/*
 * Copyright (C) 2019-2020 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/dom/attr.py"!
 * Do not change this file!
 */


#ifndef LXB_DOM_ATTR_RES_H
#define LXB_DOM_ATTR_RES_H

#include "lexbor/core/shs.h"

#include "lexbor/dom/interfaces/attr_const.h"


static const lxb_dom_attr_data_t lxb_dom_attr_res_data_default[LXB_DOM_ATTR__LAST_ENTRY] = 
{
    {{.u.short_str = "#undef", .length = 6, .next = NULL},
     LXB_DOM_ATTR__UNDEF, 1, true},
    {{.u.short_str = "active", .length = 6, .next = NULL},
     LXB_DOM_ATTR_ACTIVE, 1, true},
    {{.u.short_str = "alt", .length = 3, .next = NULL},
     LXB_DOM_ATTR_ALT, 1, true},
    {{.u.short_str = "charset", .length = 7, .next = NULL},
     LXB_DOM_ATTR_CHARSET, 1, true},
    {{.u.short_str = "checked", .length = 7, .next = NULL},
     LXB_DOM_ATTR_CHECKED, 1, true},
    {{.u.short_str = "class", .length = 5, .next = NULL},
     LXB_DOM_ATTR_CLASS, 1, true},
    {{.u.short_str = "color", .length = 5, .next = NULL},
     LXB_DOM_ATTR_COLOR, 1, true},
    {{.u.short_str = "content", .length = 7, .next = NULL},
     LXB_DOM_ATTR_CONTENT, 1, true},
    {{.u.short_str = "dir", .length = 3, .next = NULL},
     LXB_DOM_ATTR_DIR, 1, true},
    {{.u.short_str = "disabled", .length = 8, .next = NULL},
     LXB_DOM_ATTR_DISABLED, 1, true},
    {{.u.short_str = "face", .length = 4, .next = NULL},
     LXB_DOM_ATTR_FACE, 1, true},
    {{.u.short_str = "focus", .length = 5, .next = NULL},
     LXB_DOM_ATTR_FOCUS, 1, true},
    {{.u.short_str = "for", .length = 3, .next = NULL},
     LXB_DOM_ATTR_FOR, 1, true},
    {{.u.short_str = "height", .length = 6, .next = NULL},
     LXB_DOM_ATTR_HEIGHT, 1, true},
    {{.u.short_str = "hover", .length = 5, .next = NULL},
     LXB_DOM_ATTR_HOVER, 1, true},
    {{.u.short_str = "href", .length = 4, .next = NULL},
     LXB_DOM_ATTR_HREF, 1, true},
    {{.u.short_str = "html", .length = 4, .next = NULL},
     LXB_DOM_ATTR_HTML, 1, true},
    {{.u.short_str = "http-equiv", .length = 10, .next = NULL},
     LXB_DOM_ATTR_HTTP_EQUIV, 1, true},
    {{.u.short_str = "id", .length = 2, .next = NULL},
     LXB_DOM_ATTR_ID, 1, true},
    {{.u.short_str = "is", .length = 2, .next = NULL},
     LXB_DOM_ATTR_IS, 1, true},
    {{.u.short_str = "maxlength", .length = 9, .next = NULL},
     LXB_DOM_ATTR_MAXLENGTH, 1, true},
    {{.u.short_str = "placeholder", .length = 11, .next = NULL},
     LXB_DOM_ATTR_PLACEHOLDER, 1, true},
    {{.u.short_str = "pool", .length = 4, .next = NULL},
     LXB_DOM_ATTR_POOL, 1, true},
    {{.u.short_str = "public", .length = 6, .next = NULL},
     LXB_DOM_ATTR_PUBLIC, 1, true},
    {{.u.short_str = "readonly", .length = 8, .next = NULL},
     LXB_DOM_ATTR_READONLY, 1, true},
    {{.u.short_str = "required", .length = 8, .next = NULL},
     LXB_DOM_ATTR_REQUIRED, 1, true},
    {{.u.short_str = "scheme", .length = 6, .next = NULL},
     LXB_DOM_ATTR_SCHEME, 1, true},
    {{.u.short_str = "selected", .length = 8, .next = NULL},
     LXB_DOM_ATTR_SELECTED, 1, true},
    {{.u.short_str = "size", .length = 4, .next = NULL},
     LXB_DOM_ATTR_SIZE, 1, true},
    {{.u.short_str = "slot", .length = 4, .next = NULL},
     LXB_DOM_ATTR_SLOT, 1, true},
    {{.u.short_str = "src", .length = 3, .next = NULL},
     LXB_DOM_ATTR_SRC, 1, true},
    {{.u.short_str = "style", .length = 5, .next = NULL},
     LXB_DOM_ATTR_STYLE, 1, true},
    {{.u.short_str = "system", .length = 6, .next = NULL},
     LXB_DOM_ATTR_SYSTEM, 1, true},
    {{.u.short_str = "title", .length = 5, .next = NULL},
     LXB_DOM_ATTR_TITLE, 1, true},
    {{.u.short_str = "type", .length = 4, .next = NULL},
     LXB_DOM_ATTR_TYPE, 1, true},
    {{.u.short_str = "width", .length = 5, .next = NULL},
     LXB_DOM_ATTR_WIDTH, 1, true}
};

static const lexbor_shs_entry_t lxb_dom_attr_res_shs_data[40] = 
{
    {NULL, NULL, 39, 0}, 
    {"slot", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_SLOT], 4, 2}, 
    {"#undef", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR__UNDEF], 6, 0}, 
    {"active", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_ACTIVE], 6, 0}, 
    {"html", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_HTML], 4, 0}, 
    {"href", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_HREF], 4, 4}, 
    {"hover", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_HOVER], 5, 7}, 
    {"readonly", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_READONLY], 8, 0}, 
    {"disabled", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_DISABLED], 8, 0}, 
    {"color", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_COLOR], 5, 0}, 
    {"src", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_SRC], 3, 8}, 
    {"required", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_REQUIRED], 8, 0}, 
    {"id", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_ID], 2, 11}, 
    {"system", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_SYSTEM], 6, 0}, 
    {"pool", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_POOL], 4, 13}, 
    {"placeholder", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_PLACEHOLDER], 11, 0}, 
    {"selected", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_SELECTED], 8, 0}, 
    {"scheme", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_SCHEME], 6, 0}, 
    {"http-equiv", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_HTTP_EQUIV], 10, 0}, 
    {"size", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_SIZE], 4, 16}, 
    {"style", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_STYLE], 5, 0}, 
    {"class", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_CLASS], 5, 17}, 
    {"width", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_WIDTH], 5, 0}, 
    {"height", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_HEIGHT], 6, 0}, 
    {"focus", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_FOCUS], 5, 18}, 
    {"public", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_PUBLIC], 6, 0}, 
    {"content", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_CONTENT], 7, 0}, 
    {"is", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_IS], 2, 0}, 
    {"type", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_TYPE], 4, 0}, 
    {"title", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_TITLE], 5, 0}, 
    {NULL, NULL, 0, 0}, 
    {"for", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_FOR], 3, 0}, 
    {"face", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_FACE], 4, 22}, 
    {"alt", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_ALT], 3, 23}, 
    {"dir", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_DIR], 3, 0}, 
    {"charset", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_CHARSET], 7, 26}, 
    {"maxlength", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_MAXLENGTH], 9, 0}, 
    {NULL, NULL, 0, 0}, 
    {"checked", (void *) &lxb_dom_attr_res_data_default[LXB_DOM_ATTR_CHECKED], 7, 0}, 
    {NULL, NULL, 0, 0}
};


#endif /* LXB_DOM_ATTR_RES_H */
