/*
 * Copyright (C) 2018 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/tag_ns/tags.py"!
 * Do not change this file!
 */


#ifndef LXB_TAG_RES_H
#define LXB_TAG_RES_H

#endif /* LXB_TAG_RES_H */

#ifdef LXB_TAG_CONST_VERSION
#ifndef LXB_TAG_CONST_VERSION_A161EC911182C3254E7A972D5C51DF86
#error Mismatched tags version! See "lexbor/tag/const.h".
#endif /* LXB_TAG_CONST_VERSION_A161EC911182C3254E7A972D5C51DF86 */
#else
#error You need to include "lexbor/tag/const.h".
#endif /* LXB_TAG_CONST_VERSION */

static const lxb_tag_data_t lxb_tag_res_data_default[LXB_TAG__LAST_ENTRY] = 
{
    {{.u.short_str = "#undef", .length = 6, .next = NULL}, LXB_TAG__UNDEF, 1, true},
    {{.u.short_str = "#end-of-file", .length = 12, .next = NULL}, LXB_TAG__END_OF_FILE, 1, true},
    {{.u.short_str = "#text", .length = 5, .next = NULL}, LXB_TAG__TEXT, 1, true},
    {{.u.short_str = "#document", .length = 9, .next = NULL}, LXB_TAG__DOCUMENT, 1, true},
    {{.u.short_str = "!--", .length = 3, .next = NULL}, LXB_TAG__EM_COMMENT, 1, true},
    {{.u.short_str = "!doctype", .length = 8, .next = NULL}, LXB_TAG__EM_DOCTYPE, 1, true},
    {{.u.short_str = "a", .length = 1, .next = NULL}, LXB_TAG_A, 1, true},
    {{.u.short_str = "abbr", .length = 4, .next = NULL}, LXB_TAG_ABBR, 1, true},
    {{.u.short_str = "acronym", .length = 7, .next = NULL}, LXB_TAG_ACRONYM, 1, true},
    {{.u.short_str = "address", .length = 7, .next = NULL}, LXB_TAG_ADDRESS, 1, true},
    {{.u.short_str = "altglyph", .length = 8, .next = NULL}, LXB_TAG_ALTGLYPH, 1, true},
    {{.u.short_str = "altglyphdef", .length = 11, .next = NULL}, LXB_TAG_ALTGLYPHDEF, 1, true},
    {{.u.short_str = "altglyphitem", .length = 12, .next = NULL}, LXB_TAG_ALTGLYPHITEM, 1, true},
    {{.u.short_str = "animatecolor", .length = 12, .next = NULL}, LXB_TAG_ANIMATECOLOR, 1, true},
    {{.u.short_str = "animatemotion", .length = 13, .next = NULL}, LXB_TAG_ANIMATEMOTION, 1, true},
    {{.u.short_str = "animatetransform", .length = 16, .next = NULL}, LXB_TAG_ANIMATETRANSFORM, 1, true},
    {{.u.short_str = "annotation-xml", .length = 14, .next = NULL}, LXB_TAG_ANNOTATION_XML, 1, true},
    {{.u.short_str = "applet", .length = 6, .next = NULL}, LXB_TAG_APPLET, 1, true},
    {{.u.short_str = "area", .length = 4, .next = NULL}, LXB_TAG_AREA, 1, true},
    {{.u.short_str = "article", .length = 7, .next = NULL}, LXB_TAG_ARTICLE, 1, true},
    {{.u.short_str = "aside", .length = 5, .next = NULL}, LXB_TAG_ASIDE, 1, true},
    {{.u.short_str = "audio", .length = 5, .next = NULL}, LXB_TAG_AUDIO, 1, true},
    {{.u.short_str = "b", .length = 1, .next = NULL}, LXB_TAG_B, 1, true},
    {{.u.short_str = "base", .length = 4, .next = NULL}, LXB_TAG_BASE, 1, true},
    {{.u.short_str = "basefont", .length = 8, .next = NULL}, LXB_TAG_BASEFONT, 1, true},
    {{.u.short_str = "bdi", .length = 3, .next = NULL}, LXB_TAG_BDI, 1, true},
    {{.u.short_str = "bdo", .length = 3, .next = NULL}, LXB_TAG_BDO, 1, true},
    {{.u.short_str = "bgsound", .length = 7, .next = NULL}, LXB_TAG_BGSOUND, 1, true},
    {{.u.short_str = "big", .length = 3, .next = NULL}, LXB_TAG_BIG, 1, true},
    {{.u.short_str = "blink", .length = 5, .next = NULL}, LXB_TAG_BLINK, 1, true},
    {{.u.short_str = "blockquote", .length = 10, .next = NULL}, LXB_TAG_BLOCKQUOTE, 1, true},
    {{.u.short_str = "body", .length = 4, .next = NULL}, LXB_TAG_BODY, 1, true},
    {{.u.short_str = "br", .length = 2, .next = NULL}, LXB_TAG_BR, 1, true},
    {{.u.short_str = "button", .length = 6, .next = NULL}, LXB_TAG_BUTTON, 1, true},
    {{.u.short_str = "canvas", .length = 6, .next = NULL}, LXB_TAG_CANVAS, 1, true},
    {{.u.short_str = "caption", .length = 7, .next = NULL}, LXB_TAG_CAPTION, 1, true},
    {{.u.short_str = "center", .length = 6, .next = NULL}, LXB_TAG_CENTER, 1, true},
    {{.u.short_str = "cite", .length = 4, .next = NULL}, LXB_TAG_CITE, 1, true},
    {{.u.short_str = "clippath", .length = 8, .next = NULL}, LXB_TAG_CLIPPATH, 1, true},
    {{.u.short_str = "code", .length = 4, .next = NULL}, LXB_TAG_CODE, 1, true},
    {{.u.short_str = "col", .length = 3, .next = NULL}, LXB_TAG_COL, 1, true},
    {{.u.short_str = "colgroup", .length = 8, .next = NULL}, LXB_TAG_COLGROUP, 1, true},
    {{.u.short_str = "data", .length = 4, .next = NULL}, LXB_TAG_DATA, 1, true},
    {{.u.short_str = "datalist", .length = 8, .next = NULL}, LXB_TAG_DATALIST, 1, true},
    {{.u.short_str = "dd", .length = 2, .next = NULL}, LXB_TAG_DD, 1, true},
    {{.u.short_str = "del", .length = 3, .next = NULL}, LXB_TAG_DEL, 1, true},
    {{.u.short_str = "desc", .length = 4, .next = NULL}, LXB_TAG_DESC, 1, true},
    {{.u.short_str = "details", .length = 7, .next = NULL}, LXB_TAG_DETAILS, 1, true},
    {{.u.short_str = "dfn", .length = 3, .next = NULL}, LXB_TAG_DFN, 1, true},
    {{.u.short_str = "dialog", .length = 6, .next = NULL}, LXB_TAG_DIALOG, 1, true},
    {{.u.short_str = "dir", .length = 3, .next = NULL}, LXB_TAG_DIR, 1, true},
    {{.u.short_str = "div", .length = 3, .next = NULL}, LXB_TAG_DIV, 1, true},
    {{.u.short_str = "dl", .length = 2, .next = NULL}, LXB_TAG_DL, 1, true},
    {{.u.short_str = "dt", .length = 2, .next = NULL}, LXB_TAG_DT, 1, true},
    {{.u.short_str = "em", .length = 2, .next = NULL}, LXB_TAG_EM, 1, true},
    {{.u.short_str = "embed", .length = 5, .next = NULL}, LXB_TAG_EMBED, 1, true},
    {{.u.short_str = "feblend", .length = 7, .next = NULL}, LXB_TAG_FEBLEND, 1, true},
    {{.u.short_str = "fecolormatrix", .length = 13, .next = NULL}, LXB_TAG_FECOLORMATRIX, 1, true},
    {{.u.long_str = (lxb_char_t *) "fecomponenttransfer", .length = 19, .next = NULL}, LXB_TAG_FECOMPONENTTRANSFER, 1, true},
    {{.u.short_str = "fecomposite", .length = 11, .next = NULL}, LXB_TAG_FECOMPOSITE, 1, true},
    {{.u.short_str = "feconvolvematrix", .length = 16, .next = NULL}, LXB_TAG_FECONVOLVEMATRIX, 1, true},
    {{.u.long_str = (lxb_char_t *) "fediffuselighting", .length = 17, .next = NULL}, LXB_TAG_FEDIFFUSELIGHTING, 1, true},
    {{.u.long_str = (lxb_char_t *) "fedisplacementmap", .length = 17, .next = NULL}, LXB_TAG_FEDISPLACEMENTMAP, 1, true},
    {{.u.short_str = "fedistantlight", .length = 14, .next = NULL}, LXB_TAG_FEDISTANTLIGHT, 1, true},
    {{.u.short_str = "fedropshadow", .length = 12, .next = NULL}, LXB_TAG_FEDROPSHADOW, 1, true},
    {{.u.short_str = "feflood", .length = 7, .next = NULL}, LXB_TAG_FEFLOOD, 1, true},
    {{.u.short_str = "fefunca", .length = 7, .next = NULL}, LXB_TAG_FEFUNCA, 1, true},
    {{.u.short_str = "fefuncb", .length = 7, .next = NULL}, LXB_TAG_FEFUNCB, 1, true},
    {{.u.short_str = "fefuncg", .length = 7, .next = NULL}, LXB_TAG_FEFUNCG, 1, true},
    {{.u.short_str = "fefuncr", .length = 7, .next = NULL}, LXB_TAG_FEFUNCR, 1, true},
    {{.u.short_str = "fegaussianblur", .length = 14, .next = NULL}, LXB_TAG_FEGAUSSIANBLUR, 1, true},
    {{.u.short_str = "feimage", .length = 7, .next = NULL}, LXB_TAG_FEIMAGE, 1, true},
    {{.u.short_str = "femerge", .length = 7, .next = NULL}, LXB_TAG_FEMERGE, 1, true},
    {{.u.short_str = "femergenode", .length = 11, .next = NULL}, LXB_TAG_FEMERGENODE, 1, true},
    {{.u.short_str = "femorphology", .length = 12, .next = NULL}, LXB_TAG_FEMORPHOLOGY, 1, true},
    {{.u.short_str = "feoffset", .length = 8, .next = NULL}, LXB_TAG_FEOFFSET, 1, true},
    {{.u.short_str = "fepointlight", .length = 12, .next = NULL}, LXB_TAG_FEPOINTLIGHT, 1, true},
    {{.u.long_str = (lxb_char_t *) "fespecularlighting", .length = 18, .next = NULL}, LXB_TAG_FESPECULARLIGHTING, 1, true},
    {{.u.short_str = "fespotlight", .length = 11, .next = NULL}, LXB_TAG_FESPOTLIGHT, 1, true},
    {{.u.short_str = "fetile", .length = 6, .next = NULL}, LXB_TAG_FETILE, 1, true},
    {{.u.short_str = "feturbulence", .length = 12, .next = NULL}, LXB_TAG_FETURBULENCE, 1, true},
    {{.u.short_str = "fieldset", .length = 8, .next = NULL}, LXB_TAG_FIELDSET, 1, true},
    {{.u.short_str = "figcaption", .length = 10, .next = NULL}, LXB_TAG_FIGCAPTION, 1, true},
    {{.u.short_str = "figure", .length = 6, .next = NULL}, LXB_TAG_FIGURE, 1, true},
    {{.u.short_str = "font", .length = 4, .next = NULL}, LXB_TAG_FONT, 1, true},
    {{.u.short_str = "footer", .length = 6, .next = NULL}, LXB_TAG_FOOTER, 1, true},
    {{.u.short_str = "foreignobject", .length = 13, .next = NULL}, LXB_TAG_FOREIGNOBJECT, 1, true},
    {{.u.short_str = "form", .length = 4, .next = NULL}, LXB_TAG_FORM, 1, true},
    {{.u.short_str = "frame", .length = 5, .next = NULL}, LXB_TAG_FRAME, 1, true},
    {{.u.short_str = "frameset", .length = 8, .next = NULL}, LXB_TAG_FRAMESET, 1, true},
    {{.u.short_str = "glyphref", .length = 8, .next = NULL}, LXB_TAG_GLYPHREF, 1, true},
    {{.u.short_str = "h1", .length = 2, .next = NULL}, LXB_TAG_H1, 1, true},
    {{.u.short_str = "h2", .length = 2, .next = NULL}, LXB_TAG_H2, 1, true},
    {{.u.short_str = "h3", .length = 2, .next = NULL}, LXB_TAG_H3, 1, true},
    {{.u.short_str = "h4", .length = 2, .next = NULL}, LXB_TAG_H4, 1, true},
    {{.u.short_str = "h5", .length = 2, .next = NULL}, LXB_TAG_H5, 1, true},
    {{.u.short_str = "h6", .length = 2, .next = NULL}, LXB_TAG_H6, 1, true},
    {{.u.short_str = "head", .length = 4, .next = NULL}, LXB_TAG_HEAD, 1, true},
    {{.u.short_str = "header", .length = 6, .next = NULL}, LXB_TAG_HEADER, 1, true},
    {{.u.short_str = "hgroup", .length = 6, .next = NULL}, LXB_TAG_HGROUP, 1, true},
    {{.u.short_str = "hr", .length = 2, .next = NULL}, LXB_TAG_HR, 1, true},
    {{.u.short_str = "html", .length = 4, .next = NULL}, LXB_TAG_HTML, 1, true},
    {{.u.short_str = "i", .length = 1, .next = NULL}, LXB_TAG_I, 1, true},
    {{.u.short_str = "iframe", .length = 6, .next = NULL}, LXB_TAG_IFRAME, 1, true},
    {{.u.short_str = "image", .length = 5, .next = NULL}, LXB_TAG_IMAGE, 1, true},
    {{.u.short_str = "img", .length = 3, .next = NULL}, LXB_TAG_IMG, 1, true},
    {{.u.short_str = "input", .length = 5, .next = NULL}, LXB_TAG_INPUT, 1, true},
    {{.u.short_str = "ins", .length = 3, .next = NULL}, LXB_TAG_INS, 1, true},
    {{.u.short_str = "isindex", .length = 7, .next = NULL}, LXB_TAG_ISINDEX, 1, true},
    {{.u.short_str = "kbd", .length = 3, .next = NULL}, LXB_TAG_KBD, 1, true},
    {{.u.short_str = "keygen", .length = 6, .next = NULL}, LXB_TAG_KEYGEN, 1, true},
    {{.u.short_str = "label", .length = 5, .next = NULL}, LXB_TAG_LABEL, 1, true},
    {{.u.short_str = "legend", .length = 6, .next = NULL}, LXB_TAG_LEGEND, 1, true},
    {{.u.short_str = "li", .length = 2, .next = NULL}, LXB_TAG_LI, 1, true},
    {{.u.short_str = "lineargradient", .length = 14, .next = NULL}, LXB_TAG_LINEARGRADIENT, 1, true},
    {{.u.short_str = "link", .length = 4, .next = NULL}, LXB_TAG_LINK, 1, true},
    {{.u.short_str = "listing", .length = 7, .next = NULL}, LXB_TAG_LISTING, 1, true},
    {{.u.short_str = "main", .length = 4, .next = NULL}, LXB_TAG_MAIN, 1, true},
    {{.u.short_str = "malignmark", .length = 10, .next = NULL}, LXB_TAG_MALIGNMARK, 1, true},
    {{.u.short_str = "map", .length = 3, .next = NULL}, LXB_TAG_MAP, 1, true},
    {{.u.short_str = "mark", .length = 4, .next = NULL}, LXB_TAG_MARK, 1, true},
    {{.u.short_str = "marquee", .length = 7, .next = NULL}, LXB_TAG_MARQUEE, 1, true},
    {{.u.short_str = "math", .length = 4, .next = NULL}, LXB_TAG_MATH, 1, true},
    {{.u.short_str = "menu", .length = 4, .next = NULL}, LXB_TAG_MENU, 1, true},
    {{.u.short_str = "meta", .length = 4, .next = NULL}, LXB_TAG_META, 1, true},
    {{.u.short_str = "meter", .length = 5, .next = NULL}, LXB_TAG_METER, 1, true},
    {{.u.short_str = "mfenced", .length = 7, .next = NULL}, LXB_TAG_MFENCED, 1, true},
    {{.u.short_str = "mglyph", .length = 6, .next = NULL}, LXB_TAG_MGLYPH, 1, true},
    {{.u.short_str = "mi", .length = 2, .next = NULL}, LXB_TAG_MI, 1, true},
    {{.u.short_str = "mn", .length = 2, .next = NULL}, LXB_TAG_MN, 1, true},
    {{.u.short_str = "mo", .length = 2, .next = NULL}, LXB_TAG_MO, 1, true},
    {{.u.short_str = "ms", .length = 2, .next = NULL}, LXB_TAG_MS, 1, true},
    {{.u.short_str = "mtext", .length = 5, .next = NULL}, LXB_TAG_MTEXT, 1, true},
    {{.u.short_str = "multicol", .length = 8, .next = NULL}, LXB_TAG_MULTICOL, 1, true},
    {{.u.short_str = "nav", .length = 3, .next = NULL}, LXB_TAG_NAV, 1, true},
    {{.u.short_str = "nextid", .length = 6, .next = NULL}, LXB_TAG_NEXTID, 1, true},
    {{.u.short_str = "nobr", .length = 4, .next = NULL}, LXB_TAG_NOBR, 1, true},
    {{.u.short_str = "noembed", .length = 7, .next = NULL}, LXB_TAG_NOEMBED, 1, true},
    {{.u.short_str = "noframes", .length = 8, .next = NULL}, LXB_TAG_NOFRAMES, 1, true},
    {{.u.short_str = "noscript", .length = 8, .next = NULL}, LXB_TAG_NOSCRIPT, 1, true},
    {{.u.short_str = "object", .length = 6, .next = NULL}, LXB_TAG_OBJECT, 1, true},
    {{.u.short_str = "ol", .length = 2, .next = NULL}, LXB_TAG_OL, 1, true},
    {{.u.short_str = "optgroup", .length = 8, .next = NULL}, LXB_TAG_OPTGROUP, 1, true},
    {{.u.short_str = "option", .length = 6, .next = NULL}, LXB_TAG_OPTION, 1, true},
    {{.u.short_str = "output", .length = 6, .next = NULL}, LXB_TAG_OUTPUT, 1, true},
    {{.u.short_str = "p", .length = 1, .next = NULL}, LXB_TAG_P, 1, true},
    {{.u.short_str = "param", .length = 5, .next = NULL}, LXB_TAG_PARAM, 1, true},
    {{.u.short_str = "path", .length = 4, .next = NULL}, LXB_TAG_PATH, 1, true},
    {{.u.short_str = "picture", .length = 7, .next = NULL}, LXB_TAG_PICTURE, 1, true},
    {{.u.short_str = "plaintext", .length = 9, .next = NULL}, LXB_TAG_PLAINTEXT, 1, true},
    {{.u.short_str = "pre", .length = 3, .next = NULL}, LXB_TAG_PRE, 1, true},
    {{.u.short_str = "progress", .length = 8, .next = NULL}, LXB_TAG_PROGRESS, 1, true},
    {{.u.short_str = "q", .length = 1, .next = NULL}, LXB_TAG_Q, 1, true},
    {{.u.short_str = "radialgradient", .length = 14, .next = NULL}, LXB_TAG_RADIALGRADIENT, 1, true},
    {{.u.short_str = "rb", .length = 2, .next = NULL}, LXB_TAG_RB, 1, true},
    {{.u.short_str = "rp", .length = 2, .next = NULL}, LXB_TAG_RP, 1, true},
    {{.u.short_str = "rt", .length = 2, .next = NULL}, LXB_TAG_RT, 1, true},
    {{.u.short_str = "rtc", .length = 3, .next = NULL}, LXB_TAG_RTC, 1, true},
    {{.u.short_str = "ruby", .length = 4, .next = NULL}, LXB_TAG_RUBY, 1, true},
    {{.u.short_str = "s", .length = 1, .next = NULL}, LXB_TAG_S, 1, true},
    {{.u.short_str = "samp", .length = 4, .next = NULL}, LXB_TAG_SAMP, 1, true},
    {{.u.short_str = "script", .length = 6, .next = NULL}, LXB_TAG_SCRIPT, 1, true},
    {{.u.short_str = "section", .length = 7, .next = NULL}, LXB_TAG_SECTION, 1, true},
    {{.u.short_str = "select", .length = 6, .next = NULL}, LXB_TAG_SELECT, 1, true},
    {{.u.short_str = "slot", .length = 4, .next = NULL}, LXB_TAG_SLOT, 1, true},
    {{.u.short_str = "small", .length = 5, .next = NULL}, LXB_TAG_SMALL, 1, true},
    {{.u.short_str = "source", .length = 6, .next = NULL}, LXB_TAG_SOURCE, 1, true},
    {{.u.short_str = "spacer", .length = 6, .next = NULL}, LXB_TAG_SPACER, 1, true},
    {{.u.short_str = "span", .length = 4, .next = NULL}, LXB_TAG_SPAN, 1, true},
    {{.u.short_str = "strike", .length = 6, .next = NULL}, LXB_TAG_STRIKE, 1, true},
    {{.u.short_str = "strong", .length = 6, .next = NULL}, LXB_TAG_STRONG, 1, true},
    {{.u.short_str = "style", .length = 5, .next = NULL}, LXB_TAG_STYLE, 1, true},
    {{.u.short_str = "sub", .length = 3, .next = NULL}, LXB_TAG_SUB, 1, true},
    {{.u.short_str = "summary", .length = 7, .next = NULL}, LXB_TAG_SUMMARY, 1, true},
    {{.u.short_str = "sup", .length = 3, .next = NULL}, LXB_TAG_SUP, 1, true},
    {{.u.short_str = "svg", .length = 3, .next = NULL}, LXB_TAG_SVG, 1, true},
    {{.u.short_str = "table", .length = 5, .next = NULL}, LXB_TAG_TABLE, 1, true},
    {{.u.short_str = "tbody", .length = 5, .next = NULL}, LXB_TAG_TBODY, 1, true},
    {{.u.short_str = "td", .length = 2, .next = NULL}, LXB_TAG_TD, 1, true},
    {{.u.short_str = "template", .length = 8, .next = NULL}, LXB_TAG_TEMPLATE, 1, true},
    {{.u.short_str = "textarea", .length = 8, .next = NULL}, LXB_TAG_TEXTAREA, 1, true},
    {{.u.short_str = "textpath", .length = 8, .next = NULL}, LXB_TAG_TEXTPATH, 1, true},
    {{.u.short_str = "tfoot", .length = 5, .next = NULL}, LXB_TAG_TFOOT, 1, true},
    {{.u.short_str = "th", .length = 2, .next = NULL}, LXB_TAG_TH, 1, true},
    {{.u.short_str = "thead", .length = 5, .next = NULL}, LXB_TAG_THEAD, 1, true},
    {{.u.short_str = "time", .length = 4, .next = NULL}, LXB_TAG_TIME, 1, true},
    {{.u.short_str = "title", .length = 5, .next = NULL}, LXB_TAG_TITLE, 1, true},
    {{.u.short_str = "tr", .length = 2, .next = NULL}, LXB_TAG_TR, 1, true},
    {{.u.short_str = "track", .length = 5, .next = NULL}, LXB_TAG_TRACK, 1, true},
    {{.u.short_str = "tt", .length = 2, .next = NULL}, LXB_TAG_TT, 1, true},
    {{.u.short_str = "u", .length = 1, .next = NULL}, LXB_TAG_U, 1, true},
    {{.u.short_str = "ul", .length = 2, .next = NULL}, LXB_TAG_UL, 1, true},
    {{.u.short_str = "var", .length = 3, .next = NULL}, LXB_TAG_VAR, 1, true},
    {{.u.short_str = "video", .length = 5, .next = NULL}, LXB_TAG_VIDEO, 1, true},
    {{.u.short_str = "wbr", .length = 3, .next = NULL}, LXB_TAG_WBR, 1, true},
    {{.u.short_str = "xmp", .length = 3, .next = NULL}, LXB_TAG_XMP, 1, true}
};

static const lxb_tag_data_t lxb_tag_res_data_upper_default[LXB_TAG__LAST_ENTRY] = 
{
    {{.u.short_str = "#UNDEF", .length = 6, .next = NULL}, LXB_TAG__UNDEF, 1, true},
    {{.u.short_str = "#END-OF-FILE", .length = 12, .next = NULL}, LXB_TAG__END_OF_FILE, 1, true},
    {{.u.short_str = "#TEXT", .length = 5, .next = NULL}, LXB_TAG__TEXT, 1, true},
    {{.u.short_str = "#DOCUMENT", .length = 9, .next = NULL}, LXB_TAG__DOCUMENT, 1, true},
    {{.u.short_str = "!--", .length = 3, .next = NULL}, LXB_TAG__EM_COMMENT, 1, true},
    {{.u.short_str = "!DOCTYPE", .length = 8, .next = NULL}, LXB_TAG__EM_DOCTYPE, 1, true},
    {{.u.short_str = "A", .length = 1, .next = NULL}, LXB_TAG_A, 1, true},
    {{.u.short_str = "ABBR", .length = 4, .next = NULL}, LXB_TAG_ABBR, 1, true},
    {{.u.short_str = "ACRONYM", .length = 7, .next = NULL}, LXB_TAG_ACRONYM, 1, true},
    {{.u.short_str = "ADDRESS", .length = 7, .next = NULL}, LXB_TAG_ADDRESS, 1, true},
    {{.u.short_str = "ALTGLYPH", .length = 8, .next = NULL}, LXB_TAG_ALTGLYPH, 1, true},
    {{.u.short_str = "ALTGLYPHDEF", .length = 11, .next = NULL}, LXB_TAG_ALTGLYPHDEF, 1, true},
    {{.u.short_str = "ALTGLYPHITEM", .length = 12, .next = NULL}, LXB_TAG_ALTGLYPHITEM, 1, true},
    {{.u.short_str = "ANIMATECOLOR", .length = 12, .next = NULL}, LXB_TAG_ANIMATECOLOR, 1, true},
    {{.u.short_str = "ANIMATEMOTION", .length = 13, .next = NULL}, LXB_TAG_ANIMATEMOTION, 1, true},
    {{.u.short_str = "ANIMATETRANSFORM", .length = 16, .next = NULL}, LXB_TAG_ANIMATETRANSFORM, 1, true},
    {{.u.short_str = "ANNOTATION-XML", .length = 14, .next = NULL}, LXB_TAG_ANNOTATION_XML, 1, true},
    {{.u.short_str = "APPLET", .length = 6, .next = NULL}, LXB_TAG_APPLET, 1, true},
    {{.u.short_str = "AREA", .length = 4, .next = NULL}, LXB_TAG_AREA, 1, true},
    {{.u.short_str = "ARTICLE", .length = 7, .next = NULL}, LXB_TAG_ARTICLE, 1, true},
    {{.u.short_str = "ASIDE", .length = 5, .next = NULL}, LXB_TAG_ASIDE, 1, true},
    {{.u.short_str = "AUDIO", .length = 5, .next = NULL}, LXB_TAG_AUDIO, 1, true},
    {{.u.short_str = "B", .length = 1, .next = NULL}, LXB_TAG_B, 1, true},
    {{.u.short_str = "BASE", .length = 4, .next = NULL}, LXB_TAG_BASE, 1, true},
    {{.u.short_str = "BASEFONT", .length = 8, .next = NULL}, LXB_TAG_BASEFONT, 1, true},
    {{.u.short_str = "BDI", .length = 3, .next = NULL}, LXB_TAG_BDI, 1, true},
    {{.u.short_str = "BDO", .length = 3, .next = NULL}, LXB_TAG_BDO, 1, true},
    {{.u.short_str = "BGSOUND", .length = 7, .next = NULL}, LXB_TAG_BGSOUND, 1, true},
    {{.u.short_str = "BIG", .length = 3, .next = NULL}, LXB_TAG_BIG, 1, true},
    {{.u.short_str = "BLINK", .length = 5, .next = NULL}, LXB_TAG_BLINK, 1, true},
    {{.u.short_str = "BLOCKQUOTE", .length = 10, .next = NULL}, LXB_TAG_BLOCKQUOTE, 1, true},
    {{.u.short_str = "BODY", .length = 4, .next = NULL}, LXB_TAG_BODY, 1, true},
    {{.u.short_str = "BR", .length = 2, .next = NULL}, LXB_TAG_BR, 1, true},
    {{.u.short_str = "BUTTON", .length = 6, .next = NULL}, LXB_TAG_BUTTON, 1, true},
    {{.u.short_str = "CANVAS", .length = 6, .next = NULL}, LXB_TAG_CANVAS, 1, true},
    {{.u.short_str = "CAPTION", .length = 7, .next = NULL}, LXB_TAG_CAPTION, 1, true},
    {{.u.short_str = "CENTER", .length = 6, .next = NULL}, LXB_TAG_CENTER, 1, true},
    {{.u.short_str = "CITE", .length = 4, .next = NULL}, LXB_TAG_CITE, 1, true},
    {{.u.short_str = "CLIPPATH", .length = 8, .next = NULL}, LXB_TAG_CLIPPATH, 1, true},
    {{.u.short_str = "CODE", .length = 4, .next = NULL}, LXB_TAG_CODE, 1, true},
    {{.u.short_str = "COL", .length = 3, .next = NULL}, LXB_TAG_COL, 1, true},
    {{.u.short_str = "COLGROUP", .length = 8, .next = NULL}, LXB_TAG_COLGROUP, 1, true},
    {{.u.short_str = "DATA", .length = 4, .next = NULL}, LXB_TAG_DATA, 1, true},
    {{.u.short_str = "DATALIST", .length = 8, .next = NULL}, LXB_TAG_DATALIST, 1, true},
    {{.u.short_str = "DD", .length = 2, .next = NULL}, LXB_TAG_DD, 1, true},
    {{.u.short_str = "DEL", .length = 3, .next = NULL}, LXB_TAG_DEL, 1, true},
    {{.u.short_str = "DESC", .length = 4, .next = NULL}, LXB_TAG_DESC, 1, true},
    {{.u.short_str = "DETAILS", .length = 7, .next = NULL}, LXB_TAG_DETAILS, 1, true},
    {{.u.short_str = "DFN", .length = 3, .next = NULL}, LXB_TAG_DFN, 1, true},
    {{.u.short_str = "DIALOG", .length = 6, .next = NULL}, LXB_TAG_DIALOG, 1, true},
    {{.u.short_str = "DIR", .length = 3, .next = NULL}, LXB_TAG_DIR, 1, true},
    {{.u.short_str = "DIV", .length = 3, .next = NULL}, LXB_TAG_DIV, 1, true},
    {{.u.short_str = "DL", .length = 2, .next = NULL}, LXB_TAG_DL, 1, true},
    {{.u.short_str = "DT", .length = 2, .next = NULL}, LXB_TAG_DT, 1, true},
    {{.u.short_str = "EM", .length = 2, .next = NULL}, LXB_TAG_EM, 1, true},
    {{.u.short_str = "EMBED", .length = 5, .next = NULL}, LXB_TAG_EMBED, 1, true},
    {{.u.short_str = "FEBLEND", .length = 7, .next = NULL}, LXB_TAG_FEBLEND, 1, true},
    {{.u.short_str = "FECOLORMATRIX", .length = 13, .next = NULL}, LXB_TAG_FECOLORMATRIX, 1, true},
    {{.u.long_str = (lxb_char_t *) "FECOMPONENTTRANSFER", .length = 19, .next = NULL}, LXB_TAG_FECOMPONENTTRANSFER, 1, true},
    {{.u.short_str = "FECOMPOSITE", .length = 11, .next = NULL}, LXB_TAG_FECOMPOSITE, 1, true},
    {{.u.short_str = "FECONVOLVEMATRIX", .length = 16, .next = NULL}, LXB_TAG_FECONVOLVEMATRIX, 1, true},
    {{.u.long_str = (lxb_char_t *) "FEDIFFUSELIGHTING", .length = 17, .next = NULL}, LXB_TAG_FEDIFFUSELIGHTING, 1, true},
    {{.u.long_str = (lxb_char_t *) "FEDISPLACEMENTMAP", .length = 17, .next = NULL}, LXB_TAG_FEDISPLACEMENTMAP, 1, true},
    {{.u.short_str = "FEDISTANTLIGHT", .length = 14, .next = NULL}, LXB_TAG_FEDISTANTLIGHT, 1, true},
    {{.u.short_str = "FEDROPSHADOW", .length = 12, .next = NULL}, LXB_TAG_FEDROPSHADOW, 1, true},
    {{.u.short_str = "FEFLOOD", .length = 7, .next = NULL}, LXB_TAG_FEFLOOD, 1, true},
    {{.u.short_str = "FEFUNCA", .length = 7, .next = NULL}, LXB_TAG_FEFUNCA, 1, true},
    {{.u.short_str = "FEFUNCB", .length = 7, .next = NULL}, LXB_TAG_FEFUNCB, 1, true},
    {{.u.short_str = "FEFUNCG", .length = 7, .next = NULL}, LXB_TAG_FEFUNCG, 1, true},
    {{.u.short_str = "FEFUNCR", .length = 7, .next = NULL}, LXB_TAG_FEFUNCR, 1, true},
    {{.u.short_str = "FEGAUSSIANBLUR", .length = 14, .next = NULL}, LXB_TAG_FEGAUSSIANBLUR, 1, true},
    {{.u.short_str = "FEIMAGE", .length = 7, .next = NULL}, LXB_TAG_FEIMAGE, 1, true},
    {{.u.short_str = "FEMERGE", .length = 7, .next = NULL}, LXB_TAG_FEMERGE, 1, true},
    {{.u.short_str = "FEMERGENODE", .length = 11, .next = NULL}, LXB_TAG_FEMERGENODE, 1, true},
    {{.u.short_str = "FEMORPHOLOGY", .length = 12, .next = NULL}, LXB_TAG_FEMORPHOLOGY, 1, true},
    {{.u.short_str = "FEOFFSET", .length = 8, .next = NULL}, LXB_TAG_FEOFFSET, 1, true},
    {{.u.short_str = "FEPOINTLIGHT", .length = 12, .next = NULL}, LXB_TAG_FEPOINTLIGHT, 1, true},
    {{.u.long_str = (lxb_char_t *) "FESPECULARLIGHTING", .length = 18, .next = NULL}, LXB_TAG_FESPECULARLIGHTING, 1, true},
    {{.u.short_str = "FESPOTLIGHT", .length = 11, .next = NULL}, LXB_TAG_FESPOTLIGHT, 1, true},
    {{.u.short_str = "FETILE", .length = 6, .next = NULL}, LXB_TAG_FETILE, 1, true},
    {{.u.short_str = "FETURBULENCE", .length = 12, .next = NULL}, LXB_TAG_FETURBULENCE, 1, true},
    {{.u.short_str = "FIELDSET", .length = 8, .next = NULL}, LXB_TAG_FIELDSET, 1, true},
    {{.u.short_str = "FIGCAPTION", .length = 10, .next = NULL}, LXB_TAG_FIGCAPTION, 1, true},
    {{.u.short_str = "FIGURE", .length = 6, .next = NULL}, LXB_TAG_FIGURE, 1, true},
    {{.u.short_str = "FONT", .length = 4, .next = NULL}, LXB_TAG_FONT, 1, true},
    {{.u.short_str = "FOOTER", .length = 6, .next = NULL}, LXB_TAG_FOOTER, 1, true},
    {{.u.short_str = "FOREIGNOBJECT", .length = 13, .next = NULL}, LXB_TAG_FOREIGNOBJECT, 1, true},
    {{.u.short_str = "FORM", .length = 4, .next = NULL}, LXB_TAG_FORM, 1, true},
    {{.u.short_str = "FRAME", .length = 5, .next = NULL}, LXB_TAG_FRAME, 1, true},
    {{.u.short_str = "FRAMESET", .length = 8, .next = NULL}, LXB_TAG_FRAMESET, 1, true},
    {{.u.short_str = "GLYPHREF", .length = 8, .next = NULL}, LXB_TAG_GLYPHREF, 1, true},
    {{.u.short_str = "H1", .length = 2, .next = NULL}, LXB_TAG_H1, 1, true},
    {{.u.short_str = "H2", .length = 2, .next = NULL}, LXB_TAG_H2, 1, true},
    {{.u.short_str = "H3", .length = 2, .next = NULL}, LXB_TAG_H3, 1, true},
    {{.u.short_str = "H4", .length = 2, .next = NULL}, LXB_TAG_H4, 1, true},
    {{.u.short_str = "H5", .length = 2, .next = NULL}, LXB_TAG_H5, 1, true},
    {{.u.short_str = "H6", .length = 2, .next = NULL}, LXB_TAG_H6, 1, true},
    {{.u.short_str = "HEAD", .length = 4, .next = NULL}, LXB_TAG_HEAD, 1, true},
    {{.u.short_str = "HEADER", .length = 6, .next = NULL}, LXB_TAG_HEADER, 1, true},
    {{.u.short_str = "HGROUP", .length = 6, .next = NULL}, LXB_TAG_HGROUP, 1, true},
    {{.u.short_str = "HR", .length = 2, .next = NULL}, LXB_TAG_HR, 1, true},
    {{.u.short_str = "HTML", .length = 4, .next = NULL}, LXB_TAG_HTML, 1, true},
    {{.u.short_str = "I", .length = 1, .next = NULL}, LXB_TAG_I, 1, true},
    {{.u.short_str = "IFRAME", .length = 6, .next = NULL}, LXB_TAG_IFRAME, 1, true},
    {{.u.short_str = "IMAGE", .length = 5, .next = NULL}, LXB_TAG_IMAGE, 1, true},
    {{.u.short_str = "IMG", .length = 3, .next = NULL}, LXB_TAG_IMG, 1, true},
    {{.u.short_str = "INPUT", .length = 5, .next = NULL}, LXB_TAG_INPUT, 1, true},
    {{.u.short_str = "INS", .length = 3, .next = NULL}, LXB_TAG_INS, 1, true},
    {{.u.short_str = "ISINDEX", .length = 7, .next = NULL}, LXB_TAG_ISINDEX, 1, true},
    {{.u.short_str = "KBD", .length = 3, .next = NULL}, LXB_TAG_KBD, 1, true},
    {{.u.short_str = "KEYGEN", .length = 6, .next = NULL}, LXB_TAG_KEYGEN, 1, true},
    {{.u.short_str = "LABEL", .length = 5, .next = NULL}, LXB_TAG_LABEL, 1, true},
    {{.u.short_str = "LEGEND", .length = 6, .next = NULL}, LXB_TAG_LEGEND, 1, true},
    {{.u.short_str = "LI", .length = 2, .next = NULL}, LXB_TAG_LI, 1, true},
    {{.u.short_str = "LINEARGRADIENT", .length = 14, .next = NULL}, LXB_TAG_LINEARGRADIENT, 1, true},
    {{.u.short_str = "LINK", .length = 4, .next = NULL}, LXB_TAG_LINK, 1, true},
    {{.u.short_str = "LISTING", .length = 7, .next = NULL}, LXB_TAG_LISTING, 1, true},
    {{.u.short_str = "MAIN", .length = 4, .next = NULL}, LXB_TAG_MAIN, 1, true},
    {{.u.short_str = "MALIGNMARK", .length = 10, .next = NULL}, LXB_TAG_MALIGNMARK, 1, true},
    {{.u.short_str = "MAP", .length = 3, .next = NULL}, LXB_TAG_MAP, 1, true},
    {{.u.short_str = "MARK", .length = 4, .next = NULL}, LXB_TAG_MARK, 1, true},
    {{.u.short_str = "MARQUEE", .length = 7, .next = NULL}, LXB_TAG_MARQUEE, 1, true},
    {{.u.short_str = "MATH", .length = 4, .next = NULL}, LXB_TAG_MATH, 1, true},
    {{.u.short_str = "MENU", .length = 4, .next = NULL}, LXB_TAG_MENU, 1, true},
    {{.u.short_str = "META", .length = 4, .next = NULL}, LXB_TAG_META, 1, true},
    {{.u.short_str = "METER", .length = 5, .next = NULL}, LXB_TAG_METER, 1, true},
    {{.u.short_str = "MFENCED", .length = 7, .next = NULL}, LXB_TAG_MFENCED, 1, true},
    {{.u.short_str = "MGLYPH", .length = 6, .next = NULL}, LXB_TAG_MGLYPH, 1, true},
    {{.u.short_str = "MI", .length = 2, .next = NULL}, LXB_TAG_MI, 1, true},
    {{.u.short_str = "MN", .length = 2, .next = NULL}, LXB_TAG_MN, 1, true},
    {{.u.short_str = "MO", .length = 2, .next = NULL}, LXB_TAG_MO, 1, true},
    {{.u.short_str = "MS", .length = 2, .next = NULL}, LXB_TAG_MS, 1, true},
    {{.u.short_str = "MTEXT", .length = 5, .next = NULL}, LXB_TAG_MTEXT, 1, true},
    {{.u.short_str = "MULTICOL", .length = 8, .next = NULL}, LXB_TAG_MULTICOL, 1, true},
    {{.u.short_str = "NAV", .length = 3, .next = NULL}, LXB_TAG_NAV, 1, true},
    {{.u.short_str = "NEXTID", .length = 6, .next = NULL}, LXB_TAG_NEXTID, 1, true},
    {{.u.short_str = "NOBR", .length = 4, .next = NULL}, LXB_TAG_NOBR, 1, true},
    {{.u.short_str = "NOEMBED", .length = 7, .next = NULL}, LXB_TAG_NOEMBED, 1, true},
    {{.u.short_str = "NOFRAMES", .length = 8, .next = NULL}, LXB_TAG_NOFRAMES, 1, true},
    {{.u.short_str = "NOSCRIPT", .length = 8, .next = NULL}, LXB_TAG_NOSCRIPT, 1, true},
    {{.u.short_str = "OBJECT", .length = 6, .next = NULL}, LXB_TAG_OBJECT, 1, true},
    {{.u.short_str = "OL", .length = 2, .next = NULL}, LXB_TAG_OL, 1, true},
    {{.u.short_str = "OPTGROUP", .length = 8, .next = NULL}, LXB_TAG_OPTGROUP, 1, true},
    {{.u.short_str = "OPTION", .length = 6, .next = NULL}, LXB_TAG_OPTION, 1, true},
    {{.u.short_str = "OUTPUT", .length = 6, .next = NULL}, LXB_TAG_OUTPUT, 1, true},
    {{.u.short_str = "P", .length = 1, .next = NULL}, LXB_TAG_P, 1, true},
    {{.u.short_str = "PARAM", .length = 5, .next = NULL}, LXB_TAG_PARAM, 1, true},
    {{.u.short_str = "PATH", .length = 4, .next = NULL}, LXB_TAG_PATH, 1, true},
    {{.u.short_str = "PICTURE", .length = 7, .next = NULL}, LXB_TAG_PICTURE, 1, true},
    {{.u.short_str = "PLAINTEXT", .length = 9, .next = NULL}, LXB_TAG_PLAINTEXT, 1, true},
    {{.u.short_str = "PRE", .length = 3, .next = NULL}, LXB_TAG_PRE, 1, true},
    {{.u.short_str = "PROGRESS", .length = 8, .next = NULL}, LXB_TAG_PROGRESS, 1, true},
    {{.u.short_str = "Q", .length = 1, .next = NULL}, LXB_TAG_Q, 1, true},
    {{.u.short_str = "RADIALGRADIENT", .length = 14, .next = NULL}, LXB_TAG_RADIALGRADIENT, 1, true},
    {{.u.short_str = "RB", .length = 2, .next = NULL}, LXB_TAG_RB, 1, true},
    {{.u.short_str = "RP", .length = 2, .next = NULL}, LXB_TAG_RP, 1, true},
    {{.u.short_str = "RT", .length = 2, .next = NULL}, LXB_TAG_RT, 1, true},
    {{.u.short_str = "RTC", .length = 3, .next = NULL}, LXB_TAG_RTC, 1, true},
    {{.u.short_str = "RUBY", .length = 4, .next = NULL}, LXB_TAG_RUBY, 1, true},
    {{.u.short_str = "S", .length = 1, .next = NULL}, LXB_TAG_S, 1, true},
    {{.u.short_str = "SAMP", .length = 4, .next = NULL}, LXB_TAG_SAMP, 1, true},
    {{.u.short_str = "SCRIPT", .length = 6, .next = NULL}, LXB_TAG_SCRIPT, 1, true},
    {{.u.short_str = "SECTION", .length = 7, .next = NULL}, LXB_TAG_SECTION, 1, true},
    {{.u.short_str = "SELECT", .length = 6, .next = NULL}, LXB_TAG_SELECT, 1, true},
    {{.u.short_str = "SLOT", .length = 4, .next = NULL}, LXB_TAG_SLOT, 1, true},
    {{.u.short_str = "SMALL", .length = 5, .next = NULL}, LXB_TAG_SMALL, 1, true},
    {{.u.short_str = "SOURCE", .length = 6, .next = NULL}, LXB_TAG_SOURCE, 1, true},
    {{.u.short_str = "SPACER", .length = 6, .next = NULL}, LXB_TAG_SPACER, 1, true},
    {{.u.short_str = "SPAN", .length = 4, .next = NULL}, LXB_TAG_SPAN, 1, true},
    {{.u.short_str = "STRIKE", .length = 6, .next = NULL}, LXB_TAG_STRIKE, 1, true},
    {{.u.short_str = "STRONG", .length = 6, .next = NULL}, LXB_TAG_STRONG, 1, true},
    {{.u.short_str = "STYLE", .length = 5, .next = NULL}, LXB_TAG_STYLE, 1, true},
    {{.u.short_str = "SUB", .length = 3, .next = NULL}, LXB_TAG_SUB, 1, true},
    {{.u.short_str = "SUMMARY", .length = 7, .next = NULL}, LXB_TAG_SUMMARY, 1, true},
    {{.u.short_str = "SUP", .length = 3, .next = NULL}, LXB_TAG_SUP, 1, true},
    {{.u.short_str = "SVG", .length = 3, .next = NULL}, LXB_TAG_SVG, 1, true},
    {{.u.short_str = "TABLE", .length = 5, .next = NULL}, LXB_TAG_TABLE, 1, true},
    {{.u.short_str = "TBODY", .length = 5, .next = NULL}, LXB_TAG_TBODY, 1, true},
    {{.u.short_str = "TD", .length = 2, .next = NULL}, LXB_TAG_TD, 1, true},
    {{.u.short_str = "TEMPLATE", .length = 8, .next = NULL}, LXB_TAG_TEMPLATE, 1, true},
    {{.u.short_str = "TEXTAREA", .length = 8, .next = NULL}, LXB_TAG_TEXTAREA, 1, true},
    {{.u.short_str = "TEXTPATH", .length = 8, .next = NULL}, LXB_TAG_TEXTPATH, 1, true},
    {{.u.short_str = "TFOOT", .length = 5, .next = NULL}, LXB_TAG_TFOOT, 1, true},
    {{.u.short_str = "TH", .length = 2, .next = NULL}, LXB_TAG_TH, 1, true},
    {{.u.short_str = "THEAD", .length = 5, .next = NULL}, LXB_TAG_THEAD, 1, true},
    {{.u.short_str = "TIME", .length = 4, .next = NULL}, LXB_TAG_TIME, 1, true},
    {{.u.short_str = "TITLE", .length = 5, .next = NULL}, LXB_TAG_TITLE, 1, true},
    {{.u.short_str = "TR", .length = 2, .next = NULL}, LXB_TAG_TR, 1, true},
    {{.u.short_str = "TRACK", .length = 5, .next = NULL}, LXB_TAG_TRACK, 1, true},
    {{.u.short_str = "TT", .length = 2, .next = NULL}, LXB_TAG_TT, 1, true},
    {{.u.short_str = "U", .length = 1, .next = NULL}, LXB_TAG_U, 1, true},
    {{.u.short_str = "UL", .length = 2, .next = NULL}, LXB_TAG_UL, 1, true},
    {{.u.short_str = "VAR", .length = 3, .next = NULL}, LXB_TAG_VAR, 1, true},
    {{.u.short_str = "VIDEO", .length = 5, .next = NULL}, LXB_TAG_VIDEO, 1, true},
    {{.u.short_str = "WBR", .length = 3, .next = NULL}, LXB_TAG_WBR, 1, true},
    {{.u.short_str = "XMP", .length = 3, .next = NULL}, LXB_TAG_XMP, 1, true}
};

static const lexbor_shs_entry_t lxb_tag_res_shs_data_default[] = 
{
    {NULL, NULL, 262, 0}, {"radialgradient", (void *) &lxb_tag_res_data_default[153], 14, 0}, 
    {"fecomponenttransfer", (void *) &lxb_tag_res_data_default[58], 19, 0}, {"abbr", (void *) &lxb_tag_res_data_default[7], 4, 1}, 
    {"feflood", (void *) &lxb_tag_res_data_default[65], 7, 0}, {"marquee", (void *) &lxb_tag_res_data_default[121], 7, 0}, 
    {"feblend", (void *) &lxb_tag_res_data_default[56], 7, 4}, {"optgroup", (void *) &lxb_tag_res_data_default[142], 8, 0}, 
    {"video", (void *) &lxb_tag_res_data_default[193], 5, 10}, {"u", (void *) &lxb_tag_res_data_default[190], 1, 0}, 
    {"iframe", (void *) &lxb_tag_res_data_default[103], 6, 0}, {"animatecolor", (void *) &lxb_tag_res_data_default[13], 12, 0}, 
    {"output", (void *) &lxb_tag_res_data_default[144], 6, 0}, {"figcaption", (void *) &lxb_tag_res_data_default[82], 10, 0}, 
    {"mglyph", (void *) &lxb_tag_res_data_default[127], 6, 0}, {"!--", (void *) &lxb_tag_res_data_default[4], 3, 0}, 
    {"fefuncg", (void *) &lxb_tag_res_data_default[68], 7, 0}, {"aside", (void *) &lxb_tag_res_data_default[20], 5, 0}, 
    {"style", (void *) &lxb_tag_res_data_default[171], 5, 0}, {"strike", (void *) &lxb_tag_res_data_default[169], 6, 0}, 
    {"header", (void *) &lxb_tag_res_data_default[98], 6, 0}, {"glyphref", (void *) &lxb_tag_res_data_default[90], 8, 23}, 
    {"label", (void *) &lxb_tag_res_data_default[111], 5, 0}, {"feconvolvematrix", (void *) &lxb_tag_res_data_default[60], 16, 0}, 
    {"altglyphdef", (void *) &lxb_tag_res_data_default[11], 11, 0}, {"title", (void *) &lxb_tag_res_data_default[186], 5, 0}, 
    {"head", (void *) &lxb_tag_res_data_default[97], 4, 0}, {"noframes", (void *) &lxb_tag_res_data_default[138], 8, 0}, 
    {"code", (void *) &lxb_tag_res_data_default[39], 4, 30}, {"rb", (void *) &lxb_tag_res_data_default[154], 2, 5}, 
    {"blink", (void *) &lxb_tag_res_data_default[29], 5, 0}, {"image", (void *) &lxb_tag_res_data_default[104], 5, 0}, 
    {"col", (void *) &lxb_tag_res_data_default[40], 3, 8}, {"object", (void *) &lxb_tag_res_data_default[140], 6, 12}, 
    {"template", (void *) &lxb_tag_res_data_default[179], 8, 36}, {"h2", (void *) &lxb_tag_res_data_default[92], 2, 13}, 
    {"lineargradient", (void *) &lxb_tag_res_data_default[114], 14, 0}, {"math", (void *) &lxb_tag_res_data_default[122], 4, 0}, 
    {"base", (void *) &lxb_tag_res_data_default[23], 4, 45}, {"dl", (void *) &lxb_tag_res_data_default[52], 2, 14}, 
    {"del", (void *) &lxb_tag_res_data_default[45], 3, 16}, {"svg", (void *) &lxb_tag_res_data_default[175], 3, 17}, 
    {"dir", (void *) &lxb_tag_res_data_default[50], 3, 0}, {"article", (void *) &lxb_tag_res_data_default[19], 7, 0}, 
    {"strong", (void *) &lxb_tag_res_data_default[170], 6, 0}, {"dialog", (void *) &lxb_tag_res_data_default[49], 6, 0}, 
    {"details", (void *) &lxb_tag_res_data_default[47], 7, 0}, {"textpath", (void *) &lxb_tag_res_data_default[181], 8, 52}, 
    {"mark", (void *) &lxb_tag_res_data_default[120], 4, 0}, {"basefont", (void *) &lxb_tag_res_data_default[24], 8, 0}, 
    {"fediffuselighting", (void *) &lxb_tag_res_data_default[61], 17, 0}, {"fespecularlighting", (void *) &lxb_tag_res_data_default[77], 18, 0}, 
    {"blockquote", (void *) &lxb_tag_res_data_default[30], 10, 0}, {"script", (void *) &lxb_tag_res_data_default[161], 6, 58}, 
    {"malignmark", (void *) &lxb_tag_res_data_default[118], 10, 0}, {"hr", (void *) &lxb_tag_res_data_default[100], 2, 18}, 
    {"source", (void *) &lxb_tag_res_data_default[166], 6, 19}, {"mn", (void *) &lxb_tag_res_data_default[129], 2, 0}, 
    {"select", (void *) &lxb_tag_res_data_default[163], 6, 0}, {"main", (void *) &lxb_tag_res_data_default[117], 4, 20}, 
    {"fieldset", (void *) &lxb_tag_res_data_default[81], 8, 62}, {"ins", (void *) &lxb_tag_res_data_default[107], 3, 0}, 
    {"frameset", (void *) &lxb_tag_res_data_default[89], 8, 0}, {"button", (void *) &lxb_tag_res_data_default[33], 6, 0}, 
    {"fecolormatrix", (void *) &lxb_tag_res_data_default[57], 13, 0}, {"q", (void *) &lxb_tag_res_data_default[152], 1, 0}, 
    {"animatemotion", (void *) &lxb_tag_res_data_default[14], 13, 0}, {"time", (void *) &lxb_tag_res_data_default[185], 4, 21}, 
    {"table", (void *) &lxb_tag_res_data_default[176], 5, 25}, {"h6", (void *) &lxb_tag_res_data_default[96], 2, 26}, 
    {"cite", (void *) &lxb_tag_res_data_default[37], 4, 28}, {"img", (void *) &lxb_tag_res_data_default[105], 3, 34}, 
    {"fepointlight", (void *) &lxb_tag_res_data_default[76], 12, 0}, {"audio", (void *) &lxb_tag_res_data_default[21], 5, 0}, 
    {"#end-of-file", (void *) &lxb_tag_res_data_default[1], 12, 0}, {"noscript", (void *) &lxb_tag_res_data_default[139], 8, 0}, 
    {"foreignobject", (void *) &lxb_tag_res_data_default[86], 13, 0}, {"spacer", (void *) &lxb_tag_res_data_default[167], 6, 0}, 
    {"samp", (void *) &lxb_tag_res_data_default[160], 4, 0}, {"altglyphitem", (void *) &lxb_tag_res_data_default[12], 12, 0}, 
    {"dt", (void *) &lxb_tag_res_data_default[53], 2, 0}, {"data", (void *) &lxb_tag_res_data_default[42], 4, 0}, 
    {"mtext", (void *) &lxb_tag_res_data_default[132], 5, 0}, {"path", (void *) &lxb_tag_res_data_default[147], 4, 0}, 
    {"input", (void *) &lxb_tag_res_data_default[106], 5, 0}, {"th", (void *) &lxb_tag_res_data_default[183], 2, 38}, 
    {"p", (void *) &lxb_tag_res_data_default[145], 1, 0}, {"animatetransform", (void *) &lxb_tag_res_data_default[15], 16, 0}, 
    {"datalist", (void *) &lxb_tag_res_data_default[43], 8, 0}, {"small", (void *) &lxb_tag_res_data_default[165], 5, 0}, 
    {"b", (void *) &lxb_tag_res_data_default[22], 1, 46}, {"nextid", (void *) &lxb_tag_res_data_default[135], 6, 47}, 
    {"noembed", (void *) &lxb_tag_res_data_default[137], 7, 0}, {"nav", (void *) &lxb_tag_res_data_default[134], 3, 0}, 
    {"bgsound", (void *) &lxb_tag_res_data_default[27], 7, 0}, {"slot", (void *) &lxb_tag_res_data_default[164], 4, 0}, 
    {"param", (void *) &lxb_tag_res_data_default[146], 5, 0}, {"font", (void *) &lxb_tag_res_data_default[84], 4, 53}, 
    {"figure", (void *) &lxb_tag_res_data_default[83], 6, 0}, {"femerge", (void *) &lxb_tag_res_data_default[72], 7, 0}, 
    {"femergenode", (void *) &lxb_tag_res_data_default[73], 11, 0}, {"feoffset", (void *) &lxb_tag_res_data_default[75], 8, 60}, 
    {"#text", (void *) &lxb_tag_res_data_default[2], 5, 0}, {"ul", (void *) &lxb_tag_res_data_default[191], 2, 0}, 
    {"fespotlight", (void *) &lxb_tag_res_data_default[78], 11, 66}, {"form", (void *) &lxb_tag_res_data_default[87], 4, 72}, 
    {"#document", (void *) &lxb_tag_res_data_default[3], 9, 76}, {"fedistantlight", (void *) &lxb_tag_res_data_default[63], 14, 0}, 
    {"track", (void *) &lxb_tag_res_data_default[188], 5, 0}, {"h3", (void *) &lxb_tag_res_data_default[93], 2, 77}, 
    {"h1", (void *) &lxb_tag_res_data_default[91], 2, 0}, {"i", (void *) &lxb_tag_res_data_default[102], 1, 0}, 
    {"altglyph", (void *) &lxb_tag_res_data_default[10], 8, 0}, {"legend", (void *) &lxb_tag_res_data_default[112], 6, 115}, 
    {"tbody", (void *) &lxb_tag_res_data_default[177], 5, 0}, {"address", (void *) &lxb_tag_res_data_default[9], 7, 0}, 
    {"caption", (void *) &lxb_tag_res_data_default[35], 7, 0}, {"option", (void *) &lxb_tag_res_data_default[143], 6, 0}, 
    {"sup", (void *) &lxb_tag_res_data_default[174], 3, 0}, {"body", (void *) &lxb_tag_res_data_default[31], 4, 78}, 
    {"progress", (void *) &lxb_tag_res_data_default[151], 8, 122}, {"acronym", (void *) &lxb_tag_res_data_default[8], 7, 0}, 
    {"fegaussianblur", (void *) &lxb_tag_res_data_default[70], 14, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {"mi", (void *) &lxb_tag_res_data_default[128], 2, 79}, {NULL, NULL, 0, 0}, 
    {"dfn", (void *) &lxb_tag_res_data_default[48], 3, 0}, {"a", (void *) &lxb_tag_res_data_default[6], 1, 80}, 
    {"listing", (void *) &lxb_tag_res_data_default[116], 7, 87}, {"span", (void *) &lxb_tag_res_data_default[168], 4, 0}, 
    {"area", (void *) &lxb_tag_res_data_default[18], 4, 0}, {"clippath", (void *) &lxb_tag_res_data_default[38], 8, 0}, 
    {"section", (void *) &lxb_tag_res_data_default[162], 7, 0}, {"li", (void *) &lxb_tag_res_data_default[113], 2, 88}, 
    {NULL, NULL, 0, 0}, {"html", (void *) &lxb_tag_res_data_default[101], 4, 0}, 
    {NULL, NULL, 0, 0}, {"fedropshadow", (void *) &lxb_tag_res_data_default[64], 12, 0}, 
    {"embed", (void *) &lxb_tag_res_data_default[55], 5, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {"multicol", (void *) &lxb_tag_res_data_default[133], 8, 0}, 
    {"var", (void *) &lxb_tag_res_data_default[192], 3, 89}, {"rp", (void *) &lxb_tag_res_data_default[155], 2, 0}, 
    {NULL, NULL, 0, 0}, {"link", (void *) &lxb_tag_res_data_default[115], 4, 0}, 
    {"mo", (void *) &lxb_tag_res_data_default[130], 2, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {"annotation-xml", (void *) &lxb_tag_res_data_default[16], 14, 0}, 
    {"fedisplacementmap", (void *) &lxb_tag_res_data_default[62], 17, 0}, {"center", (void *) &lxb_tag_res_data_default[36], 6, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {"fefuncb", (void *) &lxb_tag_res_data_default[67], 7, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {"meter", (void *) &lxb_tag_res_data_default[125], 5, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {"tt", (void *) &lxb_tag_res_data_default[189], 2, 0}, 
    {"big", (void *) &lxb_tag_res_data_default[28], 3, 93}, {NULL, NULL, 0, 0}, 
    {"tfoot", (void *) &lxb_tag_res_data_default[182], 5, 0}, {"desc", (void *) &lxb_tag_res_data_default[46], 4, 0}, 
    {"isindex", (void *) &lxb_tag_res_data_default[108], 7, 0}, {NULL, NULL, 0, 0}, 
    {"menu", (void *) &lxb_tag_res_data_default[123], 4, 0}, {"hgroup", (void *) &lxb_tag_res_data_default[99], 6, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {"wbr", (void *) &lxb_tag_res_data_default[194], 3, 0}, {NULL, NULL, 0, 0}, 
    {"pre", (void *) &lxb_tag_res_data_default[150], 3, 94}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {"picture", (void *) &lxb_tag_res_data_default[148], 7, 0}, {"h4", (void *) &lxb_tag_res_data_default[94], 2, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {"meta", (void *) &lxb_tag_res_data_default[124], 4, 96}, {NULL, NULL, 0, 0}, 
    {"rtc", (void *) &lxb_tag_res_data_default[157], 3, 0}, {NULL, NULL, 0, 0}, 
    {"frame", (void *) &lxb_tag_res_data_default[88], 5, 0}, {"fetile", (void *) &lxb_tag_res_data_default[79], 6, 98}, 
    {"feimage", (void *) &lxb_tag_res_data_default[71], 7, 99}, {NULL, NULL, 0, 0}, 
    {"xmp", (void *) &lxb_tag_res_data_default[195], 3, 0}, {NULL, NULL, 0, 0}, 
    {"fecomposite", (void *) &lxb_tag_res_data_default[59], 11, 100}, {"feturbulence", (void *) &lxb_tag_res_data_default[80], 12, 0}, 
    {NULL, NULL, 0, 0}, {"summary", (void *) &lxb_tag_res_data_default[173], 7, 0}, 
    {"mfenced", (void *) &lxb_tag_res_data_default[126], 7, 0}, {NULL, NULL, 0, 0}, 
    {"sub", (void *) &lxb_tag_res_data_default[172], 3, 0}, {"colgroup", (void *) &lxb_tag_res_data_default[41], 8, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {"dd", (void *) &lxb_tag_res_data_default[44], 2, 103}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {"div", (void *) &lxb_tag_res_data_default[51], 3, 0}, {"textarea", (void *) &lxb_tag_res_data_default[180], 8, 0}, 
    {"!doctype", (void *) &lxb_tag_res_data_default[5], 8, 0}, {"applet", (void *) &lxb_tag_res_data_default[17], 6, 0}, 
    {NULL, NULL, 0, 0}, {"br", (void *) &lxb_tag_res_data_default[32], 2, 110}, 
    {NULL, NULL, 0, 0}, {"keygen", (void *) &lxb_tag_res_data_default[110], 6, 0}, 
    {"kbd", (void *) &lxb_tag_res_data_default[109], 3, 0}, {NULL, NULL, 0, 0}, 
    {"plaintext", (void *) &lxb_tag_res_data_default[149], 9, 0}, {"s", (void *) &lxb_tag_res_data_default[159], 1, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {"bdo", (void *) &lxb_tag_res_data_default[26], 3, 0}, {"td", (void *) &lxb_tag_res_data_default[178], 2, 0}, 
    {"fefunca", (void *) &lxb_tag_res_data_default[66], 7, 0}, {"ol", (void *) &lxb_tag_res_data_default[141], 2, 0}, 
    {"thead", (void *) &lxb_tag_res_data_default[184], 5, 0}, {"nobr", (void *) &lxb_tag_res_data_default[136], 4, 112}, 
    {NULL, NULL, 0, 0}, {"tr", (void *) &lxb_tag_res_data_default[187], 2, 0}, 
    {"map", (void *) &lxb_tag_res_data_default[119], 3, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {"#undef", (void *) &lxb_tag_res_data_default[0], 6, 113}, 
    {"em", (void *) &lxb_tag_res_data_default[54], 2, 0}, {NULL, NULL, 0, 0}, 
    {"bdi", (void *) &lxb_tag_res_data_default[25], 3, 0}, {"femorphology", (void *) &lxb_tag_res_data_default[74], 12, 0}, 
    {"ms", (void *) &lxb_tag_res_data_default[131], 2, 116}, {"footer", (void *) &lxb_tag_res_data_default[85], 6, 0}, 
    {"fefuncr", (void *) &lxb_tag_res_data_default[69], 7, 0}, {"rt", (void *) &lxb_tag_res_data_default[156], 2, 117}, 
    {NULL, NULL, 0, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}, {"h5", (void *) &lxb_tag_res_data_default[95], 2, 0}, 
    {NULL, NULL, 0, 0}, {"ruby", (void *) &lxb_tag_res_data_default[158], 4, 120}, 
    {"canvas", (void *) &lxb_tag_res_data_default[34], 6, 0}, {NULL, NULL, 0, 0}, 
    {NULL, NULL, 0, 0}
};

