/*
 Copyright (C) 2016-2017 Alexander Borisov

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

 Author: lex.borisov@gmail.com (Alexander Borisov)
*/

#ifndef MyCSS_PROPERTY_RESOURCES_NAME_H
#define MyCSS_PROPERTY_RESOURCES_NAME_H
#pragma once

static const char * mycss_property_index_type_name[] = 
{
	"",
	"align-content",
	"align-items",
	"align-self",
	"animation",
	"animation-delay",
	"animation-direction",
	"animation-duration",
	"animation-fill-mode",
	"animation-iteration-count",
	"animation-name",
	"animation-play-state",
	"animation-timing-function",
	"appearance",
	"backface-visibility",
	"background",
	"background-attachment",
	"background-clip",
	"background-color",
	"background-image",
	"background-origin",
	"background-position",
	"background-repeat",
	"background-size",
	"bookmark-label",
	"bookmark-level",
	"bookmark-state",
	"border",
	"border-block-end",
	"border-block-end-color",
	"border-block-end-style",
	"border-block-end-width",
	"border-block-start",
	"border-block-start-color",
	"border-block-start-style",
	"border-block-start-width",
	"border-bottom",
	"border-bottom-color",
	"border-bottom-left-radius",
	"border-bottom-right-radius",
	"border-bottom-style",
	"border-bottom-width",
	"border-collapse",
	"border-color",
	"border-image",
	"border-image-outset",
	"border-image-repeat",
	"border-image-slice",
	"border-image-width",
	"border-inline-end",
	"border-inline-end-color",
	"border-inline-end-style",
	"border-inline-end-width",
	"border-inline-start",
	"border-inline-start-color",
	"border-inline-start-style",
	"border-inline-start-width",
	"border-left",
	"border-left-color",
	"border-left-style",
	"border-left-width",
	"border-radius",
	"border-right",
	"border-right-color",
	"border-right-style",
	"border-right-width",
	"border-spacing",
	"border-style",
	"border-top",
	"border-top-color",
	"border-top-left-radius",
	"border-top-right-radius",
	"border-top-style",
	"border-top-width",
	"border-width",
	"bottom",
	"box-decoration-break",
	"box-shadow",
	"box-sizing",
	"box-suppress",
	"break-after",
	"break-before",
	"break-inside",
	"caption-side",
	"caret",
	"caret-color",
	"caret-shape",
	"clear",
	"clear-after",
	"color",
	"color-adjust",
	"color-interpolation",
	"color-interpolation-filters",
	"color-rendering",
	"column-count",
	"column-fill",
	"column-gap",
	"column-rule",
	"column-rule-color",
	"column-rule-style",
	"column-rule-width",
	"column-span",
	"column-width",
	"columns",
	"contain",
	"content",
	"continue",
	"counter-increment",
	"cue",
	"cue-after",
	"cue-before",
	"cursor",
	"direction",
	"display",
	"empty-cells",
	"fill",
	"fill-opacity",
	"fill-rule",
	"flex",
	"flex-basis",
	"flex-direction",
	"flex-flow",
	"flex-grow",
	"flex-shrink",
	"flex-wrap",
	"float",
	"float-displace",
	"font",
	"font-family",
	"font-feature-settings",
	"font-kerning",
	"font-language-override",
	"font-size",
	"font-size-adjust",
	"font-stretch",
	"font-style",
	"font-synthesis",
	"font-variant",
	"font-variant-alternates",
	"font-variant-caps",
	"font-variant-east-asian",
	"font-variant-ligatures",
	"font-variant-numeric",
	"font-variant-position",
	"font-weight",
	"glyph-orientation-vertical",
	"hanging-punctuation",
	"height",
	"hyphens",
	"image-rendering",
	"indent-edge-reset",
	"inline-size-step",
	"justify-content",
	"left",
	"letter-spacing",
	"line-break",
	"line-height",
	"line-height-step",
	"list-style",
	"list-style-image",
	"list-style-position",
	"list-style-type",
	"margin",
	"margin-block-end",
	"margin-block-start",
	"margin-bottom",
	"margin-inline-end",
	"margin-inline-start",
	"margin-left",
	"margin-right",
	"margin-top",
	"marker",
	"marker-end",
	"marker-mid",
	"marker-side",
	"marker-start",
	"max-height",
	"max-lines",
	"max-width",
	"min-height",
	"min-width",
	"nav-down",
	"nav-left",
	"nav-right",
	"nav-up",
	"offset-after",
	"offset-before",
	"offset-end",
	"offset-start",
	"opacity",
	"order",
	"orphans",
	"outline",
	"outline-color",
	"outline-offset",
	"outline-style",
	"outline-width",
	"overflow",
	"overflow-wrap",
	"overflow-x",
	"overflow-y",
	"padding",
	"padding-block-end",
	"padding-block-start",
	"padding-bottom",
	"padding-inline-end",
	"padding-inline-start",
	"padding-left",
	"padding-right",
	"padding-top",
	"pause",
	"pause-after",
	"pause-before",
	"perspective",
	"perspective-origin",
	"position",
	"presentation-level",
	"quotes",
	"region-fragment",
	"resize",
	"rest",
	"rest-after",
	"rest-before",
	"right",
	"ruby-align",
	"ruby-merge",
	"ruby-position",
	"scroll-padding",
	"scroll-snap-align",
	"scroll-snap-margin",
	"scroll-snap-stop",
	"scroll-snap-type",
	"shape-image-threshold",
	"shape-margin",
	"shape-outside",
	"shape-rendering",
	"speak",
	"speak-as",
	"string-set",
	"stroke",
	"stroke-dasharray",
	"stroke-dashoffset",
	"stroke-linecap",
	"stroke-linejoin",
	"stroke-miterlimit",
	"stroke-opacity",
	"stroke-width",
	"tab-size",
	"table-layout",
	"text-align",
	"text-align-all",
	"text-align-last",
	"text-combine-upright",
	"text-decoration",
	"text-decoration-color",
	"text-decoration-line",
	"text-decoration-skip",
	"text-decoration-style",
	"text-emphasis",
	"text-emphasis-color",
	"text-emphasis-position",
	"text-emphasis-style",
	"text-indent",
	"text-justify",
	"text-orientation",
	"text-overflow",
	"text-rendering",
	"text-shadow",
	"text-size-adjust",
	"text-transform",
	"text-underline-position",
	"top",
	"touch-action",
	"transform",
	"transform-box",
	"transform-origin",
	"transform-style",
	"transition",
	"transition-delay",
	"transition-duration",
	"transition-property",
	"transition-timing-function",
	"unicode-bidi",
	"user-select",
	"vertical-align",
	"visibility",
	"voice-balance",
	"voice-duration",
	"voice-family",
	"voice-pitch",
	"voice-range",
	"voice-rate",
	"voice-stress",
	"voice-volume",
	"white-space",
	"widows",
	"width",
	"will-change",
	"word-break",
	"word-spacing",
	"word-wrap",
	"wrap-flow",
	"wrap-through",
	"writing-mode",
	"z-index"
};

static const char * mycss_property_index_type_value[] = 
{
	"",
	"0",
	"0deg",
	"100",
	"200",
	"300",
	"400",
	"500",
	"600",
	"700",
	"800",
	"90",
	"900",
	"90deg",
	"absolute",
	"alias",
	"all",
	"allow-end",
	"all-petite-caps",
	"all-scroll",
	"all-small-caps",
	"alternate",
	"alternate-reverse",
	"always",
	"auto",
	"available",
	"avoid",
	"avoid-column",
	"avoid-page",
	"avoid-region",
	"backwards",
	"balance",
	"balance-all",
	"bar",
	"baseline",
	"bevel",
	"bidi-override",
	"blink",
	"block",
	"block-within-page",
	"bold",
	"bolder",
	"border-box",
	"border-edge",
	"both",
	"bottom",
	"box-decoration",
	"break",
	"break-all",
	"break-spaces",
	"break-word",
	"butt",
	"capitalize",
	"caption",
	"cell",
	"center",
	"circle",
	"clear",
	"clip",
	"clone",
	"closed",
	"collapse",
	"column",
	"column-reverse",
	"col-resize",
	"common-ligatures",
	"complex",
	"condensed",
	"contain",
	"content",
	"contents",
	"content-box",
	"content-edge",
	"contextual",
	"context-menu",
	"copy",
	"cover",
	"crispedges",
	"crosshair",
	"current",
	"currentcolor",
	"cursive",
	"dashed",
	"default",
	"descendants",
	"diagonal-fractions",
	"digits",
	"discard",
	"discretionary-ligatures",
	"dot",
	"dotted",
	"double",
	"double-circle",
	"each-line",
	"ease",
	"ease-in",
	"ease-in-out",
	"ease-out",
	"economy",
	"edges",
	"ellipsis",
	"embed",
	"end",
	"evenodd",
	"ew-resize",
	"exact",
	"expanded",
	"extra-condensed",
	"extra-expanded",
	"e-resize",
	"fantasy",
	"fast",
	"fill",
	"filled",
	"fill-box",
	"first",
	"first-except",
	"fit-content",
	"fixed",
	"flat",
	"flex",
	"flex-end",
	"flex-start",
	"flow",
	"flow-root",
	"force-end",
	"forwards",
	"fragments",
	"full-width",
	"geometricprecision",
	"grab",
	"grabbing",
	"grid",
	"groove",
	"hanging",
	"help",
	"hidden",
	"hide",
	"high",
	"historical-forms",
	"historical-ligatures",
	"horizontal",
	"horizontal-tb",
	"icon",
	"increment",
	"indent",
	"inherit",
	"initial",
	"ink",
	"inline",
	"inline-block",
	"inline-flex",
	"inline-grid",
	"inline-list-item",
	"inline-table",
	"inset",
	"inside",
	"inter-character",
	"inter-word",
	"invert",
	"isolate",
	"isolate-override",
	"italic",
	"jis04",
	"jis78",
	"jis83",
	"jis90",
	"justify",
	"justify-all",
	"keep-all",
	"large",
	"larger",
	"last",
	"layout",
	"left",
	"leftwards",
	"lighter",
	"line",
	"linear",
	"linearrgb",
	"line-through",
	"lining-nums",
	"list-container",
	"list-item",
	"literal-punctuation",
	"local",
	"loose",
	"loud",
	"low",
	"lowercase",
	"ltr",
	"mandatory",
	"manipulation",
	"manual",
	"margin-edge",
	"match-parent",
	"maximum",
	"max-content",
	"medium",
	"menu",
	"message-box",
	"middle",
	"minimum",
	"min-content",
	"miter",
	"mixed",
	"moderate",
	"monospace",
	"move",
	"nesw-resize",
	"ne-resize",
	"none",
	"nonzero",
	"normal",
	"not-allowed",
	"nowrap",
	"no-common-ligatures",
	"no-content",
	"no-contextual",
	"no-discretionary-ligatures",
	"no-display",
	"no-drop",
	"no-historical-ligatures",
	"no-punctuation",
	"no-repeat",
	"ns-resize",
	"nwse-resize",
	"nw-resize",
	"n-resize",
	"objects",
	"oblique",
	"off",
	"oldstyle-nums",
	"on",
	"open",
	"optimizelegibility",
	"optimizequality",
	"optimizespeed",
	"ordinal",
	"outset",
	"outside",
	"over",
	"overflow",
	"overline",
	"padding-box",
	"padding-edge",
	"page",
	"paginate",
	"paint",
	"pan-down",
	"pan-left",
	"pan-right",
	"pan-up",
	"pan-x",
	"pan-y",
	"paused",
	"petite-caps",
	"plaintext",
	"pointer",
	"pre",
	"preserve",
	"preserve-3d",
	"pre-line",
	"pre-wrap",
	"progress",
	"proportional-nums",
	"proportional-width",
	"proximity",
	"recto",
	"reduced",
	"region",
	"relative",
	"repeat",
	"repeat-x",
	"repeat-y",
	"reverse",
	"ridge",
	"right",
	"rightwards",
	"root",
	"round",
	"row",
	"row-resize",
	"row-reverse",
	"rtl",
	"ruby",
	"ruby-base",
	"ruby-base-container",
	"ruby-text",
	"ruby-text-container",
	"running",
	"run-in",
	"same",
	"sans-serif",
	"scroll",
	"scroll-position",
	"semi-condensed",
	"semi-expanded",
	"separate",
	"serif",
	"sesameinitial",
	"se-resize",
	"show",
	"sideways",
	"sideways-lr",
	"sideways-rl",
	"silent",
	"simplified",
	"size",
	"slashed-zero",
	"slice",
	"slow",
	"small",
	"smaller",
	"small-caps",
	"small-caption",
	"soft",
	"solid",
	"space",
	"spaces",
	"space-around",
	"space-between",
	"spell-out",
	"square",
	"srgb",
	"stacked-fractions",
	"start",
	"static",
	"status-bar",
	"step-end",
	"step-start",
	"sticky",
	"stretch",
	"strict",
	"strong",
	"style",
	"sub",
	"super",
	"sw-resize",
	"s-resize",
	"table",
	"table-caption",
	"table-cell",
	"table-column",
	"table-column-group",
	"table-footer-group",
	"table-header-group",
	"table-row",
	"table-row-group",
	"tabular-nums",
	"text",
	"text-bottom",
	"text-top",
	"thick",
	"thin",
	"titling-caps",
	"top",
	"traditional",
	"transparent",
	"triangle",
	"ultra-condensed",
	"ultra-expanded",
	"under",
	"underline",
	"underscore",
	"unicase",
	"unse",
	"unset",
	"uppercase",
	"upright",
	"verso",
	"vertical",
	"vertical-lr",
	"vertical-rl",
	"vertical-text",
	"view-box",
	"visible",
	"wait",
	"wavy",
	"weak",
	"weight",
	"wrap",
	"wrap-reverse",
	"w-resize",
	"x",
	"xx-large",
	"xx-small",
	"x-fast",
	"x-high",
	"x-large",
	"x-loud",
	"x-low",
	"x-slow",
	"x-small",
	"x-soft",
	"x-strong",
	"x-weak",
	"y",
	"zoom-in",
	"zoom-out",
	"<color>",
	"<cross-fade-function>",
	"<custom-ident>",
	"<element-function>",
	"<family-name>",
	"<generic-family>",
	"<image>",
	"<image-function>",
	"<image-list>",
	"<image-set-function>",
	"<length>",
	"<linear-gradient-function>",
	"<number>",
	"<percentage>",
	"<radial-gradient-function>",
	"<resolution>",
	"<url>",
	""
};


#endif /* MyCSS_PROPERTY_RESOURCES_NAME_H */
