/*
 Copyright (C) 2016-2017 Alexander Borisov

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

 Author: lex.borisov@gmail.com (Alexander Borisov)
*/

#ifndef MyCSS_DECLARATION_DEFAULT_RESOURCES_H
#define MyCSS_DECLARATION_DEFAULT_RESOURCES_H
#pragma once

#include "mycss/declaration/default_entries.h"

static mycss_declaration_entry_t * mycss_declaration_default_entry_index_type[MyCSS_PROPERTY_TYPE_LAST_ENTRY] = 
{
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_background_attachment,
	&mycss_declaration_default_entry_background_clip,
	&mycss_declaration_default_entry_background_color,
	&mycss_declaration_default_entry_background_image,
	&mycss_declaration_default_entry_background_origin,
	&mycss_declaration_default_entry_background_position,
	&mycss_declaration_default_entry_background_repeat,
	&mycss_declaration_default_entry_background_size,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_border_bottom_color,
	&mycss_declaration_default_entry_border_bottom_left_radius,
	&mycss_declaration_default_entry_border_bottom_right_radius,
	&mycss_declaration_default_entry_border_bottom_style,
	&mycss_declaration_default_entry_border_bottom_width,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_border_left_color,
	&mycss_declaration_default_entry_border_left_style,
	&mycss_declaration_default_entry_border_left_width,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_border_right_color,
	&mycss_declaration_default_entry_border_right_style,
	&mycss_declaration_default_entry_border_right_width,
	&mycss_declaration_default_entry_border_spacing,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_border_top_color,
	&mycss_declaration_default_entry_border_top_left_radius,
	&mycss_declaration_default_entry_border_top_right_radius,
	&mycss_declaration_default_entry_border_top_style,
	&mycss_declaration_default_entry_border_top_width,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_bottom,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_box_sizing,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_clear,
	&mycss_declaration_default_entry_clear_after,
	&mycss_declaration_default_entry_color,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_cursor,
	&mycss_declaration_default_entry_direction,
	&mycss_declaration_default_entry_display,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_float,
	&mycss_declaration_default_entry_float_displace,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_font_size,
	&mycss_declaration_default_entry_font_size_adjust,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_font_style,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_font_weight,
	&mycss_declaration_default_entry_glyph_orientation_vertical,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_height,
	&mycss_declaration_default_entry_hyphens,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_left,
	&mycss_declaration_default_entry_letter_spacing,
	&mycss_declaration_default_entry_line_break,
	&mycss_declaration_default_entry_line_height,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_margin_bottom,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_margin_left,
	&mycss_declaration_default_entry_margin_right,
	&mycss_declaration_default_entry_margin_top,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_max_height,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_max_width,
	&mycss_declaration_default_entry_min_height,
	&mycss_declaration_default_entry_min_width,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_overflow_wrap,
	&mycss_declaration_default_entry_overflow_x,
	&mycss_declaration_default_entry_overflow_y,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_padding_bottom,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_padding_left,
	&mycss_declaration_default_entry_padding_right,
	&mycss_declaration_default_entry_padding_top,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_position,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_right,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_tab_size,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_text_align,
	&mycss_declaration_default_entry_text_align_all,
	&mycss_declaration_default_entry_text_align_last,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_text_decoration_color,
	&mycss_declaration_default_entry_text_decoration_line,
	&mycss_declaration_default_entry_text_decoration_skip,
	&mycss_declaration_default_entry_text_decoration_style,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_text_justify,
	&mycss_declaration_default_entry_text_orientation,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_text_transform,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_top,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_unicode_bidi,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_vertical_align,
	&mycss_declaration_default_entry_visibility,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_white_space,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_width,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_word_break,
	&mycss_declaration_default_entry_word_spacing,
	&mycss_declaration_default_entry_word_wrap,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_undef,
	&mycss_declaration_default_entry_writing_mode,
	&mycss_declaration_default_entry_undef
};


#endif /* MyCSS_DECLARATION_DEFAULT_RESOURCES_H */
