/*
 * Copyright (C) 2020-2021 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/css/selectors/pseudo.py"!
 * Do not change this file!
 */


#ifndef LXB_CSS_SELECTOS_PSEUDO_CONST_H
#define LXB_CSS_SELECTOS_PSEUDO_CONST_H


typedef enum {
    LXB_CSS_SELECTOR_PSEUDO_CLASS__UNDEF            = 0x0000,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_ACTIVE            = 0x0001,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_ANY_LINK          = 0x0002,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_BLANK             = 0x0003,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_CHECKED           = 0x0004,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_CURRENT           = 0x0005,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_DEFAULT           = 0x0006,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_DISABLED          = 0x0007,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_EMPTY             = 0x0008,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_ENABLED           = 0x0009,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FIRST_CHILD       = 0x000a,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FIRST_OF_TYPE     = 0x000b,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FOCUS             = 0x000c,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FOCUS_VISIBLE     = 0x000d,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FOCUS_WITHIN      = 0x000e,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FULLSCREEN        = 0x000f,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUTURE            = 0x0010,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_HOVER             = 0x0011,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_IN_RANGE          = 0x0012,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_INDETERMINATE     = 0x0013,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_INVALID           = 0x0014,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_LAST_CHILD        = 0x0015,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_LAST_OF_TYPE      = 0x0016,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_LINK              = 0x0017,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_LOCAL_LINK        = 0x0018,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_ONLY_CHILD        = 0x0019,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_ONLY_OF_TYPE      = 0x001a,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_OPTIONAL          = 0x001b,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_OUT_OF_RANGE      = 0x001c,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_PAST              = 0x001d,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_PLACEHOLDER_SHOWN = 0x001e,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_READ_ONLY         = 0x001f,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_READ_WRITE        = 0x0020,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_REQUIRED          = 0x0021,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_ROOT              = 0x0022,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_SCOPE             = 0x0023,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_TARGET            = 0x0024,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_TARGET_WITHIN     = 0x0025,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_USER_INVALID      = 0x0026,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_VALID             = 0x0027,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_VISITED           = 0x0028,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_WARNING           = 0x0029,
    LXB_CSS_SELECTOR_PSEUDO_CLASS__LAST_ENTRY       = 0x002a
}
lxb_css_selector_pseudo_class_id_t;

typedef enum {
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION__UNDEF           = 0x0000,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_CURRENT          = 0x0001,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_DIR              = 0x0002,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_HAS              = 0x0003,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_IS               = 0x0004,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_LANG             = 0x0005,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_NOT              = 0x0006,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_NTH_CHILD        = 0x0007,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_NTH_COL          = 0x0008,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_NTH_LAST_CHILD   = 0x0009,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_NTH_LAST_COL     = 0x000a,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_NTH_LAST_OF_TYPE = 0x000b,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_NTH_OF_TYPE      = 0x000c,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION_WHERE            = 0x000d,
    LXB_CSS_SELECTOR_PSEUDO_CLASS_FUNCTION__LAST_ENTRY      = 0x000e
}
lxb_css_selector_pseudo_class_function_id_t;

typedef enum {
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT__UNDEF             = 0x0000,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_AFTER              = 0x0001,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_BACKDROP           = 0x0002,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_BEFORE             = 0x0003,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_FIRST_LETTER       = 0x0004,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_FIRST_LINE         = 0x0005,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_GRAMMAR_ERROR      = 0x0006,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_INACTIVE_SELECTION = 0x0007,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_MARKER             = 0x0008,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_PLACEHOLDER        = 0x0009,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_SELECTION          = 0x000a,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_SPELLING_ERROR     = 0x000b,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_TARGET_TEXT        = 0x000c,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT__LAST_ENTRY        = 0x000d
}
lxb_css_selector_pseudo_element_id_t;

typedef enum {
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_FUNCTION__UNDEF      = 0x0000,
    LXB_CSS_SELECTOR_PSEUDO_ELEMENT_FUNCTION__LAST_ENTRY = 0x0001
}
lxb_css_selector_pseudo_element_function_id_t;


#endif /* LXB_CSS_SELECTOS_PSEUDO_CONST_H */
