/*
 * Copyright (C) 2019-2020 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/encoding/res.py"!
 * Do not change this file!
 */


#ifndef LEXBOR_ENCODING_RES_H
#define LEXBOR_ENCODING_RES_H

#ifdef __cplusplus
extern "C" {
#endif


#include "lexbor/encoding/base.h"
#include "lexbor/core/shs.h"


LXB_EXTERN const lxb_encoding_data_t lxb_encoding_res_map[LXB_ENCODING_LAST_ENTRY];
LXB_EXTERN const lexbor_shs_entry_t lxb_encoding_res_shs_entities[220];


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* LEXBOR_ENCODING_RES_H */
