/*
 Copyright (C) 2016-2017 Alexander Borisov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 Author: lex.borisov@gmail.com (Alexander Borisov)
*/

#ifndef MyHTML_MyCSS_DECLARATION_INIT_H
#define MyHTML_MyCSS_DECLARATION_INIT_H
#pragma once

#include <mycss/declaration/myosi.h>
#include <mycss/entry.h>

#ifdef __cplusplus
extern "C" {
#endif

mycss_declaration_t * mycss_declaration_create(void);
mystatus_t  mycss_declaration_init(mycss_entry_t* entry, mycss_declaration_t* declaration);
void  mycss_declaration_clean_all(mycss_declaration_t* declaration);
void mycss_declaration_clean(mycss_declaration_t* declaration);
mycss_declaration_t * mycss_declaration_destroy(mycss_declaration_t* declaration, bool self_destroy);

void mycss_declaration_entry_clean(mycss_declaration_entry_t* entry);

mycss_declaration_entry_t * mycss_declaration_entry_create(mycss_declaration_t* declaration, mystatus_t* status);
void mycss_declaration_entry_clean_all(mycss_declaration_entry_t* entry);
void mycss_declaration_entry_append_to_current(mycss_declaration_t* declaration, mycss_declaration_entry_t* dec_entry);

mycss_declaration_entry_t * mycss_declaration_parse(mycss_declaration_t* declaration, myencoding_t encoding, const char* data, size_t data_size, mystatus_t* out_status);

mycss_token_type_t mycss_declaration_ending_token_type(mycss_declaration_t* declaration);
void mycss_declaration_ending_token_type_set(mycss_declaration_t* declaration, mycss_token_type_t ending_token_type);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* MyHTML_MyCSS_DECLARATION_INIT_H */

