/*
 * Copyright (C) 2019-2020 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/dom/attr.py"!
 * Do not change this file!
 */


#ifndef LXB_DOM_ATTR_CONST_H
#define LXB_DOM_ATTR_CONST_H


typedef uintptr_t lxb_dom_attr_id_t;

typedef enum {
    LXB_DOM_ATTR__UNDEF      = 0x0000,
    LXB_DOM_ATTR_ACTIVE      = 0x0001,
    LXB_DOM_ATTR_ALT         = 0x0002,
    LXB_DOM_ATTR_CHARSET     = 0x0003,
    LXB_DOM_ATTR_CHECKED     = 0x0004,
    LXB_DOM_ATTR_CLASS       = 0x0005,
    LXB_DOM_ATTR_COLOR       = 0x0006,
    LXB_DOM_ATTR_CONTENT     = 0x0007,
    LXB_DOM_ATTR_DIR         = 0x0008,
    LXB_DOM_ATTR_DISABLED    = 0x0009,
    LXB_DOM_ATTR_FACE        = 0x000a,
    LXB_DOM_ATTR_FOCUS       = 0x000b,
    LXB_DOM_ATTR_FOR         = 0x000c,
    LXB_DOM_ATTR_HEIGHT      = 0x000d,
    LXB_DOM_ATTR_HOVER       = 0x000e,
    LXB_DOM_ATTR_HREF        = 0x000f,
    LXB_DOM_ATTR_HTML        = 0x0010,
    LXB_DOM_ATTR_HTTP_EQUIV  = 0x0011,
    LXB_DOM_ATTR_ID          = 0x0012,
    LXB_DOM_ATTR_IS          = 0x0013,
    LXB_DOM_ATTR_MAXLENGTH   = 0x0014,
    LXB_DOM_ATTR_PLACEHOLDER = 0x0015,
    LXB_DOM_ATTR_POOL        = 0x0016,
    LXB_DOM_ATTR_PUBLIC      = 0x0017,
    LXB_DOM_ATTR_READONLY    = 0x0018,
    LXB_DOM_ATTR_REQUIRED    = 0x0019,
    LXB_DOM_ATTR_SCHEME      = 0x001a,
    LXB_DOM_ATTR_SELECTED    = 0x001b,
    LXB_DOM_ATTR_SIZE        = 0x001c,
    LXB_DOM_ATTR_SLOT        = 0x001d,
    LXB_DOM_ATTR_SRC         = 0x001e,
    LXB_DOM_ATTR_STYLE       = 0x001f,
    LXB_DOM_ATTR_SYSTEM      = 0x0020,
    LXB_DOM_ATTR_TITLE       = 0x0021,
    LXB_DOM_ATTR_TYPE        = 0x0022,
    LXB_DOM_ATTR_WIDTH       = 0x0023,
    LXB_DOM_ATTR__LAST_ENTRY = 0x0024
}
lxb_dom_attr_id_enum_t;


#endif /* LXB_DOM_ATTR_CONST_H */
