/* Generated by Cython 3.1.2 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "modest/include/modest/finder/finder.h",
            "modest/include/mycss/mycss.h",
            "modest/include/myencoding/encoding.h",
            "modest/include/myhtml/myhtml.h",
            "modest/include/myhtml/serialization.h",
            "modest/include/myhtml/tree.h"
        ],
        "extra_compile_args": [
            "-DMODEST_BUILD_OS=Linux",
            "-DMyCORE_OS_Linux",
            "-DMODEST_PORT_NAME=posix",
            "-DMyCORE_BUILD_WITHOUT_THREADS=YES",
            "-DMyCORE_BUILD_DEBUG=NO",
            "-pedantic",
            "-fPIC",
            "-Wno-unused-variable",
            "-Wno-unused-function",
            "-std=c99",
            "-O2"
        ],
        "include_dirs": [
            "modest/include/"
        ],
        "language": "c",
        "name": "selectolax.parser",
        "sources": [
            "selectolax/parser.pyx",
            "modest/source/mycss/entry.c",
            "modest/source/mycss/tokenizer_end.c",
            "modest/source/mycss/parser.c",
            "modest/source/mycss/an_plus_b.c",
            "modest/source/mycss/stack.c",
            "modest/source/mycss/mystring.c",
            "modest/source/mycss/convert.c",
            "modest/source/mycss/tokenizer.c",
            "modest/source/mycss/stylesheet.c",
            "modest/source/mycss/tokenizer_global.c",
            "modest/source/mycss/check.c",
            "modest/source/mycss/mycss.c",
            "modest/source/mycss/namespace/state.c",
            "modest/source/mycss/namespace/parser.c",
            "modest/source/mycss/namespace/serialization.c",
            "modest/source/mycss/namespace/init.c",
            "modest/source/mycss/media/state.c",
            "modest/source/mycss/media/init.c",
            "modest/source/mycss/selectors/state.c",
            "modest/source/mycss/selectors/function_parser.c",
            "modest/source/mycss/selectors/parser.c",
            "modest/source/mycss/selectors/serialization.c",
            "modest/source/mycss/selectors/init.c",
            "modest/source/mycss/selectors/function.c",
            "modest/source/mycss/selectors/value.c",
            "modest/source/mycss/selectors/pseudo.c",
            "modest/source/mycss/selectors/list.c",
            "modest/source/mycss/declaration/entry.c",
            "modest/source/mycss/declaration/state.c",
            "modest/source/mycss/declaration/default.c",
            "modest/source/mycss/declaration/parser.c",
            "modest/source/mycss/declaration/serialization.c",
            "modest/source/mycss/declaration/init.c",
            "modest/source/mycss/declaration/entry_destroy.c",
            "modest/source/mycss/values/destroy.c",
            "modest/source/mycss/values/color_parser.c",
            "modest/source/mycss/values/serialization.c",
            "modest/source/mycss/values/image.c",
            "modest/source/mycss/values/consume.c",
            "modest/source/mycss/values/color.c",
            "modest/source/mycss/values/units.c",
            "modest/source/mycss/values/values.c",
            "modest/source/mycss/property/parser_url.c",
            "modest/source/mycss/property/parser.c",
            "modest/source/mycss/property/serialization.c",
            "modest/source/mycss/property/init.c",
            "modest/source/mycss/property/shared.c",
            "modest/source/mycss/property/parser_image.c",
            "modest/source/mycss/property/parser_text_decoration.c",
            "modest/source/mycss/property/parser_background.c",
            "modest/source/myhtml/tokenizer_end.c",
            "modest/source/myhtml/parser.c",
            "modest/source/myhtml/tag.c",
            "modest/source/myhtml/tag_init.c",
            "modest/source/myhtml/serialization.c",
            "modest/source/myhtml/tokenizer_doctype.c",
            "modest/source/myhtml/charef.c",
            "modest/source/myhtml/mynamespace.c",
            "modest/source/myhtml/mystring.c",
            "modest/source/myhtml/stream.c",
            "modest/source/myhtml/tokenizer.c",
            "modest/source/myhtml/tokenizer_script.c",
            "modest/source/myhtml/data_process.c",
            "modest/source/myhtml/tree.c",
            "modest/source/myhtml/myhtml.c",
            "modest/source/myhtml/rules.c",
            "modest/source/myhtml/callback.c",
            "modest/source/myhtml/token.c",
            "modest/source/modest/declaration.c",
            "modest/source/modest/glue.c",
            "modest/source/modest/modest.c",
            "modest/source/modest/node/serialization.c",
            "modest/source/modest/node/property.c",
            "modest/source/modest/node/raw_property.c",
            "modest/source/modest/node/node.c",
            "modest/source/modest/layer/layer.c",
            "modest/source/modest/style/default.c",
            "modest/source/modest/style/type.c",
            "modest/source/modest/style/map.c",
            "modest/source/modest/style/raw.c",
            "modest/source/modest/style/sheet.c",
            "modest/source/modest/finder/type.c",
            "modest/source/modest/finder/pseudo_class.c",
            "modest/source/modest/finder/thread.c",
            "modest/source/modest/finder/match.c",
            "modest/source/modest/finder/finder.c",
            "modest/source/modest/render/begin.c",
            "modest/source/modest/render/tree_node.c",
            "modest/source/modest/render/binding.c",
            "modest/source/modest/render/tree.c",
            "modest/source/myurl/path.c",
            "modest/source/myurl/parser_end.c",
            "modest/source/myurl/parser.c",
            "modest/source/myurl/serialization.c",
            "modest/source/myurl/host.c",
            "modest/source/myurl/punycode.c",
            "modest/source/myurl/scheme.c",
            "modest/source/myurl/myosi.c",
            "modest/source/myurl/url.c",
            "modest/source/myurl/utils.c",
            "modest/source/myunicode/myosi.c",
            "modest/source/mycore/thread_queue.c",
            "modest/source/mycore/mystring.c",
            "modest/source/mycore/mythread.c",
            "modest/source/mycore/myosi.c",
            "modest/source/mycore/incoming.c",
            "modest/source/mycore/utils.c",
            "modest/source/mycore/utils/mchar_async.c",
            "modest/source/mycore/utils/avl_tree.c",
            "modest/source/mycore/utils/mcsimple.c",
            "modest/source/mycore/utils/mcobject.c",
            "modest/source/mycore/utils/mcsync.c",
            "modest/source/mycore/utils/mctree.c",
            "modest/source/mycore/utils/mcobject_async.c",
            "modest/source/mycore/utils/mhash.c",
            "modest/source/myport/posix/mycore/memory.c",
            "modest/source/myport/posix/mycore/io.c",
            "modest/source/myport/posix/mycore/thread.c",
            "modest/source/myport/posix/mycore/perf.c",
            "modest/source/myport/posix/mycore/utils/mcsync.c",
            "modest/source/myfont/glyf.c",
            "modest/source/myfont/vmtx.c",
            "modest/source/myfont/vhea.c",
            "modest/source/myfont/os_2.c",
            "modest/source/myfont/hmtx.c",
            "modest/source/myfont/name.c",
            "modest/source/myfont/head.c",
            "modest/source/myfont/maxp.c",
            "modest/source/myfont/loca.c",
            "modest/source/myfont/myosi.c",
            "modest/source/myfont/pclt.c",
            "modest/source/myfont/myfont.c",
            "modest/source/myfont/cmap.c",
            "modest/source/myfont/hhea.c",
            "modest/source/myencoding/detect.c",
            "modest/source/myencoding/mystring.c",
            "modest/source/myencoding/encoding.c"
        ]
    },
    "module_name": "selectolax.parser"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API) && !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_1_2"
#define CYTHON_HEX_VERSION 0x030102F0
#define CYTHON_FUTURE_DIVISION 1
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_CLINE_IN_TRACEBACK
  #define CYTHON_CLINE_IN_TRACEBACK 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CInitCode */
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if __PYX_LIMITED_VERSION_HEX < 0x03090000
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyList_GetItemRef(o, i) (likely((i) >= 0) ? PySequence_GetItem(o, i) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_ITEM(o, i)
  #endif
#elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #else
    #define __Pyx_PyList_GetItemRef(o, i) __Pyx_XNewRef(PyList_GetItem(o, i))
  #endif
#else
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_NewRef(PyList_GET_ITEM(o, i))
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static CYTHON_INLINE PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__selectolax__parser
#define __PYX_HAVE_API__selectolax__parser
/* Early includes */
#include "myhtml/myhtml.h"
#include "myhtml/tree.h"
#include "myhtml/serialization.h"
#include "myencoding/encoding.h"
#include "mycss/mycss.h"
#include "modest/finder/finder.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#if __cplusplus > 201103L
#include <type_traits>
#endif
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "selectolax/parser.pyx",
  "selectolax/modest/selection.pxi",
  "<stringsource>",
  "selectolax/modest/node.pxi",
  "selectolax/modest/../utils.pxi",
  "selectolax/modest/util.pxi",
  "selectolax/utils.pxi",
  "selectolax/parser.pxd",
  "cpython/bool.pxd",
  "cpython/type.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* Atomics.proto */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif
#if CYTHON_ATOMICS
    #define __pyx_add_acquisition_count(memview)\
             __pyx_atomic_incr_relaxed(__pyx_get_slice_count_pointer(memview))
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_atomic_decr_acq_rel(__pyx_get_slice_count_pointer(memview))
#else
    #define __pyx_add_acquisition_count(memview)\
            __pyx_add_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_sub_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* CriticalSections.proto */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_Begin1(cs, arg) (void)cs
#define __Pyx_PyCriticalSection_Begin2(cs, arg1, arg2) (void)cs
#define __Pyx_PyCriticalSection_End1(cs)
#define __Pyx_PyCriticalSection_End2(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_Begin1 PyCriticalSection_Begin
#define __Pyx_PyCriticalSection_Begin2 PyCriticalSection2_Begin
#define __Pyx_PyCriticalSection_End1 PyCriticalSection_End
#define __Pyx_PyCriticalSection_End2 PyCriticalSection2_End
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_10selectolax_6parser_HTMLParser;
struct __pyx_obj_10selectolax_6parser_Stack;
struct __pyx_obj_10selectolax_6parser_CSSSelector;
struct __pyx_obj_10selectolax_6parser_Selector;
struct __pyx_obj_10selectolax_6parser__Attributes;
struct __pyx_obj_10selectolax_6parser_Node;
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__;
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items;
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values;
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter;
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse;
struct __pyx_opt_args_10selectolax_6parser_4Node__text_deep;
struct __pyx_opt_args_10selectolax_6parser_append_text;

/* "selectolax/modest/node.pxi":293
 *         return text
 * 
 *     cdef inline _text_deep(self, myhtml_tree_node_t *node, separator='', strip=False):             # <<<<<<<<<<<<<<
 *         text = ""
 *         cdef Stack stack = Stack(_STACK_SIZE)
*/
struct __pyx_opt_args_10selectolax_6parser_4Node__text_deep {
  int __pyx_n;
  PyObject *separator;
  PyObject *strip;
};

/* "selectolax/modest/node.pxi":973
 *                 stack.push(current_node.child)
 * 
 * cdef inline str append_text(str text, str node_text, str separator='', bint strip=False):             # <<<<<<<<<<<<<<
 *     if strip:
 *         text += node_text.strip() + separator
*/
struct __pyx_opt_args_10selectolax_6parser_append_text {
  int __pyx_n;
  PyObject *separator;
  int strip;
};

/* "selectolax/parser.pxd":550
 * 
 * 
 * cdef class HTMLParser:             # <<<<<<<<<<<<<<
 *     cdef myhtml_tree_t *html_tree
 *     cdef public bint detect_encoding
*/
struct __pyx_obj_10selectolax_6parser_HTMLParser {
  PyObject_HEAD
  struct __pyx_vtabstruct_10selectolax_6parser_HTMLParser *__pyx_vtab;
  myhtml_tree_t *html_tree;
  int detect_encoding;
  int use_meta_tags;
  myencoding_t _encoding;
  PyObject *decode_errors;
  PyObject *raw_html;
  PyObject *cached_script_texts;
  PyObject *cached_script_srcs;
};


/* "selectolax/parser.pxd":570
 * 
 * 
 * cdef class Stack:             # <<<<<<<<<<<<<<
 *     cdef size_t capacity
 *     cdef size_t top
*/
struct __pyx_obj_10selectolax_6parser_Stack {
  PyObject_HEAD
  struct __pyx_vtabstruct_10selectolax_6parser_Stack *__pyx_vtab;
  size_t capacity;
  size_t top;
  myhtml_tree_node_t **_stack;
};


/* "selectolax/modest/selection.pxi":5
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class CSSSelector:
 * 
*/
struct __pyx_obj_10selectolax_6parser_CSSSelector {
  PyObject_HEAD
  struct __pyx_vtabstruct_10selectolax_6parser_CSSSelector *__pyx_vtab;
  char *c_selector;
  mycss_entry_t *css_entry;
  modest_finder_t *finder;
  mycss_selectors_list_t *selectors_list;
};


/* "selectolax/modest/selection.pxi":70
 * 
 * 
 * cdef class Selector:             # <<<<<<<<<<<<<<
 *     """An advanced CSS selector that supports additional operations.
 * 
*/
struct __pyx_obj_10selectolax_6parser_Selector {
  PyObject_HEAD
  struct __pyx_vtabstruct_10selectolax_6parser_Selector *__pyx_vtab;
  struct __pyx_obj_10selectolax_6parser_Node *node;
  PyObject *nodes;
};


/* "selectolax/modest/node.pxi":47
 *         return 0
 * 
 * cdef class _Attributes:             # <<<<<<<<<<<<<<
 *     """A dict-like object that represents attributes."""
 *     cdef myhtml_tree_node_t * node
*/
struct __pyx_obj_10selectolax_6parser__Attributes {
  PyObject_HEAD
  struct __pyx_vtabstruct_10selectolax_6parser__Attributes *__pyx_vtab;
  myhtml_tree_node_t *node;
  PyObject *decode_errors;
};


/* "selectolax/modest/node.pxi":144
 *     Node
 * 
 * cdef class Node:             # <<<<<<<<<<<<<<
 *     """A class that represents HTML node (element)."""
 *     cdef myhtml_tree_node_t *node
*/
struct __pyx_obj_10selectolax_6parser_Node {
  PyObject_HEAD
  struct __pyx_vtabstruct_10selectolax_6parser_Node *__pyx_vtab;
  myhtml_tree_node_t *node;
  struct __pyx_obj_10selectolax_6parser_HTMLParser *parser;
};


/* "selectolax/modest/node.pxi":59
 *         return obj
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef myhtml_tree_attr_t *attr = myhtml_node_attribute_first(self.node)
 *         while attr:
*/
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ {
  PyObject_HEAD
  myhtml_tree_attr_t *__pyx_v_attr;
  PyObject *__pyx_v_key;
  struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self;
};


/* "selectolax/modest/node.pxi":101
 *         return self.__iter__()
 * 
 *     def items(self):             # <<<<<<<<<<<<<<
 *         for key in self.__iter__():
 *             yield key, self[key]
*/
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items {
  PyObject_HEAD
  PyObject *__pyx_v_key;
  struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "selectolax/modest/node.pxi":105
 *             yield key, self[key]
 * 
 *     def values(self):             # <<<<<<<<<<<<<<
 *         for key in self.__iter__():
 *             yield self[key]
*/
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values {
  PyObject_HEAD
  PyObject *__pyx_v_key;
  struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "selectolax/modest/node.pxi":328
 *         return text
 * 
 *     def iter(self, include_text=False):             # <<<<<<<<<<<<<<
 *         """Iterate over nodes on the current level.
 * 
*/
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter {
  PyObject_HEAD
  PyObject *__pyx_v_include_text;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_next_node;
  myhtml_tree_node_t *__pyx_v_node;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self;
};


/* "selectolax/modest/node.pxi":353
 *             node = node.next
 * 
 *     def traverse(self, include_text=False):             # <<<<<<<<<<<<<<
 *         """Iterate over all child and next nodes starting from the current level.
 * 
*/
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse {
  PyObject_HEAD
  myhtml_tree_node_t *__pyx_v_current_node;
  PyObject *__pyx_v_include_text;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_next_node;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self;
  struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_stack;
};



/* "selectolax/parser.pyx":10
 * include "utils.pxi"
 * 
 * cdef class HTMLParser:             # <<<<<<<<<<<<<<
 *     """The HTML parser.
 * 
*/

struct __pyx_vtabstruct_10selectolax_6parser_HTMLParser {
  void (*_detect_encoding)(struct __pyx_obj_10selectolax_6parser_HTMLParser *, char *, size_t);
  int (*_parse_html)(struct __pyx_obj_10selectolax_6parser_HTMLParser *, char *, size_t);
  struct __pyx_obj_10selectolax_6parser_HTMLParser *(*from_tree)(myhtml_tree_t *, PyObject *, int, int, PyObject *, myencoding_t);
};
static struct __pyx_vtabstruct_10selectolax_6parser_HTMLParser *__pyx_vtabptr_10selectolax_6parser_HTMLParser;


/* "selectolax/modest/node.pxi":13
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class Stack:
 *     def __cinit__(self, size_t capacity=25):
*/

struct __pyx_vtabstruct_10selectolax_6parser_Stack {
  int (*is_empty)(struct __pyx_obj_10selectolax_6parser_Stack *);
  int (*push)(struct __pyx_obj_10selectolax_6parser_Stack *, myhtml_tree_node_t *);
  myhtml_tree_node_t *(*pop)(struct __pyx_obj_10selectolax_6parser_Stack *);
  int (*resize)(struct __pyx_obj_10selectolax_6parser_Stack *);
};
static struct __pyx_vtabstruct_10selectolax_6parser_Stack *__pyx_vtabptr_10selectolax_6parser_Stack;
static int __pyx_f_10selectolax_6parser_5Stack_is_empty(struct __pyx_obj_10selectolax_6parser_Stack *);
static int __pyx_f_10selectolax_6parser_5Stack_push(struct __pyx_obj_10selectolax_6parser_Stack *, myhtml_tree_node_t *);
static myhtml_tree_node_t *__pyx_f_10selectolax_6parser_5Stack_pop(struct __pyx_obj_10selectolax_6parser_Stack *);
static int __pyx_f_10selectolax_6parser_5Stack_resize(struct __pyx_obj_10selectolax_6parser_Stack *);


/* "selectolax/modest/selection.pxi":5
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class CSSSelector:
 * 
*/

struct __pyx_vtabstruct_10selectolax_6parser_CSSSelector {
  myhtml_collection_t *(*find)(struct __pyx_obj_10selectolax_6parser_CSSSelector *, myhtml_tree_node_t *);
  int (*_create_css_parser)(struct __pyx_obj_10selectolax_6parser_CSSSelector *);
  int (*_prepare_selector)(struct __pyx_obj_10selectolax_6parser_CSSSelector *, mycss_entry_t *, char const *, size_t);
};
static struct __pyx_vtabstruct_10selectolax_6parser_CSSSelector *__pyx_vtabptr_10selectolax_6parser_CSSSelector;
static myhtml_collection_t *__pyx_f_10selectolax_6parser_11CSSSelector_find(struct __pyx_obj_10selectolax_6parser_CSSSelector *, myhtml_tree_node_t *);
static int __pyx_f_10selectolax_6parser_11CSSSelector__create_css_parser(struct __pyx_obj_10selectolax_6parser_CSSSelector *);
static int __pyx_f_10selectolax_6parser_11CSSSelector__prepare_selector(struct __pyx_obj_10selectolax_6parser_CSSSelector *, mycss_entry_t *, char const *, size_t);


/* "selectolax/modest/selection.pxi":70
 * 
 * 
 * cdef class Selector:             # <<<<<<<<<<<<<<
 *     """An advanced CSS selector that supports additional operations.
 * 
*/

struct __pyx_vtabstruct_10selectolax_6parser_Selector {
  PyObject *(*css)(struct __pyx_obj_10selectolax_6parser_Selector *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10selectolax_6parser_Selector *__pyx_vtabptr_10selectolax_6parser_Selector;


/* "selectolax/modest/node.pxi":47
 *         return 0
 * 
 * cdef class _Attributes:             # <<<<<<<<<<<<<<
 *     """A dict-like object that represents attributes."""
 *     cdef myhtml_tree_node_t * node
*/

struct __pyx_vtabstruct_10selectolax_6parser__Attributes {
  struct __pyx_obj_10selectolax_6parser__Attributes *(*create)(myhtml_tree_node_t *, PyObject *);
};
static struct __pyx_vtabstruct_10selectolax_6parser__Attributes *__pyx_vtabptr_10selectolax_6parser__Attributes;


/* "selectolax/modest/node.pxi":144
 *     Node
 * 
 * cdef class Node:             # <<<<<<<<<<<<<<
 *     """A class that represents HTML node (element)."""
 *     cdef myhtml_tree_node_t *node
*/

struct __pyx_vtabstruct_10selectolax_6parser_Node {
  struct __pyx_obj_10selectolax_6parser_Node *(*__pyx_new)(myhtml_tree_node_t *, struct __pyx_obj_10selectolax_6parser_HTMLParser *);
  PyObject *(*_text_deep)(struct __pyx_obj_10selectolax_6parser_Node *, myhtml_tree_node_t *, struct __pyx_opt_args_10selectolax_6parser_4Node__text_deep *__pyx_optional_args);
};
static struct __pyx_vtabstruct_10selectolax_6parser_Node *__pyx_vtabptr_10selectolax_6parser_Node;
static CYTHON_INLINE PyObject *__pyx_f_10selectolax_6parser_4Node__text_deep(struct __pyx_obj_10selectolax_6parser_Node *, myhtml_tree_node_t *, struct __pyx_opt_args_10selectolax_6parser_4Node__text_deep *__pyx_optional_args);
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL || (CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallCFunction.proto */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject *const *args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if !CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03080000
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()
  #define __Pyx_PyFrame_GetLocalsplus(frame)  ((frame)->f_localsplus)
#else
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_string.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* RejectKeywords.proto */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00a2
    return PyList_Extend(L, v);
#elif CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* PyObjectVectorCallKwBuilder.proto */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* PyObjectVectorCallMethodKwBuilder.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_VectorcallMethod_CallFromBuilder PyObject_VectorcallMethod
#else
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* pep479.proto */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen);

/* PyObject_Unicode.proto */
#define __Pyx_PyObject_Unicode(obj)\
    (likely(PyUnicode_CheckExact(obj)) ? __Pyx_NewRef(obj) : PyObject_Str(obj))

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* BuildPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* COrdinalToPyUnicode.proto */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value);
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t width, char padding_char);

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* IncludeStdlibH.proto */
#include <stdlib.h>

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char);

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_long(long value, Py_ssize_t width, char padding_char, char format_char);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectCallNoArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
    PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple);
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* py_dict_values.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* CallUnboundCMethod0.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* PyObjectCall2Args.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* CallUnboundCMethod1.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* UnicodeConcatInPlace.proto */
# if CYTHON_COMPILING_IN_CPYTHON
    #if CYTHON_REFNANNY
        #define __Pyx_PyUnicode_ConcatInPlace(left, right) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, __pyx_refnanny)
    #else
        #define __Pyx_PyUnicode_ConcatInPlace(left, right) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right)
    #endif
    static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    );
#else
#define __Pyx_PyUnicode_ConcatInPlace __Pyx_PyUnicode_Concat
#endif
#define __Pyx_PyUnicode_ConcatInPlaceSafe(left, right) ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_ConcatInPlace(left, right))

/* RaiseUnboundLocalError.proto */
static void __Pyx_RaiseUnboundLocalError(const char *varname);

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportDottedModule.proto */
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple);
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* CallTypeTraverse.proto */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#define __Pyx_call_type_traverse(o, always_call, visit, arg) 0
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg);
#endif

/* LimitedApiGetTypeDict.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp);
#endif

/* SetItemOnTypeDict.proto */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v);
#define __Pyx_SetItemOnTypeDict(tp, k, v) __Pyx__SetItemOnTypeDict((PyTypeObject*)tp, k, v)

/* FixUpExtensionType.proto */
static CYTHON_INLINE int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);

/* ValidateBasesTuple.proto */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
static int __Pyx_MergeVtables(PyTypeObject *type);

/* DelItemOnTypeDict.proto */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k);
#define __Pyx_DelItemOnTypeDict(tp, k) __Pyx__DelItemOnTypeDict((PyTypeObject*)tp, k)

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_1_2
#define __PYX_HAVE_RT_ImportType_proto_3_1_2
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_1_2(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_1_2(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_1_2 {
   __Pyx_ImportType_CheckSize_Error_3_1_2 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_1_2 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_1_2 = 2
};
static PyTypeObject *__Pyx_ImportType_3_1_2(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_1_2 check_size);
#endif

/* FetchSharedCythonModule.proto */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* dict_setdefault.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value, int is_safe_type);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases);

/* CommonTypesMetaclass.proto */
static int __pyx_CommonTypesMetaclass_init(PyObject *module);
#define __Pyx_CommonTypesMetaclass_USED

/* PyMethodNew.proto */
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ);

/* PyVectorcallFastCallDict.proto */
#if CYTHON_METH_FASTCALL && (CYTHON_VECTORCALL || CYTHON_BACKPORT_VECTORCALL)
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_BACKPORT_VECTORCALL ||\
        (CYTHON_COMPILING_IN_LIMITED_API && CYTHON_METH_FASTCALL)
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    PyObject *defaults;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_mstate_global->__pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_mstate_global->__pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void));
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func,
                                                         PyTypeObject *defaults_type);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_BACKPORT_VECTORCALL || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* ListPack.proto */
static PyObject *__Pyx_PyList_Pack(Py_ssize_t n, ...);

/* FusedFunction.proto */
typedef struct {
    __pyx_CyFunctionObject func;
    PyObject *__signatures__;
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyMethodDef *ml;
#endif
} __pyx_FusedFunctionObject;
static PyObject *__pyx_FusedFunction_New(PyMethodDef *ml, int flags,
                                         PyObject *qualname, PyObject *closure,
                                         PyObject *module, PyObject *globals,
                                         PyObject *code);
static int __pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self);
static int __pyx_FusedFunction_init(PyObject *module);
#define __Pyx_FusedFunction_USED

/* CLineInTraceback.proto */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_mystatus_t(mystatus_t value);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* IterNextPlain.proto */
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator);
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void);
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* ReturnWithStopIteration.proto */
static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext);

/* CoroutineBase.proto */
struct __pyx_CoroutineObject;
typedef PyObject *(*__pyx_coroutine_body_t)(struct __pyx_CoroutineObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct __pyx_CoroutineObject {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    __Pyx_pyiter_sendfunc yieldfrom_am_send;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
#if CYTHON_USE_SYS_MONITORING && (CYTHON_PROFILE || CYTHON_TRACE)
    PyMonitoringState __pyx_pymonitoring_state[__Pyx_MonitoringEventTypes_CyGen_count];
    uint64_t __pyx_pymonitoring_version;
#endif
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static __Pyx_PySendResult __Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static __Pyx_PySendResult __Pyx_Coroutine_Close(PyObject *self, PyObject **retval);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);
static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen);
static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen);
static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen);
static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure);
#if __PYX_HAS_PY_AM_SEND == 2
static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send);
#endif
static PyObject *__Pyx_Coroutine_fail_reduce_ex(PyObject *self, PyObject *arg);

/* Generator.proto */
#define __Pyx_Generator_USED
#define __Pyx_Generator_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_mstate_global->__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(PyObject *module);
static CYTHON_INLINE PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self);

/* GetRuntimeVersion.proto */
static unsigned long __Pyx_get_runtime_version(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

static myhtml_collection_t *__pyx_f_10selectolax_6parser_11CSSSelector_find(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self, myhtml_tree_node_t *__pyx_v_scope); /* proto*/
static int __pyx_f_10selectolax_6parser_11CSSSelector__create_css_parser(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self); /* proto*/
static int __pyx_f_10selectolax_6parser_11CSSSelector__prepare_selector(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self, mycss_entry_t *__pyx_v_css_entry, char const *__pyx_v_selector, size_t __pyx_v_selector_size); /* proto*/
static PyObject *__pyx_f_10selectolax_6parser_8Selector_css(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_query, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_10selectolax_6parser_5Stack_is_empty(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self); /* proto*/
static int __pyx_f_10selectolax_6parser_5Stack_push(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self, myhtml_tree_node_t *__pyx_v_res); /* proto*/
static myhtml_tree_node_t *__pyx_f_10selectolax_6parser_5Stack_pop(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self); /* proto*/
static int __pyx_f_10selectolax_6parser_5Stack_resize(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self); /* proto*/
static struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_f_10selectolax_6parser_11_Attributes_create(myhtml_tree_node_t *__pyx_v_node, PyObject *__pyx_v_decode_errors); /* proto*/
static struct __pyx_obj_10selectolax_6parser_Node *__pyx_f_10selectolax_6parser_4Node_new(myhtml_tree_node_t *__pyx_v_node, struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_parser); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_10selectolax_6parser_4Node__text_deep(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, myhtml_tree_node_t *__pyx_v_node, struct __pyx_opt_args_10selectolax_6parser_4Node__text_deep *__pyx_optional_args); /* proto*/
static void __pyx_f_10selectolax_6parser_10HTMLParser__detect_encoding(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, char *__pyx_v_html, size_t __pyx_v_html_len); /* proto*/
static int __pyx_f_10selectolax_6parser_10HTMLParser__parse_html(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, char *__pyx_v_html, size_t __pyx_v_html_len); /* proto*/
static struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_f_10selectolax_6parser_10HTMLParser_from_tree(myhtml_tree_t *__pyx_v_tree, PyObject *__pyx_v_raw_html, int __pyx_v_detect_encoding, int __pyx_v_use_meta_tags, PyObject *__pyx_v_decode_errors, myencoding_t __pyx_v_encoding); /* proto*/

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.bool" */

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdio" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.type" */

/* Module declarations from "cpython" */

/* Module declarations from "cpython.object" */

/* Module declarations from "cpython.exc" */

/* Module declarations from "cython" */

/* Module declarations from "libc.stdlib" */

/* Module declarations from "selectolax.parser" */
static PyObject *__pyx_f_10selectolax_6parser_find_nodes(struct __pyx_obj_10selectolax_6parser_HTMLParser *, myhtml_tree_node_t *, PyObject *); /*proto*/
static PyLongObject *__pyx_f_10selectolax_6parser_find_matches(struct __pyx_obj_10selectolax_6parser_HTMLParser *, myhtml_tree_node_t *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_10selectolax_6parser_append_text(PyObject *, PyObject *, struct __pyx_opt_args_10selectolax_6parser_append_text *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_10selectolax_6parser___pyx_unpickle_Selector__set_state(struct __pyx_obj_10selectolax_6parser_Selector *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_fuse_0__pyx_f_10selectolax_6parser_to_bytes(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_fuse_1__pyx_f_10selectolax_6parser_to_bytes(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_fuse_2__pyx_f_10selectolax_6parser_to_bytes(struct __pyx_obj_10selectolax_6parser_Node *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "selectolax.parser"
extern int __pyx_module_is_main_selectolax__parser;
int __pyx_module_is_main_selectolax__parser = 0;

/* Implementation of "selectolax.parser" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin___import__;
/* #### Code section: string_decls ### */
static const char __pyx_k_[] = "";
static const char __pyx_k_A[] = "\200A";
static const char __pyx_k_Q[] = "\200\001\330\004\n\210+\220Q";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_i[] = "i";
static const char __pyx_k__2[] = "<";
static const char __pyx_k__4[] = "()";
static const char __pyx_k__5[] = "|";
static const char __pyx_k__9[] = "></";
static const char __pyx_k_gc[] = "gc";
static const char __pyx_k_re[] = "re";
static const char __pyx_k_AV1[] = "\320\000\031\230\021\360\020\000\005\014\320\013\034\230A\230V\2401";
static const char __pyx_k_A_2[] = "\320\004$\240A";
static const char __pyx_k_A_a[] = "\320\000\024\220A\360\n\000\005\014\210=\230\001\230\025\230a";
static const char __pyx_k__10[] = ">";
static const char __pyx_k__11[] = ".";
static const char __pyx_k__12[] = "?";
static const char __pyx_k__13[] = "\320\004 \240\001";
static const char __pyx_k_avQ[] = "\200\001\330\004&\240a\240v\250Q";
static const char __pyx_k_cls[] = "cls";
static const char __pyx_k_css[] = "css";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_src[] = "src";
static const char __pyx_k_str[] = "str";
static const char __pyx_k_tag[] = "tag";
static const char __pyx_k_val[] = "val";
static const char __pyx_k_A_t5[] = "\200A\340\010\017\210t\2205\320\030(\250\001\250\021";
static const char __pyx_k_Node[] = "Node";
static const char __pyx_k_Type[] = "Type";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_attr[] = "attr";
static const char __pyx_k_body[] = "<body";
static const char __pyx_k_bool[] = "bool";
static const char __pyx_k_deep[] = "deep";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_find[] = "find";
static const char __pyx_k_func[] = "__func__";
static const char __pyx_k_head[] = "<head";
static const char __pyx_k_html[] = "html";
static const char __pyx_k_iter[] = "__iter__";
static const char __pyx_k_keys[] = "keys";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_next[] = "next";
static const char __pyx_k_node[] = "node";
static const char __pyx_k_root[] = "root";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_sget[] = "sget";
static const char __pyx_k_spec[] = "__spec__";
static const char __pyx_k_tags[] = "tags";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_text[] = "text";
static const char __pyx_k_tree[] = "tree";
static const char __pyx_k_A_Jaq[] = "\320\004$\240A\340\010\014\210J\220a\220q";
static const char __pyx_k_A_t9A[] = "\200A\330\010\017\210t\2209\230A";
static const char __pyx_k_Stack[] = "Stack";
static const char __pyx_k_UTF_8[] = "UTF-8";
static const char __pyx_k_Union[] = "Union";
static const char __pyx_k_attrs[] = "attrs";
static const char __pyx_k_bytes[] = "bytes";
static const char __pyx_k_clone[] = "clone";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_match[] = "match";
static const char __pyx_k_nodes[] = "nodes";
static const char __pyx_k_query[] = "query";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_stack[] = "stack";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_strip[] = "strip";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_undef[] = "<-undef>";
static const char __pyx_k_utf_8[] = "utf-8";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_1_xq_a[] = "\320\004\033\2301\360\034\000\t\020\210x\220q\230\006\230a";
static const char __pyx_k_A_t5_2[] = "\200A\360\026\000\t\020\210t\2205\320\030(\250\001\250\021";
static const char __pyx_k_A_t5_3[] = "\200A\360$\000\t\020\210t\2205\320\030)\250\021";
static const char __pyx_k_Node_s[] = "<Node %s>";
static const char __pyx_k_Parser[] = "Parser";
static const char __pyx_k_body_2[] = "body";
static const char __pyx_k_c_text[] = "c_text";
static const char __pyx_k_decode[] = "decode";
static const char __pyx_k_dict_2[] = "_dict";
static const char __pyx_k_enable[] = "enable";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_errors[] = "errors";
static const char __pyx_k_head_2[] = "head";
static const char __pyx_k_html_2[] = "<html";
static const char __pyx_k_ignore[] = "ignore";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_iter_2[] = "iter";
static const char __pyx_k_kwargs[] = "kwargs";
static const char __pyx_k_length[] = "length";
static const char __pyx_k_mem_id[] = "mem_id";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_myhtml[] = "myhtml";
static const char __pyx_k_name_2[] = "name";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_q_4q_1[] = "\320\004\037\230q\330\010\t\330\014\023\2204\220q\230\001\330\017\020\330\014\023\2201";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_remove[] = "remove";
static const char __pyx_k_result[] = "result";
static const char __pyx_k_return[] = "return";
static const char __pyx_k_script[] = "script";
static const char __pyx_k_select[] = "select";
static const char __pyx_k_status[] = "status";
static const char __pyx_k_strict[] = "strict";
static const char __pyx_k_typing[] = "typing";
static const char __pyx_k_unwrap[] = "unwrap";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_values[] = "values";
static const char __pyx_k_A_1D_WA[] = "\200A\340\010\017\210|\2301\230D\240\t\250\024\250W\260A";
static const char __pyx_k_A_1D_XQ[] = "\200A\340\010\017\210|\2301\230D\240\t\250\024\250X\260Q";
static const char __pyx_k_A_t5_AQ[] = "\200A\360\024\000\t\020\210t\2205\320\030,\250A\250Q";
static const char __pyx_k_A_z_it7[] = "\200A\340\010\017\210z\230\021\230$\230i\240t\2507\260!";
static const char __pyx_k_Literal[] = "Literal";
static const char __pyx_k_compile[] = "compile";
static const char __pyx_k_default[] = "default";
static const char __pyx_k_disable[] = "disable";
static const char __pyx_k_element[] = "element";
static const char __pyx_k_getitem[] = "__getitem__";
static const char __pyx_k_html_re[] = "html_re";
static const char __pyx_k_items_2[] = " items>";
static const char __pyx_k_queries[] = "queries";
static const char __pyx_k_results[] = "results";
static const char __pyx_k_unknown[] = "unknown";
static const char __pyx_k_FRAGMENT[] = "FRAGMENT";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_Node_css[] = "Node.css";
static const char __pyx_k_Optional[] = "Optional";
static const char __pyx_k_Selector[] = "Selector";
static const char __pyx_k_add_note[] = "add_note";
static const char __pyx_k_capacity[] = "capacity";
static const char __pyx_k_defaults[] = "defaults";
static const char __pyx_k_document[] = "document";
static const char __pyx_k_endswith[] = "endswith";
static const char __pyx_k_finditer[] = "finditer";
static const char __pyx_k_fragment[] = "fragment";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_has_body[] = "has_body";
static const char __pyx_k_has_head[] = "has_head";
static const char __pyx_k_has_html[] = "has_html";
static const char __pyx_k_html_len[] = "html_len";
static const char __pyx_k_node_src[] = "node_src";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_selector[] = "selector";
static const char __pyx_k_set_name[] = "__set_name__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_traverse[] = "traverse";
static const char __pyx_k_A_t5_AQ_2[] = "\200A\330\010\017\210t\2205\230\014\240A\240Q";
static const char __pyx_k_Node_iter[] = "Node.iter";
static const char __pyx_k_Node_text[] = "Node.text";
static const char __pyx_k_ParserCls[] = "ParserCls";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_attribute[] = "attribute";
static const char __pyx_k_bytes_val[] = "bytes_val";
static const char __pyx_k_css_first[] = "css_first";
static const char __pyx_k_decompose[] = "decompose";
static const char __pyx_k_frag_type[] = "frag_type";
static const char __pyx_k_html_tree[] = "html_tree";
static const char __pyx_k_isenabled[] = "isenabled";
static const char __pyx_k_left_text[] = "left_text";
static const char __pyx_k_n_results[] = "n_results";
static const char __pyx_k_next_node[] = "next_node";
static const char __pyx_k_node_text[] = "node_text";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_q_1_Q_at1[] = "\320\004'\240q\360 \000\t\014\2101\330\014(\250\001\250\024\250Q\340\014\036\230a\230t\2401";
static const char __pyx_k_recursive[] = "recursive";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_selectors[] = "selectors";
static const char __pyx_k_separator[] = "separator";
static const char __pyx_k_src_nodes[] = "src_nodes";
static const char __pyx_k_1_t1_1_81F[] = "\320\004\033\2301\360\036\000\t\020\210t\2201\330\010\013\2101\330\014\023\2208\2301\230F\240!";
static const char __pyx_k_4G1_4vWA_Q[] = "\320\0044\260G\2701\360(\000\t\014\2104\210v\220W\230A\330\014\020\220\005\220\\\240\021\240&\250\r\260Q";
static const char __pyx_k_Attributes[] = "_Attributes";
static const char __pyx_k_HTMLParser[] = "HTMLParser";
static const char __pyx_k_IGNORECASE[] = "IGNORECASE";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_attributes[] = "attributes";
static const char __pyx_k_bytes_html[] = "bytes_html";
static const char __pyx_k_collection[] = "collection";
static const char __pyx_k_create_tag[] = "create_tag";
static const char __pyx_k_final_text[] = "final_text";
static const char __pyx_k_parser_cls[] = "parser_cls";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_right_text[] = "right_text";
static const char __pyx_k_strip_tags[] = "strip_tags";
static const char __pyx_k_text_nodes[] = "text_nodes";
static const char __pyx_k_A_T_j_A_t4q[] = "\200A\360\"\000\t\036\230T\240\021\240$\240j\260\014\270A\330\010\017\210t\2204\220q\230\001";
static const char __pyx_k_CSSSelector[] = "CSSSelector";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_Node_remove[] = "Node.remove";
static const char __pyx_k_Node_select[] = "Node.select";
static const char __pyx_k_Node_unwrap[] = "Node.unwrap";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_css_matches[] = "css_matches";
static const char __pyx_k_left_length[] = "left_length";
static const char __pyx_k_pybyte_name[] = "pybyte_name";
static const char __pyx_k_unwrap_tags[] = "unwrap_tags";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_Selector_css[] = "Selector.css";
static const char __pyx_k_T_j_A_t_QgYa[] = "\320\004+\250=\270\001\360&\000\t\036\230T\240\021\240$\240j\260\014\270A\330\010\017\210t\220:\230Q\230g\240Y\250a";
static const char __pyx_k_aq_t3a_a_1_1[] = "\320\004 \240\001\340\010\t\330\014\022\220$\220a\220q\330\014\017\210t\2203\220a\330\020\026\220a\330\014\023\2201\330\017\020\330\014\023\2201";
static const char __pyx_k_attributes_2[] = " attributes, ";
static const char __pyx_k_current_node[] = "current_node";
static const char __pyx_k_delete_empty[] = "delete_empty";
static const char __pyx_k_final_length[] = "final_length";
static const char __pyx_k_include_text[] = "include_text";
static const char __pyx_k_initializing[] = "_initializing";
static const char __pyx_k_insert_after[] = "insert_after";
static const char __pyx_k_insert_child[] = "insert_child";
static const char __pyx_k_is_coroutine[] = "_is_coroutine";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_replace_with[] = "replace_with";
static const char __pyx_k_right_length[] = "right_length";
static const char __pyx_k_stringsource[] = "<stringsource>";
static const char __pyx_k_use_setstate[] = "use_setstate";
static const char __pyx_k_Node_traverse[] = "Node.traverse";
static const char __pyx_k_arguments_got[] = " arguments, got ";
static const char __pyx_k_class_getitem[] = "__class_getitem__";
static const char __pyx_k_decode_errors[] = "decode_errors";
static const char __pyx_k_do_create_tag[] = "do_create_tag";
static const char __pyx_k_head_and_body[] = "head_and_body";
static const char __pyx_k_insert_before[] = "insert_before";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_t1_j_ARuHE_AQ[] = "\320\000\027\320\027(\250\001\330\004\007\200t\2101\330\010\016\210j\230\001\230\021\330\004\013\320\013\034\230A\230R\230u\240H\250E\260\033\270A\270Q";
static const char __pyx_k_text_contains[] = "text_contains";
static const char __pyx_k_use_meta_tags[] = "use_meta_tags";
static const char __pyx_k_4A_G1_4t1A_waq[] = "\320\0044\260A\360.\000\t\r\210G\2201\330\014\020\220\013\2304\230t\2401\240A\330\020\027\220w\230a\230q";
static const char __pyx_k_5Q_G1_4t1A_z_A[] = "\320\0045\260Q\360,\000\t\r\210G\2201\330\014\020\220\013\2304\230t\2401\240A\330\020\027\220z\240\021\240*\250A";
static const char __pyx_k_Attributes_get[] = "_Attributes.get";
static const char __pyx_k_HTMLParser_css[] = "HTMLParser.css";
static const char __pyx_k_Node_css_first[] = "Node.css_first";
static const char __pyx_k_Node_decompose[] = "Node.decompose";
static const char __pyx_k_parse_fragment[] = "parse_fragment";
static const char __pyx_k_Attributes_keys[] = "_Attributes.keys";
static const char __pyx_k_Attributes_sget[] = "_Attributes.sget";
static const char __pyx_k_HTMLParser_tags[] = "HTMLParser.tags";
static const char __pyx_k_HTMLParser_text[] = "HTMLParser.text";
static const char __pyx_k_Node_strip_tags[] = "Node.strip_tags";
static const char __pyx_k_Optional_Parser[] = "Optional[Parser]";
static const char __pyx_k_any_css_matches[] = "any_css_matches";
static const char __pyx_k_detect_encoding[] = "detect_encoding";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_scripts_contain[] = "scripts_contain";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_Attributes_items[] = "_Attributes.items";
static const char __pyx_k_HTMLParser_clone[] = "HTMLParser.clone";
static const char __pyx_k_LexborHTMLParser[] = "LexborHTMLParser";
static const char __pyx_k_Node_css_matches[] = "Node.css_matches";
static const char __pyx_k_Node_unwrap_tags[] = "Node.unwrap_tags";
static const char __pyx_k_document_no_body[] = "document_no_body";
static const char __pyx_k_document_no_head[] = "document_no_head";
static const char __pyx_k_insert_after_str[] = "insert_after[str]";
static const char __pyx_k_insert_child_str[] = "insert_child[str]";
static const char __pyx_k_merge_text_nodes[] = "merge_text_nodes";
static const char __pyx_k_preprocess_input[] = "preprocess_input";
static const char __pyx_k_replace_with_str[] = "replace_with[str]";
static const char __pyx_k_1_4t1_1_t5_Qe6_fA[] = "\320\004\035\320\0351\260\037\300\001\360\"\000\t\014\2104\210t\2201\330\014\023\2201\330\010\017\210t\2205\230\005\230Q\230e\2406\250\032\260;\270f\300A";
static const char __pyx_k_Attributes___iter[] = "_Attributes.__iter__";
static const char __pyx_k_Attributes_values[] = "_Attributes.values";
static const char __pyx_k_Expected_at_least[] = "Expected at least ";
static const char __pyx_k_HTMLParser_select[] = "HTMLParser.select";
static const char __pyx_k_Node_insert_after[] = "Node.insert_after";
static const char __pyx_k_Node_insert_child[] = "Node.insert_child";
static const char __pyx_k_Node_replace_with[] = "Node.replace_with";
static const char __pyx_k_any_text_contains[] = "any_text_contains";
static const char __pyx_k_do_parse_fragment[] = "do_parse_fragment";
static const char __pyx_k_get_fragment_type[] = "get_fragment_type";
static const char __pyx_k_html_body_head_er[] = "<html|<body|<head(?!er)";
static const char __pyx_k_insert_after_Node[] = "insert_after[Node]";
static const char __pyx_k_insert_before_str[] = "insert_before[str]";
static const char __pyx_k_insert_child_Node[] = "insert_child[Node]";
static const char __pyx_k_replace_with_Node[] = "replace_with[Node]";
static const char __pyx_k_selectolax_parser[] = "selectolax.parser";
static const char __pyx_k_HTMLParser_chars_s[] = "<HTMLParser chars=%s>";
static const char __pyx_k_Node_insert_before[] = "Node.insert_before";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_fused_sigindex_ref[] = "_fused_sigindex_ref";
static const char __pyx_k_insert_after_bytes[] = "insert_after[bytes]";
static const char __pyx_k_insert_before_Node[] = "insert_before[Node]";
static const char __pyx_k_insert_child_bytes[] = "insert_child[bytes]";
static const char __pyx_k_replace_with_bytes[] = "replace_with[bytes]";
static const char __pyx_k_A_A_HD_1_1D_Yl_IQ_q[] = "\200A\360\006\000\t\025\220A\330\010\014\210H\220D\230\001\330\014\033\2301\330\014\021\220\027\230\001\230\032\2401\240D\250\005\250Y\260l\300'\310\021\330\010\014\210I\220Q\330\010\017\210q";
static const char __pyx_k_Bad_CSS_Selectors_s[] = "Bad CSS Selectors: %s";
static const char __pyx_k_MAX_HTML_INPUT_SIZE[] = "MAX_HTML_INPUT_SIZE";
static const char __pyx_k_insert_before_bytes[] = "insert_before[bytes]";
static const char __pyx_k_script_srcs_contain[] = "script_srcs_contain";
static const char __pyx_k_HTMLParser_css_first[] = "HTMLParser.css_first";
static const char __pyx_k_Node___reduce_cython[] = "Node.__reduce_cython__";
static const char __pyx_k_Node_any_css_matches[] = "Node.any_css_matches";
static const char __pyx_k_Node_scripts_contain[] = "Node.scripts_contain";
static const char __pyx_k_Node_unwrap_line_517[] = "Node.unwrap (line 517)";
static const char __pyx_k_Tag_name_is_too_long[] = "Tag name is too long";
static const char __pyx_k_selectolax_utils_pxi[] = "selectolax/utils.pxi";
static const char __pyx_k_HTMLParser_strip_tags[] = "HTMLParser.strip_tags";
static const char __pyx_k_Node_insert_after_str[] = "Node.insert_after[str]";
static const char __pyx_k_Node_insert_child_str[] = "Node.insert_child[str]";
static const char __pyx_k_Node_merge_text_nodes[] = "Node.merge_text_nodes";
static const char __pyx_k_Node_replace_with_str[] = "Node.replace_with[str]";
static const char __pyx_k_Stack___reduce_cython[] = "Stack.__reduce_cython__";
static const char __pyx_k_attribute_longer_than[] = "attribute_longer_than";
static const char __pyx_k_pyx_unpickle_Selector[] = "__pyx_unpickle_Selector";
static const char __pyx_k_selectolax_parser_pyx[] = "selectolax/parser.pyx";
static const char __pyx_k_HTMLParser_css_matches[] = "HTMLParser.css_matches";
static const char __pyx_k_HTMLParser_unwrap_tags[] = "HTMLParser.unwrap_tags";
static const char __pyx_k_Node___setstate_cython[] = "Node.__setstate_cython__";
static const char __pyx_k_Node_insert_after_Node[] = "Node.insert_after[Node]";
static const char __pyx_k_Node_insert_before_str[] = "Node.insert_before[str]";
static const char __pyx_k_Node_insert_child_Node[] = "Node.insert_child[Node]";
static const char __pyx_k_Node_replace_with_Node[] = "Node.replace_with[Node]";
static const char __pyx_k_Selector_text_contains[] = "Selector.text_contains";
static const char __pyx_k_Can_t_init_MyCSS_object[] = "Can't init MyCSS object.";
static const char __pyx_k_Node_decompose_line_492[] = "Node.decompose (line 492)";
static const char __pyx_k_Node_insert_after_bytes[] = "Node.insert_after[bytes]";
static const char __pyx_k_Node_insert_before_Node[] = "Node.insert_before[Node]";
static const char __pyx_k_Node_insert_child_bytes[] = "Node.insert_child[bytes]";
static const char __pyx_k_Node_replace_with_bytes[] = "Node.replace_with[bytes]";
static const char __pyx_k_Stack___setstate_cython[] = "Stack.__setstate_cython__";
static const char __pyx_k_44H_WX_HD_E_vZ_z_U_Q_q_q[] = "\320\0044\3204H\310\017\320WX\340\010\020\220\001\340\010\014\210H\220D\230\001\330\014\030\230\004\230E\240\021\240%\240v\250Z\260{\300&\310\001\330\014\017\210z\230\024\230U\240#\240Q\330\020\027\220q\330\010\017\210q";
static const char __pyx_k_Can_t_init_MyHTML_object[] = "Can't init MyHTML object.";
static const char __pyx_k_Node_insert_before_bytes[] = "Node.insert_before[bytes]";
static const char __pyx_k_Node_script_srcs_contain[] = "Node.script_srcs_contain";
static const char __pyx_k_Node_strip_tags_line_554[] = "Node.strip_tags (line 554)";
static const char __pyx_k_Selector___reduce_cython[] = "Selector.__reduce_cython__";
static const char __pyx_k_Tag_name_cannot_be_empty[] = "Tag name cannot be empty";
static const char __pyx_k_document_no_head_no_body[] = "document_no_head_no_body";
static const char __pyx_k_hk_A_1_vvxxy_881A_7_nA_1[] = "\200\001\360\006\000\005\010\200\220h\230k\250\033\260A\330\010\r\210^\2301\330\010\016\320\016!\320!v\320vx\320xy\330\004\023\2208\2308\2401\240A\330\004\007\200|\2207\230!\330\010*\250!\250;\260n\300A\330\004\013\2101";
static const char __pyx_k_Node_unwrap_tags_line_580[] = "Node.unwrap_tags (line 580)";
static const char __pyx_k_Node_unwrap_tags_line_768[] = "Node.unwrap_tags (line 768)";
static const char __pyx_k_any_attribute_longer_than[] = "any_attribute_longer_than";
static const char __pyx_k_Attributes___reduce_cython[] = "_Attributes.__reduce_cython__";
static const char __pyx_k_HTMLParser___reduce_cython[] = "HTMLParser.__reduce_cython__";
static const char __pyx_k_HTMLParser_any_css_matches[] = "HTMLParser.any_css_matches";
static const char __pyx_k_HTMLParser_scripts_contain[] = "HTMLParser.scripts_contain";
static const char __pyx_k_Node_insert_after_line_688[] = "Node.insert_after (line 688)";
static const char __pyx_k_Node_insert_child_line_728[] = "Node.insert_child (line 728)";
static const char __pyx_k_Node_replace_with_line_607[] = "Node.replace_with (line 607)";
static const char __pyx_k_Selector___setstate_cython[] = "Selector.__setstate_cython__";
static const char __pyx_k_Selector_any_text_contains[] = "Selector.any_text_contains";
static const char __pyx_k_selectolax_modest_node_pxi[] = "selectolax/modest/node.pxi";
static const char __pyx_k_selectolax_modest_util_pxi[] = "selectolax/modest/util.pxi";
static const char __pyx_k_CSSSelector___reduce_cython[] = "CSSSelector.__reduce_cython__";
static const char __pyx_k_HTMLParser_merge_text_nodes[] = "HTMLParser.merge_text_nodes";
static const char __pyx_k_No_matching_signature_found[] = "No matching signature found";
static const char __pyx_k_Node_attrs___get___line_194[] = "Node.attrs.__get__ (line 194)";
static const char __pyx_k_Node_insert_before_line_648[] = "Node.insert_before (line 648)";
static const char __pyx_k_selectolax_modest_utils_pxi[] = "selectolax/modest/../utils.pxi";
static const char __pyx_k_Attributes___setstate_cython[] = "_Attributes.__setstate_cython__";
static const char __pyx_k_HTMLParser___setstate_cython[] = "HTMLParser.__setstate_cython__";
static const char __pyx_k_00DOST_HD_E_vZ_z_U_Q_WAQ_IQ_q[] = "\320\0040\3200D\300O\320ST\340\010\020\220\001\340\010\014\210H\220D\230\001\330\014\030\230\004\230E\240\021\240%\240v\250Z\260{\300&\310\001\330\014\017\210z\230\024\230U\240#\240Q\330\020\025\220W\230A\230Q\330\010\014\210I\220Q\330\010\017\210q";
static const char __pyx_k_CSSSelector___setstate_cython[] = "CSSSelector.__setstate_cython__";
static const char __pyx_k_Can_t_init_MyCSS_Entry_object[] = "Can't init MyCSS Entry object.";
static const char __pyx_k_Can_t_init_MyHTML_Tree_object[] = "Can't init MyHTML Tree object.";
static const char __pyx_k_Expected_a_string_but_s_found[] = "Expected a string, but %s found";
static const char __pyx_k_Q_d_1_Cq_Rq_wd_Ba_j_J_A_7_1_q[] = "\320\004+\320+=\270Q\340\010\022\220$\220d\230!\2301\330\010\024\220C\220q\230\001\340\010\013\210:\220R\220q\340\014\017\210w\220d\230*\240B\240a\330\020\026\220j\240\001\320!J\310\"\310A\340\014\023\2207\230!\2301\340\010\017\210q";
static const char __pyx_k_1_q_E_4q_t5_1_gQ_7_Q_gQk_WA_av[] = "\320\004\035\320\0351\260\037\300\001\360\"\000\t\020\210q\340\010(\250\004\250E\260\021\340\010\013\2104\210q\330\014\017\210t\2205\230\010\240\003\2401\330\020\031\320\031)\250\021\250$\250g\260Q\330\020\023\2207\230#\230Q\330\024 \240\006\240g\250Q\250k\270\024\270W\300A\330\024\033\230;\240a\240v\250[\270\013\3001\340\014\022\220%\220s\230!\330\020\023\2204\220x\230s\240!\330\024\035\320\035-\250Q\250f\260A\330\024\027\220w\230c\240\021\330\030$\240F\250'\260\021\260+\270T\300\027\310\001\330\030\037\230{\250!\2506\260\033\270K\300q\330\020\027\220t\2301\340\014\023\2204\220{\240!\2404\320'8\3208I\310\021\330\010\013\210:\220T\230\025\230d\240$\240i\250q\260\001\330\014\023\2204\220r\230\021\230#\230Q\230a\330\010\017\210q";
static const char __pyx_k_Can_t_parse_HTML_status_code_d[] = "Can't parse HTML (status code: %d)";
static const char __pyx_k_HTMLParser_script_srcs_contain[] = "HTMLParser.script_srcs_contain";
static const char __pyx_k_HTMLParser_strip_tags_line_235[] = "HTMLParser.strip_tags (line 235)";
static const char __pyx_k_LA_HD_4_aq_uD_d_1_t1D_QgRs_1_s[] = "\320\004L\310A\360\n\000\t\021\220\001\330\010\014\210H\220D\230\001\330\014\023\2204\220{\240$\240a\240q\330\014\017\210u\220D\230\006\230d\240&\250\003\2501\330\020\027\220t\2301\230D\240\005\240Q\240g\250R\250s\260!\2601\330\014\017\210s\220!\2206\230\022\2301\330\020\025\220W\230A\230Q\330\010\014\210I\220Q\330\010\017\210q";
static const char __pyx_k_Node_merge_text_nodes_line_921[] = "Node.merge_text_nodes (line 921)";
static const char __pyx_k_Q_4vQ_Qa_A_z_A_q_A_z_A_E_1Ja_q[] = "\320\000\034\320\034-\250Q\360\020\000\005\014\2104\210v\220Q\330\004\013\210:\220Q\220a\330\004\020\320\020!\240\021\240&\250\014\260A\340\004\007\200z\220\023\220A\330\010\017\210q\220\004\220A\330\004\007\200z\220\023\220A\330\010\014\210E\220\032\2301\230J\240a\330\010\017\210q\220\004\220A\330\004\007\200z\220\023\220A\330\010\014\210E\220\032\2301\230J\240a\330\010\017\210q\220\004\220A\330\004\007\200z\220\023\220A\330\010\014\210E\220\032\2301\230J\240a\330\010\014\210E\220\032\2301\230J\240a\330\010\017\210q\220\004\220A\330\t\023\2203\220a\330\010\014\210E\220\032\2301\230J\240a\330\010\017\210q\220\004\220A\330\t\023\2203\220a\330\010\014\210E\220\032\2301\230J\240a\330\010\017\210q\220\004\220A\330\t\023\2203\220a\330\010\017\210q\220\004\220G\2304\230q\340\010\017\210q\330\r\021\220\025\220e\2301\230M\250\021\330\r\021\220\025\220e\2301\230M\250\021";
static const char __pyx_k_Selector_attribute_longer_than[] = "Selector.attribute_longer_than";
static const char __pyx_k_T_A_G1F_a_vWA_q_t6_S_G7_q_Qg_q[] = "\200\001\360\010\000\005\016\210T\220\027\230\004\230A\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220A\330\010\022\220!\330\010\027\220q\340\010\027\220t\2306\240\027\250\005\250S\260\004\260G\2707\300!\330\004\007\200q\330\010\017\320\017)\250\024\250Q\250g\260[\300\007\300q\340\010\017\320\017)\250\024\250Q\250g\260[\300\001";
static const char __pyx_k_1_z_a_T_1F_Q_iq_D_a_s_1_y_j___q[] = "\320\000\033\2301\330\004\007\200z\220\021\220'\230\025\230a\330\010\025\220T\230\027\240\001\240\031\250'\260\021\330\t\023\2201\220F\230!\330\010\025\220Q\340\010\016\210i\220q\320\030:\270\"\270D\300\001\300\025\300a\330\004\017\210s\220!\2201\330\004\007\200y\220\002\220!\330\010\016\210j\230\001\320\031\\\320\\^\320^_\330\004\013\210<\220q";
static const char __pyx_k_5Q_0q_G1_WAQ_1_l_UXXYYgghhi_Q_w[] = "\320\0045\260Q\360(\000\t0\250q\340\010!\240\021\340\010\014\210G\2201\330\014\032\230#\230W\240A\240Q\330\014\031\320\0311\260\021\260$\260l\300&\310\r\320UX\320XY\320Yg\320gh\320hi\340\014\017\210{\230#\230Q\330\020\021\340\014\017\210w\220c\230\021\330\020\021\340\014\020\220\005\220U\230!\230:\240Q\330\020\023\2201\330\0240\260\001\260\032\2705\300\001\300\021\340\024&\240a\240z\260\025\260a\260q\340\014%\240Q\240a";
static const char __pyx_k_A8_Qhe7_Qd_Oq_x_Ql_Qd_at1_q_q_g[] = "\200A\3608\000\t\014\210:\220Q\220h\230e\2407\250!\330\014\030\230\010\240\001\240\021\330\014\023\320\023%\240Q\240d\250'\260\034\320=O\310q\330\014 \240\001\240\026\240x\250{\270#\270Q\270l\310!\330\014%\240Q\240d\250'\260\021\330\014\036\230a\230t\2401\330\r\027\220q\230\007\230q\330\014\023\320\023)\250\021\250$\250g\260\\\320AX\320X]\320]^\330\014%\240Q\240d\250'\260\021\330\014\036\230a\230t\2401\340\014\022\220)\2301\320\034O\310r\320QU\320UV\320V\\\320\\]";
static const char __pyx_k_A_4uG3a_q_4q_4uF_1_4uA_A_Cq_L_g[] = "\320\004$\240A\360$\000\t\014\2104\210u\220G\2303\230a\330\014\017\210q\330\020\"\240!\2404\240q\330\014\r\360\010\000\t\014\2104\210u\220F\230&\240\003\2401\330\014\033\2304\230u\240A\330\014\030\230\014\240A\340\014\022\220*\230C\230q\330\020\034\230L\250\001\330\020)\250\021\250$\250g\260Q\330\020\037\230q\340\014%\240Q\240d\250'\260\024\260U\270!\330\010\032\230!\2304\230q";
static const char __pyx_k_A_4w_Q_Jat9D_q_4vT_1_WAQ_Q_HD_q[] = "\200A\360\024\000\t\014\2104\210w\320\026*\250#\250Q\330\014\024\220J\230a\230t\2409\250D\260\007\260q\330\014\030\230\001\330\014\020\220\010\230\001\330\020\033\2304\230v\240T\250\021\250!\330\020\023\2201\330\024\035\230W\240A\240Q\330\014\020\220\007\320\027-\250Q\340\010\014\210H\220D\230\007\230q\330\014\020\220\t\230\021\330\020\023\2206\230\023\230A\330\024\033\2301\330\010\017\210q";
static const char __pyx_k_Can_t_allocate_memory_for_a_new[] = "Can't allocate memory for a new node.";
static const char __pyx_k_HTMLParser_unwrap_tags_line_277[] = "HTMLParser.unwrap_tags (line 277)";
static const char __pyx_k_Insert_a_node_after_the_current[] = "\n        Insert a node after the current Node.\n\n        Parameters\n        ----------\n        value : str, bytes or Node\n            The text or Node instance to insert after the Node.\n            When a text string is passed, it's treated as text. All HTML tags will be escaped.\n            Convert and pass the ``Node`` object when you want to work with HTML.\n            Does not clone the ``Node`` object.\n            All future changes to the passed ``Node`` object will also be taken into account.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser('<div>Get <img src=\"\" alt=\"Laptop\"></div>')\n        >>> img = tree.css_first('img')\n        >>> img.insert_after(img.attributes.get('alt', ''))\n        >>> tree.body.child.html\n        '<div>Get <img src=\"\" alt=\"Laptop\">Laptop</div>'\n\n        >>> html_parser = HTMLParser('<div>Get <span alt=\"Laptop\"><img src=\"/jpg\"> <div></div></span></div>')\n        >>> html_parser2 = HTMLParser('<div>Test</div>')\n        >>> img_node = html_parser.css_first('img')\n        >>> img_node.insert_after(html_parser2.body.child)\n        <div>Get <span alt=\"Laptop\"><img src=\"/jpg\"><div>Test</div> <div></div></span></div>'\n        ";
static const char __pyx_k_Insert_a_node_before_the_curren[] = "\n        Insert a node before the current Node.\n\n        Parameters\n        ----------\n        value : str, bytes or Node\n            The text or Node instance to insert before the Node.\n            When a text string is passed, it's treated as text. All HTML tags will be escaped.\n            Convert and pass the ``Node`` object when you want to work with HTML.\n            Does not clone the ``Node`` object.\n            All future changes to the passed ``Node`` object will also be taken into account.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser('<div>Get <img src=\"\" alt=\"Laptop\"></div>')\n        >>> img = tree.css_first('img')\n        >>> img.insert_before(img.attributes.get('alt', ''))\n        >>> tree.body.child.html\n        '<div>Get Laptop<img src=\"\" alt=\"Laptop\"></div>'\n\n        >>> html_parser = HTMLParser('<div>Get <span alt=\"Laptop\"><img src=\"/jpg\"> <div></div></span></div>')\n        >>> html_parser2 = HTMLParser('<div>Test</div>')\n        >>> img_node = html_parser.css_first('img')\n        >>> img_node.insert_before(html_parser2.body.child)\n        <div>Get <span alt=\"Laptop\"><div>Test</div><img src=\"/jpg\"> <div></div></span></div>'\n        ";
static const char __pyx_k_Insert_a_node_inside_at_the_end[] = "\n        Insert a node inside (at the end of) the current Node.\n\n        Parameters\n        ----------\n        value : str, bytes or Node\n            The text or Node instance to insert inside the Node.\n            When a text string is passed, it's treated as text. All HTML tags will be escaped.\n            Convert and pass the ``Node`` object when you want to work with HTML.\n            Does not clone the ``Node`` object.\n            All future changes to the passed ``Node`` object will also be taken into account.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser('<div>Get <img src=\"\"></div>')\n        >>> div = tree.css_first('div')\n        >>> div.insert_child('Laptop')\n        >>> tree.body.child.html\n        '<div>Get <img src=\"\">Laptop</div>'\n\n        >>> html_parser = HTMLParser('<div>Get <span alt=\"Laptop\"> <div>Laptop</div> </span></div>')\n        >>> html_parser2 = HTMLParser('<div>Test</div>')\n        >>> span_node = html_parser.css_first('span')\n        >>> span_node.insert_child(html_parser2.body.child)\n        <div>Get <span alt=\"Laptop\"> <div>Laptop</div> <div>Test</div> </span></div>'\n        ";
static const char __pyx_k_Node_raw_value___get___line_795[] = "Node.raw_value.__get__ (line 795)";
static const char __pyx_k_PPQ_HD_4_aq_uD_d_1_t1D_QgRs_1_s[] = "\320\004P\320PQ\360\n\000\t\033\230!\340\010\014\210H\220D\230\001\330\014\023\2204\220{\240$\240a\240q\330\014\017\210u\220D\230\006\230d\240&\250\003\2501\330\020\027\220t\2301\230D\240\005\240Q\240g\250R\250s\260!\2601\330\014\017\210s\220!\2206\230\022\2301\330\020\027\220q\330\010\017\210q";
static const char __pyx_k_The_specified_HTML_input_is_too[] = "The specified HTML input is too large to be processed (%d bytes)";
static const char __pyx_k_Unwraps_specified_tags_from_the[] = "Unwraps specified tags from the HTML tree.\n\n        Works the same as the ``unwrap`` method, but applied to a list of tags.\n\n        Parameters\n        ----------\n        tags : list\n            List of tags to remove.\n        delete_empty : bool, default False\n            Whenever to delete empty tags.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser(\"<div><a href=\"\">Hello</a> <i>world</i>!</div>\")\n        >>> tree.body.unwrap_tags(['i','a'])\n        >>> tree.body.html\n        '<body><div>Hello world!</div></body>'\n\n        Note: by default, empty tags are ignored, set \"delete_empty\" to \"True\" to change this.\n        ";
static const char __pyx_k_selectolax_modest_selection_pxi[] = "selectolax/modest/selection.pxi";
static const char __pyx_k_self_css_entry_self_finder_self[] = "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling";
static const char __pyx_k_t1_z_b_5Rq_q_q_q_5_Cq_q_3c_q_3c[] = "\200\001\330\n\013\330\020\021\330\004\n\210!\330\005\006\330\004\007\200t\2101\330\010\017\210z\230\021\230!\340\013\014\330\004\016\210b\220\010\230\001\320\0315\260R\260q\340\004\017\210q\330\004\017\210q\330\004\017\210q\330\004\010\210\t\220\027\230\t\240\021\240!\330\010\013\2105\220\001\220\023\220C\220q\330\014\027\220q\330\r\022\220!\2203\220c\230\021\330\014\027\220q\330\r\022\220!\2203\220c\230\021\330\014\027\220q\340\010\013\2109\220D\230\t\240\024\240Q\330\014\r\340\004\007\200y\220\004\220I\230T\240\021\330\010\017\210q\330\t\022\220$\220d\230)\2404\240q\330\010\017\210q\330\t\022\220$\220i\230t\2404\240q\330\010\017\210q\330\t\022\220$\220d\230)\2404\240t\2501\330\010\017\210q\330\t\022\220$\220d\230!\330\010\017\210q\330\t\r\210Y\220d\230!\330\010\017\210q\330\t\022\220$\220a\330\010\017\210q\340\010\017\210q";
static const char __pyx_k_A_4q_AQ_3avRq_AQ_q_d_Q_Qd_fMQTTU[] = "\200A\360\022\000\t\014\2104\210q\330\014\022\220*\230A\230Q\330\010\013\2103\210a\210v\220R\220q\330\014\022\220*\230A\230Q\340\010/\250q\330\010\026\220d\230'\240\021\240!\330\010!\240\021\340\010\025\220Q\330\010\025\320\025-\250Q\250d\260,\270f\300M\320QT\320TU\320Uc\320cd\320de\340\010\013\210;\220c\230\021\330\014\023\2201\340\010\013\2107\220#\220Q\330\014\020\220\005\220U\230!\230:\240Q\330\020\033\2304\230q\240\n\250%\250q\260\004\260A\330\020\026\220g\230Q\230a\340\010!\240\021\240!\340\010\017\210q";
static const char __pyx_k_A_4w_3a_Jat9D_q_D_Qe1_1_gQa_a_HD[] = "\200A\360\030\000\t\014\2104\210w\320\026+\2503\250a\330\014\024\220J\230a\230t\2409\250D\260\007\260q\330\014\031\230\021\330\014\020\220\010\230\001\330\020\034\230D\240\005\240Q\240e\2501\330\020\023\2201\330\024\036\230g\240Q\240a\330\014\020\220\007\320\027.\250a\340\010\014\210H\220D\230\007\230q\330\014\017\210v\220S\230\001\330\020\027\220q\330\010\017\210q";
static const char __pyx_k_A_5_1_U_4q_d_y_5_A_HC_8_L_a_XS_A[] = "\200A\360$\000\t\034\2305\240\001\240\021\330\0101\260\021\360\016\000\t\016\210U\220!\2204\220q\340\010\016\210d\220%\220y\240\001\330\014\033\2305\240\004\240A\340\014\020\220\014\230H\240C\320'8\270\004\270L\310\006\310a\330\024 \240\005\240X\250S\260\001\330\020\034\320\034,\250A\250\\\270\027\300\001\300\021\330\020\035\320\035-\250Q\250n\270A\270Q\330\020\023\220:\230T\240\021\330\024#\240<\250r\260\021\330\024!\240\030\250\026\250q\260\r\270R\270q\330\024\027\220{\240#\240Q\330\030\036\230k\250\021\250!\330\024\032\230!\230<\240{\260!\330\024\032\230!\230;\240b\250\r\260\\\300\035\310b\320PQ\330\024(\250\001\250\036\260~\300\\\320Q_\320_`\330\024&\240a\240|\2601\330\024\030\230\001\230\021\340\014\017\210|\2306\240\027\250\001\330\020\025\220U\230!\230<\240q\340\014\017\210|\2307\240'\250\021\330\020\025\220U\230!\230<\240q";
static const char __pyx_k_A_A_A_7_Q_aq_Qk_7_Q_aq_Qk_Zq_9Kq[] = "\200A\360\020\000\016\017\330\014\025\220]\240!\330\014\025\220[\240\001\240\030\320)A\300\023\300A\340\010\013\2107\220#\220Q\330\014\022\220,\230a\230q\340\r\016\330\014\030\320\030*\250!\330\014\025\320\025%\240Q\240k\260\021\340\010\013\2107\220#\220Q\330\014\022\220,\230a\230q\340\010\017\320\017%\240Q\240k\260\024\260Z\270q\330\010\"\240!\2409\250K\260q\330\010\021\220\035\230a\340\010\030\230\n\240!\330\014\r\330\014\020\220\001\330\014\020\220\001\330\014\020\220\001\330\014\020\220\001\330\014\020\220\001\340\010\017\210q";
static const char __pyx_k_A_Qhe7_Qd_Oq_x_Ql_AT_q_q_g_AXX_A[] = "\200A\360:\000\t\014\210:\220Q\220h\230e\2407\250!\330\014\030\230\010\240\001\240\021\330\014\023\320\023%\240Q\240d\250'\260\034\320=O\310q\330\014 \240\001\240\026\240x\250{\270#\270Q\270l\310!\330\014$\240A\240T\250\027\260\001\330\r\027\220q\230\007\230q\330\014\023\320\023)\250\021\250$\250g\260\\\320AX\320X]\320]^\330\014$\240A\240T\250\027\260\001\340\014\022\220)\2301\320\034O\310r\320QU\320UV\320V\\\320\\]";
static const char __pyx_k_A_Qhe7_Qd_Oq_x_Ql_Qd_q_q_g_AXX_Q[] = "\200A\360:\000\t\014\210:\220Q\220h\230e\2407\250!\330\014\031\230\030\240\021\240!\330\014\023\320\023%\240Q\240d\250'\260\034\320=O\310q\330\014 \240\001\240\026\240x\250{\270#\270Q\270l\310!\330\014%\240Q\240d\250'\260\021\330\r\027\220q\230\007\230q\330\014\023\320\023)\250\021\250$\250g\260\\\320AX\320X]\320]^\330\014%\240Q\240d\250'\260\021\340\014\022\220)\2301\320\034O\310r\320QU\320UV\320V\\\320\\]";
static const char __pyx_k_A_dict_like_object_that_is_simil[] = "A dict-like object that is similar to the ``attributes`` property, but operates directly on the Node data.\n\n        .. warning:: Use ``attributes`` instead, if you don't want to modify Node attributes.\n\n        Returns\n        -------\n        attributes : Attributes mapping object.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser(\"<div id='a'></div>\")\n        >>> node = tree.css_first('div')\n        >>> node.attrs\n        <div attributes, 1 items>\n        >>> node.attrs['id']\n        'a'\n        >>> node.attrs['foo'] = 'bar'\n        >>> del node.attrs['id']\n        >>> node.attributes\n        {'foo': 'bar'}\n        >>> node.attrs['id'] = 'new_id'\n        >>> node.html\n        '<div foo=\"bar\" id=\"new_id\"></div>'\n        ";
static const char __pyx_k_Can_t_obtain_raw_value_for_non_t[] = "Can't obtain raw value for non-text node.";
static const char __pyx_k_Expected_1_match_but_found_s_mat[] = "Expected 1 match, but found %s matches";
static const char __pyx_k_Expected_a_string_or_Node_instan[] = "Expected a string or Node instance, but %s found";
static const char __pyx_k_Failed_to_allocate_memory_for_st[] = "Failed to allocate memory for stack";
static const char __pyx_k_Function_call_with_ambiguous_arg[] = "Function call with ambiguous argument types";
static const char __pyx_k_Get_all_attributes_that_belong_t[] = "Get all attributes that belong to the current node.\n\n        The value of empty attributes is None.\n\n        Returns\n        -------\n        attributes : dictionary of all attributes.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser(\"<div data id='my_id'></div>\")\n        >>> node = tree.css_first('div')\n        >>> node.attributes\n        {'data': None, 'id': 'my_id'}\n        ";
static const char __pyx_k_HTMLParser_merge_text_nodes_line[] = "HTMLParser.merge_text_nodes (line 360)";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0[] = "Incompatible checksums (0x%x vs (0xc5dfe7e, 0xad8d8ab, 0xc00154f) = (node, nodes))";
static const char __pyx_k_Iterates_over_all_text_nodes_and[] = "Iterates over all text nodes and merges all text nodes that are close to each other.\n\n        This is useful for text extraction.\n        Use it when you need to strip HTML tags and merge \"dangling\" text.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser(\"<div><p><strong>J</strong>ohn</p><p>Doe</p></div>\")\n        >>> node = tree.css_first('div')\n        >>> tree.unwrap_tags([\"strong\"])\n        >>> tree.text(deep=True, separator=\" \", strip=True)\n        \"J ohn Doe\" # Text extraction produces an extra space because the strong tag was removed.\n        >>> node.merge_text_nodes()\n        >>> tree.text(deep=True, separator=\" \", strip=True)\n        \"John Doe\"\n        ";
static const char __pyx_k_Node_attributes___get___line_158[] = "Node.attributes.__get__ (line 158)";
static const char __pyx_k_Note_that_Cython_is_deliberately[] = "Note that Cython is deliberately stricter than PEP-484 and rejects subclasses of builtin types. If you need to pass subclasses then set the 'annotation_typing' directive to False.";
static const char __pyx_k_Remove_a_Node_from_the_tree_Para[] = "Remove a Node from the tree.\n\n        Parameters\n        ----------\n        recursive : bool, default True\n            Whenever to delete all its child nodes\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser(html)\n        >>> for tag in tree.css('script'):\n        >>>     tag.decompose()\n\n        ";
static const char __pyx_k_Remove_specified_tags_from_the_H[] = "Remove specified tags from the HTML tree.\n\n        Parameters\n        ----------\n        tags : list\n            List of tags to remove.\n        recursive : bool, default True\n            Whenever to delete all its child nodes\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser('<html><head></head><body><script></script><div>Hello world!</div></body></html>')\n        >>> tags = ['head', 'style', 'script', 'xmp', 'iframe', 'noembed', 'noframes']\n        >>> tree.strip_tags(tags)\n        >>> tree.html\n        '<html><body><div>Hello world!</div></body></html>'\n\n        ";
static const char __pyx_k_Remove_specified_tags_from_the_n[] = "Remove specified tags from the node.\n\n        Parameters\n        ----------\n        tags : list of str\n            List of tags to remove.\n        recursive : bool, default True\n            Whenever to delete all its child nodes\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser('<html><head></head><body><script></script><div>Hello world!</div></body></html>')\n        >>> tags = ['head', 'style', 'script', 'xmp', 'iframe', 'noembed', 'noframes']\n        >>> tree.strip_tags(tags)\n        >>> tree.html\n        '<html><body><div>Hello world!</div></body></html>'\n\n        ";
static const char __pyx_k_Replace_current_Node_with_specif[] = "Replace current Node with specified value.\n\n        Parameters\n        ----------\n        value : str, bytes or Node\n            The text or Node instance to replace the Node with.\n            When a text string is passed, it's treated as text. All HTML tags will be escaped.\n            Convert and pass the ``Node`` object when you want to work with HTML.\n            Does not clone the ``Node`` object.\n            All future changes to the passed ``Node`` object will also be taken into account.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser('<div>Get <img src=\"\" alt=\"Laptop\"></div>')\n        >>> img = tree.css_first('img')\n        >>> img.replace_with(img.attributes.get('alt', ''))\n        >>> tree.body.child.html\n        '<div>Get Laptop</div>'\n\n        >>> html_parser = HTMLParser('<div>Get <span alt=\"Laptop\"><img src=\"/jpg\"> <div></div></span></div>')\n        >>> html_parser2 = HTMLParser('<div>Test</div>')\n        >>> img_node = html_parser.css_first('img')\n        >>> img_node.replace_with(html_parser2.body.child)\n        '<div>Get <span alt=\"Laptop\"><div>Test</div> <div></div></span></div>'\n        ";
static const char __pyx_k_Replace_node_with_whatever_is_in[] = "Replace node with whatever is inside this node.\n\n        Parameters\n        ----------\n        delete_empty : bool, default False\n            Whenever to delete empty tags.\n\n        Examples\n        --------\n\n        >>>  tree = HTMLParser(\"<div>Hello <i>world</i>!</div>\")\n        >>>  tree.css_first('i').unwrap()\n        >>>  tree.html\n        '<html><head></head><body><div>Hello world!</div></body></html>'\n\n        Note: by default, empty tags are ignored, set \"delete_empty\" to \"True\" to change this.\n        ";
static const char __pyx_k_Return_the_raw_unparsed_original[] = "Return the raw (unparsed, original) value of a node.\n\n        Currently, works on text nodes only.\n\n        Returns\n        -------\n\n        raw_value : bytes\n\n        Examples\n        --------\n\n        >>> html_parser = HTMLParser('<div>&#x3C;test&#x3E;</div>')\n        >>> selector = html_parser.css_first('div')\n        >>> selector.child.html\n        '&lt;test&gt;'\n        >>> selector.child.raw_value\n        b'&#x3C;test&#x3E;'\n        ";
static const char __pyx_k_Selector_any_attribute_longer_th[] = "Selector.any_attribute_longer_than";
static const char __pyx_k_html_tree_is_still_NULL_even_aft[] = "html_tree is still NULL even after parsing ";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_self_html_tree_cannot_be_convert[] = "self.html_tree cannot be converted to a Python object for pickling";
static const char __pyx_k_self_node_cannot_be_converted_to[] = "self.node cannot be converted to a Python object for pickling";
static const char __pyx_k_Unwraps_specified_tags_from_the_2[] = "Unwraps specified tags from the HTML tree.\n\n        Works the same as th ``unwrap`` method, but applied to a list of tags.\n\n        Parameters\n        ----------\n        tags : list\n            List of tags to remove.\n        delete_empty : bool, default False\n            Whenever to delete empty tags.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser(\"<div><a href=\"\">Hello</a> <i>world</i>!</div>\")\n        >>> tree.body.unwrap_tags(['i','a'])\n        >>> tree.body.html\n        '<body><div>Hello world!</div></body>'\n\n        Note: by default, empty tags are ignored, set \"delete_empty\" to \"True\" to change this.\n        ";
static const char __pyx_k_Unwraps_specified_tags_from_the_3[] = "Unwraps specified tags from the HTML tree.\n\n        Works the same as th `unwrap` method, but applied to a list of tags.\n\n        Parameters\n        ----------\n        tags : list\n            List of tags to remove.\n        delete_empty : bool, default False\n            If True, removes empty tags.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser(\"<div><a href=\"\">Hello</a> <i>world</i>!</div>\")\n        >>> tree.head.unwrap_tags(['i','a'])\n        >>> tree.head.html\n        '<body><div>Hello world!</div></body>'\n        ";
/* #### Code section: decls ### */
static int __pyx_pf_10selectolax_6parser_11CSSSelector___init__(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self, PyObject *__pyx_v_selector); /* proto */
static void __pyx_pf_10selectolax_6parser_11CSSSelector_2__dealloc__(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11CSSSelector_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11CSSSelector_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_10selectolax_6parser_8Selector___init__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node, PyObject *__pyx_v_query); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8Selector_2css(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_query); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8Selector_7matches___get__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8Selector_11any_matches___get__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8Selector_4text_contains(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_text, PyLongObject *__pyx_v_deep, PyObject *__pyx_v_separator, PyLongObject *__pyx_v_strip); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8Selector_6any_text_contains(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_text, PyLongObject *__pyx_v_deep, PyObject *__pyx_v_separator, PyLongObject *__pyx_v_strip); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8Selector_8attribute_longer_than(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_attribute, int __pyx_v_length, PyObject *__pyx_v_start); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8Selector_10any_attribute_longer_than(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_attribute, int __pyx_v_length, PyObject *__pyx_v_start); /* proto */
static int __pyx_pf_10selectolax_6parser_8Selector_12__bool__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8Selector_14__reduce_cython__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8Selector_16__setstate_cython__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_10selectolax_6parser_5Stack___cinit__(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self, size_t __pyx_v_capacity); /* proto */
static void __pyx_pf_10selectolax_6parser_5Stack_2__dealloc__(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_5Stack_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_5Stack_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes___iter__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self); /* proto */
static int __pyx_pf_10selectolax_6parser_11_Attributes_3__setitem__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10selectolax_6parser_11_Attributes_5__delitem__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_7__getitem__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static Py_ssize_t __pyx_pf_10selectolax_6parser_11_Attributes_9__len__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_11keys(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_13items(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_16values(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_19get(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_21sget(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default); /* proto */
static int __pyx_pf_10selectolax_6parser_11_Attributes_23__contains__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_25__repr__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_27__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_29__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_10attributes___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_5attrs___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_6mem_id___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_2id___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static Py_hash_t __pyx_pf_10selectolax_6parser_4Node___hash__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_2text(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyLongObject *__pyx_v_deep, PyObject *__pyx_v_separator, PyLongObject *__pyx_v_strip); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_4iter(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_include_text); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_7traverse(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_include_text); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_3tag___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_5child___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_6parent___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_4next___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_4prev___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_10last_child___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_4html___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_10css(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_query); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_12any_css_matches(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_selectors); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_14css_matches(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_selector); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_16css_first(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_query, PyObject *__pyx_v_default, PyLongObject *__pyx_v_strict); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_18decompose(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyLongObject *__pyx_v_recursive); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_20remove(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyLongObject *__pyx_v_recursive); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_22unwrap(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_delete_empty); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_24strip_tags(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_tags, PyLongObject *__pyx_v_recursive); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_26unwrap_tags(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_tags, PyObject *__pyx_v_delete_empty); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_28replace_with(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults, PyObject *__pyx_v__fused_sigindex_ref); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_50replace_with(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_52replace_with(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_54replace_with(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_30insert_before(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults, PyObject *__pyx_v__fused_sigindex_ref); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_58insert_before(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_60insert_before(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_62insert_before(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_32insert_after(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults, PyObject *__pyx_v__fused_sigindex_ref); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_66insert_after(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_68insert_after(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_70insert_after(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_34insert_child(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults, PyObject *__pyx_v__fused_sigindex_ref); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_74insert_child(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_76insert_child(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_78insert_child(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_36unwrap_tags(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_tags, PyObject *__pyx_v_delete_empty); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_9raw_value___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_38select(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_query); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_40scripts_contain(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_query); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_42script_srcs_contain(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_queries); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_44__repr__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_46__eq__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_12text_content___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_48merge_text_nodes(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_6parser___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static int __pyx_pf_10selectolax_6parser_4Node_6parser_2__set__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10selectolax_6parser_4Node_6parser_4__del__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_82__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4Node_84__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_preprocess_input(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_decode_errors); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_2do_create_tag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tag, PyObject *__pyx_v_parser_cls); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_4get_fragment_type(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_parser_cls, PyObject *__pyx_v_tree); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_6do_parse_fragment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_parser_cls); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_8create_tag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10parse_fragment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_12preprocess_input(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_decode_errors); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_14do_create_tag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tag, PyObject *__pyx_v_parser_cls); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_16get_fragment_type(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_parser_cls, PyObject *__pyx_v_tree); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_18do_parse_fragment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_parser_cls); /* proto */
static int __pyx_pf_10selectolax_6parser_10HTMLParser___init__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_html, PyObject *__pyx_v_detect_encoding, PyObject *__pyx_v_use_meta_tags, PyObject *__pyx_v_decode_errors); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_2css(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_query); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4css_first(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_query, PyObject *__pyx_v_default, PyObject *__pyx_v_strict); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_14input_encoding___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4root___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4head___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4body___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_6tags(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_8text(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyLongObject *__pyx_v_deep, PyObject *__pyx_v_separator, PyLongObject *__pyx_v_strip); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_10strip_tags(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_tags, PyLongObject *__pyx_v_recursive); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_12unwrap_tags(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_tags, PyObject *__pyx_v_delete_empty); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4html___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_14select(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_query); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_16any_css_matches(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_selectors); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_18scripts_contain(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_query); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_20script_srcs_contain(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_queries); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_22css_matches(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_selector); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_24merge_text_nodes(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_26clone(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static void __pyx_pf_10selectolax_6parser_10HTMLParser_28__dealloc__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_30__repr__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_15detect_encoding___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static int __pyx_pf_10selectolax_6parser_10HTMLParser_15detect_encoding_2__set__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_13use_meta_tags___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static int __pyx_pf_10selectolax_6parser_10HTMLParser_13use_meta_tags_2__set__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_13decode_errors___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static int __pyx_pf_10selectolax_6parser_10HTMLParser_13decode_errors_2__set__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10selectolax_6parser_10HTMLParser_13decode_errors_4__del__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_8raw_html___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static int __pyx_pf_10selectolax_6parser_10HTMLParser_8raw_html_2__set__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10selectolax_6parser_10HTMLParser_8raw_html_4__del__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_32__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_34__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_10selectolax_6parser_20__pyx_unpickle_Selector(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_10selectolax_6parser_HTMLParser(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser_Stack(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser_CSSSelector(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser_Selector(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser__Attributes(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser_Node(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct____iter__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_1_items(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_2_values(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_3_iter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_4_traverse(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  #ifdef __Pyx_Generator_USED
  PyTypeObject *__pyx_GeneratorType;
  #endif
  #ifdef __Pyx_IterableCoroutine_USED
  PyTypeObject *__pyx_IterableCoroutineType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineAwaitType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineType;
  #endif
  PyTypeObject *__pyx_ptype_7cpython_4bool_bool;
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  PyObject *__pyx_type_10selectolax_6parser_HTMLParser;
  PyObject *__pyx_type_10selectolax_6parser_Stack;
  PyObject *__pyx_type_10selectolax_6parser_CSSSelector;
  PyObject *__pyx_type_10selectolax_6parser_Selector;
  PyObject *__pyx_type_10selectolax_6parser__Attributes;
  PyObject *__pyx_type_10selectolax_6parser_Node;
  PyObject *__pyx_type_10selectolax_6parser___pyx_scope_struct____iter__;
  PyObject *__pyx_type_10selectolax_6parser___pyx_scope_struct_1_items;
  PyObject *__pyx_type_10selectolax_6parser___pyx_scope_struct_2_values;
  PyObject *__pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter;
  PyObject *__pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse;
  PyTypeObject *__pyx_ptype_10selectolax_6parser_HTMLParser;
  PyTypeObject *__pyx_ptype_10selectolax_6parser_Stack;
  PyTypeObject *__pyx_ptype_10selectolax_6parser_CSSSelector;
  PyTypeObject *__pyx_ptype_10selectolax_6parser_Selector;
  PyTypeObject *__pyx_ptype_10selectolax_6parser__Attributes;
  PyTypeObject *__pyx_ptype_10selectolax_6parser_Node;
  PyTypeObject *__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__;
  PyTypeObject *__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items;
  PyTypeObject *__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values;
  PyTypeObject *__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter;
  PyTypeObject *__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values;
  __Pyx_CachedCFunction __pyx_umethod_PyBytes_Type__encode;
  __Pyx_CachedCFunction __pyx_umethod_PyUnicode_Type__strip;
  PyObject *__pyx_k__3;
  PyObject *__pyx_k__6;
  PyObject *__pyx_k__7;
  PyObject *__pyx_k__8;
  PyObject *__pyx_tuple[10];
  PyObject *__pyx_codeobj_tab[80];
  PyObject *__pyx_string_tab[372];
  PyObject *__pyx_float_250e_7;
  PyObject *__pyx_int_0;
  PyObject *__pyx_int_1;
  PyObject *__pyx_int_100;
  PyObject *__pyx_int_181983403;
  PyObject *__pyx_int_201332047;
  PyObject *__pyx_int_207486590;
/* #### Code section: module_state_contents ### */

#if CYTHON_USE_FREELISTS
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *__pyx_freelist_10selectolax_6parser___pyx_scope_struct____iter__[8];
int __pyx_freecount_10selectolax_6parser___pyx_scope_struct____iter__;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *__pyx_freelist_10selectolax_6parser___pyx_scope_struct_1_items[8];
int __pyx_freecount_10selectolax_6parser___pyx_scope_struct_1_items;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *__pyx_freelist_10selectolax_6parser___pyx_scope_struct_2_values[8];
int __pyx_freecount_10selectolax_6parser___pyx_scope_struct_2_values;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *__pyx_freelist_10selectolax_6parser___pyx_scope_struct_3_iter[8];
int __pyx_freecount_10selectolax_6parser___pyx_scope_struct_3_iter;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *__pyx_freelist_10selectolax_6parser___pyx_scope_struct_4_traverse[8];
int __pyx_freecount_10selectolax_6parser___pyx_scope_struct_4_traverse;
#endif
/* CommonTypesMetaclass.module_state_decls */
PyTypeObject *__pyx_CommonTypesMetaclassType;

/* CachedMethodType.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedMethodType;
#endif

/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* IterNextPlain.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
PyObject *__Pyx_GetBuiltinNext_LimitedAPI_cache;
#endif

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_ __pyx_string_tab[0]
#define __pyx_kp_u_A_dict_like_object_that_is_simil __pyx_string_tab[1]
#define __pyx_n_u_Attributes __pyx_string_tab[2]
#define __pyx_n_u_Attributes___iter __pyx_string_tab[3]
#define __pyx_n_u_Attributes___reduce_cython __pyx_string_tab[4]
#define __pyx_n_u_Attributes___setstate_cython __pyx_string_tab[5]
#define __pyx_n_u_Attributes_get __pyx_string_tab[6]
#define __pyx_n_u_Attributes_items __pyx_string_tab[7]
#define __pyx_n_u_Attributes_keys __pyx_string_tab[8]
#define __pyx_n_u_Attributes_sget __pyx_string_tab[9]
#define __pyx_n_u_Attributes_values __pyx_string_tab[10]
#define __pyx_kp_u_Bad_CSS_Selectors_s __pyx_string_tab[11]
#define __pyx_n_u_CSSSelector __pyx_string_tab[12]
#define __pyx_n_u_CSSSelector___reduce_cython __pyx_string_tab[13]
#define __pyx_n_u_CSSSelector___setstate_cython __pyx_string_tab[14]
#define __pyx_kp_u_Can_t_allocate_memory_for_a_new __pyx_string_tab[15]
#define __pyx_kp_u_Can_t_init_MyCSS_Entry_object __pyx_string_tab[16]
#define __pyx_kp_u_Can_t_init_MyCSS_object __pyx_string_tab[17]
#define __pyx_kp_u_Can_t_init_MyHTML_Tree_object __pyx_string_tab[18]
#define __pyx_kp_u_Can_t_init_MyHTML_object __pyx_string_tab[19]
#define __pyx_kp_u_Can_t_obtain_raw_value_for_non_t __pyx_string_tab[20]
#define __pyx_kp_u_Can_t_parse_HTML_status_code_d __pyx_string_tab[21]
#define __pyx_kp_u_Expected_1_match_but_found_s_mat __pyx_string_tab[22]
#define __pyx_kp_u_Expected_a_string_but_s_found __pyx_string_tab[23]
#define __pyx_kp_u_Expected_a_string_or_Node_instan __pyx_string_tab[24]
#define __pyx_kp_u_Expected_at_least __pyx_string_tab[25]
#define __pyx_n_u_FRAGMENT __pyx_string_tab[26]
#define __pyx_kp_u_Failed_to_allocate_memory_for_st __pyx_string_tab[27]
#define __pyx_kp_u_Function_call_with_ambiguous_arg __pyx_string_tab[28]
#define __pyx_kp_u_Get_all_attributes_that_belong_t __pyx_string_tab[29]
#define __pyx_n_u_HTMLParser __pyx_string_tab[30]
#define __pyx_n_u_HTMLParser___reduce_cython __pyx_string_tab[31]
#define __pyx_n_u_HTMLParser___setstate_cython __pyx_string_tab[32]
#define __pyx_n_u_HTMLParser_any_css_matches __pyx_string_tab[33]
#define __pyx_kp_u_HTMLParser_chars_s __pyx_string_tab[34]
#define __pyx_n_u_HTMLParser_clone __pyx_string_tab[35]
#define __pyx_n_u_HTMLParser_css __pyx_string_tab[36]
#define __pyx_n_u_HTMLParser_css_first __pyx_string_tab[37]
#define __pyx_n_u_HTMLParser_css_matches __pyx_string_tab[38]
#define __pyx_n_u_HTMLParser_merge_text_nodes __pyx_string_tab[39]
#define __pyx_kp_u_HTMLParser_merge_text_nodes_line __pyx_string_tab[40]
#define __pyx_n_u_HTMLParser_script_srcs_contain __pyx_string_tab[41]
#define __pyx_n_u_HTMLParser_scripts_contain __pyx_string_tab[42]
#define __pyx_n_u_HTMLParser_select __pyx_string_tab[43]
#define __pyx_n_u_HTMLParser_strip_tags __pyx_string_tab[44]
#define __pyx_kp_u_HTMLParser_strip_tags_line_235 __pyx_string_tab[45]
#define __pyx_n_u_HTMLParser_tags __pyx_string_tab[46]
#define __pyx_n_u_HTMLParser_text __pyx_string_tab[47]
#define __pyx_n_u_HTMLParser_unwrap_tags __pyx_string_tab[48]
#define __pyx_kp_u_HTMLParser_unwrap_tags_line_277 __pyx_string_tab[49]
#define __pyx_n_u_IGNORECASE __pyx_string_tab[50]
#define __pyx_kp_u_Incompatible_checksums_0x_x_vs_0 __pyx_string_tab[51]
#define __pyx_kp_u_Insert_a_node_after_the_current __pyx_string_tab[52]
#define __pyx_kp_u_Insert_a_node_before_the_curren __pyx_string_tab[53]
#define __pyx_kp_u_Insert_a_node_inside_at_the_end __pyx_string_tab[54]
#define __pyx_kp_u_Iterates_over_all_text_nodes_and __pyx_string_tab[55]
#define __pyx_n_u_KeyError __pyx_string_tab[56]
#define __pyx_n_u_LexborHTMLParser __pyx_string_tab[57]
#define __pyx_n_u_Literal __pyx_string_tab[58]
#define __pyx_n_u_MAX_HTML_INPUT_SIZE __pyx_string_tab[59]
#define __pyx_n_u_MemoryError __pyx_string_tab[60]
#define __pyx_kp_u_No_matching_signature_found __pyx_string_tab[61]
#define __pyx_n_u_Node __pyx_string_tab[62]
#define __pyx_n_u_Node___reduce_cython __pyx_string_tab[63]
#define __pyx_n_u_Node___setstate_cython __pyx_string_tab[64]
#define __pyx_n_u_Node_any_css_matches __pyx_string_tab[65]
#define __pyx_kp_u_Node_attributes___get___line_158 __pyx_string_tab[66]
#define __pyx_kp_u_Node_attrs___get___line_194 __pyx_string_tab[67]
#define __pyx_n_u_Node_css __pyx_string_tab[68]
#define __pyx_n_u_Node_css_first __pyx_string_tab[69]
#define __pyx_n_u_Node_css_matches __pyx_string_tab[70]
#define __pyx_n_u_Node_decompose __pyx_string_tab[71]
#define __pyx_kp_u_Node_decompose_line_492 __pyx_string_tab[72]
#define __pyx_n_u_Node_insert_after __pyx_string_tab[73]
#define __pyx_n_u_Node_insert_after_Node __pyx_string_tab[74]
#define __pyx_n_u_Node_insert_after_bytes __pyx_string_tab[75]
#define __pyx_kp_u_Node_insert_after_line_688 __pyx_string_tab[76]
#define __pyx_n_u_Node_insert_after_str __pyx_string_tab[77]
#define __pyx_n_u_Node_insert_before __pyx_string_tab[78]
#define __pyx_n_u_Node_insert_before_Node __pyx_string_tab[79]
#define __pyx_n_u_Node_insert_before_bytes __pyx_string_tab[80]
#define __pyx_kp_u_Node_insert_before_line_648 __pyx_string_tab[81]
#define __pyx_n_u_Node_insert_before_str __pyx_string_tab[82]
#define __pyx_n_u_Node_insert_child __pyx_string_tab[83]
#define __pyx_n_u_Node_insert_child_Node __pyx_string_tab[84]
#define __pyx_n_u_Node_insert_child_bytes __pyx_string_tab[85]
#define __pyx_kp_u_Node_insert_child_line_728 __pyx_string_tab[86]
#define __pyx_n_u_Node_insert_child_str __pyx_string_tab[87]
#define __pyx_n_u_Node_iter __pyx_string_tab[88]
#define __pyx_n_u_Node_merge_text_nodes __pyx_string_tab[89]
#define __pyx_kp_u_Node_merge_text_nodes_line_921 __pyx_string_tab[90]
#define __pyx_kp_u_Node_raw_value___get___line_795 __pyx_string_tab[91]
#define __pyx_n_u_Node_remove __pyx_string_tab[92]
#define __pyx_n_u_Node_replace_with __pyx_string_tab[93]
#define __pyx_n_u_Node_replace_with_Node __pyx_string_tab[94]
#define __pyx_n_u_Node_replace_with_bytes __pyx_string_tab[95]
#define __pyx_kp_u_Node_replace_with_line_607 __pyx_string_tab[96]
#define __pyx_n_u_Node_replace_with_str __pyx_string_tab[97]
#define __pyx_kp_u_Node_s __pyx_string_tab[98]
#define __pyx_n_u_Node_script_srcs_contain __pyx_string_tab[99]
#define __pyx_n_u_Node_scripts_contain __pyx_string_tab[100]
#define __pyx_n_u_Node_select __pyx_string_tab[101]
#define __pyx_n_u_Node_strip_tags __pyx_string_tab[102]
#define __pyx_kp_u_Node_strip_tags_line_554 __pyx_string_tab[103]
#define __pyx_n_u_Node_text __pyx_string_tab[104]
#define __pyx_n_u_Node_traverse __pyx_string_tab[105]
#define __pyx_n_u_Node_unwrap __pyx_string_tab[106]
#define __pyx_kp_u_Node_unwrap_line_517 __pyx_string_tab[107]
#define __pyx_n_u_Node_unwrap_tags __pyx_string_tab[108]
#define __pyx_kp_u_Node_unwrap_tags_line_580 __pyx_string_tab[109]
#define __pyx_kp_u_Node_unwrap_tags_line_768 __pyx_string_tab[110]
#define __pyx_kp_u_Note_that_Cython_is_deliberately __pyx_string_tab[111]
#define __pyx_n_u_Optional __pyx_string_tab[112]
#define __pyx_kp_u_Optional_Parser __pyx_string_tab[113]
#define __pyx_n_u_Parser __pyx_string_tab[114]
#define __pyx_n_u_ParserCls __pyx_string_tab[115]
#define __pyx_n_u_PickleError __pyx_string_tab[116]
#define __pyx_kp_u_Remove_a_Node_from_the_tree_Para __pyx_string_tab[117]
#define __pyx_kp_u_Remove_specified_tags_from_the_H __pyx_string_tab[118]
#define __pyx_kp_u_Remove_specified_tags_from_the_n __pyx_string_tab[119]
#define __pyx_kp_u_Replace_current_Node_with_specif __pyx_string_tab[120]
#define __pyx_kp_u_Replace_node_with_whatever_is_in __pyx_string_tab[121]
#define __pyx_kp_u_Return_the_raw_unparsed_original __pyx_string_tab[122]
#define __pyx_n_u_RuntimeError __pyx_string_tab[123]
#define __pyx_n_u_Selector __pyx_string_tab[124]
#define __pyx_n_u_Selector___reduce_cython __pyx_string_tab[125]
#define __pyx_n_u_Selector___setstate_cython __pyx_string_tab[126]
#define __pyx_n_u_Selector_any_attribute_longer_th __pyx_string_tab[127]
#define __pyx_n_u_Selector_any_text_contains __pyx_string_tab[128]
#define __pyx_n_u_Selector_attribute_longer_than __pyx_string_tab[129]
#define __pyx_n_u_Selector_css __pyx_string_tab[130]
#define __pyx_n_u_Selector_text_contains __pyx_string_tab[131]
#define __pyx_n_u_Stack __pyx_string_tab[132]
#define __pyx_n_u_Stack___reduce_cython __pyx_string_tab[133]
#define __pyx_n_u_Stack___setstate_cython __pyx_string_tab[134]
#define __pyx_kp_u_Tag_name_cannot_be_empty __pyx_string_tab[135]
#define __pyx_kp_u_Tag_name_is_too_long __pyx_string_tab[136]
#define __pyx_kp_u_The_specified_HTML_input_is_too __pyx_string_tab[137]
#define __pyx_n_u_Type __pyx_string_tab[138]
#define __pyx_n_u_TypeError __pyx_string_tab[139]
#define __pyx_kp_u_UTF_8 __pyx_string_tab[140]
#define __pyx_n_u_Union __pyx_string_tab[141]
#define __pyx_kp_u_Unwraps_specified_tags_from_the __pyx_string_tab[142]
#define __pyx_kp_u_Unwraps_specified_tags_from_the_2 __pyx_string_tab[143]
#define __pyx_kp_u_Unwraps_specified_tags_from_the_3 __pyx_string_tab[144]
#define __pyx_n_u_ValueError __pyx_string_tab[145]
#define __pyx_kp_u__10 __pyx_string_tab[146]
#define __pyx_kp_u__11 __pyx_string_tab[147]
#define __pyx_kp_u__12 __pyx_string_tab[148]
#define __pyx_kp_u__2 __pyx_string_tab[149]
#define __pyx_kp_u__4 __pyx_string_tab[150]
#define __pyx_kp_u__5 __pyx_string_tab[151]
#define __pyx_kp_u__9 __pyx_string_tab[152]
#define __pyx_kp_u_add_note __pyx_string_tab[153]
#define __pyx_n_u_any_attribute_longer_than __pyx_string_tab[154]
#define __pyx_n_u_any_css_matches __pyx_string_tab[155]
#define __pyx_n_u_any_text_contains __pyx_string_tab[156]
#define __pyx_n_u_args __pyx_string_tab[157]
#define __pyx_kp_u_arguments_got __pyx_string_tab[158]
#define __pyx_n_u_asyncio_coroutines __pyx_string_tab[159]
#define __pyx_n_u_attr __pyx_string_tab[160]
#define __pyx_n_u_attribute __pyx_string_tab[161]
#define __pyx_n_u_attribute_longer_than __pyx_string_tab[162]
#define __pyx_n_u_attributes __pyx_string_tab[163]
#define __pyx_kp_u_attributes_2 __pyx_string_tab[164]
#define __pyx_n_u_attrs __pyx_string_tab[165]
#define __pyx_kp_u_body __pyx_string_tab[166]
#define __pyx_n_u_body_2 __pyx_string_tab[167]
#define __pyx_n_u_bool __pyx_string_tab[168]
#define __pyx_n_u_bytes __pyx_string_tab[169]
#define __pyx_n_u_bytes_html __pyx_string_tab[170]
#define __pyx_n_u_bytes_val __pyx_string_tab[171]
#define __pyx_n_u_c_text __pyx_string_tab[172]
#define __pyx_n_u_capacity __pyx_string_tab[173]
#define __pyx_n_u_class_getitem __pyx_string_tab[174]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[175]
#define __pyx_n_u_clone __pyx_string_tab[176]
#define __pyx_n_u_close __pyx_string_tab[177]
#define __pyx_n_u_cls __pyx_string_tab[178]
#define __pyx_n_u_collection __pyx_string_tab[179]
#define __pyx_n_u_compile __pyx_string_tab[180]
#define __pyx_n_u_create_tag __pyx_string_tab[181]
#define __pyx_n_u_css __pyx_string_tab[182]
#define __pyx_n_u_css_first __pyx_string_tab[183]
#define __pyx_n_u_css_matches __pyx_string_tab[184]
#define __pyx_n_u_current_node __pyx_string_tab[185]
#define __pyx_n_u_d __pyx_string_tab[186]
#define __pyx_n_u_decode __pyx_string_tab[187]
#define __pyx_n_u_decode_errors __pyx_string_tab[188]
#define __pyx_n_u_decompose __pyx_string_tab[189]
#define __pyx_n_u_deep __pyx_string_tab[190]
#define __pyx_n_u_default __pyx_string_tab[191]
#define __pyx_n_u_defaults __pyx_string_tab[192]
#define __pyx_n_u_delete_empty __pyx_string_tab[193]
#define __pyx_n_u_detect_encoding __pyx_string_tab[194]
#define __pyx_n_u_dict __pyx_string_tab[195]
#define __pyx_n_u_dict_2 __pyx_string_tab[196]
#define __pyx_kp_u_disable __pyx_string_tab[197]
#define __pyx_n_u_do_create_tag __pyx_string_tab[198]
#define __pyx_n_u_do_parse_fragment __pyx_string_tab[199]
#define __pyx_n_u_document __pyx_string_tab[200]
#define __pyx_n_u_document_no_body __pyx_string_tab[201]
#define __pyx_n_u_document_no_head __pyx_string_tab[202]
#define __pyx_n_u_document_no_head_no_body __pyx_string_tab[203]
#define __pyx_n_u_element __pyx_string_tab[204]
#define __pyx_kp_u_enable __pyx_string_tab[205]
#define __pyx_n_u_encode __pyx_string_tab[206]
#define __pyx_n_u_endswith __pyx_string_tab[207]
#define __pyx_n_u_errors __pyx_string_tab[208]
#define __pyx_n_u_final_length __pyx_string_tab[209]
#define __pyx_n_u_final_text __pyx_string_tab[210]
#define __pyx_n_u_find __pyx_string_tab[211]
#define __pyx_n_u_finditer __pyx_string_tab[212]
#define __pyx_n_u_frag_type __pyx_string_tab[213]
#define __pyx_n_u_fragment __pyx_string_tab[214]
#define __pyx_n_u_func __pyx_string_tab[215]
#define __pyx_n_u_fused_sigindex_ref __pyx_string_tab[216]
#define __pyx_kp_u_gc __pyx_string_tab[217]
#define __pyx_n_u_get __pyx_string_tab[218]
#define __pyx_n_u_get_fragment_type __pyx_string_tab[219]
#define __pyx_n_u_getitem __pyx_string_tab[220]
#define __pyx_n_u_getstate __pyx_string_tab[221]
#define __pyx_n_u_has_body __pyx_string_tab[222]
#define __pyx_n_u_has_head __pyx_string_tab[223]
#define __pyx_n_u_has_html __pyx_string_tab[224]
#define __pyx_kp_u_head __pyx_string_tab[225]
#define __pyx_n_u_head_2 __pyx_string_tab[226]
#define __pyx_n_u_head_and_body __pyx_string_tab[227]
#define __pyx_n_u_html __pyx_string_tab[228]
#define __pyx_kp_u_html_2 __pyx_string_tab[229]
#define __pyx_kp_u_html_body_head_er __pyx_string_tab[230]
#define __pyx_n_u_html_len __pyx_string_tab[231]
#define __pyx_n_u_html_re __pyx_string_tab[232]
#define __pyx_n_u_html_tree __pyx_string_tab[233]
#define __pyx_kp_u_html_tree_is_still_NULL_even_aft __pyx_string_tab[234]
#define __pyx_n_u_i __pyx_string_tab[235]
#define __pyx_n_u_ignore __pyx_string_tab[236]
#define __pyx_n_u_import __pyx_string_tab[237]
#define __pyx_n_u_include_text __pyx_string_tab[238]
#define __pyx_n_u_initializing __pyx_string_tab[239]
#define __pyx_n_u_insert_after __pyx_string_tab[240]
#define __pyx_n_u_insert_after_Node __pyx_string_tab[241]
#define __pyx_n_u_insert_after_bytes __pyx_string_tab[242]
#define __pyx_n_u_insert_after_str __pyx_string_tab[243]
#define __pyx_n_u_insert_before __pyx_string_tab[244]
#define __pyx_n_u_insert_before_Node __pyx_string_tab[245]
#define __pyx_n_u_insert_before_bytes __pyx_string_tab[246]
#define __pyx_n_u_insert_before_str __pyx_string_tab[247]
#define __pyx_n_u_insert_child __pyx_string_tab[248]
#define __pyx_n_u_insert_child_Node __pyx_string_tab[249]
#define __pyx_n_u_insert_child_bytes __pyx_string_tab[250]
#define __pyx_n_u_insert_child_str __pyx_string_tab[251]
#define __pyx_n_u_is_coroutine __pyx_string_tab[252]
#define __pyx_kp_u_isenabled __pyx_string_tab[253]
#define __pyx_n_u_items __pyx_string_tab[254]
#define __pyx_kp_u_items_2 __pyx_string_tab[255]
#define __pyx_n_u_iter __pyx_string_tab[256]
#define __pyx_n_u_iter_2 __pyx_string_tab[257]
#define __pyx_n_u_key __pyx_string_tab[258]
#define __pyx_n_u_keys __pyx_string_tab[259]
#define __pyx_n_u_kwargs __pyx_string_tab[260]
#define __pyx_n_u_left_length __pyx_string_tab[261]
#define __pyx_n_u_left_text __pyx_string_tab[262]
#define __pyx_n_u_length __pyx_string_tab[263]
#define __pyx_n_u_main __pyx_string_tab[264]
#define __pyx_n_u_match __pyx_string_tab[265]
#define __pyx_n_u_mem_id __pyx_string_tab[266]
#define __pyx_n_u_merge_text_nodes __pyx_string_tab[267]
#define __pyx_n_u_module __pyx_string_tab[268]
#define __pyx_n_u_myhtml __pyx_string_tab[269]
#define __pyx_n_u_n_results __pyx_string_tab[270]
#define __pyx_n_u_name __pyx_string_tab[271]
#define __pyx_n_u_name_2 __pyx_string_tab[272]
#define __pyx_n_u_new __pyx_string_tab[273]
#define __pyx_n_u_next __pyx_string_tab[274]
#define __pyx_n_u_next_node __pyx_string_tab[275]
#define __pyx_kp_u_no_default___reduce___due_to_non __pyx_string_tab[276]
#define __pyx_n_u_node __pyx_string_tab[277]
#define __pyx_n_u_node_src __pyx_string_tab[278]
#define __pyx_n_u_node_text __pyx_string_tab[279]
#define __pyx_n_u_nodes __pyx_string_tab[280]
#define __pyx_n_u_parse_fragment __pyx_string_tab[281]
#define __pyx_n_u_parser_cls __pyx_string_tab[282]
#define __pyx_n_u_pickle __pyx_string_tab[283]
#define __pyx_n_u_pop __pyx_string_tab[284]
#define __pyx_n_u_preprocess_input __pyx_string_tab[285]
#define __pyx_n_u_pybyte_name __pyx_string_tab[286]
#define __pyx_n_u_pyx_PickleError __pyx_string_tab[287]
#define __pyx_n_u_pyx_checksum __pyx_string_tab[288]
#define __pyx_n_u_pyx_result __pyx_string_tab[289]
#define __pyx_n_u_pyx_state __pyx_string_tab[290]
#define __pyx_n_u_pyx_type __pyx_string_tab[291]
#define __pyx_n_u_pyx_unpickle_Selector __pyx_string_tab[292]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[293]
#define __pyx_n_u_qualname __pyx_string_tab[294]
#define __pyx_n_u_queries __pyx_string_tab[295]
#define __pyx_n_u_query __pyx_string_tab[296]
#define __pyx_n_u_range __pyx_string_tab[297]
#define __pyx_n_u_re __pyx_string_tab[298]
#define __pyx_n_u_recursive __pyx_string_tab[299]
#define __pyx_n_u_reduce __pyx_string_tab[300]
#define __pyx_n_u_reduce_cython __pyx_string_tab[301]
#define __pyx_n_u_reduce_ex __pyx_string_tab[302]
#define __pyx_n_u_remove __pyx_string_tab[303]
#define __pyx_n_u_replace_with __pyx_string_tab[304]
#define __pyx_n_u_replace_with_Node __pyx_string_tab[305]
#define __pyx_n_u_replace_with_bytes __pyx_string_tab[306]
#define __pyx_n_u_replace_with_str __pyx_string_tab[307]
#define __pyx_n_u_result __pyx_string_tab[308]
#define __pyx_n_u_results __pyx_string_tab[309]
#define __pyx_n_u_return __pyx_string_tab[310]
#define __pyx_n_u_right_length __pyx_string_tab[311]
#define __pyx_n_u_right_text __pyx_string_tab[312]
#define __pyx_n_u_root __pyx_string_tab[313]
#define __pyx_n_u_script __pyx_string_tab[314]
#define __pyx_n_u_script_srcs_contain __pyx_string_tab[315]
#define __pyx_n_u_scripts_contain __pyx_string_tab[316]
#define __pyx_n_u_select __pyx_string_tab[317]
#define __pyx_kp_u_selectolax_modest_node_pxi __pyx_string_tab[318]
#define __pyx_kp_u_selectolax_modest_selection_pxi __pyx_string_tab[319]
#define __pyx_kp_u_selectolax_modest_util_pxi __pyx_string_tab[320]
#define __pyx_kp_u_selectolax_modest_utils_pxi __pyx_string_tab[321]
#define __pyx_n_u_selectolax_parser __pyx_string_tab[322]
#define __pyx_kp_u_selectolax_parser __pyx_string_tab[323]
#define __pyx_kp_u_selectolax_parser_pyx __pyx_string_tab[324]
#define __pyx_kp_u_selectolax_utils_pxi __pyx_string_tab[325]
#define __pyx_n_u_selector __pyx_string_tab[326]
#define __pyx_n_u_selectors __pyx_string_tab[327]
#define __pyx_n_u_self __pyx_string_tab[328]
#define __pyx_kp_u_self_css_entry_self_finder_self __pyx_string_tab[329]
#define __pyx_kp_u_self_html_tree_cannot_be_convert __pyx_string_tab[330]
#define __pyx_kp_u_self_node_cannot_be_converted_to __pyx_string_tab[331]
#define __pyx_n_u_send __pyx_string_tab[332]
#define __pyx_n_u_separator __pyx_string_tab[333]
#define __pyx_n_u_set_name __pyx_string_tab[334]
#define __pyx_n_u_setstate __pyx_string_tab[335]
#define __pyx_n_u_setstate_cython __pyx_string_tab[336]
#define __pyx_n_u_sget __pyx_string_tab[337]
#define __pyx_n_u_spec __pyx_string_tab[338]
#define __pyx_n_u_split __pyx_string_tab[339]
#define __pyx_n_u_src __pyx_string_tab[340]
#define __pyx_n_u_src_nodes __pyx_string_tab[341]
#define __pyx_n_u_stack __pyx_string_tab[342]
#define __pyx_n_u_start __pyx_string_tab[343]
#define __pyx_n_u_state __pyx_string_tab[344]
#define __pyx_n_u_status __pyx_string_tab[345]
#define __pyx_n_u_str __pyx_string_tab[346]
#define __pyx_n_u_strict __pyx_string_tab[347]
#define __pyx_kp_u_stringsource __pyx_string_tab[348]
#define __pyx_n_u_strip __pyx_string_tab[349]
#define __pyx_n_u_strip_tags __pyx_string_tab[350]
#define __pyx_n_u_tag __pyx_string_tab[351]
#define __pyx_n_u_tags __pyx_string_tab[352]
#define __pyx_n_u_test __pyx_string_tab[353]
#define __pyx_n_u_text __pyx_string_tab[354]
#define __pyx_n_u_text_contains __pyx_string_tab[355]
#define __pyx_n_u_text_nodes __pyx_string_tab[356]
#define __pyx_n_u_throw __pyx_string_tab[357]
#define __pyx_n_u_traverse __pyx_string_tab[358]
#define __pyx_n_u_tree __pyx_string_tab[359]
#define __pyx_n_u_typing __pyx_string_tab[360]
#define __pyx_kp_u_undef __pyx_string_tab[361]
#define __pyx_n_u_unknown __pyx_string_tab[362]
#define __pyx_n_u_unwrap __pyx_string_tab[363]
#define __pyx_n_u_unwrap_tags __pyx_string_tab[364]
#define __pyx_n_u_update __pyx_string_tab[365]
#define __pyx_n_u_use_meta_tags __pyx_string_tab[366]
#define __pyx_n_u_use_setstate __pyx_string_tab[367]
#define __pyx_kp_u_utf_8 __pyx_string_tab[368]
#define __pyx_n_u_val __pyx_string_tab[369]
#define __pyx_n_u_value __pyx_string_tab[370]
#define __pyx_n_u_values __pyx_string_tab[371]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser_HTMLParser);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser_HTMLParser);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser_Stack);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser_Stack);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser_CSSSelector);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser_CSSSelector);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser_Selector);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser_Selector);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser__Attributes);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser__Attributes);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser_Node);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser_Node);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct____iter__);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct_1_items);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct_2_values);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter);
  Py_CLEAR(clear_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse);
  Py_CLEAR(clear_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse);
  Py_CLEAR(clear_module_state->__pyx_k__3);
  Py_CLEAR(clear_module_state->__pyx_k__6);
  Py_CLEAR(clear_module_state->__pyx_k__7);
  Py_CLEAR(clear_module_state->__pyx_k__8);
  for (int i=0; i<10; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<80; ++i) { Py_CLEAR(clear_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<372; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  Py_CLEAR(clear_module_state->__pyx_float_250e_7);
  Py_CLEAR(clear_module_state->__pyx_int_0);
  Py_CLEAR(clear_module_state->__pyx_int_1);
  Py_CLEAR(clear_module_state->__pyx_int_100);
  Py_CLEAR(clear_module_state->__pyx_int_181983403);
  Py_CLEAR(clear_module_state->__pyx_int_201332047);
  Py_CLEAR(clear_module_state->__pyx_int_207486590);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser_HTMLParser);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser_HTMLParser);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser_Stack);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser_Stack);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser_CSSSelector);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser_CSSSelector);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser_Selector);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser_Selector);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser__Attributes);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser__Attributes);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser_Node);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser_Node);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct____iter__);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct_1_items);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct_2_values);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter);
  Py_VISIT(traverse_module_state->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse);
  Py_VISIT(traverse_module_state->__pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse);
  Py_VISIT(traverse_module_state->__pyx_k__3);
  Py_VISIT(traverse_module_state->__pyx_k__6);
  Py_VISIT(traverse_module_state->__pyx_k__7);
  Py_VISIT(traverse_module_state->__pyx_k__8);
  for (int i=0; i<10; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<80; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<372; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_float_250e_7);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_0);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_1);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_100);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_181983403);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_201332047);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_207486590);
  return 0;
}
#endif
/* #### Code section: module_code ### */

/* "selectolax/modest/selection.pxi":13
 *     cdef mycss_selectors_list_t *selectors_list
 * 
 *     def __init__(self, str selector):             # <<<<<<<<<<<<<<
 * 
 *         selector_pybyte = selector.encode('UTF-8')
*/

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_11CSSSelector_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_10selectolax_6parser_11CSSSelector_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_selector = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_selector,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 13, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 13, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < 0) __PYX_ERR(1, 13, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, i); __PYX_ERR(1, 13, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 13, __pyx_L3_error)
    }
    __pyx_v_selector = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 13, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.CSSSelector.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_selector), (&PyUnicode_Type), 1, "selector", 1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_11CSSSelector___init__(((struct __pyx_obj_10selectolax_6parser_CSSSelector *)__pyx_v_self), __pyx_v_selector);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_11CSSSelector___init__(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self, PyObject *__pyx_v_selector) {
  PyObject *__pyx_v_selector_pybyte = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char *__pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "selectolax/modest/selection.pxi":15
 *     def __init__(self, str selector):
 * 
 *         selector_pybyte = selector.encode('UTF-8')             # <<<<<<<<<<<<<<
 *         self.c_selector = selector_pybyte
 * 
*/
  if (unlikely(__pyx_v_selector == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(1, 15, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_selector); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_selector_pybyte = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":16
 * 
 *         selector_pybyte = selector.encode('UTF-8')
 *         self.c_selector = selector_pybyte             # <<<<<<<<<<<<<<
 * 
 *         # In order to propagate errors these methods should return no value
*/
  __pyx_t_2 = __Pyx_PyBytes_AsWritableString(__pyx_v_selector_pybyte); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(1, 16, __pyx_L1_error)
  __pyx_v_self->c_selector = __pyx_t_2;

  /* "selectolax/modest/selection.pxi":19
 * 
 *         # In order to propagate errors these methods should return no value
 *         self._create_css_parser()             # <<<<<<<<<<<<<<
 *         self._prepare_selector(self.css_entry, self.c_selector, len(self.c_selector))
 *         self.finder = modest_finder_create_simple()
*/
  __pyx_t_3 = __pyx_f_10selectolax_6parser_11CSSSelector__create_css_parser(__pyx_v_self); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 19, __pyx_L1_error)

  /* "selectolax/modest/selection.pxi":20
 *         # In order to propagate errors these methods should return no value
 *         self._create_css_parser()
 *         self._prepare_selector(self.css_entry, self.c_selector, len(self.c_selector))             # <<<<<<<<<<<<<<
 *         self.finder = modest_finder_create_simple()
 * 
*/
  __pyx_t_4 = __Pyx_ssize_strlen(__pyx_v_self->c_selector); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 20, __pyx_L1_error)
  __pyx_t_3 = __pyx_f_10selectolax_6parser_11CSSSelector__prepare_selector(__pyx_v_self, __pyx_v_self->css_entry, __pyx_v_self->c_selector, __pyx_t_4); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 20, __pyx_L1_error)

  /* "selectolax/modest/selection.pxi":21
 *         self._create_css_parser()
 *         self._prepare_selector(self.css_entry, self.c_selector, len(self.c_selector))
 *         self.finder = modest_finder_create_simple()             # <<<<<<<<<<<<<<
 * 
 *     cdef myhtml_collection_t* find(self, myhtml_tree_node_t* scope):
*/
  __pyx_v_self->finder = modest_finder_create_simple();

  /* "selectolax/modest/selection.pxi":13
 *     cdef mycss_selectors_list_t *selectors_list
 * 
 *     def __init__(self, str selector):             # <<<<<<<<<<<<<<
 * 
 *         selector_pybyte = selector.encode('UTF-8')
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.CSSSelector.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_selector_pybyte);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":23
 *         self.finder = modest_finder_create_simple()
 * 
 *     cdef myhtml_collection_t* find(self, myhtml_tree_node_t* scope):             # <<<<<<<<<<<<<<
 *         """Find all possible matches."""
 * 
*/

static myhtml_collection_t *__pyx_f_10selectolax_6parser_11CSSSelector_find(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self, myhtml_tree_node_t *__pyx_v_scope) {
  myhtml_collection_t *__pyx_v_collection;
  myhtml_collection_t *__pyx_r;

  /* "selectolax/modest/selection.pxi":28
 *         cdef myhtml_collection_t *collection
 * 
 *         collection = NULL             # <<<<<<<<<<<<<<
 *         modest_finder_by_selectors_list(self.finder, scope, self.selectors_list, &collection)
 * 
*/
  __pyx_v_collection = NULL;

  /* "selectolax/modest/selection.pxi":29
 * 
 *         collection = NULL
 *         modest_finder_by_selectors_list(self.finder, scope, self.selectors_list, &collection)             # <<<<<<<<<<<<<<
 * 
 *         return collection
*/
  (void)(modest_finder_by_selectors_list(__pyx_v_self->finder, __pyx_v_scope, __pyx_v_self->selectors_list, (&__pyx_v_collection)));

  /* "selectolax/modest/selection.pxi":31
 *         modest_finder_by_selectors_list(self.finder, scope, self.selectors_list, &collection)
 * 
 *         return collection             # <<<<<<<<<<<<<<
 * 
 *     cdef int _create_css_parser(self) except -1:
*/
  __pyx_r = __pyx_v_collection;
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":23
 *         self.finder = modest_finder_create_simple()
 * 
 *     cdef myhtml_collection_t* find(self, myhtml_tree_node_t* scope):             # <<<<<<<<<<<<<<
 *         """Find all possible matches."""
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":33
 *         return collection
 * 
 *     cdef int _create_css_parser(self) except -1:             # <<<<<<<<<<<<<<
 *         cdef mystatus_t status
 * 
*/

static int __pyx_f_10selectolax_6parser_11CSSSelector__create_css_parser(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self) {
  mystatus_t __pyx_v_status;
  mycss_t *__pyx_v_mycss;
  int __pyx_r;
  int __pyx_t_1;

  /* "selectolax/modest/selection.pxi":36
 *         cdef mystatus_t status
 * 
 *         cdef mycss_t *mycss = mycss_create()             # <<<<<<<<<<<<<<
 *         status = mycss_init(mycss)
 * 
*/
  __pyx_v_mycss = mycss_create();

  /* "selectolax/modest/selection.pxi":37
 * 
 *         cdef mycss_t *mycss = mycss_create()
 *         status = mycss_init(mycss)             # <<<<<<<<<<<<<<
 * 
 *         if status != 0:
*/
  __pyx_v_status = mycss_init(__pyx_v_mycss);

  /* "selectolax/modest/selection.pxi":39
 *         status = mycss_init(mycss)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't init MyCSS object.")
 *             return -1
*/
  __pyx_t_1 = (__pyx_v_status != 0);
  if (__pyx_t_1) {

    /* "selectolax/modest/selection.pxi":40
 * 
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't init MyCSS object.")             # <<<<<<<<<<<<<<
 *             return -1
 * 
*/
    PyErr_SetObject(__pyx_builtin_RuntimeError, __pyx_mstate_global->__pyx_kp_u_Can_t_init_MyCSS_object);

    /* "selectolax/modest/selection.pxi":41
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't init MyCSS object.")
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *         self.css_entry = mycss_entry_create()
*/
    __pyx_r = -1;
    goto __pyx_L0;

    /* "selectolax/modest/selection.pxi":39
 *         status = mycss_init(mycss)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't init MyCSS object.")
 *             return -1
*/
  }

  /* "selectolax/modest/selection.pxi":43
 *             return -1
 * 
 *         self.css_entry = mycss_entry_create()             # <<<<<<<<<<<<<<
 *         status = mycss_entry_init(mycss, self.css_entry)
 * 
*/
  __pyx_v_self->css_entry = mycss_entry_create();

  /* "selectolax/modest/selection.pxi":44
 * 
 *         self.css_entry = mycss_entry_create()
 *         status = mycss_entry_init(mycss, self.css_entry)             # <<<<<<<<<<<<<<
 * 
 *         if status != 0:
*/
  __pyx_v_status = mycss_entry_init(__pyx_v_mycss, __pyx_v_self->css_entry);

  /* "selectolax/modest/selection.pxi":46
 *         status = mycss_entry_init(mycss, self.css_entry)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't init MyCSS Entry object.")
 *             return -1
*/
  __pyx_t_1 = (__pyx_v_status != 0);
  if (__pyx_t_1) {

    /* "selectolax/modest/selection.pxi":47
 * 
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't init MyCSS Entry object.")             # <<<<<<<<<<<<<<
 *             return -1
 *         return 0
*/
    PyErr_SetObject(__pyx_builtin_RuntimeError, __pyx_mstate_global->__pyx_kp_u_Can_t_init_MyCSS_Entry_object);

    /* "selectolax/modest/selection.pxi":48
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't init MyCSS Entry object.")
 *             return -1             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
    __pyx_r = -1;
    goto __pyx_L0;

    /* "selectolax/modest/selection.pxi":46
 *         status = mycss_entry_init(mycss, self.css_entry)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't init MyCSS Entry object.")
 *             return -1
*/
  }

  /* "selectolax/modest/selection.pxi":49
 *             PyErr_SetObject(RuntimeError, "Can't init MyCSS Entry object.")
 *             return -1
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _prepare_selector(self, mycss_entry_t *css_entry, const char *selector, size_t selector_size) except -1:
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":33
 *         return collection
 * 
 *     cdef int _create_css_parser(self) except -1:             # <<<<<<<<<<<<<<
 *         cdef mystatus_t status
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":51
 *         return 0
 * 
 *     cdef int _prepare_selector(self, mycss_entry_t *css_entry, const char *selector, size_t selector_size) except -1:             # <<<<<<<<<<<<<<
 *         cdef mystatus_t out_status
 *         self.selectors_list = mycss_selectors_parse(mycss_entry_selectors(css_entry), myencoding_t.MyENCODING_UTF_8,
*/

static int __pyx_f_10selectolax_6parser_11CSSSelector__prepare_selector(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self, mycss_entry_t *__pyx_v_css_entry, char const *__pyx_v_selector, size_t __pyx_v_selector_size) {
  mystatus_t __pyx_v_out_status;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  char *__pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_prepare_selector", 0);

  /* "selectolax/modest/selection.pxi":53
 *     cdef int _prepare_selector(self, mycss_entry_t *css_entry, const char *selector, size_t selector_size) except -1:
 *         cdef mystatus_t out_status
 *         self.selectors_list = mycss_selectors_parse(mycss_entry_selectors(css_entry), myencoding_t.MyENCODING_UTF_8,             # <<<<<<<<<<<<<<
 *                                                     selector, selector_size, &out_status)
 * 
*/
  __pyx_v_self->selectors_list = mycss_selectors_parse(mycss_entry_selectors(__pyx_v_css_entry), MyENCODING_UTF_8, __pyx_v_selector, __pyx_v_selector_size, (&__pyx_v_out_status));

  /* "selectolax/modest/selection.pxi":56
 *                                                     selector, selector_size, &out_status)
 * 
 *         if (self.selectors_list == NULL) or (self.selectors_list.flags and MyCSS_SELECTORS_FLAGS_SELECTOR_BAD):             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(ValueError, "Bad CSS Selectors: %s" % self.c_selector.decode('utf-8'))
 *             return -1
*/
  __pyx_t_2 = (__pyx_v_self->selectors_list == NULL);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  if (__pyx_v_self->selectors_list->flags) {
  } else {
    __pyx_t_1 = __pyx_v_self->selectors_list->flags;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_1 = MyCSS_SELECTORS_FLAGS_SELECTOR_BAD;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/selection.pxi":57
 * 
 *         if (self.selectors_list == NULL) or (self.selectors_list.flags and MyCSS_SELECTORS_FLAGS_SELECTOR_BAD):
 *             PyErr_SetObject(ValueError, "Bad CSS Selectors: %s" % self.c_selector.decode('utf-8'))             # <<<<<<<<<<<<<<
 *             return -1
 *         return 0
*/
    __pyx_t_3 = __pyx_v_self->c_selector;
    __pyx_t_4 = __Pyx_ssize_strlen(__pyx_t_3); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 57, __pyx_L1_error)
    __pyx_t_5 = __Pyx_decode_c_string(__pyx_t_3, 0, __pyx_t_4, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Bad_CSS_Selectors_s, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    PyErr_SetObject(__pyx_builtin_ValueError, __pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "selectolax/modest/selection.pxi":58
 *         if (self.selectors_list == NULL) or (self.selectors_list.flags and MyCSS_SELECTORS_FLAGS_SELECTOR_BAD):
 *             PyErr_SetObject(ValueError, "Bad CSS Selectors: %s" % self.c_selector.decode('utf-8'))
 *             return -1             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
    __pyx_r = -1;
    goto __pyx_L0;

    /* "selectolax/modest/selection.pxi":56
 *                                                     selector, selector_size, &out_status)
 * 
 *         if (self.selectors_list == NULL) or (self.selectors_list.flags and MyCSS_SELECTORS_FLAGS_SELECTOR_BAD):             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(ValueError, "Bad CSS Selectors: %s" % self.c_selector.decode('utf-8'))
 *             return -1
*/
  }

  /* "selectolax/modest/selection.pxi":59
 *             PyErr_SetObject(ValueError, "Bad CSS Selectors: %s" % self.c_selector.decode('utf-8'))
 *             return -1
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":51
 *         return 0
 * 
 *     cdef int _prepare_selector(self, mycss_entry_t *css_entry, const char *selector, size_t selector_size) except -1:             # <<<<<<<<<<<<<<
 *         cdef mystatus_t out_status
 *         self.selectors_list = mycss_selectors_parse(mycss_entry_selectors(css_entry), myencoding_t.MyENCODING_UTF_8,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser.CSSSelector._prepare_selector", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":61
 *         return 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         mycss_selectors_list_destroy(mycss_entry_selectors(self.css_entry), self.selectors_list, 1)
 *         modest_finder_destroy(self.finder, 1)
*/

/* Python wrapper */
static void __pyx_pw_10selectolax_6parser_11CSSSelector_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_10selectolax_6parser_11CSSSelector_3__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_10selectolax_6parser_11CSSSelector_2__dealloc__(((struct __pyx_obj_10selectolax_6parser_CSSSelector *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_10selectolax_6parser_11CSSSelector_2__dealloc__(struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self) {
  mycss_t *__pyx_v_mycss;
  mycss_t *__pyx_t_1;

  /* "selectolax/modest/selection.pxi":62
 * 
 *     def __dealloc__(self):
 *         mycss_selectors_list_destroy(mycss_entry_selectors(self.css_entry), self.selectors_list, 1)             # <<<<<<<<<<<<<<
 *         modest_finder_destroy(self.finder, 1)
 * 
*/
  (void)(mycss_selectors_list_destroy(mycss_entry_selectors(__pyx_v_self->css_entry), __pyx_v_self->selectors_list, 1));

  /* "selectolax/modest/selection.pxi":63
 *     def __dealloc__(self):
 *         mycss_selectors_list_destroy(mycss_entry_selectors(self.css_entry), self.selectors_list, 1)
 *         modest_finder_destroy(self.finder, 1)             # <<<<<<<<<<<<<<
 * 
 *         cdef mycss_t *mycss = self.css_entry.mycss
*/
  (void)(modest_finder_destroy(__pyx_v_self->finder, 1));

  /* "selectolax/modest/selection.pxi":65
 *         modest_finder_destroy(self.finder, 1)
 * 
 *         cdef mycss_t *mycss = self.css_entry.mycss             # <<<<<<<<<<<<<<
 *         mycss_entry_destroy(self.css_entry, 1)
 *         mycss_destroy(mycss, 1)
*/
  __pyx_t_1 = __pyx_v_self->css_entry->mycss;
  __pyx_v_mycss = __pyx_t_1;

  /* "selectolax/modest/selection.pxi":66
 * 
 *         cdef mycss_t *mycss = self.css_entry.mycss
 *         mycss_entry_destroy(self.css_entry, 1)             # <<<<<<<<<<<<<<
 *         mycss_destroy(mycss, 1)
 * 
*/
  (void)(mycss_entry_destroy(__pyx_v_self->css_entry, 1));

  /* "selectolax/modest/selection.pxi":67
 *         cdef mycss_t *mycss = self.css_entry.mycss
 *         mycss_entry_destroy(self.css_entry, 1)
 *         mycss_destroy(mycss, 1)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  (void)(mycss_destroy(__pyx_v_mycss, 1));

  /* "selectolax/modest/selection.pxi":61
 *         return 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         mycss_selectors_list_destroy(mycss_entry_selectors(self.css_entry), self.selectors_list, 1)
 *         modest_finder_destroy(self.finder, 1)
*/

  /* function exit code */
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11CSSSelector_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_11CSSSelector_4__reduce_cython__, "CSSSelector.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11CSSSelector_5__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11CSSSelector_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11CSSSelector_4__reduce_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_11CSSSelector_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_11CSSSelector_4__reduce_cython__(((struct __pyx_obj_10selectolax_6parser_CSSSelector *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11CSSSelector_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_self_css_entry_self_finder_self, 0, 0);
  __PYX_ERR(2, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.CSSSelector.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11CSSSelector_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_11CSSSelector_6__setstate_cython__, "CSSSelector.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11CSSSelector_7__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11CSSSelector_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11CSSSelector_6__setstate_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_11CSSSelector_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.CSSSelector.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_11CSSSelector_6__setstate_cython__(((struct __pyx_obj_10selectolax_6parser_CSSSelector *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11CSSSelector_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_self_css_entry_self_finder_self, 0, 0);
  __PYX_ERR(2, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.CSSSelector.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":80
 *     cdef list nodes
 * 
 *     def __init__(self, Node node, str query):             # <<<<<<<<<<<<<<
 *         """custom init, because __cinit__ doesn't accept C types"""
 *         self.node = node
*/

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_8Selector_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_8Selector___init__, "custom init, because __cinit__ doesn't accept C types");
#if CYTHON_UPDATE_DESCRIPTOR_DOC
struct wrapperbase __pyx_wrapperbase_10selectolax_6parser_8Selector___init__;
#endif
static int __pyx_pw_10selectolax_6parser_8Selector_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_v_query = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_node,&__pyx_mstate_global->__pyx_n_u_query,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 80, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 80, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 80, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < 0) __PYX_ERR(1, 80, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, i); __PYX_ERR(1, 80, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 80, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 80, __pyx_L3_error)
    }
    __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)values[0]);
    __pyx_v_query = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(1, 80, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Selector.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_node), __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, 1, "node", 0))) __PYX_ERR(1, 80, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query), (&PyUnicode_Type), 1, "query", 1))) __PYX_ERR(1, 80, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector___init__(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self), __pyx_v_node, __pyx_v_query);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_8Selector___init__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node, PyObject *__pyx_v_query) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "selectolax/modest/selection.pxi":82
 *     def __init__(self, Node node, str query):
 *         """custom init, because __cinit__ doesn't accept C types"""
 *         self.node = node             # <<<<<<<<<<<<<<
 *         self.nodes = find_nodes(node.parser, node.node, query) if query else [node, ]
 * 
*/
  __Pyx_INCREF((PyObject *)__pyx_v_node);
  __Pyx_GIVEREF((PyObject *)__pyx_v_node);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->node);
  __Pyx_DECREF((PyObject *)__pyx_v_self->node);
  __pyx_v_self->node = __pyx_v_node;

  /* "selectolax/modest/selection.pxi":83
 *         """custom init, because __cinit__ doesn't accept C types"""
 *         self.node = node
 *         self.nodes = find_nodes(node.parser, node.node, query) if query else [node, ]             # <<<<<<<<<<<<<<
 * 
 *     cpdef css(self, str query):
*/
  __pyx_t_2 = (__pyx_v_query != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_query) != 0);
  if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_2 < 0))) __PYX_ERR(1, 83, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = ((PyObject *)__pyx_v_node->parser);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = __pyx_f_10selectolax_6parser_find_nodes(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_3), __pyx_v_node->node, __pyx_v_query); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyList_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_4))) __PYX_ERR(1, 83, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_4;
    __pyx_t_4 = 0;
  } else {
    __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF((PyObject *)__pyx_v_node);
    __Pyx_GIVEREF((PyObject *)__pyx_v_node);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, ((PyObject *)__pyx_v_node)) != (0)) __PYX_ERR(1, 83, __pyx_L1_error);
    __pyx_t_1 = __pyx_t_4;
    __pyx_t_4 = 0;
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->nodes);
  __Pyx_DECREF(__pyx_v_self->nodes);
  __pyx_v_self->nodes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":80
 *     cdef list nodes
 * 
 *     def __init__(self, Node node, str query):             # <<<<<<<<<<<<<<
 *         """custom init, because __cinit__ doesn't accept C types"""
 *         self.node = node
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("selectolax.parser.Selector.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":85
 *         self.nodes = find_nodes(node.parser, node.node, query) if query else [node, ]
 * 
 *     cpdef css(self, str query):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current scope."""
 *         cdef Node current_node
*/

static PyObject *__pyx_pw_10selectolax_6parser_8Selector_3css(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_10selectolax_6parser_8Selector_css(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_query, int __pyx_skip_dispatch) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_current_node = 0;
  PyObject *__pyx_v_nodes = NULL;
  PyObject *__pyx_v_node = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("css", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_css); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 85, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_10selectolax_6parser_8Selector_3css)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_query};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 85, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "selectolax/modest/selection.pxi":88
 *         """Evaluate CSS selector against current scope."""
 *         cdef Node current_node
 *         nodes = list()             # <<<<<<<<<<<<<<
 *         for node in self.nodes:
 *             current_node = node
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_nodes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":89
 *         cdef Node current_node
 *         nodes = list()
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             current_node = node
 *             nodes.extend(find_nodes(self.node.parser, current_node.node, query))
*/
  if (unlikely(__pyx_v_self->nodes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 89, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->nodes; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_6 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 89, __pyx_L1_error)
      #endif
      if (__pyx_t_6 >= __pyx_temp) break;
    }
    __pyx_t_2 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_6);
    ++__pyx_t_6;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_node, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "selectolax/modest/selection.pxi":90
 *         nodes = list()
 *         for node in self.nodes:
 *             current_node = node             # <<<<<<<<<<<<<<
 *             nodes.extend(find_nodes(self.node.parser, current_node.node, query))
 *         self.nodes = nodes
*/
    __pyx_t_2 = __pyx_v_node;
    __Pyx_INCREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(1, 90, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_current_node, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "selectolax/modest/selection.pxi":91
 *         for node in self.nodes:
 *             current_node = node
 *             nodes.extend(find_nodes(self.node.parser, current_node.node, query))             # <<<<<<<<<<<<<<
 *         self.nodes = nodes
 *         return self
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_self->node->parser);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_4 = __pyx_f_10selectolax_6parser_find_nodes(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_2), __pyx_v_current_node->node, __pyx_v_query); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_7 = __Pyx_PyList_Extend(__pyx_v_nodes, __pyx_t_4); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(1, 91, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/modest/selection.pxi":89
 *         cdef Node current_node
 *         nodes = list()
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             current_node = node
 *             nodes.extend(find_nodes(self.node.parser, current_node.node, query))
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":92
 *             current_node = node
 *             nodes.extend(find_nodes(self.node.parser, current_node.node, query))
 *         self.nodes = nodes             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __Pyx_INCREF(__pyx_v_nodes);
  __Pyx_GIVEREF(__pyx_v_nodes);
  __Pyx_GOTREF(__pyx_v_self->nodes);
  __Pyx_DECREF(__pyx_v_self->nodes);
  __pyx_v_self->nodes = __pyx_v_nodes;

  /* "selectolax/modest/selection.pxi":93
 *             nodes.extend(find_nodes(self.node.parser, current_node.node, query))
 *         self.nodes = nodes
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":85
 *         self.nodes = find_nodes(node.parser, node.node, query) if query else [node, ]
 * 
 *     cpdef css(self, str query):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current scope."""
 *         cdef Node current_node
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("selectolax.parser.Selector.css", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_current_node);
  __Pyx_XDECREF(__pyx_v_nodes);
  __Pyx_XDECREF(__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_3css(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_8Selector_2css, "Selector.css(self, str query)\n\nEvaluate CSS selector against current scope.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_8Selector_3css = {"css", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_3css, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_2css};
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_3css(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_query = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("css (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_query,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 85, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 85, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "css", 0) < 0) __PYX_ERR(1, 85, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("css", 1, 1, 1, i); __PYX_ERR(1, 85, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 85, __pyx_L3_error)
    }
    __pyx_v_query = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("css", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 85, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Selector.css", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query), (&PyUnicode_Type), 1, "query", 1))) __PYX_ERR(1, 85, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_2css(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self), __pyx_v_query);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8Selector_2css(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_query) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("css", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10selectolax_6parser_8Selector_css(__pyx_v_self, __pyx_v_query, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.Selector.css", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":95
 *         return self
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def matches(self):
 *         """Returns all possible matches"""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_7matches_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_7matches_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_7matches___get__(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8Selector_7matches___get__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/selection.pxi":98
 *     def matches(self):
 *         """Returns all possible matches"""
 *         return self.nodes             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->nodes);
  __pyx_r = __pyx_v_self->nodes;
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":95
 *         return self
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def matches(self):
 *         """Returns all possible matches"""
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":100
 *         return self.nodes
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def any_matches(self):
 *         """Returns True if there are any matches"""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_11any_matches_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_11any_matches_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_11any_matches___get__(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8Selector_11any_matches___get__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/selection.pxi":103
 *     def any_matches(self):
 *         """Returns True if there are any matches"""
 *         return bool(self.nodes)             # <<<<<<<<<<<<<<
 * 
 *     def text_contains(self, str text, bool deep=True, str separator='', bool strip=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool);
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool); 
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_self->nodes};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 103, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":100
 *         return self.nodes
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def any_matches(self):
 *         """Returns True if there are any matches"""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Selector.any_matches.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":105
 *         return bool(self.nodes)
 * 
 *     def text_contains(self, str text, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Filter all current matches given text."""
 *         nodes = []
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_5text_contains(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_8Selector_4text_contains, "Selector.text_contains(self, str text, bool deep=True, str separator='', bool strip=False)\n\nFilter all current matches given text.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_8Selector_5text_contains = {"text_contains", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_5text_contains, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_4text_contains};
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_5text_contains(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_text = 0;
  PyLongObject *__pyx_v_deep = 0;
  PyObject *__pyx_v_separator = 0;
  PyLongObject *__pyx_v_strip = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("text_contains (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_text,&__pyx_mstate_global->__pyx_n_u_deep,&__pyx_mstate_global->__pyx_n_u_separator,&__pyx_mstate_global->__pyx_n_u_strip,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 105, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(1, 105, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 105, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 105, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 105, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "text_contains", 0) < 0) __PYX_ERR(1, 105, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject*)__pyx_mstate_global->__pyx_kp_u_));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("text_contains", 0, 1, 4, i); __PYX_ERR(1, 105, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(1, 105, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 105, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 105, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 105, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject*)__pyx_mstate_global->__pyx_kp_u_));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_text = ((PyObject*)values[0]);
    __pyx_v_deep = ((PyLongObject *)values[1]);
    __pyx_v_separator = ((PyObject*)values[2]);
    __pyx_v_strip = ((PyLongObject *)values[3]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("text_contains", 0, 1, 4, __pyx_nargs); __PYX_ERR(1, 105, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Selector.text_contains", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_text), (&PyUnicode_Type), 1, "text", 1))) __PYX_ERR(1, 105, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_deep), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "deep", 0))) __PYX_ERR(1, 105, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_separator), (&PyUnicode_Type), 1, "separator", 1))) __PYX_ERR(1, 105, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_strip), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "strip", 0))) __PYX_ERR(1, 105, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_4text_contains(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self), __pyx_v_text, __pyx_v_deep, __pyx_v_separator, __pyx_v_strip);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8Selector_4text_contains(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_text, PyLongObject *__pyx_v_deep, PyObject *__pyx_v_separator, PyLongObject *__pyx_v_strip) {
  PyObject *__pyx_v_nodes = NULL;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_v_node_text = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("text_contains", 0);

  /* "selectolax/modest/selection.pxi":107
 *     def text_contains(self, str text, bool deep=True, str separator='', bool strip=False):
 *         """Filter all current matches given text."""
 *         nodes = []             # <<<<<<<<<<<<<<
 *         cdef Node node
 *         for node in self.nodes:
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_nodes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":109
 *         nodes = []
 *         cdef Node node
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:
*/
  if (unlikely(__pyx_v_self->nodes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 109, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->nodes; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 109, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_2);
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(1, 109, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_node, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":110
 *         cdef Node node
 *         for node in self.nodes:
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)             # <<<<<<<<<<<<<<
 *             if node_text and text in node_text:
 *                 nodes.append(node)
*/
    __pyx_t_4 = ((PyObject *)__pyx_v_node);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_4, NULL};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 110, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_deep, ((PyObject *)__pyx_v_deep), __pyx_t_6, __pyx_callargs+1, 0) < 0) __PYX_ERR(1, 110, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_separator, __pyx_v_separator, __pyx_t_6, __pyx_callargs+1, 1) < 0) __PYX_ERR(1, 110, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_strip, ((PyObject *)__pyx_v_strip), __pyx_t_6, __pyx_callargs+1, 2) < 0) __PYX_ERR(1, 110, __pyx_L1_error)
      __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_text, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 110, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_node_text, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":111
 *         for node in self.nodes:
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:             # <<<<<<<<<<<<<<
 *                 nodes.append(node)
 *         self.nodes = nodes
*/
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_node_text); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 111, __pyx_L1_error)
    if (__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_text, __pyx_v_node_text, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 111, __pyx_L1_error)
    __pyx_t_7 = __pyx_t_8;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_7) {

      /* "selectolax/modest/selection.pxi":112
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:
 *                 nodes.append(node)             # <<<<<<<<<<<<<<
 *         self.nodes = nodes
 *         return self
*/
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_nodes, ((PyObject *)__pyx_v_node)); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(1, 112, __pyx_L1_error)

      /* "selectolax/modest/selection.pxi":111
 *         for node in self.nodes:
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:             # <<<<<<<<<<<<<<
 *                 nodes.append(node)
 *         self.nodes = nodes
*/
    }

    /* "selectolax/modest/selection.pxi":109
 *         nodes = []
 *         cdef Node node
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":113
 *             if node_text and text in node_text:
 *                 nodes.append(node)
 *         self.nodes = nodes             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __Pyx_INCREF(__pyx_v_nodes);
  __Pyx_GIVEREF(__pyx_v_nodes);
  __Pyx_GOTREF(__pyx_v_self->nodes);
  __Pyx_DECREF(__pyx_v_self->nodes);
  __pyx_v_self->nodes = __pyx_v_nodes;

  /* "selectolax/modest/selection.pxi":114
 *                 nodes.append(node)
 *         self.nodes = nodes
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def any_text_contains(self, str text, bool deep=True, str separator='', bool strip=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":105
 *         return bool(self.nodes)
 * 
 *     def text_contains(self, str text, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Filter all current matches given text."""
 *         nodes = []
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser.Selector.text_contains", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_nodes);
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XDECREF(__pyx_v_node_text);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":116
 *         return self
 * 
 *     def any_text_contains(self, str text, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Returns True if any node in the current search scope contains specified text"""
 *         nodes = []
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_7any_text_contains(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_8Selector_6any_text_contains, "Selector.any_text_contains(self, str text, bool deep=True, str separator='', bool strip=False)\n\nReturns True if any node in the current search scope contains specified text");
static PyMethodDef __pyx_mdef_10selectolax_6parser_8Selector_7any_text_contains = {"any_text_contains", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_7any_text_contains, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_6any_text_contains};
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_7any_text_contains(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_text = 0;
  PyLongObject *__pyx_v_deep = 0;
  PyObject *__pyx_v_separator = 0;
  PyLongObject *__pyx_v_strip = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("any_text_contains (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_text,&__pyx_mstate_global->__pyx_n_u_deep,&__pyx_mstate_global->__pyx_n_u_separator,&__pyx_mstate_global->__pyx_n_u_strip,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 116, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(1, 116, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 116, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 116, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 116, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "any_text_contains", 0) < 0) __PYX_ERR(1, 116, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject*)__pyx_mstate_global->__pyx_kp_u_));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("any_text_contains", 0, 1, 4, i); __PYX_ERR(1, 116, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(1, 116, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 116, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 116, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 116, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject*)__pyx_mstate_global->__pyx_kp_u_));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_text = ((PyObject*)values[0]);
    __pyx_v_deep = ((PyLongObject *)values[1]);
    __pyx_v_separator = ((PyObject*)values[2]);
    __pyx_v_strip = ((PyLongObject *)values[3]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("any_text_contains", 0, 1, 4, __pyx_nargs); __PYX_ERR(1, 116, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Selector.any_text_contains", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_text), (&PyUnicode_Type), 1, "text", 1))) __PYX_ERR(1, 116, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_deep), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "deep", 0))) __PYX_ERR(1, 116, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_separator), (&PyUnicode_Type), 1, "separator", 1))) __PYX_ERR(1, 116, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_strip), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "strip", 0))) __PYX_ERR(1, 116, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_6any_text_contains(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self), __pyx_v_text, __pyx_v_deep, __pyx_v_separator, __pyx_v_strip);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8Selector_6any_text_contains(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_text, PyLongObject *__pyx_v_deep, PyObject *__pyx_v_separator, PyLongObject *__pyx_v_strip) {
  CYTHON_UNUSED PyObject *__pyx_v_nodes = NULL;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_v_node_text = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("any_text_contains", 0);

  /* "selectolax/modest/selection.pxi":118
 *     def any_text_contains(self, str text, bool deep=True, str separator='', bool strip=False):
 *         """Returns True if any node in the current search scope contains specified text"""
 *         nodes = []             # <<<<<<<<<<<<<<
 *         cdef Node node
 *         for node in self.nodes:
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_nodes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":120
 *         nodes = []
 *         cdef Node node
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:
*/
  if (unlikely(__pyx_v_self->nodes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 120, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->nodes; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 120, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_2);
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 120, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(1, 120, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_node, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":121
 *         cdef Node node
 *         for node in self.nodes:
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)             # <<<<<<<<<<<<<<
 *             if node_text and text in node_text:
 *                 return True
*/
    __pyx_t_4 = ((PyObject *)__pyx_v_node);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_4, NULL};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_deep, ((PyObject *)__pyx_v_deep), __pyx_t_6, __pyx_callargs+1, 0) < 0) __PYX_ERR(1, 121, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_separator, __pyx_v_separator, __pyx_t_6, __pyx_callargs+1, 1) < 0) __PYX_ERR(1, 121, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_strip, ((PyObject *)__pyx_v_strip), __pyx_t_6, __pyx_callargs+1, 2) < 0) __PYX_ERR(1, 121, __pyx_L1_error)
      __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_text, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_node_text, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":122
 *         for node in self.nodes:
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
*/
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_node_text); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 122, __pyx_L1_error)
    if (__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_text, __pyx_v_node_text, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 122, __pyx_L1_error)
    __pyx_t_7 = __pyx_t_8;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_7) {

      /* "selectolax/modest/selection.pxi":123
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:
 *                 return True             # <<<<<<<<<<<<<<
 *         return False
 * 
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = Py_True;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "selectolax/modest/selection.pxi":122
 *         for node in self.nodes:
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
*/
    }

    /* "selectolax/modest/selection.pxi":120
 *         nodes = []
 *         cdef Node node
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             node_text = node.text(deep=deep, separator=separator, strip=strip)
 *             if node_text and text in node_text:
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":124
 *             if node_text and text in node_text:
 *                 return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     def attribute_longer_than(self, str attribute, int length, str start  = None):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":116
 *         return self
 * 
 *     def any_text_contains(self, str text, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Returns True if any node in the current search scope contains specified text"""
 *         nodes = []
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser.Selector.any_text_contains", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_nodes);
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XDECREF(__pyx_v_node_text);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":126
 *         return False
 * 
 *     def attribute_longer_than(self, str attribute, int length, str start  = None):             # <<<<<<<<<<<<<<
 *         """Filter all current matches by attribute length.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_9attribute_longer_than(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_8Selector_8attribute_longer_than, "Selector.attribute_longer_than(self, str attribute, int length, str start=None)\n\nFilter all current matches by attribute length.\n\nSimilar to `string-length` in XPath.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_8Selector_9attribute_longer_than = {"attribute_longer_than", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_9attribute_longer_than, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_8attribute_longer_than};
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_9attribute_longer_than(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_attribute = 0;
  int __pyx_v_length;
  PyObject *__pyx_v_start = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("attribute_longer_than (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_attribute,&__pyx_mstate_global->__pyx_n_u_length,&__pyx_mstate_global->__pyx_n_u_start,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 126, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 126, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 126, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 126, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "attribute_longer_than", 0) < 0) __PYX_ERR(1, 126, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject*)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("attribute_longer_than", 0, 2, 3, i); __PYX_ERR(1, 126, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 126, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 126, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 126, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject*)Py_None));
    }
    __pyx_v_attribute = ((PyObject*)values[0]);
    __pyx_v_length = __Pyx_PyLong_As_int(values[1]); if (unlikely((__pyx_v_length == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 126, __pyx_L3_error)
    __pyx_v_start = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("attribute_longer_than", 0, 2, 3, __pyx_nargs); __PYX_ERR(1, 126, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Selector.attribute_longer_than", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attribute), (&PyUnicode_Type), 1, "attribute", 1))) __PYX_ERR(1, 126, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyUnicode_Type), 1, "start", 1))) __PYX_ERR(1, 126, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_8attribute_longer_than(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self), __pyx_v_attribute, __pyx_v_length, __pyx_v_start);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8Selector_8attribute_longer_than(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_attribute, int __pyx_v_length, PyObject *__pyx_v_start) {
  PyObject *__pyx_v_nodes = NULL;
  PyObject *__pyx_v_node = NULL;
  PyObject *__pyx_v_attr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("attribute_longer_than", 0);

  /* "selectolax/modest/selection.pxi":131
 *         Similar to `string-length` in XPath.
 *         """
 *         nodes = []             # <<<<<<<<<<<<<<
 *         for node in self.nodes:
 *             attr = node.attributes.get(attribute)
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_nodes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":132
 *         """
 *         nodes = []
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:
*/
  if (unlikely(__pyx_v_self->nodes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 132, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->nodes; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 132, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_2);
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_node, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":133
 *         nodes = []
 *         for node in self.nodes:
 *             attr = node.attributes.get(attribute)             # <<<<<<<<<<<<<<
 *             if attr and start and start in attr:
 *                 attr = attr[attr.find(start) + len(start):]
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_mstate_global->__pyx_n_u_attributes); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_attribute};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_get, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 133, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_attr, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":134
 *         for node in self.nodes:
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:             # <<<<<<<<<<<<<<
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:
*/
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_attr); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 134, __pyx_L1_error)
    if (__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_start != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_start) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_8 < 0))) __PYX_ERR(1, 134, __pyx_L1_error)
    if (__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_start, __pyx_v_attr, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 134, __pyx_L1_error)
    __pyx_t_7 = __pyx_t_8;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_7) {

      /* "selectolax/modest/selection.pxi":135
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:
 *                 attr = attr[attr.find(start) + len(start):]             # <<<<<<<<<<<<<<
 *             if len(attr) > length:
 *                 nodes.append(node)
*/
      __pyx_t_5 = __pyx_v_attr;
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_start};
        __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_find, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 135, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      if (unlikely(__pyx_v_start == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(1, 135, __pyx_L1_error)
      }
      __pyx_t_9 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_start); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(1, 135, __pyx_L1_error)
      __pyx_t_5 = PyLong_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 135, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyNumber_Add(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 135, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_v_attr, 0, 0, &__pyx_t_4, NULL, NULL, 0, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 135, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_attr, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "selectolax/modest/selection.pxi":134
 *         for node in self.nodes:
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:             # <<<<<<<<<<<<<<
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:
*/
    }

    /* "selectolax/modest/selection.pxi":136
 *             if attr and start and start in attr:
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:             # <<<<<<<<<<<<<<
 *                 nodes.append(node)
 *         self.nodes = nodes
*/
    __pyx_t_9 = PyObject_Length(__pyx_v_attr); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(1, 136, __pyx_L1_error)
    __pyx_t_7 = (__pyx_t_9 > __pyx_v_length);
    if (__pyx_t_7) {

      /* "selectolax/modest/selection.pxi":137
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:
 *                 nodes.append(node)             # <<<<<<<<<<<<<<
 *         self.nodes = nodes
 *         return self
*/
      __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_nodes, __pyx_v_node); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(1, 137, __pyx_L1_error)

      /* "selectolax/modest/selection.pxi":136
 *             if attr and start and start in attr:
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:             # <<<<<<<<<<<<<<
 *                 nodes.append(node)
 *         self.nodes = nodes
*/
    }

    /* "selectolax/modest/selection.pxi":132
 *         """
 *         nodes = []
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":138
 *             if len(attr) > length:
 *                 nodes.append(node)
 *         self.nodes = nodes             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __Pyx_INCREF(__pyx_v_nodes);
  __Pyx_GIVEREF(__pyx_v_nodes);
  __Pyx_GOTREF(__pyx_v_self->nodes);
  __Pyx_DECREF(__pyx_v_self->nodes);
  __pyx_v_self->nodes = __pyx_v_nodes;

  /* "selectolax/modest/selection.pxi":139
 *                 nodes.append(node)
 *         self.nodes = nodes
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def any_attribute_longer_than(self, str attribute, int length, str start  = None):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":126
 *         return False
 * 
 *     def attribute_longer_than(self, str attribute, int length, str start  = None):             # <<<<<<<<<<<<<<
 *         """Filter all current matches by attribute length.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.Selector.attribute_longer_than", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_nodes);
  __Pyx_XDECREF(__pyx_v_node);
  __Pyx_XDECREF(__pyx_v_attr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":141
 *         return self
 * 
 *     def any_attribute_longer_than(self, str attribute, int length, str start  = None):             # <<<<<<<<<<<<<<
 *         """Returns True any href attribute longer than a specified length.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_11any_attribute_longer_than(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_8Selector_10any_attribute_longer_than, "Selector.any_attribute_longer_than(self, str attribute, int length, str start=None)\n\nReturns True any href attribute longer than a specified length.\n\nSimilar to `string-length` in XPath.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_8Selector_11any_attribute_longer_than = {"any_attribute_longer_than", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_11any_attribute_longer_than, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_10any_attribute_longer_than};
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_11any_attribute_longer_than(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_attribute = 0;
  int __pyx_v_length;
  PyObject *__pyx_v_start = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("any_attribute_longer_than (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_attribute,&__pyx_mstate_global->__pyx_n_u_length,&__pyx_mstate_global->__pyx_n_u_start,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 141, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 141, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 141, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 141, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "any_attribute_longer_than", 0) < 0) __PYX_ERR(1, 141, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject*)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("any_attribute_longer_than", 0, 2, 3, i); __PYX_ERR(1, 141, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 141, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 141, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 141, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject*)Py_None));
    }
    __pyx_v_attribute = ((PyObject*)values[0]);
    __pyx_v_length = __Pyx_PyLong_As_int(values[1]); if (unlikely((__pyx_v_length == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 141, __pyx_L3_error)
    __pyx_v_start = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("any_attribute_longer_than", 0, 2, 3, __pyx_nargs); __PYX_ERR(1, 141, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Selector.any_attribute_longer_than", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attribute), (&PyUnicode_Type), 1, "attribute", 1))) __PYX_ERR(1, 141, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyUnicode_Type), 1, "start", 1))) __PYX_ERR(1, 141, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_10any_attribute_longer_than(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self), __pyx_v_attribute, __pyx_v_length, __pyx_v_start);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8Selector_10any_attribute_longer_than(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v_attribute, int __pyx_v_length, PyObject *__pyx_v_start) {
  CYTHON_UNUSED PyObject *__pyx_v_nodes = 0;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_v_attr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("any_attribute_longer_than", 0);

  /* "selectolax/modest/selection.pxi":146
 *         Similar to `string-length` in XPath.
 *         """
 *         cdef list nodes = []             # <<<<<<<<<<<<<<
 *         cdef Node node
 *         for node in self.nodes:
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_nodes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":148
 *         cdef list nodes = []
 *         cdef Node node
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:
*/
  if (unlikely(__pyx_v_self->nodes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 148, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->nodes; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 148, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_2);
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(1, 148, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_node, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":149
 *         cdef Node node
 *         for node in self.nodes:
 *             attr = node.attributes.get(attribute)             # <<<<<<<<<<<<<<
 *             if attr and start and start in attr:
 *                 attr = attr[attr.find(start) + len(start):]
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_node), __pyx_mstate_global->__pyx_n_u_attributes); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_attribute};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_get, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 149, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_attr, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":150
 *         for node in self.nodes:
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:             # <<<<<<<<<<<<<<
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:
*/
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_attr); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 150, __pyx_L1_error)
    if (__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_start != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_start) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_8 < 0))) __PYX_ERR(1, 150, __pyx_L1_error)
    if (__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_start, __pyx_v_attr, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 150, __pyx_L1_error)
    __pyx_t_7 = __pyx_t_8;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_7) {

      /* "selectolax/modest/selection.pxi":151
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:
 *                 attr = attr[attr.find(start) + len(start):]             # <<<<<<<<<<<<<<
 *             if len(attr) > length:
 *                 return True
*/
      __pyx_t_5 = __pyx_v_attr;
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_start};
        __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_find, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 151, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      if (unlikely(__pyx_v_start == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(1, 151, __pyx_L1_error)
      }
      __pyx_t_9 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_start); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(1, 151, __pyx_L1_error)
      __pyx_t_5 = PyLong_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 151, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyNumber_Add(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 151, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_v_attr, 0, 0, &__pyx_t_4, NULL, NULL, 0, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 151, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_attr, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "selectolax/modest/selection.pxi":150
 *         for node in self.nodes:
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:             # <<<<<<<<<<<<<<
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:
*/
    }

    /* "selectolax/modest/selection.pxi":152
 *             if attr and start and start in attr:
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
*/
    __pyx_t_9 = PyObject_Length(__pyx_v_attr); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(1, 152, __pyx_L1_error)
    __pyx_t_7 = (__pyx_t_9 > __pyx_v_length);
    if (__pyx_t_7) {

      /* "selectolax/modest/selection.pxi":153
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:
 *                 return True             # <<<<<<<<<<<<<<
 *         return False
 * 
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = Py_True;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "selectolax/modest/selection.pxi":152
 *             if attr and start and start in attr:
 *                 attr = attr[attr.find(start) + len(start):]
 *             if len(attr) > length:             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
*/
    }

    /* "selectolax/modest/selection.pxi":148
 *         cdef list nodes = []
 *         cdef Node node
 *         for node in self.nodes:             # <<<<<<<<<<<<<<
 *             attr = node.attributes.get(attribute)
 *             if attr and start and start in attr:
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":154
 *             if len(attr) > length:
 *                 return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     def __bool__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":141
 *         return self
 * 
 *     def any_attribute_longer_than(self, str attribute, int length, str start  = None):             # <<<<<<<<<<<<<<
 *         """Returns True any href attribute longer than a specified length.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.Selector.any_attribute_longer_than", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_nodes);
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XDECREF(__pyx_v_attr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":156
 *         return False
 * 
 *     def __bool__(self):             # <<<<<<<<<<<<<<
 *         return bool(self.nodes)
 * 
*/

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_8Selector_13__bool__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10selectolax_6parser_8Selector_13__bool__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__bool__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_12__bool__(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_8Selector_12__bool__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__bool__", 0);

  /* "selectolax/modest/selection.pxi":157
 * 
 *     def __bool__(self):
 *         return bool(self.nodes)             # <<<<<<<<<<<<<<
 * 
 * cdef find_nodes(HTMLParser parser, myhtml_tree_node_t *node, str query):
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool);
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool); 
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_self->nodes};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 157, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_PyLong_As_int(((PyObject *)__pyx_t_1)); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 157, __pyx_L1_error)
  __Pyx_DECREF((PyObject *)__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":156
 *         return False
 * 
 *     def __bool__(self):             # <<<<<<<<<<<<<<
 *         return bool(self.nodes)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Selector.__bool__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_15__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_8Selector_14__reduce_cython__, "Selector.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_8Selector_15__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_15__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_14__reduce_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_15__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_14__reduce_cython__(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8Selector_14__reduce_cython__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.node, self.nodes)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
*/
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF((PyObject *)__pyx_v_self->node);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->node);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->node)) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->nodes);
  __Pyx_GIVEREF(__pyx_v_self->nodes);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->nodes) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.node, self.nodes)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
*/
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.node, self.nodes)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict) != (0)) __PYX_ERR(2, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.node is not None or self.nodes is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.node, self.nodes)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.node is not None or self.nodes is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, None), state
*/
  /*else*/ {
    __pyx_t_4 = (((PyObject *)__pyx_v_self->node) != Py_None);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->nodes != ((PyObject*)Py_None));
    __pyx_t_2 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.node is not None or self.nodes is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.node is not None or self.nodes is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Selector); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_207486590);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_207486590);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_207486590) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.node is not None or self.nodes is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, None), state
 *     else:
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Selector__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Selector); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_207486590);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_207486590);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_207486590) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.Selector.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Selector__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_17__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_8Selector_16__setstate_cython__, "Selector.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_8Selector_17__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_17__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_16__setstate_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_8Selector_17__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(2, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(2, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Selector.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_8Selector_16__setstate_cython__(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8Selector_16__setstate_cython__(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Selector__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_10selectolax_6parser___pyx_unpickle_Selector__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Selector__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.Selector.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":159
 *         return bool(self.nodes)
 * 
 * cdef find_nodes(HTMLParser parser, myhtml_tree_node_t *node, str query):             # <<<<<<<<<<<<<<
 *     cdef myhtml_collection_t *collection
 *     cdef CSSSelector selector = CSSSelector(query)
*/

static PyObject *__pyx_f_10selectolax_6parser_find_nodes(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_parser, myhtml_tree_node_t *__pyx_v_node, PyObject *__pyx_v_query) {
  myhtml_collection_t *__pyx_v_collection;
  struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_selector = 0;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_n = 0;
  PyObject *__pyx_v_result = 0;
  size_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  myhtml_collection_t *__pyx_t_5;
  int __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("find_nodes", 0);

  /* "selectolax/modest/selection.pxi":161
 * cdef find_nodes(HTMLParser parser, myhtml_tree_node_t *node, str query):
 *     cdef myhtml_collection_t *collection
 *     cdef CSSSelector selector = CSSSelector(query)             # <<<<<<<<<<<<<<
 *     cdef Node n
 *     cdef list result = []
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_CSSSelector);
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_CSSSelector); 
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_query};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 161, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_selector = ((struct __pyx_obj_10selectolax_6parser_CSSSelector *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":163
 *     cdef CSSSelector selector = CSSSelector(query)
 *     cdef Node n
 *     cdef list result = []             # <<<<<<<<<<<<<<
 *     collection = selector.find(node)
 * 
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":164
 *     cdef Node n
 *     cdef list result = []
 *     collection = selector.find(node)             # <<<<<<<<<<<<<<
 * 
 *     if collection == NULL:
*/
  __pyx_t_5 = __pyx_f_10selectolax_6parser_11CSSSelector_find(__pyx_v_selector, __pyx_v_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 164, __pyx_L1_error)
  __pyx_v_collection = __pyx_t_5;

  /* "selectolax/modest/selection.pxi":166
 *     collection = selector.find(node)
 * 
 *     if collection == NULL:             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
  __pyx_t_6 = (__pyx_v_collection == NULL);
  if (__pyx_t_6) {

    /* "selectolax/modest/selection.pxi":167
 * 
 *     if collection == NULL:
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     for i in range(collection.length):
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "selectolax/modest/selection.pxi":166
 *     collection = selector.find(node)
 * 
 *     if collection == NULL:             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
  }

  /* "selectolax/modest/selection.pxi":169
 *         return result
 * 
 *     for i in range(collection.length):             # <<<<<<<<<<<<<<
 *         n = Node.new(collection.list[i], parser)
 *         result.append(n)
*/
  __pyx_t_4 = __pyx_v_collection->length;
  __pyx_t_7 = __pyx_t_4;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "selectolax/modest/selection.pxi":170
 * 
 *     for i in range(collection.length):
 *         n = Node.new(collection.list[i], parser)             # <<<<<<<<<<<<<<
 *         result.append(n)
 *     myhtml_collection_destroy(collection)
*/
    __pyx_t_1 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new((__pyx_v_collection->list[__pyx_v_i]), __pyx_v_parser)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_n, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "selectolax/modest/selection.pxi":171
 *     for i in range(collection.length):
 *         n = Node.new(collection.list[i], parser)
 *         result.append(n)             # <<<<<<<<<<<<<<
 *     myhtml_collection_destroy(collection)
 *     return result
*/
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, ((PyObject *)__pyx_v_n)); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(1, 171, __pyx_L1_error)
  }

  /* "selectolax/modest/selection.pxi":172
 *         n = Node.new(collection.list[i], parser)
 *         result.append(n)
 *     myhtml_collection_destroy(collection)             # <<<<<<<<<<<<<<
 *     return result
 * 
*/
  (void)(myhtml_collection_destroy(__pyx_v_collection));

  /* "selectolax/modest/selection.pxi":173
 *         result.append(n)
 *     myhtml_collection_destroy(collection)
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":159
 *         return bool(self.nodes)
 * 
 * cdef find_nodes(HTMLParser parser, myhtml_tree_node_t *node, str query):             # <<<<<<<<<<<<<<
 *     cdef myhtml_collection_t *collection
 *     cdef CSSSelector selector = CSSSelector(query)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.find_nodes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_selector);
  __Pyx_XDECREF((PyObject *)__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/selection.pxi":176
 * 
 * 
 * cdef bool find_matches(HTMLParser parser, myhtml_tree_node_t *node, tuple selectors):             # <<<<<<<<<<<<<<
 *     cdef myhtml_collection_t *collection
 *     cdef CSSSelector selector
*/

static PyLongObject *__pyx_f_10selectolax_6parser_find_matches(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_parser, myhtml_tree_node_t *__pyx_v_node, PyObject *__pyx_v_selectors) {
  myhtml_collection_t *__pyx_v_collection;
  struct __pyx_obj_10selectolax_6parser_CSSSelector *__pyx_v_selector = 0;
  int __pyx_v_collection_size;
  PyObject *__pyx_v_query = 0;
  PyLongObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  myhtml_collection_t *__pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("find_matches", 0);

  /* "selectolax/modest/selection.pxi":182
 *     cdef str query
 * 
 *     for query in selectors:             # <<<<<<<<<<<<<<
 *         selector = CSSSelector(query)
 *         collection_size = 0
*/
  if (unlikely(__pyx_v_selectors == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 182, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_selectors; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 182, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2));
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2);
    #endif
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(1, 182, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_query, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":183
 * 
 *     for query in selectors:
 *         selector = CSSSelector(query)             # <<<<<<<<<<<<<<
 *         collection_size = 0
 *         collection = NULL
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_CSSSelector);
    __pyx_t_5 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_CSSSelector); 
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_query};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 183, __pyx_L1_error)
      __Pyx_GOTREF((PyObject *)__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_selector, ((struct __pyx_obj_10selectolax_6parser_CSSSelector *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "selectolax/modest/selection.pxi":184
 *     for query in selectors:
 *         selector = CSSSelector(query)
 *         collection_size = 0             # <<<<<<<<<<<<<<
 *         collection = NULL
 * 
*/
    __pyx_v_collection_size = 0;

    /* "selectolax/modest/selection.pxi":185
 *         selector = CSSSelector(query)
 *         collection_size = 0
 *         collection = NULL             # <<<<<<<<<<<<<<
 * 
 *         collection = selector.find(node)
*/
    __pyx_v_collection = NULL;

    /* "selectolax/modest/selection.pxi":187
 *         collection = NULL
 * 
 *         collection = selector.find(node)             # <<<<<<<<<<<<<<
 *         if collection == NULL:
 *             continue
*/
    __pyx_t_7 = __pyx_f_10selectolax_6parser_11CSSSelector_find(__pyx_v_selector, __pyx_v_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 187, __pyx_L1_error)
    __pyx_v_collection = __pyx_t_7;

    /* "selectolax/modest/selection.pxi":188
 * 
 *         collection = selector.find(node)
 *         if collection == NULL:             # <<<<<<<<<<<<<<
 *             continue
 * 
*/
    __pyx_t_8 = (__pyx_v_collection == NULL);
    if (__pyx_t_8) {

      /* "selectolax/modest/selection.pxi":189
 *         collection = selector.find(node)
 *         if collection == NULL:
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         collection_size = collection.length
*/
      goto __pyx_L3_continue;

      /* "selectolax/modest/selection.pxi":188
 * 
 *         collection = selector.find(node)
 *         if collection == NULL:             # <<<<<<<<<<<<<<
 *             continue
 * 
*/
    }

    /* "selectolax/modest/selection.pxi":191
 *             continue
 * 
 *         collection_size = collection.length             # <<<<<<<<<<<<<<
 *         myhtml_collection_destroy(collection)
 *         if collection_size > 0:
*/
    __pyx_t_6 = __pyx_v_collection->length;
    __pyx_v_collection_size = __pyx_t_6;

    /* "selectolax/modest/selection.pxi":192
 * 
 *         collection_size = collection.length
 *         myhtml_collection_destroy(collection)             # <<<<<<<<<<<<<<
 *         if collection_size > 0:
 *             return True
*/
    (void)(myhtml_collection_destroy(__pyx_v_collection));

    /* "selectolax/modest/selection.pxi":193
 *         collection_size = collection.length
 *         myhtml_collection_destroy(collection)
 *         if collection_size > 0:             # <<<<<<<<<<<<<<
 *             return True
 *     return False
*/
    __pyx_t_8 = (__pyx_v_collection_size > 0);
    if (__pyx_t_8) {

      /* "selectolax/modest/selection.pxi":194
 *         myhtml_collection_destroy(collection)
 *         if collection_size > 0:
 *             return True             # <<<<<<<<<<<<<<
 *     return False
*/
      __Pyx_XDECREF((PyObject *)__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = ((PyLongObject *)Py_True);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "selectolax/modest/selection.pxi":193
 *         collection_size = collection.length
 *         myhtml_collection_destroy(collection)
 *         if collection_size > 0:             # <<<<<<<<<<<<<<
 *             return True
 *     return False
*/
    }

    /* "selectolax/modest/selection.pxi":182
 *     cdef str query
 * 
 *     for query in selectors:             # <<<<<<<<<<<<<<
 *         selector = CSSSelector(query)
 *         collection_size = 0
*/
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/selection.pxi":195
 *         if collection_size > 0:
 *             return True
 *     return False             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = ((PyLongObject *)Py_False);
  goto __pyx_L0;

  /* "selectolax/modest/selection.pxi":176
 * 
 * 
 * cdef bool find_matches(HTMLParser parser, myhtml_tree_node_t *node, tuple selectors):             # <<<<<<<<<<<<<<
 *     cdef myhtml_collection_t *collection
 *     cdef CSSSelector selector
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.find_matches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_selector);
  __Pyx_XDECREF(__pyx_v_query);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":15
 * @cython.final
 * cdef class Stack:
 *     def __cinit__(self, size_t capacity=25):             # <<<<<<<<<<<<<<
 *         self.capacity = capacity
 *         self.top = 0
*/

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_5Stack_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_10selectolax_6parser_5Stack_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_capacity;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_capacity,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 15, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 15, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < 0) __PYX_ERR(3, 15, __pyx_L3_error)
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 15, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_capacity = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_capacity == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 15, __pyx_L3_error)
    } else {
      __pyx_v_capacity = ((size_t)25);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 0, 1, __pyx_nargs); __PYX_ERR(3, 15, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Stack.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_5Stack___cinit__(((struct __pyx_obj_10selectolax_6parser_Stack *)__pyx_v_self), __pyx_v_capacity);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_5Stack___cinit__(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self, size_t __pyx_v_capacity) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "selectolax/modest/node.pxi":16
 * cdef class Stack:
 *     def __cinit__(self, size_t capacity=25):
 *         self.capacity = capacity             # <<<<<<<<<<<<<<
 *         self.top = 0
 *         self._stack = <myhtml_tree_node_t**> malloc(capacity * sizeof(myhtml_tree_node_t))
*/
  __pyx_v_self->capacity = __pyx_v_capacity;

  /* "selectolax/modest/node.pxi":17
 *     def __cinit__(self, size_t capacity=25):
 *         self.capacity = capacity
 *         self.top = 0             # <<<<<<<<<<<<<<
 *         self._stack = <myhtml_tree_node_t**> malloc(capacity * sizeof(myhtml_tree_node_t))
 *         if self._stack == NULL:
*/
  __pyx_v_self->top = 0;

  /* "selectolax/modest/node.pxi":18
 *         self.capacity = capacity
 *         self.top = 0
 *         self._stack = <myhtml_tree_node_t**> malloc(capacity * sizeof(myhtml_tree_node_t))             # <<<<<<<<<<<<<<
 *         if self._stack == NULL:
 *             raise MemoryError("Failed to allocate memory for stack")
*/
  __pyx_v_self->_stack = ((myhtml_tree_node_t **)malloc((__pyx_v_capacity * (sizeof(myhtml_tree_node_t)))));

  /* "selectolax/modest/node.pxi":19
 *         self.top = 0
 *         self._stack = <myhtml_tree_node_t**> malloc(capacity * sizeof(myhtml_tree_node_t))
 *         if self._stack == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Failed to allocate memory for stack")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_stack == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":20
 *         self._stack = <myhtml_tree_node_t**> malloc(capacity * sizeof(myhtml_tree_node_t))
 *         if self._stack == NULL:
 *             raise MemoryError("Failed to allocate memory for stack")             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
    __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_builtin_MemoryError);
    __pyx_t_4 = __pyx_builtin_MemoryError; 
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Failed_to_allocate_memory_for_st};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 20, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(3, 20, __pyx_L1_error)

    /* "selectolax/modest/node.pxi":19
 *         self.top = 0
 *         self._stack = <myhtml_tree_node_t**> malloc(capacity * sizeof(myhtml_tree_node_t))
 *         if self._stack == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Failed to allocate memory for stack")
 * 
*/
  }

  /* "selectolax/modest/node.pxi":15
 * @cython.final
 * cdef class Stack:
 *     def __cinit__(self, size_t capacity=25):             # <<<<<<<<<<<<<<
 *         self.capacity = capacity
 *         self.top = 0
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("selectolax.parser.Stack.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":22
 *             raise MemoryError("Failed to allocate memory for stack")
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         free(self._stack)
 * 
*/

/* Python wrapper */
static void __pyx_pw_10selectolax_6parser_5Stack_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_10selectolax_6parser_5Stack_3__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_10selectolax_6parser_5Stack_2__dealloc__(((struct __pyx_obj_10selectolax_6parser_Stack *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_10selectolax_6parser_5Stack_2__dealloc__(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self) {

  /* "selectolax/modest/node.pxi":23
 * 
 *     def __dealloc__(self):
 *         free(self._stack)             # <<<<<<<<<<<<<<
 * 
 *     cdef bint is_empty(self):
*/
  free(__pyx_v_self->_stack);

  /* "selectolax/modest/node.pxi":22
 *             raise MemoryError("Failed to allocate memory for stack")
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         free(self._stack)
 * 
*/

  /* function exit code */
}

/* "selectolax/modest/node.pxi":25
 *         free(self._stack)
 * 
 *     cdef bint is_empty(self):             # <<<<<<<<<<<<<<
 *         return self.top <= 0
 * 
*/

static int __pyx_f_10selectolax_6parser_5Stack_is_empty(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self) {
  int __pyx_r;

  /* "selectolax/modest/node.pxi":26
 * 
 *     cdef bint is_empty(self):
 *         return self.top <= 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int push(self, myhtml_tree_node_t* res) except -1:
*/
  __pyx_r = (__pyx_v_self->top <= 0);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":25
 *         free(self._stack)
 * 
 *     cdef bint is_empty(self):             # <<<<<<<<<<<<<<
 *         return self.top <= 0
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":28
 *         return self.top <= 0
 * 
 *     cdef int push(self, myhtml_tree_node_t* res) except -1:             # <<<<<<<<<<<<<<
 *         if self.top >= self.capacity:
 *             if self.resize() < 0:
*/

static int __pyx_f_10selectolax_6parser_5Stack_push(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self, myhtml_tree_node_t *__pyx_v_res) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "selectolax/modest/node.pxi":29
 * 
 *     cdef int push(self, myhtml_tree_node_t* res) except -1:
 *         if self.top >= self.capacity:             # <<<<<<<<<<<<<<
 *             if self.resize() < 0:
 *                 return -1
*/
  __pyx_t_1 = (__pyx_v_self->top >= __pyx_v_self->capacity);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":30
 *     cdef int push(self, myhtml_tree_node_t* res) except -1:
 *         if self.top >= self.capacity:
 *             if self.resize() < 0:             # <<<<<<<<<<<<<<
 *                 return -1
 *         self._stack[self.top] = res
*/
    __pyx_t_2 = __pyx_f_10selectolax_6parser_5Stack_resize(__pyx_v_self); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 30, __pyx_L1_error)
    __pyx_t_1 = (__pyx_t_2 < 0);
    if (__pyx_t_1) {

      /* "selectolax/modest/node.pxi":31
 *         if self.top >= self.capacity:
 *             if self.resize() < 0:
 *                 return -1             # <<<<<<<<<<<<<<
 *         self._stack[self.top] = res
 *         self.top += 1
*/
      __pyx_r = -1;
      goto __pyx_L0;

      /* "selectolax/modest/node.pxi":30
 *     cdef int push(self, myhtml_tree_node_t* res) except -1:
 *         if self.top >= self.capacity:
 *             if self.resize() < 0:             # <<<<<<<<<<<<<<
 *                 return -1
 *         self._stack[self.top] = res
*/
    }

    /* "selectolax/modest/node.pxi":29
 * 
 *     cdef int push(self, myhtml_tree_node_t* res) except -1:
 *         if self.top >= self.capacity:             # <<<<<<<<<<<<<<
 *             if self.resize() < 0:
 *                 return -1
*/
  }

  /* "selectolax/modest/node.pxi":32
 *             if self.resize() < 0:
 *                 return -1
 *         self._stack[self.top] = res             # <<<<<<<<<<<<<<
 *         self.top += 1
 * 
*/
  (__pyx_v_self->_stack[__pyx_v_self->top]) = __pyx_v_res;

  /* "selectolax/modest/node.pxi":33
 *                 return -1
 *         self._stack[self.top] = res
 *         self.top += 1             # <<<<<<<<<<<<<<
 * 
 *     cdef myhtml_tree_node_t * pop(self):
*/
  __pyx_v_self->top = (__pyx_v_self->top + 1);

  /* "selectolax/modest/node.pxi":28
 *         return self.top <= 0
 * 
 *     cdef int push(self, myhtml_tree_node_t* res) except -1:             # <<<<<<<<<<<<<<
 *         if self.top >= self.capacity:
 *             if self.resize() < 0:
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Stack.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":35
 *         self.top += 1
 * 
 *     cdef myhtml_tree_node_t * pop(self):             # <<<<<<<<<<<<<<
 *         self.top = self.top - 1
 *         return self._stack[self.top]
*/

static myhtml_tree_node_t *__pyx_f_10selectolax_6parser_5Stack_pop(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self) {
  myhtml_tree_node_t *__pyx_r;

  /* "selectolax/modest/node.pxi":36
 * 
 *     cdef myhtml_tree_node_t * pop(self):
 *         self.top = self.top - 1             # <<<<<<<<<<<<<<
 *         return self._stack[self.top]
 * 
*/
  __pyx_v_self->top = (__pyx_v_self->top - 1);

  /* "selectolax/modest/node.pxi":37
 *     cdef myhtml_tree_node_t * pop(self):
 *         self.top = self.top - 1
 *         return self._stack[self.top]             # <<<<<<<<<<<<<<
 * 
 *     cdef int resize(self) except -1:
*/
  __pyx_r = (__pyx_v_self->_stack[__pyx_v_self->top]);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":35
 *         self.top += 1
 * 
 *     cdef myhtml_tree_node_t * pop(self):             # <<<<<<<<<<<<<<
 *         self.top = self.top - 1
 *         return self._stack[self.top]
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":39
 *         return self._stack[self.top]
 * 
 *     cdef int resize(self) except -1:             # <<<<<<<<<<<<<<
 *         self.capacity *= 2
 *         self._stack = <myhtml_tree_node_t**> realloc(<void*> self._stack, self.capacity * sizeof(myhtml_tree_node_t))
*/

static int __pyx_f_10selectolax_6parser_5Stack_resize(struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self) {
  int __pyx_r;
  int __pyx_t_1;
  PyObject *__pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "selectolax/modest/node.pxi":40
 * 
 *     cdef int resize(self) except -1:
 *         self.capacity *= 2             # <<<<<<<<<<<<<<
 *         self._stack = <myhtml_tree_node_t**> realloc(<void*> self._stack, self.capacity * sizeof(myhtml_tree_node_t))
 *         if self._stack == NULL:
*/
  __pyx_v_self->capacity = (__pyx_v_self->capacity * 2);

  /* "selectolax/modest/node.pxi":41
 *     cdef int resize(self) except -1:
 *         self.capacity *= 2
 *         self._stack = <myhtml_tree_node_t**> realloc(<void*> self._stack, self.capacity * sizeof(myhtml_tree_node_t))             # <<<<<<<<<<<<<<
 *         if self._stack == NULL:
 *             PyErr_NoMemory()
*/
  __pyx_v_self->_stack = ((myhtml_tree_node_t **)realloc(((void *)__pyx_v_self->_stack), (__pyx_v_self->capacity * (sizeof(myhtml_tree_node_t)))));

  /* "selectolax/modest/node.pxi":42
 *         self.capacity *= 2
 *         self._stack = <myhtml_tree_node_t**> realloc(<void*> self._stack, self.capacity * sizeof(myhtml_tree_node_t))
 *         if self._stack == NULL:             # <<<<<<<<<<<<<<
 *             PyErr_NoMemory()
 *             return -1
*/
  __pyx_t_1 = (__pyx_v_self->_stack == NULL);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":43
 *         self._stack = <myhtml_tree_node_t**> realloc(<void*> self._stack, self.capacity * sizeof(myhtml_tree_node_t))
 *         if self._stack == NULL:
 *             PyErr_NoMemory()             # <<<<<<<<<<<<<<
 *             return -1
 *         return 0
*/
    __pyx_t_2 = PyErr_NoMemory(); if (unlikely(__pyx_t_2 == ((PyObject *)0))) __PYX_ERR(3, 43, __pyx_L1_error)

    /* "selectolax/modest/node.pxi":44
 *         if self._stack == NULL:
 *             PyErr_NoMemory()
 *             return -1             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
    __pyx_r = -1;
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":42
 *         self.capacity *= 2
 *         self._stack = <myhtml_tree_node_t**> realloc(<void*> self._stack, self.capacity * sizeof(myhtml_tree_node_t))
 *         if self._stack == NULL:             # <<<<<<<<<<<<<<
 *             PyErr_NoMemory()
 *             return -1
*/
  }

  /* "selectolax/modest/node.pxi":45
 *             PyErr_NoMemory()
 *             return -1
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * cdef class _Attributes:
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":39
 *         return self._stack[self.top]
 * 
 *     cdef int resize(self) except -1:             # <<<<<<<<<<<<<<
 *         self.capacity *= 2
 *         self._stack = <myhtml_tree_node_t**> realloc(<void*> self._stack, self.capacity * sizeof(myhtml_tree_node_t))
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Stack.resize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_5Stack_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_5Stack_4__reduce_cython__, "Stack.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_5Stack_5__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_5Stack_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_5Stack_4__reduce_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_5Stack_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_5Stack_4__reduce_cython__(((struct __pyx_obj_10selectolax_6parser_Stack *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_5Stack_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(2, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Stack.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_5Stack_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_5Stack_6__setstate_cython__, "Stack.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_5Stack_7__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_5Stack_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_5Stack_6__setstate_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_5Stack_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Stack.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_5Stack_6__setstate_cython__(((struct __pyx_obj_10selectolax_6parser_Stack *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_5Stack_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(2, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Stack.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":52
 *     cdef unicode decode_errors
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef _Attributes create(myhtml_tree_node_t *node, unicode decode_errors):
 *         obj = <_Attributes>_Attributes.__new__(_Attributes)
*/

static struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_f_10selectolax_6parser_11_Attributes_create(myhtml_tree_node_t *__pyx_v_node, PyObject *__pyx_v_decode_errors) {
  struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_obj = NULL;
  struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create", 0);

  /* "selectolax/modest/node.pxi":54
 *     @staticmethod
 *     cdef _Attributes create(myhtml_tree_node_t *node, unicode decode_errors):
 *         obj = <_Attributes>_Attributes.__new__(_Attributes)             # <<<<<<<<<<<<<<
 *         obj.node = node
 *         obj.decode_errors = decode_errors
*/
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_10selectolax_6parser__Attributes(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser__Attributes), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 54, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_t_1);
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF((PyObject *)__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_obj = ((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":55
 *     cdef _Attributes create(myhtml_tree_node_t *node, unicode decode_errors):
 *         obj = <_Attributes>_Attributes.__new__(_Attributes)
 *         obj.node = node             # <<<<<<<<<<<<<<
 *         obj.decode_errors = decode_errors
 *         return obj
*/
  __pyx_v_obj->node = __pyx_v_node;

  /* "selectolax/modest/node.pxi":56
 *         obj = <_Attributes>_Attributes.__new__(_Attributes)
 *         obj.node = node
 *         obj.decode_errors = decode_errors             # <<<<<<<<<<<<<<
 *         return obj
 * 
*/
  __Pyx_INCREF(__pyx_v_decode_errors);
  __Pyx_GIVEREF(__pyx_v_decode_errors);
  __Pyx_GOTREF(__pyx_v_obj->decode_errors);
  __Pyx_DECREF(__pyx_v_obj->decode_errors);
  __pyx_v_obj->decode_errors = __pyx_v_decode_errors;

  /* "selectolax/modest/node.pxi":57
 *         obj.node = node
 *         obj.decode_errors = decode_errors
 *         return obj             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":52
 *     cdef unicode decode_errors
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef _Attributes create(myhtml_tree_node_t *node, unicode decode_errors):
 *         obj = <_Attributes>_Attributes.__new__(_Attributes)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser._Attributes.create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_obj);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_10selectolax_6parser_11_Attributes_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "selectolax/modest/node.pxi":59
 *         return obj
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef myhtml_tree_attr_t *attr = myhtml_node_attribute_first(self.node)
 *         while attr:
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_1__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_1__iter__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes___iter__(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes___iter__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);
  __pyx_cur_scope = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct____iter__(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(3, 59, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_10selectolax_6parser_11_Attributes_2generator, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_iter, __pyx_mstate_global->__pyx_n_u_Attributes___iter, __pyx_mstate_global->__pyx_n_u_selectolax_parser); if (unlikely(!gen)) __PYX_ERR(3, 59, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser._Attributes.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_10selectolax_6parser_11_Attributes_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *__pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  myhtml_tree_attr_t *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L7_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(3, 59, __pyx_L1_error)
  }

  /* "selectolax/modest/node.pxi":60
 * 
 *     def __iter__(self):
 *         cdef myhtml_tree_attr_t *attr = myhtml_node_attribute_first(self.node)             # <<<<<<<<<<<<<<
 *         while attr:
 *             if attr.key.data == NULL:
*/
  __pyx_cur_scope->__pyx_v_attr = myhtml_node_attribute_first(__pyx_cur_scope->__pyx_v_self->node);

  /* "selectolax/modest/node.pxi":61
 *     def __iter__(self):
 *         cdef myhtml_tree_attr_t *attr = myhtml_node_attribute_first(self.node)
 *         while attr:             # <<<<<<<<<<<<<<
 *             if attr.key.data == NULL:
 *                 attr = attr.next
*/
  while (1) {
    __pyx_t_1 = (__pyx_cur_scope->__pyx_v_attr != 0);
    if (!__pyx_t_1) break;

    /* "selectolax/modest/node.pxi":62
 *         cdef myhtml_tree_attr_t *attr = myhtml_node_attribute_first(self.node)
 *         while attr:
 *             if attr.key.data == NULL:             # <<<<<<<<<<<<<<
 *                 attr = attr.next
 *                 continue
*/
    __pyx_t_1 = (__pyx_cur_scope->__pyx_v_attr->key.data == NULL);
    if (__pyx_t_1) {

      /* "selectolax/modest/node.pxi":63
 *         while attr:
 *             if attr.key.data == NULL:
 *                 attr = attr.next             # <<<<<<<<<<<<<<
 *                 continue
 *             key = attr.key.data.decode(_ENCODING, self.decode_errors)
*/
      __pyx_t_2 = __pyx_cur_scope->__pyx_v_attr->next;
      __pyx_cur_scope->__pyx_v_attr = __pyx_t_2;

      /* "selectolax/modest/node.pxi":64
 *             if attr.key.data == NULL:
 *                 attr = attr.next
 *                 continue             # <<<<<<<<<<<<<<
 *             key = attr.key.data.decode(_ENCODING, self.decode_errors)
 *             attr = attr.next
*/
      goto __pyx_L4_continue;

      /* "selectolax/modest/node.pxi":62
 *         cdef myhtml_tree_attr_t *attr = myhtml_node_attribute_first(self.node)
 *         while attr:
 *             if attr.key.data == NULL:             # <<<<<<<<<<<<<<
 *                 attr = attr.next
 *                 continue
*/
    }

    /* "selectolax/modest/node.pxi":65
 *                 attr = attr.next
 *                 continue
 *             key = attr.key.data.decode(_ENCODING, self.decode_errors)             # <<<<<<<<<<<<<<
 *             attr = attr.next
 *             yield key
*/
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_cur_scope->__pyx_v_attr->key.data); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_cur_scope->__pyx_v_self->decode_errors};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 65, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_key);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_key, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":66
 *                 continue
 *             key = attr.key.data.decode(_ENCODING, self.decode_errors)
 *             attr = attr.next             # <<<<<<<<<<<<<<
 *             yield key
 * 
*/
    __pyx_t_2 = __pyx_cur_scope->__pyx_v_attr->next;
    __pyx_cur_scope->__pyx_v_attr = __pyx_t_2;

    /* "selectolax/modest/node.pxi":67
 *             key = attr.key.data.decode(_ENCODING, self.decode_errors)
 *             attr = attr.next
 *             yield key             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, str key, value):
*/
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_key);
    __pyx_r = __pyx_cur_scope->__pyx_v_key;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L7_resume_from_yield:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(3, 67, __pyx_L1_error)
    __pyx_L4_continue:;
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "selectolax/modest/node.pxi":59
 *         return obj
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef myhtml_tree_attr_t *attr = myhtml_node_attribute_first(self.node)
 *         while attr:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":69
 *             yield key
 * 
 *     def __setitem__(self, str key, value):             # <<<<<<<<<<<<<<
 *         value = str(value)
 *         bytes_key = key.encode(_ENCODING)
*/

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_11_Attributes_4__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10selectolax_6parser_11_Attributes_4__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyUnicode_Type), 1, "key", 1))) __PYX_ERR(3, 69, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_3__setitem__(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self), ((PyObject*)__pyx_v_key), ((PyObject *)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  goto __pyx_L5_cleaned_up;
  __pyx_L0:;
  __pyx_L5_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_11_Attributes_3__setitem__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_bytes_key = NULL;
  PyObject *__pyx_v_bytes_value = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  char *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "selectolax/modest/node.pxi":70
 * 
 *     def __setitem__(self, str key, value):
 *         value = str(value)             # <<<<<<<<<<<<<<
 *         bytes_key = key.encode(_ENCODING)
 *         bytes_value = value.encode(_ENCODING)
*/
  __pyx_t_1 = __Pyx_PyObject_Unicode(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":71
 *     def __setitem__(self, str key, value):
 *         value = str(value)
 *         bytes_key = key.encode(_ENCODING)             # <<<<<<<<<<<<<<
 *         bytes_value = value.encode(_ENCODING)
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))
*/
  if (unlikely(__pyx_v_key == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(3, 71, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_bytes_key = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":72
 *         value = str(value)
 *         bytes_key = key.encode(_ENCODING)
 *         bytes_value = value.encode(_ENCODING)             # <<<<<<<<<<<<<<
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))
 *         myhtml_attribute_add(self.node, <char*>bytes_key, len(bytes_key), <char*>bytes_value, len(bytes_value),
*/
  __pyx_t_2 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_UTF_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 72, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_bytes_value = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":73
 *         bytes_key = key.encode(_ENCODING)
 *         bytes_value = value.encode(_ENCODING)
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))             # <<<<<<<<<<<<<<
 *         myhtml_attribute_add(self.node, <char*>bytes_key, len(bytes_key), <char*>bytes_value, len(bytes_value),
 *                              MyENCODING_UTF_8)
*/
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_key); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 73, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_key); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 73, __pyx_L1_error)
  (void)(myhtml_attribute_remove_by_key(__pyx_v_self->node, ((char *)__pyx_t_4), __pyx_t_5));

  /* "selectolax/modest/node.pxi":74
 *         bytes_value = value.encode(_ENCODING)
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))
 *         myhtml_attribute_add(self.node, <char*>bytes_key, len(bytes_key), <char*>bytes_value, len(bytes_value),             # <<<<<<<<<<<<<<
 *                              MyENCODING_UTF_8)
 * 
*/
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_key); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 74, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_key); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 74, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_AsWritableString(__pyx_v_bytes_value); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(3, 74, __pyx_L1_error)
  __pyx_t_7 = PyObject_Length(__pyx_v_bytes_value); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(3, 74, __pyx_L1_error)

  /* "selectolax/modest/node.pxi":75
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))
 *         myhtml_attribute_add(self.node, <char*>bytes_key, len(bytes_key), <char*>bytes_value, len(bytes_value),
 *                              MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 * 
 *     def __delitem__(self, key):
*/
  (void)(myhtml_attribute_add(__pyx_v_self->node, ((char *)__pyx_t_4), __pyx_t_5, ((char *)__pyx_t_6), __pyx_t_7, MyENCODING_UTF_8));

  /* "selectolax/modest/node.pxi":69
 *             yield key
 * 
 *     def __setitem__(self, str key, value):             # <<<<<<<<<<<<<<
 *         value = str(value)
 *         bytes_key = key.encode(_ENCODING)
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser._Attributes.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_key);
  __Pyx_XDECREF(__pyx_v_bytes_value);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":77
 *                              MyENCODING_UTF_8)
 * 
 *     def __delitem__(self, key):             # <<<<<<<<<<<<<<
 *         try:
 *             self.__getitem__(key)
*/

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_11_Attributes_6__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static int __pyx_pw_10selectolax_6parser_11_Attributes_6__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__delitem__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_5__delitem__(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_11_Attributes_5__delitem__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_v_bytes_key = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  char *__pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__delitem__", 0);

  /* "selectolax/modest/node.pxi":78
 * 
 *     def __delitem__(self, key):
 *         try:             # <<<<<<<<<<<<<<
 *             self.__getitem__(key)
 *         except KeyError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "selectolax/modest/node.pxi":79
 *     def __delitem__(self, key):
 *         try:
 *             self.__getitem__(key)             # <<<<<<<<<<<<<<
 *         except KeyError:
 *             raise KeyError(key)
*/
      __pyx_t_5 = ((PyObject *)__pyx_v_self);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_key};
        __pyx_t_4 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_getitem, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 79, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "selectolax/modest/node.pxi":78
 * 
 *     def __delitem__(self, key):
 *         try:             # <<<<<<<<<<<<<<
 *             self.__getitem__(key)
 *         except KeyError:
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "selectolax/modest/node.pxi":80
 *         try:
 *             self.__getitem__(key)
 *         except KeyError:             # <<<<<<<<<<<<<<
 *             raise KeyError(key)
 *         bytes_key = key.encode(_ENCODING)
*/
    __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("selectolax.parser._Attributes.__delitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_5, &__pyx_t_8) < 0) __PYX_ERR(3, 80, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_8);

      /* "selectolax/modest/node.pxi":81
 *             self.__getitem__(key)
 *         except KeyError:
 *             raise KeyError(key)             # <<<<<<<<<<<<<<
 *         bytes_key = key.encode(_ENCODING)
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))
*/
      __pyx_t_10 = NULL;
      __Pyx_INCREF(__pyx_builtin_KeyError);
      __pyx_t_11 = __pyx_builtin_KeyError; 
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_key};
        __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 81, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_9);
      }
      __Pyx_Raise(__pyx_t_9, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __PYX_ERR(3, 81, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "selectolax/modest/node.pxi":78
 * 
 *     def __delitem__(self, key):
 *         try:             # <<<<<<<<<<<<<<
 *             self.__getitem__(key)
 *         except KeyError:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "selectolax/modest/node.pxi":82
 *         except KeyError:
 *             raise KeyError(key)
 *         bytes_key = key.encode(_ENCODING)             # <<<<<<<<<<<<<<
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))
 * 
*/
  __pyx_t_5 = __pyx_v_key;
  __Pyx_INCREF(__pyx_t_5);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_UTF_8};
    __pyx_t_8 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 82, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_v_bytes_key = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "selectolax/modest/node.pxi":83
 *             raise KeyError(key)
 *         bytes_key = key.encode(_ENCODING)
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, str key):
*/
  __pyx_t_12 = __Pyx_PyObject_AsWritableString(__pyx_v_bytes_key); if (unlikely((!__pyx_t_12) && PyErr_Occurred())) __PYX_ERR(3, 83, __pyx_L1_error)
  __pyx_t_13 = PyObject_Length(__pyx_v_bytes_key); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(3, 83, __pyx_L1_error)
  (void)(myhtml_attribute_remove_by_key(__pyx_v_self->node, ((char *)__pyx_t_12), __pyx_t_13));

  /* "selectolax/modest/node.pxi":77
 *                              MyENCODING_UTF_8)
 * 
 *     def __delitem__(self, key):             # <<<<<<<<<<<<<<
 *         try:
 *             self.__getitem__(key)
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("selectolax.parser._Attributes.__delitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_key);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":85
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))
 * 
 *     def __getitem__(self, str key):             # <<<<<<<<<<<<<<
 *         bytes_key = key.encode(_ENCODING)
 *         cdef myhtml_tree_attr_t * attr =  myhtml_attribute_by_key(self.node, <char*>bytes_key, len(bytes_key))
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_8__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_8__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyUnicode_Type), 1, "key", 1))) __PYX_ERR(3, 85, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_7__getitem__(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self), ((PyObject*)__pyx_v_key));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  goto __pyx_L5_cleaned_up;
  __pyx_L0:;
  __pyx_L5_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_7__getitem__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_v_bytes_key = NULL;
  myhtml_tree_attr_t *__pyx_v_attr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char *__pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "selectolax/modest/node.pxi":86
 * 
 *     def __getitem__(self, str key):
 *         bytes_key = key.encode(_ENCODING)             # <<<<<<<<<<<<<<
 *         cdef myhtml_tree_attr_t * attr =  myhtml_attribute_by_key(self.node, <char*>bytes_key, len(bytes_key))
 *         if attr != NULL:
*/
  if (unlikely(__pyx_v_key == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(3, 86, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_bytes_key = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":87
 *     def __getitem__(self, str key):
 *         bytes_key = key.encode(_ENCODING)
 *         cdef myhtml_tree_attr_t * attr =  myhtml_attribute_by_key(self.node, <char*>bytes_key, len(bytes_key))             # <<<<<<<<<<<<<<
 *         if attr != NULL:
 *             if attr.value.data != NULL:
*/
  __pyx_t_2 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_key); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(3, 87, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_key); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(3, 87, __pyx_L1_error)
  __pyx_v_attr = myhtml_attribute_by_key(__pyx_v_self->node, ((char *)__pyx_t_2), __pyx_t_3);

  /* "selectolax/modest/node.pxi":88
 *         bytes_key = key.encode(_ENCODING)
 *         cdef myhtml_tree_attr_t * attr =  myhtml_attribute_by_key(self.node, <char*>bytes_key, len(bytes_key))
 *         if attr != NULL:             # <<<<<<<<<<<<<<
 *             if attr.value.data != NULL:
 *                 return attr.value.data.decode(_ENCODING, self.decode_errors)
*/
  __pyx_t_4 = (__pyx_v_attr != NULL);
  if (__pyx_t_4) {

    /* "selectolax/modest/node.pxi":89
 *         cdef myhtml_tree_attr_t * attr =  myhtml_attribute_by_key(self.node, <char*>bytes_key, len(bytes_key))
 *         if attr != NULL:
 *             if attr.value.data != NULL:             # <<<<<<<<<<<<<<
 *                 return attr.value.data.decode(_ENCODING, self.decode_errors)
 *             elif attr.key.data != NULL:
*/
    __pyx_t_4 = (__pyx_v_attr->value.data != NULL);
    if (__pyx_t_4) {

      /* "selectolax/modest/node.pxi":90
 *         if attr != NULL:
 *             if attr.value.data != NULL:
 *                 return attr.value.data.decode(_ENCODING, self.decode_errors)             # <<<<<<<<<<<<<<
 *             elif attr.key.data != NULL:
 *                 return None
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_6 = __Pyx_PyBytes_FromString(__pyx_v_attr->value.data); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 90, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = __pyx_t_6;
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_7 = 0;
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->decode_errors};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 90, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __pyx_r = __pyx_t_1;
      __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "selectolax/modest/node.pxi":89
 *         cdef myhtml_tree_attr_t * attr =  myhtml_attribute_by_key(self.node, <char*>bytes_key, len(bytes_key))
 *         if attr != NULL:
 *             if attr.value.data != NULL:             # <<<<<<<<<<<<<<
 *                 return attr.value.data.decode(_ENCODING, self.decode_errors)
 *             elif attr.key.data != NULL:
*/
    }

    /* "selectolax/modest/node.pxi":91
 *             if attr.value.data != NULL:
 *                 return attr.value.data.decode(_ENCODING, self.decode_errors)
 *             elif attr.key.data != NULL:             # <<<<<<<<<<<<<<
 *                 return None
 *         raise KeyError(key)
*/
    __pyx_t_4 = (__pyx_v_attr->key.data != NULL);
    if (__pyx_t_4) {

      /* "selectolax/modest/node.pxi":92
 *                 return attr.value.data.decode(_ENCODING, self.decode_errors)
 *             elif attr.key.data != NULL:
 *                 return None             # <<<<<<<<<<<<<<
 *         raise KeyError(key)
 * 
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "selectolax/modest/node.pxi":91
 *             if attr.value.data != NULL:
 *                 return attr.value.data.decode(_ENCODING, self.decode_errors)
 *             elif attr.key.data != NULL:             # <<<<<<<<<<<<<<
 *                 return None
 *         raise KeyError(key)
*/
    }

    /* "selectolax/modest/node.pxi":88
 *         bytes_key = key.encode(_ENCODING)
 *         cdef myhtml_tree_attr_t * attr =  myhtml_attribute_by_key(self.node, <char*>bytes_key, len(bytes_key))
 *         if attr != NULL:             # <<<<<<<<<<<<<<
 *             if attr.value.data != NULL:
 *                 return attr.value.data.decode(_ENCODING, self.decode_errors)
*/
  }

  /* "selectolax/modest/node.pxi":93
 *             elif attr.key.data != NULL:
 *                 return None
 *         raise KeyError(key)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
*/
  __pyx_t_6 = NULL;
  __Pyx_INCREF(__pyx_builtin_KeyError);
  __pyx_t_5 = __pyx_builtin_KeyError; 
  __pyx_t_7 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_key};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 93, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(3, 93, __pyx_L1_error)

  /* "selectolax/modest/node.pxi":85
 *         myhtml_attribute_remove_by_key(self.node, <char*>bytes_key, len(bytes_key))
 * 
 *     def __getitem__(self, str key):             # <<<<<<<<<<<<<<
 *         bytes_key = key.encode(_ENCODING)
 *         cdef myhtml_tree_attr_t * attr =  myhtml_attribute_by_key(self.node, <char*>bytes_key, len(bytes_key))
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser._Attributes.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":95
 *         raise KeyError(key)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(list(self.__iter__()))
 * 
*/

/* Python wrapper */
static Py_ssize_t __pyx_pw_10selectolax_6parser_11_Attributes_10__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_10selectolax_6parser_11_Attributes_10__len__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_9__len__(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_10selectolax_6parser_11_Attributes_9__len__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "selectolax/modest/node.pxi":96
 * 
 *     def __len__(self):
 *         return len(list(self.__iter__()))             # <<<<<<<<<<<<<<
 * 
 *     def keys(self):
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_iter, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_2 = __Pyx_PySequence_ListKeepNew(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __Pyx_PyList_GET_SIZE(__pyx_t_2); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(3, 96, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":95
 *         raise KeyError(key)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(list(self.__iter__()))
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser._Attributes.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":98
 *         return len(list(self.__iter__()))
 * 
 *     def keys(self):             # <<<<<<<<<<<<<<
 *         return self.__iter__()
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_12keys(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_11_Attributes_11keys, "_Attributes.keys(self)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11_Attributes_12keys = {"keys", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_12keys, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_11keys};
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_12keys(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("keys (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("keys", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("keys", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_11keys(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_11keys(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("keys", 0);

  /* "selectolax/modest/node.pxi":99
 * 
 *     def keys(self):
 *         return self.__iter__()             # <<<<<<<<<<<<<<
 * 
 *     def items(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_iter, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 99, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":98
 *         return len(list(self.__iter__()))
 * 
 *     def keys(self):             # <<<<<<<<<<<<<<
 *         return self.__iter__()
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser._Attributes.keys", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_10selectolax_6parser_11_Attributes_15generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "selectolax/modest/node.pxi":101
 *         return self.__iter__()
 * 
 *     def items(self):             # <<<<<<<<<<<<<<
 *         for key in self.__iter__():
 *             yield key, self[key]
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_14items(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_11_Attributes_13items, "_Attributes.items(self)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11_Attributes_14items = {"items", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_14items, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_13items};
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_14items(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("items (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("items", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("items", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_13items(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_13items(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("items", 0);
  __pyx_cur_scope = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_1_items(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(3, 101, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_10selectolax_6parser_11_Attributes_15generator1, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_items, __pyx_mstate_global->__pyx_n_u_Attributes_items, __pyx_mstate_global->__pyx_n_u_selectolax_parser); if (unlikely(!gen)) __PYX_ERR(3, 101, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser._Attributes.items", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_10selectolax_6parser_11_Attributes_15generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *__pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("items", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(3, 101, __pyx_L1_error)
  }

  /* "selectolax/modest/node.pxi":102
 * 
 *     def items(self):
 *         for key in self.__iter__():             # <<<<<<<<<<<<<<
 *             yield key, self[key]
 * 
*/
  __pyx_t_2 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_iter, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2);
    __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 102, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 102, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        __pyx_t_1 = __Pyx_PyList_GetItemRef(__pyx_t_2, __pyx_t_4);
        ++__pyx_t_4;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 102, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4));
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_4);
        #endif
        ++__pyx_t_4;
      }
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 102, __pyx_L1_error)
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(3, 102, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_key);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_key, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "selectolax/modest/node.pxi":103
 *     def items(self):
 *         for key in self.__iter__():
 *             yield key, self[key]             # <<<<<<<<<<<<<<
 * 
 *     def values(self):
*/
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_cur_scope->__pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_key);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_key);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_cur_scope->__pyx_v_key) != (0)) __PYX_ERR(3, 103, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 103, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    __Pyx_XGIVEREF(__pyx_t_2);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_4;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(3, 103, __pyx_L1_error)

    /* "selectolax/modest/node.pxi":102
 * 
 *     def items(self):
 *         for key in self.__iter__():             # <<<<<<<<<<<<<<
 *             yield key, self[key]
 * 
*/
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "selectolax/modest/node.pxi":101
 *         return self.__iter__()
 * 
 *     def items(self):             # <<<<<<<<<<<<<<
 *         for key in self.__iter__():
 *             yield key, self[key]
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("items", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_10selectolax_6parser_11_Attributes_18generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "selectolax/modest/node.pxi":105
 *             yield key, self[key]
 * 
 *     def values(self):             # <<<<<<<<<<<<<<
 *         for key in self.__iter__():
 *             yield self[key]
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_17values(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_11_Attributes_16values, "_Attributes.values(self)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11_Attributes_17values = {"values", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_17values, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_16values};
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_17values(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("values (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("values", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("values", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_16values(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_16values(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("values", 0);
  __pyx_cur_scope = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_2_values(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(3, 105, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_10selectolax_6parser_11_Attributes_18generator2, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_values, __pyx_mstate_global->__pyx_n_u_Attributes_values, __pyx_mstate_global->__pyx_n_u_selectolax_parser); if (unlikely(!gen)) __PYX_ERR(3, 105, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser._Attributes.values", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_10selectolax_6parser_11_Attributes_18generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *__pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("values", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(3, 105, __pyx_L1_error)
  }

  /* "selectolax/modest/node.pxi":106
 * 
 *     def values(self):
 *         for key in self.__iter__():             # <<<<<<<<<<<<<<
 *             yield self[key]
 * 
*/
  __pyx_t_2 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_iter, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2);
    __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 106, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 106, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        __pyx_t_1 = __Pyx_PyList_GetItemRef(__pyx_t_2, __pyx_t_4);
        ++__pyx_t_4;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 106, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4));
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_4);
        #endif
        ++__pyx_t_4;
      }
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 106, __pyx_L1_error)
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(3, 106, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_key);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_key, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "selectolax/modest/node.pxi":107
 *     def values(self):
 *         for key in self.__iter__():
 *             yield self[key]             # <<<<<<<<<<<<<<
 * 
 *     def get(self, key, default=None):
*/
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_cur_scope->__pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    __Pyx_XGIVEREF(__pyx_t_2);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_4;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(3, 107, __pyx_L1_error)

    /* "selectolax/modest/node.pxi":106
 * 
 *     def values(self):
 *         for key in self.__iter__():             # <<<<<<<<<<<<<<
 *             yield self[key]
 * 
*/
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "selectolax/modest/node.pxi":105
 *             yield key, self[key]
 * 
 *     def values(self):             # <<<<<<<<<<<<<<
 *         for key in self.__iter__():
 *             yield self[key]
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("values", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":109
 *             yield self[key]
 * 
 *     def get(self, key, default=None):             # <<<<<<<<<<<<<<
 *         try:
 *             return self[key]
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_20get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_11_Attributes_19get, "_Attributes.get(self, key, default=None)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11_Attributes_20get = {"get", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_20get, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_19get};
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_20get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_default = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_default,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 109, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 109, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 109, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get", 0) < 0) __PYX_ERR(3, 109, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, i); __PYX_ERR(3, 109, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 109, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 109, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_key = values[0];
    __pyx_v_default = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, __pyx_nargs); __PYX_ERR(3, 109, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser._Attributes.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_19get(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self), __pyx_v_key, __pyx_v_default);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_19get(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);

  /* "selectolax/modest/node.pxi":110
 * 
 *     def get(self, key, default=None):
 *         try:             # <<<<<<<<<<<<<<
 *             return self[key]
 *         except KeyError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "selectolax/modest/node.pxi":111
 *     def get(self, key, default=None):
 *         try:
 *             return self[key]             # <<<<<<<<<<<<<<
 *         except KeyError:
 *             return default
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 111, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_r = __pyx_t_4;
      __pyx_t_4 = 0;
      goto __pyx_L7_try_return;

      /* "selectolax/modest/node.pxi":110
 * 
 *     def get(self, key, default=None):
 *         try:             # <<<<<<<<<<<<<<
 *             return self[key]
 *         except KeyError:
*/
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/modest/node.pxi":112
 *         try:
 *             return self[key]
 *         except KeyError:             # <<<<<<<<<<<<<<
 *             return default
 * 
*/
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_5) {
      __Pyx_AddTraceback("selectolax.parser._Attributes.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(3, 112, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "selectolax/modest/node.pxi":113
 *             return self[key]
 *         except KeyError:
 *             return default             # <<<<<<<<<<<<<<
 * 
 *     def sget(self, key, default=""):
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_default);
      __pyx_r = __pyx_v_default;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;

    /* "selectolax/modest/node.pxi":110
 * 
 *     def get(self, key, default=None):
 *         try:             # <<<<<<<<<<<<<<
 *             return self[key]
 *         except KeyError:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "selectolax/modest/node.pxi":109
 *             yield self[key]
 * 
 *     def get(self, key, default=None):             # <<<<<<<<<<<<<<
 *         try:
 *             return self[key]
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("selectolax.parser._Attributes.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":115
 *             return default
 * 
 *     def sget(self, key, default=""):             # <<<<<<<<<<<<<<
 *         """Same as get, but returns empty strings instead of None values for empty attributes."""
 *         try:
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_22sget(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_11_Attributes_21sget, "_Attributes.sget(self, key, default='')\n\nSame as get, but returns empty strings instead of None values for empty attributes.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11_Attributes_22sget = {"sget", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_22sget, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_21sget};
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_22sget(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_default = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sget (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_default,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 115, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 115, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 115, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "sget", 0) < 0) __PYX_ERR(3, 115, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_kp_u_));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("sget", 0, 1, 2, i); __PYX_ERR(3, 115, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 115, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 115, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_kp_u_));
    }
    __pyx_v_key = values[0];
    __pyx_v_default = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("sget", 0, 1, 2, __pyx_nargs); __PYX_ERR(3, 115, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser._Attributes.sget", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_21sget(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self), __pyx_v_key, __pyx_v_default);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_21sget(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default) {
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("sget", 0);

  /* "selectolax/modest/node.pxi":117
 *     def sget(self, key, default=""):
 *         """Same as get, but returns empty strings instead of None values for empty attributes."""
 *         try:             # <<<<<<<<<<<<<<
 *             val = self[key]
 *             if val is None:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "selectolax/modest/node.pxi":118
 *         """Same as get, but returns empty strings instead of None values for empty attributes."""
 *         try:
 *             val = self[key]             # <<<<<<<<<<<<<<
 *             if val is None:
 *                 val = ""
*/
      __pyx_t_4 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 118, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_val = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "selectolax/modest/node.pxi":119
 *         try:
 *             val = self[key]
 *             if val is None:             # <<<<<<<<<<<<<<
 *                 val = ""
 *             return val
*/
      __pyx_t_5 = (__pyx_v_val == Py_None);
      if (__pyx_t_5) {

        /* "selectolax/modest/node.pxi":120
 *             val = self[key]
 *             if val is None:
 *                 val = ""             # <<<<<<<<<<<<<<
 *             return val
 *         except KeyError:
*/
        __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
        __Pyx_DECREF_SET(__pyx_v_val, __pyx_mstate_global->__pyx_kp_u_);

        /* "selectolax/modest/node.pxi":119
 *         try:
 *             val = self[key]
 *             if val is None:             # <<<<<<<<<<<<<<
 *                 val = ""
 *             return val
*/
      }

      /* "selectolax/modest/node.pxi":121
 *             if val is None:
 *                 val = ""
 *             return val             # <<<<<<<<<<<<<<
 *         except KeyError:
 *             return default
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_val);
      __pyx_r = __pyx_v_val;
      goto __pyx_L7_try_return;

      /* "selectolax/modest/node.pxi":117
 *     def sget(self, key, default=""):
 *         """Same as get, but returns empty strings instead of None values for empty attributes."""
 *         try:             # <<<<<<<<<<<<<<
 *             val = self[key]
 *             if val is None:
*/
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/modest/node.pxi":122
 *                 val = ""
 *             return val
 *         except KeyError:             # <<<<<<<<<<<<<<
 *             return default
 * 
*/
    __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_6) {
      __Pyx_AddTraceback("selectolax.parser._Attributes.sget", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(3, 122, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);

      /* "selectolax/modest/node.pxi":123
 *             return val
 *         except KeyError:
 *             return default             # <<<<<<<<<<<<<<
 * 
 *     def __contains__(self, key):
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_default);
      __pyx_r = __pyx_v_default;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;

    /* "selectolax/modest/node.pxi":117
 *     def sget(self, key, default=""):
 *         """Same as get, but returns empty strings instead of None values for empty attributes."""
 *         try:             # <<<<<<<<<<<<<<
 *             val = self[key]
 *             if val is None:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "selectolax/modest/node.pxi":115
 *             return default
 * 
 *     def sget(self, key, default=""):             # <<<<<<<<<<<<<<
 *         """Same as get, but returns empty strings instead of None values for empty attributes."""
 *         try:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("selectolax.parser._Attributes.sget", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":125
 *             return default
 * 
 *     def __contains__(self, key):             # <<<<<<<<<<<<<<
 *         try:
 *             self[key]
*/

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_11_Attributes_24__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static int __pyx_pw_10selectolax_6parser_11_Attributes_24__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__contains__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_23__contains__(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_11_Attributes_23__contains__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__contains__", 0);

  /* "selectolax/modest/node.pxi":126
 * 
 *     def __contains__(self, key):
 *         try:             # <<<<<<<<<<<<<<
 *             self[key]
 *         except KeyError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "selectolax/modest/node.pxi":127
 *     def __contains__(self, key):
 *         try:
 *             self[key]             # <<<<<<<<<<<<<<
 *         except KeyError:
 *             return False
*/
      __pyx_t_4 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 127, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "selectolax/modest/node.pxi":126
 * 
 *     def __contains__(self, key):
 *         try:             # <<<<<<<<<<<<<<
 *             self[key]
 *         except KeyError:
*/
    }

    /* "selectolax/modest/node.pxi":131
 *             return False
 *         else:
 *             return True             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
    /*else:*/ {
      __pyx_r = 1;
      goto __pyx_L6_except_return;
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/modest/node.pxi":128
 *         try:
 *             self[key]
 *         except KeyError:             # <<<<<<<<<<<<<<
 *             return False
 *         else:
*/
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_5) {
      __Pyx_ErrRestore(0,0,0);

      /* "selectolax/modest/node.pxi":129
 *             self[key]
 *         except KeyError:
 *             return False             # <<<<<<<<<<<<<<
 *         else:
 *             return True
*/
      __pyx_r = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;

    /* "selectolax/modest/node.pxi":126
 * 
 *     def __contains__(self, key):
 *         try:             # <<<<<<<<<<<<<<
 *             self[key]
 *         except KeyError:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "selectolax/modest/node.pxi":125
 *             return default
 * 
 *     def __contains__(self, key):             # <<<<<<<<<<<<<<
 *         try:
 *             self[key]
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("selectolax.parser._Attributes.__contains__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":133
 *             return True
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         cdef const char *c_text
 *         c_text = myhtml_tag_name_by_id(self.node.tree, self.node.tag_id, NULL)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_26__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_26__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_25__repr__(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_25__repr__(struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self) {
  char const *__pyx_v_c_text;
  PyObject *__pyx_v_tag_name = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5[5];
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "selectolax/modest/node.pxi":135
 *     def __repr__(self):
 *         cdef const char *c_text
 *         c_text = myhtml_tag_name_by_id(self.node.tree, self.node.tag_id, NULL)             # <<<<<<<<<<<<<<
 *         tag_name = c_text.decode(_ENCODING, 'ignore') if c_text != NULL else 'unknown'
 *         return "<%s attributes, %s items>" % (tag_name, len(self))
*/
  __pyx_v_c_text = myhtml_tag_name_by_id(__pyx_v_self->node->tree, __pyx_v_self->node->tag_id, NULL);

  /* "selectolax/modest/node.pxi":136
 *         cdef const char *c_text
 *         c_text = myhtml_tag_name_by_id(self.node.tree, self.node.tag_id, NULL)
 *         tag_name = c_text.decode(_ENCODING, 'ignore') if c_text != NULL else 'unknown'             # <<<<<<<<<<<<<<
 *         return "<%s attributes, %s items>" % (tag_name, len(self))
 * 
*/
  __pyx_t_2 = (__pyx_v_c_text != NULL);
  if (__pyx_t_2) {
    __pyx_t_3 = __Pyx_ssize_strlen(__pyx_v_c_text); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(3, 136, __pyx_L1_error)
    __pyx_t_4 = __Pyx_decode_c_string(__pyx_v_c_text, 0, __pyx_t_3, NULL, ((char const *)"ignore"), PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __pyx_t_4 = 0;
  } else {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_unknown);
    __pyx_t_1 = __pyx_mstate_global->__pyx_n_u_unknown;
  }
  __pyx_v_tag_name = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":137
 *         c_text = myhtml_tag_name_by_id(self.node.tree, self.node.tag_id, NULL)
 *         tag_name = c_text.decode(_ENCODING, 'ignore') if c_text != NULL else 'unknown'
 *         return "<%s attributes, %s items>" % (tag_name, len(self))             # <<<<<<<<<<<<<<
 * 
 * ctypedef fused str_or_Node:
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_tag_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyObject_Length(((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(3, 137, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_t_3, 0, ' ', 'd'); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5[0] = __pyx_mstate_global->__pyx_kp_u__2;
  __pyx_t_5[1] = __pyx_t_1;
  __pyx_t_5[2] = __pyx_mstate_global->__pyx_kp_u_attributes_2;
  __pyx_t_5[3] = __pyx_t_4;
  __pyx_t_5[4] = __pyx_mstate_global->__pyx_kp_u_items_2;
  __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_5, 5, 1 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 13 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4) + 7, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1));
  if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":133
 *             return True
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         cdef const char *c_text
 *         c_text = myhtml_tag_name_by_id(self.node.tree, self.node.tag_id, NULL)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser._Attributes.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tag_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_28__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_11_Attributes_27__reduce_cython__, "_Attributes.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11_Attributes_28__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_28__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_27__reduce_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_28__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_27__reduce_cython__(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_27__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_self_node_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser._Attributes.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_30__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_11_Attributes_29__setstate_cython__, "_Attributes.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11_Attributes_30__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_30__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_29__setstate_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_11_Attributes_30__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser._Attributes.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_11_Attributes_29__setstate_cython__(((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_11_Attributes_29__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_self_node_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser._Attributes.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":149
 *     cdef public HTMLParser parser
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef Node new(myhtml_tree_node_t *node, HTMLParser parser):
 *         # custom __init__ for C, because __cinit__ doesn't accept C types
*/

static struct __pyx_obj_10selectolax_6parser_Node *__pyx_f_10selectolax_6parser_4Node_new(myhtml_tree_node_t *__pyx_v_node, struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_parser) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_cls = 0;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("new", 0);

  /* "selectolax/modest/node.pxi":152
 *     cdef Node new(myhtml_tree_node_t *node, HTMLParser parser):
 *         # custom __init__ for C, because __cinit__ doesn't accept C types
 *         cdef Node cls = Node.__new__(Node)             # <<<<<<<<<<<<<<
 *         cls.node = node
 *         # Keep reference to the selector object, so myhtml structures will not be garbage collected prematurely
*/
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_10selectolax_6parser_Node(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 152, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_1);
  __pyx_v_cls = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":153
 *         # custom __init__ for C, because __cinit__ doesn't accept C types
 *         cdef Node cls = Node.__new__(Node)
 *         cls.node = node             # <<<<<<<<<<<<<<
 *         # Keep reference to the selector object, so myhtml structures will not be garbage collected prematurely
 *         cls.parser = parser
*/
  __pyx_v_cls->node = __pyx_v_node;

  /* "selectolax/modest/node.pxi":155
 *         cls.node = node
 *         # Keep reference to the selector object, so myhtml structures will not be garbage collected prematurely
 *         cls.parser = parser             # <<<<<<<<<<<<<<
 *         return cls
 * 
*/
  __Pyx_INCREF((PyObject *)__pyx_v_parser);
  __Pyx_GIVEREF((PyObject *)__pyx_v_parser);
  __Pyx_GOTREF((PyObject *)__pyx_v_cls->parser);
  __Pyx_DECREF((PyObject *)__pyx_v_cls->parser);
  __pyx_v_cls->parser = __pyx_v_parser;

  /* "selectolax/modest/node.pxi":156
 *         # Keep reference to the selector object, so myhtml structures will not be garbage collected prematurely
 *         cls.parser = parser
 *         return cls             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_cls);
  __pyx_r = __pyx_v_cls;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":149
 *     cdef public HTMLParser parser
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef Node new(myhtml_tree_node_t *node, HTMLParser parser):
 *         # custom __init__ for C, because __cinit__ doesn't accept C types
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.Node.new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_cls);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":158
 *         return cls
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def attributes(self):
 *         """Get all attributes that belong to the current node.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_10attributes_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_10attributes_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_10attributes___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_10attributes___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  myhtml_tree_attr_t *__pyx_v_attr;
  PyObject *__pyx_v_attributes = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  myhtml_tree_attr_t *__pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":176
 *         {'data': None, 'id': 'my_id'}
 *         """
 *         cdef myhtml_tree_attr_t *attr = myhtml_node_attribute_first(self.node)             # <<<<<<<<<<<<<<
 *         attributes = dict()
 * 
*/
  __pyx_v_attr = myhtml_node_attribute_first(__pyx_v_self->node);

  /* "selectolax/modest/node.pxi":177
 *         """
 *         cdef myhtml_tree_attr_t *attr = myhtml_node_attribute_first(self.node)
 *         attributes = dict()             # <<<<<<<<<<<<<<
 * 
 *         while attr:
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_attributes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":179
 *         attributes = dict()
 * 
 *         while attr:             # <<<<<<<<<<<<<<
 *             if attr.key.data == NULL:
 *                 attr = attr.next
*/
  while (1) {
    __pyx_t_2 = (__pyx_v_attr != 0);
    if (!__pyx_t_2) break;

    /* "selectolax/modest/node.pxi":180
 * 
 *         while attr:
 *             if attr.key.data == NULL:             # <<<<<<<<<<<<<<
 *                 attr = attr.next
 *                 continue
*/
    __pyx_t_2 = (__pyx_v_attr->key.data == NULL);
    if (__pyx_t_2) {

      /* "selectolax/modest/node.pxi":181
 *         while attr:
 *             if attr.key.data == NULL:
 *                 attr = attr.next             # <<<<<<<<<<<<<<
 *                 continue
 *             key = attr.key.data.decode(_ENCODING, self.parser.decode_errors)
*/
      __pyx_t_3 = __pyx_v_attr->next;
      __pyx_v_attr = __pyx_t_3;

      /* "selectolax/modest/node.pxi":182
 *             if attr.key.data == NULL:
 *                 attr = attr.next
 *                 continue             # <<<<<<<<<<<<<<
 *             key = attr.key.data.decode(_ENCODING, self.parser.decode_errors)
 *             if attr.value.data:
*/
      goto __pyx_L3_continue;

      /* "selectolax/modest/node.pxi":180
 * 
 *         while attr:
 *             if attr.key.data == NULL:             # <<<<<<<<<<<<<<
 *                 attr = attr.next
 *                 continue
*/
    }

    /* "selectolax/modest/node.pxi":183
 *                 attr = attr.next
 *                 continue
 *             key = attr.key.data.decode(_ENCODING, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 *             if attr.value.data:
 *                 value = attr.value.data.decode(_ENCODING, self.parser.decode_errors)
*/
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_attr->key.data); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->parser->decode_errors};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 183, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "selectolax/modest/node.pxi":184
 *                 continue
 *             key = attr.key.data.decode(_ENCODING, self.parser.decode_errors)
 *             if attr.value.data:             # <<<<<<<<<<<<<<
 *                 value = attr.value.data.decode(_ENCODING, self.parser.decode_errors)
 *             else:
*/
    __pyx_t_2 = (__pyx_v_attr->value.data != 0);
    if (__pyx_t_2) {

      /* "selectolax/modest/node.pxi":185
 *             key = attr.key.data.decode(_ENCODING, self.parser.decode_errors)
 *             if attr.value.data:
 *                 value = attr.value.data.decode(_ENCODING, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 *             else:
 *                 value = None
*/
      __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_attr->value.data); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 185, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __pyx_t_4;
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->parser->decode_errors};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 185, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "selectolax/modest/node.pxi":184
 *                 continue
 *             key = attr.key.data.decode(_ENCODING, self.parser.decode_errors)
 *             if attr.value.data:             # <<<<<<<<<<<<<<
 *                 value = attr.value.data.decode(_ENCODING, self.parser.decode_errors)
 *             else:
*/
      goto __pyx_L6;
    }

    /* "selectolax/modest/node.pxi":187
 *                 value = attr.value.data.decode(_ENCODING, self.parser.decode_errors)
 *             else:
 *                 value = None             # <<<<<<<<<<<<<<
 *             attributes[key] = value
 * 
*/
    /*else*/ {
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_value, Py_None);
    }
    __pyx_L6:;

    /* "selectolax/modest/node.pxi":188
 *             else:
 *                 value = None
 *             attributes[key] = value             # <<<<<<<<<<<<<<
 * 
 *             attr = attr.next
*/
    if (unlikely((PyDict_SetItem(__pyx_v_attributes, __pyx_v_key, __pyx_v_value) < 0))) __PYX_ERR(3, 188, __pyx_L1_error)

    /* "selectolax/modest/node.pxi":190
 *             attributes[key] = value
 * 
 *             attr = attr.next             # <<<<<<<<<<<<<<
 * 
 *         return attributes
*/
    __pyx_t_3 = __pyx_v_attr->next;
    __pyx_v_attr = __pyx_t_3;
    __pyx_L3_continue:;
  }

  /* "selectolax/modest/node.pxi":192
 *             attr = attr.next
 * 
 *         return attributes             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_attributes);
  __pyx_r = __pyx_v_attributes;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":158
 *         return cls
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def attributes(self):
 *         """Get all attributes that belong to the current node.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.Node.attributes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_attributes);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":194
 *         return attributes
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def attrs(self):
 *         """A dict-like object that is similar to the ``attributes`` property, but operates directly on the Node data.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_5attrs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_5attrs_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_5attrs___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_5attrs___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser__Attributes *__pyx_v_attributes = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":221
 *         '<div foo="bar" id="new_id"></div>'
 *         """
 *         cdef _Attributes attributes = _Attributes.create(self.node, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 *         return attributes
 * 
*/
  __pyx_t_1 = __pyx_v_self->parser->decode_errors;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_f_10selectolax_6parser_11_Attributes_create(__pyx_v_self->node, ((PyObject*)__pyx_t_1))); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_attributes = ((struct __pyx_obj_10selectolax_6parser__Attributes *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":222
 *         """
 *         cdef _Attributes attributes = _Attributes.create(self.node, self.parser.decode_errors)
 *         return attributes             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_attributes);
  __pyx_r = ((PyObject *)__pyx_v_attributes);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":194
 *         return attributes
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def attrs(self):
 *         """A dict-like object that is similar to the ``attributes`` property, but operates directly on the Node data.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.Node.attrs.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_attributes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":224
 *         return attributes
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def mem_id(self):
 *         """Get the mem_id attribute of the node.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_6mem_id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_6mem_id_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_6mem_id___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_6mem_id___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":232
 *         text : int
 *         """
 *         return <size_t> self.node             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_FromSize_t(((size_t)__pyx_v_self->node)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":224
 *         return attributes
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def mem_id(self):
 *         """Get the mem_id attribute of the node.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.Node.mem_id.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":234
 *         return <size_t> self.node
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def id(self):
 *         """Get the id attribute of the node.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_2id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_2id_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_2id___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_2id___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  char *__pyx_v_key;
  myhtml_tree_attr_t *__pyx_v_attr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":244
 *         text : str
 *         """
 *         cdef char* key = 'id'             # <<<<<<<<<<<<<<
 *         cdef myhtml_tree_attr_t *attr
 *         attr = myhtml_attribute_by_key(self.node, key, 2)
*/
  __pyx_v_key = ((char *)"id");

  /* "selectolax/modest/node.pxi":246
 *         cdef char* key = 'id'
 *         cdef myhtml_tree_attr_t *attr
 *         attr = myhtml_attribute_by_key(self.node, key, 2)             # <<<<<<<<<<<<<<
 *         return None if attr == NULL else attr.value.data.decode(_ENCODING, self.parser.decode_errors)
 * 
*/
  __pyx_v_attr = myhtml_attribute_by_key(__pyx_v_self->node, __pyx_v_key, 2);

  /* "selectolax/modest/node.pxi":247
 *         cdef myhtml_tree_attr_t *attr
 *         attr = myhtml_attribute_by_key(self.node, key, 2)
 *         return None if attr == NULL else attr.value.data.decode(_ENCODING, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = (__pyx_v_attr == NULL);
  if (__pyx_t_2) {
    __Pyx_INCREF(Py_None);
    __pyx_t_1 = Py_None;
  } else {
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_attr->value.data); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->parser->decode_errors};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 247, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":234
 *         return <size_t> self.node
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def id(self):
 *         """Get the id attribute of the node.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.Node.id.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":249
 *         return None if attr == NULL else attr.value.data.decode(_ENCODING, self.parser.decode_errors)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return self.mem_id
 * 
*/

/* Python wrapper */
static Py_hash_t __pyx_pw_10selectolax_6parser_4Node_1__hash__(PyObject *__pyx_v_self); /*proto*/
static Py_hash_t __pyx_pw_10selectolax_6parser_4Node_1__hash__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node___hash__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_hash_t __pyx_pf_10selectolax_6parser_4Node___hash__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_hash_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "selectolax/modest/node.pxi":250
 * 
 *     def __hash__(self):
 *         return self.mem_id             # <<<<<<<<<<<<<<
 * 
 *     def text(self, bool deep=True, str separator='', bool strip=False):
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_mem_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_AsHash_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (Py_hash_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 250, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":249
 *         return None if attr == NULL else attr.value.data.decode(_ENCODING, self.parser.decode_errors)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return self.mem_id
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.Node.__hash__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":252
 *         return self.mem_id
 * 
 *     def text(self, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Returns the text of the node including text of all its child nodes.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_3text(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_2text, "Node.text(self, bool deep=True, str separator='', bool strip=False)\n\nReturns the text of the node including text of all its child nodes.\n\nParameters\n----------\nstrip : bool, default False\n    If true, calls ``str.strip()`` on each text part to remove extra white spaces.\nseparator : str, default ''\n    The separator to use when joining text from different nodes.\ndeep : bool, default True\n    If True, includes text from all child nodes.\n\nReturns\n-------\ntext : str");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_3text = {"text", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_3text, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_2text};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_3text(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyLongObject *__pyx_v_deep = 0;
  PyObject *__pyx_v_separator = 0;
  PyLongObject *__pyx_v_strip = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("text (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_deep,&__pyx_mstate_global->__pyx_n_u_separator,&__pyx_mstate_global->__pyx_n_u_strip,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 252, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 252, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 252, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 252, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "text", 0) < 0) __PYX_ERR(3, 252, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)__pyx_mstate_global->__pyx_kp_u_));
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 252, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 252, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 252, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)__pyx_mstate_global->__pyx_kp_u_));
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_deep = ((PyLongObject *)values[0]);
    __pyx_v_separator = ((PyObject*)values[1]);
    __pyx_v_strip = ((PyLongObject *)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("text", 0, 0, 3, __pyx_nargs); __PYX_ERR(3, 252, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_deep), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "deep", 0))) __PYX_ERR(3, 252, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_separator), (&PyUnicode_Type), 1, "separator", 1))) __PYX_ERR(3, 252, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_strip), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "strip", 0))) __PYX_ERR(3, 252, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_2text(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_deep, __pyx_v_separator, __pyx_v_strip);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_2text(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyLongObject *__pyx_v_deep, PyObject *__pyx_v_separator, PyLongObject *__pyx_v_strip) {
  PyObject *__pyx_v_text = NULL;
  char const *__pyx_v_c_text;
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_node_text = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  myhtml_tree_node_t *__pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  struct __pyx_opt_args_10selectolax_6parser_append_text __pyx_t_8;
  struct __pyx_opt_args_10selectolax_6parser_4Node__text_deep __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("text", 0);

  /* "selectolax/modest/node.pxi":269
 * 
 *         """
 *         text = ""             # <<<<<<<<<<<<<<
 *         cdef const char* c_text
 *         cdef myhtml_tree_node_t *node = self.node.child
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
  __pyx_v_text = __pyx_mstate_global->__pyx_kp_u_;

  /* "selectolax/modest/node.pxi":271
 *         text = ""
 *         cdef const char* c_text
 *         cdef myhtml_tree_node_t *node = self.node.child             # <<<<<<<<<<<<<<
 * 
 *         if not deep:
*/
  __pyx_t_1 = __pyx_v_self->node->child;
  __pyx_v_node = __pyx_t_1;

  /* "selectolax/modest/node.pxi":273
 *         cdef myhtml_tree_node_t *node = self.node.child
 * 
 *         if not deep:             # <<<<<<<<<<<<<<
 *             if self.node.tag_id == MyHTML_TAG__TEXT:
 *                 c_text = myhtml_node_text(self.node, NULL)
*/
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_deep)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(3, 273, __pyx_L1_error)
  __pyx_t_3 = (!__pyx_t_2);
  if (__pyx_t_3) {

    /* "selectolax/modest/node.pxi":274
 * 
 *         if not deep:
 *             if self.node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *                 c_text = myhtml_node_text(self.node, NULL)
 *                 if c_text != NULL:
*/
    __pyx_t_3 = (__pyx_v_self->node->tag_id == MyHTML_TAG__TEXT);
    if (__pyx_t_3) {

      /* "selectolax/modest/node.pxi":275
 *         if not deep:
 *             if self.node.tag_id == MyHTML_TAG__TEXT:
 *                 c_text = myhtml_node_text(self.node, NULL)             # <<<<<<<<<<<<<<
 *                 if c_text != NULL:
 *                     node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
*/
      __pyx_v_c_text = myhtml_node_text(__pyx_v_self->node, NULL);

      /* "selectolax/modest/node.pxi":276
 *             if self.node.tag_id == MyHTML_TAG__TEXT:
 *                 c_text = myhtml_node_text(self.node, NULL)
 *                 if c_text != NULL:             # <<<<<<<<<<<<<<
 *                     node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                     text = append_text(text, node_text, separator, strip)
*/
      __pyx_t_3 = (__pyx_v_c_text != NULL);
      if (__pyx_t_3) {

        /* "selectolax/modest/node.pxi":277
 *                 c_text = myhtml_node_text(self.node, NULL)
 *                 if c_text != NULL:
 *                     node_text = c_text.decode(_ENCODING, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 *                     text = append_text(text, node_text, separator, strip)
 * 
*/
        __pyx_t_6 = __Pyx_PyBytes_FromString(__pyx_v_c_text); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 277, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_5 = __pyx_t_6;
        __Pyx_INCREF(__pyx_t_5);
        __pyx_t_7 = 0;
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->parser->decode_errors};
          __pyx_t_4 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 277, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __pyx_v_node_text = __pyx_t_4;
        __pyx_t_4 = 0;

        /* "selectolax/modest/node.pxi":278
 *                 if c_text != NULL:
 *                     node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                     text = append_text(text, node_text, separator, strip)             # <<<<<<<<<<<<<<
 * 
 *             while node != NULL:
*/
        if (!(likely(PyUnicode_CheckExact(__pyx_v_text)) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_text))) __PYX_ERR(3, 278, __pyx_L1_error)
        if (!(likely(PyUnicode_CheckExact(__pyx_v_node_text)) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_node_text))) __PYX_ERR(3, 278, __pyx_L1_error)
        __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_strip)); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 278, __pyx_L1_error)
        __pyx_t_8.__pyx_n = 2;
        __pyx_t_8.separator = __pyx_v_separator;
        __pyx_t_8.strip = __pyx_t_3;
        __pyx_t_4 = __pyx_f_10selectolax_6parser_append_text(((PyObject*)__pyx_v_text), ((PyObject*)__pyx_v_node_text), &__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 278, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF_SET(__pyx_v_text, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "selectolax/modest/node.pxi":276
 *             if self.node.tag_id == MyHTML_TAG__TEXT:
 *                 c_text = myhtml_node_text(self.node, NULL)
 *                 if c_text != NULL:             # <<<<<<<<<<<<<<
 *                     node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                     text = append_text(text, node_text, separator, strip)
*/
      }

      /* "selectolax/modest/node.pxi":274
 * 
 *         if not deep:
 *             if self.node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *                 c_text = myhtml_node_text(self.node, NULL)
 *                 if c_text != NULL:
*/
    }

    /* "selectolax/modest/node.pxi":280
 *                     text = append_text(text, node_text, separator, strip)
 * 
 *             while node != NULL:             # <<<<<<<<<<<<<<
 *                 if node.tag_id == MyHTML_TAG__TEXT:
 *                     c_text = myhtml_node_text(node, NULL)
*/
    while (1) {
      __pyx_t_3 = (__pyx_v_node != NULL);
      if (!__pyx_t_3) break;

      /* "selectolax/modest/node.pxi":281
 * 
 *             while node != NULL:
 *                 if node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *                     c_text = myhtml_node_text(node, NULL)
 *                     if c_text != NULL:
*/
      __pyx_t_3 = (__pyx_v_node->tag_id == MyHTML_TAG__TEXT);
      if (__pyx_t_3) {

        /* "selectolax/modest/node.pxi":282
 *             while node != NULL:
 *                 if node.tag_id == MyHTML_TAG__TEXT:
 *                     c_text = myhtml_node_text(node, NULL)             # <<<<<<<<<<<<<<
 *                     if c_text != NULL:
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
*/
        __pyx_v_c_text = myhtml_node_text(__pyx_v_node, NULL);

        /* "selectolax/modest/node.pxi":283
 *                 if node.tag_id == MyHTML_TAG__TEXT:
 *                     c_text = myhtml_node_text(node, NULL)
 *                     if c_text != NULL:             # <<<<<<<<<<<<<<
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                         text = append_text(text, node_text, separator, strip)
*/
        __pyx_t_3 = (__pyx_v_c_text != NULL);
        if (__pyx_t_3) {

          /* "selectolax/modest/node.pxi":284
 *                     c_text = myhtml_node_text(node, NULL)
 *                     if c_text != NULL:
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 *                         text = append_text(text, node_text, separator, strip)
 *                 node = node.next
*/
          __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_c_text); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 284, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_6 = __pyx_t_5;
          __Pyx_INCREF(__pyx_t_6);
          __pyx_t_7 = 0;
          {
            PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->parser->decode_errors};
            __pyx_t_4 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 284, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
          }
          __Pyx_XDECREF_SET(__pyx_v_node_text, __pyx_t_4);
          __pyx_t_4 = 0;

          /* "selectolax/modest/node.pxi":285
 *                     if c_text != NULL:
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                         text = append_text(text, node_text, separator, strip)             # <<<<<<<<<<<<<<
 *                 node = node.next
 *         else:
*/
          if (!(likely(PyUnicode_CheckExact(__pyx_v_text))||((__pyx_v_text) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_text))) __PYX_ERR(3, 285, __pyx_L1_error)
          if (!(likely(PyUnicode_CheckExact(__pyx_v_node_text)) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_node_text))) __PYX_ERR(3, 285, __pyx_L1_error)
          __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_strip)); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 285, __pyx_L1_error)
          __pyx_t_8.__pyx_n = 2;
          __pyx_t_8.separator = __pyx_v_separator;
          __pyx_t_8.strip = __pyx_t_3;
          __pyx_t_4 = __pyx_f_10selectolax_6parser_append_text(((PyObject*)__pyx_v_text), ((PyObject*)__pyx_v_node_text), &__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 285, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF_SET(__pyx_v_text, __pyx_t_4);
          __pyx_t_4 = 0;

          /* "selectolax/modest/node.pxi":283
 *                 if node.tag_id == MyHTML_TAG__TEXT:
 *                     c_text = myhtml_node_text(node, NULL)
 *                     if c_text != NULL:             # <<<<<<<<<<<<<<
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                         text = append_text(text, node_text, separator, strip)
*/
        }

        /* "selectolax/modest/node.pxi":281
 * 
 *             while node != NULL:
 *                 if node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *                     c_text = myhtml_node_text(node, NULL)
 *                     if c_text != NULL:
*/
      }

      /* "selectolax/modest/node.pxi":286
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                         text = append_text(text, node_text, separator, strip)
 *                 node = node.next             # <<<<<<<<<<<<<<
 *         else:
 *             text = self._text_deep(self.node, separator=separator, strip=strip)
*/
      __pyx_t_1 = __pyx_v_node->next;
      __pyx_v_node = __pyx_t_1;
    }

    /* "selectolax/modest/node.pxi":273
 *         cdef myhtml_tree_node_t *node = self.node.child
 * 
 *         if not deep:             # <<<<<<<<<<<<<<
 *             if self.node.tag_id == MyHTML_TAG__TEXT:
 *                 c_text = myhtml_node_text(self.node, NULL)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":288
 *                 node = node.next
 *         else:
 *             text = self._text_deep(self.node, separator=separator, strip=strip)             # <<<<<<<<<<<<<<
 *         if separator and text and text.endswith(separator):
 *             text = text[:-len(separator)]
*/
  /*else*/ {
    __pyx_t_9.__pyx_n = 2;
    __pyx_t_9.separator = __pyx_v_separator;
    __pyx_t_9.strip = ((PyObject *)__pyx_v_strip);
    __pyx_t_4 = __pyx_f_10selectolax_6parser_4Node__text_deep(__pyx_v_self, __pyx_v_self->node, &__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 288, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF_SET(__pyx_v_text, __pyx_t_4);
    __pyx_t_4 = 0;
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":289
 *         else:
 *             text = self._text_deep(self.node, separator=separator, strip=strip)
 *         if separator and text and text.endswith(separator):             # <<<<<<<<<<<<<<
 *             text = text[:-len(separator)]
 *         return text
*/
  __pyx_t_2 = (__pyx_v_separator != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_separator) != 0);
  if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_2 < 0))) __PYX_ERR(3, 289, __pyx_L1_error)
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_text); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(3, 289, __pyx_L1_error)
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_5 = __pyx_v_text;
  __Pyx_INCREF(__pyx_t_5);
  __pyx_t_7 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_separator};
    __pyx_t_4 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_endswith, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 289, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(3, 289, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = __pyx_t_2;
  __pyx_L11_bool_binop_done:;
  if (__pyx_t_3) {

    /* "selectolax/modest/node.pxi":290
 *             text = self._text_deep(self.node, separator=separator, strip=strip)
 *         if separator and text and text.endswith(separator):
 *             text = text[:-len(separator)]             # <<<<<<<<<<<<<<
 *         return text
 * 
*/
    if (unlikely(__pyx_v_separator == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 290, __pyx_L1_error)
    }
    __pyx_t_10 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_separator); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(3, 290, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_v_text, 0, (-__pyx_t_10), NULL, NULL, NULL, 0, 1, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF_SET(__pyx_v_text, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "selectolax/modest/node.pxi":289
 *         else:
 *             text = self._text_deep(self.node, separator=separator, strip=strip)
 *         if separator and text and text.endswith(separator):             # <<<<<<<<<<<<<<
 *             text = text[:-len(separator)]
 *         return text
*/
  }

  /* "selectolax/modest/node.pxi":291
 *         if separator and text and text.endswith(separator):
 *             text = text[:-len(separator)]
 *         return text             # <<<<<<<<<<<<<<
 * 
 *     cdef inline _text_deep(self, myhtml_tree_node_t *node, separator='', strip=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_text);
  __pyx_r = __pyx_v_text;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":252
 *         return self.mem_id
 * 
 *     def text(self, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Returns the text of the node including text of all its child nodes.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser.Node.text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_text);
  __Pyx_XDECREF(__pyx_v_node_text);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":293
 *         return text
 * 
 *     cdef inline _text_deep(self, myhtml_tree_node_t *node, separator='', strip=False):             # <<<<<<<<<<<<<<
 *         text = ""
 *         cdef Stack stack = Stack(_STACK_SIZE)
*/

static CYTHON_INLINE PyObject *__pyx_f_10selectolax_6parser_4Node__text_deep(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, myhtml_tree_node_t *__pyx_v_node, struct __pyx_opt_args_10selectolax_6parser_4Node__text_deep *__pyx_optional_args) {
  PyObject *__pyx_v_separator = ((PyObject *)__pyx_mstate_global->__pyx_kp_u_);
  PyObject *__pyx_v_strip = ((PyObject *)Py_False);
  PyObject *__pyx_v_text = NULL;
  struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_stack = 0;
  myhtml_tree_node_t *__pyx_v_current_node;
  char const *__pyx_v_c_text;
  PyObject *__pyx_v_node_text = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  struct __pyx_opt_args_10selectolax_6parser_append_text __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  myhtml_tree_node_t *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_text_deep", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_separator = __pyx_optional_args->separator;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_strip = __pyx_optional_args->strip;
      }
    }
  }

  /* "selectolax/modest/node.pxi":294
 * 
 *     cdef inline _text_deep(self, myhtml_tree_node_t *node, separator='', strip=False):
 *         text = ""             # <<<<<<<<<<<<<<
 *         cdef Stack stack = Stack(_STACK_SIZE)
 *         cdef myhtml_tree_node_t* current_node = NULL
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
  __pyx_v_text = __pyx_mstate_global->__pyx_kp_u_;

  /* "selectolax/modest/node.pxi":295
 *     cdef inline _text_deep(self, myhtml_tree_node_t *node, separator='', strip=False):
 *         text = ""
 *         cdef Stack stack = Stack(_STACK_SIZE)             # <<<<<<<<<<<<<<
 *         cdef myhtml_tree_node_t* current_node = NULL
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Stack);
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Stack); 
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_int_100};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 295, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_stack = ((struct __pyx_obj_10selectolax_6parser_Stack *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":296
 *         text = ""
 *         cdef Stack stack = Stack(_STACK_SIZE)
 *         cdef myhtml_tree_node_t* current_node = NULL             # <<<<<<<<<<<<<<
 * 
 *         if node.tag_id == MyHTML_TAG__TEXT:
*/
  __pyx_v_current_node = NULL;

  /* "selectolax/modest/node.pxi":298
 *         cdef myhtml_tree_node_t* current_node = NULL
 * 
 *         if node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *             c_text = myhtml_node_text(node, NULL)
 *             if c_text != NULL:
*/
  __pyx_t_5 = (__pyx_v_node->tag_id == MyHTML_TAG__TEXT);
  if (__pyx_t_5) {

    /* "selectolax/modest/node.pxi":299
 * 
 *         if node.tag_id == MyHTML_TAG__TEXT:
 *             c_text = myhtml_node_text(node, NULL)             # <<<<<<<<<<<<<<
 *             if c_text != NULL:
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
*/
    __pyx_v_c_text = myhtml_node_text(__pyx_v_node, NULL);

    /* "selectolax/modest/node.pxi":300
 *         if node.tag_id == MyHTML_TAG__TEXT:
 *             c_text = myhtml_node_text(node, NULL)
 *             if c_text != NULL:             # <<<<<<<<<<<<<<
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                 text = append_text(text, node_text, separator, strip)
*/
    __pyx_t_5 = (__pyx_v_c_text != NULL);
    if (__pyx_t_5) {

      /* "selectolax/modest/node.pxi":301
 *             c_text = myhtml_node_text(node, NULL)
 *             if c_text != NULL:
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 *                 text = append_text(text, node_text, separator, strip)
 * 
*/
      __pyx_t_2 = __Pyx_PyBytes_FromString(__pyx_v_c_text); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 301, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __pyx_t_2;
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = 0;
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->parser->decode_errors};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 301, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __pyx_v_node_text = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "selectolax/modest/node.pxi":302
 *             if c_text != NULL:
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                 text = append_text(text, node_text, separator, strip)             # <<<<<<<<<<<<<<
 * 
 *         if node.child == NULL:
*/
      if (!(likely(PyUnicode_CheckExact(__pyx_v_node_text)) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_node_text))) __PYX_ERR(3, 302, __pyx_L1_error)
      if (!(likely(PyUnicode_CheckExact(__pyx_v_separator))||((__pyx_v_separator) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_separator))) __PYX_ERR(3, 302, __pyx_L1_error)
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_strip); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 302, __pyx_L1_error)
      __pyx_t_6.__pyx_n = 2;
      __pyx_t_6.separator = ((PyObject*)__pyx_v_separator);
      __pyx_t_6.strip = __pyx_t_5;
      __pyx_t_1 = __pyx_f_10selectolax_6parser_append_text(__pyx_v_text, ((PyObject*)__pyx_v_node_text), &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_text, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "selectolax/modest/node.pxi":300
 *         if node.tag_id == MyHTML_TAG__TEXT:
 *             c_text = myhtml_node_text(node, NULL)
 *             if c_text != NULL:             # <<<<<<<<<<<<<<
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                 text = append_text(text, node_text, separator, strip)
*/
    }

    /* "selectolax/modest/node.pxi":298
 *         cdef myhtml_tree_node_t* current_node = NULL
 * 
 *         if node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *             c_text = myhtml_node_text(node, NULL)
 *             if c_text != NULL:
*/
  }

  /* "selectolax/modest/node.pxi":304
 *                 text = append_text(text, node_text, separator, strip)
 * 
 *         if node.child == NULL:             # <<<<<<<<<<<<<<
 *             return text
 * 
*/
  __pyx_t_5 = (__pyx_v_node->child == NULL);
  if (__pyx_t_5) {

    /* "selectolax/modest/node.pxi":305
 * 
 *         if node.child == NULL:
 *             return text             # <<<<<<<<<<<<<<
 * 
 *         stack.push(node.child)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_text);
    __pyx_r = __pyx_v_text;
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":304
 *                 text = append_text(text, node_text, separator, strip)
 * 
 *         if node.child == NULL:             # <<<<<<<<<<<<<<
 *             return text
 * 
*/
  }

  /* "selectolax/modest/node.pxi":307
 *             return text
 * 
 *         stack.push(node.child)             # <<<<<<<<<<<<<<
 * 
 *         # Depth-first left-to-right tree traversal
*/
  __pyx_t_7 = __pyx_f_10selectolax_6parser_5Stack_push(__pyx_v_stack, __pyx_v_node->child); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(3, 307, __pyx_L1_error)

  /* "selectolax/modest/node.pxi":310
 * 
 *         # Depth-first left-to-right tree traversal
 *         while not stack.is_empty():             # <<<<<<<<<<<<<<
 *             current_node = stack.pop()
 * 
*/
  while (1) {
    __pyx_t_5 = __pyx_f_10selectolax_6parser_5Stack_is_empty(__pyx_v_stack); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 310, __pyx_L1_error)
    __pyx_t_8 = (!__pyx_t_5);
    if (!__pyx_t_8) break;

    /* "selectolax/modest/node.pxi":311
 *         # Depth-first left-to-right tree traversal
 *         while not stack.is_empty():
 *             current_node = stack.pop()             # <<<<<<<<<<<<<<
 * 
 *             if current_node != NULL:
*/
    __pyx_t_9 = __pyx_f_10selectolax_6parser_5Stack_pop(__pyx_v_stack); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 311, __pyx_L1_error)
    __pyx_v_current_node = __pyx_t_9;

    /* "selectolax/modest/node.pxi":313
 *             current_node = stack.pop()
 * 
 *             if current_node != NULL:             # <<<<<<<<<<<<<<
 *                 if current_node.tag_id == MyHTML_TAG__TEXT:
 *                     c_text = myhtml_node_text(current_node, NULL)
*/
    __pyx_t_8 = (__pyx_v_current_node != NULL);
    if (__pyx_t_8) {

      /* "selectolax/modest/node.pxi":314
 * 
 *             if current_node != NULL:
 *                 if current_node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *                     c_text = myhtml_node_text(current_node, NULL)
 *                     if c_text != NULL:
*/
      __pyx_t_8 = (__pyx_v_current_node->tag_id == MyHTML_TAG__TEXT);
      if (__pyx_t_8) {

        /* "selectolax/modest/node.pxi":315
 *             if current_node != NULL:
 *                 if current_node.tag_id == MyHTML_TAG__TEXT:
 *                     c_text = myhtml_node_text(current_node, NULL)             # <<<<<<<<<<<<<<
 *                     if c_text != NULL:
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
*/
        __pyx_v_c_text = myhtml_node_text(__pyx_v_current_node, NULL);

        /* "selectolax/modest/node.pxi":316
 *                 if current_node.tag_id == MyHTML_TAG__TEXT:
 *                     c_text = myhtml_node_text(current_node, NULL)
 *                     if c_text != NULL:             # <<<<<<<<<<<<<<
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                         text = append_text(text, node_text, separator, strip)
*/
        __pyx_t_8 = (__pyx_v_c_text != NULL);
        if (__pyx_t_8) {

          /* "selectolax/modest/node.pxi":317
 *                     c_text = myhtml_node_text(current_node, NULL)
 *                     if c_text != NULL:
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 *                         text = append_text(text, node_text, separator, strip)
 * 
*/
          __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_c_text); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 317, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_2 = __pyx_t_3;
          __Pyx_INCREF(__pyx_t_2);
          __pyx_t_4 = 0;
          {
            PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->parser->decode_errors};
            __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 317, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
          }
          __Pyx_XDECREF_SET(__pyx_v_node_text, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "selectolax/modest/node.pxi":318
 *                     if c_text != NULL:
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                         text = append_text(text, node_text, separator, strip)             # <<<<<<<<<<<<<<
 * 
 *             if current_node.next is not NULL:
*/
          if (!(likely(PyUnicode_CheckExact(__pyx_v_node_text)) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_node_text))) __PYX_ERR(3, 318, __pyx_L1_error)
          if (!(likely(PyUnicode_CheckExact(__pyx_v_separator))||((__pyx_v_separator) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_separator))) __PYX_ERR(3, 318, __pyx_L1_error)
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_strip); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 318, __pyx_L1_error)
          __pyx_t_6.__pyx_n = 2;
          __pyx_t_6.separator = ((PyObject*)__pyx_v_separator);
          __pyx_t_6.strip = __pyx_t_8;
          __pyx_t_1 = __pyx_f_10selectolax_6parser_append_text(__pyx_v_text, ((PyObject*)__pyx_v_node_text), &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 318, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF_SET(__pyx_v_text, ((PyObject*)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "selectolax/modest/node.pxi":316
 *                 if current_node.tag_id == MyHTML_TAG__TEXT:
 *                     c_text = myhtml_node_text(current_node, NULL)
 *                     if c_text != NULL:             # <<<<<<<<<<<<<<
 *                         node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                         text = append_text(text, node_text, separator, strip)
*/
        }

        /* "selectolax/modest/node.pxi":314
 * 
 *             if current_node != NULL:
 *                 if current_node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *                     c_text = myhtml_node_text(current_node, NULL)
 *                     if c_text != NULL:
*/
      }

      /* "selectolax/modest/node.pxi":313
 *             current_node = stack.pop()
 * 
 *             if current_node != NULL:             # <<<<<<<<<<<<<<
 *                 if current_node.tag_id == MyHTML_TAG__TEXT:
 *                     c_text = myhtml_node_text(current_node, NULL)
*/
    }

    /* "selectolax/modest/node.pxi":320
 *                         text = append_text(text, node_text, separator, strip)
 * 
 *             if current_node.next is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.next)
 * 
*/
    __pyx_t_8 = (__pyx_v_current_node->next != NULL);
    if (__pyx_t_8) {

      /* "selectolax/modest/node.pxi":321
 * 
 *             if current_node.next is not NULL:
 *                 stack.push(current_node.next)             # <<<<<<<<<<<<<<
 * 
 *             if current_node.child is not NULL:
*/
      __pyx_t_7 = __pyx_f_10selectolax_6parser_5Stack_push(__pyx_v_stack, __pyx_v_current_node->next); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(3, 321, __pyx_L1_error)

      /* "selectolax/modest/node.pxi":320
 *                         text = append_text(text, node_text, separator, strip)
 * 
 *             if current_node.next is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.next)
 * 
*/
    }

    /* "selectolax/modest/node.pxi":323
 *                 stack.push(current_node.next)
 * 
 *             if current_node.child is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.child)
 * 
*/
    __pyx_t_8 = (__pyx_v_current_node->child != NULL);
    if (__pyx_t_8) {

      /* "selectolax/modest/node.pxi":324
 * 
 *             if current_node.child is not NULL:
 *                 stack.push(current_node.child)             # <<<<<<<<<<<<<<
 * 
 *         return text
*/
      __pyx_t_7 = __pyx_f_10selectolax_6parser_5Stack_push(__pyx_v_stack, __pyx_v_current_node->child); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(3, 324, __pyx_L1_error)

      /* "selectolax/modest/node.pxi":323
 *                 stack.push(current_node.next)
 * 
 *             if current_node.child is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.child)
 * 
*/
    }
  }

  /* "selectolax/modest/node.pxi":326
 *                 stack.push(current_node.child)
 * 
 *         return text             # <<<<<<<<<<<<<<
 * 
 *     def iter(self, include_text=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_text);
  __pyx_r = __pyx_v_text;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":293
 *         return text
 * 
 *     cdef inline _text_deep(self, myhtml_tree_node_t *node, separator='', strip=False):             # <<<<<<<<<<<<<<
 *         text = ""
 *         cdef Stack stack = Stack(_STACK_SIZE)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Node._text_deep", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_text);
  __Pyx_XDECREF((PyObject *)__pyx_v_stack);
  __Pyx_XDECREF(__pyx_v_node_text);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_10selectolax_6parser_4Node_6generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "selectolax/modest/node.pxi":328
 *         return text
 * 
 *     def iter(self, include_text=False):             # <<<<<<<<<<<<<<
 *         """Iterate over nodes on the current level.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_5iter(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_4iter, "Node.iter(self, include_text=False)\n\nIterate over nodes on the current level.\n\nParameters\n----------\ninclude_text : bool\n    If True, includes text nodes as well.\n\nYields\n-------\nnode");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_5iter = {"iter", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_5iter, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_4iter};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_5iter(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_include_text = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("iter (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_include_text,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 328, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 328, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "iter", 0) < 0) __PYX_ERR(3, 328, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 328, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
    }
    __pyx_v_include_text = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("iter", 0, 0, 1, __pyx_nargs); __PYX_ERR(3, 328, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.iter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_4iter(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_include_text);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_4iter(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_include_text) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("iter", 0);
  __pyx_cur_scope = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_3_iter(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(3, 328, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_include_text = __pyx_v_include_text;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_include_text);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_include_text);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_10selectolax_6parser_4Node_6generator3, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_iter_2, __pyx_mstate_global->__pyx_n_u_Node_iter, __pyx_mstate_global->__pyx_n_u_selectolax_parser); if (unlikely(!gen)) __PYX_ERR(3, 328, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Node.iter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_10selectolax_6parser_4Node_6generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *__pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  myhtml_tree_node_t *__pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("iter", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L9_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(3, 328, __pyx_L1_error)
  }

  /* "selectolax/modest/node.pxi":341
 *         """
 * 
 *         cdef myhtml_tree_node_t *node = self.node.child             # <<<<<<<<<<<<<<
 *         cdef Node next_node
 * 
*/
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_self->node->child;
  __pyx_cur_scope->__pyx_v_node = __pyx_t_1;

  /* "selectolax/modest/node.pxi":344
 *         cdef Node next_node
 * 
 *         while node != NULL:             # <<<<<<<<<<<<<<
 *             if node.tag_id == MyHTML_TAG__TEXT and not include_text:
 *                 node = node.next
*/
  while (1) {
    __pyx_t_2 = (__pyx_cur_scope->__pyx_v_node != NULL);
    if (!__pyx_t_2) break;

    /* "selectolax/modest/node.pxi":345
 * 
 *         while node != NULL:
 *             if node.tag_id == MyHTML_TAG__TEXT and not include_text:             # <<<<<<<<<<<<<<
 *                 node = node.next
 *                 continue
*/
    __pyx_t_3 = (__pyx_cur_scope->__pyx_v_node->tag_id == MyHTML_TAG__TEXT);
    if (__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_include_text); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(3, 345, __pyx_L1_error)
    __pyx_t_4 = (!__pyx_t_3);
    __pyx_t_2 = __pyx_t_4;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_2) {

      /* "selectolax/modest/node.pxi":346
 *         while node != NULL:
 *             if node.tag_id == MyHTML_TAG__TEXT and not include_text:
 *                 node = node.next             # <<<<<<<<<<<<<<
 *                 continue
 * 
*/
      __pyx_t_1 = __pyx_cur_scope->__pyx_v_node->next;
      __pyx_cur_scope->__pyx_v_node = __pyx_t_1;

      /* "selectolax/modest/node.pxi":347
 *             if node.tag_id == MyHTML_TAG__TEXT and not include_text:
 *                 node = node.next
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             next_node = Node.new(node, self.parser)
*/
      goto __pyx_L4_continue;

      /* "selectolax/modest/node.pxi":345
 * 
 *         while node != NULL:
 *             if node.tag_id == MyHTML_TAG__TEXT and not include_text:             # <<<<<<<<<<<<<<
 *                 node = node.next
 *                 continue
*/
    }

    /* "selectolax/modest/node.pxi":349
 *                 continue
 * 
 *             next_node = Node.new(node, self.parser)             # <<<<<<<<<<<<<<
 *             yield next_node
 *             node = node.next
*/
    __pyx_t_5 = ((PyObject *)__pyx_cur_scope->__pyx_v_self->parser);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_6 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_cur_scope->__pyx_v_node, ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_5))); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 349, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XGOTREF((PyObject *)__pyx_cur_scope->__pyx_v_next_node);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_next_node, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_6));
    __Pyx_GIVEREF(__pyx_t_6);
    __pyx_t_6 = 0;

    /* "selectolax/modest/node.pxi":350
 * 
 *             next_node = Node.new(node, self.parser)
 *             yield next_node             # <<<<<<<<<<<<<<
 *             node = node.next
 * 
*/
    __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_next_node);
    __pyx_r = ((PyObject *)__pyx_cur_scope->__pyx_v_next_node);
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L9_resume_from_yield:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(3, 350, __pyx_L1_error)

    /* "selectolax/modest/node.pxi":351
 *             next_node = Node.new(node, self.parser)
 *             yield next_node
 *             node = node.next             # <<<<<<<<<<<<<<
 * 
 *     def traverse(self, include_text=False):
*/
    __pyx_t_1 = __pyx_cur_scope->__pyx_v_node->next;
    __pyx_cur_scope->__pyx_v_node = __pyx_t_1;
    __pyx_L4_continue:;
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "selectolax/modest/node.pxi":328
 *         return text
 * 
 *     def iter(self, include_text=False):             # <<<<<<<<<<<<<<
 *         """Iterate over nodes on the current level.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("iter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_10selectolax_6parser_4Node_9generator4(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "selectolax/modest/node.pxi":353
 *             node = node.next
 * 
 *     def traverse(self, include_text=False):             # <<<<<<<<<<<<<<
 *         """Iterate over all child and next nodes starting from the current level.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_8traverse(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_7traverse, "Node.traverse(self, include_text=False)\n\nIterate over all child and next nodes starting from the current level.\n\nParameters\n----------\ninclude_text : bool\n    If True, includes text nodes as well.\n\nYields\n-------\nnode");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_8traverse = {"traverse", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_8traverse, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_7traverse};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_8traverse(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_include_text = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("traverse (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_include_text,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 353, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 353, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "traverse", 0) < 0) __PYX_ERR(3, 353, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 353, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
    }
    __pyx_v_include_text = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("traverse", 0, 0, 1, __pyx_nargs); __PYX_ERR(3, 353, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.traverse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_7traverse(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_include_text);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_7traverse(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_include_text) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("traverse", 0);
  __pyx_cur_scope = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_4_traverse(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(3, 353, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_include_text = __pyx_v_include_text;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_include_text);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_include_text);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_10selectolax_6parser_4Node_9generator4, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[4]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_traverse, __pyx_mstate_global->__pyx_n_u_Node_traverse, __pyx_mstate_global->__pyx_n_u_selectolax_parser); if (unlikely(!gen)) __PYX_ERR(3, 353, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Node.traverse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_10selectolax_6parser_4Node_9generator4(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *__pyx_cur_scope = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  myhtml_tree_node_t *__pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("traverse", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L11_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(3, 353, __pyx_L1_error)
  }

  /* "selectolax/modest/node.pxi":365
 *         node
 *         """
 *         cdef Stack stack = Stack(_STACK_SIZE)             # <<<<<<<<<<<<<<
 *         cdef myhtml_tree_node_t* current_node = NULL
 *         cdef Node next_node
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Stack);
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Stack); 
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_int_100};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 365, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __Pyx_GIVEREF((PyObject *)__pyx_t_1);
  __pyx_cur_scope->__pyx_v_stack = ((struct __pyx_obj_10selectolax_6parser_Stack *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":366
 *         """
 *         cdef Stack stack = Stack(_STACK_SIZE)
 *         cdef myhtml_tree_node_t* current_node = NULL             # <<<<<<<<<<<<<<
 *         cdef Node next_node
 * 
*/
  __pyx_cur_scope->__pyx_v_current_node = NULL;

  /* "selectolax/modest/node.pxi":369
 *         cdef Node next_node
 * 
 *         stack.push(self.node)             # <<<<<<<<<<<<<<
 * 
 *         while not stack.is_empty():
*/
  __pyx_t_5 = __pyx_f_10selectolax_6parser_5Stack_push(__pyx_cur_scope->__pyx_v_stack, __pyx_cur_scope->__pyx_v_self->node); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(3, 369, __pyx_L1_error)

  /* "selectolax/modest/node.pxi":371
 *         stack.push(self.node)
 * 
 *         while not stack.is_empty():             # <<<<<<<<<<<<<<
 *             current_node = stack.pop()
 *             if current_node != NULL and not (current_node.tag_id == MyHTML_TAG__TEXT and not include_text):
*/
  while (1) {
    __pyx_t_6 = __pyx_f_10selectolax_6parser_5Stack_is_empty(__pyx_cur_scope->__pyx_v_stack); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 371, __pyx_L1_error)
    __pyx_t_7 = (!__pyx_t_6);
    if (!__pyx_t_7) break;

    /* "selectolax/modest/node.pxi":372
 * 
 *         while not stack.is_empty():
 *             current_node = stack.pop()             # <<<<<<<<<<<<<<
 *             if current_node != NULL and not (current_node.tag_id == MyHTML_TAG__TEXT and not include_text):
 *                 next_node = Node.new(current_node, self.parser)
*/
    __pyx_t_8 = __pyx_f_10selectolax_6parser_5Stack_pop(__pyx_cur_scope->__pyx_v_stack); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 372, __pyx_L1_error)
    __pyx_cur_scope->__pyx_v_current_node = __pyx_t_8;

    /* "selectolax/modest/node.pxi":373
 *         while not stack.is_empty():
 *             current_node = stack.pop()
 *             if current_node != NULL and not (current_node.tag_id == MyHTML_TAG__TEXT and not include_text):             # <<<<<<<<<<<<<<
 *                 next_node = Node.new(current_node, self.parser)
 *                 yield next_node
*/
    __pyx_t_6 = (__pyx_cur_scope->__pyx_v_current_node != NULL);
    if (__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_cur_scope->__pyx_v_current_node->tag_id == MyHTML_TAG__TEXT);
    if (__pyx_t_9) {
    } else {
      __pyx_t_6 = __pyx_t_9;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_include_text); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(3, 373, __pyx_L1_error)
    __pyx_t_10 = (!__pyx_t_9);
    __pyx_t_6 = __pyx_t_10;
    __pyx_L9_bool_binop_done:;
    __pyx_t_10 = (!__pyx_t_6);
    __pyx_t_7 = __pyx_t_10;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_7) {

      /* "selectolax/modest/node.pxi":374
 *             current_node = stack.pop()
 *             if current_node != NULL and not (current_node.tag_id == MyHTML_TAG__TEXT and not include_text):
 *                 next_node = Node.new(current_node, self.parser)             # <<<<<<<<<<<<<<
 *                 yield next_node
 * 
*/
      __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_self->parser);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_cur_scope->__pyx_v_current_node, ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_1))); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 374, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XGOTREF((PyObject *)__pyx_cur_scope->__pyx_v_next_node);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_next_node, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3));
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "selectolax/modest/node.pxi":375
 *             if current_node != NULL and not (current_node.tag_id == MyHTML_TAG__TEXT and not include_text):
 *                 next_node = Node.new(current_node, self.parser)
 *                 yield next_node             # <<<<<<<<<<<<<<
 * 
 *             if current_node.next is not NULL:
*/
      __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_next_node);
      __pyx_r = ((PyObject *)__pyx_cur_scope->__pyx_v_next_node);
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L11_resume_from_yield:;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(3, 375, __pyx_L1_error)

      /* "selectolax/modest/node.pxi":373
 *         while not stack.is_empty():
 *             current_node = stack.pop()
 *             if current_node != NULL and not (current_node.tag_id == MyHTML_TAG__TEXT and not include_text):             # <<<<<<<<<<<<<<
 *                 next_node = Node.new(current_node, self.parser)
 *                 yield next_node
*/
    }

    /* "selectolax/modest/node.pxi":377
 *                 yield next_node
 * 
 *             if current_node.next is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.next)
 * 
*/
    __pyx_t_7 = (__pyx_cur_scope->__pyx_v_current_node->next != NULL);
    if (__pyx_t_7) {

      /* "selectolax/modest/node.pxi":378
 * 
 *             if current_node.next is not NULL:
 *                 stack.push(current_node.next)             # <<<<<<<<<<<<<<
 * 
 *             if current_node.child is not NULL:
*/
      __pyx_t_5 = __pyx_f_10selectolax_6parser_5Stack_push(__pyx_cur_scope->__pyx_v_stack, __pyx_cur_scope->__pyx_v_current_node->next); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(3, 378, __pyx_L1_error)

      /* "selectolax/modest/node.pxi":377
 *                 yield next_node
 * 
 *             if current_node.next is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.next)
 * 
*/
    }

    /* "selectolax/modest/node.pxi":380
 *                 stack.push(current_node.next)
 * 
 *             if current_node.child is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.child)
 * 
*/
    __pyx_t_7 = (__pyx_cur_scope->__pyx_v_current_node->child != NULL);
    if (__pyx_t_7) {

      /* "selectolax/modest/node.pxi":381
 * 
 *             if current_node.child is not NULL:
 *                 stack.push(current_node.child)             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
      __pyx_t_5 = __pyx_f_10selectolax_6parser_5Stack_push(__pyx_cur_scope->__pyx_v_stack, __pyx_cur_scope->__pyx_v_current_node->child); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(3, 381, __pyx_L1_error)

      /* "selectolax/modest/node.pxi":380
 *                 stack.push(current_node.next)
 * 
 *             if current_node.child is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.child)
 * 
*/
    }
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "selectolax/modest/node.pxi":353
 *             node = node.next
 * 
 *     def traverse(self, include_text=False):             # <<<<<<<<<<<<<<
 *         """Iterate over all child and next nodes starting from the current level.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("traverse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":383
 *                 stack.push(current_node.child)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def tag(self):
 *         """Return the name of the current tag (e.g. div, p, img).
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_3tag_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_3tag_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_3tag___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_3tag___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  char const *__pyx_v_c_text;
  PyObject *__pyx_v_text = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":392
 *         """
 *         cdef const char *c_text
 *         c_text = myhtml_tag_name_by_id(self.node.tree, self.node.tag_id, NULL)             # <<<<<<<<<<<<<<
 *         text = None
 *         if c_text:
*/
  __pyx_v_c_text = myhtml_tag_name_by_id(__pyx_v_self->node->tree, __pyx_v_self->node->tag_id, NULL);

  /* "selectolax/modest/node.pxi":393
 *         cdef const char *c_text
 *         c_text = myhtml_tag_name_by_id(self.node.tree, self.node.tag_id, NULL)
 *         text = None             # <<<<<<<<<<<<<<
 *         if c_text:
 *             text = c_text.decode(_ENCODING, self.parser.decode_errors)
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_text = Py_None;

  /* "selectolax/modest/node.pxi":394
 *         c_text = myhtml_tag_name_by_id(self.node.tree, self.node.tag_id, NULL)
 *         text = None
 *         if c_text:             # <<<<<<<<<<<<<<
 *             text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *         return text
*/
  __pyx_t_1 = (__pyx_v_c_text != 0);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":395
 *         text = None
 *         if c_text:
 *             text = c_text.decode(_ENCODING, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 *         return text
 * 
*/
    __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_c_text); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->parser->decode_errors};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 395, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF_SET(__pyx_v_text, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "selectolax/modest/node.pxi":394
 *         c_text = myhtml_tag_name_by_id(self.node.tree, self.node.tag_id, NULL)
 *         text = None
 *         if c_text:             # <<<<<<<<<<<<<<
 *             text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *         return text
*/
  }

  /* "selectolax/modest/node.pxi":396
 *         if c_text:
 *             text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *         return text             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_text);
  __pyx_r = __pyx_v_text;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":383
 *                 stack.push(current_node.child)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def tag(self):
 *         """Return the name of the current tag (e.g. div, p, img).
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("selectolax.parser.Node.tag.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_text);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":398
 *         return text
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def child(self):
 *         """Return the child node."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_5child_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_5child_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_5child___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_5child___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":402
 *         """Return the child node."""
 *         cdef Node node
 *         if self.node.child:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.child, self.parser)
 *             return node
*/
  __pyx_t_1 = (__pyx_v_self->node->child != 0);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":403
 *         cdef Node node
 *         if self.node.child:
 *             node = Node.new(self.node.child, self.parser)             # <<<<<<<<<<<<<<
 *             return node
 *         return None
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_self->parser);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_self->node->child, ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_2))); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 403, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":404
 *         if self.node.child:
 *             node = Node.new(self.node.child, self.parser)
 *             return node             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_node);
    __pyx_r = ((PyObject *)__pyx_v_node);
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":402
 *         """Return the child node."""
 *         cdef Node node
 *         if self.node.child:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.child, self.parser)
 *             return node
*/
  }

  /* "selectolax/modest/node.pxi":405
 *             node = Node.new(self.node.child, self.parser)
 *             return node
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":398
 *         return text
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def child(self):
 *         """Return the child node."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Node.child.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":407
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def parent(self):
 *         """Return the parent node."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_6parent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_6parent_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_6parent___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_6parent___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":411
 *         """Return the parent node."""
 *         cdef Node node
 *         if self.node.parent:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.parent, self.parser)
 *             return node
*/
  __pyx_t_1 = (__pyx_v_self->node->parent != 0);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":412
 *         cdef Node node
 *         if self.node.parent:
 *             node = Node.new(self.node.parent, self.parser)             # <<<<<<<<<<<<<<
 *             return node
 *         return None
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_self->parser);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_self->node->parent, ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_2))); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":413
 *         if self.node.parent:
 *             node = Node.new(self.node.parent, self.parser)
 *             return node             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_node);
    __pyx_r = ((PyObject *)__pyx_v_node);
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":411
 *         """Return the parent node."""
 *         cdef Node node
 *         if self.node.parent:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.parent, self.parser)
 *             return node
*/
  }

  /* "selectolax/modest/node.pxi":414
 *             node = Node.new(self.node.parent, self.parser)
 *             return node
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":407
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def parent(self):
 *         """Return the parent node."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Node.parent.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":416
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def next(self):
 *         """Return next node."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_4next_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_4next_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_4next___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_4next___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":420
 *         """Return next node."""
 *         cdef Node node
 *         if self.node.next:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.next, self.parser)
 *             return node
*/
  __pyx_t_1 = (__pyx_v_self->node->next != 0);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":421
 *         cdef Node node
 *         if self.node.next:
 *             node = Node.new(self.node.next, self.parser)             # <<<<<<<<<<<<<<
 *             return node
 *         return None
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_self->parser);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_self->node->next, ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_2))); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 421, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":422
 *         if self.node.next:
 *             node = Node.new(self.node.next, self.parser)
 *             return node             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_node);
    __pyx_r = ((PyObject *)__pyx_v_node);
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":420
 *         """Return next node."""
 *         cdef Node node
 *         if self.node.next:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.next, self.parser)
 *             return node
*/
  }

  /* "selectolax/modest/node.pxi":423
 *             node = Node.new(self.node.next, self.parser)
 *             return node
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":416
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def next(self):
 *         """Return next node."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Node.next.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":425
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def prev(self):
 *         """Return previous node."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_4prev_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_4prev_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_4prev___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_4prev___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":429
 *         """Return previous node."""
 *         cdef Node node
 *         if self.node.prev:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.prev, self.parser)
 *             return node
*/
  __pyx_t_1 = (__pyx_v_self->node->prev != 0);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":430
 *         cdef Node node
 *         if self.node.prev:
 *             node = Node.new(self.node.prev, self.parser)             # <<<<<<<<<<<<<<
 *             return node
 *         return None
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_self->parser);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_self->node->prev, ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_2))); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":431
 *         if self.node.prev:
 *             node = Node.new(self.node.prev, self.parser)
 *             return node             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_node);
    __pyx_r = ((PyObject *)__pyx_v_node);
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":429
 *         """Return previous node."""
 *         cdef Node node
 *         if self.node.prev:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.prev, self.parser)
 *             return node
*/
  }

  /* "selectolax/modest/node.pxi":432
 *             node = Node.new(self.node.prev, self.parser)
 *             return node
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":425
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def prev(self):
 *         """Return previous node."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Node.prev.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":434
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def last_child(self):
 *         """Return last child node."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_10last_child_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_10last_child_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_10last_child___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_10last_child___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":438
 *         """Return last child node."""
 *         cdef Node node
 *         if self.node.last_child:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.last_child, self.parser)
 *             return node
*/
  __pyx_t_1 = (__pyx_v_self->node->last_child != 0);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":439
 *         cdef Node node
 *         if self.node.last_child:
 *             node = Node.new(self.node.last_child, self.parser)             # <<<<<<<<<<<<<<
 *             return node
 *         return None
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_self->parser);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_self->node->last_child, ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_2))); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 439, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":440
 *         if self.node.last_child:
 *             node = Node.new(self.node.last_child, self.parser)
 *             return node             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_node);
    __pyx_r = ((PyObject *)__pyx_v_node);
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":438
 *         """Return last child node."""
 *         cdef Node node
 *         if self.node.last_child:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.node.last_child, self.parser)
 *             return node
*/
  }

  /* "selectolax/modest/node.pxi":441
 *             node = Node.new(self.node.last_child, self.parser)
 *             return node
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":434
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def last_child(self):
 *         """Return last child node."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Node.last_child.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":443
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def html(self):
 *         """Return HTML representation of the current node including all its child nodes.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_4html_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_4html_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_4html___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_4html___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  mycore_string_raw_t __pyx_v_c_str;
  mystatus_t __pyx_v_status;
  PyObject *__pyx_v_html = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  char *__pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":452
 *         """
 *         cdef mycore_string_raw_t c_str
 *         c_str.data = NULL             # <<<<<<<<<<<<<<
 *         c_str.length = 0
 *         c_str.size = 0
*/
  __pyx_v_c_str.data = NULL;

  /* "selectolax/modest/node.pxi":453
 *         cdef mycore_string_raw_t c_str
 *         c_str.data = NULL
 *         c_str.length = 0             # <<<<<<<<<<<<<<
 *         c_str.size = 0
 * 
*/
  __pyx_v_c_str.length = 0;

  /* "selectolax/modest/node.pxi":454
 *         c_str.data = NULL
 *         c_str.length = 0
 *         c_str.size = 0             # <<<<<<<<<<<<<<
 * 
 *         cdef mystatus_t status
*/
  __pyx_v_c_str.size = 0;

  /* "selectolax/modest/node.pxi":457
 * 
 *         cdef mystatus_t status
 *         status = myhtml_serialization(self.node, &c_str)             # <<<<<<<<<<<<<<
 * 
 *         if status == 0 and c_str.data:
*/
  __pyx_v_status = myhtml_serialization(__pyx_v_self->node, (&__pyx_v_c_str));

  /* "selectolax/modest/node.pxi":459
 *         status = myhtml_serialization(self.node, &c_str)
 * 
 *         if status == 0 and c_str.data:             # <<<<<<<<<<<<<<
 *             html = c_str.data.decode(_ENCODING).replace('<-undef>', '')
 *             free(c_str.data)
*/
  __pyx_t_2 = (__pyx_v_status == 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_c_str.data != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":460
 * 
 *         if status == 0 and c_str.data:
 *             html = c_str.data.decode(_ENCODING).replace('<-undef>', '')             # <<<<<<<<<<<<<<
 *             free(c_str.data)
 *             return html
*/
    __pyx_t_3 = __pyx_v_c_str.data;
    __pyx_t_4 = __Pyx_ssize_strlen(__pyx_t_3); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(3, 460, __pyx_L1_error)
    __pyx_t_5 = __Pyx_decode_c_string(__pyx_t_3, 0, __pyx_t_4, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 460, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyUnicode_Replace(__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_undef, __pyx_mstate_global->__pyx_kp_u_, -1L); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 460, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_html = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "selectolax/modest/node.pxi":461
 *         if status == 0 and c_str.data:
 *             html = c_str.data.decode(_ENCODING).replace('<-undef>', '')
 *             free(c_str.data)             # <<<<<<<<<<<<<<
 *             return html
 * 
*/
    free(__pyx_v_c_str.data);

    /* "selectolax/modest/node.pxi":462
 *             html = c_str.data.decode(_ENCODING).replace('<-undef>', '')
 *             free(c_str.data)
 *             return html             # <<<<<<<<<<<<<<
 * 
 *         return None
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_html);
    __pyx_r = __pyx_v_html;
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":459
 *         status = myhtml_serialization(self.node, &c_str)
 * 
 *         if status == 0 and c_str.data:             # <<<<<<<<<<<<<<
 *             html = c_str.data.decode(_ENCODING).replace('<-undef>', '')
 *             free(c_str.data)
*/
  }

  /* "selectolax/modest/node.pxi":464
 *             return html
 * 
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     def css(self, str query):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":443
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def html(self):
 *         """Return HTML representation of the current node including all its child nodes.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser.Node.html.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_html);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":466
 *         return None
 * 
 *     def css(self, str query):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         return find_nodes(self.parser, self.node, query)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_11css(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_10css, "Node.css(self, str query)\n\nEvaluate CSS selector against current node and its child nodes.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_11css = {"css", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_11css, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_10css};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_11css(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_query = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("css (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_query,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 466, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 466, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "css", 0) < 0) __PYX_ERR(3, 466, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("css", 1, 1, 1, i); __PYX_ERR(3, 466, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 466, __pyx_L3_error)
    }
    __pyx_v_query = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("css", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 466, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.css", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query), (&PyUnicode_Type), 1, "query", 1))) __PYX_ERR(3, 466, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_10css(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_query);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_10css(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_query) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("css", 0);

  /* "selectolax/modest/node.pxi":468
 *     def css(self, str query):
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         return find_nodes(self.parser, self.node, query)             # <<<<<<<<<<<<<<
 * 
 *     def any_css_matches(self, tuple selectors):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_v_self->parser);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_10selectolax_6parser_find_nodes(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_1), __pyx_v_self->node, __pyx_v_query); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":466
 *         return None
 * 
 *     def css(self, str query):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         return find_nodes(self.parser, self.node, query)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.Node.css", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":470
 *         return find_nodes(self.parser, self.node, query)
 * 
 *     def any_css_matches(self, tuple selectors):             # <<<<<<<<<<<<<<
 *         """Returns True if any of CSS selectors matches a node"""
 *         return find_matches(self.parser, self.node, selectors)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_13any_css_matches(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_12any_css_matches, "Node.any_css_matches(self, tuple selectors)\n\nReturns True if any of CSS selectors matches a node");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_13any_css_matches = {"any_css_matches", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_13any_css_matches, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_12any_css_matches};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_13any_css_matches(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_selectors = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("any_css_matches (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_selectors,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 470, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 470, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "any_css_matches", 0) < 0) __PYX_ERR(3, 470, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("any_css_matches", 1, 1, 1, i); __PYX_ERR(3, 470, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 470, __pyx_L3_error)
    }
    __pyx_v_selectors = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("any_css_matches", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 470, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.any_css_matches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_selectors), (&PyTuple_Type), 1, "selectors", 1))) __PYX_ERR(3, 470, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_12any_css_matches(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_selectors);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_12any_css_matches(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_selectors) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("any_css_matches", 0);

  /* "selectolax/modest/node.pxi":472
 *     def any_css_matches(self, tuple selectors):
 *         """Returns True if any of CSS selectors matches a node"""
 *         return find_matches(self.parser, self.node, selectors)             # <<<<<<<<<<<<<<
 * 
 *     def css_matches(self, str selector):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_v_self->parser);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_f_10selectolax_6parser_find_matches(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_1), __pyx_v_self->node, __pyx_v_selectors)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":470
 *         return find_nodes(self.parser, self.node, query)
 * 
 *     def any_css_matches(self, tuple selectors):             # <<<<<<<<<<<<<<
 *         """Returns True if any of CSS selectors matches a node"""
 *         return find_matches(self.parser, self.node, selectors)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.Node.any_css_matches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":474
 *         return find_matches(self.parser, self.node, selectors)
 * 
 *     def css_matches(self, str selector):             # <<<<<<<<<<<<<<
 *         """Returns True if CSS selector matches a node."""
 *         return find_matches(self.parser, self.node, (selector, ))
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_15css_matches(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_14css_matches, "Node.css_matches(self, str selector)\n\nReturns True if CSS selector matches a node.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_15css_matches = {"css_matches", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_15css_matches, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_14css_matches};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_15css_matches(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_selector = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("css_matches (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_selector,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 474, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 474, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "css_matches", 0) < 0) __PYX_ERR(3, 474, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("css_matches", 1, 1, 1, i); __PYX_ERR(3, 474, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 474, __pyx_L3_error)
    }
    __pyx_v_selector = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("css_matches", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 474, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.css_matches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_selector), (&PyUnicode_Type), 1, "selector", 1))) __PYX_ERR(3, 474, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_14css_matches(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_selector);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_14css_matches(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_selector) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("css_matches", 0);

  /* "selectolax/modest/node.pxi":476
 *     def css_matches(self, str selector):
 *         """Returns True if CSS selector matches a node."""
 *         return find_matches(self.parser, self.node, (selector, ))             # <<<<<<<<<<<<<<
 * 
 *     def css_first(self, str query, default=None, bool strict=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_v_self->parser);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_selector);
  __Pyx_GIVEREF(__pyx_v_selector);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_selector) != (0)) __PYX_ERR(3, 476, __pyx_L1_error);
  __pyx_t_3 = ((PyObject *)__pyx_f_10selectolax_6parser_find_matches(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_1), __pyx_v_self->node, ((PyObject*)__pyx_t_2))); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":474
 *         return find_matches(self.parser, self.node, selectors)
 * 
 *     def css_matches(self, str selector):             # <<<<<<<<<<<<<<
 *         """Returns True if CSS selector matches a node."""
 *         return find_matches(self.parser, self.node, (selector, ))
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Node.css_matches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":478
 *         return find_matches(self.parser, self.node, (selector, ))
 * 
 *     def css_first(self, str query, default=None, bool strict=False):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         results = self.css(query)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_17css_first(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_16css_first, "Node.css_first(self, str query, default=None, bool strict=False)\n\nEvaluate CSS selector against current node and its child nodes.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_17css_first = {"css_first", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_17css_first, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_16css_first};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_17css_first(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_query = 0;
  PyObject *__pyx_v_default = 0;
  PyLongObject *__pyx_v_strict = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("css_first (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_query,&__pyx_mstate_global->__pyx_n_u_default,&__pyx_mstate_global->__pyx_n_u_strict,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 478, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 478, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 478, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 478, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "css_first", 0) < 0) __PYX_ERR(3, 478, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("css_first", 0, 1, 3, i); __PYX_ERR(3, 478, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 478, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 478, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 478, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_query = ((PyObject*)values[0]);
    __pyx_v_default = values[1];
    __pyx_v_strict = ((PyLongObject *)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("css_first", 0, 1, 3, __pyx_nargs); __PYX_ERR(3, 478, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.css_first", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query), (&PyUnicode_Type), 1, "query", 1))) __PYX_ERR(3, 478, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_strict), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "strict", 0))) __PYX_ERR(3, 478, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_16css_first(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_query, __pyx_v_default, __pyx_v_strict);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_16css_first(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_query, PyObject *__pyx_v_default, PyLongObject *__pyx_v_strict) {
  PyObject *__pyx_v_results = NULL;
  PyObject *__pyx_v_n_results = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("css_first", 0);

  /* "selectolax/modest/node.pxi":480
 *     def css_first(self, str query, default=None, bool strict=False):
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         results = self.css(query)             # <<<<<<<<<<<<<<
 *         n_results = len(results)
 * 
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_query};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_css, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 480, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_results = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":481
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         results = self.css(query)
 *         n_results = len(results)             # <<<<<<<<<<<<<<
 * 
 *         if n_results > 0:
*/
  __pyx_t_4 = PyObject_Length(__pyx_v_results); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(3, 481, __pyx_L1_error)
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_n_results = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":483
 *         n_results = len(results)
 * 
 *         if n_results > 0:             # <<<<<<<<<<<<<<
 * 
 *             if strict and n_results > 1:
*/
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_n_results, __pyx_mstate_global->__pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 483, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(3, 483, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "selectolax/modest/node.pxi":485
 *         if n_results > 0:
 * 
 *             if strict and n_results > 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Expected 1 match, but found %s matches" % n_results)
 * 
*/
    __pyx_t_6 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_strict)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 485, __pyx_L1_error)
    if (__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_n_results, __pyx_mstate_global->__pyx_int_1, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 485, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 485, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __pyx_t_6;
    __pyx_L5_bool_binop_done:;
    if (unlikely(__pyx_t_5)) {

      /* "selectolax/modest/node.pxi":486
 * 
 *             if strict and n_results > 1:
 *                 raise ValueError("Expected 1 match, but found %s matches" % n_results)             # <<<<<<<<<<<<<<
 * 
 *             return results[0]
*/
      __pyx_t_2 = NULL;
      __Pyx_INCREF(__pyx_builtin_ValueError);
      __pyx_t_7 = __pyx_builtin_ValueError; 
      __pyx_t_8 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_1_match_but_found_s_mat, __pyx_v_n_results); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 486, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 486, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(3, 486, __pyx_L1_error)

      /* "selectolax/modest/node.pxi":485
 *         if n_results > 0:
 * 
 *             if strict and n_results > 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Expected 1 match, but found %s matches" % n_results)
 * 
*/
    }

    /* "selectolax/modest/node.pxi":488
 *                 raise ValueError("Expected 1 match, but found %s matches" % n_results)
 * 
 *             return results[0]             # <<<<<<<<<<<<<<
 * 
 *         return default
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_results, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":483
 *         n_results = len(results)
 * 
 *         if n_results > 0:             # <<<<<<<<<<<<<<
 * 
 *             if strict and n_results > 1:
*/
  }

  /* "selectolax/modest/node.pxi":490
 *             return results[0]
 * 
 *         return default             # <<<<<<<<<<<<<<
 * 
 *     def decompose(self, bool recursive=True):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_default);
  __pyx_r = __pyx_v_default;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":478
 *         return find_matches(self.parser, self.node, (selector, ))
 * 
 *     def css_first(self, str query, default=None, bool strict=False):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         results = self.css(query)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("selectolax.parser.Node.css_first", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XDECREF(__pyx_v_n_results);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":492
 *         return default
 * 
 *     def decompose(self, bool recursive=True):             # <<<<<<<<<<<<<<
 *         """Remove a Node from the tree.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_19decompose(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_18decompose, "Node.decompose(self, bool recursive=True)\n\nRemove a Node from the tree.\n\nParameters\n----------\nrecursive : bool, default True\n    Whenever to delete all its child nodes\n\nExamples\n--------\n\n>>> tree = HTMLParser(html)\n>>> for tag in tree.css('script'):\n>>>     tag.decompose()");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_19decompose = {"decompose", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_19decompose, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_18decompose};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_19decompose(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyLongObject *__pyx_v_recursive = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("decompose (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_recursive,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 492, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 492, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "decompose", 0) < 0) __PYX_ERR(3, 492, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 492, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_recursive = ((PyLongObject *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("decompose", 0, 0, 1, __pyx_nargs); __PYX_ERR(3, 492, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.decompose", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recursive), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "recursive", 0))) __PYX_ERR(3, 492, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_18decompose(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_recursive);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_18decompose(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyLongObject *__pyx_v_recursive) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("decompose", 0);

  /* "selectolax/modest/node.pxi":508
 * 
 *         """
 *         if recursive:             # <<<<<<<<<<<<<<
 *             myhtml_node_delete_recursive(self.node)
 *         else:
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_recursive)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(3, 508, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":509
 *         """
 *         if recursive:
 *             myhtml_node_delete_recursive(self.node)             # <<<<<<<<<<<<<<
 *         else:
 *             myhtml_node_delete(self.node)
*/
    myhtml_node_delete_recursive(__pyx_v_self->node);

    /* "selectolax/modest/node.pxi":508
 * 
 *         """
 *         if recursive:             # <<<<<<<<<<<<<<
 *             myhtml_node_delete_recursive(self.node)
 *         else:
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":511
 *             myhtml_node_delete_recursive(self.node)
 *         else:
 *             myhtml_node_delete(self.node)             # <<<<<<<<<<<<<<
 * 
 *     def remove(self, bool recursive=True):
*/
  /*else*/ {
    myhtml_node_delete(__pyx_v_self->node);
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":492
 *         return default
 * 
 *     def decompose(self, bool recursive=True):             # <<<<<<<<<<<<<<
 *         """Remove a Node from the tree.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Node.decompose", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":513
 *             myhtml_node_delete(self.node)
 * 
 *     def remove(self, bool recursive=True):             # <<<<<<<<<<<<<<
 *         """An alias for the decompose method."""
 *         self.decompose(recursive)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_21remove(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_20remove, "Node.remove(self, bool recursive=True)\n\nAn alias for the decompose method.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_21remove = {"remove", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_21remove, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_20remove};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_21remove(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyLongObject *__pyx_v_recursive = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("remove (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_recursive,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 513, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 513, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "remove", 0) < 0) __PYX_ERR(3, 513, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 513, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_recursive = ((PyLongObject *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("remove", 0, 0, 1, __pyx_nargs); __PYX_ERR(3, 513, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.remove", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recursive), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "recursive", 0))) __PYX_ERR(3, 513, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_20remove(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_recursive);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_20remove(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyLongObject *__pyx_v_recursive) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("remove", 0);

  /* "selectolax/modest/node.pxi":515
 *     def remove(self, bool recursive=True):
 *         """An alias for the decompose method."""
 *         self.decompose(recursive)             # <<<<<<<<<<<<<<
 * 
 *     def unwrap(self, delete_empty = False):
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, ((PyObject *)__pyx_v_recursive)};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 515, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":513
 *             myhtml_node_delete(self.node)
 * 
 *     def remove(self, bool recursive=True):             # <<<<<<<<<<<<<<
 *         """An alias for the decompose method."""
 *         self.decompose(recursive)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.Node.remove", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":517
 *         self.decompose(recursive)
 * 
 *     def unwrap(self, delete_empty = False):             # <<<<<<<<<<<<<<
 *         """Replace node with whatever is inside this node.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_23unwrap(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_22unwrap, "Node.unwrap(self, delete_empty=False)\n\nReplace node with whatever is inside this node.\n\nParameters\n----------\ndelete_empty : bool, default False\n    Whenever to delete empty tags.\n\nExamples\n--------\n\n>>>  tree = HTMLParser(\"<div>Hello <i>world</i>!</div>\")\n>>>  tree.css_first('i').unwrap()\n>>>  tree.html\n'<html><head></head><body><div>Hello world!</div></body></html>'\n\nNote: by default, empty tags are ignored, set \"delete_empty\" to \"True\" to change this.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_23unwrap = {"unwrap", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_23unwrap, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_22unwrap};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_23unwrap(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_delete_empty = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unwrap (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_delete_empty,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 517, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 517, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "unwrap", 0) < 0) __PYX_ERR(3, 517, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 517, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
    }
    __pyx_v_delete_empty = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("unwrap", 0, 0, 1, __pyx_nargs); __PYX_ERR(3, 517, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.unwrap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_22unwrap(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_delete_empty);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_22unwrap(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_delete_empty) {
  myhtml_tree_node_t *__pyx_v_next_node;
  myhtml_tree_node_t *__pyx_v_current_node;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  myhtml_tree_node_t *__pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("unwrap", 0);

  /* "selectolax/modest/node.pxi":535
 *         Note: by default, empty tags are ignored, set "delete_empty" to "True" to change this.
 *         """
 *         if self.node.child == NULL:             # <<<<<<<<<<<<<<
 *             if delete_empty:
 *                 myhtml_node_delete(self.node)
*/
  __pyx_t_1 = (__pyx_v_self->node->child == NULL);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":536
 *         """
 *         if self.node.child == NULL:
 *             if delete_empty:             # <<<<<<<<<<<<<<
 *                 myhtml_node_delete(self.node)
 *             return
*/
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_delete_empty); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(3, 536, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "selectolax/modest/node.pxi":537
 *         if self.node.child == NULL:
 *             if delete_empty:
 *                 myhtml_node_delete(self.node)             # <<<<<<<<<<<<<<
 *             return
 *         cdef myhtml_tree_node_t* next_node
*/
      myhtml_node_delete(__pyx_v_self->node);

      /* "selectolax/modest/node.pxi":536
 *         """
 *         if self.node.child == NULL:
 *             if delete_empty:             # <<<<<<<<<<<<<<
 *                 myhtml_node_delete(self.node)
 *             return
*/
    }

    /* "selectolax/modest/node.pxi":538
 *             if delete_empty:
 *                 myhtml_node_delete(self.node)
 *             return             # <<<<<<<<<<<<<<
 *         cdef myhtml_tree_node_t* next_node
 *         cdef myhtml_tree_node_t* current_node
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":535
 *         Note: by default, empty tags are ignored, set "delete_empty" to "True" to change this.
 *         """
 *         if self.node.child == NULL:             # <<<<<<<<<<<<<<
 *             if delete_empty:
 *                 myhtml_node_delete(self.node)
*/
  }

  /* "selectolax/modest/node.pxi":542
 *         cdef myhtml_tree_node_t* current_node
 * 
 *         if self.node.child.next != NULL:             # <<<<<<<<<<<<<<
 *             current_node = self.node.child
 *             next_node = current_node.next
*/
  __pyx_t_1 = (__pyx_v_self->node->child->next != NULL);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":543
 * 
 *         if self.node.child.next != NULL:
 *             current_node = self.node.child             # <<<<<<<<<<<<<<
 *             next_node = current_node.next
 * 
*/
    __pyx_t_2 = __pyx_v_self->node->child;
    __pyx_v_current_node = __pyx_t_2;

    /* "selectolax/modest/node.pxi":544
 *         if self.node.child.next != NULL:
 *             current_node = self.node.child
 *             next_node = current_node.next             # <<<<<<<<<<<<<<
 * 
 *             while next_node != NULL:
*/
    __pyx_t_2 = __pyx_v_current_node->next;
    __pyx_v_next_node = __pyx_t_2;

    /* "selectolax/modest/node.pxi":546
 *             next_node = current_node.next
 * 
 *             while next_node != NULL:             # <<<<<<<<<<<<<<
 *                 next_node = current_node.next
 *                 myhtml_node_insert_before(self.node, current_node)
*/
    while (1) {
      __pyx_t_1 = (__pyx_v_next_node != NULL);
      if (!__pyx_t_1) break;

      /* "selectolax/modest/node.pxi":547
 * 
 *             while next_node != NULL:
 *                 next_node = current_node.next             # <<<<<<<<<<<<<<
 *                 myhtml_node_insert_before(self.node, current_node)
 *                 current_node = next_node
*/
      __pyx_t_2 = __pyx_v_current_node->next;
      __pyx_v_next_node = __pyx_t_2;

      /* "selectolax/modest/node.pxi":548
 *             while next_node != NULL:
 *                 next_node = current_node.next
 *                 myhtml_node_insert_before(self.node, current_node)             # <<<<<<<<<<<<<<
 *                 current_node = next_node
 *         else:
*/
      (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_current_node));

      /* "selectolax/modest/node.pxi":549
 *                 next_node = current_node.next
 *                 myhtml_node_insert_before(self.node, current_node)
 *                 current_node = next_node             # <<<<<<<<<<<<<<
 *         else:
 *             myhtml_node_insert_before(self.node, self.node.child)
*/
      __pyx_v_current_node = __pyx_v_next_node;
    }

    /* "selectolax/modest/node.pxi":542
 *         cdef myhtml_tree_node_t* current_node
 * 
 *         if self.node.child.next != NULL:             # <<<<<<<<<<<<<<
 *             current_node = self.node.child
 *             next_node = current_node.next
*/
    goto __pyx_L5;
  }

  /* "selectolax/modest/node.pxi":551
 *                 current_node = next_node
 *         else:
 *             myhtml_node_insert_before(self.node, self.node.child)             # <<<<<<<<<<<<<<
 *         myhtml_node_delete(self.node)
 * 
*/
  /*else*/ {
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_self->node->child));
  }
  __pyx_L5:;

  /* "selectolax/modest/node.pxi":552
 *         else:
 *             myhtml_node_insert_before(self.node, self.node.child)
 *         myhtml_node_delete(self.node)             # <<<<<<<<<<<<<<
 * 
 *     def strip_tags(self, list tags, bool recursive = False):
*/
  myhtml_node_delete(__pyx_v_self->node);

  /* "selectolax/modest/node.pxi":517
 *         self.decompose(recursive)
 * 
 *     def unwrap(self, delete_empty = False):             # <<<<<<<<<<<<<<
 *         """Replace node with whatever is inside this node.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Node.unwrap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":554
 *         myhtml_node_delete(self.node)
 * 
 *     def strip_tags(self, list tags, bool recursive = False):             # <<<<<<<<<<<<<<
 *         """Remove specified tags from the HTML tree.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_25strip_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_24strip_tags, "Node.strip_tags(self, list tags, bool recursive=False)\n\nRemove specified tags from the HTML tree.\n\nParameters\n----------\ntags : list\n    List of tags to remove.\nrecursive : bool, default True\n    Whenever to delete all its child nodes\n\nExamples\n--------\n\n>>> tree = HTMLParser('<html><head></head><body><script></script><div>Hello world!</div></body></html>')\n>>> tags = ['head', 'style', 'script', 'xmp', 'iframe', 'noembed', 'noframes']\n>>> tree.strip_tags(tags)\n>>> tree.html\n'<html><body><div>Hello world!</div></body></html>'");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_25strip_tags = {"strip_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_25strip_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_24strip_tags};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_25strip_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_tags = 0;
  PyLongObject *__pyx_v_recursive = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("strip_tags (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_tags,&__pyx_mstate_global->__pyx_n_u_recursive,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 554, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 554, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 554, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "strip_tags", 0) < 0) __PYX_ERR(3, 554, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("strip_tags", 0, 1, 2, i); __PYX_ERR(3, 554, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 554, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 554, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_tags = ((PyObject*)values[0]);
    __pyx_v_recursive = ((PyLongObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("strip_tags", 0, 1, 2, __pyx_nargs); __PYX_ERR(3, 554, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.strip_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tags), (&PyList_Type), 1, "tags", 1))) __PYX_ERR(3, 554, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recursive), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "recursive", 0))) __PYX_ERR(3, 554, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_24strip_tags(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_tags, __pyx_v_recursive);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_24strip_tags(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_tags, PyLongObject *__pyx_v_recursive) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_element = 0;
  PyObject *__pyx_v_tag = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("strip_tags", 0);

  /* "selectolax/modest/node.pxi":576
 *         # ensure cython can recast element to a Node so that decompose will be called sooner.
 *         cdef Node element
 *         for tag in tags:             # <<<<<<<<<<<<<<
 *             for element in self.css(tag):
 *                 element.decompose(recursive=recursive)
*/
  if (unlikely(__pyx_v_tags == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(3, 576, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_tags; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 576, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_2);
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_tag, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":577
 *         cdef Node element
 *         for tag in tags:
 *             for element in self.css(tag):             # <<<<<<<<<<<<<<
 *                 element.decompose(recursive=recursive)
 * 
*/
    __pyx_t_4 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_tag};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_css, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 577, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4);
      __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 577, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 577, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 577, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_4, __pyx_t_6);
          ++__pyx_t_6;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 577, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_6));
          #else
          __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_6);
          #endif
          ++__pyx_t_6;
        }
        if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 577, __pyx_L1_error)
      } else {
        __pyx_t_3 = __pyx_t_7(__pyx_t_4);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(3, 577, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(3, 577, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_element, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "selectolax/modest/node.pxi":578
 *         for tag in tags:
 *             for element in self.css(tag):
 *                 element.decompose(recursive=recursive)             # <<<<<<<<<<<<<<
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):
*/
      __pyx_t_8 = ((PyObject *)__pyx_v_element);
      __Pyx_INCREF(__pyx_t_8);
      __pyx_t_5 = 0;
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_8, NULL};
        __pyx_t_9 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 578, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, ((PyObject *)__pyx_v_recursive), __pyx_t_9, __pyx_callargs+1, 0) < 0) __PYX_ERR(3, 578, __pyx_L1_error)
        __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_9);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 578, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "selectolax/modest/node.pxi":577
 *         cdef Node element
 *         for tag in tags:
 *             for element in self.css(tag):             # <<<<<<<<<<<<<<
 *                 element.decompose(recursive=recursive)
 * 
*/
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/modest/node.pxi":576
 *         # ensure cython can recast element to a Node so that decompose will be called sooner.
 *         cdef Node element
 *         for tag in tags:             # <<<<<<<<<<<<<<
 *             for element in self.css(tag):
 *                 element.decompose(recursive=recursive)
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":554
 *         myhtml_node_delete(self.node)
 * 
 *     def strip_tags(self, list tags, bool recursive = False):             # <<<<<<<<<<<<<<
 *         """Remove specified tags from the HTML tree.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.strip_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_element);
  __Pyx_XDECREF(__pyx_v_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":580
 *                 element.decompose(recursive=recursive)
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):             # <<<<<<<<<<<<<<
 *         """Unwraps specified tags from the HTML tree.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_27unwrap_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_26unwrap_tags, "Node.unwrap_tags(self, list tags, delete_empty=False)\n\nUnwraps specified tags from the HTML tree.\n\nWorks the same as the ``unwrap`` method, but applied to a list of tags.\n\nParameters\n----------\ntags : list\n    List of tags to remove.\ndelete_empty : bool, default False\n    Whenever to delete empty tags.\n\nExamples\n--------\n\n>>> tree = HTMLParser(\"<div><a href=\"\">Hello</a> <i>world</i>!</div>\")\n>>> tree.body.unwrap_tags(['i','a'])\n>>> tree.body.html\n'<body><div>Hello world!</div></body>'\n\nNote: by default, empty tags are ignored, set \"delete_empty\" to \"True\" to change this.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_27unwrap_tags = {"unwrap_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_27unwrap_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_26unwrap_tags};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_27unwrap_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_tags = 0;
  PyObject *__pyx_v_delete_empty = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unwrap_tags (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_tags,&__pyx_mstate_global->__pyx_n_u_delete_empty,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 580, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 580, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 580, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "unwrap_tags", 0) < 0) __PYX_ERR(3, 580, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("unwrap_tags", 0, 1, 2, i); __PYX_ERR(3, 580, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 580, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 580, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_False));
    }
    __pyx_v_tags = ((PyObject*)values[0]);
    __pyx_v_delete_empty = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("unwrap_tags", 0, 1, 2, __pyx_nargs); __PYX_ERR(3, 580, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.unwrap_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tags), (&PyList_Type), 1, "tags", 1))) __PYX_ERR(3, 580, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_26unwrap_tags(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_tags, __pyx_v_delete_empty);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_26unwrap_tags(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_tags, PyObject *__pyx_v_delete_empty) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_element = 0;
  PyObject *__pyx_v_tag = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("unwrap_tags", 0);

  /* "selectolax/modest/node.pxi":603
 *         """
 *         cdef Node element
 *         for tag in tags:             # <<<<<<<<<<<<<<
 *             for element in self.css(tag):
 *                 element.unwrap(delete_empty)
*/
  if (unlikely(__pyx_v_tags == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(3, 603, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_tags; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 603, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_2);
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 603, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_tag, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":604
 *         cdef Node element
 *         for tag in tags:
 *             for element in self.css(tag):             # <<<<<<<<<<<<<<
 *                 element.unwrap(delete_empty)
 * 
*/
    __pyx_t_4 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_tag};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_css, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 604, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4);
      __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 604, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 604, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 604, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_4, __pyx_t_6);
          ++__pyx_t_6;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 604, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_6));
          #else
          __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_6);
          #endif
          ++__pyx_t_6;
        }
        if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 604, __pyx_L1_error)
      } else {
        __pyx_t_3 = __pyx_t_7(__pyx_t_4);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(3, 604, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(3, 604, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_element, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "selectolax/modest/node.pxi":605
 *         for tag in tags:
 *             for element in self.css(tag):
 *                 element.unwrap(delete_empty)             # <<<<<<<<<<<<<<
 * 
 *     def replace_with(self, str_or_Node value):
*/
      __pyx_t_8 = ((PyObject *)__pyx_v_element);
      __Pyx_INCREF(__pyx_t_8);
      __pyx_t_5 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_v_delete_empty};
        __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_unwrap, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 605, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "selectolax/modest/node.pxi":604
 *         cdef Node element
 *         for tag in tags:
 *             for element in self.css(tag):             # <<<<<<<<<<<<<<
 *                 element.unwrap(delete_empty)
 * 
*/
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/modest/node.pxi":603
 *         """
 *         cdef Node element
 *         for tag in tags:             # <<<<<<<<<<<<<<
 *             for element in self.css(tag):
 *                 element.unwrap(delete_empty)
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":580
 *                 element.decompose(recursive=recursive)
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):             # <<<<<<<<<<<<<<
 *         """Unwraps specified tags from the HTML tree.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("selectolax.parser.Node.unwrap_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_element);
  __Pyx_XDECREF(__pyx_v_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":607
 *                 element.unwrap(delete_empty)
 * 
 *     def replace_with(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """Replace current Node with specified value.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_29replace_with(PyObject *__pyx_v_signatures, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_28replace_with, "Node.replace_with(signatures, args, kwargs, defaults, _fused_sigindex_ref=[None])\n\nReplace current Node with specified value.\n\nParameters\n----------\nvalue : str, bytes or Node\n    The text or Node instance to replace the Node with.\n    When a text string is passed, it's treated as text. All HTML tags will be escaped.\n    Convert and pass the ``Node`` object when you want to work with HTML.\n    Does not clone the ``Node`` object.\n    All future changes to the passed ``Node`` object will also be taken into account.\n\nExamples\n--------\n\n>>> tree = HTMLParser('<div>Get <img src=\"\" alt=\"Laptop\"></div>')\n>>> img = tree.css_first('img')\n>>> img.replace_with(img.attributes.get('alt', ''))\n>>> tree.body.child.html\n'<div>Get Laptop</div>'\n\n>>> html_parser = HTMLParser('<div>Get <span alt=\"Laptop\"><img src=\"/jpg\"> <div></div></span></div>')\n>>> html_parser2 = HTMLParser('<div>Test</div>')\n>>> img_node = html_parser.css_first('img')\n>>> img_node.replace_with(html_parser2.body.child)\n'<div>Get <span alt=\"Laptop\"><div>Test</div> <div></div></span></div>'");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_29replace_with = {"replace_with", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_10selectolax_6parser_4Node_29replace_with, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_28replace_with};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_29replace_with(PyObject *__pyx_v_signatures, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  PyObject *__pyx_v__fused_sigindex_ref = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_args,&__pyx_mstate_global->__pyx_n_u_kwargs,&__pyx_mstate_global->__pyx_n_u_defaults,&__pyx_mstate_global->__pyx_n_u_fused_sigindex_ref,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 607, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(3, 607, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 607, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 607, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 607, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_fused_cpdef", 0) < 0) __PYX_ERR(3, 607, __pyx_L3_error)
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__3);
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 0, 3, 4, i); __PYX_ERR(3, 607, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(3, 607, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 607, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 607, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 607, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__3);
    }
    __pyx_v_args = values[0];
    __pyx_v_kwargs = values[1];
    __pyx_v_defaults = values[2];
    __pyx_v__fused_sigindex_ref = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 0, 3, 4, __pyx_nargs); __PYX_ERR(3, 607, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_28replace_with(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_signatures), __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults, __pyx_v__fused_sigindex_ref);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_28replace_with(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults, PyObject *__pyx_v__fused_sigindex_ref) {
  PyObject *__pyx_v_search_list = 0;
  PyObject *__pyx_v_sigindex_node = 0;
  PyObject *__pyx_v_dest_sig = NULL;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_fused_sigindex = NULL;
  PyObject *__pyx_v_sig = NULL;
  PyObject *__pyx_v_sig_series = NULL;
  PyObject *__pyx_v_last_type = NULL;
  PyObject *__pyx_v_sig_type = NULL;
  PyObject *__pyx_v_sigindex_matches = NULL;
  PyObject *__pyx_v_sigindex_candidates = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_v_found_matches = NULL;
  PyObject *__pyx_v_found_candidates = NULL;
  PyObject *__pyx_v_sn = NULL;
  PyObject *__pyx_v_type_match = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[4];
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  int __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("replace_with", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, Py_None) != (0)) __PYX_ERR(3, 607, __pyx_L1_error);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
  __pyx_t_4 = (!__pyx_t_3);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {
    __Pyx_INCREF(Py_None);
    __Pyx_DECREF_SET(__pyx_v_kwargs, Py_None);
  }
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(3, 607, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 607, __pyx_L1_error)
  __pyx_t_2 = (1 < __pyx_t_5);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 607, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  __pyx_t_4 = (__pyx_v_kwargs != Py_None);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L7_bool_binop_done;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(3, 607, __pyx_L1_error)
  }
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_mstate_global->__pyx_n_u_value, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_4;
  __pyx_L7_bool_binop_done:;
  if (likely(__pyx_t_2)) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 607, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyUnicode_From_long(2, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 607, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 607, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_t_5, 0, ' ', 'd'); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Expected_at_least;
    __pyx_t_10[1] = __pyx_t_8;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_arguments_got;
    __pyx_t_10[3] = __pyx_t_9;
    __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 4, 18 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 16 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9), 127);
    if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 607, __pyx_L1_error)
  }
  __pyx_L6:;
  while (1) {
    __pyx_t_2 = PyUnicode_Check(__pyx_v_arg); 
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_str, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    __pyx_t_2 = PyBytes_Check(__pyx_v_arg); 
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_bytes, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    __pyx_t_7 = NULL;
    __Pyx_INCREF(__pyx_builtin___import__);
    __pyx_t_11 = __pyx_builtin___import__; 
    __pyx_t_6 = PyList_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u_);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 0, __pyx_mstate_global->__pyx_kp_u_) != (0)) __PYX_ERR(3, 607, __pyx_L1_error);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[5] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_selectolax_parser, Py_None, Py_None, __pyx_t_6};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_12, (5-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_Node); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = PyObject_IsInstance(__pyx_v_arg, __pyx_t_11); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_Node, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
    goto __pyx_L10_break;
  }
  __pyx_L10_break:;
  __pyx_t_11 = __Pyx_GetItemInt(__pyx_v__fused_sigindex_ref, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __pyx_t_11;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_fused_sigindex = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_fused_sigindex == ((PyObject*)Py_None));
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_fused_sigindex, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;
    __pyx_t_5 = 0;
    if (unlikely(((PyObject *)__pyx_v_signatures) == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(3, 607, __pyx_L1_error)
    }
    __pyx_t_11 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_13), (&__pyx_t_14)); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF(__pyx_t_1);
    __pyx_t_1 = __pyx_t_11;
    __pyx_t_11 = 0;
    while (1) {
      __pyx_t_15 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_13, &__pyx_t_5, &__pyx_t_11, NULL, NULL, __pyx_t_14);
      if (unlikely(__pyx_t_15 == 0)) break;
      if (unlikely(__pyx_t_15 == -1)) __PYX_ERR(3, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_INCREF(__pyx_v_fused_sigindex);
      __Pyx_XDECREF_SET(__pyx_v_sigindex_node, __pyx_v_fused_sigindex);
      __pyx_t_9 = __pyx_v_sig;
      __Pyx_INCREF(__pyx_t_9);
      __pyx_t_12 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u__4};
        __pyx_t_7 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_strip, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __pyx_t_6 = __pyx_t_7;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_12 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u__5};
        __pyx_t_11 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_split, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
      }
      __pyx_t_7 = __Pyx_PySequence_ListKeepNew(__pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_16 = __Pyx_PyList_GET_SIZE(__pyx_t_7);
      if (unlikely(__pyx_t_16 < 1)) {
        __Pyx_RaiseNeedMoreValuesError(0+__pyx_t_16); __PYX_ERR(3, 607, __pyx_L1_error)
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_16-1); 
      ((PyVarObject*)__pyx_t_7)->ob_size--;
      #else
      __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_16-1); 
      #endif
      __Pyx_GOTREF(__pyx_t_6);
      #if !CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_9 = PySequence_GetSlice(__pyx_t_7, 0, __pyx_t_16-1); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_7);
      __pyx_t_7 = __pyx_t_9; __pyx_t_9 = NULL;
      #else
      CYTHON_UNUSED_VAR(__pyx_t_9);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_sig_series, ((PyObject*)__pyx_t_7));
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_v_last_type, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_11 = __pyx_v_sig_series; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_16 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
          #endif
          if (__pyx_t_16 >= __pyx_temp) break;
        }
        __pyx_t_6 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_16);
        ++__pyx_t_16;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_sig_type, __pyx_t_6);
        __pyx_t_6 = 0;
        if (unlikely(__pyx_v_sigindex_node == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(3, 607, __pyx_L1_error)
        }
        __pyx_t_2 = (__Pyx_PyDict_ContainsTF(__pyx_v_sig_type, __pyx_v_sigindex_node, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
        if (__pyx_t_2) {
          __pyx_t_6 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 607, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (unlikely(__pyx_v_sigindex_node == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(3, 607, __pyx_L1_error)
          }
          if (unlikely((PyDict_SetItem(__pyx_v_sigindex_node, __pyx_v_sig_type, __pyx_t_6) < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_DECREF_SET(__pyx_v_sigindex_node, __pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          if (unlikely(__pyx_v_sigindex_node == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(3, 607, __pyx_L1_error)
          }
          __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_sigindex_node, __pyx_v_sig_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 607, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_7 = __pyx_t_6;
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF_SET(__pyx_v_sigindex_node, ((PyObject*)__pyx_t_7));
          __pyx_t_7 = 0;
        }
        __pyx_L19:;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(__pyx_v_sigindex_node == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(3, 607, __pyx_L1_error)
      }
      if (unlikely((PyDict_SetItem(__pyx_v_sigindex_node, __pyx_v_last_type, __pyx_v_sig) < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely((__Pyx_SetItemInt(__pyx_v__fused_sigindex_ref, 0, __pyx_v_fused_sigindex, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1) < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
  }
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_sigindex_matches = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_fused_sigindex);
  __Pyx_GIVEREF(__pyx_v_fused_sigindex);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_v_fused_sigindex) != (0)) __PYX_ERR(3, 607, __pyx_L1_error);
  __pyx_v_sigindex_candidates = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_v_dest_sig; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_13 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
      #endif
      if (__pyx_t_13 >= __pyx_temp) break;
    }
    __pyx_t_11 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_13);
    ++__pyx_t_13;
    if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_11);
    __pyx_t_11 = 0;
    __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_found_matches, ((PyObject*)__pyx_t_11));
    __pyx_t_11 = 0;
    __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_found_candidates, ((PyObject*)__pyx_t_11));
    __pyx_t_11 = 0;
    __pyx_t_2 = (__pyx_v_dst_type == Py_None);
    if (__pyx_t_2) {
      __pyx_t_11 = __pyx_v_sigindex_matches; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        __pyx_t_7 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_5);
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_7);
        __pyx_t_7 = 0;
        if (unlikely(__pyx_v_sn == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
          __PYX_ERR(3, 607, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyDict_Values(((PyObject*)__pyx_v_sn)); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_17 = __Pyx_PyList_Extend(__pyx_v_found_matches, __pyx_t_7); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = __pyx_v_sigindex_candidates; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        __pyx_t_7 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_5);
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_7);
        __pyx_t_7 = 0;
        if (unlikely(__pyx_v_sn == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
          __PYX_ERR(3, 607, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyDict_Values(((PyObject*)__pyx_v_sn)); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_17 = __Pyx_PyList_Extend(__pyx_v_found_candidates, __pyx_t_7); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L23;
    }
    /*else*/ {
      __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_v_sigindex_matches);
      __Pyx_GIVEREF(__pyx_v_sigindex_matches);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_sigindex_matches) != (0)) __PYX_ERR(3, 607, __pyx_L1_error);
      __Pyx_INCREF(__pyx_v_sigindex_candidates);
      __Pyx_GIVEREF(__pyx_v_sigindex_candidates);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_v_sigindex_candidates) != (0)) __PYX_ERR(3, 607, __pyx_L1_error);
      __pyx_t_7 = __pyx_t_11; __Pyx_INCREF(__pyx_t_7);
      __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      for (;;) {
        if (__pyx_t_5 >= 2) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_11 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_5));
        #else
        __pyx_t_11 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_5);
        #endif
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_XDECREF_SET(__pyx_v_search_list, ((PyObject*)__pyx_t_11));
        __pyx_t_11 = 0;
        if (unlikely(__pyx_v_search_list == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(3, 607, __pyx_L1_error)
        }
        __pyx_t_11 = __pyx_v_search_list; __Pyx_INCREF(__pyx_t_11);
        __pyx_t_16 = 0;
        for (;;) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
            #endif
            if (__pyx_t_16 >= __pyx_temp) break;
          }
          __pyx_t_6 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_16);
          ++__pyx_t_16;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 607, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_6);
          __pyx_t_6 = 0;
          if (unlikely(__pyx_v_sn == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
            __PYX_ERR(3, 607, __pyx_L1_error)
          }
          __pyx_t_6 = __Pyx_PyDict_GetItemDefault(((PyObject*)__pyx_v_sn), __pyx_v_dst_type, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 607, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_type_match, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_2 = (__pyx_v_type_match != Py_None);
          if (__pyx_t_2) {
            __pyx_t_17 = __Pyx_PyList_Append(__pyx_v_found_matches, __pyx_v_type_match); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 607, __pyx_L1_error)
          }
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __pyx_L23:;
    __Pyx_INCREF(__pyx_v_found_matches);
    __Pyx_DECREF_SET(__pyx_v_sigindex_matches, __pyx_v_found_matches);
    __Pyx_INCREF(__pyx_v_found_candidates);
    __Pyx_DECREF_SET(__pyx_v_sigindex_candidates, __pyx_v_found_candidates);
    __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_found_matches) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
    if (!__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L38_bool_binop_done;
    }
    __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_found_candidates) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
    __pyx_t_2 = __pyx_t_4;
    __pyx_L38_bool_binop_done:;
    __pyx_t_4 = (!__pyx_t_2);
    if (__pyx_t_4) {
      goto __pyx_L22_break;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L40_for_end;
  __pyx_L22_break:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L40_for_end;
  __pyx_L40_for_end:;
  __Pyx_INCREF(__pyx_v_sigindex_matches);
  __pyx_v_candidates = __pyx_v_sigindex_matches;
  __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_candidates) != 0);
  if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 607, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_4);
  if (unlikely(__pyx_t_2)) {
    __pyx_t_7 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_11 = __pyx_builtin_TypeError; 
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_No_matching_signature_found};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 607, __pyx_L1_error)
  }
  __pyx_t_13 = __Pyx_PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(3, 607, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_13 > 1);
  if (unlikely(__pyx_t_2)) {
    __pyx_t_11 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_mstate_global->__pyx_kp_u_Function_call_with_ambiguous_arg};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 607, __pyx_L1_error)
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(((PyObject *)__pyx_v_signatures) == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 607, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), __Pyx_PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("selectolax.parser.Node.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_search_list);
  __Pyx_XDECREF(__pyx_v_sigindex_node);
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_fused_sigindex);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_sig_series);
  __Pyx_XDECREF(__pyx_v_last_type);
  __Pyx_XDECREF(__pyx_v_sig_type);
  __Pyx_XDECREF(__pyx_v_sigindex_matches);
  __Pyx_XDECREF(__pyx_v_sigindex_candidates);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_found_matches);
  __Pyx_XDECREF(__pyx_v_found_candidates);
  __Pyx_XDECREF(__pyx_v_sn);
  __Pyx_XDECREF(__pyx_v_type_match);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_51replace_with(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_10selectolax_6parser_4Node_51replace_with = {"__pyx_fuse_0replace_with", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_51replace_with, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_28replace_with};
static PyObject *__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_51replace_with(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("replace_with (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 607, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 607, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "replace_with", 0) < 0) __PYX_ERR(3, 607, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("replace_with", 1, 1, 1, i); __PYX_ERR(3, 607, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 607, __pyx_L3_error)
    }
    __pyx_v_value = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("replace_with", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 607, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.replace_with", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyUnicode_Type), 1, "value", 1))) __PYX_ERR(3, 607, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_50replace_with(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_50replace_with(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0replace_with", 0);

  /* "selectolax/modest/node.pxi":635
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_value); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_value); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":636
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_0__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":637
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":638
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 638, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 638, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 638, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 638, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":639
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":640
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    myhtml_node_delete(__pyx_v_self->node);

    /* "selectolax/modest/node.pxi":635
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":641
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":642
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 642, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_t_3));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":643
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *             myhtml_node_delete(self.node)
 *         else:
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":644
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    myhtml_node_delete(__pyx_v_self->node);

    /* "selectolax/modest/node.pxi":641
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":646
 *             myhtml_node_delete(self.node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def insert_before(self, str_or_Node value):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_value)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 646, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 646, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 646, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 646, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":607
 *                 element.unwrap(delete_empty)
 * 
 *     def replace_with(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """Replace current Node with specified value.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.replace_with", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_53replace_with(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_10selectolax_6parser_4Node_53replace_with = {"__pyx_fuse_1replace_with", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_53replace_with, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_28replace_with};
static PyObject *__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_53replace_with(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("replace_with (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 607, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 607, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "replace_with", 0) < 0) __PYX_ERR(3, 607, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("replace_with", 1, 1, 1, i); __PYX_ERR(3, 607, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 607, __pyx_L3_error)
    }
    __pyx_v_value = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("replace_with", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 607, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.replace_with", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyBytes_Type), 1, "value", 1))) __PYX_ERR(3, 607, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_52replace_with(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_52replace_with(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1replace_with", 0);

  /* "selectolax/modest/node.pxi":635
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_value); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_value); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":636
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_1__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":637
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":638
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 638, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 638, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 638, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 638, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":639
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":640
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    myhtml_node_delete(__pyx_v_self->node);

    /* "selectolax/modest/node.pxi":635
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":641
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":642
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 642, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_t_3));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":643
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *             myhtml_node_delete(self.node)
 *         else:
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":644
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    myhtml_node_delete(__pyx_v_self->node);

    /* "selectolax/modest/node.pxi":641
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":646
 *             myhtml_node_delete(self.node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def insert_before(self, str_or_Node value):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_value)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 646, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 646, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 646, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 646, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":607
 *                 element.unwrap(delete_empty)
 * 
 *     def replace_with(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """Replace current Node with specified value.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.replace_with", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_55replace_with(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2__pyx_mdef_10selectolax_6parser_4Node_55replace_with = {"__pyx_fuse_2replace_with", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_55replace_with, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_28replace_with};
static PyObject *__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_55replace_with(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("replace_with (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 607, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 607, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "replace_with", 0) < 0) __PYX_ERR(3, 607, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("replace_with", 1, 1, 1, i); __PYX_ERR(3, 607, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 607, __pyx_L3_error)
    }
    __pyx_v_value = ((struct __pyx_obj_10selectolax_6parser_Node *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("replace_with", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 607, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.replace_with", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, 1, "value", 0))) __PYX_ERR(3, 607, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_54replace_with(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_54replace_with(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2replace_with", 0);

  /* "selectolax/modest/node.pxi":635
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(((PyObject *)__pyx_v_value)); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(((PyObject *)__pyx_v_value)); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":636
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_2__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":637
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":638
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 638, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 638, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 638, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 638, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":639
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":640
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    myhtml_node_delete(__pyx_v_self->node);

    /* "selectolax/modest/node.pxi":635
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":641
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_value), __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":642
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
*/
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_v_value->node));

    /* "selectolax/modest/node.pxi":643
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *             myhtml_node_delete(self.node)
 *         else:
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":644
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    myhtml_node_delete(__pyx_v_self->node);

    /* "selectolax/modest/node.pxi":641
 *             myhtml_node_insert_before(self.node, node)
 *             myhtml_node_delete(self.node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":646
 *             myhtml_node_delete(self.node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def insert_before(self, str_or_Node value):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_value))), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 646, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 646, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 646, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 646, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":607
 *                 element.unwrap(delete_empty)
 * 
 *     def replace_with(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """Replace current Node with specified value.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.replace_with", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":648
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_before(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node before the current Node.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_31insert_before(PyObject *__pyx_v_signatures, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_30insert_before, "Node.insert_before(signatures, args, kwargs, defaults, _fused_sigindex_ref=[None])\n\nInsert a node before the current Node.\n\nParameters\n----------\nvalue : str, bytes or Node\n    The text or Node instance to insert before the Node.\n    When a text string is passed, it's treated as text. All HTML tags will be escaped.\n    Convert and pass the ``Node`` object when you want to work with HTML.\n    Does not clone the ``Node`` object.\n    All future changes to the passed ``Node`` object will also be taken into account.\n\nExamples\n--------\n\n>>> tree = HTMLParser('<div>Get <img src=\"\" alt=\"Laptop\"></div>')\n>>> img = tree.css_first('img')\n>>> img.insert_before(img.attributes.get('alt', ''))\n>>> tree.body.child.html\n'<div>Get Laptop<img src=\"\" alt=\"Laptop\"></div>'\n\n>>> html_parser = HTMLParser('<div>Get <span alt=\"Laptop\"><img src=\"/jpg\"> <div></div></span></div>')\n>>> html_parser2 = HTMLParser('<div>Test</div>')\n>>> img_node = html_parser.css_first('img')\n>>> img_node.insert_before(html_parser2.body.child)\n<div>Get <span alt=\"Laptop\"><div>Test</div><img src=\"/jpg\"> <div></div></span></div>'");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_31insert_before = {"insert_before", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_10selectolax_6parser_4Node_31insert_before, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_30insert_before};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_31insert_before(PyObject *__pyx_v_signatures, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  PyObject *__pyx_v__fused_sigindex_ref = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_args,&__pyx_mstate_global->__pyx_n_u_kwargs,&__pyx_mstate_global->__pyx_n_u_defaults,&__pyx_mstate_global->__pyx_n_u_fused_sigindex_ref,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 648, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(3, 648, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 648, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 648, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 648, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_fused_cpdef", 0) < 0) __PYX_ERR(3, 648, __pyx_L3_error)
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__6);
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 0, 3, 4, i); __PYX_ERR(3, 648, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(3, 648, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 648, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 648, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 648, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__6);
    }
    __pyx_v_args = values[0];
    __pyx_v_kwargs = values[1];
    __pyx_v_defaults = values[2];
    __pyx_v__fused_sigindex_ref = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 0, 3, 4, __pyx_nargs); __PYX_ERR(3, 648, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_30insert_before(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_signatures), __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults, __pyx_v__fused_sigindex_ref);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_30insert_before(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults, PyObject *__pyx_v__fused_sigindex_ref) {
  PyObject *__pyx_v_search_list = 0;
  PyObject *__pyx_v_sigindex_node = 0;
  PyObject *__pyx_v_dest_sig = NULL;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_fused_sigindex = NULL;
  PyObject *__pyx_v_sig = NULL;
  PyObject *__pyx_v_sig_series = NULL;
  PyObject *__pyx_v_last_type = NULL;
  PyObject *__pyx_v_sig_type = NULL;
  PyObject *__pyx_v_sigindex_matches = NULL;
  PyObject *__pyx_v_sigindex_candidates = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_v_found_matches = NULL;
  PyObject *__pyx_v_found_candidates = NULL;
  PyObject *__pyx_v_sn = NULL;
  PyObject *__pyx_v_type_match = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[4];
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  int __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insert_before", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, Py_None) != (0)) __PYX_ERR(3, 648, __pyx_L1_error);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
  __pyx_t_4 = (!__pyx_t_3);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {
    __Pyx_INCREF(Py_None);
    __Pyx_DECREF_SET(__pyx_v_kwargs, Py_None);
  }
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(3, 648, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 648, __pyx_L1_error)
  __pyx_t_2 = (1 < __pyx_t_5);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 648, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  __pyx_t_4 = (__pyx_v_kwargs != Py_None);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L7_bool_binop_done;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(3, 648, __pyx_L1_error)
  }
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_mstate_global->__pyx_n_u_value, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_4;
  __pyx_L7_bool_binop_done:;
  if (likely(__pyx_t_2)) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 648, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyUnicode_From_long(2, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 648, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 648, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_t_5, 0, ' ', 'd'); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Expected_at_least;
    __pyx_t_10[1] = __pyx_t_8;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_arguments_got;
    __pyx_t_10[3] = __pyx_t_9;
    __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 4, 18 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 16 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9), 127);
    if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 648, __pyx_L1_error)
  }
  __pyx_L6:;
  while (1) {
    __pyx_t_2 = PyUnicode_Check(__pyx_v_arg); 
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_str, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    __pyx_t_2 = PyBytes_Check(__pyx_v_arg); 
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_bytes, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    __pyx_t_7 = NULL;
    __Pyx_INCREF(__pyx_builtin___import__);
    __pyx_t_11 = __pyx_builtin___import__; 
    __pyx_t_6 = PyList_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u_);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 0, __pyx_mstate_global->__pyx_kp_u_) != (0)) __PYX_ERR(3, 648, __pyx_L1_error);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[5] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_selectolax_parser, Py_None, Py_None, __pyx_t_6};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_12, (5-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_Node); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = PyObject_IsInstance(__pyx_v_arg, __pyx_t_11); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_Node, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
    goto __pyx_L10_break;
  }
  __pyx_L10_break:;
  __pyx_t_11 = __Pyx_GetItemInt(__pyx_v__fused_sigindex_ref, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __pyx_t_11;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_fused_sigindex = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_fused_sigindex == ((PyObject*)Py_None));
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_fused_sigindex, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;
    __pyx_t_5 = 0;
    if (unlikely(((PyObject *)__pyx_v_signatures) == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(3, 648, __pyx_L1_error)
    }
    __pyx_t_11 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_13), (&__pyx_t_14)); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF(__pyx_t_1);
    __pyx_t_1 = __pyx_t_11;
    __pyx_t_11 = 0;
    while (1) {
      __pyx_t_15 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_13, &__pyx_t_5, &__pyx_t_11, NULL, NULL, __pyx_t_14);
      if (unlikely(__pyx_t_15 == 0)) break;
      if (unlikely(__pyx_t_15 == -1)) __PYX_ERR(3, 648, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_INCREF(__pyx_v_fused_sigindex);
      __Pyx_XDECREF_SET(__pyx_v_sigindex_node, __pyx_v_fused_sigindex);
      __pyx_t_9 = __pyx_v_sig;
      __Pyx_INCREF(__pyx_t_9);
      __pyx_t_12 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u__4};
        __pyx_t_7 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_strip, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __pyx_t_6 = __pyx_t_7;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_12 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u__5};
        __pyx_t_11 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_split, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
      }
      __pyx_t_7 = __Pyx_PySequence_ListKeepNew(__pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 648, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_16 = __Pyx_PyList_GET_SIZE(__pyx_t_7);
      if (unlikely(__pyx_t_16 < 1)) {
        __Pyx_RaiseNeedMoreValuesError(0+__pyx_t_16); __PYX_ERR(3, 648, __pyx_L1_error)
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_16-1); 
      ((PyVarObject*)__pyx_t_7)->ob_size--;
      #else
      __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_16-1); 
      #endif
      __Pyx_GOTREF(__pyx_t_6);
      #if !CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_9 = PySequence_GetSlice(__pyx_t_7, 0, __pyx_t_16-1); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 648, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_7);
      __pyx_t_7 = __pyx_t_9; __pyx_t_9 = NULL;
      #else
      CYTHON_UNUSED_VAR(__pyx_t_9);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_sig_series, ((PyObject*)__pyx_t_7));
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_v_last_type, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_11 = __pyx_v_sig_series; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_16 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
          #endif
          if (__pyx_t_16 >= __pyx_temp) break;
        }
        __pyx_t_6 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_16);
        ++__pyx_t_16;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_sig_type, __pyx_t_6);
        __pyx_t_6 = 0;
        if (unlikely(__pyx_v_sigindex_node == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(3, 648, __pyx_L1_error)
        }
        __pyx_t_2 = (__Pyx_PyDict_ContainsTF(__pyx_v_sig_type, __pyx_v_sigindex_node, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
        if (__pyx_t_2) {
          __pyx_t_6 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 648, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (unlikely(__pyx_v_sigindex_node == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(3, 648, __pyx_L1_error)
          }
          if (unlikely((PyDict_SetItem(__pyx_v_sigindex_node, __pyx_v_sig_type, __pyx_t_6) < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_DECREF_SET(__pyx_v_sigindex_node, __pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          if (unlikely(__pyx_v_sigindex_node == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(3, 648, __pyx_L1_error)
          }
          __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_sigindex_node, __pyx_v_sig_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 648, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_7 = __pyx_t_6;
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF_SET(__pyx_v_sigindex_node, ((PyObject*)__pyx_t_7));
          __pyx_t_7 = 0;
        }
        __pyx_L19:;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(__pyx_v_sigindex_node == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(3, 648, __pyx_L1_error)
      }
      if (unlikely((PyDict_SetItem(__pyx_v_sigindex_node, __pyx_v_last_type, __pyx_v_sig) < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely((__Pyx_SetItemInt(__pyx_v__fused_sigindex_ref, 0, __pyx_v_fused_sigindex, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1) < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
  }
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_sigindex_matches = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_fused_sigindex);
  __Pyx_GIVEREF(__pyx_v_fused_sigindex);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_v_fused_sigindex) != (0)) __PYX_ERR(3, 648, __pyx_L1_error);
  __pyx_v_sigindex_candidates = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_v_dest_sig; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_13 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
      #endif
      if (__pyx_t_13 >= __pyx_temp) break;
    }
    __pyx_t_11 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_13);
    ++__pyx_t_13;
    if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_11);
    __pyx_t_11 = 0;
    __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_found_matches, ((PyObject*)__pyx_t_11));
    __pyx_t_11 = 0;
    __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_found_candidates, ((PyObject*)__pyx_t_11));
    __pyx_t_11 = 0;
    __pyx_t_2 = (__pyx_v_dst_type == Py_None);
    if (__pyx_t_2) {
      __pyx_t_11 = __pyx_v_sigindex_matches; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        __pyx_t_7 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_5);
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_7);
        __pyx_t_7 = 0;
        if (unlikely(__pyx_v_sn == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
          __PYX_ERR(3, 648, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyDict_Values(((PyObject*)__pyx_v_sn)); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_17 = __Pyx_PyList_Extend(__pyx_v_found_matches, __pyx_t_7); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = __pyx_v_sigindex_candidates; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        __pyx_t_7 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_5);
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_7);
        __pyx_t_7 = 0;
        if (unlikely(__pyx_v_sn == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
          __PYX_ERR(3, 648, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyDict_Values(((PyObject*)__pyx_v_sn)); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_17 = __Pyx_PyList_Extend(__pyx_v_found_candidates, __pyx_t_7); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L23;
    }
    /*else*/ {
      __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_v_sigindex_matches);
      __Pyx_GIVEREF(__pyx_v_sigindex_matches);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_sigindex_matches) != (0)) __PYX_ERR(3, 648, __pyx_L1_error);
      __Pyx_INCREF(__pyx_v_sigindex_candidates);
      __Pyx_GIVEREF(__pyx_v_sigindex_candidates);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_v_sigindex_candidates) != (0)) __PYX_ERR(3, 648, __pyx_L1_error);
      __pyx_t_7 = __pyx_t_11; __Pyx_INCREF(__pyx_t_7);
      __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      for (;;) {
        if (__pyx_t_5 >= 2) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_11 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_5));
        #else
        __pyx_t_11 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_5);
        #endif
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 648, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_XDECREF_SET(__pyx_v_search_list, ((PyObject*)__pyx_t_11));
        __pyx_t_11 = 0;
        if (unlikely(__pyx_v_search_list == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(3, 648, __pyx_L1_error)
        }
        __pyx_t_11 = __pyx_v_search_list; __Pyx_INCREF(__pyx_t_11);
        __pyx_t_16 = 0;
        for (;;) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
            #endif
            if (__pyx_t_16 >= __pyx_temp) break;
          }
          __pyx_t_6 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_16);
          ++__pyx_t_16;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 648, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_6);
          __pyx_t_6 = 0;
          if (unlikely(__pyx_v_sn == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
            __PYX_ERR(3, 648, __pyx_L1_error)
          }
          __pyx_t_6 = __Pyx_PyDict_GetItemDefault(((PyObject*)__pyx_v_sn), __pyx_v_dst_type, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 648, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_type_match, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_2 = (__pyx_v_type_match != Py_None);
          if (__pyx_t_2) {
            __pyx_t_17 = __Pyx_PyList_Append(__pyx_v_found_matches, __pyx_v_type_match); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 648, __pyx_L1_error)
          }
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __pyx_L23:;
    __Pyx_INCREF(__pyx_v_found_matches);
    __Pyx_DECREF_SET(__pyx_v_sigindex_matches, __pyx_v_found_matches);
    __Pyx_INCREF(__pyx_v_found_candidates);
    __Pyx_DECREF_SET(__pyx_v_sigindex_candidates, __pyx_v_found_candidates);
    __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_found_matches) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
    if (!__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L38_bool_binop_done;
    }
    __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_found_candidates) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
    __pyx_t_2 = __pyx_t_4;
    __pyx_L38_bool_binop_done:;
    __pyx_t_4 = (!__pyx_t_2);
    if (__pyx_t_4) {
      goto __pyx_L22_break;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L40_for_end;
  __pyx_L22_break:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L40_for_end;
  __pyx_L40_for_end:;
  __Pyx_INCREF(__pyx_v_sigindex_matches);
  __pyx_v_candidates = __pyx_v_sigindex_matches;
  __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_candidates) != 0);
  if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 648, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_4);
  if (unlikely(__pyx_t_2)) {
    __pyx_t_7 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_11 = __pyx_builtin_TypeError; 
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_No_matching_signature_found};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 648, __pyx_L1_error)
  }
  __pyx_t_13 = __Pyx_PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(3, 648, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_13 > 1);
  if (unlikely(__pyx_t_2)) {
    __pyx_t_11 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_mstate_global->__pyx_kp_u_Function_call_with_ambiguous_arg};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 648, __pyx_L1_error)
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(((PyObject *)__pyx_v_signatures) == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 648, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), __Pyx_PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("selectolax.parser.Node.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_search_list);
  __Pyx_XDECREF(__pyx_v_sigindex_node);
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_fused_sigindex);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_sig_series);
  __Pyx_XDECREF(__pyx_v_last_type);
  __Pyx_XDECREF(__pyx_v_sig_type);
  __Pyx_XDECREF(__pyx_v_sigindex_matches);
  __Pyx_XDECREF(__pyx_v_sigindex_candidates);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_found_matches);
  __Pyx_XDECREF(__pyx_v_found_candidates);
  __Pyx_XDECREF(__pyx_v_sn);
  __Pyx_XDECREF(__pyx_v_type_match);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_59insert_before(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_10selectolax_6parser_4Node_59insert_before = {"__pyx_fuse_0insert_before", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_59insert_before, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_30insert_before};
static PyObject *__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_59insert_before(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert_before (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 648, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 648, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "insert_before", 0) < 0) __PYX_ERR(3, 648, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("insert_before", 1, 1, 1, i); __PYX_ERR(3, 648, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 648, __pyx_L3_error)
    }
    __pyx_v_value = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert_before", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 648, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.insert_before", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyUnicode_Type), 1, "value", 1))) __PYX_ERR(3, 648, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_58insert_before(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_58insert_before(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0insert_before", 0);

  /* "selectolax/modest/node.pxi":677
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_value); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_value); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":678
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val =  to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_0__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":679
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":680
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 680, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 680, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 680, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 680, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":681
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":677
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":682
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":683
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *         else:
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 683, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_t_3));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":684
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":682
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":686
 *             myhtml_node_insert_before(self.node, node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def insert_after(self, str_or_Node value):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_value)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 686, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 686, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":648
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_before(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node before the current Node.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.insert_before", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_61insert_before(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_10selectolax_6parser_4Node_61insert_before = {"__pyx_fuse_1insert_before", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_61insert_before, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_30insert_before};
static PyObject *__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_61insert_before(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert_before (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 648, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 648, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "insert_before", 0) < 0) __PYX_ERR(3, 648, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("insert_before", 1, 1, 1, i); __PYX_ERR(3, 648, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 648, __pyx_L3_error)
    }
    __pyx_v_value = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert_before", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 648, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.insert_before", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyBytes_Type), 1, "value", 1))) __PYX_ERR(3, 648, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_60insert_before(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_60insert_before(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1insert_before", 0);

  /* "selectolax/modest/node.pxi":677
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_value); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_value); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":678
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val =  to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_1__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":679
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":680
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 680, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 680, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 680, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 680, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":681
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":677
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":682
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":683
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *         else:
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 683, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_t_3));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":684
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":682
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":686
 *             myhtml_node_insert_before(self.node, node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def insert_after(self, str_or_Node value):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_value)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 686, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 686, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":648
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_before(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node before the current Node.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.insert_before", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_63insert_before(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2__pyx_mdef_10selectolax_6parser_4Node_63insert_before = {"__pyx_fuse_2insert_before", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_63insert_before, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_30insert_before};
static PyObject *__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_63insert_before(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert_before (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 648, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 648, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "insert_before", 0) < 0) __PYX_ERR(3, 648, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("insert_before", 1, 1, 1, i); __PYX_ERR(3, 648, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 648, __pyx_L3_error)
    }
    __pyx_v_value = ((struct __pyx_obj_10selectolax_6parser_Node *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert_before", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 648, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.insert_before", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, 1, "value", 0))) __PYX_ERR(3, 648, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_62insert_before(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_62insert_before(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2insert_before", 0);

  /* "selectolax/modest/node.pxi":677
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(((PyObject *)__pyx_v_value)); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(((PyObject *)__pyx_v_value)); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":678
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val =  to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_2__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":679
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":680
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 680, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 680, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 680, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 680, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":681
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":677
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val =  to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":682
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_value), __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":683
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_before(self.node, node)
 *         else:
*/
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_v_value->node));

    /* "selectolax/modest/node.pxi":684
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    (void)(myhtml_node_insert_before(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":682
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_before(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_before(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":686
 *             myhtml_node_insert_before(self.node, node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def insert_after(self, str_or_Node value):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_value))), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 686, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 686, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":648
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_before(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node before the current Node.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.insert_before", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":688
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_after(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node after the current Node.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_33insert_after(PyObject *__pyx_v_signatures, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_32insert_after, "Node.insert_after(signatures, args, kwargs, defaults, _fused_sigindex_ref=[None])\n\nInsert a node after the current Node.\n\nParameters\n----------\nvalue : str, bytes or Node\n    The text or Node instance to insert after the Node.\n    When a text string is passed, it's treated as text. All HTML tags will be escaped.\n    Convert and pass the ``Node`` object when you want to work with HTML.\n    Does not clone the ``Node`` object.\n    All future changes to the passed ``Node`` object will also be taken into account.\n\nExamples\n--------\n\n>>> tree = HTMLParser('<div>Get <img src=\"\" alt=\"Laptop\"></div>')\n>>> img = tree.css_first('img')\n>>> img.insert_after(img.attributes.get('alt', ''))\n>>> tree.body.child.html\n'<div>Get <img src=\"\" alt=\"Laptop\">Laptop</div>'\n\n>>> html_parser = HTMLParser('<div>Get <span alt=\"Laptop\"><img src=\"/jpg\"> <div></div></span></div>')\n>>> html_parser2 = HTMLParser('<div>Test</div>')\n>>> img_node = html_parser.css_first('img')\n>>> img_node.insert_after(html_parser2.body.child)\n<div>Get <span alt=\"Laptop\"><img src=\"/jpg\"><div>Test</div> <div></div></span></div>'");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_33insert_after = {"insert_after", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_10selectolax_6parser_4Node_33insert_after, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_32insert_after};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_33insert_after(PyObject *__pyx_v_signatures, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  PyObject *__pyx_v__fused_sigindex_ref = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_args,&__pyx_mstate_global->__pyx_n_u_kwargs,&__pyx_mstate_global->__pyx_n_u_defaults,&__pyx_mstate_global->__pyx_n_u_fused_sigindex_ref,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 688, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(3, 688, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 688, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 688, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 688, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_fused_cpdef", 0) < 0) __PYX_ERR(3, 688, __pyx_L3_error)
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__7);
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 0, 3, 4, i); __PYX_ERR(3, 688, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(3, 688, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 688, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 688, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 688, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__7);
    }
    __pyx_v_args = values[0];
    __pyx_v_kwargs = values[1];
    __pyx_v_defaults = values[2];
    __pyx_v__fused_sigindex_ref = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 0, 3, 4, __pyx_nargs); __PYX_ERR(3, 688, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_32insert_after(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_signatures), __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults, __pyx_v__fused_sigindex_ref);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_32insert_after(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults, PyObject *__pyx_v__fused_sigindex_ref) {
  PyObject *__pyx_v_search_list = 0;
  PyObject *__pyx_v_sigindex_node = 0;
  PyObject *__pyx_v_dest_sig = NULL;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_fused_sigindex = NULL;
  PyObject *__pyx_v_sig = NULL;
  PyObject *__pyx_v_sig_series = NULL;
  PyObject *__pyx_v_last_type = NULL;
  PyObject *__pyx_v_sig_type = NULL;
  PyObject *__pyx_v_sigindex_matches = NULL;
  PyObject *__pyx_v_sigindex_candidates = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_v_found_matches = NULL;
  PyObject *__pyx_v_found_candidates = NULL;
  PyObject *__pyx_v_sn = NULL;
  PyObject *__pyx_v_type_match = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[4];
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  int __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insert_after", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, Py_None) != (0)) __PYX_ERR(3, 688, __pyx_L1_error);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
  __pyx_t_4 = (!__pyx_t_3);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {
    __Pyx_INCREF(Py_None);
    __Pyx_DECREF_SET(__pyx_v_kwargs, Py_None);
  }
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(3, 688, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 688, __pyx_L1_error)
  __pyx_t_2 = (1 < __pyx_t_5);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 688, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  __pyx_t_4 = (__pyx_v_kwargs != Py_None);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L7_bool_binop_done;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(3, 688, __pyx_L1_error)
  }
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_mstate_global->__pyx_n_u_value, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_4;
  __pyx_L7_bool_binop_done:;
  if (likely(__pyx_t_2)) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 688, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyUnicode_From_long(2, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 688, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 688, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_t_5, 0, ' ', 'd'); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Expected_at_least;
    __pyx_t_10[1] = __pyx_t_8;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_arguments_got;
    __pyx_t_10[3] = __pyx_t_9;
    __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 4, 18 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 16 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9), 127);
    if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 688, __pyx_L1_error)
  }
  __pyx_L6:;
  while (1) {
    __pyx_t_2 = PyUnicode_Check(__pyx_v_arg); 
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_str, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    __pyx_t_2 = PyBytes_Check(__pyx_v_arg); 
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_bytes, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    __pyx_t_7 = NULL;
    __Pyx_INCREF(__pyx_builtin___import__);
    __pyx_t_11 = __pyx_builtin___import__; 
    __pyx_t_6 = PyList_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u_);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 0, __pyx_mstate_global->__pyx_kp_u_) != (0)) __PYX_ERR(3, 688, __pyx_L1_error);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[5] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_selectolax_parser, Py_None, Py_None, __pyx_t_6};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_12, (5-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_Node); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = PyObject_IsInstance(__pyx_v_arg, __pyx_t_11); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_Node, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
    goto __pyx_L10_break;
  }
  __pyx_L10_break:;
  __pyx_t_11 = __Pyx_GetItemInt(__pyx_v__fused_sigindex_ref, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __pyx_t_11;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_fused_sigindex = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_fused_sigindex == ((PyObject*)Py_None));
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_fused_sigindex, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;
    __pyx_t_5 = 0;
    if (unlikely(((PyObject *)__pyx_v_signatures) == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(3, 688, __pyx_L1_error)
    }
    __pyx_t_11 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_13), (&__pyx_t_14)); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF(__pyx_t_1);
    __pyx_t_1 = __pyx_t_11;
    __pyx_t_11 = 0;
    while (1) {
      __pyx_t_15 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_13, &__pyx_t_5, &__pyx_t_11, NULL, NULL, __pyx_t_14);
      if (unlikely(__pyx_t_15 == 0)) break;
      if (unlikely(__pyx_t_15 == -1)) __PYX_ERR(3, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_INCREF(__pyx_v_fused_sigindex);
      __Pyx_XDECREF_SET(__pyx_v_sigindex_node, __pyx_v_fused_sigindex);
      __pyx_t_9 = __pyx_v_sig;
      __Pyx_INCREF(__pyx_t_9);
      __pyx_t_12 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u__4};
        __pyx_t_7 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_strip, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __pyx_t_6 = __pyx_t_7;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_12 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u__5};
        __pyx_t_11 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_split, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
      }
      __pyx_t_7 = __Pyx_PySequence_ListKeepNew(__pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_16 = __Pyx_PyList_GET_SIZE(__pyx_t_7);
      if (unlikely(__pyx_t_16 < 1)) {
        __Pyx_RaiseNeedMoreValuesError(0+__pyx_t_16); __PYX_ERR(3, 688, __pyx_L1_error)
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_16-1); 
      ((PyVarObject*)__pyx_t_7)->ob_size--;
      #else
      __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_16-1); 
      #endif
      __Pyx_GOTREF(__pyx_t_6);
      #if !CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_9 = PySequence_GetSlice(__pyx_t_7, 0, __pyx_t_16-1); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_7);
      __pyx_t_7 = __pyx_t_9; __pyx_t_9 = NULL;
      #else
      CYTHON_UNUSED_VAR(__pyx_t_9);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_sig_series, ((PyObject*)__pyx_t_7));
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_v_last_type, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_11 = __pyx_v_sig_series; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_16 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
          #endif
          if (__pyx_t_16 >= __pyx_temp) break;
        }
        __pyx_t_6 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_16);
        ++__pyx_t_16;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_sig_type, __pyx_t_6);
        __pyx_t_6 = 0;
        if (unlikely(__pyx_v_sigindex_node == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(3, 688, __pyx_L1_error)
        }
        __pyx_t_2 = (__Pyx_PyDict_ContainsTF(__pyx_v_sig_type, __pyx_v_sigindex_node, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
        if (__pyx_t_2) {
          __pyx_t_6 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 688, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (unlikely(__pyx_v_sigindex_node == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(3, 688, __pyx_L1_error)
          }
          if (unlikely((PyDict_SetItem(__pyx_v_sigindex_node, __pyx_v_sig_type, __pyx_t_6) < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_DECREF_SET(__pyx_v_sigindex_node, __pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          if (unlikely(__pyx_v_sigindex_node == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(3, 688, __pyx_L1_error)
          }
          __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_sigindex_node, __pyx_v_sig_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 688, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_7 = __pyx_t_6;
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF_SET(__pyx_v_sigindex_node, ((PyObject*)__pyx_t_7));
          __pyx_t_7 = 0;
        }
        __pyx_L19:;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(__pyx_v_sigindex_node == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(3, 688, __pyx_L1_error)
      }
      if (unlikely((PyDict_SetItem(__pyx_v_sigindex_node, __pyx_v_last_type, __pyx_v_sig) < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely((__Pyx_SetItemInt(__pyx_v__fused_sigindex_ref, 0, __pyx_v_fused_sigindex, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1) < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
  }
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_sigindex_matches = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_fused_sigindex);
  __Pyx_GIVEREF(__pyx_v_fused_sigindex);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_v_fused_sigindex) != (0)) __PYX_ERR(3, 688, __pyx_L1_error);
  __pyx_v_sigindex_candidates = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_v_dest_sig; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_13 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
      #endif
      if (__pyx_t_13 >= __pyx_temp) break;
    }
    __pyx_t_11 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_13);
    ++__pyx_t_13;
    if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_11);
    __pyx_t_11 = 0;
    __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_found_matches, ((PyObject*)__pyx_t_11));
    __pyx_t_11 = 0;
    __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_found_candidates, ((PyObject*)__pyx_t_11));
    __pyx_t_11 = 0;
    __pyx_t_2 = (__pyx_v_dst_type == Py_None);
    if (__pyx_t_2) {
      __pyx_t_11 = __pyx_v_sigindex_matches; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        __pyx_t_7 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_5);
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_7);
        __pyx_t_7 = 0;
        if (unlikely(__pyx_v_sn == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
          __PYX_ERR(3, 688, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyDict_Values(((PyObject*)__pyx_v_sn)); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_17 = __Pyx_PyList_Extend(__pyx_v_found_matches, __pyx_t_7); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = __pyx_v_sigindex_candidates; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        __pyx_t_7 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_5);
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_7);
        __pyx_t_7 = 0;
        if (unlikely(__pyx_v_sn == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
          __PYX_ERR(3, 688, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyDict_Values(((PyObject*)__pyx_v_sn)); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_17 = __Pyx_PyList_Extend(__pyx_v_found_candidates, __pyx_t_7); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L23;
    }
    /*else*/ {
      __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_v_sigindex_matches);
      __Pyx_GIVEREF(__pyx_v_sigindex_matches);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_sigindex_matches) != (0)) __PYX_ERR(3, 688, __pyx_L1_error);
      __Pyx_INCREF(__pyx_v_sigindex_candidates);
      __Pyx_GIVEREF(__pyx_v_sigindex_candidates);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_v_sigindex_candidates) != (0)) __PYX_ERR(3, 688, __pyx_L1_error);
      __pyx_t_7 = __pyx_t_11; __Pyx_INCREF(__pyx_t_7);
      __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      for (;;) {
        if (__pyx_t_5 >= 2) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_11 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_5));
        #else
        __pyx_t_11 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_5);
        #endif
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 688, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_XDECREF_SET(__pyx_v_search_list, ((PyObject*)__pyx_t_11));
        __pyx_t_11 = 0;
        if (unlikely(__pyx_v_search_list == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(3, 688, __pyx_L1_error)
        }
        __pyx_t_11 = __pyx_v_search_list; __Pyx_INCREF(__pyx_t_11);
        __pyx_t_16 = 0;
        for (;;) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
            #endif
            if (__pyx_t_16 >= __pyx_temp) break;
          }
          __pyx_t_6 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_16);
          ++__pyx_t_16;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 688, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_6);
          __pyx_t_6 = 0;
          if (unlikely(__pyx_v_sn == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
            __PYX_ERR(3, 688, __pyx_L1_error)
          }
          __pyx_t_6 = __Pyx_PyDict_GetItemDefault(((PyObject*)__pyx_v_sn), __pyx_v_dst_type, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 688, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_type_match, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_2 = (__pyx_v_type_match != Py_None);
          if (__pyx_t_2) {
            __pyx_t_17 = __Pyx_PyList_Append(__pyx_v_found_matches, __pyx_v_type_match); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 688, __pyx_L1_error)
          }
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __pyx_L23:;
    __Pyx_INCREF(__pyx_v_found_matches);
    __Pyx_DECREF_SET(__pyx_v_sigindex_matches, __pyx_v_found_matches);
    __Pyx_INCREF(__pyx_v_found_candidates);
    __Pyx_DECREF_SET(__pyx_v_sigindex_candidates, __pyx_v_found_candidates);
    __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_found_matches) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
    if (!__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L38_bool_binop_done;
    }
    __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_found_candidates) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
    __pyx_t_2 = __pyx_t_4;
    __pyx_L38_bool_binop_done:;
    __pyx_t_4 = (!__pyx_t_2);
    if (__pyx_t_4) {
      goto __pyx_L22_break;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L40_for_end;
  __pyx_L22_break:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L40_for_end;
  __pyx_L40_for_end:;
  __Pyx_INCREF(__pyx_v_sigindex_matches);
  __pyx_v_candidates = __pyx_v_sigindex_matches;
  __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_candidates) != 0);
  if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 688, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_4);
  if (unlikely(__pyx_t_2)) {
    __pyx_t_7 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_11 = __pyx_builtin_TypeError; 
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_No_matching_signature_found};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 688, __pyx_L1_error)
  }
  __pyx_t_13 = __Pyx_PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(3, 688, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_13 > 1);
  if (unlikely(__pyx_t_2)) {
    __pyx_t_11 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_mstate_global->__pyx_kp_u_Function_call_with_ambiguous_arg};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 688, __pyx_L1_error)
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(((PyObject *)__pyx_v_signatures) == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 688, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), __Pyx_PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("selectolax.parser.Node.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_search_list);
  __Pyx_XDECREF(__pyx_v_sigindex_node);
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_fused_sigindex);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_sig_series);
  __Pyx_XDECREF(__pyx_v_last_type);
  __Pyx_XDECREF(__pyx_v_sig_type);
  __Pyx_XDECREF(__pyx_v_sigindex_matches);
  __Pyx_XDECREF(__pyx_v_sigindex_candidates);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_found_matches);
  __Pyx_XDECREF(__pyx_v_found_candidates);
  __Pyx_XDECREF(__pyx_v_sn);
  __Pyx_XDECREF(__pyx_v_type_match);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_67insert_after(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_10selectolax_6parser_4Node_67insert_after = {"__pyx_fuse_0insert_after", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_67insert_after, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_32insert_after};
static PyObject *__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_67insert_after(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert_after (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 688, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 688, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "insert_after", 0) < 0) __PYX_ERR(3, 688, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("insert_after", 1, 1, 1, i); __PYX_ERR(3, 688, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 688, __pyx_L3_error)
    }
    __pyx_v_value = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert_after", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 688, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.insert_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyUnicode_Type), 1, "value", 1))) __PYX_ERR(3, 688, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_66insert_after(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_66insert_after(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0insert_after", 0);

  /* "selectolax/modest/node.pxi":717
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_value); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_value); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":718
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_0__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 718, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":719
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":720
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 720, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 720, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 720, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 720, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":721
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    (void)(myhtml_node_insert_after(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":717
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":722
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_after(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":723
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_after(self.node, node)
 *         else:
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 723, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_t_3));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":724
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_after(self.node, node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    (void)(myhtml_node_insert_after(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":722
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_after(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":726
 *             myhtml_node_insert_after(self.node, node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def insert_child(self, str_or_Node value):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_value)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 726, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 726, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":688
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_after(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node after the current Node.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.insert_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_69insert_after(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_10selectolax_6parser_4Node_69insert_after = {"__pyx_fuse_1insert_after", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_69insert_after, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_32insert_after};
static PyObject *__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_69insert_after(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert_after (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 688, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 688, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "insert_after", 0) < 0) __PYX_ERR(3, 688, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("insert_after", 1, 1, 1, i); __PYX_ERR(3, 688, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 688, __pyx_L3_error)
    }
    __pyx_v_value = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert_after", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 688, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.insert_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyBytes_Type), 1, "value", 1))) __PYX_ERR(3, 688, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_68insert_after(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_68insert_after(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1insert_after", 0);

  /* "selectolax/modest/node.pxi":717
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_value); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_value); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":718
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_1__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 718, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":719
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":720
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 720, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 720, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 720, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 720, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":721
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    (void)(myhtml_node_insert_after(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":717
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":722
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_after(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":723
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_after(self.node, node)
 *         else:
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 723, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_t_3));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":724
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_after(self.node, node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    (void)(myhtml_node_insert_after(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":722
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_after(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":726
 *             myhtml_node_insert_after(self.node, node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def insert_child(self, str_or_Node value):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_value)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 726, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 726, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":688
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_after(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node after the current Node.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.insert_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_71insert_after(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2__pyx_mdef_10selectolax_6parser_4Node_71insert_after = {"__pyx_fuse_2insert_after", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_71insert_after, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_32insert_after};
static PyObject *__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_71insert_after(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert_after (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 688, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 688, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "insert_after", 0) < 0) __PYX_ERR(3, 688, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("insert_after", 1, 1, 1, i); __PYX_ERR(3, 688, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 688, __pyx_L3_error)
    }
    __pyx_v_value = ((struct __pyx_obj_10selectolax_6parser_Node *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert_after", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 688, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.insert_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, 1, "value", 0))) __PYX_ERR(3, 688, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_70insert_after(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_70insert_after(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2insert_after", 0);

  /* "selectolax/modest/node.pxi":717
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(((PyObject *)__pyx_v_value)); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(((PyObject *)__pyx_v_value)); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":718
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_2__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 718, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":719
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":720
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 720, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 720, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 720, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 720, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":721
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    (void)(myhtml_node_insert_after(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":717
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":722
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_after(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_value), __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":723
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_insert_after(self.node, node)
 *         else:
*/
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_v_value->node));

    /* "selectolax/modest/node.pxi":724
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_after(self.node, node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    (void)(myhtml_node_insert_after(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":722
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_insert_after(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_insert_after(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":726
 *             myhtml_node_insert_after(self.node, node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def insert_child(self, str_or_Node value):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_value))), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 726, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 726, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":688
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_after(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node after the current Node.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.insert_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":728
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_child(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node inside (at the end of) the current Node.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_35insert_child(PyObject *__pyx_v_signatures, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_34insert_child, "Node.insert_child(signatures, args, kwargs, defaults, _fused_sigindex_ref=[None])\n\nInsert a node inside (at the end of) the current Node.\n\nParameters\n----------\nvalue : str, bytes or Node\n    The text or Node instance to insert inside the Node.\n    When a text string is passed, it's treated as text. All HTML tags will be escaped.\n    Convert and pass the ``Node`` object when you want to work with HTML.\n    Does not clone the ``Node`` object.\n    All future changes to the passed ``Node`` object will also be taken into account.\n\nExamples\n--------\n\n>>> tree = HTMLParser('<div>Get <img src=\"\"></div>')\n>>> div = tree.css_first('div')\n>>> div.insert_child('Laptop')\n>>> tree.body.child.html\n'<div>Get <img src=\"\">Laptop</div>'\n\n>>> html_parser = HTMLParser('<div>Get <span alt=\"Laptop\"> <div>Laptop</div> </span></div>')\n>>> html_parser2 = HTMLParser('<div>Test</div>')\n>>> span_node = html_parser.css_first('span')\n>>> span_node.insert_child(html_parser2.body.child)\n<div>Get <span alt=\"Laptop\"> <div>Laptop</div> <div>Test</div> </span></div>'");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_35insert_child = {"insert_child", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_10selectolax_6parser_4Node_35insert_child, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_34insert_child};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_35insert_child(PyObject *__pyx_v_signatures, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  PyObject *__pyx_v__fused_sigindex_ref = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_args,&__pyx_mstate_global->__pyx_n_u_kwargs,&__pyx_mstate_global->__pyx_n_u_defaults,&__pyx_mstate_global->__pyx_n_u_fused_sigindex_ref,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 728, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(3, 728, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 728, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 728, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 728, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_fused_cpdef", 0) < 0) __PYX_ERR(3, 728, __pyx_L3_error)
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__8);
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 0, 3, 4, i); __PYX_ERR(3, 728, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(3, 728, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 728, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 728, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 728, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__8);
    }
    __pyx_v_args = values[0];
    __pyx_v_kwargs = values[1];
    __pyx_v_defaults = values[2];
    __pyx_v__fused_sigindex_ref = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 0, 3, 4, __pyx_nargs); __PYX_ERR(3, 728, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_34insert_child(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_signatures), __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults, __pyx_v__fused_sigindex_ref);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_34insert_child(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults, PyObject *__pyx_v__fused_sigindex_ref) {
  PyObject *__pyx_v_search_list = 0;
  PyObject *__pyx_v_sigindex_node = 0;
  PyObject *__pyx_v_dest_sig = NULL;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_fused_sigindex = NULL;
  PyObject *__pyx_v_sig = NULL;
  PyObject *__pyx_v_sig_series = NULL;
  PyObject *__pyx_v_last_type = NULL;
  PyObject *__pyx_v_sig_type = NULL;
  PyObject *__pyx_v_sigindex_matches = NULL;
  PyObject *__pyx_v_sigindex_candidates = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_v_found_matches = NULL;
  PyObject *__pyx_v_found_candidates = NULL;
  PyObject *__pyx_v_sn = NULL;
  PyObject *__pyx_v_type_match = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[4];
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  int __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insert_child", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, Py_None) != (0)) __PYX_ERR(3, 728, __pyx_L1_error);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
  __pyx_t_4 = (!__pyx_t_3);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {
    __Pyx_INCREF(Py_None);
    __Pyx_DECREF_SET(__pyx_v_kwargs, Py_None);
  }
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(3, 728, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 728, __pyx_L1_error)
  __pyx_t_2 = (1 < __pyx_t_5);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 728, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  __pyx_t_4 = (__pyx_v_kwargs != Py_None);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L7_bool_binop_done;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(3, 728, __pyx_L1_error)
  }
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_mstate_global->__pyx_n_u_value, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_4;
  __pyx_L7_bool_binop_done:;
  if (likely(__pyx_t_2)) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 728, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyUnicode_From_long(2, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 728, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 728, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_t_5, 0, ' ', 'd'); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Expected_at_least;
    __pyx_t_10[1] = __pyx_t_8;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_arguments_got;
    __pyx_t_10[3] = __pyx_t_9;
    __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 4, 18 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 16 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9), 127);
    if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 728, __pyx_L1_error)
  }
  __pyx_L6:;
  while (1) {
    __pyx_t_2 = PyUnicode_Check(__pyx_v_arg); 
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_str, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    __pyx_t_2 = PyBytes_Check(__pyx_v_arg); 
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_bytes, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    __pyx_t_7 = NULL;
    __Pyx_INCREF(__pyx_builtin___import__);
    __pyx_t_11 = __pyx_builtin___import__; 
    __pyx_t_6 = PyList_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u_);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 0, __pyx_mstate_global->__pyx_kp_u_) != (0)) __PYX_ERR(3, 728, __pyx_L1_error);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[5] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_selectolax_parser, Py_None, Py_None, __pyx_t_6};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_12, (5-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_Node); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = PyObject_IsInstance(__pyx_v_arg, __pyx_t_11); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (__pyx_t_2) {
      if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_mstate_global->__pyx_n_u_Node, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
      goto __pyx_L10_break;
    }
    if (unlikely((__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyLong_From_long, 1, 0, 0, 1) < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
    goto __pyx_L10_break;
  }
  __pyx_L10_break:;
  __pyx_t_11 = __Pyx_GetItemInt(__pyx_v__fused_sigindex_ref, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __pyx_t_11;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_fused_sigindex = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_fused_sigindex == ((PyObject*)Py_None));
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_fused_sigindex, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;
    __pyx_t_5 = 0;
    if (unlikely(((PyObject *)__pyx_v_signatures) == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(3, 728, __pyx_L1_error)
    }
    __pyx_t_11 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_13), (&__pyx_t_14)); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF(__pyx_t_1);
    __pyx_t_1 = __pyx_t_11;
    __pyx_t_11 = 0;
    while (1) {
      __pyx_t_15 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_13, &__pyx_t_5, &__pyx_t_11, NULL, NULL, __pyx_t_14);
      if (unlikely(__pyx_t_15 == 0)) break;
      if (unlikely(__pyx_t_15 == -1)) __PYX_ERR(3, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_INCREF(__pyx_v_fused_sigindex);
      __Pyx_XDECREF_SET(__pyx_v_sigindex_node, __pyx_v_fused_sigindex);
      __pyx_t_9 = __pyx_v_sig;
      __Pyx_INCREF(__pyx_t_9);
      __pyx_t_12 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u__4};
        __pyx_t_7 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_strip, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __pyx_t_6 = __pyx_t_7;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_12 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u__5};
        __pyx_t_11 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_split, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
      }
      __pyx_t_7 = __Pyx_PySequence_ListKeepNew(__pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_16 = __Pyx_PyList_GET_SIZE(__pyx_t_7);
      if (unlikely(__pyx_t_16 < 1)) {
        __Pyx_RaiseNeedMoreValuesError(0+__pyx_t_16); __PYX_ERR(3, 728, __pyx_L1_error)
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_16-1); 
      ((PyVarObject*)__pyx_t_7)->ob_size--;
      #else
      __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_16-1); 
      #endif
      __Pyx_GOTREF(__pyx_t_6);
      #if !CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_9 = PySequence_GetSlice(__pyx_t_7, 0, __pyx_t_16-1); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_7);
      __pyx_t_7 = __pyx_t_9; __pyx_t_9 = NULL;
      #else
      CYTHON_UNUSED_VAR(__pyx_t_9);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_sig_series, ((PyObject*)__pyx_t_7));
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_v_last_type, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_11 = __pyx_v_sig_series; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_16 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
          #endif
          if (__pyx_t_16 >= __pyx_temp) break;
        }
        __pyx_t_6 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_16);
        ++__pyx_t_16;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_sig_type, __pyx_t_6);
        __pyx_t_6 = 0;
        if (unlikely(__pyx_v_sigindex_node == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(3, 728, __pyx_L1_error)
        }
        __pyx_t_2 = (__Pyx_PyDict_ContainsTF(__pyx_v_sig_type, __pyx_v_sigindex_node, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
        if (__pyx_t_2) {
          __pyx_t_6 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 728, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (unlikely(__pyx_v_sigindex_node == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(3, 728, __pyx_L1_error)
          }
          if (unlikely((PyDict_SetItem(__pyx_v_sigindex_node, __pyx_v_sig_type, __pyx_t_6) < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_DECREF_SET(__pyx_v_sigindex_node, __pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          if (unlikely(__pyx_v_sigindex_node == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(3, 728, __pyx_L1_error)
          }
          __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_sigindex_node, __pyx_v_sig_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 728, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_7 = __pyx_t_6;
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF_SET(__pyx_v_sigindex_node, ((PyObject*)__pyx_t_7));
          __pyx_t_7 = 0;
        }
        __pyx_L19:;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(__pyx_v_sigindex_node == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(3, 728, __pyx_L1_error)
      }
      if (unlikely((PyDict_SetItem(__pyx_v_sigindex_node, __pyx_v_last_type, __pyx_v_sig) < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely((__Pyx_SetItemInt(__pyx_v__fused_sigindex_ref, 0, __pyx_v_fused_sigindex, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1) < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
  }
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_sigindex_matches = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_fused_sigindex);
  __Pyx_GIVEREF(__pyx_v_fused_sigindex);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_v_fused_sigindex) != (0)) __PYX_ERR(3, 728, __pyx_L1_error);
  __pyx_v_sigindex_candidates = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_v_dest_sig; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_13 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
      #endif
      if (__pyx_t_13 >= __pyx_temp) break;
    }
    __pyx_t_11 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_13);
    ++__pyx_t_13;
    if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_11);
    __pyx_t_11 = 0;
    __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_found_matches, ((PyObject*)__pyx_t_11));
    __pyx_t_11 = 0;
    __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_XDECREF_SET(__pyx_v_found_candidates, ((PyObject*)__pyx_t_11));
    __pyx_t_11 = 0;
    __pyx_t_2 = (__pyx_v_dst_type == Py_None);
    if (__pyx_t_2) {
      __pyx_t_11 = __pyx_v_sigindex_matches; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        __pyx_t_7 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_5);
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_7);
        __pyx_t_7 = 0;
        if (unlikely(__pyx_v_sn == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
          __PYX_ERR(3, 728, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyDict_Values(((PyObject*)__pyx_v_sn)); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_17 = __Pyx_PyList_Extend(__pyx_v_found_matches, __pyx_t_7); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = __pyx_v_sigindex_candidates; __Pyx_INCREF(__pyx_t_11);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        __pyx_t_7 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_5);
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_7);
        __pyx_t_7 = 0;
        if (unlikely(__pyx_v_sn == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
          __PYX_ERR(3, 728, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyDict_Values(((PyObject*)__pyx_v_sn)); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_17 = __Pyx_PyList_Extend(__pyx_v_found_candidates, __pyx_t_7); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L23;
    }
    /*else*/ {
      __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_v_sigindex_matches);
      __Pyx_GIVEREF(__pyx_v_sigindex_matches);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_sigindex_matches) != (0)) __PYX_ERR(3, 728, __pyx_L1_error);
      __Pyx_INCREF(__pyx_v_sigindex_candidates);
      __Pyx_GIVEREF(__pyx_v_sigindex_candidates);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_v_sigindex_candidates) != (0)) __PYX_ERR(3, 728, __pyx_L1_error);
      __pyx_t_7 = __pyx_t_11; __Pyx_INCREF(__pyx_t_7);
      __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      for (;;) {
        if (__pyx_t_5 >= 2) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_11 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_5));
        #else
        __pyx_t_11 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_5);
        #endif
        ++__pyx_t_5;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 728, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_XDECREF_SET(__pyx_v_search_list, ((PyObject*)__pyx_t_11));
        __pyx_t_11 = 0;
        if (unlikely(__pyx_v_search_list == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(3, 728, __pyx_L1_error)
        }
        __pyx_t_11 = __pyx_v_search_list; __Pyx_INCREF(__pyx_t_11);
        __pyx_t_16 = 0;
        for (;;) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
            #endif
            if (__pyx_t_16 >= __pyx_temp) break;
          }
          __pyx_t_6 = __Pyx_PyList_GetItemRef(__pyx_t_11, __pyx_t_16);
          ++__pyx_t_16;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 728, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_sn, __pyx_t_6);
          __pyx_t_6 = 0;
          if (unlikely(__pyx_v_sn == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
            __PYX_ERR(3, 728, __pyx_L1_error)
          }
          __pyx_t_6 = __Pyx_PyDict_GetItemDefault(((PyObject*)__pyx_v_sn), __pyx_v_dst_type, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 728, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_type_match, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_2 = (__pyx_v_type_match != Py_None);
          if (__pyx_t_2) {
            __pyx_t_17 = __Pyx_PyList_Append(__pyx_v_found_matches, __pyx_v_type_match); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(3, 728, __pyx_L1_error)
          }
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __pyx_L23:;
    __Pyx_INCREF(__pyx_v_found_matches);
    __Pyx_DECREF_SET(__pyx_v_sigindex_matches, __pyx_v_found_matches);
    __Pyx_INCREF(__pyx_v_found_candidates);
    __Pyx_DECREF_SET(__pyx_v_sigindex_candidates, __pyx_v_found_candidates);
    __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_found_matches) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
    if (!__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L38_bool_binop_done;
    }
    __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_found_candidates) != 0);
    if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
    __pyx_t_2 = __pyx_t_4;
    __pyx_L38_bool_binop_done:;
    __pyx_t_4 = (!__pyx_t_2);
    if (__pyx_t_4) {
      goto __pyx_L22_break;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L40_for_end;
  __pyx_L22_break:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L40_for_end;
  __pyx_L40_for_end:;
  __Pyx_INCREF(__pyx_v_sigindex_matches);
  __pyx_v_candidates = __pyx_v_sigindex_matches;
  __pyx_t_4 = (__Pyx_PyList_GET_SIZE(__pyx_v_candidates) != 0);
  if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_4 < 0))) __PYX_ERR(3, 728, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_4);
  if (unlikely(__pyx_t_2)) {
    __pyx_t_7 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_11 = __pyx_builtin_TypeError; 
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_No_matching_signature_found};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 728, __pyx_L1_error)
  }
  __pyx_t_13 = __Pyx_PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(3, 728, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_13 > 1);
  if (unlikely(__pyx_t_2)) {
    __pyx_t_11 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_mstate_global->__pyx_kp_u_Function_call_with_ambiguous_arg};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 728, __pyx_L1_error)
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(((PyObject *)__pyx_v_signatures) == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 728, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), __Pyx_PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("selectolax.parser.Node.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_search_list);
  __Pyx_XDECREF(__pyx_v_sigindex_node);
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_fused_sigindex);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_sig_series);
  __Pyx_XDECREF(__pyx_v_last_type);
  __Pyx_XDECREF(__pyx_v_sig_type);
  __Pyx_XDECREF(__pyx_v_sigindex_matches);
  __Pyx_XDECREF(__pyx_v_sigindex_candidates);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_found_matches);
  __Pyx_XDECREF(__pyx_v_found_candidates);
  __Pyx_XDECREF(__pyx_v_sn);
  __Pyx_XDECREF(__pyx_v_type_match);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_75insert_child(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_10selectolax_6parser_4Node_75insert_child = {"__pyx_fuse_0insert_child", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_75insert_child, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_34insert_child};
static PyObject *__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_75insert_child(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert_child (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 728, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 728, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "insert_child", 0) < 0) __PYX_ERR(3, 728, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("insert_child", 1, 1, 1, i); __PYX_ERR(3, 728, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 728, __pyx_L3_error)
    }
    __pyx_v_value = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert_child", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 728, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.insert_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyUnicode_Type), 1, "value", 1))) __PYX_ERR(3, 728, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_74insert_child(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_74insert_child(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0insert_child", 0);

  /* "selectolax/modest/node.pxi":757
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_value); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_value); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":758
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_0__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 758, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":759
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":760
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 760, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 760, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 760, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 760, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":761
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    (void)(myhtml_node_append_child(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":757
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":762
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_append_child(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":763
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_append_child(self.node, node)
 *         else:
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 763, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_t_3));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":764
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_append_child(self.node, node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    (void)(myhtml_node_append_child(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":762
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_append_child(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":766
 *             myhtml_node_append_child(self.node, node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_value)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 766, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":728
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_child(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node inside (at the end of) the current Node.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.insert_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_77insert_child(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_10selectolax_6parser_4Node_77insert_child = {"__pyx_fuse_1insert_child", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_77insert_child, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_34insert_child};
static PyObject *__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_77insert_child(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert_child (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 728, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 728, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "insert_child", 0) < 0) __PYX_ERR(3, 728, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("insert_child", 1, 1, 1, i); __PYX_ERR(3, 728, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 728, __pyx_L3_error)
    }
    __pyx_v_value = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert_child", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 728, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.insert_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyBytes_Type), 1, "value", 1))) __PYX_ERR(3, 728, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_76insert_child(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_76insert_child(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1insert_child", 0);

  /* "selectolax/modest/node.pxi":757
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_value); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_value); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":758
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_1__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 758, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":759
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":760
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 760, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 760, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 760, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 760, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":761
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    (void)(myhtml_node_append_child(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":757
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":762
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_append_child(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":763
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_append_child(self.node, node)
 *         else:
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 763, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_t_3));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":764
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_append_child(self.node, node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    (void)(myhtml_node_append_child(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":762
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_append_child(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":766
 *             myhtml_node_append_child(self.node, node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_value)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 766, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":728
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_child(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node inside (at the end of) the current Node.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.insert_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_79insert_child(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2__pyx_mdef_10selectolax_6parser_4Node_79insert_child = {"__pyx_fuse_2insert_child", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_79insert_child, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_34insert_child};
static PyObject *__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_79insert_child(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert_child (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 728, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 728, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "insert_child", 0) < 0) __PYX_ERR(3, 728, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("insert_child", 1, 1, 1, i); __PYX_ERR(3, 728, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 728, __pyx_L3_error)
    }
    __pyx_v_value = ((struct __pyx_obj_10selectolax_6parser_Node *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert_child", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 728, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.insert_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, 1, "value", 0))) __PYX_ERR(3, 728, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_78insert_child(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_78insert_child(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value) {
  myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_bytes_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2insert_child", 0);

  /* "selectolax/modest/node.pxi":757
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
  __pyx_t_2 = PyUnicode_Check(((PyObject *)__pyx_v_value)); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(((PyObject *)__pyx_v_value)); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":758
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)             # <<<<<<<<<<<<<<
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
*/
    __pyx_t_3 = __pyx_fuse_2__pyx_f_10selectolax_6parser_to_bytes(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 758, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":759
 *         if isinstance(value, (str, bytes, unicode)):
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)             # <<<<<<<<<<<<<<
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)
*/
    __pyx_v_node = myhtml_node_create(__pyx_v_self->parser->html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML);

    /* "selectolax/modest/node.pxi":760
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):
*/
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(3, 760, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_bytes_val); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 760, __pyx_L1_error)
    if (unlikely(__pyx_v_bytes_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(3, 760, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_GET_SIZE(__pyx_v_bytes_val); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 760, __pyx_L1_error)
    (void)(myhtml_node_text_set(__pyx_v_node, ((char *)__pyx_t_4), __pyx_t_5, MyENCODING_UTF_8));

    /* "selectolax/modest/node.pxi":761
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
*/
    (void)(myhtml_node_append_child(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":757
 *         """
 *         cdef myhtml_tree_node_t *node
 *         if isinstance(value, (str, bytes, unicode)):             # <<<<<<<<<<<<<<
 *             bytes_val = to_bytes(value)
 *             node = myhtml_node_create(self.parser.html_tree, MyHTML_TAG__TEXT, MyHTML_NAMESPACE_HTML)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":762
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_append_child(self.node, node)
*/
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_value), __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/modest/node.pxi":763
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)             # <<<<<<<<<<<<<<
 *             myhtml_node_append_child(self.node, node)
 *         else:
*/
    __pyx_v_node = myhtml_node_clone_deep(__pyx_v_self->parser->html_tree, ((myhtml_tree_node_t *)__pyx_v_value->node));

    /* "selectolax/modest/node.pxi":764
 *         elif isinstance(value, Node):
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_append_child(self.node, node)             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
*/
    (void)(myhtml_node_append_child(__pyx_v_self->node, __pyx_v_node));

    /* "selectolax/modest/node.pxi":762
 *             myhtml_node_text_set(node, <char*> bytes_val, len(bytes_val), MyENCODING_UTF_8)
 *             myhtml_node_append_child(self.node, node)
 *         elif isinstance(value, Node):             # <<<<<<<<<<<<<<
 *             node = myhtml_node_clone_deep(self.parser.html_tree, <myhtml_tree_node_t *> value.node)
 *             myhtml_node_append_child(self.node, node)
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":766
 *             myhtml_node_append_child(self.node, node)
 *         else:
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)             # <<<<<<<<<<<<<<
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_7 = __pyx_builtin_TypeError; 
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_value))), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_or_Node_instan, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 766, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":728
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_child(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node inside (at the end of) the current Node.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("selectolax.parser.Node.insert_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":768
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):             # <<<<<<<<<<<<<<
 *         """Unwraps specified tags from the HTML tree.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_37unwrap_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_36unwrap_tags, "Node.unwrap_tags(self, list tags, delete_empty=False)\n\nUnwraps specified tags from the HTML tree.\n\nWorks the same as th ``unwrap`` method, but applied to a list of tags.\n\nParameters\n----------\ntags : list\n    List of tags to remove.\ndelete_empty : bool, default False\n    Whenever to delete empty tags.\n\nExamples\n--------\n\n>>> tree = HTMLParser(\"<div><a href=\"\">Hello</a> <i>world</i>!</div>\")\n>>> tree.body.unwrap_tags(['i','a'])\n>>> tree.body.html\n'<body><div>Hello world!</div></body>'\n\nNote: by default, empty tags are ignored, set \"delete_empty\" to \"True\" to change this.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_37unwrap_tags = {"unwrap_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_37unwrap_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_36unwrap_tags};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_37unwrap_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_tags = 0;
  PyObject *__pyx_v_delete_empty = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unwrap_tags (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_tags,&__pyx_mstate_global->__pyx_n_u_delete_empty,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 768, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 768, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 768, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "unwrap_tags", 0) < 0) __PYX_ERR(3, 768, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("unwrap_tags", 0, 1, 2, i); __PYX_ERR(3, 768, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 768, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 768, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_False));
    }
    __pyx_v_tags = ((PyObject*)values[0]);
    __pyx_v_delete_empty = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("unwrap_tags", 0, 1, 2, __pyx_nargs); __PYX_ERR(3, 768, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.unwrap_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tags), (&PyList_Type), 1, "tags", 1))) __PYX_ERR(3, 768, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_36unwrap_tags(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_tags, __pyx_v_delete_empty);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_36unwrap_tags(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_tags, PyObject *__pyx_v_delete_empty) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_element = 0;
  PyObject *__pyx_v_tag = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("unwrap_tags", 0);

  /* "selectolax/modest/node.pxi":791
 *         """
 *         cdef Node element
 *         for tag in tags:             # <<<<<<<<<<<<<<
 *             for element in self.css(tag):
 *                 element.unwrap(delete_empty)
*/
  if (unlikely(__pyx_v_tags == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(3, 791, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_tags; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 791, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_2);
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 791, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_tag, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":792
 *         cdef Node element
 *         for tag in tags:
 *             for element in self.css(tag):             # <<<<<<<<<<<<<<
 *                 element.unwrap(delete_empty)
 * 
*/
    __pyx_t_4 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_tag};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_css, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 792, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4);
      __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 792, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 792, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 792, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_4, __pyx_t_6);
          ++__pyx_t_6;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 792, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_6));
          #else
          __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_6);
          #endif
          ++__pyx_t_6;
        }
        if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 792, __pyx_L1_error)
      } else {
        __pyx_t_3 = __pyx_t_7(__pyx_t_4);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(3, 792, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(3, 792, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_element, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "selectolax/modest/node.pxi":793
 *         for tag in tags:
 *             for element in self.css(tag):
 *                 element.unwrap(delete_empty)             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
      __pyx_t_8 = ((PyObject *)__pyx_v_element);
      __Pyx_INCREF(__pyx_t_8);
      __pyx_t_5 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_v_delete_empty};
        __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_unwrap, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 793, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "selectolax/modest/node.pxi":792
 *         cdef Node element
 *         for tag in tags:
 *             for element in self.css(tag):             # <<<<<<<<<<<<<<
 *                 element.unwrap(delete_empty)
 * 
*/
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/modest/node.pxi":791
 *         """
 *         cdef Node element
 *         for tag in tags:             # <<<<<<<<<<<<<<
 *             for element in self.css(tag):
 *                 element.unwrap(delete_empty)
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":768
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):             # <<<<<<<<<<<<<<
 *         """Unwraps specified tags from the HTML tree.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("selectolax.parser.Node.unwrap_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_element);
  __Pyx_XDECREF(__pyx_v_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":795
 *                 element.unwrap(delete_empty)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def raw_value(self):
 *         """Return the raw (unparsed, original) value of a node.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_9raw_value_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_9raw_value_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_9raw_value___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_9raw_value___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  int __pyx_v_begin;
  int __pyx_v_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":816
 *         b'&#x3C;test&#x3E;'
 *         """
 *         cdef int begin = self.node.token.element_begin             # <<<<<<<<<<<<<<
 *         cdef int length = self.node.token.element_length
 *         if self.node.tag_id != MyHTML_TAG__TEXT:
*/
  __pyx_t_1 = __pyx_v_self->node->token->element_begin;
  __pyx_v_begin = __pyx_t_1;

  /* "selectolax/modest/node.pxi":817
 *         """
 *         cdef int begin = self.node.token.element_begin
 *         cdef int length = self.node.token.element_length             # <<<<<<<<<<<<<<
 *         if self.node.tag_id != MyHTML_TAG__TEXT:
 *             raise ValueError("Can't obtain raw value for non-text node.")
*/
  __pyx_t_1 = __pyx_v_self->node->token->element_length;
  __pyx_v_length = __pyx_t_1;

  /* "selectolax/modest/node.pxi":818
 *         cdef int begin = self.node.token.element_begin
 *         cdef int length = self.node.token.element_length
 *         if self.node.tag_id != MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *             raise ValueError("Can't obtain raw value for non-text node.")
 *         return self.parser.raw_html[begin:begin + length]
*/
  __pyx_t_2 = (__pyx_v_self->node->tag_id != MyHTML_TAG__TEXT);
  if (unlikely(__pyx_t_2)) {

    /* "selectolax/modest/node.pxi":819
 *         cdef int length = self.node.token.element_length
 *         if self.node.tag_id != MyHTML_TAG__TEXT:
 *             raise ValueError("Can't obtain raw value for non-text node.")             # <<<<<<<<<<<<<<
 *         return self.parser.raw_html[begin:begin + length]
 * 
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_builtin_ValueError);
    __pyx_t_5 = __pyx_builtin_ValueError; 
    __pyx_t_1 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Can_t_obtain_raw_value_for_non_t};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_1, (2-__pyx_t_1) | (__pyx_t_1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 819, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 819, __pyx_L1_error)

    /* "selectolax/modest/node.pxi":818
 *         cdef int begin = self.node.token.element_begin
 *         cdef int length = self.node.token.element_length
 *         if self.node.tag_id != MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *             raise ValueError("Can't obtain raw value for non-text node.")
 *         return self.parser.raw_html[begin:begin + length]
*/
  }

  /* "selectolax/modest/node.pxi":820
 *         if self.node.tag_id != MyHTML_TAG__TEXT:
 *             raise ValueError("Can't obtain raw value for non-text node.")
 *         return self.parser.raw_html[begin:begin + length]             # <<<<<<<<<<<<<<
 * 
 *     def select(self, query=None):
*/
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->parser->raw_html == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(3, 820, __pyx_L1_error)
  }
  __pyx_t_3 = PySequence_GetSlice(__pyx_v_self->parser->raw_html, __pyx_v_begin, (__pyx_v_begin + __pyx_v_length)); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":795
 *                 element.unwrap(delete_empty)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def raw_value(self):
 *         """Return the raw (unparsed, original) value of a node.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.Node.raw_value.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":822
 *         return self.parser.raw_html[begin:begin + length]
 * 
 *     def select(self, query=None):             # <<<<<<<<<<<<<<
 *         """Select nodes given a CSS selector.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_39select(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_38select, "Node.select(self, query=None)\n\nSelect nodes given a CSS selector.\n\nWorks similarly to the ``css`` method, but supports chained filtering and extra features.\n\nParameters\n----------\nquery : str or None\n    The CSS selector to use when searching for nodes.\n\nReturns\n-------\nselector : The `Selector` class.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_39select = {"select", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_39select, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_38select};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_39select(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_query = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("select (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_query,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 822, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 822, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "select", 0) < 0) __PYX_ERR(3, 822, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 822, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_query = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("select", 0, 0, 1, __pyx_nargs); __PYX_ERR(3, 822, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.select", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_38select(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_query);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_38select(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_query) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("select", 0);

  /* "selectolax/modest/node.pxi":836
 *         selector : The `Selector` class.
 *         """
 *         return Selector(self, query)             # <<<<<<<<<<<<<<
 * 
 *     def scripts_contain(self, str query):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector);
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector); 
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, ((PyObject *)__pyx_v_self), __pyx_v_query};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 836, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":822
 *         return self.parser.raw_html[begin:begin + length]
 * 
 *     def select(self, query=None):             # <<<<<<<<<<<<<<
 *         """Select nodes given a CSS selector.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Node.select", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":838
 *         return Selector(self, query)
 * 
 *     def scripts_contain(self, str query):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script tags contain specified text.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_41scripts_contain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_40scripts_contain, "Node.scripts_contain(self, str query)\n\nReturns True if any of the script tags contain specified text.\n\nCaches script tags on the first call to improve performance.\n\nParameters\n----------\nquery : str\n    The query to check.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_41scripts_contain = {"scripts_contain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_41scripts_contain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_40scripts_contain};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_41scripts_contain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_query = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("scripts_contain (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_query,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 838, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 838, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "scripts_contain", 0) < 0) __PYX_ERR(3, 838, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("scripts_contain", 1, 1, 1, i); __PYX_ERR(3, 838, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 838, __pyx_L3_error)
    }
    __pyx_v_query = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("scripts_contain", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 838, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.scripts_contain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query), (&PyUnicode_Type), 1, "query", 1))) __PYX_ERR(3, 838, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_40scripts_contain(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_query);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_40scripts_contain(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_query) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_v_nodes = NULL;
  PyObject *__pyx_v_text_nodes = NULL;
  PyObject *__pyx_v_node_text = NULL;
  PyObject *__pyx_v_text = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("scripts_contain", 0);

  /* "selectolax/modest/node.pxi":850
 *         """
 *         cdef Node node
 *         if self.parser.cached_script_texts is None:             # <<<<<<<<<<<<<<
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             text_nodes = []
*/
  __pyx_t_1 = (__pyx_v_self->parser->cached_script_texts == Py_None);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":851
 *         cdef Node node
 *         if self.parser.cached_script_texts is None:
 *             nodes = find_nodes(self.parser, self.node, 'script')             # <<<<<<<<<<<<<<
 *             text_nodes = []
 *             for node in nodes:
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_self->parser);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = __pyx_f_10selectolax_6parser_find_nodes(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_2), __pyx_v_self->node, __pyx_mstate_global->__pyx_n_u_script); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 851, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_nodes = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":852
 *         if self.parser.cached_script_texts is None:
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             text_nodes = []             # <<<<<<<<<<<<<<
 *             for node in nodes:
 *                 node_text = node.text(deep=True)
*/
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 852, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_text_nodes = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":853
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             text_nodes = []
 *             for node in nodes:             # <<<<<<<<<<<<<<
 *                 node_text = node.text(deep=True)
 *                 if node_text:
*/
    if (likely(PyList_CheckExact(__pyx_v_nodes)) || PyTuple_CheckExact(__pyx_v_nodes)) {
      __pyx_t_3 = __pyx_v_nodes; __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = 0;
      __pyx_t_5 = NULL;
    } else {
      __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_nodes); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 853, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_5)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 853, __pyx_L1_error)
            #endif
            if (__pyx_t_4 >= __pyx_temp) break;
          }
          __pyx_t_2 = __Pyx_PyList_GetItemRef(__pyx_t_3, __pyx_t_4);
          ++__pyx_t_4;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_3);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 853, __pyx_L1_error)
            #endif
            if (__pyx_t_4 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4));
          #else
          __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_4);
          #endif
          ++__pyx_t_4;
        }
        if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 853, __pyx_L1_error)
      } else {
        __pyx_t_2 = __pyx_t_5(__pyx_t_3);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(3, 853, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(3, 853, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_node, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "selectolax/modest/node.pxi":854
 *             text_nodes = []
 *             for node in nodes:
 *                 node_text = node.text(deep=True)             # <<<<<<<<<<<<<<
 *                 if node_text:
 *                     text_nodes.append(node_text)
*/
      __pyx_t_6 = ((PyObject *)__pyx_v_node);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_7 = 0;
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_6, NULL};
        __pyx_t_8 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 854, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_deep, Py_True, __pyx_t_8, __pyx_callargs+1, 0) < 0) __PYX_ERR(3, 854, __pyx_L1_error)
        __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_text, __pyx_callargs+__pyx_t_7, (1-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_8);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 854, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_node_text, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "selectolax/modest/node.pxi":855
 *             for node in nodes:
 *                 node_text = node.text(deep=True)
 *                 if node_text:             # <<<<<<<<<<<<<<
 *                     text_nodes.append(node_text)
 *             self.parser.cached_script_texts = text_nodes
*/
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_node_text); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(3, 855, __pyx_L1_error)
      if (__pyx_t_1) {

        /* "selectolax/modest/node.pxi":856
 *                 node_text = node.text(deep=True)
 *                 if node_text:
 *                     text_nodes.append(node_text)             # <<<<<<<<<<<<<<
 *             self.parser.cached_script_texts = text_nodes
 * 
*/
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_text_nodes, __pyx_v_node_text); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 856, __pyx_L1_error)

        /* "selectolax/modest/node.pxi":855
 *             for node in nodes:
 *                 node_text = node.text(deep=True)
 *                 if node_text:             # <<<<<<<<<<<<<<
 *                     text_nodes.append(node_text)
 *             self.parser.cached_script_texts = text_nodes
*/
      }

      /* "selectolax/modest/node.pxi":853
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             text_nodes = []
 *             for node in nodes:             # <<<<<<<<<<<<<<
 *                 node_text = node.text(deep=True)
 *                 if node_text:
*/
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":857
 *                 if node_text:
 *                     text_nodes.append(node_text)
 *             self.parser.cached_script_texts = text_nodes             # <<<<<<<<<<<<<<
 * 
 *         for text in self.parser.cached_script_texts:
*/
    __Pyx_INCREF(__pyx_v_text_nodes);
    __Pyx_GIVEREF(__pyx_v_text_nodes);
    __Pyx_GOTREF(__pyx_v_self->parser->cached_script_texts);
    __Pyx_DECREF(__pyx_v_self->parser->cached_script_texts);
    __pyx_v_self->parser->cached_script_texts = __pyx_v_text_nodes;

    /* "selectolax/modest/node.pxi":850
 *         """
 *         cdef Node node
 *         if self.parser.cached_script_texts is None:             # <<<<<<<<<<<<<<
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             text_nodes = []
*/
  }

  /* "selectolax/modest/node.pxi":859
 *             self.parser.cached_script_texts = text_nodes
 * 
 *         for text in self.parser.cached_script_texts:             # <<<<<<<<<<<<<<
 *             if query in text:
 *                 return True
*/
  if (likely(PyList_CheckExact(__pyx_v_self->parser->cached_script_texts)) || PyTuple_CheckExact(__pyx_v_self->parser->cached_script_texts)) {
    __pyx_t_3 = __pyx_v_self->parser->cached_script_texts; __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_self->parser->cached_script_texts); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 859, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 859, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 859, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        __pyx_t_2 = __Pyx_PyList_GetItemRef(__pyx_t_3, __pyx_t_4);
        ++__pyx_t_4;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_3);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 859, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4));
        #else
        __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_4);
        #endif
        ++__pyx_t_4;
      }
      if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 859, __pyx_L1_error)
    } else {
      __pyx_t_2 = __pyx_t_5(__pyx_t_3);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(3, 859, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_text, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "selectolax/modest/node.pxi":860
 * 
 *         for text in self.parser.cached_script_texts:
 *             if query in text:             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
*/
    __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_v_query, __pyx_v_text, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(3, 860, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "selectolax/modest/node.pxi":861
 *         for text in self.parser.cached_script_texts:
 *             if query in text:
 *                 return True             # <<<<<<<<<<<<<<
 *         return False
 * 
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = Py_True;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      goto __pyx_L0;

      /* "selectolax/modest/node.pxi":860
 * 
 *         for text in self.parser.cached_script_texts:
 *             if query in text:             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
*/
    }

    /* "selectolax/modest/node.pxi":859
 *             self.parser.cached_script_texts = text_nodes
 * 
 *         for text in self.parser.cached_script_texts:             # <<<<<<<<<<<<<<
 *             if query in text:
 *                 return True
*/
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "selectolax/modest/node.pxi":862
 *             if query in text:
 *                 return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     def script_srcs_contain(self, tuple queries):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":838
 *         return Selector(self, query)
 * 
 *     def scripts_contain(self, str query):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script tags contain specified text.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("selectolax.parser.Node.scripts_contain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XDECREF(__pyx_v_nodes);
  __Pyx_XDECREF(__pyx_v_text_nodes);
  __Pyx_XDECREF(__pyx_v_node_text);
  __Pyx_XDECREF(__pyx_v_text);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":864
 *         return False
 * 
 *     def script_srcs_contain(self, tuple queries):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script SRCs attributes contain on of the specified text.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_43script_srcs_contain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_42script_srcs_contain, "Node.script_srcs_contain(self, tuple queries)\n\nReturns True if any of the script SRCs attributes contain on of the specified text.\n\nCaches values on the first call to improve performance.\n\nParameters\n----------\nqueries : tuple of str");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_43script_srcs_contain = {"script_srcs_contain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_43script_srcs_contain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_42script_srcs_contain};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_43script_srcs_contain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_queries = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("script_srcs_contain (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_queries,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(3, 864, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 864, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "script_srcs_contain", 0) < 0) __PYX_ERR(3, 864, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("script_srcs_contain", 1, 1, 1, i); __PYX_ERR(3, 864, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 864, __pyx_L3_error)
    }
    __pyx_v_queries = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("script_srcs_contain", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 864, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.script_srcs_contain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_queries), (&PyTuple_Type), 1, "queries", 1))) __PYX_ERR(3, 864, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_42script_srcs_contain(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v_queries);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_42script_srcs_contain(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_queries) {
  PyObject *__pyx_v_nodes = NULL;
  PyObject *__pyx_v_src_nodes = NULL;
  PyObject *__pyx_v_node = NULL;
  PyObject *__pyx_v_node_src = NULL;
  PyObject *__pyx_v_text = NULL;
  PyObject *__pyx_v_query = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("script_srcs_contain", 0);

  /* "selectolax/modest/node.pxi":874
 * 
 *         """
 *         if self.parser.cached_script_srcs is None:             # <<<<<<<<<<<<<<
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             src_nodes = []
*/
  __pyx_t_1 = (__pyx_v_self->parser->cached_script_srcs == Py_None);
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":875
 *         """
 *         if self.parser.cached_script_srcs is None:
 *             nodes = find_nodes(self.parser, self.node, 'script')             # <<<<<<<<<<<<<<
 *             src_nodes = []
 *             for node in nodes:
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_self->parser);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = __pyx_f_10selectolax_6parser_find_nodes(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_2), __pyx_v_self->node, __pyx_mstate_global->__pyx_n_u_script); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 875, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_nodes = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":876
 *         if self.parser.cached_script_srcs is None:
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             src_nodes = []             # <<<<<<<<<<<<<<
 *             for node in nodes:
 *                 node_src = node.attrs.get('src')
*/
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 876, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_src_nodes = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":877
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             src_nodes = []
 *             for node in nodes:             # <<<<<<<<<<<<<<
 *                 node_src = node.attrs.get('src')
 *                 if node_src:
*/
    if (likely(PyList_CheckExact(__pyx_v_nodes)) || PyTuple_CheckExact(__pyx_v_nodes)) {
      __pyx_t_3 = __pyx_v_nodes; __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = 0;
      __pyx_t_5 = NULL;
    } else {
      __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_nodes); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 877, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 877, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_5)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 877, __pyx_L1_error)
            #endif
            if (__pyx_t_4 >= __pyx_temp) break;
          }
          __pyx_t_2 = __Pyx_PyList_GetItemRef(__pyx_t_3, __pyx_t_4);
          ++__pyx_t_4;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_3);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 877, __pyx_L1_error)
            #endif
            if (__pyx_t_4 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4));
          #else
          __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_4);
          #endif
          ++__pyx_t_4;
        }
        if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 877, __pyx_L1_error)
      } else {
        __pyx_t_2 = __pyx_t_5(__pyx_t_3);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(3, 877, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_node, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "selectolax/modest/node.pxi":878
 *             src_nodes = []
 *             for node in nodes:
 *                 node_src = node.attrs.get('src')             # <<<<<<<<<<<<<<
 *                 if node_src:
 *                     src_nodes.append(node_src)
*/
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_mstate_global->__pyx_n_u_attrs); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 878, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_6 = __pyx_t_7;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_8 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_n_u_src};
        __pyx_t_2 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_get, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 878, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_node_src, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "selectolax/modest/node.pxi":879
 *             for node in nodes:
 *                 node_src = node.attrs.get('src')
 *                 if node_src:             # <<<<<<<<<<<<<<
 *                     src_nodes.append(node_src)
 *             self.parser.cached_script_srcs = src_nodes
*/
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_node_src); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(3, 879, __pyx_L1_error)
      if (__pyx_t_1) {

        /* "selectolax/modest/node.pxi":880
 *                 node_src = node.attrs.get('src')
 *                 if node_src:
 *                     src_nodes.append(node_src)             # <<<<<<<<<<<<<<
 *             self.parser.cached_script_srcs = src_nodes
 * 
*/
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_src_nodes, __pyx_v_node_src); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 880, __pyx_L1_error)

        /* "selectolax/modest/node.pxi":879
 *             for node in nodes:
 *                 node_src = node.attrs.get('src')
 *                 if node_src:             # <<<<<<<<<<<<<<
 *                     src_nodes.append(node_src)
 *             self.parser.cached_script_srcs = src_nodes
*/
      }

      /* "selectolax/modest/node.pxi":877
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             src_nodes = []
 *             for node in nodes:             # <<<<<<<<<<<<<<
 *                 node_src = node.attrs.get('src')
 *                 if node_src:
*/
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":881
 *                 if node_src:
 *                     src_nodes.append(node_src)
 *             self.parser.cached_script_srcs = src_nodes             # <<<<<<<<<<<<<<
 * 
 *         for text in self.parser.cached_script_srcs:
*/
    __Pyx_INCREF(__pyx_v_src_nodes);
    __Pyx_GIVEREF(__pyx_v_src_nodes);
    __Pyx_GOTREF(__pyx_v_self->parser->cached_script_srcs);
    __Pyx_DECREF(__pyx_v_self->parser->cached_script_srcs);
    __pyx_v_self->parser->cached_script_srcs = __pyx_v_src_nodes;

    /* "selectolax/modest/node.pxi":874
 * 
 *         """
 *         if self.parser.cached_script_srcs is None:             # <<<<<<<<<<<<<<
 *             nodes = find_nodes(self.parser, self.node, 'script')
 *             src_nodes = []
*/
  }

  /* "selectolax/modest/node.pxi":883
 *             self.parser.cached_script_srcs = src_nodes
 * 
 *         for text in self.parser.cached_script_srcs:             # <<<<<<<<<<<<<<
 *             for query in queries:
 *                 if query in text:
*/
  if (likely(PyList_CheckExact(__pyx_v_self->parser->cached_script_srcs)) || PyTuple_CheckExact(__pyx_v_self->parser->cached_script_srcs)) {
    __pyx_t_3 = __pyx_v_self->parser->cached_script_srcs; __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_self->parser->cached_script_srcs); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 883, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 883, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 883, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        __pyx_t_2 = __Pyx_PyList_GetItemRef(__pyx_t_3, __pyx_t_4);
        ++__pyx_t_4;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_3);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 883, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4));
        #else
        __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_4);
        #endif
        ++__pyx_t_4;
      }
      if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 883, __pyx_L1_error)
    } else {
      __pyx_t_2 = __pyx_t_5(__pyx_t_3);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(3, 883, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_text, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "selectolax/modest/node.pxi":884
 * 
 *         for text in self.parser.cached_script_srcs:
 *             for query in queries:             # <<<<<<<<<<<<<<
 *                 if query in text:
 *                     return True
*/
    if (unlikely(__pyx_v_queries == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(3, 884, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_v_queries; __Pyx_INCREF(__pyx_t_2);
    __pyx_t_10 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(3, 884, __pyx_L1_error)
        #endif
        if (__pyx_t_10 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_7 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_10));
      #else
      __pyx_t_7 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_10);
      #endif
      ++__pyx_t_10;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 884, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_XDECREF_SET(__pyx_v_query, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "selectolax/modest/node.pxi":885
 *         for text in self.parser.cached_script_srcs:
 *             for query in queries:
 *                 if query in text:             # <<<<<<<<<<<<<<
 *                     return True
 *         return False
*/
      __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_v_query, __pyx_v_text, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(3, 885, __pyx_L1_error)
      if (__pyx_t_1) {

        /* "selectolax/modest/node.pxi":886
 *             for query in queries:
 *                 if query in text:
 *                     return True             # <<<<<<<<<<<<<<
 *         return False
 * 
*/
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(Py_True);
        __pyx_r = Py_True;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        goto __pyx_L0;

        /* "selectolax/modest/node.pxi":885
 *         for text in self.parser.cached_script_srcs:
 *             for query in queries:
 *                 if query in text:             # <<<<<<<<<<<<<<
 *                     return True
 *         return False
*/
      }

      /* "selectolax/modest/node.pxi":884
 * 
 *         for text in self.parser.cached_script_srcs:
 *             for query in queries:             # <<<<<<<<<<<<<<
 *                 if query in text:
 *                     return True
*/
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "selectolax/modest/node.pxi":883
 *             self.parser.cached_script_srcs = src_nodes
 * 
 *         for text in self.parser.cached_script_srcs:             # <<<<<<<<<<<<<<
 *             for query in queries:
 *                 if query in text:
*/
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "selectolax/modest/node.pxi":887
 *                 if query in text:
 *                     return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":864
 *         return False
 * 
 *     def script_srcs_contain(self, tuple queries):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script SRCs attributes contain on of the specified text.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("selectolax.parser.Node.script_srcs_contain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_nodes);
  __Pyx_XDECREF(__pyx_v_src_nodes);
  __Pyx_XDECREF(__pyx_v_node);
  __Pyx_XDECREF(__pyx_v_node_src);
  __Pyx_XDECREF(__pyx_v_text);
  __Pyx_XDECREF(__pyx_v_query);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":889
 *         return False
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<Node %s>' % self.tag
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_45__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_45__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_44__repr__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_44__repr__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "selectolax/modest/node.pxi":890
 * 
 *     def __repr__(self):
 *         return '<Node %s>' % self.tag             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, other):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_tag); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Node_s, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":889
 *         return False
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<Node %s>' % self.tag
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.Node.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":892
 *         return '<Node %s>' % self.tag
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, str):
 *             return self.html == other
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_47__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_47__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_46__eq__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_46__eq__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "selectolax/modest/node.pxi":893
 * 
 *     def __eq__(self, other):
 *         if isinstance(other, str):             # <<<<<<<<<<<<<<
 *             return self.html == other
 *         if not isinstance(other, Node):
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_other); 
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":894
 *     def __eq__(self, other):
 *         if isinstance(other, str):
 *             return self.html == other             # <<<<<<<<<<<<<<
 *         if not isinstance(other, Node):
 *             return False
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_html); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_v_other, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 894, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":893
 * 
 *     def __eq__(self, other):
 *         if isinstance(other, str):             # <<<<<<<<<<<<<<
 *             return self.html == other
 *         if not isinstance(other, Node):
*/
  }

  /* "selectolax/modest/node.pxi":895
 *         if isinstance(other, str):
 *             return self.html == other
 *         if not isinstance(other, Node):             # <<<<<<<<<<<<<<
 *             return False
 *         return self.html == other.html
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node); 
  __pyx_t_4 = (!__pyx_t_1);
  if (__pyx_t_4) {

    /* "selectolax/modest/node.pxi":896
 *             return self.html == other
 *         if not isinstance(other, Node):
 *             return False             # <<<<<<<<<<<<<<
 *         return self.html == other.html
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "selectolax/modest/node.pxi":895
 *         if isinstance(other, str):
 *             return self.html == other
 *         if not isinstance(other, Node):             # <<<<<<<<<<<<<<
 *             return False
 *         return self.html == other.html
*/
  }

  /* "selectolax/modest/node.pxi":897
 *         if not isinstance(other, Node):
 *             return False
 *         return self.html == other.html             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_html); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_mstate_global->__pyx_n_u_html); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 897, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":892
 *         return '<Node %s>' % self.tag
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, str):
 *             return self.html == other
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.Node.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":899
 *         return self.html == other.html
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def text_content(self):
 *         """Returns the text of the node if it is a text node.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_12text_content_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_12text_content_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_12text_content___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_12text_content___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  PyObject *__pyx_v_text = NULL;
  char const *__pyx_v_c_text;
  CYTHON_UNUSED myhtml_tree_node_t *__pyx_v_node;
  PyObject *__pyx_v_node_text = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  myhtml_tree_node_t *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/modest/node.pxi":910
 *         text : str or None.
 *         """
 *         text = ""             # <<<<<<<<<<<<<<
 *         cdef const char* c_text
 *         cdef myhtml_tree_node_t *node = self.node.child
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
  __pyx_v_text = __pyx_mstate_global->__pyx_kp_u_;

  /* "selectolax/modest/node.pxi":912
 *         text = ""
 *         cdef const char* c_text
 *         cdef myhtml_tree_node_t *node = self.node.child             # <<<<<<<<<<<<<<
 * 
 *         if self.node.tag_id == MyHTML_TAG__TEXT:
*/
  __pyx_t_1 = __pyx_v_self->node->child;
  __pyx_v_node = __pyx_t_1;

  /* "selectolax/modest/node.pxi":914
 *         cdef myhtml_tree_node_t *node = self.node.child
 * 
 *         if self.node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *             c_text = myhtml_node_text(self.node, NULL)
 *             if c_text != NULL:
*/
  __pyx_t_2 = (__pyx_v_self->node->tag_id == MyHTML_TAG__TEXT);
  if (__pyx_t_2) {

    /* "selectolax/modest/node.pxi":915
 * 
 *         if self.node.tag_id == MyHTML_TAG__TEXT:
 *             c_text = myhtml_node_text(self.node, NULL)             # <<<<<<<<<<<<<<
 *             if c_text != NULL:
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
*/
    __pyx_v_c_text = myhtml_node_text(__pyx_v_self->node, NULL);

    /* "selectolax/modest/node.pxi":916
 *         if self.node.tag_id == MyHTML_TAG__TEXT:
 *             c_text = myhtml_node_text(self.node, NULL)
 *             if c_text != NULL:             # <<<<<<<<<<<<<<
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                 return append_text(text, node_text)
*/
    __pyx_t_2 = (__pyx_v_c_text != NULL);
    if (__pyx_t_2) {

      /* "selectolax/modest/node.pxi":917
 *             c_text = myhtml_node_text(self.node, NULL)
 *             if c_text != NULL:
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)             # <<<<<<<<<<<<<<
 *                 return append_text(text, node_text)
 *         return None
*/
      __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_c_text); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 917, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = __pyx_t_5;
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_UTF_8, __pyx_v_self->parser->decode_errors};
        __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 917, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __pyx_v_node_text = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "selectolax/modest/node.pxi":918
 *             if c_text != NULL:
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                 return append_text(text, node_text)             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
      __Pyx_XDECREF(__pyx_r);
      if (!(likely(PyUnicode_CheckExact(__pyx_v_node_text)) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_node_text))) __PYX_ERR(3, 918, __pyx_L1_error)
      __pyx_t_3 = __pyx_f_10selectolax_6parser_append_text(__pyx_v_text, ((PyObject*)__pyx_v_node_text), NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_r = __pyx_t_3;
      __pyx_t_3 = 0;
      goto __pyx_L0;

      /* "selectolax/modest/node.pxi":916
 *         if self.node.tag_id == MyHTML_TAG__TEXT:
 *             c_text = myhtml_node_text(self.node, NULL)
 *             if c_text != NULL:             # <<<<<<<<<<<<<<
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                 return append_text(text, node_text)
*/
    }

    /* "selectolax/modest/node.pxi":914
 *         cdef myhtml_tree_node_t *node = self.node.child
 * 
 *         if self.node.tag_id == MyHTML_TAG__TEXT:             # <<<<<<<<<<<<<<
 *             c_text = myhtml_node_text(self.node, NULL)
 *             if c_text != NULL:
*/
  }

  /* "selectolax/modest/node.pxi":919
 *                 node_text = c_text.decode(_ENCODING, self.parser.decode_errors)
 *                 return append_text(text, node_text)
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     def merge_text_nodes(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":899
 *         return self.html == other.html
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def text_content(self):
 *         """Returns the text of the node if it is a text node.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.Node.text_content.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_text);
  __Pyx_XDECREF(__pyx_v_node_text);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":921
 *         return None
 * 
 *     def merge_text_nodes(self):             # <<<<<<<<<<<<<<
 *         """Iterates over all text nodes and merges all text nodes that are close to each other.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_49merge_text_nodes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_48merge_text_nodes, "Node.merge_text_nodes(self)\n\nIterates over all text nodes and merges all text nodes that are close to each other.\n\nThis is useful for text extraction.\nUse it when you need to strip HTML tags and merge \"dangling\" text.\n\nExamples\n--------\n\n>>> tree = HTMLParser(\"<div><p><strong>J</strong>ohn</p><p>Doe</p></div>\")\n>>> node = tree.css_first('div')\n>>> tree.unwrap_tags([\"strong\"])\n>>> tree.text(deep=True, separator=\" \", strip=True)\n\"J ohn Doe\" # Text extraction produces an extra space because the strong tag was removed.\n>>> node.merge_text_nodes()\n>>> tree.text(deep=True, separator=\" \", strip=True)\n\"John Doe\"");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_49merge_text_nodes = {"merge_text_nodes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_49merge_text_nodes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_48merge_text_nodes};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_49merge_text_nodes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("merge_text_nodes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("merge_text_nodes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("merge_text_nodes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_48merge_text_nodes(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_48merge_text_nodes(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser_Stack *__pyx_v_stack = 0;
  myhtml_tree_node_t *__pyx_v_current_node;
  char const *__pyx_v_left_text;
  char const *__pyx_v_right_text;
  char *__pyx_v_final_text;
  size_t __pyx_v_left_length;
  size_t __pyx_v_right_length;
  size_t __pyx_v_final_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  myhtml_tree_node_t *__pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("merge_text_nodes", 0);

  /* "selectolax/modest/node.pxi":939
 *         "John Doe"
 *         """
 *         cdef Stack stack = Stack(_STACK_SIZE)             # <<<<<<<<<<<<<<
 *         cdef myhtml_tree_node_t * current_node = NULL
 *         cdef Node next_node
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Stack);
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Stack); 
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_int_100};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 939, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_stack = ((struct __pyx_obj_10selectolax_6parser_Stack *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/modest/node.pxi":940
 *         """
 *         cdef Stack stack = Stack(_STACK_SIZE)
 *         cdef myhtml_tree_node_t * current_node = NULL             # <<<<<<<<<<<<<<
 *         cdef Node next_node
 *         cdef const char* left_text
*/
  __pyx_v_current_node = NULL;

  /* "selectolax/modest/node.pxi":947
 *         cdef size_t left_length, right_length, final_length
 * 
 *         stack.push(self.node)             # <<<<<<<<<<<<<<
 * 
 *         while not stack.is_empty():
*/
  __pyx_t_5 = __pyx_f_10selectolax_6parser_5Stack_push(__pyx_v_stack, __pyx_v_self->node); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(3, 947, __pyx_L1_error)

  /* "selectolax/modest/node.pxi":949
 *         stack.push(self.node)
 * 
 *         while not stack.is_empty():             # <<<<<<<<<<<<<<
 *             current_node = stack.pop()
 * 
*/
  while (1) {
    __pyx_t_6 = __pyx_f_10selectolax_6parser_5Stack_is_empty(__pyx_v_stack); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 949, __pyx_L1_error)
    __pyx_t_7 = (!__pyx_t_6);
    if (!__pyx_t_7) break;

    /* "selectolax/modest/node.pxi":950
 * 
 *         while not stack.is_empty():
 *             current_node = stack.pop()             # <<<<<<<<<<<<<<
 * 
 *             if (current_node.tag_id == MyHTML_TAG__TEXT and current_node.prev and
*/
    __pyx_t_8 = __pyx_f_10selectolax_6parser_5Stack_pop(__pyx_v_stack); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 950, __pyx_L1_error)
    __pyx_v_current_node = __pyx_t_8;

    /* "selectolax/modest/node.pxi":952
 *             current_node = stack.pop()
 * 
 *             if (current_node.tag_id == MyHTML_TAG__TEXT and current_node.prev and             # <<<<<<<<<<<<<<
 *                     current_node.prev.tag_id == MyHTML_TAG__TEXT):
 *                 left_text = myhtml_node_text(current_node.prev, &left_length)
*/
    __pyx_t_6 = (__pyx_v_current_node->tag_id == MyHTML_TAG__TEXT);
    if (__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_current_node->prev != 0);
    if (__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L6_bool_binop_done;
    }

    /* "selectolax/modest/node.pxi":953
 * 
 *             if (current_node.tag_id == MyHTML_TAG__TEXT and current_node.prev and
 *                     current_node.prev.tag_id == MyHTML_TAG__TEXT):             # <<<<<<<<<<<<<<
 *                 left_text = myhtml_node_text(current_node.prev, &left_length)
 *                 right_text = myhtml_node_text(current_node, &right_length)
*/
    __pyx_t_6 = (__pyx_v_current_node->prev->tag_id == MyHTML_TAG__TEXT);
    __pyx_t_7 = __pyx_t_6;
    __pyx_L6_bool_binop_done:;

    /* "selectolax/modest/node.pxi":952
 *             current_node = stack.pop()
 * 
 *             if (current_node.tag_id == MyHTML_TAG__TEXT and current_node.prev and             # <<<<<<<<<<<<<<
 *                     current_node.prev.tag_id == MyHTML_TAG__TEXT):
 *                 left_text = myhtml_node_text(current_node.prev, &left_length)
*/
    if (__pyx_t_7) {

      /* "selectolax/modest/node.pxi":954
 *             if (current_node.tag_id == MyHTML_TAG__TEXT and current_node.prev and
 *                     current_node.prev.tag_id == MyHTML_TAG__TEXT):
 *                 left_text = myhtml_node_text(current_node.prev, &left_length)             # <<<<<<<<<<<<<<
 *                 right_text = myhtml_node_text(current_node, &right_length)
 *                 if left_text and right_text:
*/
      __pyx_v_left_text = myhtml_node_text(__pyx_v_current_node->prev, (&__pyx_v_left_length));

      /* "selectolax/modest/node.pxi":955
 *                     current_node.prev.tag_id == MyHTML_TAG__TEXT):
 *                 left_text = myhtml_node_text(current_node.prev, &left_length)
 *                 right_text = myhtml_node_text(current_node, &right_length)             # <<<<<<<<<<<<<<
 *                 if left_text and right_text:
 *                     final_length = left_length + right_length
*/
      __pyx_v_right_text = myhtml_node_text(__pyx_v_current_node, (&__pyx_v_right_length));

      /* "selectolax/modest/node.pxi":956
 *                 left_text = myhtml_node_text(current_node.prev, &left_length)
 *                 right_text = myhtml_node_text(current_node, &right_length)
 *                 if left_text and right_text:             # <<<<<<<<<<<<<<
 *                     final_length = left_length + right_length
 *                     final_text = <char *>malloc(final_length + 1)
*/
      __pyx_t_6 = (__pyx_v_left_text != 0);
      if (__pyx_t_6) {
      } else {
        __pyx_t_7 = __pyx_t_6;
        goto __pyx_L10_bool_binop_done;
      }
      __pyx_t_6 = (__pyx_v_right_text != 0);
      __pyx_t_7 = __pyx_t_6;
      __pyx_L10_bool_binop_done:;
      if (__pyx_t_7) {

        /* "selectolax/modest/node.pxi":957
 *                 right_text = myhtml_node_text(current_node, &right_length)
 *                 if left_text and right_text:
 *                     final_length = left_length + right_length             # <<<<<<<<<<<<<<
 *                     final_text = <char *>malloc(final_length + 1)
 *                     if final_text == NULL:
*/
        __pyx_v_final_length = (__pyx_v_left_length + __pyx_v_right_length);

        /* "selectolax/modest/node.pxi":958
 *                 if left_text and right_text:
 *                     final_length = left_length + right_length
 *                     final_text = <char *>malloc(final_length + 1)             # <<<<<<<<<<<<<<
 *                     if final_text == NULL:
 *                         raise MemoryError("Can't allocate memory for a new node.")
*/
        __pyx_v_final_text = ((char *)malloc((__pyx_v_final_length + 1)));

        /* "selectolax/modest/node.pxi":959
 *                     final_length = left_length + right_length
 *                     final_text = <char *>malloc(final_length + 1)
 *                     if final_text == NULL:             # <<<<<<<<<<<<<<
 *                         raise MemoryError("Can't allocate memory for a new node.")
 *                     memcpy(final_text, left_text, left_length)
*/
        __pyx_t_7 = (__pyx_v_final_text == NULL);
        if (unlikely(__pyx_t_7)) {

          /* "selectolax/modest/node.pxi":960
 *                     final_text = <char *>malloc(final_length + 1)
 *                     if final_text == NULL:
 *                         raise MemoryError("Can't allocate memory for a new node.")             # <<<<<<<<<<<<<<
 *                     memcpy(final_text, left_text, left_length)
 *                     memcpy(final_text + left_length, right_text, right_length + 1)
*/
          __pyx_t_3 = NULL;
          __Pyx_INCREF(__pyx_builtin_MemoryError);
          __pyx_t_2 = __pyx_builtin_MemoryError; 
          __pyx_t_4 = 1;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Can_t_allocate_memory_for_a_new};
            __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 960, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
          }
          __Pyx_Raise(__pyx_t_1, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __PYX_ERR(3, 960, __pyx_L1_error)

          /* "selectolax/modest/node.pxi":959
 *                     final_length = left_length + right_length
 *                     final_text = <char *>malloc(final_length + 1)
 *                     if final_text == NULL:             # <<<<<<<<<<<<<<
 *                         raise MemoryError("Can't allocate memory for a new node.")
 *                     memcpy(final_text, left_text, left_length)
*/
        }

        /* "selectolax/modest/node.pxi":961
 *                     if final_text == NULL:
 *                         raise MemoryError("Can't allocate memory for a new node.")
 *                     memcpy(final_text, left_text, left_length)             # <<<<<<<<<<<<<<
 *                     memcpy(final_text + left_length, right_text, right_length + 1)
 *                     myhtml_node_text_set(current_node, <const char *>final_text, final_length, MyENCODING_UTF_8)
*/
        (void)(memcpy(__pyx_v_final_text, __pyx_v_left_text, __pyx_v_left_length));

        /* "selectolax/modest/node.pxi":962
 *                         raise MemoryError("Can't allocate memory for a new node.")
 *                     memcpy(final_text, left_text, left_length)
 *                     memcpy(final_text + left_length, right_text, right_length + 1)             # <<<<<<<<<<<<<<
 *                     myhtml_node_text_set(current_node, <const char *>final_text, final_length, MyENCODING_UTF_8)
 *                     myhtml_node_delete(current_node.prev)
*/
        (void)(memcpy((__pyx_v_final_text + __pyx_v_left_length), __pyx_v_right_text, (__pyx_v_right_length + 1)));

        /* "selectolax/modest/node.pxi":963
 *                     memcpy(final_text, left_text, left_length)
 *                     memcpy(final_text + left_length, right_text, right_length + 1)
 *                     myhtml_node_text_set(current_node, <const char *>final_text, final_length, MyENCODING_UTF_8)             # <<<<<<<<<<<<<<
 *                     myhtml_node_delete(current_node.prev)
 *                     free(final_text)
*/
        (void)(myhtml_node_text_set(__pyx_v_current_node, ((char const *)__pyx_v_final_text), __pyx_v_final_length, MyENCODING_UTF_8));

        /* "selectolax/modest/node.pxi":964
 *                     memcpy(final_text + left_length, right_text, right_length + 1)
 *                     myhtml_node_text_set(current_node, <const char *>final_text, final_length, MyENCODING_UTF_8)
 *                     myhtml_node_delete(current_node.prev)             # <<<<<<<<<<<<<<
 *                     free(final_text)
 * 
*/
        myhtml_node_delete(__pyx_v_current_node->prev);

        /* "selectolax/modest/node.pxi":965
 *                     myhtml_node_text_set(current_node, <const char *>final_text, final_length, MyENCODING_UTF_8)
 *                     myhtml_node_delete(current_node.prev)
 *                     free(final_text)             # <<<<<<<<<<<<<<
 * 
 *             if current_node.next is not NULL:
*/
        free(__pyx_v_final_text);

        /* "selectolax/modest/node.pxi":956
 *                 left_text = myhtml_node_text(current_node.prev, &left_length)
 *                 right_text = myhtml_node_text(current_node, &right_length)
 *                 if left_text and right_text:             # <<<<<<<<<<<<<<
 *                     final_length = left_length + right_length
 *                     final_text = <char *>malloc(final_length + 1)
*/
      }

      /* "selectolax/modest/node.pxi":952
 *             current_node = stack.pop()
 * 
 *             if (current_node.tag_id == MyHTML_TAG__TEXT and current_node.prev and             # <<<<<<<<<<<<<<
 *                     current_node.prev.tag_id == MyHTML_TAG__TEXT):
 *                 left_text = myhtml_node_text(current_node.prev, &left_length)
*/
    }

    /* "selectolax/modest/node.pxi":967
 *                     free(final_text)
 * 
 *             if current_node.next is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.next)
 * 
*/
    __pyx_t_7 = (__pyx_v_current_node->next != NULL);
    if (__pyx_t_7) {

      /* "selectolax/modest/node.pxi":968
 * 
 *             if current_node.next is not NULL:
 *                 stack.push(current_node.next)             # <<<<<<<<<<<<<<
 * 
 *             if current_node.child is not NULL:
*/
      __pyx_t_5 = __pyx_f_10selectolax_6parser_5Stack_push(__pyx_v_stack, __pyx_v_current_node->next); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(3, 968, __pyx_L1_error)

      /* "selectolax/modest/node.pxi":967
 *                     free(final_text)
 * 
 *             if current_node.next is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.next)
 * 
*/
    }

    /* "selectolax/modest/node.pxi":970
 *                 stack.push(current_node.next)
 * 
 *             if current_node.child is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.child)
 * 
*/
    __pyx_t_7 = (__pyx_v_current_node->child != NULL);
    if (__pyx_t_7) {

      /* "selectolax/modest/node.pxi":971
 * 
 *             if current_node.child is not NULL:
 *                 stack.push(current_node.child)             # <<<<<<<<<<<<<<
 * 
 * cdef inline str append_text(str text, str node_text, str separator='', bint strip=False):
*/
      __pyx_t_5 = __pyx_f_10selectolax_6parser_5Stack_push(__pyx_v_stack, __pyx_v_current_node->child); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(3, 971, __pyx_L1_error)

      /* "selectolax/modest/node.pxi":970
 *                 stack.push(current_node.next)
 * 
 *             if current_node.child is not NULL:             # <<<<<<<<<<<<<<
 *                 stack.push(current_node.child)
 * 
*/
    }
  }

  /* "selectolax/modest/node.pxi":921
 *         return None
 * 
 *     def merge_text_nodes(self):             # <<<<<<<<<<<<<<
 *         """Iterates over all text nodes and merges all text nodes that are close to each other.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.Node.merge_text_nodes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_stack);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":147
 *     """A class that represents HTML node (element)."""
 *     cdef myhtml_tree_node_t *node
 *     cdef public HTMLParser parser             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_6parser_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_4Node_6parser_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_6parser___get__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_6parser___get__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->parser);
  __pyx_r = ((PyObject *)__pyx_v_self->parser);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_4Node_6parser_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10selectolax_6parser_4Node_6parser_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_6parser_2__set__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_4Node_6parser_2__set__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser))))) __PYX_ERR(3, 147, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->parser);
  __Pyx_DECREF((PyObject *)__pyx_v_self->parser);
  __pyx_v_self->parser = ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.Node.parser.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_4Node_6parser_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10selectolax_6parser_4Node_6parser_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_6parser_4__del__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_4Node_6parser_4__del__(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->parser);
  __Pyx_DECREF((PyObject *)__pyx_v_self->parser);
  __pyx_v_self->parser = ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_83__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_82__reduce_cython__, "Node.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_83__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_83__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_82__reduce_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_83__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_82__reduce_cython__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_82__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_self_node_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Node.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_4Node_85__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4Node_84__setstate_cython__, "Node.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_4Node_85__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_85__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_84__setstate_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_4Node_85__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.Node.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4Node_84__setstate_cython__(((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4Node_84__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_self_node_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.Node.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":973
 *                 stack.push(current_node.child)
 * 
 * cdef inline str append_text(str text, str node_text, str separator='', bint strip=False):             # <<<<<<<<<<<<<<
 *     if strip:
 *         text += node_text.strip() + separator
*/

static CYTHON_INLINE PyObject *__pyx_f_10selectolax_6parser_append_text(PyObject *__pyx_v_text, PyObject *__pyx_v_node_text, struct __pyx_opt_args_10selectolax_6parser_append_text *__pyx_optional_args) {
  PyObject *__pyx_v_separator = ((PyObject*)__pyx_mstate_global->__pyx_kp_u_);
  int __pyx_v_strip = ((int)0);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("append_text", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_separator = __pyx_optional_args->separator;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_strip = __pyx_optional_args->strip;
      }
    }
  }
  __Pyx_INCREF(__pyx_v_text);

  /* "selectolax/modest/node.pxi":974
 * 
 * cdef inline str append_text(str text, str node_text, str separator='', bint strip=False):
 *     if strip:             # <<<<<<<<<<<<<<
 *         text += node_text.strip() + separator
 *     else:
*/
  if (__pyx_v_strip) {

    /* "selectolax/modest/node.pxi":975
 * cdef inline str append_text(str text, str node_text, str separator='', bint strip=False):
 *     if strip:
 *         text += node_text.strip() + separator             # <<<<<<<<<<<<<<
 *     else:
 *         text += node_text + separator
*/
    __pyx_t_1 = __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyUnicode_Type__strip, __pyx_v_node_text); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 975, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyUnicode_ConcatInPlaceSafe(__pyx_t_1, __pyx_v_separator); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 975, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlaceSafe(__pyx_v_text, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 975, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_text, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "selectolax/modest/node.pxi":974
 * 
 * cdef inline str append_text(str text, str node_text, str separator='', bint strip=False):
 *     if strip:             # <<<<<<<<<<<<<<
 *         text += node_text.strip() + separator
 *     else:
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":977
 *         text += node_text.strip() + separator
 *     else:
 *         text += node_text + separator             # <<<<<<<<<<<<<<
 * 
 *     return text
*/
  /*else*/ {
    __pyx_t_1 = __Pyx_PyUnicode_ConcatSafe(__pyx_v_node_text, __pyx_v_separator); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 977, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyUnicode_ConcatInPlaceSafe(__pyx_v_text, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 977, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_text, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":979
 *         text += node_text + separator
 * 
 *     return text             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_text);
  __pyx_r = __pyx_v_text;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":973
 *                 stack.push(current_node.child)
 * 
 * cdef inline str append_text(str text, str node_text, str separator='', bint strip=False):             # <<<<<<<<<<<<<<
 *     if strip:
 *         text += node_text.strip() + separator
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.append_text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_text);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/node.pxi":982
 * 
 * 
 * cdef inline bytes to_bytes(str_or_Node value):             # <<<<<<<<<<<<<<
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):
*/

static CYTHON_INLINE PyObject *__pyx_fuse_0__pyx_f_10selectolax_6parser_to_bytes(PyObject *__pyx_v_value) {
  PyObject *__pyx_v_bytes_val = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0to_bytes", 0);

  /* "selectolax/modest/node.pxi":984
 * cdef inline bytes to_bytes(str_or_Node value):
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_value); 
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":985
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):
 *         bytes_val = <bytes>value.encode("utf-8")             # <<<<<<<<<<<<<<
 *     elif isinstance(value, bytes):
 *         bytes_val = <bytes>value
*/
    if (unlikely(__pyx_v_value == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
      __PYX_ERR(3, 985, __pyx_L1_error)
    }
    __pyx_t_2 = PyUnicode_AsUTF8String(__pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 985, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":984
 * cdef inline bytes to_bytes(str_or_Node value):
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":986
 *     if isinstance(value, unicode):
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value
 *     return bytes_val
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_value); 
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":987
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):
 *         bytes_val = <bytes>value             # <<<<<<<<<<<<<<
 *     return bytes_val
*/
    __pyx_t_3 = __pyx_v_value;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":986
 *     if isinstance(value, unicode):
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value
 *     return bytes_val
*/
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":988
 *     elif isinstance(value, bytes):
 *         bytes_val = <bytes>value
 *     return bytes_val             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_bytes_val)) { __Pyx_RaiseUnboundLocalError("bytes_val"); __PYX_ERR(3, 988, __pyx_L1_error) }
  __Pyx_INCREF(__pyx_v_bytes_val);
  __pyx_r = __pyx_v_bytes_val;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":982
 * 
 * 
 * cdef inline bytes to_bytes(str_or_Node value):             # <<<<<<<<<<<<<<
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static CYTHON_INLINE PyObject *__pyx_fuse_1__pyx_f_10selectolax_6parser_to_bytes(PyObject *__pyx_v_value) {
  PyObject *__pyx_v_bytes_val = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1to_bytes", 0);

  /* "selectolax/modest/node.pxi":984
 * cdef inline bytes to_bytes(str_or_Node value):
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_value); 
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":985
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):
 *         bytes_val = <bytes>value.encode("utf-8")             # <<<<<<<<<<<<<<
 *     elif isinstance(value, bytes):
 *         bytes_val = <bytes>value
*/
    __pyx_t_2 = __Pyx_CallUnboundCMethod1(&__pyx_mstate_global->__pyx_umethod_PyBytes_Type__encode, __pyx_v_value, __pyx_mstate_global->__pyx_kp_u_utf_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 985, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":984
 * cdef inline bytes to_bytes(str_or_Node value):
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":986
 *     if isinstance(value, unicode):
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value
 *     return bytes_val
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_value); 
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":987
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):
 *         bytes_val = <bytes>value             # <<<<<<<<<<<<<<
 *     return bytes_val
*/
    __pyx_t_3 = __pyx_v_value;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":986
 *     if isinstance(value, unicode):
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value
 *     return bytes_val
*/
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":988
 *     elif isinstance(value, bytes):
 *         bytes_val = <bytes>value
 *     return bytes_val             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_bytes_val)) { __Pyx_RaiseUnboundLocalError("bytes_val"); __PYX_ERR(3, 988, __pyx_L1_error) }
  __Pyx_INCREF(__pyx_v_bytes_val);
  __pyx_r = __pyx_v_bytes_val;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":982
 * 
 * 
 * cdef inline bytes to_bytes(str_or_Node value):             # <<<<<<<<<<<<<<
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static CYTHON_INLINE PyObject *__pyx_fuse_2__pyx_f_10selectolax_6parser_to_bytes(struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_value) {
  PyObject *__pyx_v_bytes_val = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2to_bytes", 0);

  /* "selectolax/modest/node.pxi":984
 * cdef inline bytes to_bytes(str_or_Node value):
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):
*/
  __pyx_t_1 = PyUnicode_Check(((PyObject *)__pyx_v_value)); 
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":985
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):
 *         bytes_val = <bytes>value.encode("utf-8")             # <<<<<<<<<<<<<<
 *     elif isinstance(value, bytes):
 *         bytes_val = <bytes>value
*/
    __pyx_t_3 = ((PyObject *)__pyx_v_value);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_utf_8};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 985, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_3 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":984
 * cdef inline bytes to_bytes(str_or_Node value):
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):
*/
    goto __pyx_L3;
  }

  /* "selectolax/modest/node.pxi":986
 *     if isinstance(value, unicode):
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value
 *     return bytes_val
*/
  __pyx_t_1 = PyBytes_Check(((PyObject *)__pyx_v_value)); 
  if (__pyx_t_1) {

    /* "selectolax/modest/node.pxi":987
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):
 *         bytes_val = <bytes>value             # <<<<<<<<<<<<<<
 *     return bytes_val
*/
    __pyx_t_3 = ((PyObject *)__pyx_v_value);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_bytes_val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/modest/node.pxi":986
 *     if isinstance(value, unicode):
 *         bytes_val = <bytes>value.encode("utf-8")
 *     elif isinstance(value, bytes):             # <<<<<<<<<<<<<<
 *         bytes_val = <bytes>value
 *     return bytes_val
*/
  }
  __pyx_L3:;

  /* "selectolax/modest/node.pxi":988
 *     elif isinstance(value, bytes):
 *         bytes_val = <bytes>value
 *     return bytes_val             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_bytes_val)) { __Pyx_RaiseUnboundLocalError("bytes_val"); __PYX_ERR(3, 988, __pyx_L1_error) }
  __Pyx_INCREF(__pyx_v_bytes_val);
  __pyx_r = __pyx_v_bytes_val;
  goto __pyx_L0;

  /* "selectolax/modest/node.pxi":982
 * 
 * 
 * cdef inline bytes to_bytes(str_or_Node value):             # <<<<<<<<<<<<<<
 *     cdef bytes bytes_val
 *     if isinstance(value, unicode):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/utils.pxi":19
 * 
 * 
 * def preprocess_input(html, decode_errors='ignore'):             # <<<<<<<<<<<<<<
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_1preprocess_input(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_preprocess_input, "preprocess_input(html, decode_errors='ignore')");
static PyMethodDef __pyx_mdef_10selectolax_6parser_1preprocess_input = {"preprocess_input", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_1preprocess_input, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_preprocess_input};
static PyObject *__pyx_pw_10selectolax_6parser_1preprocess_input(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_html = 0;
  PyObject *__pyx_v_decode_errors = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("preprocess_input (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_html,&__pyx_mstate_global->__pyx_n_u_decode_errors,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(4, 19, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(4, 19, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(4, 19, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "preprocess_input", 0) < 0) __PYX_ERR(4, 19, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)((PyObject*)__pyx_mstate_global->__pyx_n_u_ignore)));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("preprocess_input", 0, 1, 2, i); __PYX_ERR(4, 19, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(4, 19, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(4, 19, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)((PyObject*)__pyx_mstate_global->__pyx_n_u_ignore)));
    }
    __pyx_v_html = values[0];
    __pyx_v_decode_errors = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("preprocess_input", 0, 1, 2, __pyx_nargs); __PYX_ERR(4, 19, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.preprocess_input", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_preprocess_input(__pyx_self, __pyx_v_html, __pyx_v_decode_errors);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_preprocess_input(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_decode_errors) {
  PyObject *__pyx_v_bytes_html = NULL;
  PyObject *__pyx_v_html_len = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("preprocess_input", 0);

  /* "selectolax/utils.pxi":20
 * 
 * def preprocess_input(html, decode_errors='ignore'):
 *     if isinstance(html, (str, unicode)):             # <<<<<<<<<<<<<<
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_html); 
  if (__pyx_t_1) {

    /* "selectolax/utils.pxi":21
 * def preprocess_input(html, decode_errors='ignore'):
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)             # <<<<<<<<<<<<<<
 *     elif isinstance(html, bytes):
 *         bytes_html = html
*/
    __pyx_t_3 = __pyx_v_html;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 21, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_errors, __pyx_v_decode_errors, __pyx_t_5, __pyx_callargs+2, 0) < 0) __PYX_ERR(4, 21, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 21, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_v_bytes_html = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "selectolax/utils.pxi":20
 * 
 * def preprocess_input(html, decode_errors='ignore'):
 *     if isinstance(html, (str, unicode)):             # <<<<<<<<<<<<<<
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):
*/
    goto __pyx_L3;
  }

  /* "selectolax/utils.pxi":22
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):             # <<<<<<<<<<<<<<
 *         bytes_html = html
 *     else:
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_html); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/utils.pxi":23
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):
 *         bytes_html = html             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)
*/
    __Pyx_INCREF(__pyx_v_html);
    __pyx_v_bytes_html = __pyx_v_html;

    /* "selectolax/utils.pxi":22
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):             # <<<<<<<<<<<<<<
 *         bytes_html = html
 *     else:
*/
    goto __pyx_L3;
  }

  /* "selectolax/utils.pxi":25
 *         bytes_html = html
 *     else:
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)             # <<<<<<<<<<<<<<
 *     html_len = len(bytes_html)
 *     if html_len > MAX_HTML_INPUT_SIZE:
*/
  /*else*/ {
    __pyx_t_5 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_3 = __pyx_builtin_TypeError; 
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_html)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_but_s_found, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(4, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_7};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 25, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(4, 25, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/utils.pxi":26
 *     else:
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)
 *     html_len = len(bytes_html)             # <<<<<<<<<<<<<<
 *     if html_len > MAX_HTML_INPUT_SIZE:
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)
*/
  __pyx_t_8 = PyObject_Length(__pyx_v_bytes_html); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(4, 26, __pyx_L1_error)
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_html_len = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "selectolax/utils.pxi":27
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)
 *     html_len = len(bytes_html)
 *     if html_len > MAX_HTML_INPUT_SIZE:             # <<<<<<<<<<<<<<
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)
 *     return bytes_html, html_len
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_MAX_HTML_INPUT_SIZE); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_html_len, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(4, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_1)) {

    /* "selectolax/utils.pxi":28
 *     html_len = len(bytes_html)
 *     if html_len > MAX_HTML_INPUT_SIZE:
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)             # <<<<<<<<<<<<<<
 *     return bytes_html, html_len
 * 
*/
    __pyx_t_2 = NULL;
    __Pyx_INCREF(__pyx_builtin_ValueError);
    __pyx_t_7 = __pyx_builtin_ValueError; 
    __pyx_t_5 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_The_specified_HTML_input_is_too, __pyx_v_html_len); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 28, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 28, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(4, 28, __pyx_L1_error)

    /* "selectolax/utils.pxi":27
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)
 *     html_len = len(bytes_html)
 *     if html_len > MAX_HTML_INPUT_SIZE:             # <<<<<<<<<<<<<<
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)
 *     return bytes_html, html_len
*/
  }

  /* "selectolax/utils.pxi":29
 *     if html_len > MAX_HTML_INPUT_SIZE:
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)
 *     return bytes_html, html_len             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_bytes_html);
  __Pyx_GIVEREF(__pyx_v_bytes_html);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_bytes_html) != (0)) __PYX_ERR(4, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_html_len);
  __Pyx_GIVEREF(__pyx_v_html_len);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_html_len) != (0)) __PYX_ERR(4, 29, __pyx_L1_error);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "selectolax/utils.pxi":19
 * 
 * 
 * def preprocess_input(html, decode_errors='ignore'):             # <<<<<<<<<<<<<<
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("selectolax.parser.preprocess_input", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_html);
  __Pyx_XDECREF(__pyx_v_html_len);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/utils.pxi":32
 * 
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_3do_create_tag(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_2do_create_tag, "do_create_tag(tag: str, parser_cls: ParserCls)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_3do_create_tag = {"do_create_tag", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_3do_create_tag, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_2do_create_tag};
static PyObject *__pyx_pw_10selectolax_6parser_3do_create_tag(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_v_parser_cls = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("do_create_tag (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_tag,&__pyx_mstate_global->__pyx_n_u_parser_cls,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(4, 32, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(4, 32, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(4, 32, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "do_create_tag", 0) < 0) __PYX_ERR(4, 32, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("do_create_tag", 1, 2, 2, i); __PYX_ERR(4, 32, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(4, 32, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(4, 32, __pyx_L3_error)
    }
    __pyx_v_tag = values[0];
    __pyx_v_parser_cls = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("do_create_tag", 1, 2, 2, __pyx_nargs); __PYX_ERR(4, 32, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.do_create_tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_2do_create_tag(__pyx_self, __pyx_v_tag, __pyx_v_parser_cls);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_2do_create_tag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tag, PyObject *__pyx_v_parser_cls) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9[5];
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("do_create_tag", 0);

  /* "selectolax/utils.pxi":33
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):
 *     if not tag:             # <<<<<<<<<<<<<<
 *         raise ValueError("Tag name cannot be empty")
 *     return do_parse_fragment(f"<{tag}></{tag}>", parser_cls)[0]
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_tag); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(4, 33, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_1);
  if (unlikely(__pyx_t_2)) {

    /* "selectolax/utils.pxi":34
 * def do_create_tag(tag: str, parser_cls: ParserCls):
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")             # <<<<<<<<<<<<<<
 *     return do_parse_fragment(f"<{tag}></{tag}>", parser_cls)[0]
 * 
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_builtin_ValueError);
    __pyx_t_5 = __pyx_builtin_ValueError; 
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Tag_name_cannot_be_empty};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 34, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(4, 34, __pyx_L1_error)

    /* "selectolax/utils.pxi":33
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):
 *     if not tag:             # <<<<<<<<<<<<<<
 *         raise ValueError("Tag name cannot be empty")
 *     return do_parse_fragment(f"<{tag}></{tag}>", parser_cls)[0]
*/
  }

  /* "selectolax/utils.pxi":35
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")
 *     return do_parse_fragment(f"<{tag}></{tag}>", parser_cls)[0]             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_do_parse_fragment); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_v_tag, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(4, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_tag, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(4, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9[0] = __pyx_mstate_global->__pyx_kp_u__2;
  __pyx_t_9[1] = __pyx_t_7;
  __pyx_t_9[2] = __pyx_mstate_global->__pyx_kp_u__9;
  __pyx_t_9[3] = __pyx_t_8;
  __pyx_t_9[4] = __pyx_mstate_global->__pyx_kp_u__10;
  __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_9, 5, 1 * 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 3 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8));
  if (unlikely(!__pyx_t_10)) __PYX_ERR(4, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_t_10, __pyx_v_parser_cls};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 35, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "selectolax/utils.pxi":32
 * 
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("selectolax.parser.do_create_tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/utils.pxi":38
 * 
 * 
 * def get_fragment_type(             # <<<<<<<<<<<<<<
 *     html: str,
 *     parser_cls: ParserCls,
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_5get_fragment_type(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_4get_fragment_type, "get_fragment_type(html: str, parser_cls: ParserCls, tree: Optional[Parser] = None) -> FRAGMENT");
static PyMethodDef __pyx_mdef_10selectolax_6parser_5get_fragment_type = {"get_fragment_type", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_5get_fragment_type, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4get_fragment_type};
static PyObject *__pyx_pw_10selectolax_6parser_5get_fragment_type(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_html = 0;
  PyObject *__pyx_v_parser_cls = 0;
  PyObject *__pyx_v_tree = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_fragment_type (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_html,&__pyx_mstate_global->__pyx_n_u_parser_cls,&__pyx_mstate_global->__pyx_n_u_tree,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(4, 38, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(4, 38, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(4, 38, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(4, 38, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_fragment_type", 0) < 0) __PYX_ERR(4, 38, __pyx_L3_error)

      /* "selectolax/utils.pxi":41
 *     html: str,
 *     parser_cls: ParserCls,
 *     tree: Optional[Parser] = None,             # <<<<<<<<<<<<<<
 * ) -> FRAGMENT:
 *     if not tree:
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_fragment_type", 0, 2, 3, i); __PYX_ERR(4, 38, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(4, 38, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(4, 38, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(4, 38, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_html = values[0];
    __pyx_v_parser_cls = values[1];
    __pyx_v_tree = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_fragment_type", 0, 2, 3, __pyx_nargs); __PYX_ERR(4, 38, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.get_fragment_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_4get_fragment_type(__pyx_self, __pyx_v_html, __pyx_v_parser_cls, __pyx_v_tree);

  /* "selectolax/utils.pxi":38
 * 
 * 
 * def get_fragment_type(             # <<<<<<<<<<<<<<
 *     html: str,
 *     parser_cls: ParserCls,
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_4get_fragment_type(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_parser_cls, PyObject *__pyx_v_tree) {
  PyObject *__pyx_v_re = NULL;
  PyObject *__pyx_v_html_re = NULL;
  int __pyx_v_has_html;
  int __pyx_v_has_head;
  int __pyx_v_has_body;
  PyObject *__pyx_v_match = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_fragment_type", 0);
  __Pyx_INCREF(__pyx_v_tree);

  /* "selectolax/utils.pxi":43
 *     tree: Optional[Parser] = None,
 * ) -> FRAGMENT:
 *     if not tree:             # <<<<<<<<<<<<<<
 *         tree = parser_cls(html)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_tree); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(4, 43, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_1);
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":44
 * ) -> FRAGMENT:
 *     if not tree:
 *         tree = parser_cls(html)             # <<<<<<<<<<<<<<
 * 
 *     import re
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_v_parser_cls);
    __pyx_t_5 = __pyx_v_parser_cls; 
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_html};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 44, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF_SET(__pyx_v_tree, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/utils.pxi":43
 *     tree: Optional[Parser] = None,
 * ) -> FRAGMENT:
 *     if not tree:             # <<<<<<<<<<<<<<
 *         tree = parser_cls(html)
 * 
*/
  }

  /* "selectolax/utils.pxi":46
 *         tree = parser_cls(html)
 * 
 *     import re             # <<<<<<<<<<<<<<
 *     html_re = re.compile(r"<html|<body|<head(?!er)", re.IGNORECASE)
 * 
*/
  __pyx_t_3 = __Pyx_ImportDottedModule(__pyx_mstate_global->__pyx_n_u_re, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_re = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "selectolax/utils.pxi":47
 * 
 *     import re
 *     html_re = re.compile(r"<html|<body|<head(?!er)", re.IGNORECASE)             # <<<<<<<<<<<<<<
 * 
 *     has_html = False
*/
  __pyx_t_5 = __pyx_v_re;
  __Pyx_INCREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_re, __pyx_mstate_global->__pyx_n_u_IGNORECASE); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_html_body_head_er, __pyx_t_4};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_compile, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 47, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_html_re = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "selectolax/utils.pxi":49
 *     html_re = re.compile(r"<html|<body|<head(?!er)", re.IGNORECASE)
 * 
 *     has_html = False             # <<<<<<<<<<<<<<
 *     has_head = False
 *     has_body = False
*/
  __pyx_v_has_html = 0;

  /* "selectolax/utils.pxi":50
 * 
 *     has_html = False
 *     has_head = False             # <<<<<<<<<<<<<<
 *     has_body = False
 *     for match in html_re.finditer(html):
*/
  __pyx_v_has_head = 0;

  /* "selectolax/utils.pxi":51
 *     has_html = False
 *     has_head = False
 *     has_body = False             # <<<<<<<<<<<<<<
 *     for match in html_re.finditer(html):
 *         if match[0] == "<html":
*/
  __pyx_v_has_body = 0;

  /* "selectolax/utils.pxi":52
 *     has_head = False
 *     has_body = False
 *     for match in html_re.finditer(html):             # <<<<<<<<<<<<<<
 *         if match[0] == "<html":
 *             has_html = True
*/
  __pyx_t_4 = __pyx_v_html_re;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_html};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_finditer, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4);
    __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(4, 52, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(4, 52, __pyx_L1_error)
          #endif
          if (__pyx_t_7 >= __pyx_temp) break;
        }
        __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_4, __pyx_t_7);
        ++__pyx_t_7;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(4, 52, __pyx_L1_error)
          #endif
          if (__pyx_t_7 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_7));
        #else
        __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_7);
        #endif
        ++__pyx_t_7;
      }
      if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 52, __pyx_L1_error)
    } else {
      __pyx_t_3 = __pyx_t_8(__pyx_t_4);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(4, 52, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_match, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/utils.pxi":53
 *     has_body = False
 *     for match in html_re.finditer(html):
 *         if match[0] == "<html":             # <<<<<<<<<<<<<<
 *             has_html = True
 *         elif match[0] == "<head":
*/
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_match, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 53, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_html_2, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(4, 53, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "selectolax/utils.pxi":54
 *     for match in html_re.finditer(html):
 *         if match[0] == "<html":
 *             has_html = True             # <<<<<<<<<<<<<<
 *         elif match[0] == "<head":
 *             has_head = True
*/
      __pyx_v_has_html = 1;

      /* "selectolax/utils.pxi":53
 *     has_body = False
 *     for match in html_re.finditer(html):
 *         if match[0] == "<html":             # <<<<<<<<<<<<<<
 *             has_html = True
 *         elif match[0] == "<head":
*/
      goto __pyx_L6;
    }

    /* "selectolax/utils.pxi":55
 *         if match[0] == "<html":
 *             has_html = True
 *         elif match[0] == "<head":             # <<<<<<<<<<<<<<
 *             has_head = True
 *         elif match[0] == "<body":
*/
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_match, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_head, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(4, 55, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "selectolax/utils.pxi":56
 *             has_html = True
 *         elif match[0] == "<head":
 *             has_head = True             # <<<<<<<<<<<<<<
 *         elif match[0] == "<body":
 *             has_body = True
*/
      __pyx_v_has_head = 1;

      /* "selectolax/utils.pxi":55
 *         if match[0] == "<html":
 *             has_html = True
 *         elif match[0] == "<head":             # <<<<<<<<<<<<<<
 *             has_head = True
 *         elif match[0] == "<body":
*/
      goto __pyx_L6;
    }

    /* "selectolax/utils.pxi":57
 *         elif match[0] == "<head":
 *             has_head = True
 *         elif match[0] == "<body":             # <<<<<<<<<<<<<<
 *             has_body = True
 * 
*/
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_match, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_body, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(4, 57, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "selectolax/utils.pxi":58
 *             has_head = True
 *         elif match[0] == "<body":
 *             has_body = True             # <<<<<<<<<<<<<<
 * 
 *         if has_html and has_head and has_body:
*/
      __pyx_v_has_body = 1;

      /* "selectolax/utils.pxi":57
 *         elif match[0] == "<head":
 *             has_head = True
 *         elif match[0] == "<body":             # <<<<<<<<<<<<<<
 *             has_body = True
 * 
*/
    }
    __pyx_L6:;

    /* "selectolax/utils.pxi":60
 *             has_body = True
 * 
 *         if has_html and has_head and has_body:             # <<<<<<<<<<<<<<
 *             break
 * 
*/
    if (__pyx_v_has_html) {
    } else {
      __pyx_t_2 = __pyx_v_has_html;
      goto __pyx_L8_bool_binop_done;
    }
    if (__pyx_v_has_head) {
    } else {
      __pyx_t_2 = __pyx_v_has_head;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_2 = __pyx_v_has_body;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_2) {

      /* "selectolax/utils.pxi":61
 * 
 *         if has_html and has_head and has_body:
 *             break             # <<<<<<<<<<<<<<
 * 
 *     if has_html and has_head and has_body:
*/
      goto __pyx_L5_break;

      /* "selectolax/utils.pxi":60
 *             has_body = True
 * 
 *         if has_html and has_head and has_body:             # <<<<<<<<<<<<<<
 *             break
 * 
*/
    }

    /* "selectolax/utils.pxi":52
 *     has_head = False
 *     has_body = False
 *     for match in html_re.finditer(html):             # <<<<<<<<<<<<<<
 *         if match[0] == "<html":
 *             has_html = True
*/
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  goto __pyx_L11_for_end;
  __pyx_L5_break:;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  goto __pyx_L11_for_end;
  __pyx_L11_for_end:;

  /* "selectolax/utils.pxi":63
 *             break
 * 
 *     if has_html and has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "document"
 *     elif has_html and not has_head and has_body:
*/
  if (__pyx_v_has_html) {
  } else {
    __pyx_t_2 = __pyx_v_has_html;
    goto __pyx_L13_bool_binop_done;
  }
  if (__pyx_v_has_head) {
  } else {
    __pyx_t_2 = __pyx_v_has_head;
    goto __pyx_L13_bool_binop_done;
  }
  __pyx_t_2 = __pyx_v_has_body;
  __pyx_L13_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":64
 * 
 *     if has_html and has_head and has_body:
 *         return "document"             # <<<<<<<<<<<<<<
 *     elif has_html and not has_head and has_body:
 *         return "document_no_head"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_document);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_document;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":63
 *             break
 * 
 *     if has_html and has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "document"
 *     elif has_html and not has_head and has_body:
*/
  }

  /* "selectolax/utils.pxi":65
 *     if has_html and has_head and has_body:
 *         return "document"
 *     elif has_html and not has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:
*/
  if (__pyx_v_has_html) {
  } else {
    __pyx_t_2 = __pyx_v_has_html;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_head);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = __pyx_v_has_body;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":66
 *         return "document"
 *     elif has_html and not has_head and has_body:
 *         return "document_no_head"             # <<<<<<<<<<<<<<
 *     elif has_html and has_head and not has_body:
 *         return "document_no_body"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_document_no_head);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_document_no_head;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":65
 *     if has_html and has_head and has_body:
 *         return "document"
 *     elif has_html and not has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:
*/
  }

  /* "selectolax/utils.pxi":67
 *     elif has_html and not has_head and has_body:
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:
*/
  if (__pyx_v_has_html) {
  } else {
    __pyx_t_2 = __pyx_v_has_html;
    goto __pyx_L19_bool_binop_done;
  }
  if (__pyx_v_has_head) {
  } else {
    __pyx_t_2 = __pyx_v_has_head;
    goto __pyx_L19_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_body);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L19_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":68
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:
 *         return "document_no_body"             # <<<<<<<<<<<<<<
 *     elif has_html and not has_head and not has_body:
 *         return "document_no_head_no_body"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_document_no_body);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_document_no_body;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":67
 *     elif has_html and not has_head and has_body:
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:
*/
  }

  /* "selectolax/utils.pxi":69
 *     elif has_html and has_head and not has_body:
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:
*/
  if (__pyx_v_has_html) {
  } else {
    __pyx_t_2 = __pyx_v_has_html;
    goto __pyx_L22_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_head);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L22_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_body);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L22_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":70
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:
 *         return "document_no_head_no_body"             # <<<<<<<<<<<<<<
 *     elif has_head and not has_body:
 *         return "head"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_document_no_head_no_body);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_document_no_head_no_body;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":69
 *     elif has_html and has_head and not has_body:
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:
*/
  }

  /* "selectolax/utils.pxi":71
 *     elif has_html and not has_head and not has_body:
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "head"
 *     elif not has_head and has_body:
*/
  if (__pyx_v_has_head) {
  } else {
    __pyx_t_2 = __pyx_v_has_head;
    goto __pyx_L25_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_body);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L25_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":72
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:
 *         return "head"             # <<<<<<<<<<<<<<
 *     elif not has_head and has_body:
 *         return "body"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_head_2);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_head_2;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":71
 *     elif has_html and not has_head and not has_body:
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "head"
 *     elif not has_head and has_body:
*/
  }

  /* "selectolax/utils.pxi":73
 *     elif has_head and not has_body:
 *         return "head"
 *     elif not has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "body"
 *     elif has_head and has_body:
*/
  __pyx_t_1 = (!__pyx_v_has_head);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L27_bool_binop_done;
  }
  __pyx_t_2 = __pyx_v_has_body;
  __pyx_L27_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":74
 *         return "head"
 *     elif not has_head and has_body:
 *         return "body"             # <<<<<<<<<<<<<<
 *     elif has_head and has_body:
 *         return "head_and_body"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_body_2);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_body_2;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":73
 *     elif has_head and not has_body:
 *         return "head"
 *     elif not has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "body"
 *     elif has_head and has_body:
*/
  }

  /* "selectolax/utils.pxi":75
 *     elif not has_head and has_body:
 *         return "body"
 *     elif has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "head_and_body"
 *     else:
*/
  if (__pyx_v_has_head) {
  } else {
    __pyx_t_2 = __pyx_v_has_head;
    goto __pyx_L29_bool_binop_done;
  }
  __pyx_t_2 = __pyx_v_has_body;
  __pyx_L29_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":76
 *         return "body"
 *     elif has_head and has_body:
 *         return "head_and_body"             # <<<<<<<<<<<<<<
 *     else:
 *         return "fragment"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_head_and_body);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_head_and_body;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":75
 *     elif not has_head and has_body:
 *         return "body"
 *     elif has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "head_and_body"
 *     else:
*/
  }

  /* "selectolax/utils.pxi":78
 *         return "head_and_body"
 *     else:
 *         return "fragment"             # <<<<<<<<<<<<<<
 * 
 * 
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_fragment);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_fragment;
    goto __pyx_L0;
  }

  /* "selectolax/utils.pxi":38
 * 
 * 
 * def get_fragment_type(             # <<<<<<<<<<<<<<
 *     html: str,
 *     parser_cls: ParserCls,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.get_fragment_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_re);
  __Pyx_XDECREF(__pyx_v_html_re);
  __Pyx_XDECREF(__pyx_v_match);
  __Pyx_XDECREF(__pyx_v_tree);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/utils.pxi":81
 * 
 * 
 * def do_parse_fragment(html: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     """
 *     Given HTML, parse it into a list of Nodes, such that the nodes
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_7do_parse_fragment(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_6do_parse_fragment, "do_parse_fragment(html: str, parser_cls: ParserCls)\n\nGiven HTML, parse it into a list of Nodes, such that the nodes\ncorrespond to the given HTML.\n\nFor contrast, HTMLParser adds `<html>`, `<head>`, and `<body>` tags\nif they are missing. This function does not add these tags.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_7do_parse_fragment = {"do_parse_fragment", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_7do_parse_fragment, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_6do_parse_fragment};
static PyObject *__pyx_pw_10selectolax_6parser_7do_parse_fragment(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_html = 0;
  PyObject *__pyx_v_parser_cls = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("do_parse_fragment (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_html,&__pyx_mstate_global->__pyx_n_u_parser_cls,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(4, 81, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(4, 81, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(4, 81, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "do_parse_fragment", 0) < 0) __PYX_ERR(4, 81, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("do_parse_fragment", 1, 2, 2, i); __PYX_ERR(4, 81, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(4, 81, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(4, 81, __pyx_L3_error)
    }
    __pyx_v_html = values[0];
    __pyx_v_parser_cls = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("do_parse_fragment", 1, 2, 2, __pyx_nargs); __PYX_ERR(4, 81, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.do_parse_fragment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_6do_parse_fragment(__pyx_self, __pyx_v_html, __pyx_v_parser_cls);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_6do_parse_fragment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_parser_cls) {
  PyObject *__pyx_v_tree = NULL;
  PyObject *__pyx_v_frag_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("do_parse_fragment", 0);
  __Pyx_INCREF(__pyx_v_html);

  /* "selectolax/utils.pxi":89
 *     if they are missing. This function does not add these tags.
 *     """
 *     html = html.strip()             # <<<<<<<<<<<<<<
 *     tree = parser_cls(html)
 *     frag_type = get_fragment_type(html, parser_cls, tree)
*/
  __pyx_t_2 = __pyx_v_html;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_strip, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF_SET(__pyx_v_html, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/utils.pxi":90
 *     """
 *     html = html.strip()
 *     tree = parser_cls(html)             # <<<<<<<<<<<<<<
 *     frag_type = get_fragment_type(html, parser_cls, tree)
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_parser_cls);
  __pyx_t_4 = __pyx_v_parser_cls; 
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_html};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_tree = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "selectolax/utils.pxi":91
 *     html = html.strip()
 *     tree = parser_cls(html)
 *     frag_type = get_fragment_type(html, parser_cls, tree)             # <<<<<<<<<<<<<<
 * 
 *     if frag_type == "document":
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_fragment_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_v_html, __pyx_v_parser_cls, __pyx_v_tree};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+__pyx_t_3, (4-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_frag_type = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "selectolax/utils.pxi":93
 *     frag_type = get_fragment_type(html, parser_cls, tree)
 * 
 *     if frag_type == "document":             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     if frag_type == "document_no_head":
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_document, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(4, 93, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":94
 * 
 *     if frag_type == "document":
 *         return [tree.root]             # <<<<<<<<<<<<<<
 *     if frag_type == "document_no_head":
 *         tree.head.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 94, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 94, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_t_1) != (0)) __PYX_ERR(4, 94, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":93
 *     frag_type = get_fragment_type(html, parser_cls, tree)
 * 
 *     if frag_type == "document":             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     if frag_type == "document_no_head":
*/
  }

  /* "selectolax/utils.pxi":95
 *     if frag_type == "document":
 *         return [tree.root]
 *     if frag_type == "document_no_head":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_document_no_head, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(4, 95, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":96
 *         return [tree.root]
 *     if frag_type == "document_no_head":
 *         tree.head.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     if frag_type == "document_no_body":
*/
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, NULL};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 96, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_6, __pyx_callargs+1, 0) < 0) __PYX_ERR(4, 96, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 96, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "selectolax/utils.pxi":97
 *     if frag_type == "document_no_head":
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]             # <<<<<<<<<<<<<<
 *     if frag_type == "document_no_body":
 *         tree.body.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 97, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 97, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_t_2) != (0)) __PYX_ERR(4, 97, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":95
 *     if frag_type == "document":
 *         return [tree.root]
 *     if frag_type == "document_no_head":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]
*/
  }

  /* "selectolax/utils.pxi":98
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]
 *     if frag_type == "document_no_body":             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_document_no_body, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(4, 98, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":99
 *         return [tree.root]
 *     if frag_type == "document_no_body":
 *         tree.body.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     if frag_type == "document_no_head_no_body":
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 99, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = __pyx_t_6;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
      __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_1, __pyx_callargs+1, 0) < 0) __PYX_ERR(4, 99, __pyx_L1_error)
      __pyx_t_4 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/utils.pxi":100
 *     if frag_type == "document_no_body":
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]             # <<<<<<<<<<<<<<
 *     if frag_type == "document_no_head_no_body":
 *         tree.head.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 100, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyList_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 100, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 0, __pyx_t_4) != (0)) __PYX_ERR(4, 100, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":98
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]
 *     if frag_type == "document_no_body":             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
*/
  }

  /* "selectolax/utils.pxi":101
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
 *     if frag_type == "document_no_head_no_body":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         tree.body.decompose(recursive=True)
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_document_no_head_no_body, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(4, 101, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":102
 *         return [tree.root]
 *     if frag_type == "document_no_head_no_body":
 *         tree.head.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __pyx_t_1;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_4, NULL};
      __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 102, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_2, __pyx_callargs+1, 0) < 0) __PYX_ERR(4, 102, __pyx_L1_error)
      __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 102, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "selectolax/utils.pxi":103
 *     if frag_type == "document_no_head_no_body":
 *         tree.head.decompose(recursive=True)
 *         tree.body.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     elif frag_type == "head":
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, NULL};
      __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_4, __pyx_callargs+1, 0) < 0) __PYX_ERR(4, 103, __pyx_L1_error)
      __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "selectolax/utils.pxi":104
 *         tree.head.decompose(recursive=True)
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]             # <<<<<<<<<<<<<<
 *     elif frag_type == "head":
 *         tree.body.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_6);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_t_6) != (0)) __PYX_ERR(4, 104, __pyx_L1_error);
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":101
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
 *     if frag_type == "document_no_head_no_body":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         tree.body.decompose(recursive=True)
*/
  }

  /* "selectolax/utils.pxi":105
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
 *     elif frag_type == "head":             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_head_2, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(4, 105, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":106
 *         return [tree.root]
 *     elif frag_type == "head":
 *         tree.body.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.head]
 *     elif frag_type == "body":
*/
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_6, NULL};
      __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_1, __pyx_callargs+1, 0) < 0) __PYX_ERR(4, 106, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "selectolax/utils.pxi":107
 *     elif frag_type == "head":
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]             # <<<<<<<<<<<<<<
 *     elif frag_type == "body":
 *         tree.head.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_t_2) != (0)) __PYX_ERR(4, 107, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":105
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
 *     elif frag_type == "head":             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]
*/
  }

  /* "selectolax/utils.pxi":108
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]
 *     elif frag_type == "body":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_body_2, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(4, 108, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":109
 *         return [tree.head]
 *     elif frag_type == "body":
 *         tree.head.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.body]
 *     elif frag_type == "head_and_body":
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 109, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_6, __pyx_callargs+1, 0) < 0) __PYX_ERR(4, 109, __pyx_L1_error)
      __pyx_t_4 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 109, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/utils.pxi":110
 *     elif frag_type == "body":
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]             # <<<<<<<<<<<<<<
 *     elif frag_type == "head_and_body":
 *         return [tree.head, tree.body]
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_t_4) != (0)) __PYX_ERR(4, 110, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":108
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]
 *     elif frag_type == "body":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]
*/
  }

  /* "selectolax/utils.pxi":111
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]
 *     elif frag_type == "head_and_body":             # <<<<<<<<<<<<<<
 *         return [tree.head, tree.body]
 *     else:
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_head_and_body, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(4, 111, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":112
 *         return [tree.body]
 *     elif frag_type == "head_and_body":
 *         return [tree.head, tree.body]             # <<<<<<<<<<<<<<
 *     else:
 *         return [
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyList_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 0, __pyx_t_1) != (0)) __PYX_ERR(4, 112, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 1, __pyx_t_4) != (0)) __PYX_ERR(4, 112, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":111
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]
 *     elif frag_type == "head_and_body":             # <<<<<<<<<<<<<<
 *         return [tree.head, tree.body]
 *     else:
*/
  }

  /* "selectolax/utils.pxi":114
 *         return [tree.head, tree.body]
 *     else:
 *         return [             # <<<<<<<<<<<<<<
 *             *tree.head.iter(include_text=True),
 *             *tree.body.iter(include_text=True),
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);

    /* "selectolax/utils.pxi":115
 *     else:
 *         return [
 *             *tree.head.iter(include_text=True),             # <<<<<<<<<<<<<<
 *             *tree.body.iter(include_text=True),
 *         ]
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 115, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, NULL};
      __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(4, 115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_include_text, Py_True, __pyx_t_7, __pyx_callargs+1, 0) < 0) __PYX_ERR(4, 115, __pyx_L1_error)
      __pyx_t_4 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_iter_2, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __pyx_t_6 = __Pyx_PySequence_ListKeepNew(__pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/utils.pxi":116
 *         return [
 *             *tree.head.iter(include_text=True),
 *             *tree.body.iter(include_text=True),             # <<<<<<<<<<<<<<
 *         ]
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(4, 116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_2 = __pyx_t_7;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
      __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_include_text, Py_True, __pyx_t_1, __pyx_callargs+1, 0) < 0) __PYX_ERR(4, 116, __pyx_L1_error)
      __pyx_t_4 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_iter_2, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    if (__Pyx_PyList_Extend(__pyx_t_6, __pyx_t_4) < 0) __PYX_ERR(4, 116, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "selectolax/utils.pxi":81
 * 
 * 
 * def do_parse_fragment(html: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     """
 *     Given HTML, parse it into a list of Nodes, such that the nodes
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("selectolax.parser.do_parse_fragment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tree);
  __Pyx_XDECREF(__pyx_v_frag_type);
  __Pyx_XDECREF(__pyx_v_html);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/util.pxi":4
 * 
 * 
 * def create_tag(tag: str):             # <<<<<<<<<<<<<<
 *     """
 *     Given an HTML tag name, e.g. `"div"`, create a single empty node for that tag,
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_9create_tag(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_8create_tag, "create_tag(tag: str)\n\nGiven an HTML tag name, e.g. `\"div\"`, create a single empty node for that tag,\ne.g. `\"<div></div>\"`.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_9create_tag = {"create_tag", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_9create_tag, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8create_tag};
static PyObject *__pyx_pw_10selectolax_6parser_9create_tag(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_tag = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_tag (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_tag,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(5, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(5, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "create_tag", 0) < 0) __PYX_ERR(5, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("create_tag", 1, 1, 1, i); __PYX_ERR(5, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(5, 4, __pyx_L3_error)
    }
    __pyx_v_tag = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_tag", 1, 1, 1, __pyx_nargs); __PYX_ERR(5, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.create_tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_8create_tag(__pyx_self, __pyx_v_tag);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_8create_tag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_tag", 0);

  /* "selectolax/modest/util.pxi":9
 *     e.g. `"<div></div>"`.
 *     """
 *     return do_create_tag(tag, HTMLParser)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_do_create_tag); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_v_tag, ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser)};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/util.pxi":4
 * 
 * 
 * def create_tag(tag: str):             # <<<<<<<<<<<<<<
 *     """
 *     Given an HTML tag name, e.g. `"div"`, create a single empty node for that tag,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.create_tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/modest/util.pxi":12
 * 
 * 
 * def parse_fragment(html: str):             # <<<<<<<<<<<<<<
 *     """
 *     Given HTML, parse it into a list of Nodes, such that the nodes
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_11parse_fragment(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10parse_fragment, "parse_fragment(html: str)\n\nGiven HTML, parse it into a list of Nodes, such that the nodes\ncorrespond to the given HTML.\n\nFor contrast, HTMLParser adds `<html>`, `<head>`, and `<body>` tags\nif they are missing. This function does not add these tags.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_11parse_fragment = {"parse_fragment", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11parse_fragment, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10parse_fragment};
static PyObject *__pyx_pw_10selectolax_6parser_11parse_fragment(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_html = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("parse_fragment (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_html,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(5, 12, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(5, 12, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "parse_fragment", 0) < 0) __PYX_ERR(5, 12, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("parse_fragment", 1, 1, 1, i); __PYX_ERR(5, 12, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(5, 12, __pyx_L3_error)
    }
    __pyx_v_html = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("parse_fragment", 1, 1, 1, __pyx_nargs); __PYX_ERR(5, 12, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.parse_fragment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_10parse_fragment(__pyx_self, __pyx_v_html);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10parse_fragment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("parse_fragment", 0);

  /* "selectolax/modest/util.pxi":20
 *     if they are missing. This function does not add these tags.
 *     """
 *     return do_parse_fragment(html, HTMLParser)             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_do_parse_fragment); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_v_html, ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser)};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 20, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/modest/util.pxi":12
 * 
 * 
 * def parse_fragment(html: str):             # <<<<<<<<<<<<<<
 *     """
 *     Given HTML, parse it into a list of Nodes, such that the nodes
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.parse_fragment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/utils.pxi":19
 * 
 * 
 * def preprocess_input(html, decode_errors='ignore'):             # <<<<<<<<<<<<<<
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_13preprocess_input(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_12preprocess_input, "preprocess_input(html, decode_errors='ignore')");
static PyMethodDef __pyx_mdef_10selectolax_6parser_13preprocess_input = {"preprocess_input", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_13preprocess_input, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_12preprocess_input};
static PyObject *__pyx_pw_10selectolax_6parser_13preprocess_input(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_html = 0;
  PyObject *__pyx_v_decode_errors = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("preprocess_input (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_html,&__pyx_mstate_global->__pyx_n_u_decode_errors,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(6, 19, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(6, 19, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(6, 19, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "preprocess_input", 0) < 0) __PYX_ERR(6, 19, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)((PyObject*)__pyx_mstate_global->__pyx_n_u_ignore)));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("preprocess_input", 0, 1, 2, i); __PYX_ERR(6, 19, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(6, 19, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(6, 19, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)((PyObject*)__pyx_mstate_global->__pyx_n_u_ignore)));
    }
    __pyx_v_html = values[0];
    __pyx_v_decode_errors = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("preprocess_input", 0, 1, 2, __pyx_nargs); __PYX_ERR(6, 19, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.preprocess_input", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_12preprocess_input(__pyx_self, __pyx_v_html, __pyx_v_decode_errors);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_12preprocess_input(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_decode_errors) {
  PyObject *__pyx_v_bytes_html = NULL;
  PyObject *__pyx_v_html_len = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("preprocess_input", 0);

  /* "selectolax/utils.pxi":20
 * 
 * def preprocess_input(html, decode_errors='ignore'):
 *     if isinstance(html, (str, unicode)):             # <<<<<<<<<<<<<<
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_html); 
  if (__pyx_t_1) {

    /* "selectolax/utils.pxi":21
 * def preprocess_input(html, decode_errors='ignore'):
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)             # <<<<<<<<<<<<<<
 *     elif isinstance(html, bytes):
 *         bytes_html = html
*/
    __pyx_t_3 = __pyx_v_html;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 21, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_errors, __pyx_v_decode_errors, __pyx_t_5, __pyx_callargs+2, 0) < 0) __PYX_ERR(6, 21, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 21, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_v_bytes_html = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "selectolax/utils.pxi":20
 * 
 * def preprocess_input(html, decode_errors='ignore'):
 *     if isinstance(html, (str, unicode)):             # <<<<<<<<<<<<<<
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):
*/
    goto __pyx_L3;
  }

  /* "selectolax/utils.pxi":22
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):             # <<<<<<<<<<<<<<
 *         bytes_html = html
 *     else:
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_html); 
  if (likely(__pyx_t_1)) {

    /* "selectolax/utils.pxi":23
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):
 *         bytes_html = html             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)
*/
    __Pyx_INCREF(__pyx_v_html);
    __pyx_v_bytes_html = __pyx_v_html;

    /* "selectolax/utils.pxi":22
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
 *     elif isinstance(html, bytes):             # <<<<<<<<<<<<<<
 *         bytes_html = html
 *     else:
*/
    goto __pyx_L3;
  }

  /* "selectolax/utils.pxi":25
 *         bytes_html = html
 *     else:
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)             # <<<<<<<<<<<<<<
 *     html_len = len(bytes_html)
 *     if html_len > MAX_HTML_INPUT_SIZE:
*/
  /*else*/ {
    __pyx_t_5 = NULL;
    __Pyx_INCREF(__pyx_builtin_TypeError);
    __pyx_t_3 = __pyx_builtin_TypeError; 
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_html)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Expected_a_string_but_s_found, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_7};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 25, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(6, 25, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "selectolax/utils.pxi":26
 *     else:
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)
 *     html_len = len(bytes_html)             # <<<<<<<<<<<<<<
 *     if html_len > MAX_HTML_INPUT_SIZE:
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)
*/
  __pyx_t_8 = PyObject_Length(__pyx_v_bytes_html); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(6, 26, __pyx_L1_error)
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_html_len = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "selectolax/utils.pxi":27
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)
 *     html_len = len(bytes_html)
 *     if html_len > MAX_HTML_INPUT_SIZE:             # <<<<<<<<<<<<<<
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)
 *     return bytes_html, html_len
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_MAX_HTML_INPUT_SIZE); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_html_len, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_1)) {

    /* "selectolax/utils.pxi":28
 *     html_len = len(bytes_html)
 *     if html_len > MAX_HTML_INPUT_SIZE:
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)             # <<<<<<<<<<<<<<
 *     return bytes_html, html_len
 * 
*/
    __pyx_t_2 = NULL;
    __Pyx_INCREF(__pyx_builtin_ValueError);
    __pyx_t_7 = __pyx_builtin_ValueError; 
    __pyx_t_5 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_The_specified_HTML_input_is_too, __pyx_v_html_len); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 28, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 28, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(6, 28, __pyx_L1_error)

    /* "selectolax/utils.pxi":27
 *         raise TypeError("Expected a string, but %s found" % type(html).__name__)
 *     html_len = len(bytes_html)
 *     if html_len > MAX_HTML_INPUT_SIZE:             # <<<<<<<<<<<<<<
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)
 *     return bytes_html, html_len
*/
  }

  /* "selectolax/utils.pxi":29
 *     if html_len > MAX_HTML_INPUT_SIZE:
 *         raise ValueError("The specified HTML input is too large to be processed (%d bytes)" % html_len)
 *     return bytes_html, html_len             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_bytes_html);
  __Pyx_GIVEREF(__pyx_v_bytes_html);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_bytes_html) != (0)) __PYX_ERR(6, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_html_len);
  __Pyx_GIVEREF(__pyx_v_html_len);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_html_len) != (0)) __PYX_ERR(6, 29, __pyx_L1_error);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "selectolax/utils.pxi":19
 * 
 * 
 * def preprocess_input(html, decode_errors='ignore'):             # <<<<<<<<<<<<<<
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("selectolax.parser.preprocess_input", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_html);
  __Pyx_XDECREF(__pyx_v_html_len);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/utils.pxi":32
 * 
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_15do_create_tag(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_14do_create_tag, "do_create_tag(tag: str, parser_cls: ParserCls)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_15do_create_tag = {"do_create_tag", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_15do_create_tag, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_14do_create_tag};
static PyObject *__pyx_pw_10selectolax_6parser_15do_create_tag(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_v_parser_cls = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("do_create_tag (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_tag,&__pyx_mstate_global->__pyx_n_u_parser_cls,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(6, 32, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(6, 32, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(6, 32, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "do_create_tag", 0) < 0) __PYX_ERR(6, 32, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("do_create_tag", 1, 2, 2, i); __PYX_ERR(6, 32, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(6, 32, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(6, 32, __pyx_L3_error)
    }
    __pyx_v_tag = values[0];
    __pyx_v_parser_cls = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("do_create_tag", 1, 2, 2, __pyx_nargs); __PYX_ERR(6, 32, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.do_create_tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_14do_create_tag(__pyx_self, __pyx_v_tag, __pyx_v_parser_cls);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_14do_create_tag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tag, PyObject *__pyx_v_parser_cls) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9[5];
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("do_create_tag", 0);

  /* "selectolax/utils.pxi":33
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):
 *     if not tag:             # <<<<<<<<<<<<<<
 *         raise ValueError("Tag name cannot be empty")
 *     return do_parse_fragment(f"<{tag}></{tag}>", parser_cls)[0]
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_tag); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 33, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_1);
  if (unlikely(__pyx_t_2)) {

    /* "selectolax/utils.pxi":34
 * def do_create_tag(tag: str, parser_cls: ParserCls):
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")             # <<<<<<<<<<<<<<
 *     return do_parse_fragment(f"<{tag}></{tag}>", parser_cls)[0]
 * 
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_builtin_ValueError);
    __pyx_t_5 = __pyx_builtin_ValueError; 
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Tag_name_cannot_be_empty};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 34, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(6, 34, __pyx_L1_error)

    /* "selectolax/utils.pxi":33
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):
 *     if not tag:             # <<<<<<<<<<<<<<
 *         raise ValueError("Tag name cannot be empty")
 *     return do_parse_fragment(f"<{tag}></{tag}>", parser_cls)[0]
*/
  }

  /* "selectolax/utils.pxi":35
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")
 *     return do_parse_fragment(f"<{tag}></{tag}>", parser_cls)[0]             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_do_parse_fragment); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_v_tag, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_tag, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9[0] = __pyx_mstate_global->__pyx_kp_u__2;
  __pyx_t_9[1] = __pyx_t_7;
  __pyx_t_9[2] = __pyx_mstate_global->__pyx_kp_u__9;
  __pyx_t_9[3] = __pyx_t_8;
  __pyx_t_9[4] = __pyx_mstate_global->__pyx_kp_u__10;
  __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_9, 5, 1 * 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 3 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8));
  if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_t_10, __pyx_v_parser_cls};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 35, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "selectolax/utils.pxi":32
 * 
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("selectolax.parser.do_create_tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/utils.pxi":38
 * 
 * 
 * def get_fragment_type(             # <<<<<<<<<<<<<<
 *     html: str,
 *     parser_cls: ParserCls,
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_17get_fragment_type(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_16get_fragment_type, "get_fragment_type(html: str, parser_cls: ParserCls, tree: Optional[Parser] = None) -> FRAGMENT");
static PyMethodDef __pyx_mdef_10selectolax_6parser_17get_fragment_type = {"get_fragment_type", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_17get_fragment_type, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_16get_fragment_type};
static PyObject *__pyx_pw_10selectolax_6parser_17get_fragment_type(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_html = 0;
  PyObject *__pyx_v_parser_cls = 0;
  PyObject *__pyx_v_tree = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_fragment_type (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_html,&__pyx_mstate_global->__pyx_n_u_parser_cls,&__pyx_mstate_global->__pyx_n_u_tree,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(6, 38, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(6, 38, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(6, 38, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(6, 38, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_fragment_type", 0) < 0) __PYX_ERR(6, 38, __pyx_L3_error)

      /* "selectolax/utils.pxi":41
 *     html: str,
 *     parser_cls: ParserCls,
 *     tree: Optional[Parser] = None,             # <<<<<<<<<<<<<<
 * ) -> FRAGMENT:
 *     if not tree:
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_fragment_type", 0, 2, 3, i); __PYX_ERR(6, 38, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(6, 38, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(6, 38, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(6, 38, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_html = values[0];
    __pyx_v_parser_cls = values[1];
    __pyx_v_tree = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_fragment_type", 0, 2, 3, __pyx_nargs); __PYX_ERR(6, 38, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.get_fragment_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_16get_fragment_type(__pyx_self, __pyx_v_html, __pyx_v_parser_cls, __pyx_v_tree);

  /* "selectolax/utils.pxi":38
 * 
 * 
 * def get_fragment_type(             # <<<<<<<<<<<<<<
 *     html: str,
 *     parser_cls: ParserCls,
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_16get_fragment_type(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_parser_cls, PyObject *__pyx_v_tree) {
  PyObject *__pyx_v_re = NULL;
  PyObject *__pyx_v_html_re = NULL;
  int __pyx_v_has_html;
  int __pyx_v_has_head;
  int __pyx_v_has_body;
  PyObject *__pyx_v_match = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_fragment_type", 0);
  __Pyx_INCREF(__pyx_v_tree);

  /* "selectolax/utils.pxi":43
 *     tree: Optional[Parser] = None,
 * ) -> FRAGMENT:
 *     if not tree:             # <<<<<<<<<<<<<<
 *         tree = parser_cls(html)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_tree); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 43, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_1);
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":44
 * ) -> FRAGMENT:
 *     if not tree:
 *         tree = parser_cls(html)             # <<<<<<<<<<<<<<
 * 
 *     import re
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_v_parser_cls);
    __pyx_t_5 = __pyx_v_parser_cls; 
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_html};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 44, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF_SET(__pyx_v_tree, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/utils.pxi":43
 *     tree: Optional[Parser] = None,
 * ) -> FRAGMENT:
 *     if not tree:             # <<<<<<<<<<<<<<
 *         tree = parser_cls(html)
 * 
*/
  }

  /* "selectolax/utils.pxi":46
 *         tree = parser_cls(html)
 * 
 *     import re             # <<<<<<<<<<<<<<
 *     html_re = re.compile(r"<html|<body|<head(?!er)", re.IGNORECASE)
 * 
*/
  __pyx_t_3 = __Pyx_ImportDottedModule(__pyx_mstate_global->__pyx_n_u_re, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_re = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "selectolax/utils.pxi":47
 * 
 *     import re
 *     html_re = re.compile(r"<html|<body|<head(?!er)", re.IGNORECASE)             # <<<<<<<<<<<<<<
 * 
 *     has_html = False
*/
  __pyx_t_5 = __pyx_v_re;
  __Pyx_INCREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_re, __pyx_mstate_global->__pyx_n_u_IGNORECASE); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_html_body_head_er, __pyx_t_4};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_compile, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 47, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_html_re = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "selectolax/utils.pxi":49
 *     html_re = re.compile(r"<html|<body|<head(?!er)", re.IGNORECASE)
 * 
 *     has_html = False             # <<<<<<<<<<<<<<
 *     has_head = False
 *     has_body = False
*/
  __pyx_v_has_html = 0;

  /* "selectolax/utils.pxi":50
 * 
 *     has_html = False
 *     has_head = False             # <<<<<<<<<<<<<<
 *     has_body = False
 *     for match in html_re.finditer(html):
*/
  __pyx_v_has_head = 0;

  /* "selectolax/utils.pxi":51
 *     has_html = False
 *     has_head = False
 *     has_body = False             # <<<<<<<<<<<<<<
 *     for match in html_re.finditer(html):
 *         if match[0] == "<html":
*/
  __pyx_v_has_body = 0;

  /* "selectolax/utils.pxi":52
 *     has_head = False
 *     has_body = False
 *     for match in html_re.finditer(html):             # <<<<<<<<<<<<<<
 *         if match[0] == "<html":
 *             has_html = True
*/
  __pyx_t_4 = __pyx_v_html_re;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_html};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_finditer, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4);
    __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 52, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(6, 52, __pyx_L1_error)
          #endif
          if (__pyx_t_7 >= __pyx_temp) break;
        }
        __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_4, __pyx_t_7);
        ++__pyx_t_7;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(6, 52, __pyx_L1_error)
          #endif
          if (__pyx_t_7 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_7));
        #else
        __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_7);
        #endif
        ++__pyx_t_7;
      }
      if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 52, __pyx_L1_error)
    } else {
      __pyx_t_3 = __pyx_t_8(__pyx_t_4);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(6, 52, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_match, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/utils.pxi":53
 *     has_body = False
 *     for match in html_re.finditer(html):
 *         if match[0] == "<html":             # <<<<<<<<<<<<<<
 *             has_html = True
 *         elif match[0] == "<head":
*/
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_match, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 53, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_html_2, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(6, 53, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "selectolax/utils.pxi":54
 *     for match in html_re.finditer(html):
 *         if match[0] == "<html":
 *             has_html = True             # <<<<<<<<<<<<<<
 *         elif match[0] == "<head":
 *             has_head = True
*/
      __pyx_v_has_html = 1;

      /* "selectolax/utils.pxi":53
 *     has_body = False
 *     for match in html_re.finditer(html):
 *         if match[0] == "<html":             # <<<<<<<<<<<<<<
 *             has_html = True
 *         elif match[0] == "<head":
*/
      goto __pyx_L6;
    }

    /* "selectolax/utils.pxi":55
 *         if match[0] == "<html":
 *             has_html = True
 *         elif match[0] == "<head":             # <<<<<<<<<<<<<<
 *             has_head = True
 *         elif match[0] == "<body":
*/
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_match, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_head, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(6, 55, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "selectolax/utils.pxi":56
 *             has_html = True
 *         elif match[0] == "<head":
 *             has_head = True             # <<<<<<<<<<<<<<
 *         elif match[0] == "<body":
 *             has_body = True
*/
      __pyx_v_has_head = 1;

      /* "selectolax/utils.pxi":55
 *         if match[0] == "<html":
 *             has_html = True
 *         elif match[0] == "<head":             # <<<<<<<<<<<<<<
 *             has_head = True
 *         elif match[0] == "<body":
*/
      goto __pyx_L6;
    }

    /* "selectolax/utils.pxi":57
 *         elif match[0] == "<head":
 *             has_head = True
 *         elif match[0] == "<body":             # <<<<<<<<<<<<<<
 *             has_body = True
 * 
*/
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_match, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_body, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(6, 57, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "selectolax/utils.pxi":58
 *             has_head = True
 *         elif match[0] == "<body":
 *             has_body = True             # <<<<<<<<<<<<<<
 * 
 *         if has_html and has_head and has_body:
*/
      __pyx_v_has_body = 1;

      /* "selectolax/utils.pxi":57
 *         elif match[0] == "<head":
 *             has_head = True
 *         elif match[0] == "<body":             # <<<<<<<<<<<<<<
 *             has_body = True
 * 
*/
    }
    __pyx_L6:;

    /* "selectolax/utils.pxi":60
 *             has_body = True
 * 
 *         if has_html and has_head and has_body:             # <<<<<<<<<<<<<<
 *             break
 * 
*/
    if (__pyx_v_has_html) {
    } else {
      __pyx_t_2 = __pyx_v_has_html;
      goto __pyx_L8_bool_binop_done;
    }
    if (__pyx_v_has_head) {
    } else {
      __pyx_t_2 = __pyx_v_has_head;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_2 = __pyx_v_has_body;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_2) {

      /* "selectolax/utils.pxi":61
 * 
 *         if has_html and has_head and has_body:
 *             break             # <<<<<<<<<<<<<<
 * 
 *     if has_html and has_head and has_body:
*/
      goto __pyx_L5_break;

      /* "selectolax/utils.pxi":60
 *             has_body = True
 * 
 *         if has_html and has_head and has_body:             # <<<<<<<<<<<<<<
 *             break
 * 
*/
    }

    /* "selectolax/utils.pxi":52
 *     has_head = False
 *     has_body = False
 *     for match in html_re.finditer(html):             # <<<<<<<<<<<<<<
 *         if match[0] == "<html":
 *             has_html = True
*/
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  goto __pyx_L11_for_end;
  __pyx_L5_break:;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  goto __pyx_L11_for_end;
  __pyx_L11_for_end:;

  /* "selectolax/utils.pxi":63
 *             break
 * 
 *     if has_html and has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "document"
 *     elif has_html and not has_head and has_body:
*/
  if (__pyx_v_has_html) {
  } else {
    __pyx_t_2 = __pyx_v_has_html;
    goto __pyx_L13_bool_binop_done;
  }
  if (__pyx_v_has_head) {
  } else {
    __pyx_t_2 = __pyx_v_has_head;
    goto __pyx_L13_bool_binop_done;
  }
  __pyx_t_2 = __pyx_v_has_body;
  __pyx_L13_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":64
 * 
 *     if has_html and has_head and has_body:
 *         return "document"             # <<<<<<<<<<<<<<
 *     elif has_html and not has_head and has_body:
 *         return "document_no_head"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_document);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_document;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":63
 *             break
 * 
 *     if has_html and has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "document"
 *     elif has_html and not has_head and has_body:
*/
  }

  /* "selectolax/utils.pxi":65
 *     if has_html and has_head and has_body:
 *         return "document"
 *     elif has_html and not has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:
*/
  if (__pyx_v_has_html) {
  } else {
    __pyx_t_2 = __pyx_v_has_html;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_head);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = __pyx_v_has_body;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":66
 *         return "document"
 *     elif has_html and not has_head and has_body:
 *         return "document_no_head"             # <<<<<<<<<<<<<<
 *     elif has_html and has_head and not has_body:
 *         return "document_no_body"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_document_no_head);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_document_no_head;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":65
 *     if has_html and has_head and has_body:
 *         return "document"
 *     elif has_html and not has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:
*/
  }

  /* "selectolax/utils.pxi":67
 *     elif has_html and not has_head and has_body:
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:
*/
  if (__pyx_v_has_html) {
  } else {
    __pyx_t_2 = __pyx_v_has_html;
    goto __pyx_L19_bool_binop_done;
  }
  if (__pyx_v_has_head) {
  } else {
    __pyx_t_2 = __pyx_v_has_head;
    goto __pyx_L19_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_body);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L19_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":68
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:
 *         return "document_no_body"             # <<<<<<<<<<<<<<
 *     elif has_html and not has_head and not has_body:
 *         return "document_no_head_no_body"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_document_no_body);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_document_no_body;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":67
 *     elif has_html and not has_head and has_body:
 *         return "document_no_head"
 *     elif has_html and has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:
*/
  }

  /* "selectolax/utils.pxi":69
 *     elif has_html and has_head and not has_body:
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:
*/
  if (__pyx_v_has_html) {
  } else {
    __pyx_t_2 = __pyx_v_has_html;
    goto __pyx_L22_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_head);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L22_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_body);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L22_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":70
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:
 *         return "document_no_head_no_body"             # <<<<<<<<<<<<<<
 *     elif has_head and not has_body:
 *         return "head"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_document_no_head_no_body);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_document_no_head_no_body;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":69
 *     elif has_html and has_head and not has_body:
 *         return "document_no_body"
 *     elif has_html and not has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:
*/
  }

  /* "selectolax/utils.pxi":71
 *     elif has_html and not has_head and not has_body:
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "head"
 *     elif not has_head and has_body:
*/
  if (__pyx_v_has_head) {
  } else {
    __pyx_t_2 = __pyx_v_has_head;
    goto __pyx_L25_bool_binop_done;
  }
  __pyx_t_1 = (!__pyx_v_has_body);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L25_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":72
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:
 *         return "head"             # <<<<<<<<<<<<<<
 *     elif not has_head and has_body:
 *         return "body"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_head_2);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_head_2;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":71
 *     elif has_html and not has_head and not has_body:
 *         return "document_no_head_no_body"
 *     elif has_head and not has_body:             # <<<<<<<<<<<<<<
 *         return "head"
 *     elif not has_head and has_body:
*/
  }

  /* "selectolax/utils.pxi":73
 *     elif has_head and not has_body:
 *         return "head"
 *     elif not has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "body"
 *     elif has_head and has_body:
*/
  __pyx_t_1 = (!__pyx_v_has_head);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L27_bool_binop_done;
  }
  __pyx_t_2 = __pyx_v_has_body;
  __pyx_L27_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":74
 *         return "head"
 *     elif not has_head and has_body:
 *         return "body"             # <<<<<<<<<<<<<<
 *     elif has_head and has_body:
 *         return "head_and_body"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_body_2);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_body_2;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":73
 *     elif has_head and not has_body:
 *         return "head"
 *     elif not has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "body"
 *     elif has_head and has_body:
*/
  }

  /* "selectolax/utils.pxi":75
 *     elif not has_head and has_body:
 *         return "body"
 *     elif has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "head_and_body"
 *     else:
*/
  if (__pyx_v_has_head) {
  } else {
    __pyx_t_2 = __pyx_v_has_head;
    goto __pyx_L29_bool_binop_done;
  }
  __pyx_t_2 = __pyx_v_has_body;
  __pyx_L29_bool_binop_done:;
  if (__pyx_t_2) {

    /* "selectolax/utils.pxi":76
 *         return "body"
 *     elif has_head and has_body:
 *         return "head_and_body"             # <<<<<<<<<<<<<<
 *     else:
 *         return "fragment"
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_head_and_body);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_head_and_body;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":75
 *     elif not has_head and has_body:
 *         return "body"
 *     elif has_head and has_body:             # <<<<<<<<<<<<<<
 *         return "head_and_body"
 *     else:
*/
  }

  /* "selectolax/utils.pxi":78
 *         return "head_and_body"
 *     else:
 *         return "fragment"             # <<<<<<<<<<<<<<
 * 
 * 
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_fragment);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_fragment;
    goto __pyx_L0;
  }

  /* "selectolax/utils.pxi":38
 * 
 * 
 * def get_fragment_type(             # <<<<<<<<<<<<<<
 *     html: str,
 *     parser_cls: ParserCls,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.get_fragment_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_re);
  __Pyx_XDECREF(__pyx_v_html_re);
  __Pyx_XDECREF(__pyx_v_match);
  __Pyx_XDECREF(__pyx_v_tree);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/utils.pxi":81
 * 
 * 
 * def do_parse_fragment(html: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     """
 *     Given HTML, parse it into a list of Nodes, such that the nodes
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_19do_parse_fragment(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_18do_parse_fragment, "do_parse_fragment(html: str, parser_cls: ParserCls)\n\nGiven HTML, parse it into a list of Nodes, such that the nodes\ncorrespond to the given HTML.\n\nFor contrast, HTMLParser adds `<html>`, `<head>`, and `<body>` tags\nif they are missing. This function does not add these tags.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_19do_parse_fragment = {"do_parse_fragment", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_19do_parse_fragment, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_18do_parse_fragment};
static PyObject *__pyx_pw_10selectolax_6parser_19do_parse_fragment(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_html = 0;
  PyObject *__pyx_v_parser_cls = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("do_parse_fragment (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_html,&__pyx_mstate_global->__pyx_n_u_parser_cls,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(6, 81, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(6, 81, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(6, 81, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "do_parse_fragment", 0) < 0) __PYX_ERR(6, 81, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("do_parse_fragment", 1, 2, 2, i); __PYX_ERR(6, 81, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(6, 81, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(6, 81, __pyx_L3_error)
    }
    __pyx_v_html = values[0];
    __pyx_v_parser_cls = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("do_parse_fragment", 1, 2, 2, __pyx_nargs); __PYX_ERR(6, 81, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.do_parse_fragment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_18do_parse_fragment(__pyx_self, __pyx_v_html, __pyx_v_parser_cls);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_18do_parse_fragment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_html, PyObject *__pyx_v_parser_cls) {
  PyObject *__pyx_v_tree = NULL;
  PyObject *__pyx_v_frag_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("do_parse_fragment", 0);
  __Pyx_INCREF(__pyx_v_html);

  /* "selectolax/utils.pxi":89
 *     if they are missing. This function does not add these tags.
 *     """
 *     html = html.strip()             # <<<<<<<<<<<<<<
 *     tree = parser_cls(html)
 *     frag_type = get_fragment_type(html, parser_cls, tree)
*/
  __pyx_t_2 = __pyx_v_html;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_strip, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF_SET(__pyx_v_html, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/utils.pxi":90
 *     """
 *     html = html.strip()
 *     tree = parser_cls(html)             # <<<<<<<<<<<<<<
 *     frag_type = get_fragment_type(html, parser_cls, tree)
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_parser_cls);
  __pyx_t_4 = __pyx_v_parser_cls; 
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_html};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_tree = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "selectolax/utils.pxi":91
 *     html = html.strip()
 *     tree = parser_cls(html)
 *     frag_type = get_fragment_type(html, parser_cls, tree)             # <<<<<<<<<<<<<<
 * 
 *     if frag_type == "document":
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_fragment_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_v_html, __pyx_v_parser_cls, __pyx_v_tree};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+__pyx_t_3, (4-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_frag_type = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "selectolax/utils.pxi":93
 *     frag_type = get_fragment_type(html, parser_cls, tree)
 * 
 *     if frag_type == "document":             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     if frag_type == "document_no_head":
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_document, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(6, 93, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":94
 * 
 *     if frag_type == "document":
 *         return [tree.root]             # <<<<<<<<<<<<<<
 *     if frag_type == "document_no_head":
 *         tree.head.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 94, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 94, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_t_1) != (0)) __PYX_ERR(6, 94, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":93
 *     frag_type = get_fragment_type(html, parser_cls, tree)
 * 
 *     if frag_type == "document":             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     if frag_type == "document_no_head":
*/
  }

  /* "selectolax/utils.pxi":95
 *     if frag_type == "document":
 *         return [tree.root]
 *     if frag_type == "document_no_head":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_document_no_head, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(6, 95, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":96
 *         return [tree.root]
 *     if frag_type == "document_no_head":
 *         tree.head.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     if frag_type == "document_no_body":
*/
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, NULL};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 96, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_6, __pyx_callargs+1, 0) < 0) __PYX_ERR(6, 96, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 96, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "selectolax/utils.pxi":97
 *     if frag_type == "document_no_head":
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]             # <<<<<<<<<<<<<<
 *     if frag_type == "document_no_body":
 *         tree.body.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 97, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 97, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_t_2) != (0)) __PYX_ERR(6, 97, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":95
 *     if frag_type == "document":
 *         return [tree.root]
 *     if frag_type == "document_no_head":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]
*/
  }

  /* "selectolax/utils.pxi":98
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]
 *     if frag_type == "document_no_body":             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_document_no_body, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(6, 98, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":99
 *         return [tree.root]
 *     if frag_type == "document_no_body":
 *         tree.body.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     if frag_type == "document_no_head_no_body":
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 99, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = __pyx_t_6;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
      __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_1, __pyx_callargs+1, 0) < 0) __PYX_ERR(6, 99, __pyx_L1_error)
      __pyx_t_4 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/utils.pxi":100
 *     if frag_type == "document_no_body":
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]             # <<<<<<<<<<<<<<
 *     if frag_type == "document_no_head_no_body":
 *         tree.head.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 100, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyList_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 100, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 0, __pyx_t_4) != (0)) __PYX_ERR(6, 100, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":98
 *         tree.head.decompose(recursive=True)
 *         return [tree.root]
 *     if frag_type == "document_no_body":             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
*/
  }

  /* "selectolax/utils.pxi":101
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
 *     if frag_type == "document_no_head_no_body":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         tree.body.decompose(recursive=True)
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_document_no_head_no_body, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(6, 101, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":102
 *         return [tree.root]
 *     if frag_type == "document_no_head_no_body":
 *         tree.head.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __pyx_t_1;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_4, NULL};
      __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 102, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_2, __pyx_callargs+1, 0) < 0) __PYX_ERR(6, 102, __pyx_L1_error)
      __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 102, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "selectolax/utils.pxi":103
 *     if frag_type == "document_no_head_no_body":
 *         tree.head.decompose(recursive=True)
 *         tree.body.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.root]
 *     elif frag_type == "head":
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, NULL};
      __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_4, __pyx_callargs+1, 0) < 0) __PYX_ERR(6, 103, __pyx_L1_error)
      __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "selectolax/utils.pxi":104
 *         tree.head.decompose(recursive=True)
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]             # <<<<<<<<<<<<<<
 *     elif frag_type == "head":
 *         tree.body.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_6);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_t_6) != (0)) __PYX_ERR(6, 104, __pyx_L1_error);
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":101
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
 *     if frag_type == "document_no_head_no_body":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         tree.body.decompose(recursive=True)
*/
  }

  /* "selectolax/utils.pxi":105
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
 *     elif frag_type == "head":             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_head_2, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(6, 105, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":106
 *         return [tree.root]
 *     elif frag_type == "head":
 *         tree.body.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.head]
 *     elif frag_type == "body":
*/
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_6, NULL};
      __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_1, __pyx_callargs+1, 0) < 0) __PYX_ERR(6, 106, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "selectolax/utils.pxi":107
 *     elif frag_type == "head":
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]             # <<<<<<<<<<<<<<
 *     elif frag_type == "body":
 *         tree.head.decompose(recursive=True)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_t_2) != (0)) __PYX_ERR(6, 107, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":105
 *         tree.body.decompose(recursive=True)
 *         return [tree.root]
 *     elif frag_type == "head":             # <<<<<<<<<<<<<<
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]
*/
  }

  /* "selectolax/utils.pxi":108
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]
 *     elif frag_type == "body":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_body_2, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(6, 108, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":109
 *         return [tree.head]
 *     elif frag_type == "body":
 *         tree.head.decompose(recursive=True)             # <<<<<<<<<<<<<<
 *         return [tree.body]
 *     elif frag_type == "head_and_body":
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 109, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_recursive, Py_True, __pyx_t_6, __pyx_callargs+1, 0) < 0) __PYX_ERR(6, 109, __pyx_L1_error)
      __pyx_t_4 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_decompose, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 109, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/utils.pxi":110
 *     elif frag_type == "body":
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]             # <<<<<<<<<<<<<<
 *     elif frag_type == "head_and_body":
 *         return [tree.head, tree.body]
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_t_4) != (0)) __PYX_ERR(6, 110, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":108
 *         tree.body.decompose(recursive=True)
 *         return [tree.head]
 *     elif frag_type == "body":             # <<<<<<<<<<<<<<
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]
*/
  }

  /* "selectolax/utils.pxi":111
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]
 *     elif frag_type == "head_and_body":             # <<<<<<<<<<<<<<
 *         return [tree.head, tree.body]
 *     else:
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_frag_type, __pyx_mstate_global->__pyx_n_u_head_and_body, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(6, 111, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "selectolax/utils.pxi":112
 *         return [tree.body]
 *     elif frag_type == "head_and_body":
 *         return [tree.head, tree.body]             # <<<<<<<<<<<<<<
 *     else:
 *         return [
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyList_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 0, __pyx_t_1) != (0)) __PYX_ERR(6, 112, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 1, __pyx_t_4) != (0)) __PYX_ERR(6, 112, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "selectolax/utils.pxi":111
 *         tree.head.decompose(recursive=True)
 *         return [tree.body]
 *     elif frag_type == "head_and_body":             # <<<<<<<<<<<<<<
 *         return [tree.head, tree.body]
 *     else:
*/
  }

  /* "selectolax/utils.pxi":114
 *         return [tree.head, tree.body]
 *     else:
 *         return [             # <<<<<<<<<<<<<<
 *             *tree.head.iter(include_text=True),
 *             *tree.body.iter(include_text=True),
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);

    /* "selectolax/utils.pxi":115
 *     else:
 *         return [
 *             *tree.head.iter(include_text=True),             # <<<<<<<<<<<<<<
 *             *tree.body.iter(include_text=True),
 *         ]
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_head_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 115, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, NULL};
      __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_include_text, Py_True, __pyx_t_7, __pyx_callargs+1, 0) < 0) __PYX_ERR(6, 115, __pyx_L1_error)
      __pyx_t_4 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_iter_2, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __pyx_t_6 = __Pyx_PySequence_ListKeepNew(__pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "selectolax/utils.pxi":116
 *         return [
 *             *tree.head.iter(include_text=True),
 *             *tree.body.iter(include_text=True),             # <<<<<<<<<<<<<<
 *         ]
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_tree, __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_2 = __pyx_t_7;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
      __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_include_text, Py_True, __pyx_t_1, __pyx_callargs+1, 0) < 0) __PYX_ERR(6, 116, __pyx_L1_error)
      __pyx_t_4 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_iter_2, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    if (__Pyx_PyList_Extend(__pyx_t_6, __pyx_t_4) < 0) __PYX_ERR(6, 116, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "selectolax/utils.pxi":81
 * 
 * 
 * def do_parse_fragment(html: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     """
 *     Given HTML, parse it into a list of Nodes, such that the nodes
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("selectolax.parser.do_parse_fragment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tree);
  __Pyx_XDECREF(__pyx_v_frag_type);
  __Pyx_XDECREF(__pyx_v_html);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":26
 *         Same as in builtin's str.decode, i.e 'strict', 'ignore' or 'replace'.
 *     """
 *     def __init__(self, html, detect_encoding=True, use_meta_tags=True, decode_errors = 'ignore'):             # <<<<<<<<<<<<<<
 * 
 *         cdef size_t html_len
*/

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_10HTMLParser_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_10selectolax_6parser_10HTMLParser_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_html = 0;
  PyObject *__pyx_v_detect_encoding = 0;
  PyObject *__pyx_v_use_meta_tags = 0;
  PyObject *__pyx_v_decode_errors = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_html,&__pyx_mstate_global->__pyx_n_u_detect_encoding,&__pyx_mstate_global->__pyx_n_u_use_meta_tags,&__pyx_mstate_global->__pyx_n_u_decode_errors,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 26, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 26, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 26, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 26, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 26, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < 0) __PYX_ERR(0, 26, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_ignore));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 4, i); __PYX_ERR(0, 26, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 26, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 26, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 26, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 26, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_ignore));
    }
    __pyx_v_html = values[0];
    __pyx_v_detect_encoding = values[1];
    __pyx_v_use_meta_tags = values[2];
    __pyx_v_decode_errors = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 4, __pyx_nargs); __PYX_ERR(0, 26, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser___init__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_html, __pyx_v_detect_encoding, __pyx_v_use_meta_tags, __pyx_v_decode_errors);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_10HTMLParser___init__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_html, PyObject *__pyx_v_detect_encoding, PyObject *__pyx_v_use_meta_tags, PyObject *__pyx_v_decode_errors) {
  size_t __pyx_v_html_len;
  char *__pyx_v_html_chars;
  PyObject *__pyx_v_bytes_html = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *(*__pyx_t_7)(PyObject *);
  char *__pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "selectolax/parser.pyx":31
 *         cdef char* html_chars
 * 
 *         self.detect_encoding = detect_encoding             # <<<<<<<<<<<<<<
 *         self.use_meta_tags = use_meta_tags
 *         self.decode_errors = decode_errors
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_detect_encoding); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_self->detect_encoding = __pyx_t_1;

  /* "selectolax/parser.pyx":32
 * 
 *         self.detect_encoding = detect_encoding
 *         self.use_meta_tags = use_meta_tags             # <<<<<<<<<<<<<<
 *         self.decode_errors = decode_errors
 *         self._encoding = MyENCODING_UTF_8
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_use_meta_tags); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 32, __pyx_L1_error)
  __pyx_v_self->use_meta_tags = __pyx_t_1;

  /* "selectolax/parser.pyx":33
 *         self.detect_encoding = detect_encoding
 *         self.use_meta_tags = use_meta_tags
 *         self.decode_errors = decode_errors             # <<<<<<<<<<<<<<
 *         self._encoding = MyENCODING_UTF_8
 * 
*/
  __pyx_t_2 = __pyx_v_decode_errors;
  __Pyx_INCREF(__pyx_t_2);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->decode_errors);
  __Pyx_DECREF(__pyx_v_self->decode_errors);
  __pyx_v_self->decode_errors = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "selectolax/parser.pyx":34
 *         self.use_meta_tags = use_meta_tags
 *         self.decode_errors = decode_errors
 *         self._encoding = MyENCODING_UTF_8             # <<<<<<<<<<<<<<
 * 
 *         bytes_html, html_len = preprocess_input(html, decode_errors)
*/
  __pyx_v_self->_encoding = MyENCODING_UTF_8;

  /* "selectolax/parser.pyx":36
 *         self._encoding = MyENCODING_UTF_8
 * 
 *         bytes_html, html_len = preprocess_input(html, decode_errors)             # <<<<<<<<<<<<<<
 *         html_chars = <char*> bytes_html
 * 
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_preprocess_input); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_html, __pyx_v_decode_errors};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
    PyObject* sequence = __pyx_t_2;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 36, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_3);
    } else {
      __pyx_t_4 = __Pyx_PyList_GetItemRef(sequence, 0);
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 36, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyList_GetItemRef(sequence, 1);
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 36, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_3);
    }
    #else
    __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_6 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_7 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6);
    index = 0; __pyx_t_4 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_4)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    index = 1; __pyx_t_3 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_6), 2) < 0) __PYX_ERR(0, 36, __pyx_L1_error)
    __pyx_t_7 = NULL;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 36, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_3); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_bytes_html = __pyx_t_4;
  __pyx_t_4 = 0;
  __pyx_v_html_len = __pyx_t_5;

  /* "selectolax/parser.pyx":37
 * 
 *         bytes_html, html_len = preprocess_input(html, decode_errors)
 *         html_chars = <char*> bytes_html             # <<<<<<<<<<<<<<
 * 
 *         if detect_encoding and isinstance(html, bytes):
*/
  __pyx_t_8 = __Pyx_PyObject_AsWritableString(__pyx_v_bytes_html); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 37, __pyx_L1_error)
  __pyx_v_html_chars = ((char *)__pyx_t_8);

  /* "selectolax/parser.pyx":39
 *         html_chars = <char*> bytes_html
 * 
 *         if detect_encoding and isinstance(html, bytes):             # <<<<<<<<<<<<<<
 *             self._detect_encoding(html_chars, html_len)
 * 
*/
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_detect_encoding); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 39, __pyx_L1_error)
  if (__pyx_t_9) {
  } else {
    __pyx_t_1 = __pyx_t_9;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_9 = PyBytes_Check(__pyx_v_html); 
  __pyx_t_1 = __pyx_t_9;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":40
 * 
 *         if detect_encoding and isinstance(html, bytes):
 *             self._detect_encoding(html_chars, html_len)             # <<<<<<<<<<<<<<
 * 
 *         self._parse_html(html_chars, html_len)
*/
    ((struct __pyx_vtabstruct_10selectolax_6parser_HTMLParser *)__pyx_v_self->__pyx_vtab)->_detect_encoding(__pyx_v_self, __pyx_v_html_chars, __pyx_v_html_len); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)

    /* "selectolax/parser.pyx":39
 *         html_chars = <char*> bytes_html
 * 
 *         if detect_encoding and isinstance(html, bytes):             # <<<<<<<<<<<<<<
 *             self._detect_encoding(html_chars, html_len)
 * 
*/
  }

  /* "selectolax/parser.pyx":42
 *             self._detect_encoding(html_chars, html_len)
 * 
 *         self._parse_html(html_chars, html_len)             # <<<<<<<<<<<<<<
 * 
 *         self.raw_html = bytes_html
*/
  __pyx_t_10 = ((struct __pyx_vtabstruct_10selectolax_6parser_HTMLParser *)__pyx_v_self->__pyx_vtab)->_parse_html(__pyx_v_self, __pyx_v_html_chars, __pyx_v_html_len); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 42, __pyx_L1_error)

  /* "selectolax/parser.pyx":44
 *         self._parse_html(html_chars, html_len)
 * 
 *         self.raw_html = bytes_html             # <<<<<<<<<<<<<<
 *         self.cached_script_texts = None
 *         self.cached_script_srcs = None
*/
  __pyx_t_2 = __pyx_v_bytes_html;
  __Pyx_INCREF(__pyx_t_2);
  if (!(likely(PyBytes_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_2))) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->raw_html);
  __Pyx_DECREF(__pyx_v_self->raw_html);
  __pyx_v_self->raw_html = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "selectolax/parser.pyx":45
 * 
 *         self.raw_html = bytes_html
 *         self.cached_script_texts = None             # <<<<<<<<<<<<<<
 *         self.cached_script_srcs = None
 * 
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->cached_script_texts);
  __Pyx_DECREF(__pyx_v_self->cached_script_texts);
  __pyx_v_self->cached_script_texts = Py_None;

  /* "selectolax/parser.pyx":46
 *         self.raw_html = bytes_html
 *         self.cached_script_texts = None
 *         self.cached_script_srcs = None             # <<<<<<<<<<<<<<
 * 
 *     def css(self, str query):
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->cached_script_srcs);
  __Pyx_DECREF(__pyx_v_self->cached_script_srcs);
  __pyx_v_self->cached_script_srcs = Py_None;

  /* "selectolax/parser.pyx":26
 *         Same as in builtin's str.decode, i.e 'strict', 'ignore' or 'replace'.
 *     """
 *     def __init__(self, html, detect_encoding=True, use_meta_tags=True, decode_errors = 'ignore'):             # <<<<<<<<<<<<<<
 * 
 *         cdef size_t html_len
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bytes_html);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":48
 *         self.cached_script_srcs = None
 * 
 *     def css(self, str query):             # <<<<<<<<<<<<<<
 *         """A CSS selector.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_3css(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_2css, "HTMLParser.css(self, str query)\n\nA CSS selector.\n\nMatches pattern `query` against HTML tree.\n`CSS selectors reference <https://www.w3schools.com/cssref/css_selectors.asp>`_.\n\nParameters\n----------\nquery : str\n    CSS selector (e.g. \"div > :nth-child(2n+1):not(:has(a))\").\n\nReturns\n-------\nselector : list of `Node` objects");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_3css = {"css", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_3css, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_2css};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_3css(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_query = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("css (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_query,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 48, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 48, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "css", 0) < 0) __PYX_ERR(0, 48, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("css", 1, 1, 1, i); __PYX_ERR(0, 48, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 48, __pyx_L3_error)
    }
    __pyx_v_query = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("css", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 48, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.css", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query), (&PyUnicode_Type), 1, "query", 1))) __PYX_ERR(0, 48, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_2css(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_query);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_2css(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_query) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("css", 0);

  /* "selectolax/parser.pyx":65
 *         """
 * 
 *         cdef Node node = Node.new(self.html_tree.node_html, self)             # <<<<<<<<<<<<<<
 *         return node.css(query)
 * 
*/
  __pyx_t_1 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_self->html_tree->node_html, __pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/parser.pyx":66
 * 
 *         cdef Node node = Node.new(self.html_tree.node_html, self)
 *         return node.css(query)             # <<<<<<<<<<<<<<
 * 
 *     def css_first(self, str query, default=None, strict=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_node);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_query};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_css, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":48
 *         self.cached_script_srcs = None
 * 
 *     def css(self, str query):             # <<<<<<<<<<<<<<
 *         """A CSS selector.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.css", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":68
 *         return node.css(query)
 * 
 *     def css_first(self, str query, default=None, strict=False):             # <<<<<<<<<<<<<<
 *         """Same as `css` but returns only the first match.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_5css_first(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_4css_first, "HTMLParser.css_first(self, str query, default=None, strict=False)\n\nSame as `css` but returns only the first match.\n\nParameters\n----------\n\nquery : str\ndefault : bool, default None\n    Default value to return if there is no match.\nstrict: bool, default True\n    Set to True if you want to check if there is strictly only one match in the document.\n\n\nReturns\n-------\nselector : `Node` object");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_5css_first = {"css_first", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_5css_first, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_4css_first};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_5css_first(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_query = 0;
  PyObject *__pyx_v_default = 0;
  PyObject *__pyx_v_strict = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("css_first (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_query,&__pyx_mstate_global->__pyx_n_u_default,&__pyx_mstate_global->__pyx_n_u_strict,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 68, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 68, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 68, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 68, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "css_first", 0) < 0) __PYX_ERR(0, 68, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("css_first", 0, 1, 3, i); __PYX_ERR(0, 68, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 68, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 68, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 68, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_False));
    }
    __pyx_v_query = ((PyObject*)values[0]);
    __pyx_v_default = values[1];
    __pyx_v_strict = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("css_first", 0, 1, 3, __pyx_nargs); __PYX_ERR(0, 68, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.css_first", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query), (&PyUnicode_Type), 1, "query", 1))) __PYX_ERR(0, 68, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_4css_first(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_query, __pyx_v_default, __pyx_v_strict);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4css_first(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_query, PyObject *__pyx_v_default, PyObject *__pyx_v_strict) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("css_first", 0);

  /* "selectolax/parser.pyx":87
 *         """
 * 
 *         cdef Node node = Node.new(self.html_tree.node_html, self)             # <<<<<<<<<<<<<<
 *         return node.css_first(query, default, strict)
 * 
*/
  __pyx_t_1 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_self->html_tree->node_html, __pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/parser.pyx":88
 * 
 *         cdef Node node = Node.new(self.html_tree.node_html, self)
 *         return node.css_first(query, default, strict)             # <<<<<<<<<<<<<<
 * 
 *     cdef void _detect_encoding(self, char* html, size_t html_len) nogil:
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_node);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_2, __pyx_v_query, __pyx_v_default, __pyx_v_strict};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_css_first, __pyx_callargs+__pyx_t_3, (4-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":68
 *         return node.css(query)
 * 
 *     def css_first(self, str query, default=None, strict=False):             # <<<<<<<<<<<<<<
 *         """Same as `css` but returns only the first match.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.css_first", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":90
 *         return node.css_first(query, default, strict)
 * 
 *     cdef void _detect_encoding(self, char* html, size_t html_len) nogil:             # <<<<<<<<<<<<<<
 *         cdef myencoding_t encoding = MyENCODING_DEFAULT
 * 
*/

static void __pyx_f_10selectolax_6parser_10HTMLParser__detect_encoding(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, char *__pyx_v_html, size_t __pyx_v_html_len) {
  myencoding_t __pyx_v_encoding;
  int __pyx_t_1;

  /* "selectolax/parser.pyx":91
 * 
 *     cdef void _detect_encoding(self, char* html, size_t html_len) nogil:
 *         cdef myencoding_t encoding = MyENCODING_DEFAULT             # <<<<<<<<<<<<<<
 * 
 *         if self.use_meta_tags:
*/
  __pyx_v_encoding = MyENCODING_DEFAULT;

  /* "selectolax/parser.pyx":93
 *         cdef myencoding_t encoding = MyENCODING_DEFAULT
 * 
 *         if self.use_meta_tags:             # <<<<<<<<<<<<<<
 *             encoding = myencoding_prescan_stream_to_determine_encoding(html, html_len)
 *             if encoding != MyENCODING_DEFAULT and encoding != MyENCODING_NOT_DETERMINED:
*/
  if (__pyx_v_self->use_meta_tags) {

    /* "selectolax/parser.pyx":94
 * 
 *         if self.use_meta_tags:
 *             encoding = myencoding_prescan_stream_to_determine_encoding(html, html_len)             # <<<<<<<<<<<<<<
 *             if encoding != MyENCODING_DEFAULT and encoding != MyENCODING_NOT_DETERMINED:
 *                 self._encoding = encoding
*/
    __pyx_v_encoding = myencoding_prescan_stream_to_determine_encoding(__pyx_v_html, __pyx_v_html_len);

    /* "selectolax/parser.pyx":95
 *         if self.use_meta_tags:
 *             encoding = myencoding_prescan_stream_to_determine_encoding(html, html_len)
 *             if encoding != MyENCODING_DEFAULT and encoding != MyENCODING_NOT_DETERMINED:             # <<<<<<<<<<<<<<
 *                 self._encoding = encoding
 *                 return
*/
    switch (__pyx_v_encoding) {
      case MyENCODING_DEFAULT:
      case MyENCODING_NOT_DETERMINED:
      __pyx_t_1 = 0;
      break;
      default:
      __pyx_t_1 = 1;
      break;
    }
    if (__pyx_t_1) {

      /* "selectolax/parser.pyx":96
 *             encoding = myencoding_prescan_stream_to_determine_encoding(html, html_len)
 *             if encoding != MyENCODING_DEFAULT and encoding != MyENCODING_NOT_DETERMINED:
 *                 self._encoding = encoding             # <<<<<<<<<<<<<<
 *                 return
 * 
*/
      __pyx_v_self->_encoding = __pyx_v_encoding;

      /* "selectolax/parser.pyx":97
 *             if encoding != MyENCODING_DEFAULT and encoding != MyENCODING_NOT_DETERMINED:
 *                 self._encoding = encoding
 *                 return             # <<<<<<<<<<<<<<
 * 
 *         if not myencoding_detect_bom(html, html_len, &encoding):
*/
      goto __pyx_L0;

      /* "selectolax/parser.pyx":95
 *         if self.use_meta_tags:
 *             encoding = myencoding_prescan_stream_to_determine_encoding(html, html_len)
 *             if encoding != MyENCODING_DEFAULT and encoding != MyENCODING_NOT_DETERMINED:             # <<<<<<<<<<<<<<
 *                 self._encoding = encoding
 *                 return
*/
    }

    /* "selectolax/parser.pyx":93
 *         cdef myencoding_t encoding = MyENCODING_DEFAULT
 * 
 *         if self.use_meta_tags:             # <<<<<<<<<<<<<<
 *             encoding = myencoding_prescan_stream_to_determine_encoding(html, html_len)
 *             if encoding != MyENCODING_DEFAULT and encoding != MyENCODING_NOT_DETERMINED:
*/
  }

  /* "selectolax/parser.pyx":99
 *                 return
 * 
 *         if not myencoding_detect_bom(html, html_len, &encoding):             # <<<<<<<<<<<<<<
 *             myencoding_detect(html, html_len, &encoding)
 * 
*/
  __pyx_t_1 = (!myencoding_detect_bom(__pyx_v_html, __pyx_v_html_len, (&__pyx_v_encoding)));
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":100
 * 
 *         if not myencoding_detect_bom(html, html_len, &encoding):
 *             myencoding_detect(html, html_len, &encoding)             # <<<<<<<<<<<<<<
 * 
 *         self._encoding = encoding
*/
    (void)(myencoding_detect(__pyx_v_html, __pyx_v_html_len, (&__pyx_v_encoding)));

    /* "selectolax/parser.pyx":99
 *                 return
 * 
 *         if not myencoding_detect_bom(html, html_len, &encoding):             # <<<<<<<<<<<<<<
 *             myencoding_detect(html, html_len, &encoding)
 * 
*/
  }

  /* "selectolax/parser.pyx":102
 *             myencoding_detect(html, html_len, &encoding)
 * 
 *         self._encoding = encoding             # <<<<<<<<<<<<<<
 * 
 *     cdef int _parse_html(self, char* html, size_t html_len) except -1:
*/
  __pyx_v_self->_encoding = __pyx_v_encoding;

  /* "selectolax/parser.pyx":90
 *         return node.css_first(query, default, strict)
 * 
 *     cdef void _detect_encoding(self, char* html, size_t html_len) nogil:             # <<<<<<<<<<<<<<
 *         cdef myencoding_t encoding = MyENCODING_DEFAULT
 * 
*/

  /* function exit code */
  __pyx_L0:;
}

/* "selectolax/parser.pyx":104
 *         self._encoding = encoding
 * 
 *     cdef int _parse_html(self, char* html, size_t html_len) except -1:             # <<<<<<<<<<<<<<
 *         cdef myhtml_t* myhtml
 *         cdef mystatus_t status
*/

static int __pyx_f_10selectolax_6parser_10HTMLParser__parse_html(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, char *__pyx_v_html, size_t __pyx_v_html_len) {
  myhtml_t *__pyx_v_myhtml;
  mystatus_t __pyx_v_status;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_parse_html", 0);

  /* "selectolax/parser.pyx":108
 *         cdef mystatus_t status
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             myhtml = myhtml_create()
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
*/
  {
      PyThreadState *_save;
      _save = NULL;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "selectolax/parser.pyx":109
 * 
 *         with nogil:
 *             myhtml = myhtml_create()             # <<<<<<<<<<<<<<
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
 * 
*/
        __pyx_v_myhtml = myhtml_create();

        /* "selectolax/parser.pyx":110
 *         with nogil:
 *             myhtml = myhtml_create()
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)             # <<<<<<<<<<<<<<
 * 
 *         if status != 0:
*/
        __pyx_v_status = myhtml_init(__pyx_v_myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0);
      }

      /* "selectolax/parser.pyx":108
 *         cdef mystatus_t status
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             myhtml = myhtml_create()
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "selectolax/parser.pyx":112
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't init MyHTML object.")
 *             return -1
*/
  __pyx_t_1 = (__pyx_v_status != 0);
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":113
 * 
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't init MyHTML object.")             # <<<<<<<<<<<<<<
 *             return -1
 * 
*/
    PyErr_SetObject(__pyx_builtin_RuntimeError, __pyx_mstate_global->__pyx_kp_u_Can_t_init_MyHTML_object);

    /* "selectolax/parser.pyx":114
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't init MyHTML object.")
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_r = -1;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":112
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't init MyHTML object.")
 *             return -1
*/
  }

  /* "selectolax/parser.pyx":116
 *             return -1
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self.html_tree = myhtml_tree_create()
 *             status = myhtml_tree_init(self.html_tree, myhtml)
*/
  {
      PyThreadState *_save;
      _save = NULL;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "selectolax/parser.pyx":117
 * 
 *         with nogil:
 *             self.html_tree = myhtml_tree_create()             # <<<<<<<<<<<<<<
 *             status = myhtml_tree_init(self.html_tree, myhtml)
 * 
*/
        __pyx_v_self->html_tree = myhtml_tree_create();

        /* "selectolax/parser.pyx":118
 *         with nogil:
 *             self.html_tree = myhtml_tree_create()
 *             status = myhtml_tree_init(self.html_tree, myhtml)             # <<<<<<<<<<<<<<
 * 
 *         if status != 0:
*/
        __pyx_v_status = myhtml_tree_init(__pyx_v_self->html_tree, __pyx_v_myhtml);
      }

      /* "selectolax/parser.pyx":116
 *             return -1
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self.html_tree = myhtml_tree_create()
 *             status = myhtml_tree_init(self.html_tree, myhtml)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "selectolax/parser.pyx":120
 *             status = myhtml_tree_init(self.html_tree, myhtml)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't init MyHTML Tree object.")
 *             return -1
*/
  __pyx_t_1 = (__pyx_v_status != 0);
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":121
 * 
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't init MyHTML Tree object.")             # <<<<<<<<<<<<<<
 *             return -1
 * 
*/
    PyErr_SetObject(__pyx_builtin_RuntimeError, __pyx_mstate_global->__pyx_kp_u_Can_t_init_MyHTML_Tree_object);

    /* "selectolax/parser.pyx":122
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't init MyHTML Tree object.")
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_r = -1;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":120
 *             status = myhtml_tree_init(self.html_tree, myhtml)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't init MyHTML Tree object.")
 *             return -1
*/
  }

  /* "selectolax/parser.pyx":124
 *             return -1
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             status = myhtml_parse(self.html_tree, self._encoding, html, html_len)
 * 
*/
  {
      PyThreadState *_save;
      _save = NULL;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "selectolax/parser.pyx":125
 * 
 *         with nogil:
 *             status = myhtml_parse(self.html_tree, self._encoding, html, html_len)             # <<<<<<<<<<<<<<
 * 
 *         if status != 0:
*/
        __pyx_v_status = myhtml_parse(__pyx_v_self->html_tree, __pyx_v_self->_encoding, __pyx_v_html, __pyx_v_html_len);
      }

      /* "selectolax/parser.pyx":124
 *             return -1
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             status = myhtml_parse(self.html_tree, self._encoding, html, html_len)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          goto __pyx_L13;
        }
        __pyx_L13:;
      }
  }

  /* "selectolax/parser.pyx":127
 *             status = myhtml_parse(self.html_tree, self._encoding, html, html_len)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't parse HTML (status code: %d)" % status)
 *             return -1
*/
  __pyx_t_1 = (__pyx_v_status != 0);
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":128
 * 
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't parse HTML (status code: %d)" % status)             # <<<<<<<<<<<<<<
 *             return -1
 * 
*/
    __pyx_t_2 = __Pyx_PyLong_From_mystatus_t(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Can_t_parse_HTML_status_code_d, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    PyErr_SetObject(__pyx_builtin_RuntimeError, __pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "selectolax/parser.pyx":129
 *         if status != 0:
 *             PyErr_SetObject(RuntimeError, "Can't parse HTML (status code: %d)" % status)
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *         if self.html_tree.node_html == NULL:
*/
    __pyx_r = -1;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":127
 *             status = myhtml_parse(self.html_tree, self._encoding, html, html_len)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "Can't parse HTML (status code: %d)" % status)
 *             return -1
*/
  }

  /* "selectolax/parser.pyx":131
 *             return -1
 * 
 *         if self.html_tree.node_html == NULL:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "html_tree is still NULL even after parsing ")
 *             return -1
*/
  __pyx_t_1 = (__pyx_v_self->html_tree->node_html == NULL);
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":132
 * 
 *         if self.html_tree.node_html == NULL:
 *             PyErr_SetObject(RuntimeError, "html_tree is still NULL even after parsing ")             # <<<<<<<<<<<<<<
 *             return -1
 *         return 0
*/
    PyErr_SetObject(__pyx_builtin_RuntimeError, __pyx_mstate_global->__pyx_kp_u_html_tree_is_still_NULL_even_aft);

    /* "selectolax/parser.pyx":133
 *         if self.html_tree.node_html == NULL:
 *             PyErr_SetObject(RuntimeError, "html_tree is still NULL even after parsing ")
 *             return -1             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
    __pyx_r = -1;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":131
 *             return -1
 * 
 *         if self.html_tree.node_html == NULL:             # <<<<<<<<<<<<<<
 *             PyErr_SetObject(RuntimeError, "html_tree is still NULL even after parsing ")
 *             return -1
*/
  }

  /* "selectolax/parser.pyx":134
 *             PyErr_SetObject(RuntimeError, "html_tree is still NULL even after parsing ")
 *             return -1
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":104
 *         self._encoding = encoding
 * 
 *     cdef int _parse_html(self, char* html, size_t html_len) except -1:             # <<<<<<<<<<<<<<
 *         cdef myhtml_t* myhtml
 *         cdef mystatus_t status
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser._parse_html", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":136
 *         return 0
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def input_encoding(self):
 *         """Return encoding of the HTML document.
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_14input_encoding_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_14input_encoding_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_14input_encoding___get__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_14input_encoding___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  char const *__pyx_v_encoding;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/parser.pyx":143
 *         """
 *         cdef const char* encoding
 *         encoding = myencoding_name_by_id(self._encoding, NULL)             # <<<<<<<<<<<<<<
 * 
 *         if encoding != NULL:
*/
  __pyx_v_encoding = myencoding_name_by_id(__pyx_v_self->_encoding, NULL);

  /* "selectolax/parser.pyx":145
 *         encoding = myencoding_name_by_id(self._encoding, NULL)
 * 
 *         if encoding != NULL:             # <<<<<<<<<<<<<<
 *             return encoding.decode('utf-8')
 *         else:
*/
  __pyx_t_1 = (__pyx_v_encoding != NULL);
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":146
 * 
 *         if encoding != NULL:
 *             return encoding.decode('utf-8')             # <<<<<<<<<<<<<<
 *         else:
 *             return 'unknown'
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_ssize_strlen(__pyx_v_encoding); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 146, __pyx_L1_error)
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_encoding, 0, __pyx_t_2, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":145
 *         encoding = myencoding_name_by_id(self._encoding, NULL)
 * 
 *         if encoding != NULL:             # <<<<<<<<<<<<<<
 *             return encoding.decode('utf-8')
 *         else:
*/
  }

  /* "selectolax/parser.pyx":148
 *             return encoding.decode('utf-8')
 *         else:
 *             return 'unknown'             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_unknown);
    __pyx_r = __pyx_mstate_global->__pyx_n_u_unknown;
    goto __pyx_L0;
  }

  /* "selectolax/parser.pyx":136
 *         return 0
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def input_encoding(self):
 *         """Return encoding of the HTML document.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.input_encoding.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":150
 *             return 'unknown'
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def root(self):
 *         """Returns root node."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_4root_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_4root_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_4root___get__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4root___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/parser.pyx":153
 *     def root(self):
 *         """Returns root node."""
 *         if self.html_tree and self.html_tree.node_html:             # <<<<<<<<<<<<<<
 *             try:
 *                 return Node.new(self.html_tree.node_html, self)
*/
  __pyx_t_2 = (__pyx_v_self->html_tree != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->html_tree->node_html != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":154
 *         """Returns root node."""
 *         if self.html_tree and self.html_tree.node_html:
 *             try:             # <<<<<<<<<<<<<<
 *                 return Node.new(self.html_tree.node_html, self)
 *             except Exception:
*/
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "selectolax/parser.pyx":155
 *         if self.html_tree and self.html_tree.node_html:
 *             try:
 *                 return Node.new(self.html_tree.node_html, self)             # <<<<<<<<<<<<<<
 *             except Exception:
 *                 # If Node creation or initialization fails, return None
*/
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_6 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_self->html_tree->node_html, __pyx_v_self)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 155, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_r = __pyx_t_6;
        __pyx_t_6 = 0;
        goto __pyx_L10_try_return;

        /* "selectolax/parser.pyx":154
 *         """Returns root node."""
 *         if self.html_tree and self.html_tree.node_html:
 *             try:             # <<<<<<<<<<<<<<
 *                 return Node.new(self.html_tree.node_html, self)
 *             except Exception:
*/
      }
      __pyx_L6_error:;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "selectolax/parser.pyx":156
 *             try:
 *                 return Node.new(self.html_tree.node_html, self)
 *             except Exception:             # <<<<<<<<<<<<<<
 *                 # If Node creation or initialization fails, return None
 *                 return None
*/
      __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
      if (__pyx_t_7) {
        __Pyx_ErrRestore(0,0,0);

        /* "selectolax/parser.pyx":158
 *             except Exception:
 *                 # If Node creation or initialization fails, return None
 *                 return None             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
        __Pyx_XDECREF(__pyx_r);
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);
        goto __pyx_L9_except_return;
      }
      goto __pyx_L8_except_error;

      /* "selectolax/parser.pyx":154
 *         """Returns root node."""
 *         if self.html_tree and self.html_tree.node_html:
 *             try:             # <<<<<<<<<<<<<<
 *                 return Node.new(self.html_tree.node_html, self)
 *             except Exception:
*/
      __pyx_L8_except_error:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L10_try_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
      __pyx_L9_except_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
    }

    /* "selectolax/parser.pyx":153
 *     def root(self):
 *         """Returns root node."""
 *         if self.html_tree and self.html_tree.node_html:             # <<<<<<<<<<<<<<
 *             try:
 *                 return Node.new(self.html_tree.node_html, self)
*/
  }

  /* "selectolax/parser.pyx":159
 *                 # If Node creation or initialization fails, return None
 *                 return None
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/parser.pyx":150
 *             return 'unknown'
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def root(self):
 *         """Returns root node."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.root.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":161
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def head(self):
 *         """Returns head node."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_4head_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_4head_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_4head___get__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4head___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  myhtml_tree_node_t *__pyx_v_head;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/parser.pyx":165
 *         """Returns head node."""
 *         cdef myhtml_tree_node_t* head
 *         head = myhtml_tree_get_node_head(self.html_tree)             # <<<<<<<<<<<<<<
 * 
 *         if head != NULL:
*/
  __pyx_v_head = myhtml_tree_get_node_head(__pyx_v_self->html_tree);

  /* "selectolax/parser.pyx":167
 *         head = myhtml_tree_get_node_head(self.html_tree)
 * 
 *         if head != NULL:             # <<<<<<<<<<<<<<
 *             return Node.new(head, self)
 *         return None
*/
  __pyx_t_1 = (__pyx_v_head != NULL);
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":168
 * 
 *         if head != NULL:
 *             return Node.new(head, self)             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_head, __pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":167
 *         head = myhtml_tree_get_node_head(self.html_tree)
 * 
 *         if head != NULL:             # <<<<<<<<<<<<<<
 *             return Node.new(head, self)
 *         return None
*/
  }

  /* "selectolax/parser.pyx":169
 *         if head != NULL:
 *             return Node.new(head, self)
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/parser.pyx":161
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def head(self):
 *         """Returns head node."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.head.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":171
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def body(self):
 *         """Returns document body."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_4body_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_4body_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_4body___get__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4body___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  myhtml_tree_node_t *__pyx_v_body;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/parser.pyx":175
 *         """Returns document body."""
 *         cdef myhtml_tree_node_t* body
 *         body = myhtml_tree_get_node_body(self.html_tree)             # <<<<<<<<<<<<<<
 * 
 *         if body != NULL:
*/
  __pyx_v_body = myhtml_tree_get_node_body(__pyx_v_self->html_tree);

  /* "selectolax/parser.pyx":177
 *         body = myhtml_tree_get_node_body(self.html_tree)
 * 
 *         if body != NULL:             # <<<<<<<<<<<<<<
 *             return Node.new(body, self)
 *         return None
*/
  __pyx_t_1 = (__pyx_v_body != NULL);
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":178
 * 
 *         if body != NULL:
 *             return Node.new(body, self)             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_body, __pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":177
 *         body = myhtml_tree_get_node_body(self.html_tree)
 * 
 *         if body != NULL:             # <<<<<<<<<<<<<<
 *             return Node.new(body, self)
 *         return None
*/
  }

  /* "selectolax/parser.pyx":179
 *         if body != NULL:
 *             return Node.new(body, self)
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     def tags(self, str name):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/parser.pyx":171
 *         return None
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def body(self):
 *         """Returns document body."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.body.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":181
 *         return None
 * 
 *     def tags(self, str name):             # <<<<<<<<<<<<<<
 *         """Returns a list of tags that match specified name.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_7tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_6tags, "HTMLParser.tags(self, str name)\n\nReturns a list of tags that match specified name.\n\nParameters\n----------\nname : str (e.g. div)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_7tags = {"tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_7tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_6tags};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_7tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("tags (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name_2,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 181, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 181, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "tags", 0) < 0) __PYX_ERR(0, 181, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("tags", 1, 1, 1, i); __PYX_ERR(0, 181, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 181, __pyx_L3_error)
    }
    __pyx_v_name = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("tags", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 181, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 181, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_6tags(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_6tags(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_name) {
  myhtml_collection_t *__pyx_v_collection;
  PyObject *__pyx_v_pybyte_name = NULL;
  mystatus_t __pyx_v_status;
  PyObject *__pyx_v_result = NULL;
  size_t __pyx_v_i;
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  char const *__pyx_t_8;
  size_t __pyx_t_9;
  size_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("tags", 0);

  /* "selectolax/parser.pyx":190
 *         """
 *         # Validate tag name
 *         if not name:             # <<<<<<<<<<<<<<
 *             raise ValueError("Tag name cannot be empty")
 *         if len(name) > 100:  # Reasonable limit for tag names
*/
  __pyx_t_1 = (__pyx_v_name != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_name) != 0);
  if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_1 < 0))) __PYX_ERR(0, 190, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_1);
  if (unlikely(__pyx_t_2)) {

    /* "selectolax/parser.pyx":191
 *         # Validate tag name
 *         if not name:
 *             raise ValueError("Tag name cannot be empty")             # <<<<<<<<<<<<<<
 *         if len(name) > 100:  # Reasonable limit for tag names
 *             raise ValueError("Tag name is too long")
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_builtin_ValueError);
    __pyx_t_5 = __pyx_builtin_ValueError; 
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Tag_name_cannot_be_empty};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 191, __pyx_L1_error)

    /* "selectolax/parser.pyx":190
 *         """
 *         # Validate tag name
 *         if not name:             # <<<<<<<<<<<<<<
 *             raise ValueError("Tag name cannot be empty")
 *         if len(name) > 100:  # Reasonable limit for tag names
*/
  }

  /* "selectolax/parser.pyx":192
 *         if not name:
 *             raise ValueError("Tag name cannot be empty")
 *         if len(name) > 100:  # Reasonable limit for tag names             # <<<<<<<<<<<<<<
 *             raise ValueError("Tag name is too long")
 * 
*/
  if (unlikely(__pyx_v_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 192, __pyx_L1_error)
  }
  __pyx_t_7 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_name); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 192, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_7 > 0x64);
  if (unlikely(__pyx_t_2)) {

    /* "selectolax/parser.pyx":193
 *             raise ValueError("Tag name cannot be empty")
 *         if len(name) > 100:  # Reasonable limit for tag names
 *             raise ValueError("Tag name is too long")             # <<<<<<<<<<<<<<
 * 
 *         cdef myhtml_collection_t* collection = NULL
*/
    __pyx_t_5 = NULL;
    __Pyx_INCREF(__pyx_builtin_ValueError);
    __pyx_t_4 = __pyx_builtin_ValueError; 
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_Tag_name_is_too_long};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 193, __pyx_L1_error)

    /* "selectolax/parser.pyx":192
 *         if not name:
 *             raise ValueError("Tag name cannot be empty")
 *         if len(name) > 100:  # Reasonable limit for tag names             # <<<<<<<<<<<<<<
 *             raise ValueError("Tag name is too long")
 * 
*/
  }

  /* "selectolax/parser.pyx":195
 *             raise ValueError("Tag name is too long")
 * 
 *         cdef myhtml_collection_t* collection = NULL             # <<<<<<<<<<<<<<
 *         pybyte_name = name.encode('UTF-8')
 *         cdef mystatus_t status = 0
*/
  __pyx_v_collection = NULL;

  /* "selectolax/parser.pyx":196
 * 
 *         cdef myhtml_collection_t* collection = NULL
 *         pybyte_name = name.encode('UTF-8')             # <<<<<<<<<<<<<<
 *         cdef mystatus_t status = 0
 * 
*/
  if (unlikely(__pyx_v_name == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 196, __pyx_L1_error)
  }
  __pyx_t_3 = PyUnicode_AsUTF8String(__pyx_v_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_pybyte_name = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "selectolax/parser.pyx":197
 *         cdef myhtml_collection_t* collection = NULL
 *         pybyte_name = name.encode('UTF-8')
 *         cdef mystatus_t status = 0             # <<<<<<<<<<<<<<
 * 
 *         result = list()
*/
  __pyx_v_status = 0;

  /* "selectolax/parser.pyx":199
 *         cdef mystatus_t status = 0
 * 
 *         result = list()             # <<<<<<<<<<<<<<
 *         collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)
 * 
*/
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_result = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "selectolax/parser.pyx":200
 * 
 *         result = list()
 *         collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)             # <<<<<<<<<<<<<<
 * 
 *         if collection == NULL:
*/
  __pyx_t_8 = __Pyx_PyBytes_AsString(__pyx_v_pybyte_name); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyBytes_GET_SIZE(__pyx_v_pybyte_name); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 200, __pyx_L1_error)
  __pyx_v_collection = myhtml_get_nodes_by_name(__pyx_v_self->html_tree, NULL, __pyx_t_8, __pyx_t_7, (&__pyx_v_status));

  /* "selectolax/parser.pyx":202
 *         collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)
 * 
 *         if collection == NULL:             # <<<<<<<<<<<<<<
 *             return result
 * 
*/
  __pyx_t_2 = (__pyx_v_collection == NULL);
  if (__pyx_t_2) {

    /* "selectolax/parser.pyx":203
 * 
 *         if collection == NULL:
 *             return result             # <<<<<<<<<<<<<<
 * 
 *         if status == 0:
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":202
 *         collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)
 * 
 *         if collection == NULL:             # <<<<<<<<<<<<<<
 *             return result
 * 
*/
  }

  /* "selectolax/parser.pyx":205
 *             return result
 * 
 *         if status == 0:             # <<<<<<<<<<<<<<
 *             for i in range(collection.length):
 *                 node = Node.new(collection.list[i], self)
*/
  __pyx_t_2 = (__pyx_v_status == 0);
  if (__pyx_t_2) {

    /* "selectolax/parser.pyx":206
 * 
 *         if status == 0:
 *             for i in range(collection.length):             # <<<<<<<<<<<<<<
 *                 node = Node.new(collection.list[i], self)
 *                 result.append(node)
*/
    __pyx_t_6 = __pyx_v_collection->length;
    __pyx_t_9 = __pyx_t_6;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "selectolax/parser.pyx":207
 *         if status == 0:
 *             for i in range(collection.length):
 *                 node = Node.new(collection.list[i], self)             # <<<<<<<<<<<<<<
 *                 result.append(node)
 * 
*/
      __pyx_t_3 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new((__pyx_v_collection->list[__pyx_v_i]), __pyx_v_self)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_node, ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "selectolax/parser.pyx":208
 *             for i in range(collection.length):
 *                 node = Node.new(collection.list[i], self)
 *                 result.append(node)             # <<<<<<<<<<<<<<
 * 
 *         myhtml_collection_destroy(collection)
*/
      __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_result, ((PyObject *)__pyx_v_node)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 208, __pyx_L1_error)
    }

    /* "selectolax/parser.pyx":205
 *             return result
 * 
 *         if status == 0:             # <<<<<<<<<<<<<<
 *             for i in range(collection.length):
 *                 node = Node.new(collection.list[i], self)
*/
  }

  /* "selectolax/parser.pyx":210
 *                 result.append(node)
 * 
 *         myhtml_collection_destroy(collection)             # <<<<<<<<<<<<<<
 * 
 *         return result
*/
  (void)(myhtml_collection_destroy(__pyx_v_collection));

  /* "selectolax/parser.pyx":212
 *         myhtml_collection_destroy(collection)
 * 
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def text(self, bool deep=True, str separator='', bool strip=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":181
 *         return None
 * 
 *     def tags(self, str name):             # <<<<<<<<<<<<<<
 *         """Returns a list of tags that match specified name.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pybyte_name);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":214
 *         return result
 * 
 *     def text(self, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Returns the text of the node including text of all its child nodes.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_9text(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_8text, "HTMLParser.text(self, bool deep=True, str separator='', bool strip=False)\n\nReturns the text of the node including text of all its child nodes.\n\nParameters\n----------\nstrip : bool, default False\n    If true, calls ``str.strip()`` on each text part to remove extra white spaces.\nseparator : str, default ''\n    The separator to use when joining text from different nodes.\ndeep : bool, default True\n    If True, includes text from all child nodes.\n\nReturns\n-------\ntext : str");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_9text = {"text", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_9text, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_8text};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_9text(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyLongObject *__pyx_v_deep = 0;
  PyObject *__pyx_v_separator = 0;
  PyLongObject *__pyx_v_strip = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("text (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_deep,&__pyx_mstate_global->__pyx_n_u_separator,&__pyx_mstate_global->__pyx_n_u_strip,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 214, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 214, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 214, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 214, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "text", 0) < 0) __PYX_ERR(0, 214, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)__pyx_mstate_global->__pyx_kp_u_));
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 214, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 214, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 214, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)__pyx_mstate_global->__pyx_kp_u_));
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_deep = ((PyLongObject *)values[0]);
    __pyx_v_separator = ((PyObject*)values[1]);
    __pyx_v_strip = ((PyLongObject *)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("text", 0, 0, 3, __pyx_nargs); __PYX_ERR(0, 214, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_deep), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "deep", 0))) __PYX_ERR(0, 214, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_separator), (&PyUnicode_Type), 1, "separator", 1))) __PYX_ERR(0, 214, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_strip), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "strip", 0))) __PYX_ERR(0, 214, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_8text(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_deep, __pyx_v_separator, __pyx_v_strip);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_8text(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyLongObject *__pyx_v_deep, PyObject *__pyx_v_separator, PyLongObject *__pyx_v_strip) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("text", 0);

  /* "selectolax/parser.pyx":231
 * 
 *         """
 *         if not self.body:             # <<<<<<<<<<<<<<
 *             return ""
 *         return self.body.text(deep=deep, separator=separator, strip=strip)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (!__pyx_t_2);
  if (__pyx_t_3) {

    /* "selectolax/parser.pyx":232
 *         """
 *         if not self.body:
 *             return ""             # <<<<<<<<<<<<<<
 *         return self.body.text(deep=deep, separator=separator, strip=strip)
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
    __pyx_r = __pyx_mstate_global->__pyx_kp_u_;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":231
 * 
 *         """
 *         if not self.body:             # <<<<<<<<<<<<<<
 *             return ""
 *         return self.body.text(deep=deep, separator=separator, strip=strip)
*/
  }

  /* "selectolax/parser.pyx":233
 *         if not self.body:
 *             return ""
 *         return self.body.text(deep=deep, separator=separator, strip=strip)             # <<<<<<<<<<<<<<
 * 
 *     def strip_tags(self, list tags, bool recursive = False):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_body_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __pyx_t_5;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_4, NULL};
    __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_deep, ((PyObject *)__pyx_v_deep), __pyx_t_7, __pyx_callargs+1, 0) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_separator, __pyx_v_separator, __pyx_t_7, __pyx_callargs+1, 1) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_strip, ((PyObject *)__pyx_v_strip), __pyx_t_7, __pyx_callargs+1, 2) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_text, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":214
 *         return result
 * 
 *     def text(self, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Returns the text of the node including text of all its child nodes.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":235
 *         return self.body.text(deep=deep, separator=separator, strip=strip)
 * 
 *     def strip_tags(self, list tags, bool recursive = False):             # <<<<<<<<<<<<<<
 *         """Remove specified tags from the node.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_11strip_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_10strip_tags, "HTMLParser.strip_tags(self, list tags, bool recursive=False)\n\nRemove specified tags from the node.\n\nParameters\n----------\ntags : list of str\n    List of tags to remove.\nrecursive : bool, default True\n    Whenever to delete all its child nodes\n\nExamples\n--------\n\n>>> tree = HTMLParser('<html><head></head><body><script></script><div>Hello world!</div></body></html>')\n>>> tags = ['head', 'style', 'script', 'xmp', 'iframe', 'noembed', 'noframes']\n>>> tree.strip_tags(tags)\n>>> tree.html\n'<html><body><div>Hello world!</div></body></html>'");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_11strip_tags = {"strip_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_11strip_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_10strip_tags};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_11strip_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_tags = 0;
  PyLongObject *__pyx_v_recursive = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("strip_tags (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_tags,&__pyx_mstate_global->__pyx_n_u_recursive,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 235, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 235, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 235, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "strip_tags", 0) < 0) __PYX_ERR(0, 235, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("strip_tags", 0, 1, 2, i); __PYX_ERR(0, 235, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 235, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 235, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_tags = ((PyObject*)values[0]);
    __pyx_v_recursive = ((PyLongObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("strip_tags", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 235, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.strip_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tags), (&PyList_Type), 1, "tags", 1))) __PYX_ERR(0, 235, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recursive), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "recursive", 0))) __PYX_ERR(0, 235, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_10strip_tags(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_tags, __pyx_v_recursive);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_10strip_tags(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_tags, PyLongObject *__pyx_v_recursive) {
  myhtml_collection_t *__pyx_v_collection;
  mystatus_t __pyx_v_status;
  PyObject *__pyx_v_tag = NULL;
  PyObject *__pyx_v_pybyte_name = NULL;
  size_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  char const *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  size_t __pyx_t_9;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("strip_tags", 0);

  /* "selectolax/parser.pyx":255
 * 
 *         """
 *         cdef myhtml_collection_t* collection = NULL             # <<<<<<<<<<<<<<
 * 
 *         cdef mystatus_t status = 0
*/
  __pyx_v_collection = NULL;

  /* "selectolax/parser.pyx":257
 *         cdef myhtml_collection_t* collection = NULL
 * 
 *         cdef mystatus_t status = 0             # <<<<<<<<<<<<<<
 * 
 *         for tag in tags:
*/
  __pyx_v_status = 0;

  /* "selectolax/parser.pyx":259
 *         cdef mystatus_t status = 0
 * 
 *         for tag in tags:             # <<<<<<<<<<<<<<
 *             pybyte_name = tag.encode('UTF-8')
 *             collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)
*/
  if (unlikely(__pyx_v_tags == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 259, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_tags; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 259, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_2);
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_tag, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/parser.pyx":260
 * 
 *         for tag in tags:
 *             pybyte_name = tag.encode('UTF-8')             # <<<<<<<<<<<<<<
 *             collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)
 * 
*/
    __pyx_t_4 = __pyx_v_tag;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_pybyte_name, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/parser.pyx":261
 *         for tag in tags:
 *             pybyte_name = tag.encode('UTF-8')
 *             collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)             # <<<<<<<<<<<<<<
 * 
 *             if collection == NULL:
*/
    __pyx_t_6 = __Pyx_PyObject_AsString(__pyx_v_pybyte_name); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 261, __pyx_L1_error)
    __pyx_t_7 = PyObject_Length(__pyx_v_pybyte_name); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 261, __pyx_L1_error)
    __pyx_v_collection = myhtml_get_nodes_by_name(__pyx_v_self->html_tree, NULL, __pyx_t_6, __pyx_t_7, (&__pyx_v_status));

    /* "selectolax/parser.pyx":263
 *             collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)
 * 
 *             if collection == NULL:             # <<<<<<<<<<<<<<
 *                 continue
 * 
*/
    __pyx_t_8 = (__pyx_v_collection == NULL);
    if (__pyx_t_8) {

      /* "selectolax/parser.pyx":264
 * 
 *             if collection == NULL:
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             if status != 0:
*/
      goto __pyx_L3_continue;

      /* "selectolax/parser.pyx":263
 *             collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)
 * 
 *             if collection == NULL:             # <<<<<<<<<<<<<<
 *                 continue
 * 
*/
    }

    /* "selectolax/parser.pyx":266
 *                 continue
 * 
 *             if status != 0:             # <<<<<<<<<<<<<<
 *                 continue
 * 
*/
    __pyx_t_8 = (__pyx_v_status != 0);
    if (__pyx_t_8) {

      /* "selectolax/parser.pyx":267
 * 
 *             if status != 0:
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             for i in range(collection.length):
*/
      goto __pyx_L3_continue;

      /* "selectolax/parser.pyx":266
 *                 continue
 * 
 *             if status != 0:             # <<<<<<<<<<<<<<
 *                 continue
 * 
*/
    }

    /* "selectolax/parser.pyx":269
 *                 continue
 * 
 *             for i in range(collection.length):             # <<<<<<<<<<<<<<
 *                 if recursive:
 *                     myhtml_node_delete_recursive(collection.list[i])
*/
    __pyx_t_5 = __pyx_v_collection->length;
    __pyx_t_9 = __pyx_t_5;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "selectolax/parser.pyx":270
 * 
 *             for i in range(collection.length):
 *                 if recursive:             # <<<<<<<<<<<<<<
 *                     myhtml_node_delete_recursive(collection.list[i])
 *                 else:
*/
      __pyx_t_8 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_recursive)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 270, __pyx_L1_error)
      if (__pyx_t_8) {

        /* "selectolax/parser.pyx":271
 *             for i in range(collection.length):
 *                 if recursive:
 *                     myhtml_node_delete_recursive(collection.list[i])             # <<<<<<<<<<<<<<
 *                 else:
 *                     myhtml_node_delete(collection.list[i])
*/
        myhtml_node_delete_recursive((__pyx_v_collection->list[__pyx_v_i]));

        /* "selectolax/parser.pyx":270
 * 
 *             for i in range(collection.length):
 *                 if recursive:             # <<<<<<<<<<<<<<
 *                     myhtml_node_delete_recursive(collection.list[i])
 *                 else:
*/
        goto __pyx_L9;
      }

      /* "selectolax/parser.pyx":273
 *                     myhtml_node_delete_recursive(collection.list[i])
 *                 else:
 *                     myhtml_node_delete(collection.list[i])             # <<<<<<<<<<<<<<
 * 
 *             myhtml_collection_destroy(collection)
*/
      /*else*/ {
        myhtml_node_delete((__pyx_v_collection->list[__pyx_v_i]));
      }
      __pyx_L9:;
    }

    /* "selectolax/parser.pyx":275
 *                     myhtml_node_delete(collection.list[i])
 * 
 *             myhtml_collection_destroy(collection)             # <<<<<<<<<<<<<<
 * 
 *     def unwrap_tags(self, list tags, delete_empty : bool = False):
*/
    (void)(myhtml_collection_destroy(__pyx_v_collection));

    /* "selectolax/parser.pyx":259
 *         cdef mystatus_t status = 0
 * 
 *         for tag in tags:             # <<<<<<<<<<<<<<
 *             pybyte_name = tag.encode('UTF-8')
 *             collection = myhtml_get_nodes_by_name(self.html_tree, NULL, pybyte_name, len(pybyte_name), &status)
*/
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "selectolax/parser.pyx":235
 *         return self.body.text(deep=deep, separator=separator, strip=strip)
 * 
 *     def strip_tags(self, list tags, bool recursive = False):             # <<<<<<<<<<<<<<
 *         """Remove specified tags from the node.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.strip_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tag);
  __Pyx_XDECREF(__pyx_v_pybyte_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":277
 *             myhtml_collection_destroy(collection)
 * 
 *     def unwrap_tags(self, list tags, delete_empty : bool = False):             # <<<<<<<<<<<<<<
 *         """Unwraps specified tags from the HTML tree.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_13unwrap_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_12unwrap_tags, "HTMLParser.unwrap_tags(self, list tags, delete_empty: bool = False)\n\nUnwraps specified tags from the HTML tree.\n\nWorks the same as th `unwrap` method, but applied to a list of tags.\n\nParameters\n----------\ntags : list\n    List of tags to remove.\ndelete_empty : bool, default False\n    If True, removes empty tags.\n\nExamples\n--------\n\n>>> tree = HTMLParser(\"<div><a href=\"\">Hello</a> <i>world</i>!</div>\")\n>>> tree.head.unwrap_tags(['i','a'])\n>>> tree.head.html\n'<body><div>Hello world!</div></body>'");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_13unwrap_tags = {"unwrap_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_13unwrap_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_12unwrap_tags};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_13unwrap_tags(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_tags = 0;
  PyObject *__pyx_v_delete_empty = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unwrap_tags (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_tags,&__pyx_mstate_global->__pyx_n_u_delete_empty,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 277, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 277, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 277, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "unwrap_tags", 0) < 0) __PYX_ERR(0, 277, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("unwrap_tags", 0, 1, 2, i); __PYX_ERR(0, 277, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 277, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 277, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_False));
    }
    __pyx_v_tags = ((PyObject*)values[0]);
    __pyx_v_delete_empty = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("unwrap_tags", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 277, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.unwrap_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tags), (&PyList_Type), 1, "tags", 1))) __PYX_ERR(0, 277, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_12unwrap_tags(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_tags, __pyx_v_delete_empty);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_12unwrap_tags(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_tags, PyObject *__pyx_v_delete_empty) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("unwrap_tags", 0);

  /* "selectolax/parser.pyx":297
 *         '<body><div>Hello world!</div></body>'
 *         """
 *         if self.root is not None:             # <<<<<<<<<<<<<<
 *             self.root.unwrap_tags(tags, delete_empty=delete_empty)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "selectolax/parser.pyx":298
 *         """
 *         if self.root is not None:
 *             self.root.unwrap_tags(tags, delete_empty=delete_empty)             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 298, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_v_tags};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_delete_empty, __pyx_v_delete_empty, __pyx_t_6, __pyx_callargs+2, 0) < 0) __PYX_ERR(0, 298, __pyx_L1_error)
      __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_unwrap_tags, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "selectolax/parser.pyx":297
 *         '<body><div>Hello world!</div></body>'
 *         """
 *         if self.root is not None:             # <<<<<<<<<<<<<<
 *             self.root.unwrap_tags(tags, delete_empty=delete_empty)
 * 
*/
  }

  /* "selectolax/parser.pyx":277
 *             myhtml_collection_destroy(collection)
 * 
 *     def unwrap_tags(self, list tags, delete_empty : bool = False):             # <<<<<<<<<<<<<<
 *         """Unwraps specified tags from the HTML tree.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.unwrap_tags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":300
 *             self.root.unwrap_tags(tags, delete_empty=delete_empty)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def html(self):
 *         """Return HTML representation of the page."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_4html_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_4html_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_4html___get__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_4html___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "selectolax/parser.pyx":304
 *         """Return HTML representation of the page."""
 *         cdef Node node
 *         if self.html_tree != NULL and self.html_tree.document != NULL:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.html_tree.document, self)
 *             return node.html
*/
  __pyx_t_2 = (__pyx_v_self->html_tree != NULL);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->html_tree->document != NULL);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":305
 *         cdef Node node
 *         if self.html_tree != NULL and self.html_tree.document != NULL:
 *             node = Node.new(self.html_tree.document, self)             # <<<<<<<<<<<<<<
 *             return node.html
 *         return None
*/
    __pyx_t_3 = ((PyObject *)__pyx_f_10selectolax_6parser_4Node_new(__pyx_v_self->html_tree->document, __pyx_v_self)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "selectolax/parser.pyx":306
 *         if self.html_tree != NULL and self.html_tree.document != NULL:
 *             node = Node.new(self.html_tree.document, self)
 *             return node.html             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_node), __pyx_mstate_global->__pyx_n_u_html); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 306, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":304
 *         """Return HTML representation of the page."""
 *         cdef Node node
 *         if self.html_tree != NULL and self.html_tree.document != NULL:             # <<<<<<<<<<<<<<
 *             node = Node.new(self.html_tree.document, self)
 *             return node.html
*/
  }

  /* "selectolax/parser.pyx":307
 *             node = Node.new(self.html_tree.document, self)
 *             return node.html
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     def select(self, query=None):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "selectolax/parser.pyx":300
 *             self.root.unwrap_tags(tags, delete_empty=delete_empty)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def html(self):
 *         """Return HTML representation of the page."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.html.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":309
 *         return None
 * 
 *     def select(self, query=None):             # <<<<<<<<<<<<<<
 *         """Select nodes given a CSS selector.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_15select(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_14select, "HTMLParser.select(self, query=None)\n\nSelect nodes given a CSS selector.\n\nWorks similarly to  the ``css`` method, but supports chained filtering and extra features.\n\nParameters\n----------\nquery : str or None\n    The CSS selector to use when searching for nodes.\n\nReturns\n-------\nselector : The `Selector` class.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_15select = {"select", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_15select, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_14select};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_15select(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_query = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("select (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_query,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 309, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 309, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "select", 0) < 0) __PYX_ERR(0, 309, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 309, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_query = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("select", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 309, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.select", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_14select(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_query);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_14select(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_query) {
  struct __pyx_obj_10selectolax_6parser_Node *__pyx_v_node = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("select", 0);

  /* "selectolax/parser.pyx":324
 *         """
 *         cdef Node node
 *         node = self.root             # <<<<<<<<<<<<<<
 *         if node:
 *             return Selector(node, query)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(0, 324, __pyx_L1_error)
  __pyx_v_node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "selectolax/parser.pyx":325
 *         cdef Node node
 *         node = self.root
 *         if node:             # <<<<<<<<<<<<<<
 *             return Selector(node, query)
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 325, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "selectolax/parser.pyx":326
 *         node = self.root
 *         if node:
 *             return Selector(node, query)             # <<<<<<<<<<<<<<
 * 
 *     def any_css_matches(self, tuple selectors):
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = NULL;
    __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector);
    __pyx_t_4 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector); 
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, ((PyObject *)__pyx_v_node), __pyx_v_query};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_GOTREF((PyObject *)__pyx_t_1);
    }
    __pyx_r = ((PyObject *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "selectolax/parser.pyx":325
 *         cdef Node node
 *         node = self.root
 *         if node:             # <<<<<<<<<<<<<<
 *             return Selector(node, query)
 * 
*/
  }

  /* "selectolax/parser.pyx":309
 *         return None
 * 
 *     def select(self, query=None):             # <<<<<<<<<<<<<<
 *         """Select nodes given a CSS selector.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.select", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":328
 *             return Selector(node, query)
 * 
 *     def any_css_matches(self, tuple selectors):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the specified CSS selectors matches a node."""
 *         return self.root.any_css_matches(selectors)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_17any_css_matches(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_16any_css_matches, "HTMLParser.any_css_matches(self, tuple selectors)\n\nReturns True if any of the specified CSS selectors matches a node.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_17any_css_matches = {"any_css_matches", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_17any_css_matches, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_16any_css_matches};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_17any_css_matches(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_selectors = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("any_css_matches (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_selectors,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 328, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 328, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "any_css_matches", 0) < 0) __PYX_ERR(0, 328, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("any_css_matches", 1, 1, 1, i); __PYX_ERR(0, 328, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 328, __pyx_L3_error)
    }
    __pyx_v_selectors = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("any_css_matches", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 328, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.any_css_matches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_selectors), (&PyTuple_Type), 1, "selectors", 1))) __PYX_ERR(0, 328, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_16any_css_matches(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_selectors);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_16any_css_matches(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_selectors) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("any_css_matches", 0);

  /* "selectolax/parser.pyx":330
 *     def any_css_matches(self, tuple selectors):
 *         """Returns True if any of the specified CSS selectors matches a node."""
 *         return self.root.any_css_matches(selectors)             # <<<<<<<<<<<<<<
 * 
 *     def scripts_contain(self, str query):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 330, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_selectors};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_any_css_matches, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":328
 *             return Selector(node, query)
 * 
 *     def any_css_matches(self, tuple selectors):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the specified CSS selectors matches a node."""
 *         return self.root.any_css_matches(selectors)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.any_css_matches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":332
 *         return self.root.any_css_matches(selectors)
 * 
 *     def scripts_contain(self, str query):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script tags contain specified text.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_19scripts_contain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_18scripts_contain, "HTMLParser.scripts_contain(self, str query)\n\nReturns True if any of the script tags contain specified text.\n\nCaches script tags on the first call to improve performance.\n\nParameters\n----------\nquery : str\n    The query to check.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_19scripts_contain = {"scripts_contain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_19scripts_contain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_18scripts_contain};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_19scripts_contain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_query = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("scripts_contain (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_query,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 332, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 332, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "scripts_contain", 0) < 0) __PYX_ERR(0, 332, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("scripts_contain", 1, 1, 1, i); __PYX_ERR(0, 332, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 332, __pyx_L3_error)
    }
    __pyx_v_query = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("scripts_contain", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 332, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.scripts_contain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query), (&PyUnicode_Type), 1, "query", 1))) __PYX_ERR(0, 332, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_18scripts_contain(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_query);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_18scripts_contain(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_query) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("scripts_contain", 0);

  /* "selectolax/parser.pyx":343
 * 
 *         """
 *         return self.root.scripts_contain(query)             # <<<<<<<<<<<<<<
 * 
 *     def script_srcs_contain(self, tuple queries):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_query};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_scripts_contain, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":332
 *         return self.root.any_css_matches(selectors)
 * 
 *     def scripts_contain(self, str query):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script tags contain specified text.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.scripts_contain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":345
 *         return self.root.scripts_contain(query)
 * 
 *     def script_srcs_contain(self, tuple queries):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script SRCs attributes contain on of the specified text.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_21script_srcs_contain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_20script_srcs_contain, "HTMLParser.script_srcs_contain(self, tuple queries)\n\nReturns True if any of the script SRCs attributes contain on of the specified text.\n\nCaches values on the first call to improve performance.\n\nParameters\n----------\nqueries : tuple of str");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_21script_srcs_contain = {"script_srcs_contain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_21script_srcs_contain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_20script_srcs_contain};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_21script_srcs_contain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_queries = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("script_srcs_contain (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_queries,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 345, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 345, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "script_srcs_contain", 0) < 0) __PYX_ERR(0, 345, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("script_srcs_contain", 1, 1, 1, i); __PYX_ERR(0, 345, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 345, __pyx_L3_error)
    }
    __pyx_v_queries = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("script_srcs_contain", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 345, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.script_srcs_contain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_queries), (&PyTuple_Type), 1, "queries", 1))) __PYX_ERR(0, 345, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_20script_srcs_contain(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_queries);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_20script_srcs_contain(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_queries) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("script_srcs_contain", 0);

  /* "selectolax/parser.pyx":355
 * 
 *         """
 *         return self.root.script_srcs_contain(queries)             # <<<<<<<<<<<<<<
 * 
 *     def css_matches(self, str selector):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 355, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_queries};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_script_srcs_contain, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":345
 *         return self.root.scripts_contain(query)
 * 
 *     def script_srcs_contain(self, tuple queries):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script SRCs attributes contain on of the specified text.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.script_srcs_contain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":357
 *         return self.root.script_srcs_contain(queries)
 * 
 *     def css_matches(self, str selector):             # <<<<<<<<<<<<<<
 *         return self.root.css_matches(selector)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_23css_matches(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_22css_matches, "HTMLParser.css_matches(self, str selector)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_23css_matches = {"css_matches", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_23css_matches, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_22css_matches};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_23css_matches(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_selector = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("css_matches (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_selector,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 357, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 357, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "css_matches", 0) < 0) __PYX_ERR(0, 357, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("css_matches", 1, 1, 1, i); __PYX_ERR(0, 357, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 357, __pyx_L3_error)
    }
    __pyx_v_selector = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("css_matches", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 357, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.css_matches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_selector), (&PyUnicode_Type), 1, "selector", 1))) __PYX_ERR(0, 357, __pyx_L1_error)
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_22css_matches(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v_selector);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_22css_matches(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_selector) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("css_matches", 0);

  /* "selectolax/parser.pyx":358
 * 
 *     def css_matches(self, str selector):
 *         return self.root.css_matches(selector)             # <<<<<<<<<<<<<<
 * 
 *     def merge_text_nodes(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_selector};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_css_matches, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":357
 *         return self.root.script_srcs_contain(queries)
 * 
 *     def css_matches(self, str selector):             # <<<<<<<<<<<<<<
 *         return self.root.css_matches(selector)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.css_matches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":360
 *         return self.root.css_matches(selector)
 * 
 *     def merge_text_nodes(self):             # <<<<<<<<<<<<<<
 *         """Iterates over all text nodes and merges all text nodes that are close to each other.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_25merge_text_nodes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_24merge_text_nodes, "HTMLParser.merge_text_nodes(self)\n\nIterates over all text nodes and merges all text nodes that are close to each other.\n\nThis is useful for text extraction.\nUse it when you need to strip HTML tags and merge \"dangling\" text.\n\nExamples\n--------\n\n>>> tree = HTMLParser(\"<div><p><strong>J</strong>ohn</p><p>Doe</p></div>\")\n>>> node = tree.css_first('div')\n>>> tree.unwrap_tags([\"strong\"])\n>>> tree.text(deep=True, separator=\" \", strip=True)\n\"J ohn Doe\" # Text extraction produces an extra space because the strong tag was removed.\n>>> node.merge_text_nodes()\n>>> tree.text(deep=True, separator=\" \", strip=True)\n\"John Doe\"");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_25merge_text_nodes = {"merge_text_nodes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_25merge_text_nodes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_24merge_text_nodes};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_25merge_text_nodes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("merge_text_nodes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("merge_text_nodes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("merge_text_nodes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_24merge_text_nodes(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_24merge_text_nodes(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("merge_text_nodes", 0);

  /* "selectolax/parser.pyx":378
 *         "John Doe"
 *         """
 *         return self.root.merge_text_nodes()             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 378, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_merge_text_nodes, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":360
 *         return self.root.css_matches(selector)
 * 
 *     def merge_text_nodes(self):             # <<<<<<<<<<<<<<
 *         """Iterates over all text nodes and merges all text nodes that are close to each other.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.merge_text_nodes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":380
 *         return self.root.merge_text_nodes()
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef HTMLParser from_tree(
 *             myhtml_tree_t * tree, bytes raw_html, bint detect_encoding, bint use_meta_tags, str decode_errors,
*/

static struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_f_10selectolax_6parser_10HTMLParser_from_tree(myhtml_tree_t *__pyx_v_tree, PyObject *__pyx_v_raw_html, int __pyx_v_detect_encoding, int __pyx_v_use_meta_tags, PyObject *__pyx_v_decode_errors, myencoding_t __pyx_v_encoding) {
  struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_obj = NULL;
  struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_tree", 0);

  /* "selectolax/parser.pyx":385
 *             myencoding_t encoding
 *     ):
 *         obj = <HTMLParser> HTMLParser.__new__(HTMLParser)             # <<<<<<<<<<<<<<
 *         obj.html_tree = tree
 *         obj.raw_html = raw_html
*/
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_10selectolax_6parser_HTMLParser(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 385, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_t_1);
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF((PyObject *)__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_obj = ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "selectolax/parser.pyx":386
 *     ):
 *         obj = <HTMLParser> HTMLParser.__new__(HTMLParser)
 *         obj.html_tree = tree             # <<<<<<<<<<<<<<
 *         obj.raw_html = raw_html
 *         obj.detect_encoding = detect_encoding
*/
  __pyx_v_obj->html_tree = __pyx_v_tree;

  /* "selectolax/parser.pyx":387
 *         obj = <HTMLParser> HTMLParser.__new__(HTMLParser)
 *         obj.html_tree = tree
 *         obj.raw_html = raw_html             # <<<<<<<<<<<<<<
 *         obj.detect_encoding = detect_encoding
 *         obj.use_meta_tags = use_meta_tags
*/
  __Pyx_INCREF(__pyx_v_raw_html);
  __Pyx_GIVEREF(__pyx_v_raw_html);
  __Pyx_GOTREF(__pyx_v_obj->raw_html);
  __Pyx_DECREF(__pyx_v_obj->raw_html);
  __pyx_v_obj->raw_html = __pyx_v_raw_html;

  /* "selectolax/parser.pyx":388
 *         obj.html_tree = tree
 *         obj.raw_html = raw_html
 *         obj.detect_encoding = detect_encoding             # <<<<<<<<<<<<<<
 *         obj.use_meta_tags = use_meta_tags
 *         obj.decode_errors = decode_errors
*/
  __pyx_v_obj->detect_encoding = __pyx_v_detect_encoding;

  /* "selectolax/parser.pyx":389
 *         obj.raw_html = raw_html
 *         obj.detect_encoding = detect_encoding
 *         obj.use_meta_tags = use_meta_tags             # <<<<<<<<<<<<<<
 *         obj.decode_errors = decode_errors
 *         obj._encoding = encoding
*/
  __pyx_v_obj->use_meta_tags = __pyx_v_use_meta_tags;

  /* "selectolax/parser.pyx":390
 *         obj.detect_encoding = detect_encoding
 *         obj.use_meta_tags = use_meta_tags
 *         obj.decode_errors = decode_errors             # <<<<<<<<<<<<<<
 *         obj._encoding = encoding
 *         obj.cached_script_texts = None
*/
  __Pyx_INCREF(__pyx_v_decode_errors);
  __Pyx_GIVEREF(__pyx_v_decode_errors);
  __Pyx_GOTREF(__pyx_v_obj->decode_errors);
  __Pyx_DECREF(__pyx_v_obj->decode_errors);
  __pyx_v_obj->decode_errors = __pyx_v_decode_errors;

  /* "selectolax/parser.pyx":391
 *         obj.use_meta_tags = use_meta_tags
 *         obj.decode_errors = decode_errors
 *         obj._encoding = encoding             # <<<<<<<<<<<<<<
 *         obj.cached_script_texts = None
 *         obj.cached_script_srcs = None
*/
  __pyx_v_obj->_encoding = __pyx_v_encoding;

  /* "selectolax/parser.pyx":392
 *         obj.decode_errors = decode_errors
 *         obj._encoding = encoding
 *         obj.cached_script_texts = None             # <<<<<<<<<<<<<<
 *         obj.cached_script_srcs = None
 *         return obj
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_obj->cached_script_texts);
  __Pyx_DECREF(__pyx_v_obj->cached_script_texts);
  __pyx_v_obj->cached_script_texts = Py_None;

  /* "selectolax/parser.pyx":393
 *         obj._encoding = encoding
 *         obj.cached_script_texts = None
 *         obj.cached_script_srcs = None             # <<<<<<<<<<<<<<
 *         return obj
 * 
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_obj->cached_script_srcs);
  __Pyx_DECREF(__pyx_v_obj->cached_script_srcs);
  __pyx_v_obj->cached_script_srcs = Py_None;

  /* "selectolax/parser.pyx":394
 *         obj.cached_script_texts = None
 *         obj.cached_script_srcs = None
 *         return obj             # <<<<<<<<<<<<<<
 * 
 *     def clone(self):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":380
 *         return self.root.merge_text_nodes()
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef HTMLParser from_tree(
 *             myhtml_tree_t * tree, bytes raw_html, bint detect_encoding, bint use_meta_tags, str decode_errors,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.from_tree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_obj);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":396
 *         return obj
 * 
 *     def clone(self):             # <<<<<<<<<<<<<<
 *         """Clone the current tree."""
 *         cdef myhtml_t* myhtml
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_27clone(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_26clone, "HTMLParser.clone(self)\n\nClone the current tree.");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_27clone = {"clone", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_27clone, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_26clone};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_27clone(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("clone", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("clone", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_26clone(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_26clone(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  myhtml_t *__pyx_v_myhtml;
  mystatus_t __pyx_v_status;
  myhtml_tree_t *__pyx_v_html_tree;
  myhtml_tree_node_t *__pyx_v_node;
  struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_cls = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);

  /* "selectolax/parser.pyx":404
 *         cdef HTMLParser cls
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             myhtml = myhtml_create()
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
*/
  {
      PyThreadState *_save;
      _save = NULL;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "selectolax/parser.pyx":405
 * 
 *         with nogil:
 *             myhtml = myhtml_create()             # <<<<<<<<<<<<<<
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
 * 
*/
        __pyx_v_myhtml = myhtml_create();

        /* "selectolax/parser.pyx":406
 *         with nogil:
 *             myhtml = myhtml_create()
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)             # <<<<<<<<<<<<<<
 * 
 *         if status != 0:
*/
        __pyx_v_status = myhtml_init(__pyx_v_myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0);
      }

      /* "selectolax/parser.pyx":404
 *         cdef HTMLParser cls
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             myhtml = myhtml_create()
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "selectolax/parser.pyx":408
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Can't init MyHTML object.")
 * 
*/
  __pyx_t_1 = (__pyx_v_status != 0);
  if (unlikely(__pyx_t_1)) {

    /* "selectolax/parser.pyx":409
 * 
 *         if status != 0:
 *             raise RuntimeError("Can't init MyHTML object.")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_builtin_RuntimeError);
    __pyx_t_4 = __pyx_builtin_RuntimeError; 
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Can_t_init_MyHTML_object};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 409, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 409, __pyx_L1_error)

    /* "selectolax/parser.pyx":408
 *             status = myhtml_init(myhtml, MyHTML_OPTIONS_DEFAULT, 1, 0)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Can't init MyHTML object.")
 * 
*/
  }

  /* "selectolax/parser.pyx":411
 *             raise RuntimeError("Can't init MyHTML object.")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             html_tree = myhtml_tree_create()
 *             status = myhtml_tree_init(html_tree, myhtml)
*/
  {
      PyThreadState *_save;
      _save = NULL;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "selectolax/parser.pyx":412
 * 
 *         with nogil:
 *             html_tree = myhtml_tree_create()             # <<<<<<<<<<<<<<
 *             status = myhtml_tree_init(html_tree, myhtml)
 * 
*/
        __pyx_v_html_tree = myhtml_tree_create();

        /* "selectolax/parser.pyx":413
 *         with nogil:
 *             html_tree = myhtml_tree_create()
 *             status = myhtml_tree_init(html_tree, myhtml)             # <<<<<<<<<<<<<<
 * 
 *         if status != 0:
*/
        __pyx_v_status = myhtml_tree_init(__pyx_v_html_tree, __pyx_v_myhtml);
      }

      /* "selectolax/parser.pyx":411
 *             raise RuntimeError("Can't init MyHTML object.")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             html_tree = myhtml_tree_create()
 *             status = myhtml_tree_init(html_tree, myhtml)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "selectolax/parser.pyx":415
 *             status = myhtml_tree_init(html_tree, myhtml)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Can't init MyHTML Tree object.")
 * 
*/
  __pyx_t_1 = (__pyx_v_status != 0);
  if (unlikely(__pyx_t_1)) {

    /* "selectolax/parser.pyx":416
 * 
 *         if status != 0:
 *             raise RuntimeError("Can't init MyHTML Tree object.")             # <<<<<<<<<<<<<<
 * 
 *         node = myhtml_node_clone_deep(html_tree, self.html_tree.node_html)
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_builtin_RuntimeError);
    __pyx_t_3 = __pyx_builtin_RuntimeError; 
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Can_t_init_MyHTML_Tree_object};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 416, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 416, __pyx_L1_error)

    /* "selectolax/parser.pyx":415
 *             status = myhtml_tree_init(html_tree, myhtml)
 * 
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Can't init MyHTML Tree object.")
 * 
*/
  }

  /* "selectolax/parser.pyx":418
 *             raise RuntimeError("Can't init MyHTML Tree object.")
 * 
 *         node = myhtml_node_clone_deep(html_tree, self.html_tree.node_html)             # <<<<<<<<<<<<<<
 *         myhtml_tree_node_add_child(html_tree.document, node)
 *         html_tree.node_html = node
*/
  __pyx_v_node = myhtml_node_clone_deep(__pyx_v_html_tree, __pyx_v_self->html_tree->node_html);

  /* "selectolax/parser.pyx":419
 * 
 *         node = myhtml_node_clone_deep(html_tree, self.html_tree.node_html)
 *         myhtml_tree_node_add_child(html_tree.document, node)             # <<<<<<<<<<<<<<
 *         html_tree.node_html = node
 * 
*/
  myhtml_tree_node_add_child(__pyx_v_html_tree->document, __pyx_v_node);

  /* "selectolax/parser.pyx":420
 *         node = myhtml_node_clone_deep(html_tree, self.html_tree.node_html)
 *         myhtml_tree_node_add_child(html_tree.document, node)
 *         html_tree.node_html = node             # <<<<<<<<<<<<<<
 * 
 *         cls = HTMLParser.from_tree(
*/
  __pyx_v_html_tree->node_html = __pyx_v_node;

  /* "selectolax/parser.pyx":424
 *         cls = HTMLParser.from_tree(
 *             html_tree,
 *             self.raw_html,             # <<<<<<<<<<<<<<
 *             self.detect_encoding,
 *             self.use_meta_tags,
*/
  __pyx_t_2 = __pyx_v_self->raw_html;
  __Pyx_INCREF(__pyx_t_2);

  /* "selectolax/parser.pyx":427
 *             self.detect_encoding,
 *             self.use_meta_tags,
 *             self.decode_errors,             # <<<<<<<<<<<<<<
 *             self._encoding
 *         )
*/
  __pyx_t_3 = __pyx_v_self->decode_errors;
  __Pyx_INCREF(__pyx_t_3);

  /* "selectolax/parser.pyx":422
 *         html_tree.node_html = node
 * 
 *         cls = HTMLParser.from_tree(             # <<<<<<<<<<<<<<
 *             html_tree,
 *             self.raw_html,
*/
  __pyx_t_4 = ((PyObject *)__pyx_f_10selectolax_6parser_10HTMLParser_from_tree(__pyx_v_html_tree, ((PyObject*)__pyx_t_2), __pyx_v_self->detect_encoding, __pyx_v_self->use_meta_tags, ((PyObject*)__pyx_t_3), __pyx_v_self->_encoding)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_cls = ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "selectolax/parser.pyx":430
 *             self._encoding
 *         )
 *         return cls             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_cls);
  __pyx_r = ((PyObject *)__pyx_v_cls);
  goto __pyx_L0;

  /* "selectolax/parser.pyx":396
 *         return obj
 * 
 *     def clone(self):             # <<<<<<<<<<<<<<
 *         """Clone the current tree."""
 *         cdef myhtml_t* myhtml
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_cls);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pyx":432
 *         return cls
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         cdef myhtml_t* myhtml
 * 
*/

/* Python wrapper */
static void __pyx_pw_10selectolax_6parser_10HTMLParser_29__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_10selectolax_6parser_10HTMLParser_29__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_10selectolax_6parser_10HTMLParser_28__dealloc__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_10selectolax_6parser_10HTMLParser_28__dealloc__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  myhtml_t *__pyx_v_myhtml;
  int __pyx_t_1;
  myhtml_t *__pyx_t_2;

  /* "selectolax/parser.pyx":435
 *         cdef myhtml_t* myhtml
 * 
 *         if self.html_tree != NULL:             # <<<<<<<<<<<<<<
 *             myhtml = self.html_tree.myhtml
 *             myhtml_tree_destroy(self.html_tree)
*/
  __pyx_t_1 = (__pyx_v_self->html_tree != NULL);
  if (__pyx_t_1) {

    /* "selectolax/parser.pyx":436
 * 
 *         if self.html_tree != NULL:
 *             myhtml = self.html_tree.myhtml             # <<<<<<<<<<<<<<
 *             myhtml_tree_destroy(self.html_tree)
 *             self.html_tree = NULL  # Prevent double-free
*/
    __pyx_t_2 = __pyx_v_self->html_tree->myhtml;
    __pyx_v_myhtml = __pyx_t_2;

    /* "selectolax/parser.pyx":437
 *         if self.html_tree != NULL:
 *             myhtml = self.html_tree.myhtml
 *             myhtml_tree_destroy(self.html_tree)             # <<<<<<<<<<<<<<
 *             self.html_tree = NULL  # Prevent double-free
 *             if myhtml != NULL:
*/
    (void)(myhtml_tree_destroy(__pyx_v_self->html_tree));

    /* "selectolax/parser.pyx":438
 *             myhtml = self.html_tree.myhtml
 *             myhtml_tree_destroy(self.html_tree)
 *             self.html_tree = NULL  # Prevent double-free             # <<<<<<<<<<<<<<
 *             if myhtml != NULL:
 *                 myhtml_destroy(myhtml)
*/
    __pyx_v_self->html_tree = NULL;

    /* "selectolax/parser.pyx":439
 *             myhtml_tree_destroy(self.html_tree)
 *             self.html_tree = NULL  # Prevent double-free
 *             if myhtml != NULL:             # <<<<<<<<<<<<<<
 *                 myhtml_destroy(myhtml)
 * 
*/
    __pyx_t_1 = (__pyx_v_myhtml != NULL);
    if (__pyx_t_1) {

      /* "selectolax/parser.pyx":440
 *             self.html_tree = NULL  # Prevent double-free
 *             if myhtml != NULL:
 *                 myhtml_destroy(myhtml)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
      (void)(myhtml_destroy(__pyx_v_myhtml));

      /* "selectolax/parser.pyx":439
 *             myhtml_tree_destroy(self.html_tree)
 *             self.html_tree = NULL  # Prevent double-free
 *             if myhtml != NULL:             # <<<<<<<<<<<<<<
 *                 myhtml_destroy(myhtml)
 * 
*/
    }

    /* "selectolax/parser.pyx":435
 *         cdef myhtml_t* myhtml
 * 
 *         if self.html_tree != NULL:             # <<<<<<<<<<<<<<
 *             myhtml = self.html_tree.myhtml
 *             myhtml_tree_destroy(self.html_tree)
*/
  }

  /* "selectolax/parser.pyx":432
 *         return cls
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         cdef myhtml_t* myhtml
 * 
*/

  /* function exit code */
}

/* "selectolax/parser.pyx":442
 *                 myhtml_destroy(myhtml)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<HTMLParser chars=%s>' % len(self.root.html)
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_31__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_31__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_30__repr__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_30__repr__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "selectolax/parser.pyx":443
 * 
 *     def __repr__(self):
 *         return '<HTMLParser chars=%s>' % len(self.root.html)             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_root); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_html); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_HTMLParser_chars_s, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "selectolax/parser.pyx":442
 *                 myhtml_destroy(myhtml)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<HTMLParser chars=%s>' % len(self.root.html)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pxd":552
 * cdef class HTMLParser:
 *     cdef myhtml_tree_t *html_tree
 *     cdef public bint detect_encoding             # <<<<<<<<<<<<<<
 *     cdef public bint use_meta_tags
 *     cdef myencoding_t _encoding
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_15detect_encoding_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_15detect_encoding_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_15detect_encoding___get__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_15detect_encoding___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->detect_encoding); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.detect_encoding.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_10HTMLParser_15detect_encoding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10selectolax_6parser_10HTMLParser_15detect_encoding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_15detect_encoding_2__set__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_10HTMLParser_15detect_encoding_2__set__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(7, 552, __pyx_L1_error)
  __pyx_v_self->detect_encoding = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.detect_encoding.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "selectolax/parser.pxd":553
 *     cdef myhtml_tree_t *html_tree
 *     cdef public bint detect_encoding
 *     cdef public bint use_meta_tags             # <<<<<<<<<<<<<<
 *     cdef myencoding_t _encoding
 *     cdef public unicode decode_errors
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_13use_meta_tags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_13use_meta_tags_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_13use_meta_tags___get__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_13use_meta_tags___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->use_meta_tags); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.use_meta_tags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_10HTMLParser_13use_meta_tags_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10selectolax_6parser_10HTMLParser_13use_meta_tags_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_13use_meta_tags_2__set__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_10HTMLParser_13use_meta_tags_2__set__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(7, 553, __pyx_L1_error)
  __pyx_v_self->use_meta_tags = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.use_meta_tags.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "selectolax/parser.pxd":555
 *     cdef public bint use_meta_tags
 *     cdef myencoding_t _encoding
 *     cdef public unicode decode_errors             # <<<<<<<<<<<<<<
 *     cdef public bytes raw_html
 *     cdef object cached_script_texts
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_13decode_errors_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_13decode_errors_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_13decode_errors___get__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_13decode_errors___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->decode_errors);
  __pyx_r = __pyx_v_self->decode_errors;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_10HTMLParser_13decode_errors_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10selectolax_6parser_10HTMLParser_13decode_errors_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_13decode_errors_2__set__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_10HTMLParser_13decode_errors_2__set__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(7, 555, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->decode_errors);
  __Pyx_DECREF(__pyx_v_self->decode_errors);
  __pyx_v_self->decode_errors = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.decode_errors.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_10HTMLParser_13decode_errors_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10selectolax_6parser_10HTMLParser_13decode_errors_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_13decode_errors_4__del__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_10HTMLParser_13decode_errors_4__del__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->decode_errors);
  __Pyx_DECREF(__pyx_v_self->decode_errors);
  __pyx_v_self->decode_errors = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "selectolax/parser.pxd":556
 *     cdef myencoding_t _encoding
 *     cdef public unicode decode_errors
 *     cdef public bytes raw_html             # <<<<<<<<<<<<<<
 *     cdef object cached_script_texts
 *     cdef object cached_script_srcs
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_8raw_html_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_8raw_html_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_8raw_html___get__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_8raw_html___get__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->raw_html);
  __pyx_r = __pyx_v_self->raw_html;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_10HTMLParser_8raw_html_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10selectolax_6parser_10HTMLParser_8raw_html_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_8raw_html_2__set__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_10HTMLParser_8raw_html_2__set__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(7, 556, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->raw_html);
  __Pyx_DECREF(__pyx_v_self->raw_html);
  __pyx_v_self->raw_html = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.raw_html.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10selectolax_6parser_10HTMLParser_8raw_html_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10selectolax_6parser_10HTMLParser_8raw_html_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_8raw_html_4__del__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10selectolax_6parser_10HTMLParser_8raw_html_4__del__(struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->raw_html);
  __Pyx_DECREF(__pyx_v_self->raw_html);
  __pyx_v_self->raw_html = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_33__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_32__reduce_cython__, "HTMLParser.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_33__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_33__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_32__reduce_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_33__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_32__reduce_cython__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_32__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_self_html_tree_cannot_be_convert, 0, 0);
  __PYX_ERR(2, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_35__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_10HTMLParser_34__setstate_cython__, "HTMLParser.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_10HTMLParser_35__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_35__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_34__setstate_cython__};
static PyObject *__pyx_pw_10selectolax_6parser_10HTMLParser_35__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(2, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_10HTMLParser_34__setstate_cython__(((struct __pyx_obj_10selectolax_6parser_HTMLParser *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_10HTMLParser_34__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_10selectolax_6parser_HTMLParser *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_mstate_global->__pyx_kp_u_self_html_tree_cannot_be_convert, 0, 0);
  __PYX_ERR(2, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("selectolax.parser.HTMLParser.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Selector(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

/* Python wrapper */
static PyObject *__pyx_pw_10selectolax_6parser_21__pyx_unpickle_Selector(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10selectolax_6parser_20__pyx_unpickle_Selector, "__pyx_unpickle_Selector(__pyx_type, long __pyx_checksum, __pyx_state)");
static PyMethodDef __pyx_mdef_10selectolax_6parser_21__pyx_unpickle_Selector = {"__pyx_unpickle_Selector", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_21__pyx_unpickle_Selector, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_20__pyx_unpickle_Selector};
static PyObject *__pyx_pw_10selectolax_6parser_21__pyx_unpickle_Selector(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Selector (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(2, 1, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_Selector", 0) < 0) __PYX_ERR(2, 1, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Selector", 1, 3, 3, i); __PYX_ERR(2, 1, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 1, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 1, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 1, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Selector", 1, 3, 3, __pyx_nargs); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("selectolax.parser.__pyx_unpickle_Selector", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10selectolax_6parser_20__pyx_unpickle_Selector(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10selectolax_6parser_20__pyx_unpickle_Selector(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Selector", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xc5dfe7e, 0xad8d8ab, 0xc00154f):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xc5dfe7e, 0xad8d8ab, 0xc00154f) = (node, nodes))" % __pyx_checksum
*/
  __pyx_t_1 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xc5dfe7e, 0xad8d8ab, 0xc00154f):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xc5dfe7e, 0xad8d8ab, 0xc00154f) = (node, nodes))" % __pyx_checksum
 *     __pyx_result = Selector.__new__(__pyx_type)
*/
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_mstate_global->__pyx_n_u_PickleError) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
    __pyx_t_3 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xc5dfe7e, 0xad8d8ab, 0xc00154f):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xc5dfe7e, 0xad8d8ab, 0xc00154f) = (node, nodes))" % __pyx_checksum             # <<<<<<<<<<<<<<
 *     __pyx_result = Selector.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
    __pyx_t_3 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Incompatible_checksums_0x_x_vs_0, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_v___pyx_PickleError, __pyx_t_1, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xc5dfe7e, 0xad8d8ab, 0xc00154f):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xc5dfe7e, 0xad8d8ab, 0xc00154f) = (node, nodes))" % __pyx_checksum
*/
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xc5dfe7e, 0xad8d8ab, 0xc00154f) = (node, nodes))" % __pyx_checksum
 *     __pyx_result = Selector.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Selector__set_state(<Selector> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v___pyx_result = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xc5dfe7e, 0xad8d8ab, 0xc00154f) = (node, nodes))" % __pyx_checksum
 *     __pyx_result = Selector.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Selector__set_state(<Selector> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_2 = (__pyx_v___pyx_state != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = Selector.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Selector__set_state(<Selector> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Selector__set_state(Selector __pyx_result, tuple __pyx_state):
*/
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_10selectolax_6parser___pyx_unpickle_Selector__set_state(((struct __pyx_obj_10selectolax_6parser_Selector *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xc5dfe7e, 0xad8d8ab, 0xc00154f) = (node, nodes))" % __pyx_checksum
 *     __pyx_result = Selector.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Selector__set_state(<Selector> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Selector__set_state(<Selector> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Selector__set_state(Selector __pyx_result, tuple __pyx_state):
 *     __pyx_result.node = __pyx_state[0]; __pyx_result.nodes = __pyx_state[1]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Selector(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("selectolax.parser.__pyx_unpickle_Selector", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Selector__set_state(<Selector> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Selector__set_state(Selector __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.node = __pyx_state[0]; __pyx_result.nodes = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
*/

static PyObject *__pyx_f_10selectolax_6parser___pyx_unpickle_Selector__set_state(struct __pyx_obj_10selectolax_6parser_Selector *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Selector__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Selector__set_state(Selector __pyx_result, tuple __pyx_state):
 *     __pyx_result.node = __pyx_state[0]; __pyx_result.nodes = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyTuple_GET_ITEM(__pyx_v___pyx_state, 0);
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->node);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->node);
  __pyx_v___pyx_result->node = ((struct __pyx_obj_10selectolax_6parser_Node *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyTuple_GET_ITEM(__pyx_v___pyx_state, 1);
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->nodes);
  __Pyx_DECREF(__pyx_v___pyx_result->nodes);
  __pyx_v___pyx_result->nodes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Selector__set_state(Selector __pyx_result, tuple __pyx_state):
 *     __pyx_result.node = __pyx_state[0]; __pyx_result.nodes = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_3 > 2);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.node = __pyx_state[0]; __pyx_result.nodes = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])             # <<<<<<<<<<<<<<
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __pyx_t_6;
    __Pyx_INCREF(__pyx_t_5);
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __Pyx_PyTuple_GET_ITEM(__pyx_v___pyx_state, 2)};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_update, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_Selector__set_state(Selector __pyx_result, tuple __pyx_state):
 *     __pyx_result.node = __pyx_state[0]; __pyx_result.nodes = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
*/
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_Selector__set_state(<Selector> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Selector__set_state(Selector __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.node = __pyx_state[0]; __pyx_result.nodes = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("selectolax.parser.__pyx_unpickle_Selector__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */
static struct __pyx_vtabstruct_10selectolax_6parser_HTMLParser __pyx_vtable_10selectolax_6parser_HTMLParser;

static PyObject *__pyx_tp_new_10selectolax_6parser_HTMLParser(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_10selectolax_6parser_HTMLParser *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)o);
  p->__pyx_vtab = __pyx_vtabptr_10selectolax_6parser_HTMLParser;
  p->decode_errors = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->raw_html = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->cached_script_texts = Py_None; Py_INCREF(Py_None);
  p->cached_script_srcs = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser_HTMLParser(PyObject *o) {
  struct __pyx_obj_10selectolax_6parser_HTMLParser *p = (struct __pyx_obj_10selectolax_6parser_HTMLParser *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_10selectolax_6parser_HTMLParser) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_10selectolax_6parser_10HTMLParser_29__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->decode_errors);
  Py_CLEAR(p->raw_html);
  Py_CLEAR(p->cached_script_texts);
  Py_CLEAR(p->cached_script_srcs);
  #if CYTHON_USE_TYPE_SLOTS
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static int __pyx_tp_traverse_10selectolax_6parser_HTMLParser(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10selectolax_6parser_HTMLParser *p = (struct __pyx_obj_10selectolax_6parser_HTMLParser *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->cached_script_texts) {
    e = (*v)(p->cached_script_texts, a); if (e) return e;
  }
  if (p->cached_script_srcs) {
    e = (*v)(p->cached_script_srcs, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_10selectolax_6parser_HTMLParser(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_10selectolax_6parser_HTMLParser *p = (struct __pyx_obj_10selectolax_6parser_HTMLParser *)o;
  tmp = ((PyObject*)p->cached_script_texts);
  p->cached_script_texts = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->cached_script_srcs);
  p->cached_script_srcs = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_10selectolax_6parser_10HTMLParser_input_encoding(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_10HTMLParser_14input_encoding_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_10HTMLParser_root(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_10HTMLParser_4root_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_10HTMLParser_head(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_10HTMLParser_4head_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_10HTMLParser_body(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_10HTMLParser_4body_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_10HTMLParser_html(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_10HTMLParser_4html_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_10HTMLParser_detect_encoding(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_10HTMLParser_15detect_encoding_1__get__(o);
}

static int __pyx_setprop_10selectolax_6parser_10HTMLParser_detect_encoding(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10selectolax_6parser_10HTMLParser_15detect_encoding_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10selectolax_6parser_10HTMLParser_use_meta_tags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_10HTMLParser_13use_meta_tags_1__get__(o);
}

static int __pyx_setprop_10selectolax_6parser_10HTMLParser_use_meta_tags(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10selectolax_6parser_10HTMLParser_13use_meta_tags_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10selectolax_6parser_10HTMLParser_decode_errors(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_10HTMLParser_13decode_errors_1__get__(o);
}

static int __pyx_setprop_10selectolax_6parser_10HTMLParser_decode_errors(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10selectolax_6parser_10HTMLParser_13decode_errors_3__set__(o, v);
  }
  else {
    return __pyx_pw_10selectolax_6parser_10HTMLParser_13decode_errors_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10selectolax_6parser_10HTMLParser_raw_html(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_10HTMLParser_8raw_html_1__get__(o);
}

static int __pyx_setprop_10selectolax_6parser_10HTMLParser_raw_html(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10selectolax_6parser_10HTMLParser_8raw_html_3__set__(o, v);
  }
  else {
    return __pyx_pw_10selectolax_6parser_10HTMLParser_8raw_html_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_10selectolax_6parser_HTMLParser[] = {
  {"css", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_3css, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_2css},
  {"css_first", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_5css_first, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_4css_first},
  {"tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_7tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_6tags},
  {"text", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_9text, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_8text},
  {"strip_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_11strip_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_10strip_tags},
  {"unwrap_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_13unwrap_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_12unwrap_tags},
  {"select", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_15select, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_14select},
  {"any_css_matches", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_17any_css_matches, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_16any_css_matches},
  {"scripts_contain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_19scripts_contain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_18scripts_contain},
  {"script_srcs_contain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_21script_srcs_contain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_20script_srcs_contain},
  {"css_matches", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_23css_matches, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_22css_matches},
  {"merge_text_nodes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_25merge_text_nodes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_24merge_text_nodes},
  {"clone", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_27clone, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_26clone},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_33__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_32__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_10HTMLParser_35__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_10HTMLParser_34__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_10selectolax_6parser_HTMLParser[] = {
  {"input_encoding", __pyx_getprop_10selectolax_6parser_10HTMLParser_input_encoding, 0, PyDoc_STR("Return encoding of the HTML document.\n\n        Returns `unknown` in case the encoding is not determined.\n        "), 0},
  {"root", __pyx_getprop_10selectolax_6parser_10HTMLParser_root, 0, PyDoc_STR("Returns root node."), 0},
  {"head", __pyx_getprop_10selectolax_6parser_10HTMLParser_head, 0, PyDoc_STR("Returns head node."), 0},
  {"body", __pyx_getprop_10selectolax_6parser_10HTMLParser_body, 0, PyDoc_STR("Returns document body."), 0},
  {"html", __pyx_getprop_10selectolax_6parser_10HTMLParser_html, 0, PyDoc_STR("Return HTML representation of the page."), 0},
  {"detect_encoding", __pyx_getprop_10selectolax_6parser_10HTMLParser_detect_encoding, __pyx_setprop_10selectolax_6parser_10HTMLParser_detect_encoding, PyDoc_STR("detect_encoding: 'bool'"), 0},
  {"use_meta_tags", __pyx_getprop_10selectolax_6parser_10HTMLParser_use_meta_tags, __pyx_setprop_10selectolax_6parser_10HTMLParser_use_meta_tags, PyDoc_STR("use_meta_tags: 'bool'"), 0},
  {"decode_errors", __pyx_getprop_10selectolax_6parser_10HTMLParser_decode_errors, __pyx_setprop_10selectolax_6parser_10HTMLParser_decode_errors, PyDoc_STR("decode_errors: str"), 0},
  {"raw_html", __pyx_getprop_10selectolax_6parser_10HTMLParser_raw_html, __pyx_setprop_10selectolax_6parser_10HTMLParser_raw_html, PyDoc_STR("raw_html: bytes"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser_HTMLParser_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser_HTMLParser},
  {Py_tp_repr, (void *)__pyx_pw_10selectolax_6parser_10HTMLParser_31__repr__},
  {Py_tp_doc, (void *)PyDoc_STR("HTMLParser(html, detect_encoding=True, use_meta_tags=True, decode_errors='ignore')\n\nThe HTML parser.\n\nUse this class to parse raw HTML.\n\nParameters\n----------\n\nhtml : str (unicode) or bytes\ndetect_encoding : bool, default True\n    If `True` and html type is `bytes` then encoding will be detected automatically.\nuse_meta_tags : bool, default True\n    Whether to use meta tags in encoding detection process.\ndecode_errors : str, default 'ignore'\n    Same as in builtin's str.decode, i.e 'strict', 'ignore' or 'replace'.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_10selectolax_6parser_HTMLParser},
  {Py_tp_clear, (void *)__pyx_tp_clear_10selectolax_6parser_HTMLParser},
  {Py_tp_methods, (void *)__pyx_methods_10selectolax_6parser_HTMLParser},
  {Py_tp_getset, (void *)__pyx_getsets_10selectolax_6parser_HTMLParser},
  {Py_tp_init, (void *)__pyx_pw_10selectolax_6parser_10HTMLParser_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser_HTMLParser},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser_HTMLParser_spec = {
  "selectolax.parser.HTMLParser",
  sizeof(struct __pyx_obj_10selectolax_6parser_HTMLParser),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_10selectolax_6parser_HTMLParser_slots,
};
#else

static PyTypeObject __pyx_type_10selectolax_6parser_HTMLParser = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""HTMLParser", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser_HTMLParser), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser_HTMLParser, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_10selectolax_6parser_10HTMLParser_31__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("HTMLParser(html, detect_encoding=True, use_meta_tags=True, decode_errors='ignore')\n\nThe HTML parser.\n\nUse this class to parse raw HTML.\n\nParameters\n----------\n\nhtml : str (unicode) or bytes\ndetect_encoding : bool, default True\n    If `True` and html type is `bytes` then encoding will be detected automatically.\nuse_meta_tags : bool, default True\n    Whether to use meta tags in encoding detection process.\ndecode_errors : str, default 'ignore'\n    Same as in builtin's str.decode, i.e 'strict', 'ignore' or 'replace'."), /*tp_doc*/
  __pyx_tp_traverse_10selectolax_6parser_HTMLParser, /*tp_traverse*/
  __pyx_tp_clear_10selectolax_6parser_HTMLParser, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10selectolax_6parser_HTMLParser, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_10selectolax_6parser_HTMLParser, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_10selectolax_6parser_10HTMLParser_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser_HTMLParser, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_10selectolax_6parser_Stack __pyx_vtable_10selectolax_6parser_Stack;

static PyObject *__pyx_tp_new_10selectolax_6parser_Stack(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_10selectolax_6parser_Stack *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  o = (*t->tp_alloc)(t, 0);
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_10selectolax_6parser_Stack *)o);
  p->__pyx_vtab = __pyx_vtabptr_10selectolax_6parser_Stack;
  if (unlikely(__pyx_pw_10selectolax_6parser_5Stack_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_10selectolax_6parser_Stack(PyObject *o) {
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_10selectolax_6parser_5Stack_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  #if CYTHON_USE_TYPE_SLOTS
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static PyMethodDef __pyx_methods_10selectolax_6parser_Stack[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_5Stack_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_5Stack_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_5Stack_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_5Stack_6__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser_Stack_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser_Stack},
  {Py_tp_methods, (void *)__pyx_methods_10selectolax_6parser_Stack},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser_Stack},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser_Stack_spec = {
  "selectolax.parser.Stack",
  sizeof(struct __pyx_obj_10selectolax_6parser_Stack),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER,
  __pyx_type_10selectolax_6parser_Stack_slots,
};
#else

static PyTypeObject __pyx_type_10selectolax_6parser_Stack = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""Stack", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser_Stack), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser_Stack, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10selectolax_6parser_Stack, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser_Stack, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_10selectolax_6parser_CSSSelector __pyx_vtable_10selectolax_6parser_CSSSelector;

static PyObject *__pyx_tp_new_10selectolax_6parser_CSSSelector(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_10selectolax_6parser_CSSSelector *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  o = (*t->tp_alloc)(t, 0);
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_10selectolax_6parser_CSSSelector *)o);
  p->__pyx_vtab = __pyx_vtabptr_10selectolax_6parser_CSSSelector;
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser_CSSSelector(PyObject *o) {
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_10selectolax_6parser_11CSSSelector_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  #if CYTHON_USE_TYPE_SLOTS
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static PyMethodDef __pyx_methods_10selectolax_6parser_CSSSelector[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11CSSSelector_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11CSSSelector_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11CSSSelector_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11CSSSelector_6__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser_CSSSelector_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser_CSSSelector},
  {Py_tp_doc, (void *)PyDoc_STR("CSSSelector(str selector)")},
  {Py_tp_methods, (void *)__pyx_methods_10selectolax_6parser_CSSSelector},
  {Py_tp_init, (void *)__pyx_pw_10selectolax_6parser_11CSSSelector_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser_CSSSelector},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser_CSSSelector_spec = {
  "selectolax.parser.CSSSelector",
  sizeof(struct __pyx_obj_10selectolax_6parser_CSSSelector),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER,
  __pyx_type_10selectolax_6parser_CSSSelector_slots,
};
#else

static PyTypeObject __pyx_type_10selectolax_6parser_CSSSelector = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""CSSSelector", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser_CSSSelector), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser_CSSSelector, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  PyDoc_STR("CSSSelector(str selector)"), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10selectolax_6parser_CSSSelector, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_10selectolax_6parser_11CSSSelector_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser_CSSSelector, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_10selectolax_6parser_Selector __pyx_vtable_10selectolax_6parser_Selector;

static PyObject *__pyx_tp_new_10selectolax_6parser_Selector(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_10selectolax_6parser_Selector *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_10selectolax_6parser_Selector *)o);
  p->__pyx_vtab = __pyx_vtabptr_10selectolax_6parser_Selector;
  p->node = ((struct __pyx_obj_10selectolax_6parser_Node *)Py_None); Py_INCREF(Py_None);
  p->nodes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser_Selector(PyObject *o) {
  struct __pyx_obj_10selectolax_6parser_Selector *p = (struct __pyx_obj_10selectolax_6parser_Selector *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_10selectolax_6parser_Selector) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->node);
  Py_CLEAR(p->nodes);
  #if CYTHON_USE_TYPE_SLOTS
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static int __pyx_tp_traverse_10selectolax_6parser_Selector(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10selectolax_6parser_Selector *p = (struct __pyx_obj_10selectolax_6parser_Selector *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->node) {
    e = (*v)(((PyObject *)p->node), a); if (e) return e;
  }
  if (p->nodes) {
    e = (*v)(p->nodes, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_10selectolax_6parser_Selector(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_10selectolax_6parser_Selector *p = (struct __pyx_obj_10selectolax_6parser_Selector *)o;
  tmp = ((PyObject*)p->node);
  p->node = ((struct __pyx_obj_10selectolax_6parser_Node *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->nodes);
  p->nodes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_10selectolax_6parser_8Selector_matches(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_8Selector_7matches_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_8Selector_any_matches(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_8Selector_11any_matches_1__get__(o);
}

static PyMethodDef __pyx_methods_10selectolax_6parser_Selector[] = {
  {"text_contains", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_5text_contains, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_4text_contains},
  {"any_text_contains", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_7any_text_contains, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_6any_text_contains},
  {"attribute_longer_than", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_9attribute_longer_than, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_8attribute_longer_than},
  {"any_attribute_longer_than", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_11any_attribute_longer_than, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_10any_attribute_longer_than},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_15__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_14__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_8Selector_17__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_8Selector_16__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_10selectolax_6parser_Selector[] = {
  {"matches", __pyx_getprop_10selectolax_6parser_8Selector_matches, 0, PyDoc_STR("Returns all possible matches"), 0},
  {"any_matches", __pyx_getprop_10selectolax_6parser_8Selector_any_matches, 0, PyDoc_STR("Returns True if there are any matches"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser_Selector_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser_Selector},
  {Py_nb_bool, (void *)__pyx_pw_10selectolax_6parser_8Selector_13__bool__},
  {Py_tp_doc, (void *)PyDoc_STR("Selector(Node node, str query)\n\nAn advanced CSS selector that supports additional operations.\n\nThink of it as a toolkit that mimics some of the features of XPath.\n\nPlease note, this is an experimental feature that can change in the future.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_10selectolax_6parser_Selector},
  {Py_tp_clear, (void *)__pyx_tp_clear_10selectolax_6parser_Selector},
  {Py_tp_methods, (void *)__pyx_methods_10selectolax_6parser_Selector},
  {Py_tp_getset, (void *)__pyx_getsets_10selectolax_6parser_Selector},
  {Py_tp_init, (void *)__pyx_pw_10selectolax_6parser_8Selector_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser_Selector},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser_Selector_spec = {
  "selectolax.parser.Selector",
  sizeof(struct __pyx_obj_10selectolax_6parser_Selector),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_10selectolax_6parser_Selector_slots,
};
#else

static PyNumberMethods __pyx_tp_as_number_Selector = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  __pyx_pw_10selectolax_6parser_8Selector_13__bool__, /*nb_bool*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  0, /*nb_int*/
  0, /*nb_long (reserved)*/
  0, /*nb_float*/
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  0, /*nb_matrix_multiply*/
  0, /*nb_inplace_matrix_multiply*/
};

static PyTypeObject __pyx_type_10selectolax_6parser_Selector = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""Selector", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser_Selector), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser_Selector, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  &__pyx_tp_as_number_Selector, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("Selector(Node node, str query)\n\nAn advanced CSS selector that supports additional operations.\n\nThink of it as a toolkit that mimics some of the features of XPath.\n\nPlease note, this is an experimental feature that can change in the future."), /*tp_doc*/
  __pyx_tp_traverse_10selectolax_6parser_Selector, /*tp_traverse*/
  __pyx_tp_clear_10selectolax_6parser_Selector, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10selectolax_6parser_Selector, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_10selectolax_6parser_Selector, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_10selectolax_6parser_8Selector_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser_Selector, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_10selectolax_6parser__Attributes __pyx_vtable_10selectolax_6parser__Attributes;

static PyObject *__pyx_tp_new_10selectolax_6parser__Attributes(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_10selectolax_6parser__Attributes *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_10selectolax_6parser__Attributes *)o);
  p->__pyx_vtab = __pyx_vtabptr_10selectolax_6parser__Attributes;
  p->decode_errors = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser__Attributes(PyObject *o) {
  struct __pyx_obj_10selectolax_6parser__Attributes *p = (struct __pyx_obj_10selectolax_6parser__Attributes *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_10selectolax_6parser__Attributes) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  Py_CLEAR(p->decode_errors);
  #if CYTHON_USE_TYPE_SLOTS
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static PyObject *__pyx_sq_item_10selectolax_6parser__Attributes(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyLong_FromSsize_t(i); if(!x) return 0;
  #if CYTHON_USE_TYPE_SLOTS || (!CYTHON_USE_TYPE_SPECS && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  #else
  r = ((binaryfunc)PyType_GetSlot(Py_TYPE(o), Py_mp_subscript))(o, x);
  #endif
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_10selectolax_6parser__Attributes(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_10selectolax_6parser_11_Attributes_4__setitem__(o, i, v);
  }
  else {
    return __pyx_pw_10selectolax_6parser_11_Attributes_6__delitem__(o, i);
  }
}

static PyMethodDef __pyx_methods_10selectolax_6parser__Attributes[] = {
  {"keys", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_12keys, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_11keys},
  {"items", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_14items, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_13items},
  {"values", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_17values, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_16values},
  {"get", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_20get, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_19get},
  {"sget", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_22sget, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_21sget},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_28__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_27__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_11_Attributes_30__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_11_Attributes_29__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser__Attributes_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser__Attributes},
  {Py_tp_repr, (void *)__pyx_pw_10selectolax_6parser_11_Attributes_26__repr__},
  {Py_sq_length, (void *)__pyx_pw_10selectolax_6parser_11_Attributes_10__len__},
  {Py_sq_item, (void *)__pyx_sq_item_10selectolax_6parser__Attributes},
  {Py_sq_contains, (void *)__pyx_pw_10selectolax_6parser_11_Attributes_24__contains__},
  {Py_mp_length, (void *)__pyx_pw_10selectolax_6parser_11_Attributes_10__len__},
  {Py_mp_subscript, (void *)__pyx_pw_10selectolax_6parser_11_Attributes_8__getitem__},
  {Py_mp_ass_subscript, (void *)__pyx_mp_ass_subscript_10selectolax_6parser__Attributes},
  {Py_tp_doc, (void *)PyDoc_STR("A dict-like object that represents attributes.")},
  {Py_tp_iter, (void *)__pyx_pw_10selectolax_6parser_11_Attributes_1__iter__},
  {Py_tp_methods, (void *)__pyx_methods_10selectolax_6parser__Attributes},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser__Attributes},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser__Attributes_spec = {
  "selectolax.parser._Attributes",
  sizeof(struct __pyx_obj_10selectolax_6parser__Attributes),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_10selectolax_6parser__Attributes_slots,
};
#else

static PySequenceMethods __pyx_tp_as_sequence__Attributes = {
  __pyx_pw_10selectolax_6parser_11_Attributes_10__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_10selectolax_6parser__Attributes, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  __pyx_pw_10selectolax_6parser_11_Attributes_24__contains__, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__Attributes = {
  __pyx_pw_10selectolax_6parser_11_Attributes_10__len__, /*mp_length*/
  __pyx_pw_10selectolax_6parser_11_Attributes_8__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_10selectolax_6parser__Attributes, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_10selectolax_6parser__Attributes = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""_Attributes", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser__Attributes), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser__Attributes, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_10selectolax_6parser_11_Attributes_26__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence__Attributes, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__Attributes, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("A dict-like object that represents attributes."), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_10selectolax_6parser_11_Attributes_1__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10selectolax_6parser__Attributes, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser__Attributes, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_10selectolax_6parser_Node __pyx_vtable_10selectolax_6parser_Node;

static PyObject *__pyx_tp_new_10selectolax_6parser_Node(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_10selectolax_6parser_Node *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_10selectolax_6parser_Node *)o);
  p->__pyx_vtab = __pyx_vtabptr_10selectolax_6parser_Node;
  p->parser = ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser_Node(PyObject *o) {
  struct __pyx_obj_10selectolax_6parser_Node *p = (struct __pyx_obj_10selectolax_6parser_Node *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_10selectolax_6parser_Node) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->parser);
  #if CYTHON_USE_TYPE_SLOTS
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static int __pyx_tp_traverse_10selectolax_6parser_Node(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10selectolax_6parser_Node *p = (struct __pyx_obj_10selectolax_6parser_Node *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->parser) {
    e = (*v)(((PyObject *)p->parser), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_10selectolax_6parser_Node(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_10selectolax_6parser_Node *p = (struct __pyx_obj_10selectolax_6parser_Node *)o;
  tmp = ((PyObject*)p->parser);
  p->parser = ((struct __pyx_obj_10selectolax_6parser_HTMLParser *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_tp_richcompare_10selectolax_6parser_Node(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_10selectolax_6parser_4Node_47__eq__(o1, o2);
    }
    case Py_NE: {
      PyObject *ret;
      ret = __pyx_pw_10selectolax_6parser_4Node_47__eq__(o1, o2);
      if (likely(ret && ret != Py_NotImplemented)) {
        int b = __Pyx_PyObject_IsTrue(ret);
        Py_DECREF(ret);
        if (unlikely(b < 0)) return NULL;
        ret = (b) ? Py_False : Py_True;
        Py_INCREF(ret);
      }
      return ret;
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_attributes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_10attributes_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_attrs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_5attrs_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_mem_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_6mem_id_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_2id_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_tag(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_3tag_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_child(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_5child_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_parent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_6parent_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_next(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_4next_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_prev(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_4prev_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_last_child(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_10last_child_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_html(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_4html_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_raw_value(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_9raw_value_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_text_content(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_12text_content_1__get__(o);
}

static PyObject *__pyx_getprop_10selectolax_6parser_4Node_parser(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10selectolax_6parser_4Node_6parser_1__get__(o);
}

static int __pyx_setprop_10selectolax_6parser_4Node_parser(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10selectolax_6parser_4Node_6parser_3__set__(o, v);
  }
  else {
    return __pyx_pw_10selectolax_6parser_4Node_6parser_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_10selectolax_6parser_Node[] = {
  {"text", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_3text, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_2text},
  {"iter", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_5iter, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_4iter},
  {"traverse", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_8traverse, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_7traverse},
  {"css", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_11css, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_10css},
  {"any_css_matches", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_13any_css_matches, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_12any_css_matches},
  {"css_matches", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_15css_matches, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_14css_matches},
  {"css_first", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_17css_first, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_16css_first},
  {"decompose", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_19decompose, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_18decompose},
  {"remove", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_21remove, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_20remove},
  {"unwrap", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_23unwrap, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_22unwrap},
  {"strip_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_25strip_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_24strip_tags},
  {"unwrap_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_27unwrap_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_26unwrap_tags},
  {"unwrap_tags", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_37unwrap_tags, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_36unwrap_tags},
  {"select", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_39select, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_38select},
  {"scripts_contain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_41scripts_contain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_40scripts_contain},
  {"script_srcs_contain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_43script_srcs_contain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_42script_srcs_contain},
  {"merge_text_nodes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_49merge_text_nodes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_48merge_text_nodes},
  {"__pyx_fuse_0replace_with", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_51replace_with, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_28replace_with},
  {"__pyx_fuse_1replace_with", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_53replace_with, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_28replace_with},
  {"__pyx_fuse_2replace_with", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_55replace_with, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_28replace_with},
  {"__pyx_fuse_0insert_before", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_59insert_before, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_30insert_before},
  {"__pyx_fuse_1insert_before", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_61insert_before, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_30insert_before},
  {"__pyx_fuse_2insert_before", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_63insert_before, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_30insert_before},
  {"__pyx_fuse_0insert_after", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_67insert_after, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_32insert_after},
  {"__pyx_fuse_1insert_after", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_69insert_after, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_32insert_after},
  {"__pyx_fuse_2insert_after", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_71insert_after, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_32insert_after},
  {"__pyx_fuse_0insert_child", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_0__pyx_pw_10selectolax_6parser_4Node_75insert_child, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_34insert_child},
  {"__pyx_fuse_1insert_child", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_1__pyx_pw_10selectolax_6parser_4Node_77insert_child, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_34insert_child},
  {"__pyx_fuse_2insert_child", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_fuse_2__pyx_pw_10selectolax_6parser_4Node_79insert_child, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_34insert_child},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_83__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_82__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10selectolax_6parser_4Node_85__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10selectolax_6parser_4Node_84__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_10selectolax_6parser_Node[] = {
  {"attributes", __pyx_getprop_10selectolax_6parser_4Node_attributes, 0, PyDoc_STR("Get all attributes that belong to the current node.\n\n        The value of empty attributes is None.\n\n        Returns\n        -------\n        attributes : dictionary of all attributes.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser(\"<div data id='my_id'></div>\")\n        >>> node = tree.css_first('div')\n        >>> node.attributes\n        {'data': None, 'id': 'my_id'}\n        "), 0},
  {"attrs", __pyx_getprop_10selectolax_6parser_4Node_attrs, 0, PyDoc_STR("A dict-like object that is similar to the ``attributes`` property, but operates directly on the Node data.\n\n        .. warning:: Use ``attributes`` instead, if you don't want to modify Node attributes.\n\n        Returns\n        -------\n        attributes : Attributes mapping object.\n\n        Examples\n        --------\n\n        >>> tree = HTMLParser(\"<div id='a'></div>\")\n        >>> node = tree.css_first('div')\n        >>> node.attrs\n        <div attributes, 1 items>\n        >>> node.attrs['id']\n        'a'\n        >>> node.attrs['foo'] = 'bar'\n        >>> del node.attrs['id']\n        >>> node.attributes\n        {'foo': 'bar'}\n        >>> node.attrs['id'] = 'new_id'\n        >>> node.html\n        '<div foo=\"bar\" id=\"new_id\"></div>'\n        "), 0},
  {"mem_id", __pyx_getprop_10selectolax_6parser_4Node_mem_id, 0, PyDoc_STR("Get the mem_id attribute of the node.\n\n        Returns\n        -------\n        text : int\n        "), 0},
  {"id", __pyx_getprop_10selectolax_6parser_4Node_id, 0, PyDoc_STR("Get the id attribute of the node.\n\n        Returns None if id does not set.\n\n        Returns\n        -------\n        text : str\n        "), 0},
  {"tag", __pyx_getprop_10selectolax_6parser_4Node_tag, 0, PyDoc_STR("Return the name of the current tag (e.g. div, p, img).\n\n        Returns\n        -------\n        text : str\n        "), 0},
  {"child", __pyx_getprop_10selectolax_6parser_4Node_child, 0, PyDoc_STR("Return the child node."), 0},
  {"parent", __pyx_getprop_10selectolax_6parser_4Node_parent, 0, PyDoc_STR("Return the parent node."), 0},
  {"next", __pyx_getprop_10selectolax_6parser_4Node_next, 0, PyDoc_STR("Return next node."), 0},
  {"prev", __pyx_getprop_10selectolax_6parser_4Node_prev, 0, PyDoc_STR("Return previous node."), 0},
  {"last_child", __pyx_getprop_10selectolax_6parser_4Node_last_child, 0, PyDoc_STR("Return last child node."), 0},
  {"html", __pyx_getprop_10selectolax_6parser_4Node_html, 0, PyDoc_STR("Return HTML representation of the current node including all its child nodes.\n\n        Returns\n        -------\n        text : str\n        "), 0},
  {"raw_value", __pyx_getprop_10selectolax_6parser_4Node_raw_value, 0, PyDoc_STR("Return the raw (unparsed, original) value of a node.\n\n        Currently, works on text nodes only.\n\n        Returns\n        -------\n\n        raw_value : bytes\n\n        Examples\n        --------\n\n        >>> html_parser = HTMLParser('<div>&#x3C;test&#x3E;</div>')\n        >>> selector = html_parser.css_first('div')\n        >>> selector.child.html\n        '&lt;test&gt;'\n        >>> selector.child.raw_value\n        b'&#x3C;test&#x3E;'\n        "), 0},
  {"text_content", __pyx_getprop_10selectolax_6parser_4Node_text_content, 0, PyDoc_STR("Returns the text of the node if it is a text node.\n\n        Returns None for other nodes.\n        Unlike the ``text`` method, does not include child nodes.\n\n        Returns\n        -------\n        text : str or None.\n        "), 0},
  {"parser", __pyx_getprop_10selectolax_6parser_4Node_parser, __pyx_setprop_10selectolax_6parser_4Node_parser, PyDoc_STR("parser: selectolax.parser.HTMLParser"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser_Node_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser_Node},
  {Py_tp_repr, (void *)__pyx_pw_10selectolax_6parser_4Node_45__repr__},
  {Py_tp_hash, (void *)__pyx_pw_10selectolax_6parser_4Node_1__hash__},
  {Py_tp_doc, (void *)PyDoc_STR("A class that represents HTML node (element).")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_10selectolax_6parser_Node},
  {Py_tp_clear, (void *)__pyx_tp_clear_10selectolax_6parser_Node},
  {Py_tp_richcompare, (void *)__pyx_tp_richcompare_10selectolax_6parser_Node},
  {Py_tp_methods, (void *)__pyx_methods_10selectolax_6parser_Node},
  {Py_tp_getset, (void *)__pyx_getsets_10selectolax_6parser_Node},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser_Node},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser_Node_spec = {
  "selectolax.parser.Node",
  sizeof(struct __pyx_obj_10selectolax_6parser_Node),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_10selectolax_6parser_Node_slots,
};
#else

static PyTypeObject __pyx_type_10selectolax_6parser_Node = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""Node", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser_Node), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser_Node, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_10selectolax_6parser_4Node_45__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  __pyx_pw_10selectolax_6parser_4Node_1__hash__, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("A class that represents HTML node (element)."), /*tp_doc*/
  __pyx_tp_traverse_10selectolax_6parser_Node, /*tp_traverse*/
  __pyx_tp_clear_10selectolax_6parser_Node, /*tp_clear*/
  __pyx_tp_richcompare_10selectolax_6parser_Node, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10selectolax_6parser_Node, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_10selectolax_6parser_Node, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser_Node, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct____iter__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct____iter__ > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__)))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct____iter__[--__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct____iter__];
    memset(o, 0, sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct____iter__(PyObject *o) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct____iter__) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_key);
  Py_CLEAR(p->__pyx_v_self);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct____iter__ < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__)))) {
    __pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct____iter__[__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct____iter__++] = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct____iter__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__ *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_key) {
    e = (*v)(p->__pyx_v_key, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser___pyx_scope_struct____iter___slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct____iter__},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct____iter__},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct____iter__},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser___pyx_scope_struct____iter___spec = {
  "selectolax.parser.__pyx_scope_struct____iter__",
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_10selectolax_6parser___pyx_scope_struct____iter___slots,
};
#else

static PyTypeObject __pyx_type_10selectolax_6parser___pyx_scope_struct____iter__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""__pyx_scope_struct____iter__", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct____iter__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct____iter__, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct____iter__, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser___pyx_scope_struct____iter__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_1_items(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_1_items > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items)))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct_1_items[--__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_1_items];
    memset(o, 0, sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_1_items(PyObject *o) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_1_items) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_key);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_1_items < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items)))) {
    __pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct_1_items[__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_1_items++] = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_1_items(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_key) {
    e = (*v)(p->__pyx_v_key, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser___pyx_scope_struct_1_items_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_1_items},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_1_items},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_1_items},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser___pyx_scope_struct_1_items_spec = {
  "selectolax.parser.__pyx_scope_struct_1_items",
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_10selectolax_6parser___pyx_scope_struct_1_items_slots,
};
#else

static PyTypeObject __pyx_type_10selectolax_6parser___pyx_scope_struct_1_items = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""__pyx_scope_struct_1_items", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_1_items), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_1_items, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_1_items, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser___pyx_scope_struct_1_items, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_2_values(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_2_values > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values)))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct_2_values[--__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_2_values];
    memset(o, 0, sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_2_values(PyObject *o) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_2_values) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_key);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_2_values < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values)))) {
    __pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct_2_values[__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_2_values++] = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_2_values(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_key) {
    e = (*v)(p->__pyx_v_key, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser___pyx_scope_struct_2_values_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_2_values},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_2_values},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_2_values},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser___pyx_scope_struct_2_values_spec = {
  "selectolax.parser.__pyx_scope_struct_2_values",
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_10selectolax_6parser___pyx_scope_struct_2_values_slots,
};
#else

static PyTypeObject __pyx_type_10selectolax_6parser___pyx_scope_struct_2_values = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""__pyx_scope_struct_2_values", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_2_values), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_2_values, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_2_values, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser___pyx_scope_struct_2_values, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_3_iter(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_3_iter > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter)))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct_3_iter[--__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_3_iter];
    memset(o, 0, sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_3_iter(PyObject *o) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_3_iter) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_include_text);
  Py_CLEAR(p->__pyx_v_next_node);
  Py_CLEAR(p->__pyx_v_self);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_3_iter < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter)))) {
    __pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct_3_iter[__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_3_iter++] = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_3_iter(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_include_text) {
    e = (*v)(p->__pyx_v_include_text, a); if (e) return e;
  }
  if (p->__pyx_v_next_node) {
    e = (*v)(((PyObject *)p->__pyx_v_next_node), a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_3_iter},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_3_iter},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_3_iter},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter_spec = {
  "selectolax.parser.__pyx_scope_struct_3_iter",
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter_slots,
};
#else

static PyTypeObject __pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""__pyx_scope_struct_3_iter", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_3_iter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_3_iter, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_3_iter, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser___pyx_scope_struct_3_iter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_4_traverse(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_4_traverse > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse)))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct_4_traverse[--__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_4_traverse];
    memset(o, 0, sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_4_traverse(PyObject *o) {
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_4_traverse) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_include_text);
  Py_CLEAR(p->__pyx_v_next_node);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_stack);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_4_traverse < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse)))) {
    __pyx_mstate_global->__pyx_freelist_10selectolax_6parser___pyx_scope_struct_4_traverse[__pyx_mstate_global->__pyx_freecount_10selectolax_6parser___pyx_scope_struct_4_traverse++] = ((struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_4_traverse(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *p = (struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_include_text) {
    e = (*v)(p->__pyx_v_include_text, a); if (e) return e;
  }
  if (p->__pyx_v_next_node) {
    e = (*v)(((PyObject *)p->__pyx_v_next_node), a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_stack) {
    e = (*v)(((PyObject *)p->__pyx_v_stack), a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_4_traverse},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_4_traverse},
  {Py_tp_new, (void *)__pyx_tp_new_10selectolax_6parser___pyx_scope_struct_4_traverse},
  {0, 0},
};
static PyType_Spec __pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse_spec = {
  "selectolax.parser.__pyx_scope_struct_4_traverse",
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse_slots,
};
#else

static PyTypeObject __pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse = {
  PyVarObject_HEAD_INIT(0, 0)
  "selectolax.parser.""__pyx_scope_struct_4_traverse", /*tp_name*/
  sizeof(struct __pyx_obj_10selectolax_6parser___pyx_scope_struct_4_traverse), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10selectolax_6parser___pyx_scope_struct_4_traverse, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_10selectolax_6parser___pyx_scope_struct_4_traverse, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10selectolax_6parser___pyx_scope_struct_4_traverse, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_10selectolax_6parser_HTMLParser = &__pyx_vtable_10selectolax_6parser_HTMLParser;
  __pyx_vtable_10selectolax_6parser_HTMLParser._detect_encoding = (void (*)(struct __pyx_obj_10selectolax_6parser_HTMLParser *, char *, size_t))__pyx_f_10selectolax_6parser_10HTMLParser__detect_encoding;
  __pyx_vtable_10selectolax_6parser_HTMLParser._parse_html = (int (*)(struct __pyx_obj_10selectolax_6parser_HTMLParser *, char *, size_t))__pyx_f_10selectolax_6parser_10HTMLParser__parse_html;
  __pyx_vtable_10selectolax_6parser_HTMLParser.from_tree = (struct __pyx_obj_10selectolax_6parser_HTMLParser *(*)(myhtml_tree_t *, PyObject *, int, int, PyObject *, myencoding_t))__pyx_f_10selectolax_6parser_10HTMLParser_from_tree;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser_HTMLParser_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser)) __PYX_ERR(0, 10, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser_HTMLParser_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser = &__pyx_type_10selectolax_6parser_HTMLParser;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_vtabptr_10selectolax_6parser_HTMLParser) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_HTMLParser, (PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_HTMLParser) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  __pyx_vtabptr_10selectolax_6parser_Stack = &__pyx_vtable_10selectolax_6parser_Stack;
  __pyx_vtable_10selectolax_6parser_Stack.is_empty = (int (*)(struct __pyx_obj_10selectolax_6parser_Stack *))__pyx_f_10selectolax_6parser_5Stack_is_empty;
  __pyx_vtable_10selectolax_6parser_Stack.push = (int (*)(struct __pyx_obj_10selectolax_6parser_Stack *, myhtml_tree_node_t *))__pyx_f_10selectolax_6parser_5Stack_push;
  __pyx_vtable_10selectolax_6parser_Stack.pop = (myhtml_tree_node_t *(*)(struct __pyx_obj_10selectolax_6parser_Stack *))__pyx_f_10selectolax_6parser_5Stack_pop;
  __pyx_vtable_10selectolax_6parser_Stack.resize = (int (*)(struct __pyx_obj_10selectolax_6parser_Stack *))__pyx_f_10selectolax_6parser_5Stack_resize;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser_Stack_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack)) __PYX_ERR(3, 13, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser_Stack_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack) < 0) __PYX_ERR(3, 13, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack = &__pyx_type_10selectolax_6parser_Stack;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack) < 0) __PYX_ERR(3, 13, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack, __pyx_vtabptr_10selectolax_6parser_Stack) < 0) __PYX_ERR(3, 13, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack) < 0) __PYX_ERR(3, 13, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Stack, (PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack) < 0) __PYX_ERR(3, 13, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_Stack) < 0) __PYX_ERR(3, 13, __pyx_L1_error)
  __pyx_vtabptr_10selectolax_6parser_CSSSelector = &__pyx_vtable_10selectolax_6parser_CSSSelector;
  __pyx_vtable_10selectolax_6parser_CSSSelector.find = (myhtml_collection_t *(*)(struct __pyx_obj_10selectolax_6parser_CSSSelector *, myhtml_tree_node_t *))__pyx_f_10selectolax_6parser_11CSSSelector_find;
  __pyx_vtable_10selectolax_6parser_CSSSelector._create_css_parser = (int (*)(struct __pyx_obj_10selectolax_6parser_CSSSelector *))__pyx_f_10selectolax_6parser_11CSSSelector__create_css_parser;
  __pyx_vtable_10selectolax_6parser_CSSSelector._prepare_selector = (int (*)(struct __pyx_obj_10selectolax_6parser_CSSSelector *, mycss_entry_t *, char const *, size_t))__pyx_f_10selectolax_6parser_11CSSSelector__prepare_selector;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser_CSSSelector_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector)) __PYX_ERR(1, 5, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser_CSSSelector_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector) < 0) __PYX_ERR(1, 5, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector = &__pyx_type_10selectolax_6parser_CSSSelector;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector) < 0) __PYX_ERR(1, 5, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector, __pyx_vtabptr_10selectolax_6parser_CSSSelector) < 0) __PYX_ERR(1, 5, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector) < 0) __PYX_ERR(1, 5, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_CSSSelector, (PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector) < 0) __PYX_ERR(1, 5, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_CSSSelector) < 0) __PYX_ERR(1, 5, __pyx_L1_error)
  __pyx_vtabptr_10selectolax_6parser_Selector = &__pyx_vtable_10selectolax_6parser_Selector;
  __pyx_vtable_10selectolax_6parser_Selector.css = (PyObject *(*)(struct __pyx_obj_10selectolax_6parser_Selector *, PyObject *, int __pyx_skip_dispatch))__pyx_f_10selectolax_6parser_8Selector_css;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser_Selector_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector)) __PYX_ERR(1, 70, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser_Selector_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector) < 0) __PYX_ERR(1, 70, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector = &__pyx_type_10selectolax_6parser_Selector;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector) < 0) __PYX_ERR(1, 70, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_UPDATE_DESCRIPTOR_DOC
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector, "__init__"); if (unlikely(!wrapper)) __PYX_ERR(1, 70, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_10selectolax_6parser_8Selector___init__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_10selectolax_6parser_8Selector___init__.doc = __pyx_doc_10selectolax_6parser_8Selector___init__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_10selectolax_6parser_8Selector___init__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector, __pyx_vtabptr_10selectolax_6parser_Selector) < 0) __PYX_ERR(1, 70, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector) < 0) __PYX_ERR(1, 70, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Selector, (PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector) < 0) __PYX_ERR(1, 70, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_Selector) < 0) __PYX_ERR(1, 70, __pyx_L1_error)
  __pyx_vtabptr_10selectolax_6parser__Attributes = &__pyx_vtable_10selectolax_6parser__Attributes;
  __pyx_vtable_10selectolax_6parser__Attributes.create = (struct __pyx_obj_10selectolax_6parser__Attributes *(*)(myhtml_tree_node_t *, PyObject *))__pyx_f_10selectolax_6parser_11_Attributes_create;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser__Attributes_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes)) __PYX_ERR(3, 47, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser__Attributes_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes) < 0) __PYX_ERR(3, 47, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes = &__pyx_type_10selectolax_6parser__Attributes;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes) < 0) __PYX_ERR(3, 47, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes, __pyx_vtabptr_10selectolax_6parser__Attributes) < 0) __PYX_ERR(3, 47, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes) < 0) __PYX_ERR(3, 47, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Attributes, (PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes) < 0) __PYX_ERR(3, 47, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser__Attributes) < 0) __PYX_ERR(3, 47, __pyx_L1_error)
  __pyx_vtabptr_10selectolax_6parser_Node = &__pyx_vtable_10selectolax_6parser_Node;
  __pyx_vtable_10selectolax_6parser_Node.__pyx_new = (struct __pyx_obj_10selectolax_6parser_Node *(*)(myhtml_tree_node_t *, struct __pyx_obj_10selectolax_6parser_HTMLParser *))__pyx_f_10selectolax_6parser_4Node_new;
  __pyx_vtable_10selectolax_6parser_Node._text_deep = (PyObject *(*)(struct __pyx_obj_10selectolax_6parser_Node *, myhtml_tree_node_t *, struct __pyx_opt_args_10selectolax_6parser_4Node__text_deep *__pyx_optional_args))__pyx_f_10selectolax_6parser_4Node__text_deep;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_Node = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser_Node_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_Node)) __PYX_ERR(3, 144, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser_Node_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser_Node) < 0) __PYX_ERR(3, 144, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser_Node = &__pyx_type_10selectolax_6parser_Node;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser_Node) < 0) __PYX_ERR(3, 144, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser_Node->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser_Node->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser_Node->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_10selectolax_6parser_Node, __pyx_vtabptr_10selectolax_6parser_Node) < 0) __PYX_ERR(3, 144, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_10selectolax_6parser_Node) < 0) __PYX_ERR(3, 144, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Node, (PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_Node) < 0) __PYX_ERR(3, 144, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_10selectolax_6parser_Node) < 0) __PYX_ERR(3, 144, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__ = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser___pyx_scope_struct____iter___spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__)) __PYX_ERR(3, 59, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser___pyx_scope_struct____iter___spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__) < 0) __PYX_ERR(3, 59, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__ = &__pyx_type_10selectolax_6parser___pyx_scope_struct____iter__;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__) < 0) __PYX_ERR(3, 59, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct____iter__->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser___pyx_scope_struct_1_items_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items)) __PYX_ERR(3, 101, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser___pyx_scope_struct_1_items_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items) < 0) __PYX_ERR(3, 101, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items = &__pyx_type_10selectolax_6parser___pyx_scope_struct_1_items;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items) < 0) __PYX_ERR(3, 101, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_1_items->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser___pyx_scope_struct_2_values_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values)) __PYX_ERR(3, 105, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser___pyx_scope_struct_2_values_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values) < 0) __PYX_ERR(3, 105, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values = &__pyx_type_10selectolax_6parser___pyx_scope_struct_2_values;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values) < 0) __PYX_ERR(3, 105, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_2_values->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter)) __PYX_ERR(3, 328, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter) < 0) __PYX_ERR(3, 328, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter = &__pyx_type_10selectolax_6parser___pyx_scope_struct_3_iter;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter) < 0) __PYX_ERR(3, 328, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_3_iter->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse)) __PYX_ERR(3, 353, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse_spec, __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse) < 0) __PYX_ERR(3, 353, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse = &__pyx_type_10selectolax_6parser___pyx_scope_struct_4_traverse;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse) < 0) __PYX_ERR(3, 353, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse->tp_dictoffset && __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_10selectolax_6parser___pyx_scope_struct_4_traverse->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType_3_1_2(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyLongObject), __PYX_GET_STRUCT_ALIGNMENT_3_1_2(PyLongObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyLongObject), __PYX_GET_STRUCT_ALIGNMENT_3_1_2(PyLongObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_1_2); if (!__pyx_mstate->__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(8, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_4type_type = __Pyx_ImportType_3_1_2(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_1_2(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_1_2(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_1_2); if (!__pyx_mstate->__pyx_ptype_7cpython_4type_type) __PYX_ERR(9, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_parser(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_parser},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, Py_MOD_GIL_USED},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "parser",
      0, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit_parser(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_parser(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x03090000
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_parser(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'parser' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "parser" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 2, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 2, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif

__Pyx_RefNannySetupContext("PyInit_parser", 0);
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 2, __pyx_L1_error)
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #if 0 || defined(__Pyx_CyFunction_USED) || defined(__Pyx_FusedFunction_USED) || defined(__Pyx_Coroutine_USED) || defined(__Pyx_Generator_USED) || defined(__Pyx_AsyncGen_USED)
  if (__pyx_CommonTypesMetaclass_init(__pyx_m) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  if (__pyx_module_is_main_selectolax__parser) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name, __pyx_mstate_global->__pyx_n_u_main) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 2, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "selectolax.parser")) {
      if (unlikely((PyDict_SetItemString(modules, "selectolax.parser", __pyx_m) < 0))) __PYX_ERR(0, 2, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  (void)__Pyx_modinit_variable_export_code(__pyx_mstate);
  (void)__Pyx_modinit_function_export_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_type_init_code(__pyx_mstate) < 0))) __PYX_ERR(0, 2, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 2, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code(__pyx_mstate);
  (void)__Pyx_modinit_function_import_code(__pyx_mstate);
  /*--- Execution code ---*/

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11CSSSelector_5__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_CSSSelector___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[5])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.css_entry,self.finder,self.selectors_list cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11CSSSelector_7__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_CSSSelector___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[6])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/selection.pxi":85
 *         self.nodes = find_nodes(node.parser, node.node, query) if query else [node, ]
 * 
 *     cpdef css(self, str query):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current scope."""
 *         cdef Node current_node
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_8Selector_3css, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Selector_css, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[7])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector, __pyx_mstate_global->__pyx_n_u_css, __pyx_t_2) < 0) __PYX_ERR(1, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/selection.pxi":105
 *         return bool(self.nodes)
 * 
 *     def text_contains(self, str text, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Filter all current matches given text."""
 *         nodes = []
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_8Selector_5text_contains, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Selector_text_contains, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[8])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[1]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector, __pyx_mstate_global->__pyx_n_u_text_contains, __pyx_t_2) < 0) __PYX_ERR(1, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/selection.pxi":116
 *         return self
 * 
 *     def any_text_contains(self, str text, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Returns True if any node in the current search scope contains specified text"""
 *         nodes = []
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_8Selector_7any_text_contains, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Selector_any_text_contains, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[9])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[1]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector, __pyx_mstate_global->__pyx_n_u_any_text_contains, __pyx_t_2) < 0) __PYX_ERR(1, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/selection.pxi":126
 *         return False
 * 
 *     def attribute_longer_than(self, str attribute, int length, str start  = None):             # <<<<<<<<<<<<<<
 *         """Filter all current matches by attribute length.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_8Selector_9attribute_longer_than, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Selector_attribute_longer_than, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[10])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[2]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector, __pyx_mstate_global->__pyx_n_u_attribute_longer_than, __pyx_t_2) < 0) __PYX_ERR(1, 126, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/selection.pxi":141
 *         return self
 * 
 *     def any_attribute_longer_than(self, str attribute, int length, str start  = None):             # <<<<<<<<<<<<<<
 *         """Returns True any href attribute longer than a specified length.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_8Selector_11any_attribute_longer_than, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Selector_any_attribute_longer_th, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[11])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[2]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector, __pyx_mstate_global->__pyx_n_u_any_attribute_longer_than, __pyx_t_2) < 0) __PYX_ERR(1, 141, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_8Selector_15__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Selector___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[12])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Selector, (type(self), 0xc5dfe7e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Selector__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_8Selector_17__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Selector___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[13])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Selector, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_5Stack_5__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stack___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[14])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_5Stack_7__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stack___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[15])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":98
 *         return len(list(self.__iter__()))
 * 
 *     def keys(self):             # <<<<<<<<<<<<<<
 *         return self.__iter__()
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11_Attributes_12keys, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Attributes_keys, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[16])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser__Attributes, __pyx_mstate_global->__pyx_n_u_keys, __pyx_t_2) < 0) __PYX_ERR(3, 98, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":101
 *         return self.__iter__()
 * 
 *     def items(self):             # <<<<<<<<<<<<<<
 *         for key in self.__iter__():
 *             yield key, self[key]
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11_Attributes_14items, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Attributes_items, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser__Attributes, __pyx_mstate_global->__pyx_n_u_items, __pyx_t_2) < 0) __PYX_ERR(3, 101, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":105
 *             yield key, self[key]
 * 
 *     def values(self):             # <<<<<<<<<<<<<<
 *         for key in self.__iter__():
 *             yield self[key]
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11_Attributes_17values, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Attributes_values, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser__Attributes, __pyx_mstate_global->__pyx_n_u_values, __pyx_t_2) < 0) __PYX_ERR(3, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":109
 *             yield self[key]
 * 
 *     def get(self, key, default=None):             # <<<<<<<<<<<<<<
 *         try:
 *             return self[key]
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11_Attributes_20get, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Attributes_get, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[17])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[2]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser__Attributes, __pyx_mstate_global->__pyx_n_u_get, __pyx_t_2) < 0) __PYX_ERR(3, 109, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":115
 *             return default
 * 
 *     def sget(self, key, default=""):             # <<<<<<<<<<<<<<
 *         """Same as get, but returns empty strings instead of None values for empty attributes."""
 *         try:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11_Attributes_22sget, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Attributes_sget, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[18])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[3]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser__Attributes, __pyx_mstate_global->__pyx_n_u_sget, __pyx_t_2) < 0) __PYX_ERR(3, 115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11_Attributes_28__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Attributes___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[19])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11_Attributes_30__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Attributes___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[20])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":252
 *         return self.mem_id
 * 
 *     def text(self, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Returns the text of the node including text of all its child nodes.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_3text, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_text, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[21])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[1]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_text, __pyx_t_2) < 0) __PYX_ERR(3, 252, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":328
 *         return text
 * 
 *     def iter(self, include_text=False):             # <<<<<<<<<<<<<<
 *         """Iterate over nodes on the current level.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_5iter, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_iter, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[4]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_iter_2, __pyx_t_2) < 0) __PYX_ERR(3, 328, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":353
 *             node = node.next
 * 
 *     def traverse(self, include_text=False):             # <<<<<<<<<<<<<<
 *         """Iterate over all child and next nodes starting from the current level.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_8traverse, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_traverse, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[4])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[4]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_traverse, __pyx_t_2) < 0) __PYX_ERR(3, 353, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":466
 *         return None
 * 
 *     def css(self, str query):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         return find_nodes(self.parser, self.node, query)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_11css, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_css, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[22])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_css, __pyx_t_2) < 0) __PYX_ERR(3, 466, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":470
 *         return find_nodes(self.parser, self.node, query)
 * 
 *     def any_css_matches(self, tuple selectors):             # <<<<<<<<<<<<<<
 *         """Returns True if any of CSS selectors matches a node"""
 *         return find_matches(self.parser, self.node, selectors)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_13any_css_matches, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_any_css_matches, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[23])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_any_css_matches, __pyx_t_2) < 0) __PYX_ERR(3, 470, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":474
 *         return find_matches(self.parser, self.node, selectors)
 * 
 *     def css_matches(self, str selector):             # <<<<<<<<<<<<<<
 *         """Returns True if CSS selector matches a node."""
 *         return find_matches(self.parser, self.node, (selector, ))
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_15css_matches, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_css_matches, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[24])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_css_matches, __pyx_t_2) < 0) __PYX_ERR(3, 474, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":478
 *         return find_matches(self.parser, self.node, (selector, ))
 * 
 *     def css_first(self, str query, default=None, bool strict=False):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         results = self.css(query)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_17css_first, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_css_first, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[25])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[5]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_css_first, __pyx_t_2) < 0) __PYX_ERR(3, 478, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":492
 *         return default
 * 
 *     def decompose(self, bool recursive=True):             # <<<<<<<<<<<<<<
 *         """Remove a Node from the tree.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_19decompose, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_decompose, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[26])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[6]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_decompose, __pyx_t_2) < 0) __PYX_ERR(3, 492, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":513
 *             myhtml_node_delete(self.node)
 * 
 *     def remove(self, bool recursive=True):             # <<<<<<<<<<<<<<
 *         """An alias for the decompose method."""
 *         self.decompose(recursive)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_21remove, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_remove, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[27])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[6]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_remove, __pyx_t_2) < 0) __PYX_ERR(3, 513, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":517
 *         self.decompose(recursive)
 * 
 *     def unwrap(self, delete_empty = False):             # <<<<<<<<<<<<<<
 *         """Replace node with whatever is inside this node.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_23unwrap, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_unwrap, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[28])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[4]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_unwrap, __pyx_t_2) < 0) __PYX_ERR(3, 517, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":554
 *         myhtml_node_delete(self.node)
 * 
 *     def strip_tags(self, list tags, bool recursive = False):             # <<<<<<<<<<<<<<
 *         """Remove specified tags from the HTML tree.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_25strip_tags, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_strip_tags, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[29])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[4]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_strip_tags, __pyx_t_2) < 0) __PYX_ERR(3, 554, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":580
 *                 element.decompose(recursive=recursive)
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):             # <<<<<<<<<<<<<<
 *         """Unwraps specified tags from the HTML tree.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_27unwrap_tags, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_unwrap_tags, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[30])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 580, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[4]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_unwrap_tags, __pyx_t_2) < 0) __PYX_ERR(3, 580, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":607
 *                 element.unwrap(delete_empty)
 * 
 *     def replace_with(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """Replace current Node with specified value.
 * 
*/
  __pyx_t_2 = __Pyx_PyList_Pack(1, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_mstate_global->__pyx_k__3 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_fuse_0__pyx_mdef_10selectolax_6parser_4Node_51replace_with, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_replace_with_str, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[31])); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_str, __pyx_t_3) < 0) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_fuse_1__pyx_mdef_10selectolax_6parser_4Node_53replace_with, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_replace_with_bytes, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[32])); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_bytes, __pyx_t_3) < 0) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_fuse_2__pyx_mdef_10selectolax_6parser_4Node_55replace_with, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_replace_with_Node, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[33])); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Node, __pyx_t_3) < 0) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_29replace_with, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_replace_with, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[34])); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_mstate_global->__pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_3)->__signatures__ = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_replace_with, __pyx_t_3) < 0) __PYX_ERR(3, 607, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "selectolax/modest/node.pxi":648
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_before(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node before the current Node.
*/
  __pyx_t_3 = __Pyx_PyList_Pack(1, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_mstate_global->__pyx_k__6 = __pyx_t_3;
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_FusedFunction_New(&__pyx_fuse_0__pyx_mdef_10selectolax_6parser_4Node_59insert_before, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_before_str, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[35])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_str, __pyx_t_2) < 0) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_FusedFunction_New(&__pyx_fuse_1__pyx_mdef_10selectolax_6parser_4Node_61insert_before, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_before_bytes, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[36])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_bytes, __pyx_t_2) < 0) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_FusedFunction_New(&__pyx_fuse_2__pyx_mdef_10selectolax_6parser_4Node_63insert_before, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_before_Node, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[37])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_Node, __pyx_t_2) < 0) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_FusedFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_31insert_before, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_before, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[38])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_2)->__signatures__ = __pyx_t_3;
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_insert_before, __pyx_t_2) < 0) __PYX_ERR(3, 648, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":688
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_after(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node after the current Node.
*/
  __pyx_t_2 = __Pyx_PyList_Pack(1, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_mstate_global->__pyx_k__7 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_fuse_0__pyx_mdef_10selectolax_6parser_4Node_67insert_after, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_after_str, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[39])); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_str, __pyx_t_3) < 0) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_fuse_1__pyx_mdef_10selectolax_6parser_4Node_69insert_after, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_after_bytes, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[40])); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_bytes, __pyx_t_3) < 0) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_fuse_2__pyx_mdef_10selectolax_6parser_4Node_71insert_after, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_after_Node, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[41])); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Node, __pyx_t_3) < 0) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_33insert_after, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_after, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[42])); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_mstate_global->__pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_3)->__signatures__ = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_insert_after, __pyx_t_3) < 0) __PYX_ERR(3, 688, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "selectolax/modest/node.pxi":728
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def insert_child(self, str_or_Node value):             # <<<<<<<<<<<<<<
 *         """
 *         Insert a node inside (at the end of) the current Node.
*/
  __pyx_t_3 = __Pyx_PyList_Pack(1, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_mstate_global->__pyx_k__8 = __pyx_t_3;
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_FusedFunction_New(&__pyx_fuse_0__pyx_mdef_10selectolax_6parser_4Node_75insert_child, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_child_str, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[43])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_str, __pyx_t_2) < 0) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_FusedFunction_New(&__pyx_fuse_1__pyx_mdef_10selectolax_6parser_4Node_77insert_child, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_child_bytes, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[44])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_bytes, __pyx_t_2) < 0) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_FusedFunction_New(&__pyx_fuse_2__pyx_mdef_10selectolax_6parser_4Node_79insert_child, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_child_Node, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[45])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_Node, __pyx_t_2) < 0) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_FusedFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_35insert_child, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_insert_child, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[46])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_2)->__signatures__ = __pyx_t_3;
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_insert_child, __pyx_t_2) < 0) __PYX_ERR(3, 728, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":768
 *             raise TypeError("Expected a string or Node instance, but %s found" % type(value).__name__)
 * 
 *     def unwrap_tags(self, list tags, delete_empty = False):             # <<<<<<<<<<<<<<
 *         """Unwraps specified tags from the HTML tree.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_37unwrap_tags, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_unwrap_tags, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[47])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 768, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[4]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_unwrap_tags, __pyx_t_2) < 0) __PYX_ERR(3, 768, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":822
 *         return self.parser.raw_html[begin:begin + length]
 * 
 *     def select(self, query=None):             # <<<<<<<<<<<<<<
 *         """Select nodes given a CSS selector.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_39select, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_select, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[48])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[2]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_select, __pyx_t_2) < 0) __PYX_ERR(3, 822, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":838
 *         return Selector(self, query)
 * 
 *     def scripts_contain(self, str query):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script tags contain specified text.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_41scripts_contain, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_scripts_contain, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[49])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 838, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_scripts_contain, __pyx_t_2) < 0) __PYX_ERR(3, 838, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":864
 *         return False
 * 
 *     def script_srcs_contain(self, tuple queries):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script SRCs attributes contain on of the specified text.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_43script_srcs_contain, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_script_srcs_contain, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[50])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_script_srcs_contain, __pyx_t_2) < 0) __PYX_ERR(3, 864, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/node.pxi":921
 *         return None
 * 
 *     def merge_text_nodes(self):             # <<<<<<<<<<<<<<
 *         """Iterates over all text nodes and merges all text nodes that are close to each other.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_49merge_text_nodes, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node_merge_text_nodes, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[51])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 921, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_Node, __pyx_mstate_global->__pyx_n_u_merge_text_nodes, __pyx_t_2) < 0) __PYX_ERR(3, 921, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_83__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[52])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.node cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_4Node_85__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[53])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/utils.pxi":1
 * from typing import Literal, Optional, Union, Type             # <<<<<<<<<<<<<<
 * 
 * MAX_HTML_INPUT_SIZE = 250e+7
*/
  __pyx_t_2 = __Pyx_PyList_Pack(4, __pyx_mstate_global->__pyx_n_u_Literal, __pyx_mstate_global->__pyx_n_u_Optional, __pyx_mstate_global->__pyx_n_u_Union, __pyx_mstate_global->__pyx_n_u_Type); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_typing, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_Literal); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Literal, __pyx_t_2) < 0) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_Optional); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Optional, __pyx_t_2) < 0) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_Union); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Union, __pyx_t_2) < 0) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_Type); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Type, __pyx_t_2) < 0) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "selectolax/utils.pxi":3
 * from typing import Literal, Optional, Union, Type
 * 
 * MAX_HTML_INPUT_SIZE = 250e+7             # <<<<<<<<<<<<<<
 * 
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_MAX_HTML_INPUT_SIZE, __pyx_mstate_global->__pyx_float_250e_7) < 0) __PYX_ERR(4, 3, __pyx_L1_error)

  /* "selectolax/utils.pxi":5
 * MAX_HTML_INPUT_SIZE = 250e+7
 * 
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]             # <<<<<<<<<<<<<<
 * Parser = Union["HTMLParser", "LexborHTMLParser"]
 * FRAGMENT = Literal[
*/
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_Union); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Type); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_HTMLParser); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Type); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_LexborHTMLParser); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_Pack(2, __pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ParserCls, __pyx_t_5) < 0) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "selectolax/utils.pxi":6
 * 
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]
 * Parser = Union["HTMLParser", "LexborHTMLParser"]             # <<<<<<<<<<<<<<
 * FRAGMENT = Literal[
 *     "document",
*/
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_Union); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_mstate_global->__pyx_tuple[7]); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Parser, __pyx_t_2) < 0) __PYX_ERR(4, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/utils.pxi":7
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]
 * Parser = Union["HTMLParser", "LexborHTMLParser"]
 * FRAGMENT = Literal[             # <<<<<<<<<<<<<<
 *     "document",
 *     "fragment",
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Literal); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[8]); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_FRAGMENT, __pyx_t_5) < 0) __PYX_ERR(4, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "selectolax/utils.pxi":19
 * 
 * 
 * def preprocess_input(html, decode_errors='ignore'):             # <<<<<<<<<<<<<<
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
*/
  __pyx_t_5 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_1preprocess_input, 0, __pyx_mstate_global->__pyx_n_u_preprocess_input, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[54])); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_5, __pyx_mstate_global->__pyx_tuple[9]);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_preprocess_input, __pyx_t_5) < 0) __PYX_ERR(4, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "selectolax/utils.pxi":32
 * 
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")
*/
  __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_tag, __pyx_mstate_global->__pyx_n_u_str) < 0) __PYX_ERR(4, 32, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_parser_cls, __pyx_mstate_global->__pyx_n_u_ParserCls) < 0) __PYX_ERR(4, 32, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_3do_create_tag, 0, __pyx_mstate_global->__pyx_n_u_do_create_tag, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[55])); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_do_create_tag, __pyx_t_2) < 0) __PYX_ERR(4, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/utils.pxi":38
 * 
 * 
 * def get_fragment_type(             # <<<<<<<<<<<<<<
 *     html: str,
 *     parser_cls: ParserCls,
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_html, __pyx_mstate_global->__pyx_n_u_str) < 0) __PYX_ERR(4, 38, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_parser_cls, __pyx_mstate_global->__pyx_n_u_ParserCls) < 0) __PYX_ERR(4, 38, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_tree, __pyx_mstate_global->__pyx_kp_u_Optional_Parser) < 0) __PYX_ERR(4, 38, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_FRAGMENT) < 0) __PYX_ERR(4, 38, __pyx_L1_error)
  __pyx_t_5 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_5get_fragment_type, 0, __pyx_mstate_global->__pyx_n_u_get_fragment_type, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[56])); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_5, __pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_5, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_get_fragment_type, __pyx_t_5) < 0) __PYX_ERR(4, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "selectolax/utils.pxi":81
 * 
 * 
 * def do_parse_fragment(html: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     """
 *     Given HTML, parse it into a list of Nodes, such that the nodes
*/
  __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_html, __pyx_mstate_global->__pyx_n_u_str) < 0) __PYX_ERR(4, 81, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_parser_cls, __pyx_mstate_global->__pyx_n_u_ParserCls) < 0) __PYX_ERR(4, 81, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_7do_parse_fragment, 0, __pyx_mstate_global->__pyx_n_u_do_parse_fragment, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[57])); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_do_parse_fragment, __pyx_t_2) < 0) __PYX_ERR(4, 81, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/modest/util.pxi":4
 * 
 * 
 * def create_tag(tag: str):             # <<<<<<<<<<<<<<
 *     """
 *     Given an HTML tag name, e.g. `"div"`, create a single empty node for that tag,
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_tag, __pyx_mstate_global->__pyx_n_u_str) < 0) __PYX_ERR(5, 4, __pyx_L1_error)
  __pyx_t_5 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_9create_tag, 0, __pyx_mstate_global->__pyx_n_u_create_tag, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[58])); if (unlikely(!__pyx_t_5)) __PYX_ERR(5, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_5, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_create_tag, __pyx_t_5) < 0) __PYX_ERR(5, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "selectolax/modest/util.pxi":12
 * 
 * 
 * def parse_fragment(html: str):             # <<<<<<<<<<<<<<
 *     """
 *     Given HTML, parse it into a list of Nodes, such that the nodes
*/
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(5, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_html, __pyx_mstate_global->__pyx_n_u_str) < 0) __PYX_ERR(5, 12, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_11parse_fragment, 0, __pyx_mstate_global->__pyx_n_u_parse_fragment, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[59])); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_parse_fragment, __pyx_t_2) < 0) __PYX_ERR(5, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/utils.pxi":1
 * from typing import Literal, Optional, Union, Type             # <<<<<<<<<<<<<<
 * 
 * MAX_HTML_INPUT_SIZE = 250e+7
*/
  __pyx_t_2 = __Pyx_PyList_Pack(4, __pyx_mstate_global->__pyx_n_u_Literal, __pyx_mstate_global->__pyx_n_u_Optional, __pyx_mstate_global->__pyx_n_u_Union, __pyx_mstate_global->__pyx_n_u_Type); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_typing, __pyx_t_2, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_Literal); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Literal, __pyx_t_2) < 0) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_Optional); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Optional, __pyx_t_2) < 0) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_Union); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Union, __pyx_t_2) < 0) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_Type); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Type, __pyx_t_2) < 0) __PYX_ERR(6, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "selectolax/utils.pxi":3
 * from typing import Literal, Optional, Union, Type
 * 
 * MAX_HTML_INPUT_SIZE = 250e+7             # <<<<<<<<<<<<<<
 * 
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_MAX_HTML_INPUT_SIZE, __pyx_mstate_global->__pyx_float_250e_7) < 0) __PYX_ERR(6, 3, __pyx_L1_error)

  /* "selectolax/utils.pxi":5
 * MAX_HTML_INPUT_SIZE = 250e+7
 * 
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]             # <<<<<<<<<<<<<<
 * Parser = Union["HTMLParser", "LexborHTMLParser"]
 * FRAGMENT = Literal[
*/
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_Union); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Type); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_HTMLParser); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Type); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_LexborHTMLParser); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_Pack(2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ParserCls, __pyx_t_4) < 0) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/utils.pxi":6
 * 
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]
 * Parser = Union["HTMLParser", "LexborHTMLParser"]             # <<<<<<<<<<<<<<
 * FRAGMENT = Literal[
 *     "document",
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_Union); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_tuple[7]); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Parser, __pyx_t_2) < 0) __PYX_ERR(6, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/utils.pxi":7
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]
 * Parser = Union["HTMLParser", "LexborHTMLParser"]
 * FRAGMENT = Literal[             # <<<<<<<<<<<<<<
 *     "document",
 *     "fragment",
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Literal); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[8]); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_FRAGMENT, __pyx_t_4) < 0) __PYX_ERR(6, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/utils.pxi":19
 * 
 * 
 * def preprocess_input(html, decode_errors='ignore'):             # <<<<<<<<<<<<<<
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_13preprocess_input, 0, __pyx_mstate_global->__pyx_n_u_preprocess_input, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[60])); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_mstate_global->__pyx_tuple[9]);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_preprocess_input, __pyx_t_4) < 0) __PYX_ERR(6, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/utils.pxi":32
 * 
 * 
 * def do_create_tag(tag: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     if not tag:
 *         raise ValueError("Tag name cannot be empty")
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_tag, __pyx_mstate_global->__pyx_n_u_str) < 0) __PYX_ERR(6, 32, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_parser_cls, __pyx_mstate_global->__pyx_n_u_ParserCls) < 0) __PYX_ERR(6, 32, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_15do_create_tag, 0, __pyx_mstate_global->__pyx_n_u_do_create_tag, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[61])); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_do_create_tag, __pyx_t_2) < 0) __PYX_ERR(6, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/utils.pxi":38
 * 
 * 
 * def get_fragment_type(             # <<<<<<<<<<<<<<
 *     html: str,
 *     parser_cls: ParserCls,
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_html, __pyx_mstate_global->__pyx_n_u_str) < 0) __PYX_ERR(6, 38, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_parser_cls, __pyx_mstate_global->__pyx_n_u_ParserCls) < 0) __PYX_ERR(6, 38, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_tree, __pyx_mstate_global->__pyx_kp_u_Optional_Parser) < 0) __PYX_ERR(6, 38, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_FRAGMENT) < 0) __PYX_ERR(6, 38, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_17get_fragment_type, 0, __pyx_mstate_global->__pyx_n_u_get_fragment_type, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[62])); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_get_fragment_type, __pyx_t_4) < 0) __PYX_ERR(6, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/utils.pxi":81
 * 
 * 
 * def do_parse_fragment(html: str, parser_cls: ParserCls):             # <<<<<<<<<<<<<<
 *     """
 *     Given HTML, parse it into a list of Nodes, such that the nodes
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_html, __pyx_mstate_global->__pyx_n_u_str) < 0) __PYX_ERR(6, 81, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_parser_cls, __pyx_mstate_global->__pyx_n_u_ParserCls) < 0) __PYX_ERR(6, 81, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_19do_parse_fragment, 0, __pyx_mstate_global->__pyx_n_u_do_parse_fragment, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[63])); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_do_parse_fragment, __pyx_t_2) < 0) __PYX_ERR(6, 81, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/parser.pyx":48
 *         self.cached_script_srcs = None
 * 
 *     def css(self, str query):             # <<<<<<<<<<<<<<
 *         """A CSS selector.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_3css, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_css, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[64])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_css, __pyx_t_2) < 0) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/parser.pyx":68
 *         return node.css(query)
 * 
 *     def css_first(self, str query, default=None, strict=False):             # <<<<<<<<<<<<<<
 *         """Same as `css` but returns only the first match.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_5css_first, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_css_first, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[65])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[5]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_css_first, __pyx_t_2) < 0) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/parser.pyx":181
 *         return None
 * 
 *     def tags(self, str name):             # <<<<<<<<<<<<<<
 *         """Returns a list of tags that match specified name.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_7tags, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_tags, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[66])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_tags, __pyx_t_2) < 0) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/parser.pyx":214
 *         return result
 * 
 *     def text(self, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Returns the text of the node including text of all its child nodes.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_9text, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_text, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[67])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[1]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_text, __pyx_t_2) < 0) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/parser.pyx":235
 *         return self.body.text(deep=deep, separator=separator, strip=strip)
 * 
 *     def strip_tags(self, list tags, bool recursive = False):             # <<<<<<<<<<<<<<
 *         """Remove specified tags from the node.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_11strip_tags, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_strip_tags, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[68])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[4]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_strip_tags, __pyx_t_2) < 0) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "selectolax/parser.pyx":277
 *             myhtml_collection_destroy(collection)
 * 
 *     def unwrap_tags(self, list tags, delete_empty : bool = False):             # <<<<<<<<<<<<<<
 *         """Unwraps specified tags from the HTML tree.
 * 
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_delete_empty, __pyx_mstate_global->__pyx_n_u_bool) < 0) __PYX_ERR(0, 277, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_13unwrap_tags, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_unwrap_tags, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[69])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_mstate_global->__pyx_tuple[4]);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_unwrap_tags, __pyx_t_4) < 0) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/parser.pyx":309
 *         return None
 * 
 *     def select(self, query=None):             # <<<<<<<<<<<<<<
 *         """Select nodes given a CSS selector.
 * 
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_15select, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_select, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[70])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_mstate_global->__pyx_tuple[2]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_select, __pyx_t_4) < 0) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/parser.pyx":328
 *             return Selector(node, query)
 * 
 *     def any_css_matches(self, tuple selectors):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the specified CSS selectors matches a node."""
 *         return self.root.any_css_matches(selectors)
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_17any_css_matches, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_any_css_matches, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[71])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_any_css_matches, __pyx_t_4) < 0) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/parser.pyx":332
 *         return self.root.any_css_matches(selectors)
 * 
 *     def scripts_contain(self, str query):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script tags contain specified text.
 * 
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_19scripts_contain, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_scripts_contain, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[72])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_scripts_contain, __pyx_t_4) < 0) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/parser.pyx":345
 *         return self.root.scripts_contain(query)
 * 
 *     def script_srcs_contain(self, tuple queries):             # <<<<<<<<<<<<<<
 *         """Returns True if any of the script SRCs attributes contain on of the specified text.
 * 
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_21script_srcs_contain, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_script_srcs_contain, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[73])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_script_srcs_contain, __pyx_t_4) < 0) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/parser.pyx":357
 *         return self.root.script_srcs_contain(queries)
 * 
 *     def css_matches(self, str selector):             # <<<<<<<<<<<<<<
 *         return self.root.css_matches(selector)
 * 
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_23css_matches, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_css_matches, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[74])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_css_matches, __pyx_t_4) < 0) __PYX_ERR(0, 357, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/parser.pyx":360
 *         return self.root.css_matches(selector)
 * 
 *     def merge_text_nodes(self):             # <<<<<<<<<<<<<<
 *         """Iterates over all text nodes and merges all text nodes that are close to each other.
 * 
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_25merge_text_nodes, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_merge_text_nodes, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[75])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_merge_text_nodes, __pyx_t_4) < 0) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/parser.pyx":396
 *         return obj
 * 
 *     def clone(self):             # <<<<<<<<<<<<<<
 *         """Clone the current tree."""
 *         cdef myhtml_t* myhtml
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_27clone, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser_clone, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[76])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_10selectolax_6parser_HTMLParser, __pyx_mstate_global->__pyx_n_u_clone, __pyx_t_4) < 0) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_33__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[77])); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_4) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.html_tree cannot be converted to a Python object for pickling"
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_10HTMLParser_35__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_HTMLParser___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[78])); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_4) < 0) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Selector(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10selectolax_6parser_21__pyx_unpickle_Selector, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Selector, NULL, __pyx_mstate_global->__pyx_n_u_selectolax_parser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[79])); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Selector, __pyx_t_4) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "selectolax/parser.pyx":2
 * 
 * from cpython.bool cimport bool             # <<<<<<<<<<<<<<
 * from cpython.exc cimport PyErr_SetObject
 * 
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_attributes___get___line_158, __pyx_mstate_global->__pyx_kp_u_Get_all_attributes_that_belong_t) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_attrs___get___line_194, __pyx_mstate_global->__pyx_kp_u_A_dict_like_object_that_is_simil) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_decompose_line_492, __pyx_mstate_global->__pyx_kp_u_Remove_a_Node_from_the_tree_Para) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_unwrap_line_517, __pyx_mstate_global->__pyx_kp_u_Replace_node_with_whatever_is_in) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_strip_tags_line_554, __pyx_mstate_global->__pyx_kp_u_Remove_specified_tags_from_the_H) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_unwrap_tags_line_580, __pyx_mstate_global->__pyx_kp_u_Unwraps_specified_tags_from_the) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_replace_with_line_607, __pyx_mstate_global->__pyx_kp_u_Replace_current_Node_with_specif) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_insert_before_line_648, __pyx_mstate_global->__pyx_kp_u_Insert_a_node_before_the_curren) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_insert_after_line_688, __pyx_mstate_global->__pyx_kp_u_Insert_a_node_after_the_current) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_insert_child_line_728, __pyx_mstate_global->__pyx_kp_u_Insert_a_node_inside_at_the_end) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_unwrap_tags_line_768, __pyx_mstate_global->__pyx_kp_u_Unwraps_specified_tags_from_the_2) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_raw_value___get___line_795, __pyx_mstate_global->__pyx_kp_u_Return_the_raw_unparsed_original) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Node_merge_text_nodes_line_921, __pyx_mstate_global->__pyx_kp_u_Iterates_over_all_text_nodes_and) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_HTMLParser_strip_tags_line_235, __pyx_mstate_global->__pyx_kp_u_Remove_specified_tags_from_the_n) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_HTMLParser_unwrap_tags_line_277, __pyx_mstate_global->__pyx_kp_u_Unwraps_specified_tags_from_the_3) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_HTMLParser_merge_text_nodes_line, __pyx_mstate_global->__pyx_kp_u_Iterates_over_all_text_nodes_and) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_4) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init selectolax.parser", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init selectolax.parser");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */

typedef struct {
    const char *s;
#if 1231 <= 65535
    const unsigned short n;
#elif 1231 / 2 < INT_MAX
    const unsigned int n;
#elif 1231 / 2 < LONG_MAX
    const unsigned long n;
#else
    const Py_ssize_t n;
#endif
#if 1 <= 31
    const unsigned int encoding : 5;
#elif 1 <= 255
    const unsigned char encoding;
#elif 1 <= 65535
    const unsigned short encoding;
#else
    const Py_ssize_t encoding;
#endif
    const unsigned int is_unicode : 1;
    const unsigned int intern : 1;
} __Pyx_StringTabEntry;
static const char * const __pyx_string_tab_encodings[] = { 0 };
static const __Pyx_StringTabEntry __pyx_string_tab[] = {
  {__pyx_k_, sizeof(__pyx_k_), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_ */
  {__pyx_k_A_dict_like_object_that_is_simil, sizeof(__pyx_k_A_dict_like_object_that_is_simil), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_A_dict_like_object_that_is_simil */
  {__pyx_k_Attributes, sizeof(__pyx_k_Attributes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Attributes */
  {__pyx_k_Attributes___iter, sizeof(__pyx_k_Attributes___iter), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Attributes___iter */
  {__pyx_k_Attributes___reduce_cython, sizeof(__pyx_k_Attributes___reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Attributes___reduce_cython */
  {__pyx_k_Attributes___setstate_cython, sizeof(__pyx_k_Attributes___setstate_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Attributes___setstate_cython */
  {__pyx_k_Attributes_get, sizeof(__pyx_k_Attributes_get), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Attributes_get */
  {__pyx_k_Attributes_items, sizeof(__pyx_k_Attributes_items), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Attributes_items */
  {__pyx_k_Attributes_keys, sizeof(__pyx_k_Attributes_keys), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Attributes_keys */
  {__pyx_k_Attributes_sget, sizeof(__pyx_k_Attributes_sget), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Attributes_sget */
  {__pyx_k_Attributes_values, sizeof(__pyx_k_Attributes_values), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Attributes_values */
  {__pyx_k_Bad_CSS_Selectors_s, sizeof(__pyx_k_Bad_CSS_Selectors_s), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Bad_CSS_Selectors_s */
  {__pyx_k_CSSSelector, sizeof(__pyx_k_CSSSelector), 0, 1, 1}, /* PyObject cname: __pyx_n_u_CSSSelector */
  {__pyx_k_CSSSelector___reduce_cython, sizeof(__pyx_k_CSSSelector___reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_CSSSelector___reduce_cython */
  {__pyx_k_CSSSelector___setstate_cython, sizeof(__pyx_k_CSSSelector___setstate_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_CSSSelector___setstate_cython */
  {__pyx_k_Can_t_allocate_memory_for_a_new, sizeof(__pyx_k_Can_t_allocate_memory_for_a_new), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Can_t_allocate_memory_for_a_new */
  {__pyx_k_Can_t_init_MyCSS_Entry_object, sizeof(__pyx_k_Can_t_init_MyCSS_Entry_object), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Can_t_init_MyCSS_Entry_object */
  {__pyx_k_Can_t_init_MyCSS_object, sizeof(__pyx_k_Can_t_init_MyCSS_object), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Can_t_init_MyCSS_object */
  {__pyx_k_Can_t_init_MyHTML_Tree_object, sizeof(__pyx_k_Can_t_init_MyHTML_Tree_object), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Can_t_init_MyHTML_Tree_object */
  {__pyx_k_Can_t_init_MyHTML_object, sizeof(__pyx_k_Can_t_init_MyHTML_object), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Can_t_init_MyHTML_object */
  {__pyx_k_Can_t_obtain_raw_value_for_non_t, sizeof(__pyx_k_Can_t_obtain_raw_value_for_non_t), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Can_t_obtain_raw_value_for_non_t */
  {__pyx_k_Can_t_parse_HTML_status_code_d, sizeof(__pyx_k_Can_t_parse_HTML_status_code_d), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Can_t_parse_HTML_status_code_d */
  {__pyx_k_Expected_1_match_but_found_s_mat, sizeof(__pyx_k_Expected_1_match_but_found_s_mat), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Expected_1_match_but_found_s_mat */
  {__pyx_k_Expected_a_string_but_s_found, sizeof(__pyx_k_Expected_a_string_but_s_found), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Expected_a_string_but_s_found */
  {__pyx_k_Expected_a_string_or_Node_instan, sizeof(__pyx_k_Expected_a_string_or_Node_instan), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Expected_a_string_or_Node_instan */
  {__pyx_k_Expected_at_least, sizeof(__pyx_k_Expected_at_least), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Expected_at_least */
  {__pyx_k_FRAGMENT, sizeof(__pyx_k_FRAGMENT), 0, 1, 1}, /* PyObject cname: __pyx_n_u_FRAGMENT */
  {__pyx_k_Failed_to_allocate_memory_for_st, sizeof(__pyx_k_Failed_to_allocate_memory_for_st), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Failed_to_allocate_memory_for_st */
  {__pyx_k_Function_call_with_ambiguous_arg, sizeof(__pyx_k_Function_call_with_ambiguous_arg), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Function_call_with_ambiguous_arg */
  {__pyx_k_Get_all_attributes_that_belong_t, sizeof(__pyx_k_Get_all_attributes_that_belong_t), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Get_all_attributes_that_belong_t */
  {__pyx_k_HTMLParser, sizeof(__pyx_k_HTMLParser), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser */
  {__pyx_k_HTMLParser___reduce_cython, sizeof(__pyx_k_HTMLParser___reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser___reduce_cython */
  {__pyx_k_HTMLParser___setstate_cython, sizeof(__pyx_k_HTMLParser___setstate_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser___setstate_cython */
  {__pyx_k_HTMLParser_any_css_matches, sizeof(__pyx_k_HTMLParser_any_css_matches), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_any_css_matches */
  {__pyx_k_HTMLParser_chars_s, sizeof(__pyx_k_HTMLParser_chars_s), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_HTMLParser_chars_s */
  {__pyx_k_HTMLParser_clone, sizeof(__pyx_k_HTMLParser_clone), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_clone */
  {__pyx_k_HTMLParser_css, sizeof(__pyx_k_HTMLParser_css), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_css */
  {__pyx_k_HTMLParser_css_first, sizeof(__pyx_k_HTMLParser_css_first), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_css_first */
  {__pyx_k_HTMLParser_css_matches, sizeof(__pyx_k_HTMLParser_css_matches), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_css_matches */
  {__pyx_k_HTMLParser_merge_text_nodes, sizeof(__pyx_k_HTMLParser_merge_text_nodes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_merge_text_nodes */
  {__pyx_k_HTMLParser_merge_text_nodes_line, sizeof(__pyx_k_HTMLParser_merge_text_nodes_line), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_HTMLParser_merge_text_nodes_line */
  {__pyx_k_HTMLParser_script_srcs_contain, sizeof(__pyx_k_HTMLParser_script_srcs_contain), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_script_srcs_contain */
  {__pyx_k_HTMLParser_scripts_contain, sizeof(__pyx_k_HTMLParser_scripts_contain), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_scripts_contain */
  {__pyx_k_HTMLParser_select, sizeof(__pyx_k_HTMLParser_select), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_select */
  {__pyx_k_HTMLParser_strip_tags, sizeof(__pyx_k_HTMLParser_strip_tags), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_strip_tags */
  {__pyx_k_HTMLParser_strip_tags_line_235, sizeof(__pyx_k_HTMLParser_strip_tags_line_235), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_HTMLParser_strip_tags_line_235 */
  {__pyx_k_HTMLParser_tags, sizeof(__pyx_k_HTMLParser_tags), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_tags */
  {__pyx_k_HTMLParser_text, sizeof(__pyx_k_HTMLParser_text), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_text */
  {__pyx_k_HTMLParser_unwrap_tags, sizeof(__pyx_k_HTMLParser_unwrap_tags), 0, 1, 1}, /* PyObject cname: __pyx_n_u_HTMLParser_unwrap_tags */
  {__pyx_k_HTMLParser_unwrap_tags_line_277, sizeof(__pyx_k_HTMLParser_unwrap_tags_line_277), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_HTMLParser_unwrap_tags_line_277 */
  {__pyx_k_IGNORECASE, sizeof(__pyx_k_IGNORECASE), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IGNORECASE */
  {__pyx_k_Incompatible_checksums_0x_x_vs_0, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Incompatible_checksums_0x_x_vs_0 */
  {__pyx_k_Insert_a_node_after_the_current, sizeof(__pyx_k_Insert_a_node_after_the_current), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Insert_a_node_after_the_current */
  {__pyx_k_Insert_a_node_before_the_curren, sizeof(__pyx_k_Insert_a_node_before_the_curren), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Insert_a_node_before_the_curren */
  {__pyx_k_Insert_a_node_inside_at_the_end, sizeof(__pyx_k_Insert_a_node_inside_at_the_end), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Insert_a_node_inside_at_the_end */
  {__pyx_k_Iterates_over_all_text_nodes_and, sizeof(__pyx_k_Iterates_over_all_text_nodes_and), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Iterates_over_all_text_nodes_and */
  {__pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_KeyError */
  {__pyx_k_LexborHTMLParser, sizeof(__pyx_k_LexborHTMLParser), 0, 1, 1}, /* PyObject cname: __pyx_n_u_LexborHTMLParser */
  {__pyx_k_Literal, sizeof(__pyx_k_Literal), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Literal */
  {__pyx_k_MAX_HTML_INPUT_SIZE, sizeof(__pyx_k_MAX_HTML_INPUT_SIZE), 0, 1, 1}, /* PyObject cname: __pyx_n_u_MAX_HTML_INPUT_SIZE */
  {__pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_MemoryError */
  {__pyx_k_No_matching_signature_found, sizeof(__pyx_k_No_matching_signature_found), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_No_matching_signature_found */
  {__pyx_k_Node, sizeof(__pyx_k_Node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node */
  {__pyx_k_Node___reduce_cython, sizeof(__pyx_k_Node___reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node___reduce_cython */
  {__pyx_k_Node___setstate_cython, sizeof(__pyx_k_Node___setstate_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node___setstate_cython */
  {__pyx_k_Node_any_css_matches, sizeof(__pyx_k_Node_any_css_matches), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_any_css_matches */
  {__pyx_k_Node_attributes___get___line_158, sizeof(__pyx_k_Node_attributes___get___line_158), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_attributes___get___line_158 */
  {__pyx_k_Node_attrs___get___line_194, sizeof(__pyx_k_Node_attrs___get___line_194), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_attrs___get___line_194 */
  {__pyx_k_Node_css, sizeof(__pyx_k_Node_css), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_css */
  {__pyx_k_Node_css_first, sizeof(__pyx_k_Node_css_first), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_css_first */
  {__pyx_k_Node_css_matches, sizeof(__pyx_k_Node_css_matches), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_css_matches */
  {__pyx_k_Node_decompose, sizeof(__pyx_k_Node_decompose), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_decompose */
  {__pyx_k_Node_decompose_line_492, sizeof(__pyx_k_Node_decompose_line_492), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_decompose_line_492 */
  {__pyx_k_Node_insert_after, sizeof(__pyx_k_Node_insert_after), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_after */
  {__pyx_k_Node_insert_after_Node, sizeof(__pyx_k_Node_insert_after_Node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_after_Node */
  {__pyx_k_Node_insert_after_bytes, sizeof(__pyx_k_Node_insert_after_bytes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_after_bytes */
  {__pyx_k_Node_insert_after_line_688, sizeof(__pyx_k_Node_insert_after_line_688), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_insert_after_line_688 */
  {__pyx_k_Node_insert_after_str, sizeof(__pyx_k_Node_insert_after_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_after_str */
  {__pyx_k_Node_insert_before, sizeof(__pyx_k_Node_insert_before), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_before */
  {__pyx_k_Node_insert_before_Node, sizeof(__pyx_k_Node_insert_before_Node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_before_Node */
  {__pyx_k_Node_insert_before_bytes, sizeof(__pyx_k_Node_insert_before_bytes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_before_bytes */
  {__pyx_k_Node_insert_before_line_648, sizeof(__pyx_k_Node_insert_before_line_648), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_insert_before_line_648 */
  {__pyx_k_Node_insert_before_str, sizeof(__pyx_k_Node_insert_before_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_before_str */
  {__pyx_k_Node_insert_child, sizeof(__pyx_k_Node_insert_child), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_child */
  {__pyx_k_Node_insert_child_Node, sizeof(__pyx_k_Node_insert_child_Node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_child_Node */
  {__pyx_k_Node_insert_child_bytes, sizeof(__pyx_k_Node_insert_child_bytes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_child_bytes */
  {__pyx_k_Node_insert_child_line_728, sizeof(__pyx_k_Node_insert_child_line_728), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_insert_child_line_728 */
  {__pyx_k_Node_insert_child_str, sizeof(__pyx_k_Node_insert_child_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_insert_child_str */
  {__pyx_k_Node_iter, sizeof(__pyx_k_Node_iter), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_iter */
  {__pyx_k_Node_merge_text_nodes, sizeof(__pyx_k_Node_merge_text_nodes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_merge_text_nodes */
  {__pyx_k_Node_merge_text_nodes_line_921, sizeof(__pyx_k_Node_merge_text_nodes_line_921), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_merge_text_nodes_line_921 */
  {__pyx_k_Node_raw_value___get___line_795, sizeof(__pyx_k_Node_raw_value___get___line_795), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_raw_value___get___line_795 */
  {__pyx_k_Node_remove, sizeof(__pyx_k_Node_remove), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_remove */
  {__pyx_k_Node_replace_with, sizeof(__pyx_k_Node_replace_with), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_replace_with */
  {__pyx_k_Node_replace_with_Node, sizeof(__pyx_k_Node_replace_with_Node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_replace_with_Node */
  {__pyx_k_Node_replace_with_bytes, sizeof(__pyx_k_Node_replace_with_bytes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_replace_with_bytes */
  {__pyx_k_Node_replace_with_line_607, sizeof(__pyx_k_Node_replace_with_line_607), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_replace_with_line_607 */
  {__pyx_k_Node_replace_with_str, sizeof(__pyx_k_Node_replace_with_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_replace_with_str */
  {__pyx_k_Node_s, sizeof(__pyx_k_Node_s), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_s */
  {__pyx_k_Node_script_srcs_contain, sizeof(__pyx_k_Node_script_srcs_contain), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_script_srcs_contain */
  {__pyx_k_Node_scripts_contain, sizeof(__pyx_k_Node_scripts_contain), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_scripts_contain */
  {__pyx_k_Node_select, sizeof(__pyx_k_Node_select), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_select */
  {__pyx_k_Node_strip_tags, sizeof(__pyx_k_Node_strip_tags), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_strip_tags */
  {__pyx_k_Node_strip_tags_line_554, sizeof(__pyx_k_Node_strip_tags_line_554), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_strip_tags_line_554 */
  {__pyx_k_Node_text, sizeof(__pyx_k_Node_text), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_text */
  {__pyx_k_Node_traverse, sizeof(__pyx_k_Node_traverse), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_traverse */
  {__pyx_k_Node_unwrap, sizeof(__pyx_k_Node_unwrap), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_unwrap */
  {__pyx_k_Node_unwrap_line_517, sizeof(__pyx_k_Node_unwrap_line_517), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_unwrap_line_517 */
  {__pyx_k_Node_unwrap_tags, sizeof(__pyx_k_Node_unwrap_tags), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Node_unwrap_tags */
  {__pyx_k_Node_unwrap_tags_line_580, sizeof(__pyx_k_Node_unwrap_tags_line_580), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_unwrap_tags_line_580 */
  {__pyx_k_Node_unwrap_tags_line_768, sizeof(__pyx_k_Node_unwrap_tags_line_768), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Node_unwrap_tags_line_768 */
  {__pyx_k_Note_that_Cython_is_deliberately, sizeof(__pyx_k_Note_that_Cython_is_deliberately), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Note_that_Cython_is_deliberately */
  {__pyx_k_Optional, sizeof(__pyx_k_Optional), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Optional */
  {__pyx_k_Optional_Parser, sizeof(__pyx_k_Optional_Parser), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Optional_Parser */
  {__pyx_k_Parser, sizeof(__pyx_k_Parser), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Parser */
  {__pyx_k_ParserCls, sizeof(__pyx_k_ParserCls), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ParserCls */
  {__pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_PickleError */
  {__pyx_k_Remove_a_Node_from_the_tree_Para, sizeof(__pyx_k_Remove_a_Node_from_the_tree_Para), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Remove_a_Node_from_the_tree_Para */
  {__pyx_k_Remove_specified_tags_from_the_H, sizeof(__pyx_k_Remove_specified_tags_from_the_H), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Remove_specified_tags_from_the_H */
  {__pyx_k_Remove_specified_tags_from_the_n, sizeof(__pyx_k_Remove_specified_tags_from_the_n), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Remove_specified_tags_from_the_n */
  {__pyx_k_Replace_current_Node_with_specif, sizeof(__pyx_k_Replace_current_Node_with_specif), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Replace_current_Node_with_specif */
  {__pyx_k_Replace_node_with_whatever_is_in, sizeof(__pyx_k_Replace_node_with_whatever_is_in), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Replace_node_with_whatever_is_in */
  {__pyx_k_Return_the_raw_unparsed_original, sizeof(__pyx_k_Return_the_raw_unparsed_original), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Return_the_raw_unparsed_original */
  {__pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_RuntimeError */
  {__pyx_k_Selector, sizeof(__pyx_k_Selector), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Selector */
  {__pyx_k_Selector___reduce_cython, sizeof(__pyx_k_Selector___reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Selector___reduce_cython */
  {__pyx_k_Selector___setstate_cython, sizeof(__pyx_k_Selector___setstate_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Selector___setstate_cython */
  {__pyx_k_Selector_any_attribute_longer_th, sizeof(__pyx_k_Selector_any_attribute_longer_th), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Selector_any_attribute_longer_th */
  {__pyx_k_Selector_any_text_contains, sizeof(__pyx_k_Selector_any_text_contains), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Selector_any_text_contains */
  {__pyx_k_Selector_attribute_longer_than, sizeof(__pyx_k_Selector_attribute_longer_than), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Selector_attribute_longer_than */
  {__pyx_k_Selector_css, sizeof(__pyx_k_Selector_css), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Selector_css */
  {__pyx_k_Selector_text_contains, sizeof(__pyx_k_Selector_text_contains), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Selector_text_contains */
  {__pyx_k_Stack, sizeof(__pyx_k_Stack), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Stack */
  {__pyx_k_Stack___reduce_cython, sizeof(__pyx_k_Stack___reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Stack___reduce_cython */
  {__pyx_k_Stack___setstate_cython, sizeof(__pyx_k_Stack___setstate_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Stack___setstate_cython */
  {__pyx_k_Tag_name_cannot_be_empty, sizeof(__pyx_k_Tag_name_cannot_be_empty), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Tag_name_cannot_be_empty */
  {__pyx_k_Tag_name_is_too_long, sizeof(__pyx_k_Tag_name_is_too_long), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Tag_name_is_too_long */
  {__pyx_k_The_specified_HTML_input_is_too, sizeof(__pyx_k_The_specified_HTML_input_is_too), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_The_specified_HTML_input_is_too */
  {__pyx_k_Type, sizeof(__pyx_k_Type), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Type */
  {__pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_TypeError */
  {__pyx_k_UTF_8, sizeof(__pyx_k_UTF_8), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_UTF_8 */
  {__pyx_k_Union, sizeof(__pyx_k_Union), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Union */
  {__pyx_k_Unwraps_specified_tags_from_the, sizeof(__pyx_k_Unwraps_specified_tags_from_the), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Unwraps_specified_tags_from_the */
  {__pyx_k_Unwraps_specified_tags_from_the_2, sizeof(__pyx_k_Unwraps_specified_tags_from_the_2), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Unwraps_specified_tags_from_the_2 */
  {__pyx_k_Unwraps_specified_tags_from_the_3, sizeof(__pyx_k_Unwraps_specified_tags_from_the_3), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Unwraps_specified_tags_from_the_3 */
  {__pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ValueError */
  {__pyx_k__10, sizeof(__pyx_k__10), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__10 */
  {__pyx_k__11, sizeof(__pyx_k__11), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__11 */
  {__pyx_k__12, sizeof(__pyx_k__12), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__12 */
  {__pyx_k__2, sizeof(__pyx_k__2), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__2 */
  {__pyx_k__4, sizeof(__pyx_k__4), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__4 */
  {__pyx_k__5, sizeof(__pyx_k__5), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__5 */
  {__pyx_k__9, sizeof(__pyx_k__9), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__9 */
  {__pyx_k_add_note, sizeof(__pyx_k_add_note), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_add_note */
  {__pyx_k_any_attribute_longer_than, sizeof(__pyx_k_any_attribute_longer_than), 0, 1, 1}, /* PyObject cname: __pyx_n_u_any_attribute_longer_than */
  {__pyx_k_any_css_matches, sizeof(__pyx_k_any_css_matches), 0, 1, 1}, /* PyObject cname: __pyx_n_u_any_css_matches */
  {__pyx_k_any_text_contains, sizeof(__pyx_k_any_text_contains), 0, 1, 1}, /* PyObject cname: __pyx_n_u_any_text_contains */
  {__pyx_k_args, sizeof(__pyx_k_args), 0, 1, 1}, /* PyObject cname: __pyx_n_u_args */
  {__pyx_k_arguments_got, sizeof(__pyx_k_arguments_got), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_arguments_got */
  {__pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 1, 1}, /* PyObject cname: __pyx_n_u_asyncio_coroutines */
  {__pyx_k_attr, sizeof(__pyx_k_attr), 0, 1, 1}, /* PyObject cname: __pyx_n_u_attr */
  {__pyx_k_attribute, sizeof(__pyx_k_attribute), 0, 1, 1}, /* PyObject cname: __pyx_n_u_attribute */
  {__pyx_k_attribute_longer_than, sizeof(__pyx_k_attribute_longer_than), 0, 1, 1}, /* PyObject cname: __pyx_n_u_attribute_longer_than */
  {__pyx_k_attributes, sizeof(__pyx_k_attributes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_attributes */
  {__pyx_k_attributes_2, sizeof(__pyx_k_attributes_2), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_attributes_2 */
  {__pyx_k_attrs, sizeof(__pyx_k_attrs), 0, 1, 1}, /* PyObject cname: __pyx_n_u_attrs */
  {__pyx_k_body, sizeof(__pyx_k_body), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_body */
  {__pyx_k_body_2, sizeof(__pyx_k_body_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_body_2 */
  {__pyx_k_bool, sizeof(__pyx_k_bool), 0, 1, 1}, /* PyObject cname: __pyx_n_u_bool */
  {__pyx_k_bytes, sizeof(__pyx_k_bytes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_bytes */
  {__pyx_k_bytes_html, sizeof(__pyx_k_bytes_html), 0, 1, 1}, /* PyObject cname: __pyx_n_u_bytes_html */
  {__pyx_k_bytes_val, sizeof(__pyx_k_bytes_val), 0, 1, 1}, /* PyObject cname: __pyx_n_u_bytes_val */
  {__pyx_k_c_text, sizeof(__pyx_k_c_text), 0, 1, 1}, /* PyObject cname: __pyx_n_u_c_text */
  {__pyx_k_capacity, sizeof(__pyx_k_capacity), 0, 1, 1}, /* PyObject cname: __pyx_n_u_capacity */
  {__pyx_k_class_getitem, sizeof(__pyx_k_class_getitem), 0, 1, 1}, /* PyObject cname: __pyx_n_u_class_getitem */
  {__pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 1, 1}, /* PyObject cname: __pyx_n_u_cline_in_traceback */
  {__pyx_k_clone, sizeof(__pyx_k_clone), 0, 1, 1}, /* PyObject cname: __pyx_n_u_clone */
  {__pyx_k_close, sizeof(__pyx_k_close), 0, 1, 1}, /* PyObject cname: __pyx_n_u_close */
  {__pyx_k_cls, sizeof(__pyx_k_cls), 0, 1, 1}, /* PyObject cname: __pyx_n_u_cls */
  {__pyx_k_collection, sizeof(__pyx_k_collection), 0, 1, 1}, /* PyObject cname: __pyx_n_u_collection */
  {__pyx_k_compile, sizeof(__pyx_k_compile), 0, 1, 1}, /* PyObject cname: __pyx_n_u_compile */
  {__pyx_k_create_tag, sizeof(__pyx_k_create_tag), 0, 1, 1}, /* PyObject cname: __pyx_n_u_create_tag */
  {__pyx_k_css, sizeof(__pyx_k_css), 0, 1, 1}, /* PyObject cname: __pyx_n_u_css */
  {__pyx_k_css_first, sizeof(__pyx_k_css_first), 0, 1, 1}, /* PyObject cname: __pyx_n_u_css_first */
  {__pyx_k_css_matches, sizeof(__pyx_k_css_matches), 0, 1, 1}, /* PyObject cname: __pyx_n_u_css_matches */
  {__pyx_k_current_node, sizeof(__pyx_k_current_node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_current_node */
  {__pyx_k_d, sizeof(__pyx_k_d), 0, 1, 1}, /* PyObject cname: __pyx_n_u_d */
  {__pyx_k_decode, sizeof(__pyx_k_decode), 0, 1, 1}, /* PyObject cname: __pyx_n_u_decode */
  {__pyx_k_decode_errors, sizeof(__pyx_k_decode_errors), 0, 1, 1}, /* PyObject cname: __pyx_n_u_decode_errors */
  {__pyx_k_decompose, sizeof(__pyx_k_decompose), 0, 1, 1}, /* PyObject cname: __pyx_n_u_decompose */
  {__pyx_k_deep, sizeof(__pyx_k_deep), 0, 1, 1}, /* PyObject cname: __pyx_n_u_deep */
  {__pyx_k_default, sizeof(__pyx_k_default), 0, 1, 1}, /* PyObject cname: __pyx_n_u_default */
  {__pyx_k_defaults, sizeof(__pyx_k_defaults), 0, 1, 1}, /* PyObject cname: __pyx_n_u_defaults */
  {__pyx_k_delete_empty, sizeof(__pyx_k_delete_empty), 0, 1, 1}, /* PyObject cname: __pyx_n_u_delete_empty */
  {__pyx_k_detect_encoding, sizeof(__pyx_k_detect_encoding), 0, 1, 1}, /* PyObject cname: __pyx_n_u_detect_encoding */
  {__pyx_k_dict, sizeof(__pyx_k_dict), 0, 1, 1}, /* PyObject cname: __pyx_n_u_dict */
  {__pyx_k_dict_2, sizeof(__pyx_k_dict_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_dict_2 */
  {__pyx_k_disable, sizeof(__pyx_k_disable), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_disable */
  {__pyx_k_do_create_tag, sizeof(__pyx_k_do_create_tag), 0, 1, 1}, /* PyObject cname: __pyx_n_u_do_create_tag */
  {__pyx_k_do_parse_fragment, sizeof(__pyx_k_do_parse_fragment), 0, 1, 1}, /* PyObject cname: __pyx_n_u_do_parse_fragment */
  {__pyx_k_document, sizeof(__pyx_k_document), 0, 1, 1}, /* PyObject cname: __pyx_n_u_document */
  {__pyx_k_document_no_body, sizeof(__pyx_k_document_no_body), 0, 1, 1}, /* PyObject cname: __pyx_n_u_document_no_body */
  {__pyx_k_document_no_head, sizeof(__pyx_k_document_no_head), 0, 1, 1}, /* PyObject cname: __pyx_n_u_document_no_head */
  {__pyx_k_document_no_head_no_body, sizeof(__pyx_k_document_no_head_no_body), 0, 1, 1}, /* PyObject cname: __pyx_n_u_document_no_head_no_body */
  {__pyx_k_element, sizeof(__pyx_k_element), 0, 1, 1}, /* PyObject cname: __pyx_n_u_element */
  {__pyx_k_enable, sizeof(__pyx_k_enable), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_enable */
  {__pyx_k_encode, sizeof(__pyx_k_encode), 0, 1, 1}, /* PyObject cname: __pyx_n_u_encode */
  {__pyx_k_endswith, sizeof(__pyx_k_endswith), 0, 1, 1}, /* PyObject cname: __pyx_n_u_endswith */
  {__pyx_k_errors, sizeof(__pyx_k_errors), 0, 1, 1}, /* PyObject cname: __pyx_n_u_errors */
  {__pyx_k_final_length, sizeof(__pyx_k_final_length), 0, 1, 1}, /* PyObject cname: __pyx_n_u_final_length */
  {__pyx_k_final_text, sizeof(__pyx_k_final_text), 0, 1, 1}, /* PyObject cname: __pyx_n_u_final_text */
  {__pyx_k_find, sizeof(__pyx_k_find), 0, 1, 1}, /* PyObject cname: __pyx_n_u_find */
  {__pyx_k_finditer, sizeof(__pyx_k_finditer), 0, 1, 1}, /* PyObject cname: __pyx_n_u_finditer */
  {__pyx_k_frag_type, sizeof(__pyx_k_frag_type), 0, 1, 1}, /* PyObject cname: __pyx_n_u_frag_type */
  {__pyx_k_fragment, sizeof(__pyx_k_fragment), 0, 1, 1}, /* PyObject cname: __pyx_n_u_fragment */
  {__pyx_k_func, sizeof(__pyx_k_func), 0, 1, 1}, /* PyObject cname: __pyx_n_u_func */
  {__pyx_k_fused_sigindex_ref, sizeof(__pyx_k_fused_sigindex_ref), 0, 1, 1}, /* PyObject cname: __pyx_n_u_fused_sigindex_ref */
  {__pyx_k_gc, sizeof(__pyx_k_gc), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_gc */
  {__pyx_k_get, sizeof(__pyx_k_get), 0, 1, 1}, /* PyObject cname: __pyx_n_u_get */
  {__pyx_k_get_fragment_type, sizeof(__pyx_k_get_fragment_type), 0, 1, 1}, /* PyObject cname: __pyx_n_u_get_fragment_type */
  {__pyx_k_getitem, sizeof(__pyx_k_getitem), 0, 1, 1}, /* PyObject cname: __pyx_n_u_getitem */
  {__pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 1, 1}, /* PyObject cname: __pyx_n_u_getstate */
  {__pyx_k_has_body, sizeof(__pyx_k_has_body), 0, 1, 1}, /* PyObject cname: __pyx_n_u_has_body */
  {__pyx_k_has_head, sizeof(__pyx_k_has_head), 0, 1, 1}, /* PyObject cname: __pyx_n_u_has_head */
  {__pyx_k_has_html, sizeof(__pyx_k_has_html), 0, 1, 1}, /* PyObject cname: __pyx_n_u_has_html */
  {__pyx_k_head, sizeof(__pyx_k_head), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_head */
  {__pyx_k_head_2, sizeof(__pyx_k_head_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_head_2 */
  {__pyx_k_head_and_body, sizeof(__pyx_k_head_and_body), 0, 1, 1}, /* PyObject cname: __pyx_n_u_head_and_body */
  {__pyx_k_html, sizeof(__pyx_k_html), 0, 1, 1}, /* PyObject cname: __pyx_n_u_html */
  {__pyx_k_html_2, sizeof(__pyx_k_html_2), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_html_2 */
  {__pyx_k_html_body_head_er, sizeof(__pyx_k_html_body_head_er), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_html_body_head_er */
  {__pyx_k_html_len, sizeof(__pyx_k_html_len), 0, 1, 1}, /* PyObject cname: __pyx_n_u_html_len */
  {__pyx_k_html_re, sizeof(__pyx_k_html_re), 0, 1, 1}, /* PyObject cname: __pyx_n_u_html_re */
  {__pyx_k_html_tree, sizeof(__pyx_k_html_tree), 0, 1, 1}, /* PyObject cname: __pyx_n_u_html_tree */
  {__pyx_k_html_tree_is_still_NULL_even_aft, sizeof(__pyx_k_html_tree_is_still_NULL_even_aft), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_html_tree_is_still_NULL_even_aft */
  {__pyx_k_i, sizeof(__pyx_k_i), 0, 1, 1}, /* PyObject cname: __pyx_n_u_i */
  {__pyx_k_ignore, sizeof(__pyx_k_ignore), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ignore */
  {__pyx_k_import, sizeof(__pyx_k_import), 0, 1, 1}, /* PyObject cname: __pyx_n_u_import */
  {__pyx_k_include_text, sizeof(__pyx_k_include_text), 0, 1, 1}, /* PyObject cname: __pyx_n_u_include_text */
  {__pyx_k_initializing, sizeof(__pyx_k_initializing), 0, 1, 1}, /* PyObject cname: __pyx_n_u_initializing */
  {__pyx_k_insert_after, sizeof(__pyx_k_insert_after), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_after */
  {__pyx_k_insert_after_Node, sizeof(__pyx_k_insert_after_Node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_after_Node */
  {__pyx_k_insert_after_bytes, sizeof(__pyx_k_insert_after_bytes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_after_bytes */
  {__pyx_k_insert_after_str, sizeof(__pyx_k_insert_after_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_after_str */
  {__pyx_k_insert_before, sizeof(__pyx_k_insert_before), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_before */
  {__pyx_k_insert_before_Node, sizeof(__pyx_k_insert_before_Node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_before_Node */
  {__pyx_k_insert_before_bytes, sizeof(__pyx_k_insert_before_bytes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_before_bytes */
  {__pyx_k_insert_before_str, sizeof(__pyx_k_insert_before_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_before_str */
  {__pyx_k_insert_child, sizeof(__pyx_k_insert_child), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_child */
  {__pyx_k_insert_child_Node, sizeof(__pyx_k_insert_child_Node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_child_Node */
  {__pyx_k_insert_child_bytes, sizeof(__pyx_k_insert_child_bytes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_child_bytes */
  {__pyx_k_insert_child_str, sizeof(__pyx_k_insert_child_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_insert_child_str */
  {__pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 1, 1}, /* PyObject cname: __pyx_n_u_is_coroutine */
  {__pyx_k_isenabled, sizeof(__pyx_k_isenabled), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_isenabled */
  {__pyx_k_items, sizeof(__pyx_k_items), 0, 1, 1}, /* PyObject cname: __pyx_n_u_items */
  {__pyx_k_items_2, sizeof(__pyx_k_items_2), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_items_2 */
  {__pyx_k_iter, sizeof(__pyx_k_iter), 0, 1, 1}, /* PyObject cname: __pyx_n_u_iter */
  {__pyx_k_iter_2, sizeof(__pyx_k_iter_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_iter_2 */
  {__pyx_k_key, sizeof(__pyx_k_key), 0, 1, 1}, /* PyObject cname: __pyx_n_u_key */
  {__pyx_k_keys, sizeof(__pyx_k_keys), 0, 1, 1}, /* PyObject cname: __pyx_n_u_keys */
  {__pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 1, 1}, /* PyObject cname: __pyx_n_u_kwargs */
  {__pyx_k_left_length, sizeof(__pyx_k_left_length), 0, 1, 1}, /* PyObject cname: __pyx_n_u_left_length */
  {__pyx_k_left_text, sizeof(__pyx_k_left_text), 0, 1, 1}, /* PyObject cname: __pyx_n_u_left_text */
  {__pyx_k_length, sizeof(__pyx_k_length), 0, 1, 1}, /* PyObject cname: __pyx_n_u_length */
  {__pyx_k_main, sizeof(__pyx_k_main), 0, 1, 1}, /* PyObject cname: __pyx_n_u_main */
  {__pyx_k_match, sizeof(__pyx_k_match), 0, 1, 1}, /* PyObject cname: __pyx_n_u_match */
  {__pyx_k_mem_id, sizeof(__pyx_k_mem_id), 0, 1, 1}, /* PyObject cname: __pyx_n_u_mem_id */
  {__pyx_k_merge_text_nodes, sizeof(__pyx_k_merge_text_nodes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_merge_text_nodes */
  {__pyx_k_module, sizeof(__pyx_k_module), 0, 1, 1}, /* PyObject cname: __pyx_n_u_module */
  {__pyx_k_myhtml, sizeof(__pyx_k_myhtml), 0, 1, 1}, /* PyObject cname: __pyx_n_u_myhtml */
  {__pyx_k_n_results, sizeof(__pyx_k_n_results), 0, 1, 1}, /* PyObject cname: __pyx_n_u_n_results */
  {__pyx_k_name, sizeof(__pyx_k_name), 0, 1, 1}, /* PyObject cname: __pyx_n_u_name */
  {__pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_name_2 */
  {__pyx_k_new, sizeof(__pyx_k_new), 0, 1, 1}, /* PyObject cname: __pyx_n_u_new */
  {__pyx_k_next, sizeof(__pyx_k_next), 0, 1, 1}, /* PyObject cname: __pyx_n_u_next */
  {__pyx_k_next_node, sizeof(__pyx_k_next_node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_next_node */
  {__pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_no_default___reduce___due_to_non */
  {__pyx_k_node, sizeof(__pyx_k_node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_node */
  {__pyx_k_node_src, sizeof(__pyx_k_node_src), 0, 1, 1}, /* PyObject cname: __pyx_n_u_node_src */
  {__pyx_k_node_text, sizeof(__pyx_k_node_text), 0, 1, 1}, /* PyObject cname: __pyx_n_u_node_text */
  {__pyx_k_nodes, sizeof(__pyx_k_nodes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_nodes */
  {__pyx_k_parse_fragment, sizeof(__pyx_k_parse_fragment), 0, 1, 1}, /* PyObject cname: __pyx_n_u_parse_fragment */
  {__pyx_k_parser_cls, sizeof(__pyx_k_parser_cls), 0, 1, 1}, /* PyObject cname: __pyx_n_u_parser_cls */
  {__pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pickle */
  {__pyx_k_pop, sizeof(__pyx_k_pop), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pop */
  {__pyx_k_preprocess_input, sizeof(__pyx_k_preprocess_input), 0, 1, 1}, /* PyObject cname: __pyx_n_u_preprocess_input */
  {__pyx_k_pybyte_name, sizeof(__pyx_k_pybyte_name), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pybyte_name */
  {__pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_PickleError */
  {__pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_checksum */
  {__pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_result */
  {__pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_state */
  {__pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_type */
  {__pyx_k_pyx_unpickle_Selector, sizeof(__pyx_k_pyx_unpickle_Selector), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_unpickle_Selector */
  {__pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_vtable */
  {__pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 1, 1}, /* PyObject cname: __pyx_n_u_qualname */
  {__pyx_k_queries, sizeof(__pyx_k_queries), 0, 1, 1}, /* PyObject cname: __pyx_n_u_queries */
  {__pyx_k_query, sizeof(__pyx_k_query), 0, 1, 1}, /* PyObject cname: __pyx_n_u_query */
  {__pyx_k_range, sizeof(__pyx_k_range), 0, 1, 1}, /* PyObject cname: __pyx_n_u_range */
  {__pyx_k_re, sizeof(__pyx_k_re), 0, 1, 1}, /* PyObject cname: __pyx_n_u_re */
  {__pyx_k_recursive, sizeof(__pyx_k_recursive), 0, 1, 1}, /* PyObject cname: __pyx_n_u_recursive */
  {__pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 1, 1}, /* PyObject cname: __pyx_n_u_reduce */
  {__pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_reduce_cython */
  {__pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 1, 1}, /* PyObject cname: __pyx_n_u_reduce_ex */
  {__pyx_k_remove, sizeof(__pyx_k_remove), 0, 1, 1}, /* PyObject cname: __pyx_n_u_remove */
  {__pyx_k_replace_with, sizeof(__pyx_k_replace_with), 0, 1, 1}, /* PyObject cname: __pyx_n_u_replace_with */
  {__pyx_k_replace_with_Node, sizeof(__pyx_k_replace_with_Node), 0, 1, 1}, /* PyObject cname: __pyx_n_u_replace_with_Node */
  {__pyx_k_replace_with_bytes, sizeof(__pyx_k_replace_with_bytes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_replace_with_bytes */
  {__pyx_k_replace_with_str, sizeof(__pyx_k_replace_with_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_replace_with_str */
  {__pyx_k_result, sizeof(__pyx_k_result), 0, 1, 1}, /* PyObject cname: __pyx_n_u_result */
  {__pyx_k_results, sizeof(__pyx_k_results), 0, 1, 1}, /* PyObject cname: __pyx_n_u_results */
  {__pyx_k_return, sizeof(__pyx_k_return), 0, 1, 1}, /* PyObject cname: __pyx_n_u_return */
  {__pyx_k_right_length, sizeof(__pyx_k_right_length), 0, 1, 1}, /* PyObject cname: __pyx_n_u_right_length */
  {__pyx_k_right_text, sizeof(__pyx_k_right_text), 0, 1, 1}, /* PyObject cname: __pyx_n_u_right_text */
  {__pyx_k_root, sizeof(__pyx_k_root), 0, 1, 1}, /* PyObject cname: __pyx_n_u_root */
  {__pyx_k_script, sizeof(__pyx_k_script), 0, 1, 1}, /* PyObject cname: __pyx_n_u_script */
  {__pyx_k_script_srcs_contain, sizeof(__pyx_k_script_srcs_contain), 0, 1, 1}, /* PyObject cname: __pyx_n_u_script_srcs_contain */
  {__pyx_k_scripts_contain, sizeof(__pyx_k_scripts_contain), 0, 1, 1}, /* PyObject cname: __pyx_n_u_scripts_contain */
  {__pyx_k_select, sizeof(__pyx_k_select), 0, 1, 1}, /* PyObject cname: __pyx_n_u_select */
  {__pyx_k_selectolax_modest_node_pxi, sizeof(__pyx_k_selectolax_modest_node_pxi), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_selectolax_modest_node_pxi */
  {__pyx_k_selectolax_modest_selection_pxi, sizeof(__pyx_k_selectolax_modest_selection_pxi), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_selectolax_modest_selection_pxi */
  {__pyx_k_selectolax_modest_util_pxi, sizeof(__pyx_k_selectolax_modest_util_pxi), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_selectolax_modest_util_pxi */
  {__pyx_k_selectolax_modest_utils_pxi, sizeof(__pyx_k_selectolax_modest_utils_pxi), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_selectolax_modest_utils_pxi */
  {__pyx_k_selectolax_parser, sizeof(__pyx_k_selectolax_parser), 0, 1, 1}, /* PyObject cname: __pyx_n_u_selectolax_parser */
  {__pyx_k_selectolax_parser, sizeof(__pyx_k_selectolax_parser), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_selectolax_parser */
  {__pyx_k_selectolax_parser_pyx, sizeof(__pyx_k_selectolax_parser_pyx), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_selectolax_parser_pyx */
  {__pyx_k_selectolax_utils_pxi, sizeof(__pyx_k_selectolax_utils_pxi), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_selectolax_utils_pxi */
  {__pyx_k_selector, sizeof(__pyx_k_selector), 0, 1, 1}, /* PyObject cname: __pyx_n_u_selector */
  {__pyx_k_selectors, sizeof(__pyx_k_selectors), 0, 1, 1}, /* PyObject cname: __pyx_n_u_selectors */
  {__pyx_k_self, sizeof(__pyx_k_self), 0, 1, 1}, /* PyObject cname: __pyx_n_u_self */
  {__pyx_k_self_css_entry_self_finder_self, sizeof(__pyx_k_self_css_entry_self_finder_self), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_self_css_entry_self_finder_self */
  {__pyx_k_self_html_tree_cannot_be_convert, sizeof(__pyx_k_self_html_tree_cannot_be_convert), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_self_html_tree_cannot_be_convert */
  {__pyx_k_self_node_cannot_be_converted_to, sizeof(__pyx_k_self_node_cannot_be_converted_to), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_self_node_cannot_be_converted_to */
  {__pyx_k_send, sizeof(__pyx_k_send), 0, 1, 1}, /* PyObject cname: __pyx_n_u_send */
  {__pyx_k_separator, sizeof(__pyx_k_separator), 0, 1, 1}, /* PyObject cname: __pyx_n_u_separator */
  {__pyx_k_set_name, sizeof(__pyx_k_set_name), 0, 1, 1}, /* PyObject cname: __pyx_n_u_set_name */
  {__pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 1, 1}, /* PyObject cname: __pyx_n_u_setstate */
  {__pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_setstate_cython */
  {__pyx_k_sget, sizeof(__pyx_k_sget), 0, 1, 1}, /* PyObject cname: __pyx_n_u_sget */
  {__pyx_k_spec, sizeof(__pyx_k_spec), 0, 1, 1}, /* PyObject cname: __pyx_n_u_spec */
  {__pyx_k_split, sizeof(__pyx_k_split), 0, 1, 1}, /* PyObject cname: __pyx_n_u_split */
  {__pyx_k_src, sizeof(__pyx_k_src), 0, 1, 1}, /* PyObject cname: __pyx_n_u_src */
  {__pyx_k_src_nodes, sizeof(__pyx_k_src_nodes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_src_nodes */
  {__pyx_k_stack, sizeof(__pyx_k_stack), 0, 1, 1}, /* PyObject cname: __pyx_n_u_stack */
  {__pyx_k_start, sizeof(__pyx_k_start), 0, 1, 1}, /* PyObject cname: __pyx_n_u_start */
  {__pyx_k_state, sizeof(__pyx_k_state), 0, 1, 1}, /* PyObject cname: __pyx_n_u_state */
  {__pyx_k_status, sizeof(__pyx_k_status), 0, 1, 1}, /* PyObject cname: __pyx_n_u_status */
  {__pyx_k_str, sizeof(__pyx_k_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_str */
  {__pyx_k_strict, sizeof(__pyx_k_strict), 0, 1, 1}, /* PyObject cname: __pyx_n_u_strict */
  {__pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_stringsource */
  {__pyx_k_strip, sizeof(__pyx_k_strip), 0, 1, 1}, /* PyObject cname: __pyx_n_u_strip */
  {__pyx_k_strip_tags, sizeof(__pyx_k_strip_tags), 0, 1, 1}, /* PyObject cname: __pyx_n_u_strip_tags */
  {__pyx_k_tag, sizeof(__pyx_k_tag), 0, 1, 1}, /* PyObject cname: __pyx_n_u_tag */
  {__pyx_k_tags, sizeof(__pyx_k_tags), 0, 1, 1}, /* PyObject cname: __pyx_n_u_tags */
  {__pyx_k_test, sizeof(__pyx_k_test), 0, 1, 1}, /* PyObject cname: __pyx_n_u_test */
  {__pyx_k_text, sizeof(__pyx_k_text), 0, 1, 1}, /* PyObject cname: __pyx_n_u_text */
  {__pyx_k_text_contains, sizeof(__pyx_k_text_contains), 0, 1, 1}, /* PyObject cname: __pyx_n_u_text_contains */
  {__pyx_k_text_nodes, sizeof(__pyx_k_text_nodes), 0, 1, 1}, /* PyObject cname: __pyx_n_u_text_nodes */
  {__pyx_k_throw, sizeof(__pyx_k_throw), 0, 1, 1}, /* PyObject cname: __pyx_n_u_throw */
  {__pyx_k_traverse, sizeof(__pyx_k_traverse), 0, 1, 1}, /* PyObject cname: __pyx_n_u_traverse */
  {__pyx_k_tree, sizeof(__pyx_k_tree), 0, 1, 1}, /* PyObject cname: __pyx_n_u_tree */
  {__pyx_k_typing, sizeof(__pyx_k_typing), 0, 1, 1}, /* PyObject cname: __pyx_n_u_typing */
  {__pyx_k_undef, sizeof(__pyx_k_undef), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_undef */
  {__pyx_k_unknown, sizeof(__pyx_k_unknown), 0, 1, 1}, /* PyObject cname: __pyx_n_u_unknown */
  {__pyx_k_unwrap, sizeof(__pyx_k_unwrap), 0, 1, 1}, /* PyObject cname: __pyx_n_u_unwrap */
  {__pyx_k_unwrap_tags, sizeof(__pyx_k_unwrap_tags), 0, 1, 1}, /* PyObject cname: __pyx_n_u_unwrap_tags */
  {__pyx_k_update, sizeof(__pyx_k_update), 0, 1, 1}, /* PyObject cname: __pyx_n_u_update */
  {__pyx_k_use_meta_tags, sizeof(__pyx_k_use_meta_tags), 0, 1, 1}, /* PyObject cname: __pyx_n_u_use_meta_tags */
  {__pyx_k_use_setstate, sizeof(__pyx_k_use_setstate), 0, 1, 1}, /* PyObject cname: __pyx_n_u_use_setstate */
  {__pyx_k_utf_8, sizeof(__pyx_k_utf_8), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_utf_8 */
  {__pyx_k_val, sizeof(__pyx_k_val), 0, 1, 1}, /* PyObject cname: __pyx_n_u_val */
  {__pyx_k_value, sizeof(__pyx_k_value), 0, 1, 1}, /* PyObject cname: __pyx_n_u_value */
  {__pyx_k_values, sizeof(__pyx_k_values), 0, 1, 1}, /* PyObject cname: __pyx_n_u_values */
  {0, 0, 0, 0, 0}
};
/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry const *t, PyObject **target, const char* const* encoding_names);

/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(1, 40, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(1, 57, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(2, 2, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_range); if (!__pyx_builtin_range) __PYX_ERR(1, 169, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(3, 20, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(3, 80, __pyx_L1_error)
  __pyx_builtin___import__ = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_import); if (!__pyx_builtin___import__) __PYX_ERR(3, 607, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xc5dfe7e, 0xad8d8ab, 0xc00154f):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xc5dfe7e, 0xad8d8ab, 0xc00154f) = (node, nodes))" % __pyx_checksum
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_Pack(3, __pyx_mstate_global->__pyx_int_207486590, __pyx_mstate_global->__pyx_int_181983403, __pyx_mstate_global->__pyx_int_201332047); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);

  /* "selectolax/modest/selection.pxi":105
 *         return bool(self.nodes)
 * 
 *     def text_contains(self, str text, bool deep=True, str separator='', bool strip=False):             # <<<<<<<<<<<<<<
 *         """Filter all current matches given text."""
 *         nodes = []
*/
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(3, Py_True, __pyx_mstate_global->__pyx_kp_u_, Py_False); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(1, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);

  /* "selectolax/modest/selection.pxi":126
 *         return False
 * 
 *     def attribute_longer_than(self, str attribute, int length, str start  = None):             # <<<<<<<<<<<<<<
 *         """Filter all current matches by attribute length.
 * 
*/
  __pyx_mstate_global->__pyx_tuple[2] = PyTuple_Pack(1, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[2])) __PYX_ERR(1, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[2]);

  /* "selectolax/modest/node.pxi":115
 *             return default
 * 
 *     def sget(self, key, default=""):             # <<<<<<<<<<<<<<
 *         """Same as get, but returns empty strings instead of None values for empty attributes."""
 *         try:
*/
  __pyx_mstate_global->__pyx_tuple[3] = PyTuple_Pack(1, __pyx_mstate_global->__pyx_kp_u_); if (unlikely(!__pyx_mstate_global->__pyx_tuple[3])) __PYX_ERR(3, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[3]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[3]);

  /* "selectolax/modest/node.pxi":328
 *         return text
 * 
 *     def iter(self, include_text=False):             # <<<<<<<<<<<<<<
 *         """Iterate over nodes on the current level.
 * 
*/
  __pyx_mstate_global->__pyx_tuple[4] = PyTuple_Pack(1, Py_False); if (unlikely(!__pyx_mstate_global->__pyx_tuple[4])) __PYX_ERR(3, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[4]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[4]);

  /* "selectolax/modest/node.pxi":478
 *         return find_matches(self.parser, self.node, (selector, ))
 * 
 *     def css_first(self, str query, default=None, bool strict=False):             # <<<<<<<<<<<<<<
 *         """Evaluate CSS selector against current node and its child nodes."""
 *         results = self.css(query)
*/
  __pyx_mstate_global->__pyx_tuple[5] = PyTuple_Pack(2, Py_None, Py_False); if (unlikely(!__pyx_mstate_global->__pyx_tuple[5])) __PYX_ERR(3, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[5]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[5]);

  /* "selectolax/modest/node.pxi":492
 *         return default
 * 
 *     def decompose(self, bool recursive=True):             # <<<<<<<<<<<<<<
 *         """Remove a Node from the tree.
 * 
*/
  __pyx_mstate_global->__pyx_tuple[6] = PyTuple_Pack(1, Py_True); if (unlikely(!__pyx_mstate_global->__pyx_tuple[6])) __PYX_ERR(3, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[6]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[6]);

  /* "selectolax/utils.pxi":6
 * 
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]
 * Parser = Union["HTMLParser", "LexborHTMLParser"]             # <<<<<<<<<<<<<<
 * FRAGMENT = Literal[
 *     "document",
*/
  __pyx_mstate_global->__pyx_tuple[7] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_n_u_HTMLParser, __pyx_mstate_global->__pyx_n_u_LexborHTMLParser); if (unlikely(!__pyx_mstate_global->__pyx_tuple[7])) __PYX_ERR(4, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[7]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[7]);

  /* "selectolax/utils.pxi":7
 * ParserCls = Union[Type["HTMLParser"], Type["LexborHTMLParser"]]
 * Parser = Union["HTMLParser", "LexborHTMLParser"]
 * FRAGMENT = Literal[             # <<<<<<<<<<<<<<
 *     "document",
 *     "fragment",
*/
  __pyx_mstate_global->__pyx_tuple[8] = PyTuple_Pack(8, __pyx_mstate_global->__pyx_n_u_document, __pyx_mstate_global->__pyx_n_u_fragment, __pyx_mstate_global->__pyx_n_u_head_2, __pyx_mstate_global->__pyx_n_u_body_2, __pyx_mstate_global->__pyx_n_u_head_and_body, __pyx_mstate_global->__pyx_n_u_document_no_head, __pyx_mstate_global->__pyx_n_u_document_no_body, __pyx_mstate_global->__pyx_n_u_document_no_head_no_body); if (unlikely(!__pyx_mstate_global->__pyx_tuple[8])) __PYX_ERR(4, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[8]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[8]);

  /* "selectolax/utils.pxi":19
 * 
 * 
 * def preprocess_input(html, decode_errors='ignore'):             # <<<<<<<<<<<<<<
 *     if isinstance(html, (str, unicode)):
 *         bytes_html = html.encode('UTF-8', errors=decode_errors)
*/
  __pyx_mstate_global->__pyx_tuple[9] = PyTuple_Pack(1, ((PyObject*)__pyx_mstate_global->__pyx_n_u_ignore)); if (unlikely(!__pyx_mstate_global->__pyx_tuple[9])) __PYX_ERR(4, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[9]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[9]);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.method_name = &__pyx_mstate->__pyx_n_u_get;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.method_name = &__pyx_mstate->__pyx_n_u_values;
  __pyx_mstate->__pyx_umethod_PyBytes_Type__encode.type = (PyObject*)(&PyBytes_Type);
  __pyx_mstate->__pyx_umethod_PyBytes_Type__encode.method_name = &__pyx_mstate->__pyx_n_u_encode;
  __pyx_mstate->__pyx_umethod_PyUnicode_Type__strip.type = (PyObject*)(&PyUnicode_Type);
  __pyx_mstate->__pyx_umethod_PyUnicode_Type__strip.method_name = &__pyx_mstate->__pyx_n_u_strip;
  if (__Pyx_InitStrings(__pyx_string_tab, __pyx_mstate->__pyx_string_tab, __pyx_string_tab_encodings) < 0) __PYX_ERR(0, 2, __pyx_L1_error);
  __pyx_mstate->__pyx_float_250e_7 = PyFloat_FromDouble(250e+7); if (unlikely(!__pyx_mstate->__pyx_float_250e_7)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_mstate->__pyx_int_0 = PyLong_FromLong(0); if (unlikely(!__pyx_mstate->__pyx_int_0)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_mstate->__pyx_int_1 = PyLong_FromLong(1); if (unlikely(!__pyx_mstate->__pyx_int_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_mstate->__pyx_int_100 = PyLong_FromLong(100); if (unlikely(!__pyx_mstate->__pyx_int_100)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_mstate->__pyx_int_181983403 = PyLong_FromLong(181983403L); if (unlikely(!__pyx_mstate->__pyx_int_181983403)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_mstate->__pyx_int_201332047 = PyLong_FromLong(201332047L); if (unlikely(!__pyx_mstate->__pyx_int_201332047)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_mstate->__pyx_int_207486590 = PyLong_FromLong(207486590L); if (unlikely(!__pyx_mstate->__pyx_int_207486590)) __PYX_ERR(0, 2, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */
\
        typedef struct {
            unsigned int argcount : 3;
            unsigned int num_posonly_args : 1;
            unsigned int num_kwonly_args : 1;
            unsigned int nlocals : 4;
            unsigned int flags : 10;
            unsigned int first_line : 10;
            unsigned int line_table_length : 13;
        } __Pyx_PyCode_New_function_description;
/* NewCodeObj.proto */
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        const char *line_table,
        PyObject *tuple_dedup_map
);


static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  PyObject* tuple_dedup_map = PyDict_New();
  if (unlikely(!tuple_dedup_map)) return -1;
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 59, 2};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attr, __pyx_mstate->__pyx_n_u_key};
    __pyx_mstate_global->__pyx_codeobj_tab[0] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_iter, __pyx_k_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[0])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 101, 2};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key};
    __pyx_mstate_global->__pyx_codeobj_tab[1] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_items, __pyx_k_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[1])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 105, 2};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key};
    __pyx_mstate_global->__pyx_codeobj_tab[2] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_values, __pyx_k_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[2])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 328, 5};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_include_text, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_next_node};
    __pyx_mstate_global->__pyx_codeobj_tab[3] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_iter_2, __pyx_k__13, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[3])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 353, 5};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_include_text, __pyx_mstate->__pyx_n_u_stack, __pyx_mstate->__pyx_n_u_current_node, __pyx_mstate->__pyx_n_u_next_node};
    __pyx_mstate_global->__pyx_codeobj_tab[4] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_traverse, __pyx_k_A_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[4])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[5] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[5])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[6] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[6])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 85, 58};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_query};
    __pyx_mstate_global->__pyx_codeobj_tab[7] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_selection_pxi, __pyx_mstate->__pyx_n_u_css, __pyx_k_A_A_HD_1_1D_Yl_IQ_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[7])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 105, 80};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_text, __pyx_mstate->__pyx_n_u_deep, __pyx_mstate->__pyx_n_u_separator, __pyx_mstate->__pyx_n_u_strip, __pyx_mstate->__pyx_n_u_nodes, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_node_text};
    __pyx_mstate_global->__pyx_codeobj_tab[8] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_selection_pxi, __pyx_mstate->__pyx_n_u_text_contains, __pyx_k_00DOST_HD_E_vZ_z_U_Q_WAQ_IQ_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[8])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 116, 69};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_text, __pyx_mstate->__pyx_n_u_deep, __pyx_mstate->__pyx_n_u_separator, __pyx_mstate->__pyx_n_u_strip, __pyx_mstate->__pyx_n_u_nodes, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_node_text};
    __pyx_mstate_global->__pyx_codeobj_tab[9] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_selection_pxi, __pyx_mstate->__pyx_n_u_any_text_contains, __pyx_k_44H_WX_HD_E_vZ_z_U_Q_q_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[9])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 126, 108};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attribute, __pyx_mstate->__pyx_n_u_length, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_nodes, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_attr};
    __pyx_mstate_global->__pyx_codeobj_tab[10] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_selection_pxi, __pyx_mstate->__pyx_n_u_attribute_longer_than, __pyx_k_LA_HD_4_aq_uD_d_1_t1D_QgRs_1_s, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[10])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 141, 98};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attribute, __pyx_mstate->__pyx_n_u_length, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_nodes, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_attr};
    __pyx_mstate_global->__pyx_codeobj_tab[11] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_selection_pxi, __pyx_mstate->__pyx_n_u_any_attribute_longer_than, __pyx_k_PPQ_HD_4_aq_uD_d_1_t1D_QgRs_1_s, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[11])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 107};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[12] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_T_A_G1F_a_vWA_q_t6_S_G7_q_Qg_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[12])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16, 11};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[13] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_avQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[13])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[14] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[14])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[15] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[15])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 98, 11};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[16] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_keys, __pyx_k_A_t9A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[16])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 109, 25};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_default};
    __pyx_mstate_global->__pyx_codeobj_tab[17] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_get, __pyx_k_q_4q_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[17])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 115, 44};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_default, __pyx_mstate->__pyx_n_u_val};
    __pyx_mstate_global->__pyx_codeobj_tab[18] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_sget, __pyx_k_aq_t3a_a_1_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[18])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[19] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[19])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[20] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[20])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 252, 239};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_deep, __pyx_mstate->__pyx_n_u_separator, __pyx_mstate->__pyx_n_u_strip, __pyx_mstate->__pyx_n_u_text, __pyx_mstate->__pyx_n_u_c_text, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_node_text};
    __pyx_mstate_global->__pyx_codeobj_tab[21] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_text, __pyx_k_1_q_E_4q_t5_1_gQ_7_Q_gQk_WA_av, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[21])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 466, 19};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_query};
    __pyx_mstate_global->__pyx_codeobj_tab[22] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_css, __pyx_k_A_z_it7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[22])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 470, 19};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_selectors};
    __pyx_mstate_global->__pyx_codeobj_tab[23] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_any_css_matches, __pyx_k_A_1D_WA, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[23])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 474, 19};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_selector};
    __pyx_mstate_global->__pyx_codeobj_tab[24] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_css_matches, __pyx_k_A_1D_XQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[24])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 478, 78};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_query, __pyx_mstate->__pyx_n_u_default, __pyx_mstate->__pyx_n_u_strict, __pyx_mstate->__pyx_n_u_results, __pyx_mstate->__pyx_n_u_n_results};
    __pyx_mstate_global->__pyx_codeobj_tab[25] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_css_first, __pyx_k_Q_d_1_Cq_Rq_wd_Ba_j_J_A_7_1_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[25])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 492, 30};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_recursive};
    __pyx_mstate_global->__pyx_codeobj_tab[26] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_decompose, __pyx_k_q_1_Q_at1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[26])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 513, 14};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_recursive};
    __pyx_mstate_global->__pyx_codeobj_tab[27] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_remove, __pyx_k_A_Jaq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[27])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 517, 126};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_delete_empty, __pyx_mstate->__pyx_n_u_next_node, __pyx_mstate->__pyx_n_u_current_node};
    __pyx_mstate_global->__pyx_codeobj_tab[28] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_unwrap, __pyx_k_A_4uG3a_q_4q_4uF_1_4uA_A_Cq_L_g, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[28])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 554, 38};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_tags, __pyx_mstate->__pyx_n_u_recursive, __pyx_mstate->__pyx_n_u_element, __pyx_mstate->__pyx_n_u_tag};
    __pyx_mstate_global->__pyx_codeobj_tab[29] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_strip_tags, __pyx_k_5Q_G1_4t1A_z_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[29])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 580, 36};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_tags, __pyx_mstate->__pyx_n_u_delete_empty, __pyx_mstate->__pyx_n_u_element, __pyx_mstate->__pyx_n_u_tag};
    __pyx_mstate_global->__pyx_codeobj_tab[30] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_unwrap_tags, __pyx_k_4A_G1_4t1A_waq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[30])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 607, 162};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[31] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_replace_with_str, __pyx_k_A8_Qhe7_Qd_Oq_x_Ql_Qd_at1_q_q_g, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[31])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 607, 162};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[32] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_replace_with_bytes, __pyx_k_A8_Qhe7_Qd_Oq_x_Ql_Qd_at1_q_q_g, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[32])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 607, 162};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[33] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_replace_with_Node, __pyx_k_A8_Qhe7_Qd_Oq_x_Ql_Qd_at1_q_q_g, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[33])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 607, 162};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_bytes_val};
    __pyx_mstate_global->__pyx_codeobj_tab[34] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_replace_with_str, __pyx_k_A8_Qhe7_Qd_Oq_x_Ql_Qd_at1_q_q_g, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[34])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 648, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[35] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_before_str, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_Qd_q_q_g_AXX_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[35])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 648, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[36] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_before_bytes, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_Qd_q_q_g_AXX_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[36])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 648, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[37] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_before_Node, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_Qd_q_q_g_AXX_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[37])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 648, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_bytes_val};
    __pyx_mstate_global->__pyx_codeobj_tab[38] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_before_str, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_Qd_q_q_g_AXX_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[38])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 688, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[39] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_after_str, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_AT_q_q_g_AXX_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[39])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 688, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[40] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_after_bytes, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_AT_q_q_g_AXX_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[40])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 688, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[41] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_after_Node, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_AT_q_q_g_AXX_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[41])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 688, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_bytes_val};
    __pyx_mstate_global->__pyx_codeobj_tab[42] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_after_str, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_AT_q_q_g_AXX_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[42])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 728, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[43] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_child_str, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_AT_q_q_g_AXX_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[43])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 728, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[44] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_child_bytes, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_AT_q_q_g_AXX_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[44])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 728, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value};
    __pyx_mstate_global->__pyx_codeobj_tab[45] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_child_Node, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_AT_q_q_g_AXX_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[45])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 728, 144};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_bytes_val};
    __pyx_mstate_global->__pyx_codeobj_tab[46] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_insert_child_str, __pyx_k_A_Qhe7_Qd_Oq_x_Ql_AT_q_q_g_AXX_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[46])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 768, 36};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_tags, __pyx_mstate->__pyx_n_u_delete_empty, __pyx_mstate->__pyx_n_u_element, __pyx_mstate->__pyx_n_u_tag};
    __pyx_mstate_global->__pyx_codeobj_tab[47] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_unwrap_tags, __pyx_k_4A_G1_4t1A_waq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[47])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 822, 18};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_query};
    __pyx_mstate_global->__pyx_codeobj_tab[48] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_select, __pyx_k_1_xq_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[48])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 838, 114};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_query, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_nodes, __pyx_mstate->__pyx_n_u_text_nodes, __pyx_mstate->__pyx_n_u_node_text, __pyx_mstate->__pyx_n_u_text};
    __pyx_mstate_global->__pyx_codeobj_tab[49] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_scripts_contain, __pyx_k_A_4w_3a_Jat9D_q_D_Qe1_1_gQa_a_HD, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[49])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 864, 121};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_queries, __pyx_mstate->__pyx_n_u_nodes, __pyx_mstate->__pyx_n_u_src_nodes, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_node_src, __pyx_mstate->__pyx_n_u_text, __pyx_mstate->__pyx_n_u_query};
    __pyx_mstate_global->__pyx_codeobj_tab[50] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_script_srcs_contain, __pyx_k_A_4w_Q_Jat9D_q_4vT_1_WAQ_Q_HD_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[50])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 10, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 921, 271};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_stack, __pyx_mstate->__pyx_n_u_current_node, __pyx_mstate->__pyx_n_u_next_node, __pyx_mstate->__pyx_n_u_left_text, __pyx_mstate->__pyx_n_u_right_text, __pyx_mstate->__pyx_n_u_final_text, __pyx_mstate->__pyx_n_u_left_length, __pyx_mstate->__pyx_n_u_right_length, __pyx_mstate->__pyx_n_u_final_length};
    __pyx_mstate_global->__pyx_codeobj_tab[51] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_node_pxi, __pyx_mstate->__pyx_n_u_merge_text_nodes, __pyx_k_A_5_1_U_4q_d_y_5_A_HC_8_L_a_XS_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[51])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[52] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[52])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[53] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[53])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 19, 108};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_html, __pyx_mstate->__pyx_n_u_decode_errors, __pyx_mstate->__pyx_n_u_bytes_html, __pyx_mstate->__pyx_n_u_html_len};
    __pyx_mstate_global->__pyx_codeobj_tab[54] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_utils_pxi, __pyx_mstate->__pyx_n_u_preprocess_input, __pyx_k_1_z_a_T_1F_Q_iq_D_a_s_1_y_j___q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[54])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 32, 46};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_tag, __pyx_mstate->__pyx_n_u_parser_cls};
    __pyx_mstate_global->__pyx_codeobj_tab[55] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_utils_pxi, __pyx_mstate->__pyx_n_u_do_create_tag, __pyx_k_t1_j_ARuHE_AQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[55])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 38, 264};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_html, __pyx_mstate->__pyx_n_u_parser_cls, __pyx_mstate->__pyx_n_u_tree, __pyx_mstate->__pyx_n_u_re, __pyx_mstate->__pyx_n_u_html_re, __pyx_mstate->__pyx_n_u_has_html, __pyx_mstate->__pyx_n_u_has_head, __pyx_mstate->__pyx_n_u_has_body, __pyx_mstate->__pyx_n_u_match};
    __pyx_mstate_global->__pyx_codeobj_tab[56] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_utils_pxi, __pyx_mstate->__pyx_n_u_get_fragment_type, __pyx_k_t1_z_b_5Rq_q_q_q_5_Cq_q_3c_q_3c, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[56])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 81, 275};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_html, __pyx_mstate->__pyx_n_u_parser_cls, __pyx_mstate->__pyx_n_u_tree, __pyx_mstate->__pyx_n_u_frag_type};
    __pyx_mstate_global->__pyx_codeobj_tab[57] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_utils_pxi, __pyx_mstate->__pyx_n_u_do_parse_fragment, __pyx_k_Q_4vQ_Qa_A_z_A_q_A_z_A_E_1Ja_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[57])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 4, 18};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_tag};
    __pyx_mstate_global->__pyx_codeobj_tab[58] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_util_pxi, __pyx_mstate->__pyx_n_u_create_tag, __pyx_k_A_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[58])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 12, 19};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_html};
    __pyx_mstate_global->__pyx_codeobj_tab[59] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_modest_util_pxi, __pyx_mstate->__pyx_n_u_parse_fragment, __pyx_k_AV1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[59])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 19, 108};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_html, __pyx_mstate->__pyx_n_u_decode_errors, __pyx_mstate->__pyx_n_u_bytes_html, __pyx_mstate->__pyx_n_u_html_len};
    __pyx_mstate_global->__pyx_codeobj_tab[60] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_utils_pxi, __pyx_mstate->__pyx_n_u_preprocess_input, __pyx_k_1_z_a_T_1F_Q_iq_D_a_s_1_y_j___q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[60])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 32, 46};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_tag, __pyx_mstate->__pyx_n_u_parser_cls};
    __pyx_mstate_global->__pyx_codeobj_tab[61] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_utils_pxi, __pyx_mstate->__pyx_n_u_do_create_tag, __pyx_k_t1_j_ARuHE_AQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[61])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 38, 264};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_html, __pyx_mstate->__pyx_n_u_parser_cls, __pyx_mstate->__pyx_n_u_tree, __pyx_mstate->__pyx_n_u_re, __pyx_mstate->__pyx_n_u_html_re, __pyx_mstate->__pyx_n_u_has_html, __pyx_mstate->__pyx_n_u_has_head, __pyx_mstate->__pyx_n_u_has_body, __pyx_mstate->__pyx_n_u_match};
    __pyx_mstate_global->__pyx_codeobj_tab[62] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_utils_pxi, __pyx_mstate->__pyx_n_u_get_fragment_type, __pyx_k_t1_z_b_5Rq_q_q_q_5_Cq_q_3c_q_3c, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[62])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 81, 275};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_html, __pyx_mstate->__pyx_n_u_parser_cls, __pyx_mstate->__pyx_n_u_tree, __pyx_mstate->__pyx_n_u_frag_type};
    __pyx_mstate_global->__pyx_codeobj_tab[63] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_utils_pxi, __pyx_mstate->__pyx_n_u_do_parse_fragment, __pyx_k_Q_4vQ_Qa_A_z_A_q_A_z_A_E_1Ja_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[63])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 48, 30};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_query, __pyx_mstate->__pyx_n_u_node};
    __pyx_mstate_global->__pyx_codeobj_tab[64] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_css, __pyx_k_A_T_j_A_t4q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[64])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 68, 39};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_query, __pyx_mstate->__pyx_n_u_default, __pyx_mstate->__pyx_n_u_strict, __pyx_mstate->__pyx_n_u_node};
    __pyx_mstate_global->__pyx_codeobj_tab[65] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_css_first, __pyx_k_T_j_A_t_QgYa, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[65])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 181, 173};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name_2, __pyx_mstate->__pyx_n_u_collection, __pyx_mstate->__pyx_n_u_pybyte_name, __pyx_mstate->__pyx_n_u_status, __pyx_mstate->__pyx_n_u_result, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_node};
    __pyx_mstate_global->__pyx_codeobj_tab[66] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_tags, __pyx_k_A_4q_AQ_3avRq_AQ_q_d_Q_Qd_fMQTTU, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[66])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 214, 49};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_deep, __pyx_mstate->__pyx_n_u_separator, __pyx_mstate->__pyx_n_u_strip};
    __pyx_mstate_global->__pyx_codeobj_tab[67] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_text, __pyx_k_1_4t1_1_t5_Qe6_fA, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[67])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 235, 141};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_tags, __pyx_mstate->__pyx_n_u_recursive, __pyx_mstate->__pyx_n_u_collection, __pyx_mstate->__pyx_n_u_status, __pyx_mstate->__pyx_n_u_tag, __pyx_mstate->__pyx_n_u_pybyte_name, __pyx_mstate->__pyx_n_u_i};
    __pyx_mstate_global->__pyx_codeobj_tab[68] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_strip_tags, __pyx_k_5Q_0q_G1_WAQ_1_l_UXXYYgghhi_Q_w, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[68])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 277, 35};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_tags, __pyx_mstate->__pyx_n_u_delete_empty};
    __pyx_mstate_global->__pyx_codeobj_tab[69] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_unwrap_tags, __pyx_k_4G1_4vWA_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[69])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 309, 30};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_query, __pyx_mstate->__pyx_n_u_node};
    __pyx_mstate_global->__pyx_codeobj_tab[70] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_select, __pyx_k_1_t1_1_81F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[70])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 328, 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_selectors};
    __pyx_mstate_global->__pyx_codeobj_tab[71] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_any_css_matches, __pyx_k_A_t5, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[71])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 332, 18};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_query};
    __pyx_mstate_global->__pyx_codeobj_tab[72] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_scripts_contain, __pyx_k_A_t5_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[72])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 345, 18};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_queries};
    __pyx_mstate_global->__pyx_codeobj_tab[73] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_script_srcs_contain, __pyx_k_A_t5_AQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[73])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 357, 15};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_selector};
    __pyx_mstate_global->__pyx_codeobj_tab[74] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_css_matches, __pyx_k_A_t5_AQ_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[74])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 360, 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[75] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_merge_text_nodes, __pyx_k_A_t5_3, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[75])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 396, 163};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_myhtml, __pyx_mstate->__pyx_n_u_status, __pyx_mstate->__pyx_n_u_html_tree, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_cls};
    __pyx_mstate_global->__pyx_codeobj_tab[76] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_selectolax_parser_pyx, __pyx_mstate->__pyx_n_u_clone, __pyx_k_A_A_A_7_Q_aq_Qk_7_Q_aq_Qk_Zq_9Kq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[76])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[77] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[77])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3, 9};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[78] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[78])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 75};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_PickleError, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[79] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_pyx_unpickle_Selector, __pyx_k_hk_A_1_vvxxy_881A_7_nA_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[79])) goto bad;
  }
  Py_DECREF(tuple_dedup_map);
  return 0;
  bad:
  Py_DECREF(tuple_dedup_map);
  return -1;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else

if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2, __pyx_L1_error)

  /* CachedMethodType.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
{
    PyObject *typesModule=NULL;
    typesModule = PyImport_ImportModule("types");
    if (typesModule) {
        __pyx_mstate_global->__Pyx_CachedMethodType = PyObject_GetAttrString(typesModule, "MethodType");
        Py_DECREF(typesModule);
    }
} // error handling follows
#endif

if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* TupleAndListFromArray */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < 0)) {
            Py_DECREF(res);
            return NULL;
        }
        Py_INCREF(src[i]);
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject *const *args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject *const *args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
    PyObject *kwdefs;
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object"))) {
        return NULL;
    }
    if (
            co->co_kwonlyargcount == 0 &&
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED)
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
        vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
        if (f) {
            return f(func, args, _nargs, NULL);
        }
        #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
        if (__Pyx_CyFunction_CheckExact(func)) {
            __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
            if (f) return f(func, args, _nargs, NULL);
        }
        #elif CYTHON_COMPILING_IN_LIMITED_API && CYTHON_VECTORCALL
        return PyObject_Vectorcall(func, args, _nargs, NULL);
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* UnpackUnboundCMethod */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywords */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    Py_ssize_t pos = 0;
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (PyDict_Next(kwds, &pos, &key, NULL)) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            #if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(key);
            #endif
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
            #if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(key);
            #endif
        }
    }
    __Pyx_END_CRITICAL_SECTION();
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    PyObject *extra_info = __pyx_mstate_global->__pyx_empty_unicode;
    int from_annotation_subclass = 0;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (!exact) {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    } else if (exact == 2) {
        if (__Pyx_TypeCheck(obj, type)) {
            from_annotation_subclass = 1;
            extra_info = __pyx_mstate_global->__pyx_kp_u_Note_that_Cython_is_deliberately;
        }
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")"
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        "%s%U"
#endif
        , name, type_name, obj_type_name
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        , (from_annotation_subclass ? ". " : ""), extra_info
#endif
        );
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (exact == 2 && from_annotation_subclass) {
        PyObject *res;
        PyObject *vargs[2];
        vargs[0] = PyErr_GetRaisedException();
        vargs[1] = extra_info;
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_kp_u_add_note, vargs, 2, NULL);
        Py_XDECREF(res);
        PyErr_SetRaisedException(vargs[0]);
    }
#endif
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* decode_c_string */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* RejectKeywords */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds) {
    PyObject *key = NULL;
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds))) {
        key = __Pyx_PySequence_ITEM(kwds, 0);
    } else {
        Py_ssize_t pos = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
        if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return;
#endif
        PyDict_Next(kwds, &pos, &key, NULL);
        Py_INCREF(key);
    }
    if (likely(key)) {
        PyErr_Format(PyExc_TypeError,
            "%s() got an unexpected keyword argument '%U'",
            function_name, key);
        Py_DECREF(key);
    }
}

/* RaiseException */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* PyObjectVectorCallKwBuilder */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    Py_INCREF(key);
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* PyObjectVectorCallMethodKwBuilder */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames) {
    PyObject *result;
    PyObject *obj = PyObject_GetAttr(args[0], name);
    if (unlikely(!obj))
        return NULL;
    result = __Pyx_Object_Vectorcall_CallFromBuilder(obj, args+1, nargsf-1, kwnames);
    Py_DECREF(obj);
    return result;
}
#endif

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
    __Pyx_TypeName obj_type_name;
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyLong_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyLong_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is unsliceable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
bad:
    return NULL;
}

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C0000
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#elif __PYX_LIMITED_VERSION_HEX > 0x030C0000
    local_value = PyErr_GetRaisedException();
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
#if __PYX_LIMITED_VERSION_HEX > 0x030C0000
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
#else
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
#endif // __PYX_LIMITED_VERSION_HEX > 0x030C0000
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#elif __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    PyErr_SetHandledException(local_value);
    Py_XDECREF(local_value);
    Py_XDECREF(local_type);
    Py_XDECREF(local_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
#if __PYX_LIMITED_VERSION_HEX <= 0x030C0000
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
#endif
}

/* pep479 */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen) {
    PyObject *exc, *val, *tb, *cur_exc, *new_exc;
    __Pyx_PyThreadState_declare
    int is_async_stopiteration = 0;
    CYTHON_MAYBE_UNUSED_VAR(in_async_gen);
    __Pyx_PyThreadState_assign
    cur_exc = __Pyx_PyErr_CurrentExceptionType();
    if (likely(!__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopIteration))) {
        if (in_async_gen && unlikely(__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopAsyncIteration))) {
            is_async_stopiteration = 1;
        } else {
            return;
        }
    }
    __Pyx_GetException(&exc, &val, &tb);
    Py_XDECREF(exc);
    Py_XDECREF(tb);
    new_exc = PyObject_CallFunction(PyExc_RuntimeError, "s",
        is_async_stopiteration ? "async generator raised StopAsyncIteration" :
        in_async_gen ? "async generator raised StopIteration" :
        "generator raised StopIteration");
    if (!new_exc) {
        Py_XDECREF(val);
        return;
    }
    PyException_SetCause(new_exc, val); // steals ref to val
    PyErr_SetObject(PyExc_RuntimeError, new_exc);
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE && !CYTHON_AVOID_BORROWED_REFS && !CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            return __Pyx_PyList_GetItemRef(o, n);
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}

/* PyObjectCallOneArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_mstate_global->__pyx_n_u_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result;
    if (unlikely(!s)) return NULL;
    result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* CIntToDigits */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp = PySequence_Repeat(padding, uoffset - prepend_sign);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyUnicode_Concat(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyUnicode_Concat(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* COrdinalToPyUnicode */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value) {
    return value <= 1114111;
}
static PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t ulength, char padding_char) {
    if (likely(ulength <= 250)) {
        char chars[256];
        if (value <= 255) {
            memset(chars, padding_char, (size_t) (ulength - 1));
            chars[ulength-1] = (char) value;
            return PyUnicode_DecodeLatin1(chars, ulength, NULL);
        }
        char *cpos = chars + sizeof(chars);
        if (value < 0x800) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xc0 | (value & 0x1f));
        } else if (value < 0x10000) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xe0 | (value & 0x0f));
        } else {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xf0 | (value & 0x07));
        }
        cpos -= ulength;
        memset(cpos, padding_char, (size_t) (ulength - 1));
        return PyUnicode_DecodeUTF8(cpos, chars + sizeof(chars) - cpos, NULL);
    }
    if (value <= 127 && CYTHON_USE_UNICODE_INTERNALS) {
        const char chars[1] = {(char) value};
        return __Pyx_PyUnicode_BuildFromAscii(ulength, chars, 1, 0, padding_char);
    }
    {
        PyObject *uchar, *padding_uchar, *padding, *result;
        padding_uchar = PyUnicode_FromOrdinal(padding_char);
        if (unlikely(!padding_uchar)) return NULL;
        padding = PySequence_Repeat(padding_uchar, ulength - 1);
        Py_DECREF(padding_uchar);
        if (unlikely(!padding)) return NULL;
        uchar = PyUnicode_FromOrdinal(value);
        if (unlikely(!uchar)) {
            Py_DECREF(padding);
            return NULL;
        }
        result = PyUnicode_Concat(padding, uchar);
        Py_DECREF(padding);
        Py_DECREF(uchar);
        return result;
    }
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(Py_ssize_t)*3+2];
    char *dpos, *end = digits + sizeof(Py_ssize_t)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    Py_ssize_t remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_ssize_t neg_one = (Py_ssize_t) -1, const_zero = (Py_ssize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'c') {
        if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                     !(sizeof(value) <= 2 || value & ~ (Py_ssize_t) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
            PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
            return NULL;
        }
        if (width <= 1) {
            return PyUnicode_FromOrdinal((int) value);
        }
        return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
    }
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (Py_ssize_t) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (Py_ssize_t) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (Py_ssize_t) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
        Py_INCREF(values[i]);
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* DictGetItem */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    if (unlikely(__Pyx_PyDict_GetItemRef(d, key, &value) == 0)) { // no value, no error
        if (unlikely(PyTuple_Check(key))) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args)) {
                PyErr_SetObject(PyExc_KeyError, args);
                Py_DECREF(args);
            }
        } else {
            PyErr_SetObject(PyExc_KeyError, key);
        }
    }
    return value;
}
#endif

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_long(long value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(long)*3+2];
    char *dpos, *end = digits + sizeof(long)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    long remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'c') {
        if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                     !(sizeof(value) <= 2 || value & ~ (long) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
            PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
            return NULL;
        }
        if (width <= 1) {
            return PyUnicode_FromOrdinal((int) value);
        }
        return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
    }
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (long) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (long) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (long) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (unlikely(!j)) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            Py_INCREF(v);
#if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
            PyList_SetItem(o, n, v);
#else
            PyObject* old = PyList_GET_ITEM(o, n);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
#endif
            return 1;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_ass_subscript) {
            int r;
            PyObject *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return -1;
            r = mm->mp_ass_subscript(o, key, v);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return sm->sq_ass_item(o, i, v);
        }
    }
#else
    if (is_list || !PyMapping_Check(o))
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyLong_FromSsize_t(i), v);
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* PyObjectCallNoArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(tp);
    PyErr_Format(PyExc_AttributeError,
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else {
      Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(t);
 #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely(size < 0)) return;
 #endif
      if (size < index) {
        __Pyx_RaiseNeedMoreValuesError(size);
      } else {
        __Pyx_RaiseTooManyValuesError(index);
      }
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
        PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple) {
    if (likely(is_tuple || PyTuple_Check(tuple))) {
        Py_ssize_t size;
        if (has_known_size) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        size = __Pyx_PyTuple_GET_SIZE(tuple);
        if (likely(size == 2)) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        if (size >= 0) {
            __Pyx_UnpackTupleError(tuple, 2);
        }
        return -1;
    } else {
        return __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple);
    }
}
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
    value1 = __Pyx_PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = __Pyx_PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = __Pyx_PyObject_GetIterNextFunc(iter);
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
#if CYTHON_COMPILING_IN_PYPY
#include <string.h>
#endif
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#else
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE int __Pyx_dict_iter_next_source_is_dict(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem) {
    PyObject *key, *value;
    if (unlikely(orig_length != PyDict_Size(iter_obj))) {
        PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
        return -1;
    }
    if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
        return 0;
    }
    if (pitem) {
        PyObject* tuple = PyTuple_New(2);
        if (unlikely(!tuple)) {
            return -1;
        }
        Py_INCREF(key);
        Py_INCREF(value);
        #if CYTHON_ASSUME_SAFE_MACROS
        PyTuple_SET_ITEM(tuple, 0, key);
        PyTuple_SET_ITEM(tuple, 1, value);
        #else
        if (unlikely(PyTuple_SetItem(tuple, 0, key) < 0)) {
            Py_DECREF(value);
            Py_DECREF(tuple);
            return -1;
        }
        if (unlikely(PyTuple_SetItem(tuple, 1, value) < 0)) {
            Py_DECREF(tuple);
            return -1;
        }
        #endif
        *pitem = tuple;
    } else {
        if (pkey) {
            Py_INCREF(key);
            *pkey = key;
        }
        if (pvalue) {
            Py_INCREF(value);
            *pvalue = value;
        }
    }
    return 1;
}
#endif
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        int result;
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_BEGIN_CRITICAL_SECTION(iter_obj);
#endif
        result = __Pyx_dict_iter_next_source_is_dict(iter_obj, orig_length, ppos, pkey, pvalue, pitem);
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_END_CRITICAL_SECTION();
#endif
        return result;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(tuple_size < 0)) return -1;
        #endif
        if (unlikely(pos >= tuple_size)) return 0;
        *ppos = pos + 1;
        #if CYTHON_ASSUME_SAFE_MACROS
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        #else
        next_item = PyTuple_GetItem(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        #endif
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t list_size = __Pyx_PyList_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(list_size < 0)) return -1;
        #endif
        if (unlikely(pos >= list_size)) return 0;
        *ppos = pos + 1;
        #if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
        next_item = PyList_GetItemRef(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        #elif CYTHON_ASSUME_SAFE_MACROS
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
        #else
        next_item = PyList_GetItem(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        Py_INCREF(next_item);
        #endif
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* CallUnboundCMethod0 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        if (likely(cfunc->flag == METH_NOARGS))
            return __Pyx_CallCFunction(cfunc, self, NULL);
        if (likely(cfunc->flag == METH_FASTCALL))
            return __Pyx_CallCFunctionFast(cfunc, self, NULL, 0);
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, NULL, 0, NULL);
        if (likely(cfunc->flag == (METH_VARARGS | METH_KEYWORDS)))
            return __Pyx_CallCFunctionWithKeywords(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple, NULL);
        if (cfunc->flag == METH_VARARGS)
            return __Pyx_CallCFunction(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple);
        return __Pyx__CallUnboundCMethod0(cfunc, self);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod0(&tmp_cfunc, self);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod0(cfunc, self);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *result;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
    result = __Pyx_PyObject_CallOneArg(cfunc->method, self);
    return result;
}

/* py_dict_values */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_values, d);
}

/* PyObjectCall2Args */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    int was_initialized =  __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return __Pyx_CallCFunction(cfunc, self, arg);
        } else if (flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, &arg, 1);
        } else if (flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, &arg, 1, NULL);
        }
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod1(&tmp_cfunc, self, arg);
    }
#endif
    PyObject* result = __Pyx__CallUnboundCMethod1(cfunc, self, arg);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *args = PyTuple_New(1);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
    } else
#endif
    {
        result = __Pyx_PyObject_Call2Args(cfunc->method, self, arg);
    }
    return result;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyBytes_CheckExact(key) || PyUnicode_CheckExact(key) || PyLong_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* UnicodeConcatInPlace */
# if CYTHON_COMPILING_IN_CPYTHON
static int
__Pyx_unicode_modifiable(PyObject *unicode)
{
    if (Py_REFCNT(unicode) != 1)
        return 0;
    if (!PyUnicode_CheckExact(unicode))
        return 0;
    if (PyUnicode_CHECK_INTERNED(unicode))
        return 0;
    return 1;
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    ) {
    PyObject *left = *p_left;
    Py_ssize_t left_len, right_len, new_len;
    if (unlikely(__Pyx_PyUnicode_READY(left) == -1))
        return NULL;
    if (unlikely(__Pyx_PyUnicode_READY(right) == -1))
        return NULL;
    left_len = PyUnicode_GET_LENGTH(left);
    if (left_len == 0) {
        Py_INCREF(right);
        return right;
    }
    right_len = PyUnicode_GET_LENGTH(right);
    if (right_len == 0) {
        Py_INCREF(left);
        return left;
    }
    if (unlikely(left_len > PY_SSIZE_T_MAX - right_len)) {
        PyErr_SetString(PyExc_OverflowError,
                        "strings are too large to concat");
        return NULL;
    }
    new_len = left_len + right_len;
    if (__Pyx_unicode_modifiable(left)
            && PyUnicode_CheckExact(right)
            && PyUnicode_KIND(right) <= PyUnicode_KIND(left)
            && !(PyUnicode_IS_ASCII(left) && !PyUnicode_IS_ASCII(right))) {
        int ret;
        __Pyx_GIVEREF(*p_left);
        ret = PyUnicode_Resize(p_left, new_len);
        __Pyx_GOTREF(*p_left);
        if (unlikely(ret != 0))
            return NULL;
        #if PY_VERSION_HEX >= 0x030d0000
        if (unlikely(PyUnicode_CopyCharacters(*p_left, left_len, right, 0, right_len) < 0)) return NULL;
        #else
        _PyUnicode_FastCopyCharacters(*p_left, left_len, right, 0, right_len);
        #endif
        __Pyx_INCREF(*p_left);
        __Pyx_GIVEREF(*p_left);
        return *p_left;
    } else {
        return __Pyx_PyUnicode_Concat(left, right);
    }
  }
#endif

/* RaiseUnboundLocalError */
static void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    if (level == -1) {
        const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
        if (package_sep != (0)) {
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_mstate_global->__pyx_d, empty_dict, from_list, 1);
            if (unlikely(!module)) {
                if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                    goto bad;
                PyErr_Clear();
            }
        }
        level = 0;
    }
    if (!module) {
        module = PyImport_ImportModuleLevelObject(
            name, __pyx_mstate_global->__pyx_d, empty_dict, from_list, level);
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    return module;
}

/* ImportDottedModule */
static PyObject *__Pyx__ImportDottedModule_Error(PyObject *name, PyObject *parts_tuple, Py_ssize_t count) {
    PyObject *partial_name = NULL, *slice = NULL, *sep = NULL;
    Py_ssize_t size;
    if (unlikely(PyErr_Occurred())) {
        PyErr_Clear();
    }
#if CYTHON_ASSUME_SAFE_SIZE
    size = PyTuple_GET_SIZE(parts_tuple);
#else
    size = PyTuple_Size(parts_tuple);
    if (size < 0) goto bad;
#endif
    if (likely(size == count)) {
        partial_name = name;
    } else {
        slice = PySequence_GetSlice(parts_tuple, 0, count);
        if (unlikely(!slice))
            goto bad;
        sep = PyUnicode_FromStringAndSize(".", 1);
        if (unlikely(!sep))
            goto bad;
        partial_name = PyUnicode_Join(sep, slice);
    }
    PyErr_Format(
        PyExc_ModuleNotFoundError,
        "No module named '%U'", partial_name);
bad:
    Py_XDECREF(sep);
    Py_XDECREF(slice);
    Py_XDECREF(partial_name);
    return NULL;
}
static PyObject *__Pyx__ImportDottedModule_Lookup(PyObject *name) {
    PyObject *imported_module;
#if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
        CYTHON_COMPILING_IN_GRAAL
    PyObject *modules = PyImport_GetModuleDict();
    if (unlikely(!modules))
        return NULL;
    imported_module = __Pyx_PyDict_GetItemStr(modules, name);
    Py_XINCREF(imported_module);
#else
    imported_module = PyImport_GetModule(name);
#endif
    return imported_module;
}
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple) {
    Py_ssize_t i, nparts;
#if CYTHON_ASSUME_SAFE_SIZE
    nparts = PyTuple_GET_SIZE(parts_tuple);
#else
    nparts = PyTuple_Size(parts_tuple);
    if (nparts < 0) return NULL;
#endif
    for (i=1; i < nparts && module; i++) {
        PyObject *part, *submodule;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        part = PyTuple_GET_ITEM(parts_tuple, i);
#else
        part = __Pyx_PySequence_ITEM(parts_tuple, i);
        if (!part) return NULL;
#endif
        submodule = __Pyx_PyObject_GetAttrStrNoError(module, part);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_DECREF(part);
#endif
        Py_DECREF(module);
        module = submodule;
    }
    if (unlikely(!module)) {
        return __Pyx__ImportDottedModule_Error(name, parts_tuple, i);
    }
    return module;
}
static PyObject *__Pyx__ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
    PyObject *imported_module;
    PyObject *module = __Pyx_Import(name, NULL, 0);
    if (!parts_tuple || unlikely(!module))
        return module;
    imported_module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(imported_module)) {
        Py_DECREF(module);
        return imported_module;
    }
    PyErr_Clear();
    return __Pyx_ImportDottedModule_WalkParts(module, name, parts_tuple);
}
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(module)) {
        PyObject *spec = __Pyx_PyObject_GetAttrStrNoError(module, __pyx_mstate_global->__pyx_n_u_spec);
        if (likely(spec)) {
            PyObject *unsafe = __Pyx_PyObject_GetAttrStrNoError(spec, __pyx_mstate_global->__pyx_n_u_initializing);
            if (likely(!unsafe || !__Pyx_PyObject_IsTrue(unsafe))) {
                Py_DECREF(spec);
                spec = NULL;
            }
            Py_XDECREF(unsafe);
        }
        if (likely(!spec)) {
            PyErr_Clear();
            return module;
        }
        Py_DECREF(spec);
        Py_DECREF(module);
    } else if (PyErr_Occurred()) {
        PyErr_Clear();
    }
#endif
    return __Pyx__ImportDottedModule(name, parts_tuple);
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u__11);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                CYTHON_COMPILING_IN_GRAAL
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
    }
    return value;
}

/* HasAttr */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!PyUnicode_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_PyObject_GetAttrStrNoError(o, n);
    if (!r) {
        return (unlikely(PyErr_Occurred())) ? -1 : 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}
#endif

/* CallTypeTraverse */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg) {
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
    if (__Pyx_get_runtime_version() < 0x03090000) return 0;
    #endif
    if (!always_call) {
        PyTypeObject *base = __Pyx_PyObject_GetSlot(o, tp_base, PyTypeObject*);
        unsigned long flags = PyType_GetFlags(base);
        if (flags & Py_TPFLAGS_HEAPTYPE) {
            return 0;
        }
    }
    Py_VISIT((PyObject*)Py_TYPE(o));
    return 0;
}
#endif

/* LimitedApiGetTypeDict */
#if CYTHON_COMPILING_IN_LIMITED_API
static Py_ssize_t __Pyx_GetTypeDictOffset(void) {
    PyObject *tp_dictoffset_o;
    Py_ssize_t tp_dictoffset;
    tp_dictoffset_o = PyObject_GetAttrString((PyObject*)(&PyType_Type), "__dictoffset__");
    if (unlikely(!tp_dictoffset_o)) return -1;
    tp_dictoffset = PyLong_AsSsize_t(tp_dictoffset_o);
    Py_DECREF(tp_dictoffset_o);
    if (unlikely(tp_dictoffset == 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' doesn't have a dictoffset");
        return -1;
    } else if (unlikely(tp_dictoffset < 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' has an unexpected negative dictoffset. "
            "Please report this as Cython bug");
        return -1;
    }
    return tp_dictoffset;
}
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp) {
    static Py_ssize_t tp_dictoffset = 0;
    if (unlikely(tp_dictoffset == 0)) {
        tp_dictoffset = __Pyx_GetTypeDictOffset();
        if (unlikely(tp_dictoffset == -1 && PyErr_Occurred())) {
            tp_dictoffset = 0; // try again next time?
            return NULL;
        }
    }
    return *(PyObject**)((char*)tp + tp_dictoffset);
}
#endif

/* SetItemOnTypeDict */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_SetItem(tp_dict, k, v);
    if (likely(!result)) {
        PyType_Modified(tp);
        if (unlikely(PyObject_HasAttr(v, __pyx_mstate_global->__pyx_n_u_set_name))) {
            PyObject *setNameResult = PyObject_CallMethodObjArgs(v, __pyx_mstate_global->__pyx_n_u_set_name,  (PyObject *) tp, k, NULL);
            if (!setNameResult) return -1;
            Py_DECREF(setNameResult);
        }
    }
    return result;
}

/* FixUpExtensionType */
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if __PYX_LIMITED_VERSION_HEX > 0x030900B1
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#else
    const PyType_Slot *slot = spec->slots;
    int changed = 0;
#if !CYTHON_COMPILING_IN_LIMITED_API
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
#if !CYTHON_COMPILING_IN_CPYTHON
        const
#endif  // !CYTHON_COMPILING_IN_CPYTHON)
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif  // CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                    Py_DECREF(descr);
                    if (unlikely(set_item_result < 0)) {
                        return -1;
                    }
                    changed = 1;
                }
            }
            memb++;
        }
    }
#endif  // !CYTHON_COMPILING_IN_LIMITED_API
    slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_getset)
        slot++;
    if (slot && slot->slot == Py_tp_getset) {
        PyGetSetDef *getset = (PyGetSetDef*) slot->pfunc;
        while (getset && getset->name) {
            if (getset->name[0] == '_' && getset->name[1] == '_' && strcmp(getset->name, "__module__") == 0) {
                PyObject *descr = PyDescr_NewGetSet(type, getset);
                if (unlikely(!descr))
                    return -1;
                #if CYTHON_COMPILING_IN_LIMITED_API
                PyObject *pyname = PyUnicode_FromString(getset->name);
                if (unlikely(!pyname)) {
                    Py_DECREF(descr);
                    return -1;
                }
                int set_item_result = __Pyx_SetItemOnTypeDict(type, pyname, descr);
                Py_DECREF(pyname);
                #else
                CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
                int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                #endif
                Py_DECREF(descr);
                if (unlikely(set_item_result < 0)) {
                    return -1;
                }
                changed = 1;
            }
            ++getset;
        }
    }
    if (changed)
        PyType_Modified(type);
#endif  // PY_VERSION_HEX > 0x030900B1
    return 0;
}

/* ValidateBasesTuple */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_SIZE
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (unlikely(n < 0)) return -1;
#endif
    for (i = 1; i < n; i++)
    {
        PyTypeObject *b;
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !CYTHON_USE_TYPE_SLOTS
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
CYTHON_UNUSED static int __Pyx_PyType_HasMultipleInheritance(PyTypeObject *t) {
    while (t) {
        PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
        if (bases) {
            return 1;
        }
        t = __Pyx_PyType_GetSlot(t, tp_base, PyTypeObject*);
    }
    return 0;
}
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !CYTHON_COMPILING_IN_CPYTHON || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    if (!__Pyx_PyType_HasMultipleInheritance(t)) {
        return PyType_Ready(t);
    }
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400) &&\
                !CYTHON_COMPILING_IN_GRAAL
        gc = PyImport_GetModule(__pyx_mstate_global->__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_mstate_global->__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i=0;
    Py_ssize_t size;
    void** base_vtables;
    __Pyx_TypeName tp_base_name = NULL;
    __Pyx_TypeName base_name = NULL;
    void* unknown = (void*)-1;
    PyObject* bases = __Pyx_PyType_GetSlot(type, tp_bases, PyObject*);
    int base_depth = 0;
    {
        PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        while (base) {
            base_depth += 1;
            base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
        }
    }
    base_vtables = (void**) PyMem_Malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
#if CYTHON_COMPILING_IN_LIMITED_API
    size = PyTuple_Size(bases);
    if (size < 0) goto other_failure;
#else
    size = PyTuple_GET_SIZE(bases);
#endif
    for (i = 1; i < size; i++) {
        PyObject *basei;
        void* base_vtable;
#if CYTHON_AVOID_BORROWED_REFS
        basei = PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#else
        basei = PyTuple_GET_ITEM(bases, i);
#endif
        base_vtable = __Pyx_GetVtable((PyTypeObject*)basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
            }
        }
    }
    PyErr_Clear();
    PyMem_Free(base_vtables);
    return 0;
bad:
    {
        PyTypeObject* basei = NULL;
        PyTypeObject* tp_base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        tp_base_name = __Pyx_PyType_GetFullyQualifiedName(tp_base);
#if CYTHON_AVOID_BORROWED_REFS
        basei = (PyTypeObject*)PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = (PyTypeObject*)PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#else
        basei = (PyTypeObject*)PyTuple_GET_ITEM(bases, i);
#endif
        base_name = __Pyx_PyType_GetFullyQualifiedName(basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
    }
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
really_bad: // bad has failed!
#endif
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
other_failure:
#endif
    PyMem_Free(base_vtables);
    return -1;
}

/* DelItemOnTypeDict */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_DelItem(tp_dict, k);
    if (likely(!result)) PyType_Modified(tp);
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_mstate_global->__pyx_n_u_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_mstate_global->__pyx_n_u_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython);
            if (likely(reduce_cython)) {
                ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_mstate_global->__pyx_n_u_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetFullyQualifiedName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_1_2
#define __PYX_HAVE_RT_ImportType_3_1_2
static PyTypeObject *__Pyx_ImportType_3_1_2(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_1_2 check_size)
{
    PyObject *result = 0;
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !CYTHON_COMPILING_IN_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    if (size == 0) {
        return (PyTypeObject *)result;
    }
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_1_2 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_1_2 && (size_t)basicsize > size) {
        if (PyErr_WarnFormat(NULL, 0,
                "%.200s.%.200s size changed, may indicate binary incompatibility. "
                "Expected %zd from C header, got %zd from PyObject",
                module_name, class_name, size, basicsize) < 0) {
            goto bad;
        }
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* FetchSharedCythonModule */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef(__PYX_ABI_MODULE_NAME);
}

/* dict_setdefault */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value,
                                                       int is_safe_type) {
    PyObject* value;
    CYTHON_MAYBE_UNUSED_VAR(is_safe_type);
#if CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethod(d, "setdefault", "OO", key, default_value);
#elif PY_VERSION_HEX >= 0x030d0000
    PyDict_SetDefaultRef(d, key, default_value, &value);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* FetchCommonType */
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject* __Pyx_PyType_FromMetaclass(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *result = __Pyx_PyType_FromModuleAndSpec(module, spec, bases);
    if (result && metaclass) {
        PyObject *old_tp = (PyObject*)Py_TYPE(result);
    Py_INCREF((PyObject*)metaclass);
#if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        Py_SET_TYPE(result, metaclass);
#else
        result->ob_type = metaclass;
#endif
        Py_DECREF(old_tp);
    }
    return result;
}
#else
#define __Pyx_PyType_FromMetaclass(me, mo, s, b) PyType_FromMetaclass(me, mo, s, b)
#endif
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t expected_basicsize) {
    Py_ssize_t basicsize;
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (expected_basicsize == 0) {
        return 0; // size is inherited, nothing useful to check
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
    if (unlikely(!py_basicsize)) return -1;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = NULL;
    if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) return -1;
#else
    basicsize = ((PyTypeObject*) cached_type)->tp_basicsize;
#endif
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module = NULL, *cached_type = NULL, *abi_module_dict, *new_cached_type, *py_object_name;
    int get_item_ref_result;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    py_object_name = PyUnicode_FromString(object_name);
    if (!py_object_name) return NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) goto done;
    abi_module_dict = PyModule_GetDict(abi_module);
    if (!abi_module_dict) goto done;
    get_item_ref_result = __Pyx_PyDict_GetItemRef(abi_module_dict, py_object_name, &cached_type);
    if (get_item_ref_result == 1) {
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else if (unlikely(get_item_ref_result == -1)) {
        goto bad;
    }
    CYTHON_UNUSED_VAR(module);
    cached_type = __Pyx_PyType_FromMetaclass(metaclass, abi_module, spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    new_cached_type = __Pyx_PyDict_SetDefault(abi_module_dict, py_object_name, cached_type, 1);
    if (unlikely(new_cached_type != cached_type)) {
        if (unlikely(!new_cached_type)) goto bad;
        Py_DECREF(cached_type);
        cached_type = new_cached_type;
        if (__Pyx_VerifyCachedType(
                cached_type,
                object_name,
                spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else {
        Py_DECREF(new_cached_type);
    }
done:
    Py_XDECREF(abi_module);
    Py_DECREF(py_object_name);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CommonTypesMetaclass */
PyObject* __pyx_CommonTypesMetaclass_get_module(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED void* context) {
    return PyUnicode_FromString(__PYX_ABI_MODULE_NAME);
}
static PyGetSetDef __pyx_CommonTypesMetaclass_getset[] = {
    {"__module__", __pyx_CommonTypesMetaclass_get_module, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_CommonTypesMetaclass_slots[] = {
    {Py_tp_getset, (void *)__pyx_CommonTypesMetaclass_getset},
    {0, 0}
};
static PyType_Spec __pyx_CommonTypesMetaclass_spec = {
    __PYX_TYPE_MODULE_PREFIX "_common_types_metatype",
    0,
    0,
#if PY_VERSION_HEX >= 0x030A0000
    Py_TPFLAGS_IMMUTABLETYPE |
    Py_TPFLAGS_DISALLOW_INSTANTIATION |
#endif
    Py_TPFLAGS_DEFAULT,
    __pyx_CommonTypesMetaclass_slots
};
static int __pyx_CommonTypesMetaclass_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    PyObject *bases = PyTuple_Pack(1, &PyType_Type);
    if (unlikely(!bases)) {
        return -1;
    }
    mstate->__pyx_CommonTypesMetaclassType = __Pyx_FetchCommonTypeFromSpec(NULL, module, &__pyx_CommonTypesMetaclass_spec, bases);
    if (unlikely(mstate->__pyx_CommonTypesMetaclassType == NULL)) {
        return -1;
    }
    return 0;
}

/* PyMethodNew */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *result;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    #if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    {
        PyObject *args[] = {func, self};
        result = PyObject_Vectorcall(__pyx_mstate_global->__Pyx_CachedMethodType, args, 2, NULL);
    }
    #else
    result = PyObject_CallFunctionObjArgs(__pyx_mstate_global->__Pyx_CachedMethodType, func, self, NULL);
    #endif
    return result;
}
#else
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#endif

/* PyVectorcallFastCallDict */
#if CYTHON_METH_FASTCALL && (CYTHON_VECTORCALL || CYTHON_BACKPORT_VECTORCALL)
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i, pos;
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    #if !CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t nkw = PyDict_Size(kw);
    if (unlikely(nkw == -1)) return NULL;
    #else
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    #endif
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (PyDict_Next(kw, &pos, &key, &value)) {
        keys_are_strings &=
        #if CYTHON_COMPILING_IN_LIMITED_API
            PyType_GetFlags(Py_TYPE(key));
        #else
            Py_TYPE(key)->tp_flags;
        #endif
        Py_INCREF(key);
        Py_INCREF(value);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(PyTuple_SetItem(kwnames, i, key) < 0)) goto cleanup;
        #else
        PyTuple_SET_ITEM(kwnames, i, key);
        #endif
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    Py_ssize_t kw_size =
        likely(kw == NULL) ?
        0 :
#if !CYTHON_ASSUME_SAFE_SIZE
        PyDict_Size(kw);
#else
        PyDict_GET_SIZE(kw);
#endif
    if (kw_size == 0) {
        return vc(func, args, nargs, NULL);
    }
#if !CYTHON_ASSUME_SAFE_SIZE
    else if (unlikely(kw_size == -1)) {
        return NULL;
    }
#endif
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunctionNoMethod(PyObject *func, void (*cfunc)(void)) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if ((PyObject*)Py_TYPE(func) == __pyx_mstate_global->__Pyx_CachedMethodType) {
        int result;
        PyObject *newFunc = PyObject_GetAttr(func, __pyx_mstate_global->__pyx_n_u_func);
        if (unlikely(!newFunc)) {
            PyErr_Clear(); // It's only an optimization, so don't throw an error
            return 0;
        }
        result = __Pyx__IsSameCyOrCFunctionNoMethod(newFunc, cfunc);
        Py_DECREF(newFunc);
        return result;
    }
    return __Pyx__IsSameCyOrCFunctionNoMethod(func, cfunc);
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if (PyMethod_Check(func)) {
        func = PyMethod_GET_FUNCTION(func);
    }
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure) {
    PyObject *result;
    CYTHON_UNUSED_VAR(closure);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_doc_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#else
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_name_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    Py_INCREF(op->func_qualname);
    result = op->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_dict_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_dict_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_dict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_defaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_kwdefaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_annotations_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine_value(__pyx_CyFunctionObject *op) {
    int is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
    if (is_coroutine) {
        PyObject *is_coroutine_value, *module, *fromlist, *marker = __pyx_mstate_global->__pyx_n_u_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(marker);
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_mstate_global->__pyx_n_u_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        is_coroutine_value = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(is_coroutine_value)) {
            return is_coroutine_value;
        }
ignore:
        PyErr_Clear();
    }
    return __Pyx_PyBool_FromLong(is_coroutine);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    result = __Pyx_CyFunction_get_is_coroutine_value(op);
    if (unlikely(!result))
        return NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    if (op->func_is_coroutine) {
        Py_DECREF(result);
        result = __Pyx_NewRef(op->func_is_coroutine);
    } else {
        op->func_is_coroutine = __Pyx_NewRef(result);
    }
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static void __Pyx_CyFunction_raise_argument_count_error(__pyx_CyFunctionObject *func, const char* message, Py_ssize_t size) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        py_name, message, size);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        name, message, size);
#endif
}
static void __Pyx_CyFunction_raise_type_error(__pyx_CyFunctionObject *func, const char* message) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s",
        py_name, message);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s",
        name, message);
#endif
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {"func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {"func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {"__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {"func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {"__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {"_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
    {"__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#if CYTHON_METH_FASTCALL
#if CYTHON_BACKPORT_VECTORCALL || CYTHON_COMPILING_IN_LIMITED_API
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {"__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(args);
    __Pyx_BEGIN_CRITICAL_SECTION(m);
    Py_INCREF(m->func_qualname);
    result = m->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    Py_CLEAR(m->defaults);
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    {
        int e = __Pyx_call_type_traverse((PyObject*)m, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
    Py_VISIT(m->func_dict);
    __Pyx_VISIT_CONST(m->func_name);
    __Pyx_VISIT_CONST(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    __Pyx_VISIT_CONST(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
#endif
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    Py_VISIT(m->defaults);
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
    PyObject *repr;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    repr = PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
    __Pyx_END_CRITICAL_SECTION();
    return repr;
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void(*)(void))meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes no arguments", size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes exactly one argument", size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
    __Pyx_CyFunction_raise_type_error(
        (__pyx_CyFunctionObject*)func, "takes no keyword arguments");
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL && (CYTHON_VECTORCALL || CYTHON_BACKPORT_VECTORCALL)
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_SIZE
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(argc < 0)) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL && (CYTHON_VECTORCALL || CYTHON_BACKPORT_VECTORCALL)
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            __Pyx_CyFunction_raise_type_error(
                cyfunc, "needs an argument");
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(__Pyx_PyTuple_GET_SIZE(kwnames))) {
        __Pyx_CyFunction_raise_type_error(
            cyfunc, "takes no keyword arguments");
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes no arguments", nargs);
        return NULL;
    }
    return meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes exactly one argument", nargs);
        return NULL;
    }
    return meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCMethod)(void(*)(void))meth)(self, cls, args, (size_t)nargs, kwnames);
}
#endif
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if CYTHON_METH_FASTCALL
#if defined(Py_TPFLAGS_HAVE_VECTORCALL)
    Py_TPFLAGS_HAVE_VECTORCALL |
#elif defined(_Py_TPFLAGS_HAVE_VECTORCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
#endif // CYTHON_METH_FASTCALL
#if PY_VERSION_HEX >= 0x030A0000
    Py_TPFLAGS_IMMUTABLETYPE |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
static int __pyx_CyFunction_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_CyFunctionType_spec, NULL);
    if (unlikely(mstate->__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func, PyTypeObject *defaults_type) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_CallObject((PyObject*)defaults_type, NULL); // _PyObject_New(defaults_type);
    if (unlikely(!m->defaults))
        return NULL;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* ListPack */
static PyObject *__Pyx_PyList_Pack(Py_ssize_t n, ...) {
    va_list va;
    PyObject *l = PyList_New(n);
    va_start(va, n);
    if (unlikely(!l)) goto end;
    for (Py_ssize_t i=0; i<n; ++i) {
        PyObject *arg = va_arg(va, PyObject*);
        Py_INCREF(arg);
        if (__Pyx_PyList_SET_ITEM(l, i, arg) != (0)) {
            Py_CLEAR(l);
            goto end;
        }
    }
    end:
    va_end(va);
    return l;
}

/* FusedFunction */
static PyObject *
__pyx_FusedFunction_New(PyMethodDef *ml, int flags,
                        PyObject *qualname, PyObject *closure,
                        PyObject *module, PyObject *globals,
                        PyObject *code)
{
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_FusedFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        __pyx_FusedFunctionObject *fusedfunc = (__pyx_FusedFunctionObject *) op;
        fusedfunc->__signatures__ = NULL;
        fusedfunc->self = NULL;
        #if CYTHON_COMPILING_IN_LIMITED_API
        fusedfunc->ml = ml;
        #endif
        PyObject_GC_Track(op);
    }
    return op;
}
static void
__pyx_FusedFunction_dealloc(__pyx_FusedFunctionObject *self)
{
    PyObject_GC_UnTrack(self);
    Py_CLEAR(self->self);
    Py_CLEAR(self->__signatures__);
    __Pyx__CyFunction_dealloc((__pyx_CyFunctionObject *) self);
}
static int
__pyx_FusedFunction_traverse(__pyx_FusedFunctionObject *self,
                             visitproc visit,
                             void *arg)
{
    Py_VISIT(self->self);
    Py_VISIT(self->__signatures__);
    return __Pyx_CyFunction_traverse((__pyx_CyFunctionObject *) self, visit, arg);
}
static int
__pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self)
{
    Py_CLEAR(self->self);
    Py_CLEAR(self->__signatures__);
    return __Pyx_CyFunction_clear((__pyx_CyFunctionObject *) self);
}
static __pyx_FusedFunctionObject *
__pyx_FusedFunction_descr_get_locked(__pyx_FusedFunctionObject *func, PyObject *obj)
{
    PyObject *module;
    __pyx_FusedFunctionObject *meth;
    #if CYTHON_COMPILING_IN_LIMITED_API
    module = __Pyx_CyFunction_get_module((__pyx_CyFunctionObject *) func, NULL);
    if ((unlikely(!module))) return NULL;
    #else
    module = ((PyCFunctionObject *) func)->m_module;
    #endif
    meth = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_New(
        #if CYTHON_COMPILING_IN_LIMITED_API
                    func->ml,
        #else
                    ((PyCFunctionObject *) func)->m_ml,
        #endif
                    ((__pyx_CyFunctionObject *) func)->flags,
                    ((__pyx_CyFunctionObject *) func)->func_qualname,
                    ((__pyx_CyFunctionObject *) func)->func_closure,
                    module,
                    ((__pyx_CyFunctionObject *) func)->func_globals,
                    ((__pyx_CyFunctionObject *) func)->func_code);
    #if CYTHON_COMPILING_IN_LIMITED_API
    Py_DECREF(module);
    #endif
    if (unlikely(!meth))
        return NULL;
    Py_XINCREF(func->func.defaults);
    meth->func.defaults = func->func.defaults;
    __Pyx_CyFunction_SetClassObj(meth, __Pyx_CyFunction_GetClassObj(func));
    Py_XINCREF(func->__signatures__);
    meth->__signatures__ = func->__signatures__;
    Py_XINCREF(func->func.defaults_tuple);
    meth->func.defaults_tuple = func->func.defaults_tuple;
    Py_XINCREF(obj);
    meth->self = obj;
    return meth;
}
static PyObject *
__pyx_FusedFunction_descr_get(PyObject *self, PyObject *obj, PyObject *type)
{
    __pyx_FusedFunctionObject *func, *meth;
    func = (__pyx_FusedFunctionObject *) self;
    if (func->self || func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(self);
        return self;
    }
    if (obj == Py_None)
        obj = NULL;
    if (func->func.flags & __Pyx_CYFUNCTION_CLASSMETHOD)
        obj = type;
    if (obj == NULL) {
        Py_INCREF(self);
        return self;
    }
    __Pyx_BEGIN_CRITICAL_SECTION(func);
    meth = __pyx_FusedFunction_descr_get_locked(func, obj);
    __Pyx_END_CRITICAL_SECTION()
    return (PyObject *) meth;
}
static PyObject *
_obj_to_string(PyObject *obj)
{
    if (PyUnicode_CheckExact(obj))
        return __Pyx_NewRef(obj);
    else if (PyType_Check(obj))
        return PyObject_GetAttr(obj, __pyx_mstate_global->__pyx_n_u_name);
    else
        return PyObject_Str(obj);
}
static PyObject *
__pyx_FusedFunction_getitem(__pyx_FusedFunctionObject *self, PyObject *idx)
{
    PyObject *signature = NULL;
    PyObject *unbound_result_func;
    PyObject *result_func = NULL;
    if (unlikely(self->__signatures__ == NULL)) {
        PyErr_SetString(PyExc_TypeError, "Function is not fused");
        return NULL;
    }
    if (PyTuple_Check(idx)) {
        Py_ssize_t n = __Pyx_PyTuple_GET_SIZE(idx);
        PyObject *list;
        int i;
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(n < 0)) return NULL;
        #endif
        list = PyList_New(n);
        if (unlikely(!list))
            return NULL;
        for (i = 0; i < n; i++) {
            PyObject *string;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            PyObject *item = PyTuple_GET_ITEM(idx, i);
#else
            PyObject *item = __Pyx_PySequence_ITEM(idx, i);  if (unlikely(!item)) goto __pyx_err;
#endif
            string = _obj_to_string(item);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
            Py_DECREF(item);
#endif
            if (unlikely(!string)) goto __pyx_err;
	    if (__Pyx_PyList_SET_ITEM(list, i, string) < 0) goto __pyx_err;
        }
        signature = PyUnicode_Join(__pyx_mstate_global->__pyx_kp_u__5, list);
__pyx_err:;
        Py_DECREF(list);
    } else {
        signature = _obj_to_string(idx);
    }
    if (unlikely(!signature))
        return NULL;
    unbound_result_func = PyObject_GetItem(self->__signatures__, signature);
    if (likely(unbound_result_func)) {
        if (self->self) {
            __pyx_FusedFunctionObject *unbound = (__pyx_FusedFunctionObject *) unbound_result_func;
            __Pyx_CyFunction_SetClassObj(unbound, __Pyx_CyFunction_GetClassObj(self));
            result_func = __pyx_FusedFunction_descr_get(unbound_result_func,
                                                        self->self, self->self);
        } else {
            result_func = unbound_result_func;
            Py_INCREF(result_func);
        }
    }
    Py_DECREF(signature);
    Py_XDECREF(unbound_result_func);
    return result_func;
}
static PyObject *
__pyx_FusedFunction_callfunction(PyObject *func, PyObject *args, PyObject *kw)
{
     __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    int static_specialized = (cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD &&
                              !((__pyx_FusedFunctionObject *) func)->__signatures__);
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !static_specialized) {
        return __Pyx_CyFunction_CallAsMethod(func, args, kw);
    } else {
        return __Pyx_CyFunction_Call(func, args, kw);
    }
}
static PyObject *
__pyx_FusedFunction_call(PyObject *func, PyObject *args, PyObject *kw)
{
    __pyx_FusedFunctionObject *binding_func = (__pyx_FusedFunctionObject *) func;
    Py_ssize_t argc = __Pyx_PyTuple_GET_SIZE(args);
    PyObject *new_args = NULL;
    __pyx_FusedFunctionObject *new_func = NULL;
    PyObject *result = NULL;
    int is_staticmethod = binding_func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD;
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(argc < 0)) return NULL;
    #endif
    if (binding_func->self) {
        PyObject *self;
        Py_ssize_t i;
        new_args = PyTuple_New(argc + 1);
        if (unlikely(!new_args))
            return NULL;
        self = binding_func->self;
        Py_INCREF(self);
        if (__Pyx_PyTuple_SET_ITEM(new_args, 0, self)) goto bad;
        self = NULL;
        for (i = 0; i < argc; i++) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            PyObject *item = PyTuple_GET_ITEM(args, i);
            Py_INCREF(item);
#else
            PyObject *item = __Pyx_PySequence_ITEM(args, i);  if (unlikely(!item)) goto bad;
#endif
        if (__Pyx_PyTuple_SET_ITEM(new_args, i + 1, item)) goto bad;
        }
        args = new_args;
    }
    if (binding_func->__signatures__) {
        PyObject *tup;
        if (is_staticmethod && binding_func->func.flags & __Pyx_CYFUNCTION_CCLASS) {
            tup = PyTuple_Pack(3, args,
                               kw == NULL ? Py_None : kw,
                               binding_func->func.defaults_tuple);
            if (unlikely(!tup)) goto bad;
            new_func = (__pyx_FusedFunctionObject *) __Pyx_CyFunction_CallMethod(
                func, binding_func->__signatures__, tup, NULL);
        } else {
            tup = PyTuple_Pack(4, binding_func->__signatures__, args,
                               kw == NULL ? Py_None : kw,
                               binding_func->func.defaults_tuple);
            if (unlikely(!tup)) goto bad;
            new_func = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_callfunction(func, tup, NULL);
        }
        Py_DECREF(tup);
        if (unlikely(!new_func))
            goto bad;
        __Pyx_CyFunction_SetClassObj(new_func, __Pyx_CyFunction_GetClassObj(binding_func));
        func = (PyObject *) new_func;
    }
    result = __pyx_FusedFunction_callfunction(func, args, kw);
bad:
    Py_XDECREF(new_args);
    Py_XDECREF((PyObject *) new_func);
    return result;
}
static PyMemberDef __pyx_FusedFunction_members[] = {
    {"__signatures__",
     T_OBJECT,
     offsetof(__pyx_FusedFunctionObject, __signatures__),
     READONLY,
     0},
    {"__self__", T_OBJECT_EX, offsetof(__pyx_FusedFunctionObject, self), READONLY, 0},
    #if !CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
    #endif
    {0, 0, 0, 0, 0},
};
static PyGetSetDef __pyx_FusedFunction_getsets[] = {
    {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    #if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
    #endif
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_FusedFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__pyx_FusedFunction_dealloc},
    {Py_tp_call, (void *)__pyx_FusedFunction_call},
    {Py_tp_traverse, (void *)__pyx_FusedFunction_traverse},
    {Py_tp_clear, (void *)__pyx_FusedFunction_clear},
    {Py_tp_members, (void *)__pyx_FusedFunction_members},
    {Py_tp_getset, (void *)__pyx_FusedFunction_getsets},
    {Py_tp_descr_get, (void *)__pyx_FusedFunction_descr_get},
    {Py_mp_subscript, (void *)__pyx_FusedFunction_getitem},
    {0, 0},
};
static PyType_Spec __pyx_FusedFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "fused_cython_function",
    sizeof(__pyx_FusedFunctionObject),
    0,
#if PY_VERSION_HEX >= 0x030A0000
    Py_TPFLAGS_IMMUTABLETYPE |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    __pyx_FusedFunctionType_slots
};
static int __pyx_FusedFunction_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    PyObject *bases = PyTuple_Pack(1, mstate->__pyx_CyFunctionType);
    if (unlikely(!bases)) {
        return -1;
    }
    mstate->__pyx_FusedFunctionType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_FusedFunctionType_spec, bases);
    Py_DECREF(bases);
    if (unlikely(mstate->__pyx_FusedFunctionType == NULL)) {
        return -1;
    }
    return 0;
}

/* CLineInTraceback */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_mstate_global->__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __Pyx_BEGIN_CRITICAL_SECTION(*cython_runtime_dict);
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback))
        Py_XINCREF(use_cline);
        __Pyx_END_CRITICAL_SECTION();
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_cython_runtime, __pyx_mstate_global->__pyx_n_u_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_INCREF(use_cline);
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_mstate_global->__pyx_cython_runtime, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    Py_XDECREF(use_cline);
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
    __Pyx_CachedCodeObjectType* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = code_cache->entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__find_code_object;
    return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
    if (old_count < 0) {
        __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
        return NULL;
    }
#endif
    __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
#endif
    return result;
#endif
}
static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
{
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            code_cache->entries = entries;
            code_cache->max_count = 64;
            code_cache->count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
        __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_INCREF(code_object);
        Py_DECREF(tmp);
        return;
    }
    if (code_cache->count == code_cache->max_count) {
        int new_max = code_cache->max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        code_cache->entries = entries;
        code_cache->max_count = new_max;
    }
    for (i=code_cache->count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    code_cache->count++;
    Py_INCREF(code_object);
}
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__insert_code_object;
    return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type expected = 0;
    if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
        return;
    }
#endif
    __pyx__insert_code_object(code_cache, code_line, code_object);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
#endif
#endif
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        (void) __pyx_cfilenm;
        (void) __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!code_object) {
        code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
        if (unlikely(!code_object)) goto bad;
        py_py_line = PyLong_FromLong(py_line);
        if (unlikely(!py_py_line)) goto bad;
        py_funcname = PyUnicode_FromString(funcname);
        if (unlikely(!py_funcname)) goto bad;
        dict = PyDict_New();
        if (unlikely(!dict)) goto bad;
        {
            PyObject *old_code_object = code_object;
            code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
            Py_DECREF(old_code_object);
        }
        if (unlikely(!code_object)) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
    } else {
        dict = PyDict_New();
    }
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    if (c_line) {
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
    }
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyLong_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyLong_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 2 * PyLong_SHIFT)) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 3 * PyLong_SHIFT)) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 4 * PyLong_SHIFT)) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (size_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(size_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(size_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(size_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        size_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (size_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (size_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (size_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (size_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(size_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((size_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(size_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((size_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((size_t) 1) << (sizeof(size_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (size_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyLong_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if defined(HAVE_LONG_LONG) && !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_mystatus_t(mystatus_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const mystatus_t neg_one = (mystatus_t) -1, const_zero = (mystatus_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(mystatus_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(mystatus_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if defined(HAVE_LONG_LONG) && !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(mystatus_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(mystatus_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(mystatus_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(mystatus_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(mystatus_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
static __Pyx_TypeName
__Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
{
    PyObject *module = NULL, *name = NULL, *result = NULL;
    #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
    name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_qualname);
    #else
    name = PyType_GetQualName(tp);
    #endif
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
    module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_module);
    if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
    if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
        result = name;
        name = NULL;
        goto done;
    }
    result = PyUnicode_FromFormat("%U.%U", module, name);
    if (unlikely(result == NULL)) goto bad;
  done:
    Py_XDECREF(name);
    Py_XDECREF(module);
    return result;
  bad:
    PyErr_Clear();
    if (name) {
        result = name;
        name = NULL;
    } else {
        result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__12);
    }
    goto done;
}
#endif

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* IterNextPlain */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void) {
    if (unlikely(!__pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache))
        __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache = __Pyx_GetBuiltinName(__pyx_mstate_global->__pyx_n_u_next);
    return __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator) {
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    PyObject *result;
    PyObject *next = __Pyx_GetBuiltinNext_LimitedAPI();
    if (unlikely(!next)) return NULL;
    result = PyObject_CallFunctionObjArgs(next, iterator, NULL);
    return result;
#else
    (void)__Pyx_GetBuiltinName; // only for early limited API
    iternextfunc iternext = __Pyx_PyObject_GetIterNextFunc(iterator);
    assert(iternext);
    return iternext(iterator);
#endif
}

/* PyObjectCallMethod1 */
#if !(CYTHON_VECTORCALL && __PYX_LIMITED_VERSION_HEX >= 0x030C0000)
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
#endif
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
#if CYTHON_VECTORCALL && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[2] = {obj, arg};
    (void) __Pyx_PyObject_GetMethod;
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_Call2Args;
    return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
#endif
}

/* ReturnWithStopIteration */
static void __Pyx__ReturnWithStopIteration(PyObject* value, int async);
static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext) {
    if (value == Py_None) {
        if (async || !iternext)
            PyErr_SetNone(async ? PyExc_StopAsyncIteration : PyExc_StopIteration);
        return;
    }
    __Pyx__ReturnWithStopIteration(value, async);
}
static void __Pyx__ReturnWithStopIteration(PyObject* value, int async) {
#if CYTHON_COMPILING_IN_CPYTHON
    __Pyx_PyThreadState_declare
#endif
    PyObject *exc;
    PyObject *exc_type = async ? PyExc_StopAsyncIteration : PyExc_StopIteration;
#if CYTHON_COMPILING_IN_CPYTHON
    if ((PY_VERSION_HEX >= (0x030C00A6)) || unlikely(PyTuple_Check(value) || PyExceptionInstance_Check(value))) {
        if (PY_VERSION_HEX >= (0x030e00A1)) {
            exc = __Pyx_PyObject_CallOneArg(exc_type, value);
        } else {
            PyObject *args_tuple = PyTuple_New(1);
            if (unlikely(!args_tuple)) return;
            Py_INCREF(value);
            PyTuple_SET_ITEM(args_tuple, 0, value);
            exc = PyObject_Call(exc_type, args_tuple, NULL);
            Py_DECREF(args_tuple);
        }
        if (unlikely(!exc)) return;
    } else {
        Py_INCREF(value);
        exc = value;
    }
    #if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    #if CYTHON_USE_EXC_INFO_STACK
    if (!__pyx_tstate->exc_info->exc_value)
    #else
    if (!__pyx_tstate->exc_type)
    #endif
    {
        Py_INCREF(exc_type);
        __Pyx_ErrRestore(exc_type, exc, NULL);
        return;
    }
    #endif
#else
    exc = __Pyx_PyObject_CallOneArg(exc_type, value);
    if (unlikely(!exc)) return;
#endif
    PyErr_SetObject(exc_type, exc);
    Py_DECREF(exc);
}

/* CoroutineBase */
#if !CYTHON_COMPILING_IN_LIMITED_API
#include <frameobject.h>
#if PY_VERSION_HEX >= 0x030b00a6 && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#endif // CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void
__Pyx_Coroutine_Undelegate(__pyx_CoroutineObject *gen) {
#if CYTHON_USE_AM_SEND
    gen->yieldfrom_am_send = NULL;
#endif
    Py_CLEAR(gen->yieldfrom);
}
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    CYTHON_UNUSED_VAR(__pyx_tstate);
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
        else if (likely(__Pyx_IS_TYPE(ev, (PyTypeObject*)PyExc_StopIteration))) {
            #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
            value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
            if (unlikely(!value)) goto limited_api_failure;
            #else
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            #endif
            Py_DECREF(ev);
        }
        else if (unlikely(PyTuple_Check(ev))) {
            Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(ev);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely(tuple_size < 0)) {
                Py_XDECREF(tb);
                Py_DECREF(ev);
                Py_DECREF(et);
                return -1;
            }
            #endif
            if (tuple_size >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#elif CYTHON_ASSUME_SAFE_MACROS
                value = PySequence_ITEM(ev, 0);
#else
                value = PySequence_GetItem(ev, 0);
                if (!value) goto limited_api_failure;
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
#else
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
#endif
    Py_DECREF(ev);
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!value)) return -1;
#endif
    *pvalue = value;
    return 0;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL || !CYTHON_ASSUME_SAFE_MACROS
  limited_api_failure:
    Py_XDECREF(et);
    Py_XDECREF(tb);
    Py_XDECREF(ev);
    return -1;
#endif
}
static CYTHON_INLINE
__Pyx_PySendResult __Pyx_Coroutine_status_from_result(PyObject **retval) {
    if (*retval) {
        return PYGEN_NEXT;
    } else if (likely(__Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, retval) == 0)) {
        return PYGEN_RETURN;
    } else {
        return PYGEN_ERROR;
    }
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_CLEAR(exc_state->exc_value);
#else
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
#endif
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(__pyx_CoroutineObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
static void __Pyx_Coroutine_AlreadyTerminatedError(PyObject *gen, PyObject *value, int closing) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    CYTHON_MAYBE_UNUSED_VAR(closing);
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
__Pyx_PySendResult __Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, PyObject **result, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(__Pyx_Coroutine_get_is_running(self));  // Callers should ensure is_running
    if (unlikely(self->resume_label == -1)) {
        __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
        return PYGEN_ERROR;
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_value) {
        #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        #else
        PyObject *exc_tb;
        #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
        exc_tb = PyException_GetTraceback(exc_state->exc_value);
        #elif PY_VERSION_HEX >= 0x030B00a4
        exc_tb = ((PyBaseExceptionObject*) exc_state->exc_value)->traceback;
        #else
        exc_tb = exc_state->exc_traceback;
        #endif
        if (exc_tb) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
            #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
            Py_DECREF(exc_tb);
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    retval = self->body(self, tstate, value);
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    *result = retval;
    if (self->resume_label == -1) {
        return likely(retval) ? PYGEN_RETURN : PYGEN_ERROR;
    }
    return PYGEN_NEXT;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(exc_state);
#else
    PyObject *exc_tb;
    #if PY_VERSION_HEX >= 0x030B00a4
    if (!exc_state->exc_value) return;
    exc_tb = PyException_GetTraceback(exc_state->exc_value);
    #else
    exc_tb = exc_state->exc_traceback;
    #endif
    if (likely(exc_tb)) {
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
        #if PY_VERSION_HEX >= 0x030B00a4
        Py_DECREF(exc_tb);
        #endif
    }
#endif
}
#define __Pyx_Coroutine_MethodReturnFromResult(gen, result, retval, iternext)\
    ((result) == PYGEN_NEXT ? (retval) : __Pyx__Coroutine_MethodReturnFromResult(gen, result, retval, iternext))
static PyObject *
__Pyx__Coroutine_MethodReturnFromResult(PyObject* gen, __Pyx_PySendResult result, PyObject *retval, int iternext) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if (likely(result == PYGEN_RETURN)) {
        int is_async = 0;
        #ifdef __Pyx_AsyncGen_USED
        is_async = __Pyx_AsyncGen_CheckExact(gen);
        #endif
        __Pyx_ReturnWithStopIteration(retval, is_async, iternext);
        Py_XDECREF(retval);
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
#if PY_VERSION_HEX < 0x030d00A1
            _PyGen_SetStopIterationValue(result);
#else
            if (!PyTuple_Check(result) && !PyExceptionInstance_Check(result)) {
                PyErr_SetObject(PyExc_StopIteration, result);
            } else {
                PyObject *exc = __Pyx_PyObject_CallOneArg(PyExc_StopIteration, result);
                if (likely(exc != NULL)) {
                    PyErr_SetObject(PyExc_StopIteration, exc);
                    Py_DECREF(exc);
                }
            }
#endif
        }
        Py_DECREF(result);
        result = NULL;
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE __Pyx_PySendResult
__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen, PyObject** retval) {
    __Pyx_PySendResult result;
    PyObject *val = NULL;
    assert(__Pyx_Coroutine_get_is_running(gen));
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    result = __Pyx_Coroutine_SendEx(gen, val, retval, 0);
    Py_XDECREF(val);
    return result;
}
#if CYTHON_USE_AM_SEND
static __Pyx_PySendResult
__Pyx_Coroutine_SendToDelegate(__pyx_CoroutineObject *gen, __Pyx_pyiter_sendfunc gen_am_send, PyObject *value, PyObject **retval) {
    PyObject *ret = NULL;
    __Pyx_PySendResult delegate_result, result;
    assert(__Pyx_Coroutine_get_is_running(gen));
    delegate_result = gen_am_send(gen->yieldfrom, value, &ret);
    if (delegate_result == PYGEN_NEXT) {
        assert (ret != NULL);
        *retval = ret;
        return PYGEN_NEXT;
    }
    assert (delegate_result != PYGEN_ERROR || ret == NULL);
    __Pyx_Coroutine_Undelegate(gen);
    result = __Pyx_Coroutine_SendEx(gen, ret, retval, 0);
    Py_XDECREF(ret);
    return result;
}
#endif
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result = __Pyx_Coroutine_AmSend(self, value, &retval);
    return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
}
static __Pyx_PySendResult
__Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval) {
    __Pyx_PySendResult result;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
        return PYGEN_ERROR;
    }
    #if CYTHON_USE_AM_SEND
    if (gen->yieldfrom_am_send) {
        result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, value, retval);
    } else
    #endif
    if (gen->yieldfrom) {
        PyObject *yf = gen->yieldfrom;
        PyObject *ret;
      #if !CYTHON_USE_AM_SEND
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
      #endif
        {
            #if !CYTHON_COMPILING_IN_LIMITED_API || __PYX_LIMITED_VERSION_HEX >= 0x03080000
            if (value == Py_None && PyIter_Check(yf))
                ret = __Pyx_PyIter_Next_Plain(yf);
            else
            #endif
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_mstate_global->__pyx_n_u_send, value);
        }
        if (likely(ret)) {
            __Pyx_Coroutine_unset_is_running(gen);
            *retval = ret;
            return PYGEN_NEXT;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, retval);
    } else {
        result = __Pyx_Coroutine_SendEx(gen, value, retval, 0);
    }
    __Pyx_Coroutine_unset_is_running(gen);
    return result;
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    __Pyx_PySendResult result;
    PyObject *retval = NULL;
    CYTHON_UNUSED_VAR(gen);
    assert(__Pyx_Coroutine_get_is_running(gen));
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        result = __Pyx_Coroutine_Close(yf, &retval);
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        result = __Pyx_Coroutine_Close(yf, &retval);
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        result = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf);
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
        result = PYGEN_RETURN;
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
        result = PYGEN_RETURN;
    } else
    #endif
    {
        PyObject *meth;
        result = PYGEN_RETURN;
        meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_close);
        if (unlikely(!meth)) {
            if (unlikely(PyErr_Occurred())) {
                PyErr_WriteUnraisable(yf);
            }
        } else {
            retval = __Pyx_PyObject_CallNoArg(meth);
            Py_DECREF(meth);
            if (unlikely(!retval)) {
                result = PYGEN_ERROR;
            }
        }
    }
    Py_XDECREF(retval);
    return result == PYGEN_ERROR ? -1 : 0;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __Pyx_PySendResult result;
    PyObject *retval = NULL;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    }
    #if CYTHON_USE_AM_SEND
    if (gen->yieldfrom_am_send) {
        result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, Py_None, &retval);
    } else
    #endif
    if (gen->yieldfrom) {
        PyObject *yf = gen->yieldfrom;
        PyObject *ret;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && (PY_VERSION_HEX < 0x030A00A3 || !CYTHON_USE_AM_SEND)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
            ret = __Pyx_PyIter_Next_Plain(yf);
        if (likely(ret)) {
            __Pyx_Coroutine_unset_is_running(gen);
            return ret;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, &retval);
    } else {
        result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
    }
    __Pyx_Coroutine_unset_is_running(gen);
    return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 1);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, PyObject *arg) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result;
    CYTHON_UNUSED_VAR(arg);
    result = __Pyx_Coroutine_Close(self, &retval);
    if (unlikely(result == PYGEN_ERROR))
        return NULL;
    Py_XDECREF(retval);
    Py_RETURN_NONE;
}
static __Pyx_PySendResult
__Pyx_Coroutine_Close(PyObject *self, PyObject **retval) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PySendResult result;
    PyObject *yf;
    int err = 0;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
        return PYGEN_ERROR;
    }
    yf = gen->yieldfrom;
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    result = __Pyx_Coroutine_SendEx(gen, NULL, retval, 1);
    if (result == PYGEN_ERROR) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_Coroutine_unset_is_running(gen);
        if (!__Pyx_PyErr_Occurred()) {
            return PYGEN_RETURN;
        } else if (likely(__Pyx_PyErr_ExceptionMatches2(PyExc_GeneratorExit, PyExc_StopIteration))) {
            __Pyx_PyErr_Clear();
            return PYGEN_RETURN;
        }
        return PYGEN_ERROR;
    } else if (likely(result == PYGEN_RETURN && *retval == Py_None)) {
        __Pyx_Coroutine_unset_is_running(gen);
        return PYGEN_RETURN;
    } else {
        const char *msg;
        Py_DECREF(*retval);
        *retval = NULL;
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
            msg = "async generator ignored GeneratorExit";
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        __Pyx_Coroutine_unset_is_running(gen);
        return PYGEN_ERROR;
    }
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen)))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    yf = gen->yieldfrom;
    if (yf) {
        __Pyx_PySendResult result;
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                goto propagate_exception;
            goto throw_here;
        }
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (unlikely(PyErr_Occurred())) {
                    __Pyx_Coroutine_unset_is_running(gen);
                    return NULL;
                }
                __Pyx_Coroutine_Undelegate(gen);
                goto throw_here;
            }
            if (likely(args)) {
                ret = __Pyx_PyObject_Call(meth, args, NULL);
            } else {
                PyObject *cargs[4] = {NULL, typ, val, tb};
                ret = __Pyx_PyObject_FastCall(meth, cargs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
            }
            Py_DECREF(meth);
        }
        Py_DECREF(yf);
        if (ret) {
            __Pyx_Coroutine_unset_is_running(gen);
            return ret;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, &ret);
        __Pyx_Coroutine_unset_is_running(gen);
        return __Pyx_Coroutine_MethodReturnFromResult(self, result, ret, 0);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
propagate_exception:
    {
        PyObject *retval = NULL;
        __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, NULL, &retval, 0);
        __Pyx_Coroutine_unset_is_running(gen);
        return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
    }
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (unlikely(!PyArg_UnpackTuple(args, "throw", 1, 3, &typ, &val, &tb)))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_VISIT(exc_state->exc_value);
#else
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
#endif
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    {
        int e = __Pyx_call_type_traverse((PyObject*)gen, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if CYTHON_USE_TP_FINALIZE
        if (unlikely(PyObject_CallFinalizerFromDealloc(self)))
#else
        {
            destructor del = __Pyx_PyObject_GetSlot(gen, tp_del, destructor);
            if (del) del(self);
        }
        if (unlikely(Py_REFCNT(self) > 0))
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    __Pyx_PyHeapTypeObject_GC_Del(gen);
}
#if CYTHON_USE_TP_FINALIZE
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *retval = NULL;
        __Pyx_PySendResult result = __Pyx_Coroutine_Close(self, &retval);
        if (result == PYGEN_ERROR) {
            PyErr_WriteUnraisable(self);
        } else {
            Py_XDECREF(retval);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
}
#endif
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_name;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_name, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_qualname;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_qualname, value);
    return 0;
}
static PyObject *
__Pyx__Coroutine_get_frame(__pyx_CoroutineObject *self)
{
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyObject *frame;
    #if PY_VERSION_HEX >= 0x030d0000
    Py_BEGIN_CRITICAL_SECTION(self);
    #endif
    frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        PyObject *globals = PyDict_New();
        if (unlikely(!globals)) return NULL;
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            globals,                        /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        Py_DECREF(globals);
        if (unlikely(!frame))
            return NULL;
        if (unlikely(self->gi_frame)) {
            Py_DECREF(frame);
            frame = self->gi_frame;
        } else {
            self->gi_frame = frame;
        }
    }
    Py_INCREF(frame);
    #if PY_VERSION_HEX >= 0x030d0000
    Py_END_CRITICAL_SECTION();
    #endif
    return frame;
#else
    CYTHON_UNUSED_VAR(self);
    Py_RETURN_NONE;
#endif
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, void *context) {
    CYTHON_UNUSED_VAR(context);
    PyObject *frame = self->gi_frame;
    if (frame)
        return __Pyx_NewRef(frame);
    return __Pyx__Coroutine_get_frame(self);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->yieldfrom_am_send = NULL;
    #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_LIMITED_API
    gen->gi_exc_state.exc_value = NULL;
    #else
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
    #endif
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}
static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen) {
    char result;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_BEGIN_CRITICAL_SECTION(gen);
    #endif
    result = gen->is_running;
    gen->is_running = 1;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_END_CRITICAL_SECTION();
    #endif
    return result;
}
static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen) {
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_BEGIN_CRITICAL_SECTION(gen);
    #endif
    assert(gen->is_running);
    gen->is_running = 0;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_END_CRITICAL_SECTION();
    #endif
}
static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen) {
    char result;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_BEGIN_CRITICAL_SECTION(gen);
    #endif
    result = gen->is_running;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_END_CRITICAL_SECTION();
    #endif
    return result;
}
static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure) {
    CYTHON_UNUSED_VAR(closure);
    char result = __Pyx_Coroutine_get_is_running((__pyx_CoroutineObject*)gen);
    if (result) Py_RETURN_TRUE;
    else Py_RETURN_FALSE;
}
#if __PYX_HAS_PY_AM_SEND == 2
static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send) {
    Py_ssize_t ptr_offset = (char*)(type->tp_as_async) - (char*)type;
    if (ptr_offset < 0 || ptr_offset > type->tp_basicsize) {
        return;
    }
    memcpy((void*)static_amsend_methods, (void*)(type->tp_as_async), sizeof(*type->tp_as_async));
    static_amsend_methods->am_send = am_send;
    type->tp_as_async = __Pyx_SlotTpAsAsync(static_amsend_methods);
}
#endif
static PyObject *__Pyx_Coroutine_fail_reduce_ex(PyObject *self, PyObject *arg) {
    CYTHON_UNUSED_VAR(arg);
    __Pyx_TypeName self_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE((PyObject*)self));
    PyErr_Format(PyExc_TypeError, "cannot pickle '" __Pyx_FMT_TYPENAME "' object",
                         self_type_name);
    __Pyx_DECREF_TypeName(self_type_name);
    return NULL;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {"__reduce_ex__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_O, 0},
    {"__reduce__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_NOARGS, 0},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {"gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     PyDoc_STR("object being iterated by 'yield from', or None")},
    {"gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {"__module__", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_modulename), 0, 0},
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CoroutineObject, gi_weakreflist), READONLY, 0},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {"__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     PyDoc_STR("name of the generator"), 0},
    {"__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     PyDoc_STR("qualified name of the generator"), 0},
    {"gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     PyDoc_STR("Frame of the generator"), 0},
    {"gi_running", __Pyx_Coroutine_get_is_running_getter, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_GeneratorType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_Coroutine_dealloc},
    {Py_tp_traverse, (void *)__Pyx_Coroutine_traverse},
    {Py_tp_iter, (void *)PyObject_SelfIter},
    {Py_tp_iternext, (void *)__Pyx_Generator_Next},
    {Py_tp_methods, (void *)__pyx_Generator_methods},
    {Py_tp_members, (void *)__pyx_Generator_memberlist},
    {Py_tp_getset, (void *)__pyx_Generator_getsets},
    {Py_tp_getattro, (void *) PyObject_GenericGetAttr},
#if CYTHON_USE_TP_FINALIZE
    {Py_tp_finalize, (void *)__Pyx_Coroutine_del},
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    {Py_am_send, (void *)__Pyx_Coroutine_AmSend},
#endif
    {0, 0},
};
static PyType_Spec __pyx_GeneratorType_spec = {
    __PYX_TYPE_MODULE_PREFIX "generator",
    sizeof(__pyx_CoroutineObject),
    0,
#if PY_VERSION_HEX >= 0x030A0000
    Py_TPFLAGS_IMMUTABLETYPE |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE | __Pyx_TPFLAGS_HAVE_AM_SEND,
    __pyx_GeneratorType_slots
};
#if __PYX_HAS_PY_AM_SEND == 2
static __Pyx_PyAsyncMethodsStruct __pyx_Generator_as_async;
#endif
static int __pyx_Generator_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_GeneratorType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_GeneratorType_spec, NULL);
    if (unlikely(!mstate->__pyx_GeneratorType)) {
        return -1;
    }
#if __PYX_HAS_PY_AM_SEND == 2
    __Pyx_SetBackportTypeAmSend(mstate->__pyx_GeneratorType, &__pyx_Generator_as_async, &__Pyx_Coroutine_AmSend);
#endif
    return 0;
}
static PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *retval = NULL;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    }
    __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
    __Pyx_Coroutine_unset_is_running(gen);
    (void) result;
    assert (result == PYGEN_RETURN || result == PYGEN_ERROR);
    assert ((result == PYGEN_RETURN && retval != NULL) || (result == PYGEN_ERROR && retval == NULL));
    return retval;
}

/* GetRuntimeVersion */
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    return Py_Version & ~0xFFUL;
#else
    static unsigned long __Pyx_cached_runtime_version = 0;
    if (__Pyx_cached_runtime_version == 0) {
        const char* rt_version = Py_GetVersion();
        unsigned long version = 0;
        unsigned long factor = 0x01000000UL;
        unsigned int digit = 0;
        int i = 0;
        while (factor) {
            while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                ++i;
            }
            version += factor * digit;
            if (rt_version[i] != '.')
                break;
            digit = 0;
            factor >>= 8;
            ++i;
        }
        __Pyx_cached_runtime_version = version;
    }
    return __Pyx_cached_runtime_version;
#endif
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* NewCodeObj */
#if CYTHON_COMPILING_IN_LIMITED_API
    static PyObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                       PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                       PyObject *fv, PyObject *cell, PyObject* fn,
                                       PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
#elif PY_VERSION_HEX >= 0x030B0000
  static PyCodeObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                         PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                         PyObject *fv, PyObject *cell, PyObject* fn,
                                         PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, __pyx_mstate_global->__pyx_empty_bytes);
    return result;
  }
#elif PY_VERSION_HEX >= 0x030800B2 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        const char *line_table,
        PyObject *tuple_dedup_map
) {
    PyObject *code_obj = NULL, *varnames_tuple_dedup = NULL, *code_bytes = NULL, *line_table_bytes = NULL;
    Py_ssize_t var_count = (Py_ssize_t) descr.nlocals;
    PyObject *varnames_tuple = PyTuple_New(var_count);
    if (unlikely(!varnames_tuple)) return NULL;
    for (Py_ssize_t i=0; i < var_count; i++) {
        Py_INCREF(varnames[i]);
        if (__Pyx_PyTuple_SET_ITEM(varnames_tuple, i, varnames[i]) != (0)) goto done;
    }
    #if CYTHON_COMPILING_IN_LIMITED_API
    varnames_tuple_dedup = PyDict_GetItem(tuple_dedup_map, varnames_tuple);
    if (!varnames_tuple_dedup) {
        if (unlikely(PyDict_SetItem(tuple_dedup_map, varnames_tuple, varnames_tuple) < 0)) goto done;
        varnames_tuple_dedup = varnames_tuple;
    }
    #else
    varnames_tuple_dedup = PyDict_SetDefault(tuple_dedup_map, varnames_tuple, varnames_tuple);
    if (unlikely(!varnames_tuple_dedup)) goto done;
    #endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(varnames_tuple_dedup);
    #endif
    if (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table != NULL
        && !CYTHON_COMPILING_IN_GRAAL) {
        line_table_bytes = PyBytes_FromStringAndSize(line_table, descr.line_table_length);
        if (unlikely(!line_table_bytes)) goto done;
        Py_ssize_t code_len = (descr.line_table_length * 2 + 4) & ~3;
        code_bytes = PyBytes_FromStringAndSize(NULL, code_len);
        if (unlikely(!code_bytes)) goto done;
        char* c_code_bytes = PyBytes_AsString(code_bytes);
        if (unlikely(!c_code_bytes)) goto done;
        memset(c_code_bytes, 0, (size_t) code_len);
    }
    code_obj = (PyObject*) __Pyx__PyCode_New(
        (int) descr.argcount,
        (int) descr.num_posonly_args,
        (int) descr.num_kwonly_args,
        (int) descr.nlocals,
        0,
        (int) descr.flags,
        code_bytes ? code_bytes : __pyx_mstate_global->__pyx_empty_bytes,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        varnames_tuple_dedup,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        filename,
        funcname,
        (int) descr.first_line,
        (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table_bytes) ? line_table_bytes : __pyx_mstate_global->__pyx_empty_bytes
    );
done:
    Py_XDECREF(code_bytes);
    Py_XDECREF(line_table_bytes);
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(varnames_tuple_dedup);
    #endif
    Py_DECREF(varnames_tuple);
    return code_obj;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry const *t, PyObject **target, const char* const* encoding_names) {
    while (t->s) {
        PyObject *str;
        if (t->is_unicode) {
            if (t->intern) {
                str = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                str = PyUnicode_Decode(t->s, t->n - 1, encoding_names[t->encoding], NULL);
            } else {
                str = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            str = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        if (!str)
            return -1;
        *target = str;
        if (PyObject_Hash(str) == -1)
            return -1;
        ++t;
        ++target;
    }
    return 0;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


/* MultiPhaseInitModuleState */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
#ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
#else
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
#endif
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
#error "Module state with PEP489 requires atomics. Currently that's one of\
 C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
#endif
#if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#define __Pyx_ModuleStateLookup_Lock()
#define __Pyx_ModuleStateLookup_Unlock()
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
#define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(__cplusplus) && __cplusplus >= 201103L
#include <mutex>
static std::mutex __Pyx_ModuleStateLookup_mutex;
#define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
#define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
#include <threads.h>
static mtx_t __Pyx_ModuleStateLookup_mutex;
static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
    mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
}
#define __Pyx_ModuleStateLookup_Lock()\
  call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
  mtx_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(HAVE_PTHREAD_H)
#include <pthread.h>
static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
#define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(_WIN32)
#include <Windows.h>  // synchapi.h on its own doesn't work
static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
#define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#else
#error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
 Requires C standard >= C11, or C++ standard >= C++11,\
 or pthreads, or the Windows 32 API, or Python >= 3.13."
#endif
typedef struct {
    int64_t id;
    PyObject *module;
} __Pyx_InterpreterIdAndModule;
typedef struct {
    char interpreter_id_as_index;
    Py_ssize_t count;
    Py_ssize_t allocated;
    __Pyx_InterpreterIdAndModule table[1];
} __Pyx_ModuleStateLookupData;
#define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
#else
static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
#endif
static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
        __Pyx_InterpreterIdAndModule* table,
        Py_ssize_t count,
        int64_t interpreterId) {
    __Pyx_InterpreterIdAndModule* begin = table;
    __Pyx_InterpreterIdAndModule* end = begin + count;
    if (begin->id == interpreterId) {
        return begin;
    }
    while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
        __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
        if (halfway->id == interpreterId) {
            return halfway;
        }
        if (halfway->id < interpreterId) {
            begin = halfway;
        } else {
            end = halfway;
        }
    }
    for (; begin < end; ++begin) {
        if (begin->id >= interpreterId) return begin;
    }
    return begin;
}
static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return NULL;
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
    {
        __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        if (likely(data)) {
            __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
            if (likely(data == new_data)) {
                goto read_finished;
            }
        }
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        __Pyx_ModuleStateLookup_Lock();
        __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
        data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
        __Pyx_ModuleStateLookup_Unlock();
    }
  read_finished:;
#else
    __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_InterpreterIdAndModule* found = NULL;
    if (unlikely(!data)) goto end;
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            found = data->table+interpreter_id;
        }
    } else {
        found = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
    }
  end:
    {
        PyObject *result=NULL;
        if (found && found->id == interpreter_id) {
            result = found->module;
        }
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
#endif
        return result;
    }
}
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
    while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
}
#else
#define __Pyx_ModuleStateLookup_wait_until_no_readers()
#endif
static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
    Py_ssize_t to_allocate = (*old_data)->allocated;
    while (to_allocate <= interpreter_id) {
        if (to_allocate == 0) to_allocate = 1;
        else to_allocate *= 2;
    }
    __Pyx_ModuleStateLookupData *new_data = *old_data;
    if (to_allocate != (*old_data)->allocated) {
         new_data = (__Pyx_ModuleStateLookupData *)realloc(
            *old_data,
            sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
        if (!new_data) {
            PyErr_NoMemory();
            return -1;
        }
        for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
            new_data->table[i].id = i;
            new_data->table[i].module = NULL;
        }
        new_data->allocated = to_allocate;
    }
    new_data->table[interpreter_id].module = module;
    if (new_data->count < interpreter_id+1) {
        new_data->count = interpreter_id+1;
    }
    *old_data = new_data;
    return 0;
}
static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
    __Pyx_InterpreterIdAndModule *read = data->table;
    __Pyx_InterpreterIdAndModule *write = data->table;
    __Pyx_InterpreterIdAndModule *end = read + data->count;
    for (; read<end; ++read) {
        if (read->module) {
            write->id = read->id;
            write->module = read->module;
            ++write;
        }
    }
    data->count = write - data->table;
    for (; write<end; ++write) {
        write->id = 0;
        write->module = NULL;
    }
    data->interpreter_id_as_index = 0;
}
static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    int result = 0;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_ModuleStateLookupData *new_data = old_data;
    if (!new_data) {
        new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
        if (!new_data) {
            result = -1;
            PyErr_NoMemory();
            goto end;
        }
        new_data->allocated = 1;
        new_data->interpreter_id_as_index = 1;
    }
    __Pyx_ModuleStateLookup_wait_until_no_readers();
    if (new_data->interpreter_id_as_index) {
        if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
            result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
            goto end;
        }
        __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
    }
    {
        Py_ssize_t insert_at = 0;
        {
            __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                new_data->table, new_data->count, interpreter_id);
            assert(lower_bound);
            insert_at = lower_bound - new_data->table;
            if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                lower_bound->module = module;
                goto end;  // already in table, nothing more to do
            }
        }
        if (new_data->count+1 >= new_data->allocated) {
            Py_ssize_t to_allocate = (new_data->count+1)*2;
            new_data =
                (__Pyx_ModuleStateLookupData*)realloc(
                    new_data,
                    sizeof(__Pyx_ModuleStateLookupData) +
                    (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
            if (!new_data) {
                result = -1;
                new_data = old_data;
                PyErr_NoMemory();
                goto end;
            }
            new_data->allocated = to_allocate;
        }
        ++new_data->count;
        int64_t last_id = interpreter_id;
        PyObject *last_module = module;
        for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
            int64_t current_id = new_data->table[i].id;
            new_data->table[i].id = last_id;
            last_id = current_id;
            PyObject *current_module = new_data->table[i].module;
            new_data->table[i].module = last_module;
            last_module = current_module;
        }
    }
  end:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
#else
    __Pyx_ModuleStateLookup_data = new_data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return result;
}
static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
#endif
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            data->table[interpreter_id].module = NULL;
        }
        goto done;
    }
    {
        __Pyx_ModuleStateLookup_wait_until_no_readers();
        __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
        if (!lower_bound) goto done;
        if (lower_bound->id != interpreter_id) goto done;
        __Pyx_InterpreterIdAndModule *end = data->table+data->count;
        for (;lower_bound<end-1; ++lower_bound) {
            lower_bound->id = (lower_bound+1)->id;
            lower_bound->module = (lower_bound+1)->module;
        }
    }
    --data->count;
    if (data->count == 0) {
        free(data);
        data = NULL;
    }
  done:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
#else
    __Pyx_ModuleStateLookup_data = data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return 0;
}
#endif

/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
